// InputDialogBox.cpp : implementation file
//

#include "stdafx.h"
#include <kch_theparser_type2.h>

#include "InputDialogBox.h"

void SetBoolToEdit(CEditInput* pEdit, ExpTypeless* data);

// CInputDialogBox dialog

IMPLEMENT_DYNAMIC(CInputDialogBox, CDialog)

CInputDialogBox::CInputDialogBox(CWnd* pParent /*=NULL*/)
	: CDialog(CInputDialogBox::IDD, pParent)
	, cControl(0), pPrompt(NULL), pEditbox(NULL)
	, MaxCharSize(0)
	, MaxStringIndex(0)
{

}

CInputDialogBox::~CInputDialogBox()
{
	if(this->pPrompt) 
	{
		delete [] this->pPrompt;
		this->pPrompt = NULL;
	}

	if(this->pEditbox)
	{
		delete [] this->pEditbox;
		this->pEditbox = NULL;
	}

}

void CInputDialogBox::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
}


BEGIN_MESSAGE_MAP(CInputDialogBox, CDialog)
	ON_WM_SIZE()
	ON_BN_CLICKED(IDOK, &CInputDialogBox::OnBnClickedOk)
	ON_BN_CLICKED(IDCANCEL, &CInputDialogBox::OnBnClickedCancel)
	ON_BN_CLICKED(IDC_BTN_BREAK, &CInputDialogBox::OnBnClickedBtnBreak)
END_MESSAGE_MAP()


// CInputDialogBox message handlers

int CInputDialogBox::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CDialog::OnCreate(lpCreateStruct) == -1)
		return -1;

	// TODO:  Add your specialized creation code here
	
	return 0;
}


void CInputDialogBox::InitializePrompt(StrDataVec& sdv)
{
	this->cControl = int(sdv.size());
	
	CString fmt_bool, fmt_intg, fmt_real, fmt_cplx, fmt_vctr, fmt_prompt;
			
	fmt_prompt.LoadStringW(IDS_PROMPT_INPUT); fmt_bool.LoadStringW(IDS_PROMPT_BOOL); 
	fmt_intg.LoadStringW(IDS_PROMPT_INTG);    fmt_real.LoadStringW(IDS_PROMPT_REAL);
	fmt_cplx.LoadStringW(IDS_PROMPT_CPLX);    fmt_vctr.LoadStringW(IDS_PROMPT_VCTR); 
				
	int strlength = 0;     CString first;
	this->MaxCharSize = 0; this->MaxStringIndex = 0;
	
	for(int index=0; index<this->cControl; ++index)
	{
		if(sdv[index].first == "VAR_BOOL")
		{
			LeftHandSide<bool> *pLHS = dynamic_cast<LeftHandSide<bool>*>(sdv[index].second);

			if(pLHS)
				first.FormatMessageW(fmt_bool, index+1, (LPCSTR)pLHS->GetFullName().c_str());
			else 
				HandleSyntaxError(L"Unsupported boolean type for input.\r\n");
		}
		else if(sdv[index].first == "VAR_INTG")
		{
			LeftHandSide<integer> *pLHS = dynamic_cast<LeftHandSide<integer>*>(sdv[index].second);

			if(pLHS)
				first.FormatMessageW(fmt_intg, index+1, (LPCSTR)pLHS->GetFullName().c_str());
			else 
				HandleSyntaxError(L"Unsupported integer type for input.\r\n");
		}
		else if(sdv[index].first == "VAR_REAL")
		{			
			LeftHandSide<realnum> *pLHS = dynamic_cast<LeftHandSide<realnum>*>(sdv[index].second);

			if(pLHS)
				first.FormatMessageW(fmt_real, index+1, (LPCSTR)pLHS->GetFullName().c_str());
			else 
				HandleSyntaxError(L"Unsupported real number type for input.\r\n");

		}
		else if(sdv[index].first == "VAR_CPLX")
		{
			LeftHandSide<Complex> *pLHS = dynamic_cast<LeftHandSide<Complex>*>(sdv[index].second);

			if(pLHS)
				first.FormatMessageW(fmt_cplx, index+1, (LPCSTR)pLHS->GetFullName().c_str());
			else 
				HandleSyntaxError(L"Unsupported complex number type for input.\r\n");

		}
		else if(sdv[index].first == "VAR_VCTR")
		{
			LeftHandSide<Vector> *pLHS = dynamic_cast<LeftHandSide<Vector>*>(sdv[index].second);

			if(pLHS)
				first.FormatMessageW(fmt_vctr, index+1, (LPCSTR)pLHS->GetFullName().c_str());
			else 
				HandleSyntaxError(L"Unsupported vector type for input.\r\n");
		}
		else
			first.FormatMessageW(fmt_prompt, index+1, sdv[index].first);
				
		strlength = CountCharacters(first);

		if(strlength>this->MaxCharSize) 
		{
			this->MaxStringIndex = index; this->MaxCharSize = strlength;
		}
		
		this->strdata.push_back(StrDataPair(first, sdv[index].second));
	}

	if(this->cControl>0)
	{
		if(pPrompt==NULL) pPrompt = new CStatic[this->cControl];
		if(pEditbox==NULL) pEditbox = new CEditInput[this->cControl];
	}
}

BOOL CInputDialogBox::OnInitDialog()
{
	CDialog::OnInitDialog();

	// TODO:  Add extra initialization here
	CClientDC dc(this); CFont *pOldFont, *pFont = this->GetFont();
	
	pOldFont = (CFont*) dc.SelectObject(pFont);
	CRect rect, rectedit; DataType type;
	
	for(int i=0; i<cControl; ++i)
	{
		CSize size = dc.GetTextExtent(this->strdata[i].first); dc.LPtoDP(&size);

		rect.left = 5;  
		rect.top = (size.cy+8) * i + 6;
		rect.right = rect.left + size.cx; 
		rect.bottom = rect.top + size.cy + 16;
		
		pPrompt[i].Create(this->strdata[i].first, 
			WS_CHILD|WS_VISIBLE|SS_LEFT, rect, this, UINT(i+WM_USER+100));
		pPrompt[i].SetFont(pFont);
		
		rectedit.left = int(rect.right); 
		rectedit.top = rect.top-2;
		rectedit.right = int(rectedit.left+160); 
		rectedit.bottom = rectedit.top+rect.Height()-10;

		pEditbox[i].Create(WS_CHILD | WS_VISIBLE | WS_BORDER | ES_AUTOHSCROLL | WS_TABSTOP, 
			rectedit, this, UINT(i+WM_USER+500));

		pEditbox[i].SetFont(pFont); pEditbox[i].SetMargins(4, 2);
		
		type = this->strdata[i].second->datatype;
		
		std::wstring msg;

		if(type == TypeBool)  SetBoolToEdit(&pEditbox[i], this->strdata[i].second);
		else if(type == TypeIntg)
			pEditbox[i].SetWindowText(CString(PrepareDataForEdit<integer>(this->strdata[i].second).c_str()));
		else if(type == TypeReal)
			pEditbox[i].SetWindowText(CString(PrepareDataForEdit<realnum>(this->strdata[i].second).c_str()));
		else if(type == TypeCplx)
			pEditbox[i].SetWindowText(CString(PrepareDataForEdit<Complex>(this->strdata[i].second).c_str()));
		else if(type == TypeVctr)
			pEditbox[i].SetWindowText(CString(PrepareDataForEdit<Vector>(this->strdata[i].second).c_str()));
		else
			throw RunTimeException(_T("Unsupported data type for runtime input.\r\n"));
	}
		
	CRect dlgrect, rc;

	this->GetWindowRect(&dlgrect);

	int dlgWidth = dlgrect.Width(); int dlgHeight = dlgrect.Height();

	pEditbox[this->MaxStringIndex].GetWindowRect(&rectedit);
	pEditbox[this->cControl-1].GetWindowRect(&rc);

	CButton *pBtnOk = (CButton*)this->GetDlgItem(IDOK);

	dlgrect.right = rectedit.right + 14 ;
	pBtnOk->GetWindowRect(&rect); 
	dlgrect.bottom = rc.bottom + rect.Height() + 16;
	
	this->MoveWindow(&dlgrect); 

	this->GotoDlgCtrl(&pEditbox[0]);
	dc.SelectObject(pOldFont);
			
	return FALSE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}


void CInputDialogBox::OnSize(UINT nType, int cx, int cy)
{
	CDialog::OnSize(nType, cx, cy);

	// TODO: Add your message handler code here
	
	CRect rectOk, rectCancel, dlgrect, rectBreak;
	CButton *pBtnOk, *pBtnCancel, *pBtnBreak;

	pBtnOk = (CButton*)this->GetDlgItem(IDOK);
	pBtnCancel = (CButton*)this->GetDlgItem(IDCANCEL);
	pBtnBreak = (CButton*)this->GetDlgItem(IDC_BTN_BREAK);
	
	this->GetClientRect(&dlgrect);

	if(pBtnOk)
	{
		pBtnOk->GetWindowRect(&rectOk);
		int okWidth = rectOk.Width();
		int okHeight = rectOk.Height();

		rectOk.left = dlgrect.Width()/2 - okWidth/2;
		rectOk.top = dlgrect.Height() - okHeight - 1;
		rectOk.right = rectOk.left+okWidth;
		rectOk.bottom = rectOk.top+okHeight;
	
		pBtnOk->MoveWindow(&rectOk);
			
		rectBreak.right = rectOk.left - 4;
		rectBreak.left = rectBreak.right - okWidth;
		rectBreak.top = rectOk.top;
		rectBreak.bottom = rectOk.bottom;
	
		pBtnBreak->MoveWindow(&rectBreak);
			
		pBtnCancel->GetWindowRect(&rectCancel);
		int cancelWidth = rectCancel.Width();
		int cancelHeight = rectCancel.Height();

		rectCancel.left = rectOk.right + 4;
		rectCancel.top = rectOk.top;
		rectCancel.right = rectCancel.left+cancelWidth;
		rectCancel.bottom = rectCancel.top+cancelHeight;
	
		pBtnCancel->MoveWindow(&rectCancel);
	}
}

void CInputDialogBox::OnBnClickedOk()
{
	// TODO: Add your control notification handler code here
	//OnOK();
	
	bool SetVarData(CEdit *pEdit, ExpTypeless* data);
	
	for(int i=0; i<this->cControl; ++i)
	{
		try
		{
			if(SetVarData(&pEditbox[i], this->strdata[i].second)==false)
			{
				::AfxMessageBox(_T("Parsing failed.\r\n")); return;
			}
		}
		catch(...)
		{
			if(::AfxMessageBox(_T("Failed to parse input data!\r\nDo you want to try again?"), 
				MB_YESNO|MB_ICONQUESTION)==IDYES)
			{
				this->GotoDlgCtrl(&this->pEditbox[i]); return;
			}
			else
			{
				this->strdata.clear(); this->EndDialog(1); return;
			}
		}
	}

	this->strdata.clear(); this->EndDialog(1);
}

void CInputDialogBox::OnBnClickedCancel()
{
	// TODO: Add your control notification handler code here
	//OnCancel();
	this->strdata.clear();
	this->EndDialog(0);
}

void CInputDialogBox::OnBnClickedBtnBreak()
{
	// TODO: Add your control notification handler code here

	CString msg;

	msg.LoadStringW(IDS_WANT_BREAK);

	if(::AfxMessageBox(msg, MB_YESNO)==IDYES)
	{
		extern void PutString(UINT);
		//PutString(IDS_PRO_ABORTED);
		//PutString(IDS_STK_UNWIND);
		this->strdata.clear();
		this->EndDialog(2);
	}
}
