%option noyywrap
%option prefix="keyword"

%{
	#include "KeywordParsing.h"
	#include "KeywordParser.hpp"
	
	void DeleteKeywordBuffer() 
		{ keyword_delete_buffer(YY_CURRENT_BUFFER); }	
		
	void ResetKeywordLexer() { yy_start = 0; yy_init = 1; }
%}

ZERO			[0][\x00]		
DIGIT			[1-9][\x00]
DIGITZ			{ZERO}|{DIGIT}
DIGITS			{DIGITZ}+

ATSYMBOL								[@][\x00]
EQUAL									[=][\x00]
NEWLINE									[\n][\x00]

SINGLEQ									([\x27][\x00])|([\xB9\xBC\xC8][\x02])|([\x18\x19\x32][\x20])|([\x8C][\xA7])|([\x01][\x03])
BASIC_LATIN								[A-Za-z_][\x00]
LATIN_1									[\xC0-\xD6\xD8-\xF6\xF8-\xFF\xB4\xB8][\x00]
LATIN_EXTENDED_A						[\x00-\x7F][\x01]
LATIN_EXTENDED_B						([\x80-\xFF][\x01])|([\x00-\x4F][\x02])
GREEK									[\x70-\xFF][\x03]
CYRILLIC								[\x00-\xFF][\x04]
CYRILLIC_SUPPLEMENT						[\x00-\x2F][\x05]
ARMENIAN								[\x30-\x8F][\x05]
HEBREW									[\x90-\xFF][\x05]
ARABIC									[\x00-\xFF][\x06]
SYRIAC									[\x00-\x4F][\x07]
ARABIC_SUPPLEMENT						[\x50-\x7F][\x07]
THAANA									[\x80-\xBF][\x07]
NKO										[\xC0-\xFF][\x07]
DEVANAGARI								[\x00-\x7F][\x09]
BENGALI									[\x80-\xFF][\x09]
GURMUKHI								[\x00-\x7F][\x0A]
GUJARATI								[\x80-\xFF][\x0A]
ORIYA									[\x00-\x7F][\x0B]
TAMIL									[\x80-\xFF][\x0B]
TELUGU									[\x00-\x7F][\x0C]
KANNADA									[\x80-\xFF][\x0C]
MALAYALAM								[\x00-\x7F][\x0D]
SINHALA									[\x80-\xFF][\x0D]
TIBETAN									[\x00-\xFF][\x0F]
GEORGIAN								[\xA0-\xFF][\x10]
HANGUL_JAMO								[\x00-\xFF][\x11]
ETHIOPIC								([\x00-\xFF][\x12])|([\x00-\x7F][\x13])
ETHIOPIC_SUPPLEMENT						[\x80-\x9F][\x13]
CHEROKEE								[\xA0-\xFF][\x13]
CANADIAN_SYLLABICS						([\x00-\xFF][\x14\x15])|([\x00-\x7F][\x16])
OGHAM									[\x80-\x9F][\x16]
RUNIC									[\xA0-\xFF][\x16]
TAGALOG									[\x00-\x1F][\x17]
HANUNOO									[\x20-\x3F][\x17]
BUHID									[\x40-\x5F][\x17]
TAGBANWA								[\x60-\x7F][\x17]
KHMER									[\x80-\xFF][\x17]
MONGOLIAN								[\x00-\xAF][\x18]
LIMBU									[\x00-\x4F][\x19]
BUGINESE								[\x00-\x1F][\x1A]
BALINESE								[\x00-\x7F][\x1B]
LATIN_EXTENDED_ADDITIONAL				[\x00-\xFF][\x1E]
GREEK_EXTENDED							[\x00-\xFF][\x1F]
GLAGOLITIC								[\x00-\x5F][\x2C]
LATIN_EXTENDED_C						[\x60-\x7F][\x2C]
COPTIC									[\x80-\xFF][\X2C]
GEORGIAN_SUPPLEMENT						[\x00-\x2F][\x2D]
TIFINAGH								[\x30-\x7F][\x2D]
ETHIOPIC_EXTENDED						[\x80-\xDF][\x2D]
CJK_RADICALS							[\x80-\xFF][\x2E]
KANGXI_RADICALS							[\x00-\xDF][\x2F]
HIRAGANA								[\x40-\x9F][\x30]
KATAKANA								[\xA0-\xFF][\x30]
BOPOMOFO								[\x00-\x2F][\x31]
HANGUL_COMPATIBILITY_JAMO				[\x30-\x8F][\x31]
KANBUN									[\x90-\x9F][\x31]
BOPOMOFO_EXTENDED						[\xA0-\xBF][\x31]
CJK_STROKES								[\xC0-\xEF][\x31]
KATAKANA_PHONETIC_EXTENSIONS			[\xF0-\xFF][\x31]
CJK_UNIFIED_IDEOGRAPHS_EXTENSTION_A		([\x00-\xFF][\x34-\x4C])|([\x00-\xBF][\x4D])
CJK_UNIFIED_IDEOGRAPHS					([\x00-\xFF][\x4E-\x9E])|([\x00-\xBF][\x9F])
YI_SYLLABLES							([\x00-\xFF][\xA0-\xA3])|([\x00-\x8F][\xA4])
YI_RADICALS								[\x90-\xCF][\xA4]
LATIN_EXTENDED_D						[\x20-\xFF][\xA7]
SYLOTI_NAGRI							[\x00-\x2F][\xA8]
PHAGS_PA								[\x40-\x7F][\xA8]
HANGUL_SYLLABLES						([\x00-\xFF][\xAC-\xD6])|([\x00-\xAF][\xD7])
CJK_COMPATIBILITY_IDEOGRAPHS			[\x00-\xFF][\xF9\xFA]
ARMENIAN_LIGATURES						[\x00-\x4F][\xFB]
ARABIC_PRESENTATION_FORM_A				([\x50-\xFF][\xFB])|([\x00-\xFF][\xFC\xFD])
ARABIC_PRESENTATION_FORM_B				[\x70-\xFF][\xFE]

LETTER									{SINGLEQ}|{BASIC_LATIN}|{LATIN_1}|{LATIN_EXTENDED_A}|{LATIN_EXTENDED_B}|{GREEK}|{CYRILLIC}|{CYRILLIC_SUPPLEMENT}|{ARMENIAN}|{HEBREW}|{ARABIC}|{SYRIAC}|{ARABIC_SUPPLEMENT}|{THAANA}|{NKO}|{DEVANAGARI}|{BENGALI}|{GURMUKHI}|{GUJARATI}|{ORIYA}|{TAMIL}|{TELUGU}|{KANNADA}|{MALAYALAM}|{SINHALA}|{TIBETAN}|{GEORGIAN}|{HANGUL_JAMO}|{ETHIOPIC}|{ETHIOPIC_SUPPLEMENT}|{CHEROKEE}|{CANADIAN_SYLLABICS}|{OGHAM}|{RUNIC}|{TAGALOG}|{HANUNOO}|{BUHID}|{TAGBANWA}|{KHMER}|{MONGOLIAN}|{LIMBU}|{BUGINESE}|{BALINESE}|{LATIN_EXTENDED_ADDITIONAL}|{GREEK_EXTENDED}|{GLAGOLITIC}|{LATIN_EXTENDED_C}|{COPTIC}|{GEORGIAN_SUPPLEMENT}|{TIFINAGH}|{ETHIOPIC_EXTENDED}|{CJK_RADICALS}|{KANGXI_RADICALS}|{HIRAGANA}|{KATAKANA}|{BOPOMOFO}|{HANGUL_COMPATIBILITY_JAMO}|{KANBUN}|{BOPOMOFO_EXTENDED}|{CJK_STROKES}|{KATAKANA_PHONETIC_EXTENSIONS}|{CJK_UNIFIED_IDEOGRAPHS_EXTENSTION_A}|{CJK_UNIFIED_IDEOGRAPHS}|{YI_SYLLABLES}|{YI_RADICALS}|{LATIN_EXTENDED_D}|{SYLOTI_NAGRI}|{PHAGS_PA}|{HANGUL_SYLLABLES}|{CJK_COMPATIBILITY_IDEOGRAPHS}|{ARMENIAN_LIGATURES}|{ARABIC_PRESENTATION_FORM_A}|{ARABIC_PRESENTATION_FORM_B}
LETTERS									{LETTER}+({DIGITS}|{LETTER})*

KEYID									{ATSYMBOL}{BASIC_LATIN}+

ANYCAR									(.){2}
COMMENT									([/][\x00][/][\x00])+{ANYCAR}*[\n][\x00]

%%
	SET_USER_ABORT;
	
{KEYID}									{ 
											std::wstring str = CharToWideString(keywordtext, keywordleng);
											
											keywordlval.id = gTransKeywordTemplate.GetKeywordID(str);
											if(keywordlval.id != keyword_null) return TOKEN_ID;
											
											keywordlval.id = gTransOprTypeTemplate.GetKeywordID(str);
											if(keywordlval.id != keyword_null) return OPTR_ID;
										}
{LETTERS}								{ 
											std::wstring str = CharToWideString(keywordtext, keywordleng);
											
											::lstrcpy(keywordlval.bufr, str.c_str());
											return TOKEN_KEYWORD;
										}
{COMMENT}								{ /* eat up */ }
{EQUAL}									{ return '=';  }
[;][\x00]								{ return ';'; }

{ANYCAR}|([\n][\x00])					/* eat up any other characters */

%%

bool PrepareKeywordLexer(char* scanbuffer, size_t charcount)
{
	if(keyword_scan_buffer(scanbuffer, charcount))
		return true;
	else return false;
}
