#include "SyntaxAnalyzer.h"

#include <kch_autodelete.h>
#include <kch_basetype.h>

#include <string>

UnicodeToken gTokenMap;

void KCH_EXP PutText(char* text, int len)
{
	TCHAR buffer[512]; CString msg;

	::memcpy((void*)buffer, (void*)text, len);
	buffer[len/2]=L'\0';  msg=buffer;
	::AfxMessageBox(msg);
}

std::wstring KCH_EXP CharToWString(char* const buffer, int len)
{
	std::wstring result; UniChar unicode;
	for(int i=0; i<len; i+=2)
	{
		unicode.byte.low = buffer[i]; 
		unicode.byte.high = buffer[i+1];
		result.push_back(unicode.wchar);
	}
	return result;

}

void KCH_EXP CharToWString(std::wstring& result, char* const buffer, int len)
{
	UniChar unicode;
	for(int i=0; i<len; i+=2)
	{
		unicode.byte.low = buffer[i]; 
		unicode.byte.high = buffer[i+1];
		result.push_back(unicode.wchar);
	}
}

KCH_EXP std::wstring TrimString(const std::wstring& str)
{
	int begin = int(str.find_first_not_of(_T(" \t")));
	int end = int(str.find_last_not_of(_T(" \t")));
	
	if(begin != std::wstring::npos && 
		end !=std::wstring::npos) 
		return str.substr(begin, end-begin+1);
	else return std::wstring();
}

KCH_EXP std::wstring CharToWideStringTrim(char* const buffer, int len)
{
	return TrimString(CharToWString(buffer, len));
}

std::wstring KCH_EXP CharToWideString(char* const src, int len)
{
	int charsize = len/2 + 1;
	
	AutoDeleteArray<wchar_t> buffer(charsize);
	::memcpy((void*)buffer.DataPtr, (void*)src, len);
	buffer.DataPtr[len/2] = L'\0';
			
	return std::wstring(buffer.DataPtr);
}

std::string KCH_EXP TrimToAscii(const char* buffer, int len)
{
	std::string rtn;
	for(int i=0; i<len; i+=2) rtn.push_back(buffer[i]);
	return rtn;
}

integer KCH_EXP ToIntLiteral(const char* text, int len, char thousand)
{
	integer rtn; int rlt=0, j=0;
	std::string buffer = TrimToAscii(text, len);
	int size = int(buffer.size());
	
	AutoDeleteArray<char> buf(size+1);
		
	for(int i=0; i<size; ++i)
	{
		if(buffer[i]==thousand) continue;
		else buf.DataPtr[j++]=buffer[i];
	}

	buf.DataPtr[j] = '\0'; rtn=::_atoi64(buf.DataPtr);
	if(rtn== _I64_MAX || rtn == _I64_MIN || errno==ERANGE || errno == EINVAL)
	{		
		::_set_errno(0);
		throw UnicodeException(_T("Integer is either too big or too small.\r\n"));
	}
	else return rtn;
}

realnum KCH_EXP ToRealLiteral(const char* text, int len, char thousand, char decimalpoint)
{
	std::string buffer = TrimToAscii(text, len);

	int rlt=0, j=0; _CRT_DOUBLE dbl;
	int size = int(buffer.size());

	AutoDeleteArray<char> buf(size+1);
		
	for(int i=0; i<size; ++i)
	{
		if(buffer[i]==thousand) continue;
		else if(buffer[i]==decimalpoint)
		{	buf.DataPtr[j++] = '.'; continue; }
		else
			buf.DataPtr[j++] = buffer[i];
	}
	
	buf.DataPtr[j] = '\0'; rlt = ::_atodbl(&dbl, buf.DataPtr);

	if(rlt == 0) return dbl.x;
	else throw UnicodeException(_T("Real number is either too big or too small.\r\n"));
}

KCH_EXP void PutErrorMsgString(char const* msg)
{
	size_t msglen = ::_mbslen((const unsigned char*)msg)*2;

	AutoDeleteArray<wchar_t> msgbuf(int(msglen+2));

	size_t cvt; mbstowcs_s(&cvt, msgbuf.DataPtr, msglen+2, msg, msglen);

	msgbuf.DataPtr[cvt]=L'\0';

	::AfxMessageBox(msgbuf.DataPtr);
}

KCH_EXP bool LoadFileContents(const CString& filepath, CString& contents)
{
	try
	{
		CFile thefile(filepath, CFile::modeRead|CFile::shareDenyWrite);
		CArchive ar(&thefile, CArchive::load); 	ar>>contents; 
		ar.Close(); thefile.Close();  return true;

	}
	catch(CFileException* fe)
	{
		fe->Delete(); return false;
	}
	catch(CException *e)
	{
		e->Delete(); return false;
	}
}

KCH_EXP bool LoadFileContents(const std::wstring& filepath, std::wstring& contents)
{
	CString filecontents;

	if(LoadFileContents(CString(filepath.c_str()), filecontents))
	{
		contents = filecontents.GetBuffer(); return true;
	}
	return false;
}

void UnicodeToken::InsertToken(const char *ascii, const wchar_t *unicode)
{
	theMap.insert(UnicodeToken::TokenPair(std::string(ascii), std::wstring(unicode)));
}

std::wstring UnicodeToken::TranslateToken(const char* ascii)
{
	UnicodeToken::TokenMapItr itr = this->theMap.find(std::string(ascii));
	if(itr == this->theMap.end())
	{
		TRACE(ascii); return std::wstring(L"NOT-IN-THE-MAP");
	}
	else return itr->second;
}

void KCH_EXP PlantTokenMap()
{
	gTokenMap.InsertToken("SOL_INVERSE",		L"inverse");
	gTokenMap.InsertToken("SOL_LUMETHOD",		L"lumethod");
	gTokenMap.InsertToken("SOL_LUDECOMPO",		L"ludecompo");
	gTokenMap.InsertToken("SOL_GAUSSJORDAN",	L"gaussjordan");
	gTokenMap.InsertToken("SOL_SOLVECRAMER",	L"solvecramer");
	gTokenMap.InsertToken("SOL_DETCRAMER",		L"detcramer");

	gTokenMap.InsertToken("TKN_EOF",			L"'%>'");

	gTokenMap.InsertToken("'='",				L"'='");
	gTokenMap.InsertToken("'?'",				L"'?'");
	gTokenMap.InsertToken("':'",				L"':'");
	gTokenMap.InsertToken("'<'",				L"'<'");
	gTokenMap.InsertToken("'>'",				L"'>'");
	gTokenMap.InsertToken("'|'",				L"'|'");
	gTokenMap.InsertToken("'@'",				L"'@'");
	gTokenMap.InsertToken("'#'",				L"'#'");
	gTokenMap.InsertToken("'+'",				L"'+'");
	gTokenMap.InsertToken("'-'",				L"'-'");
	gTokenMap.InsertToken("'*'",				L"'*'");
	gTokenMap.InsertToken("'/'",				L"'/'");
	gTokenMap.InsertToken("'%'",				L"'%'");
	gTokenMap.InsertToken("'^'",				L"'^'");
	gTokenMap.InsertToken("'!'",				L"'!'");
	gTokenMap.InsertToken("'.'",				L"'.'");
	gTokenMap.InsertToken("'&'",				L"'&'");
	gTokenMap.InsertToken("'~'",				L"'~'");
	gTokenMap.InsertToken("'{'",				L"'{'");
	gTokenMap.InsertToken("'}'",				L"'}'");
	gTokenMap.InsertToken("','",				L"','");
	gTokenMap.InsertToken("'['",				L"'['");
	gTokenMap.InsertToken("']'",				L"']'");
	gTokenMap.InsertToken("'('",				L"'('");
	gTokenMap.InsertToken("')'",				L"')'");
	gTokenMap.InsertToken("';'",				L"';'");

	gTokenMap.InsertToken("OPR_OUTPUT",			L"'<<'");
	gTokenMap.InsertToken("OPR_INPUT",			L"'>>'");
	gTokenMap.InsertToken("ARY_OPEN",			L"'{:'");
	gTokenMap.InsertToken("ARY_CLOSE",			L"':}'");

	gTokenMap.InsertToken("OPR_XPR",			L"'<-'");
	gTokenMap.InsertToken("OPR_CRS",			L"'%='");
	gTokenMap.InsertToken("OPR_DIV",			L"'/='");
	gTokenMap.InsertToken("OPR_MUL",			L"'*='");
	gTokenMap.InsertToken("OPR_MNS",			L"'-='");
	gTokenMap.InsertToken("OPR_PLS",			L"'+='");
	gTokenMap.InsertToken("OPR_DEC",			L"'--'");
	gTokenMap.InsertToken("OPR_INC",			L"'++'");
	gTokenMap.InsertToken("OPR_SWAP",			L"'<>'");
	gTokenMap.InsertToken("OPR_SOL",			L"'??'");
	gTokenMap.InsertToken("OPR_PRO",			L"'<*>'");

	gTokenMap.InsertToken("OR",					L"'||'");
	gTokenMap.InsertToken("AND",				L"'&&'");
	gTokenMap.InsertToken("NE",					L"'!='");
	gTokenMap.InsertToken("EQ",					L"'=='");
	gTokenMap.InsertToken("GE",					L"'>='");
	gTokenMap.InsertToken("LE",					L"'<='");

	gTokenMap.InsertToken("$end",				L"end of file");
	gTokenMap.InsertToken("error",				L"error rule");
	gTokenMap.InsertToken("$undefined",			L"undefined lexical token");
	gTokenMap.InsertToken("STMT_CLEAR",			L"clear");
	gTokenMap.InsertToken("STMT_IF",			L"if");
	gTokenMap.InsertToken("STMT_DO",			L"do");
	gTokenMap.InsertToken("STMT_FOR",			L"for");
	gTokenMap.InsertToken("STMT_WHILE",			L"while");

	gTokenMap.InsertToken("STMT_EXIT",			L"exit");
	gTokenMap.InsertToken("STMT_IFTRUE",		L"if_true");
	gTokenMap.InsertToken("STMT_IFFALSE",		L"if_false");
	gTokenMap.InsertToken("STMT_GOTO",			L"goto");
	gTokenMap.InsertToken("STMT_BREAK",			L"break");
	gTokenMap.InsertToken("STMT_SKIP",			L"continue");

	gTokenMap.InsertToken("TKN_NEW",			L"new");
	gTokenMap.InsertToken("TKN_DELETE",			L"delete");

	gTokenMap.InsertToken("TKN_TRY",			L"try");
	gTokenMap.InsertToken("TKN_CATCH",			L"catch");
	gTokenMap.InsertToken("TKN_THROW",			L"throw");

	gTokenMap.InsertToken("STMT_SELECT",		L"select");
	gTokenMap.InsertToken("LABEL_CHOICE",		L"choice label");
	gTokenMap.InsertToken("LABEL_ALWAYS",		L"always label");
	gTokenMap.InsertToken("END_LINE",			L"endl operator");
	gTokenMap.InsertToken("SET_PREC",			L"fmt_precision setting");

	gTokenMap.InsertToken("DECLARE",			L"declare");
	gTokenMap.InsertToken("TKN_STRING",			L"literal string");

	gTokenMap.InsertToken("PTR_BOOL",			L"pointer to boolean");
	gTokenMap.InsertToken("PTR_INTG",			L"pointer to integer");
	gTokenMap.InsertToken("PTR_REAL",			L"pointer to real");
	gTokenMap.InsertToken("PTR_CPLX",			L"pointer to complex");
	gTokenMap.InsertToken("PTR_VCTR",			L"pointer to vector");
	gTokenMap.InsertToken("PTR_STRN",			L"pointer to string");
	gTokenMap.InsertToken("PTR_USER",			L"pointer to user defined type");

	gTokenMap.InsertToken("PTR_AP1I",			L"pointer to 1D integer array");
	gTokenMap.InsertToken("PTR_AP1R",			L"pointer to 1D real array");
	gTokenMap.InsertToken("PTR_AP1C",			L"pointer to 1D complex array");
	gTokenMap.InsertToken("PTR_AP1V",			L"pointer to 1D vector array");

	gTokenMap.InsertToken("PTR_AP2I",			L"pointer to 2D integer array");
	gTokenMap.InsertToken("PTR_AP2R",			L"pointer to 2D real array");
	gTokenMap.InsertToken("PTR_AP2C",			L"pointer to 2D complex array");
	gTokenMap.InsertToken("PTR_AP2V",			L"pointer to 2D vector array");
	
	gTokenMap.InsertToken("PTR_AP3I",			L"pointer to 3D integer array");
	gTokenMap.InsertToken("PTR_AP3R",			L"pointer to 3D real array");
	gTokenMap.InsertToken("PTR_AP3C",			L"pointer to 3D complex array");
	gTokenMap.InsertToken("PTR_AP3V",			L"pointer to 3D vector array");
	
	gTokenMap.InsertToken("PTR_AP4I",			L"pointer to 4D integer array");
	gTokenMap.InsertToken("PTR_AP4R",			L"pointer to 4D real array");
	gTokenMap.InsertToken("PTR_AP4C",			L"pointer to 4D complex array");
	gTokenMap.InsertToken("PTR_AP4V",			L"pointer to 4D vector array");

	gTokenMap.InsertToken("PTR_AP5I",			L"pointer to 5D integer array");
	gTokenMap.InsertToken("PTR_AP5R",			L"pointer to 5D real array");
	gTokenMap.InsertToken("PTR_AP5C",			L"pointer to 5D complex array");
	gTokenMap.InsertToken("PTR_AP5V",			L"pointer to 5D vector array");

	gTokenMap.InsertToken("REAL",				L"literal real");
	gTokenMap.InsertToken("INTG",				L"literal integer");
	gTokenMap.InsertToken("BOOLN",				L"literal boolean");

	gTokenMap.InsertToken("TYPE_CLASS",			L"type class");
	gTokenMap.InsertToken("TKN_EXTENDS",		L"extends");
	gTokenMap.InsertToken("DEF_LITERAL",		L"literal");

	gTokenMap.InsertToken("TYPE_FILE",			L"type file");
	gTokenMap.InsertToken("TYPE_VOID",			L"type void");
	gTokenMap.InsertToken("TYPE_BOOL",			L"type bool");
	gTokenMap.InsertToken("TYPE_INTG",			L"type integer");
	gTokenMap.InsertToken("TYPE_REAL",			L"type real");
	gTokenMap.InsertToken("TYPE_CPLX",			L"type complex");
	gTokenMap.InsertToken("TYPE_VCTR",			L"type vector");
	gTokenMap.InsertToken("TYPE_STRN",			L"type string");
	gTokenMap.InsertToken("TYPE_UDT",			L"type user defined");
	
	gTokenMap.InsertToken("RTN_VOID",			L"return void");
	gTokenMap.InsertToken("RTN_BOOL",			L"return bool");
	gTokenMap.InsertToken("RTN_REAL",			L"return real");
	gTokenMap.InsertToken("RTN_STRN",			L"return string");
	gTokenMap.InsertToken("RTN_CPLX",			L"return complex");
	gTokenMap.InsertToken("RTN_VCTR",			L"return vector");
	gTokenMap.InsertToken("RTN_INTG",			L"return integer");

	gTokenMap.InsertToken("RTN_BPTR",			L"return bool pointer ");
	gTokenMap.InsertToken("RTN_SPTR",			L"return string pointer");
	gTokenMap.InsertToken("RTN_IPTR",			L"return integer pointer ");
	gTokenMap.InsertToken("RTN_RPTR",			L"return real number pointer");
	gTokenMap.InsertToken("RTN_CPTR",			L"return complex number pointer");
	gTokenMap.InsertToken("RTN_VPTR",			L"return vector pointer");

	gTokenMap.InsertToken("RTN_AI1D",			L"return 1D integer array");
	gTokenMap.InsertToken("RTN_AI2D",			L"return 2D integer array");
	gTokenMap.InsertToken("RTN_AI3D",			L"return 3D integer array");
	gTokenMap.InsertToken("RTN_AI4D",			L"return 4D integer array");
	gTokenMap.InsertToken("RTN_AI5D",			L"return 5D integer array");

	gTokenMap.InsertToken("RTN_AR1D",			L"return 1D real array");
	gTokenMap.InsertToken("RTN_AR2D",			L"return 2D real array");
	gTokenMap.InsertToken("RTN_AR3D",			L"return 3D real array");
	gTokenMap.InsertToken("RTN_AR4D",			L"return 4D real array");
	gTokenMap.InsertToken("RTN_AR5D",			L"return 5D real array");

	gTokenMap.InsertToken("RTN_AC1D",			L"return 1D complex array");
	gTokenMap.InsertToken("RTN_AC2D",			L"return 2D complex array");
	gTokenMap.InsertToken("RTN_AC3D",			L"return 3D complex array");
	gTokenMap.InsertToken("RTN_AC4D",			L"return 4D complex array");
	gTokenMap.InsertToken("RTN_AC5D",			L"return 5D complex array");

	gTokenMap.InsertToken("RTN_AV1D",			L"return 1D vector array");
	gTokenMap.InsertToken("RTN_AV2D",			L"return 2D vector array");
	gTokenMap.InsertToken("RTN_AV3D",			L"return 3D vector array");
	gTokenMap.InsertToken("RTN_AV4D",			L"return 4D vector array");
	gTokenMap.InsertToken("RTN_AV5D",			L"return 5D vector array");

	gTokenMap.InsertToken("CPLX_R",				L"real part of complex");
	gTokenMap.InsertToken("CPLX_I",				L"imaginary part of complex");
	
	gTokenMap.InsertToken("VCTR_X",				L"x component of vector");
	gTokenMap.InsertToken("VCTR_Y",				L"y component of vector");
	gTokenMap.InsertToken("VCTR_Z",				L"z component of vector");

	gTokenMap.InsertToken("DEF_VUNIT",			L"set unit vectors (uvctr)");
	gTokenMap.InsertToken("DEF_CUNIT",			L"set unit complex (ucplx)");
	gTokenMap.InsertToken("DEF_STATIC",			L"static");

	gTokenMap.InsertToken("VCTR_UNIT",			L"vector component unit");
	gTokenMap.InsertToken("CPLX_UNIT",			L"complex component unit");
	
	gTokenMap.InsertToken("TYPE_UNDF",			L"undefined symbol");

	gTokenMap.InsertToken("VAR_VOID",			L"variable of type void");
	gTokenMap.InsertToken("VAR_BOOL",			L"variable of type bool");
	gTokenMap.InsertToken("VAR_INTG",			L"variable of type integer");
	gTokenMap.InsertToken("VAR_REAL",			L"variable of type real");
	gTokenMap.InsertToken("VAR_CPLX",			L"variable of type complex");
	gTokenMap.InsertToken("VAR_VCTR",			L"variable of type vector");
	gTokenMap.InsertToken("VAR_STRN",			L"variable of type string");
	gTokenMap.InsertToken("VAR_USER",			L"variable of type user defined");

	gTokenMap.InsertToken("LTL_STRN",			L"literal variable of type string");
	gTokenMap.InsertToken("LTL_INTG",			L"literal variable of type integer");
	gTokenMap.InsertToken("LTL_REAL",			L"literal variable of type real");
	gTokenMap.InsertToken("LTL_CPLX",			L"literal variable of type complex");
	gTokenMap.InsertToken("LTL_VCTR",			L"literal variable of type vector");

	gTokenMap.InsertToken("MEM_LTL_STRN",		L"literal member variable of type string");
	gTokenMap.InsertToken("MEM_LTL_INTG",		L"literal member variable of type integer");
	gTokenMap.InsertToken("MEM_LTL_REAL",		L"literal member variable of type real");
	gTokenMap.InsertToken("MEM_LTL_CPLX",		L"literal member variable of type complex");
	gTokenMap.InsertToken("MEM_LTL_VCTR",		L"literal member variable of type vector");

	gTokenMap.InsertToken("MEM_USER",			L"member variable of user defined type");
	gTokenMap.InsertToken("MEM_BOOL",			L"member variable of type bool");
	gTokenMap.InsertToken("MEM_STRN",			L"member variable of type string");
	gTokenMap.InsertToken("MEM_INTG",			L"member variable of type integer");
	gTokenMap.InsertToken("MEM_REAL",			L"member variable of type real");
	gTokenMap.InsertToken("MEM_CPLX",			L"member variable of type complex");
	gTokenMap.InsertToken("MEM_VCTR",			L"member variable of type vector");

	gTokenMap.InsertToken("MEM_BPTR",			L"member pointer variable of type bool");
	gTokenMap.InsertToken("MEM_SPTR",			L"member pointer variable of type string");
	gTokenMap.InsertToken("MEM_IPTR",			L"member pointer variable of type integer");
	gTokenMap.InsertToken("MEM_RPTR",			L"member pointer variable of type real");
	gTokenMap.InsertToken("MEM_CPTR",			L"member pointer variable of type complex");
	gTokenMap.InsertToken("MEM_VPTR",			L"member pointer variable of type vector");
	gTokenMap.InsertToken("MEM_UPTR",			L"member pointer variable of user defined type");

	gTokenMap.InsertToken("MEM_AI1D",			L"1D array member of type integer");
	gTokenMap.InsertToken("MEM_AI2D",			L"2D array member of type integer");
	gTokenMap.InsertToken("MEM_AI3D",			L"3D array member of type integer");
	gTokenMap.InsertToken("MEM_AI4D",			L"4D array member of type integer");
	gTokenMap.InsertToken("MEM_AI5D",			L"5D array member of type integer");

	gTokenMap.InsertToken("MEM_AR1D",			L"1D array member of type real");
	gTokenMap.InsertToken("MEM_AR2D",			L"2D array member of type real");
	gTokenMap.InsertToken("MEM_AR3D",			L"3D array member of type real");
	gTokenMap.InsertToken("MEM_AR4D",			L"4D array member of type real");
	gTokenMap.InsertToken("MEM_AR5D",			L"5D array member of type real");

	gTokenMap.InsertToken("MEM_AC1D",			L"1D array member of type complex");
	gTokenMap.InsertToken("MEM_AC2D",			L"2D array member of type complex");
	gTokenMap.InsertToken("MEM_AC3D",			L"3D array member of type complex");
	gTokenMap.InsertToken("MEM_AC4D",			L"4D array member of type complex");
	gTokenMap.InsertToken("MEM_AC5D",			L"5D array member of type complex");

	gTokenMap.InsertToken("MEM_AV1D",			L"1D array member of type vector");
	gTokenMap.InsertToken("MEM_AV2D",			L"2D array member of type vector");
	gTokenMap.InsertToken("MEM_AV3D",			L"3D array member of type vector");
	gTokenMap.InsertToken("MEM_AV4D",			L"4D array member of type vector");
	gTokenMap.InsertToken("MEM_AV5D",			L"5D array member of type vector");

	gTokenMap.InsertToken("ARY_INTG1D",			L"variable of 1D integer array");
	gTokenMap.InsertToken("ARY_REAL1D",			L"variable of 1D real array");
	gTokenMap.InsertToken("ARY_CPLX1D",			L"variable of 1D complex array");
	gTokenMap.InsertToken("ARY_VCTR1D",			L"variable of 1D vector array");

	gTokenMap.InsertToken("ARY_INTG2D",			L"variable of 2D integer array");
	gTokenMap.InsertToken("ARY_REAL2D",			L"variable of 2D real array");
	gTokenMap.InsertToken("ARY_CPLX2D",			L"variable of 2D complex array");
	gTokenMap.InsertToken("ARY_VCTR2D",			L"variable of 2D vector array");

	gTokenMap.InsertToken("ARY_INTG3D",			L"variable of 3D integer array");
	gTokenMap.InsertToken("ARY_REAL3D",			L"variable of 3D real array");
	gTokenMap.InsertToken("ARY_CPLX3D",			L"variable of 3D complex array");
	gTokenMap.InsertToken("ARY_VCTR3D",			L"variable of 3D vector array");

	gTokenMap.InsertToken("ARY_INTG4D",			L"variable of 4D integer array");
	gTokenMap.InsertToken("ARY_REAL4D",			L"variable of 4D real array");
	gTokenMap.InsertToken("ARY_CPLX4D",			L"variable of 4D complex array");
	gTokenMap.InsertToken("ARY_VCTR4D",			L"variable of 4D vector array");

	gTokenMap.InsertToken("ARY_INTG5D",			L"variable of 5D integer array");
	gTokenMap.InsertToken("ARY_REAL5D",			L"variable of 5D real array");
	gTokenMap.InsertToken("ARY_CPLX5D",			L"variable of 5D complex array"); 
	gTokenMap.InsertToken("ARY_VCTR5D",			L"variable of 5D vector array");

	gTokenMap.InsertToken("VID_VOID",			L"variable's name of type void");
	gTokenMap.InsertToken("VID_BOOL",			L"variable's name of type bool");
	gTokenMap.InsertToken("VID_INTG",			L"variable's name of type integer");
	gTokenMap.InsertToken("VID_REAL",			L"variable's name of type real");
	gTokenMap.InsertToken("VID_CPLX",			L"variable's name of type complex");
	gTokenMap.InsertToken("VID_VCTR",			L"variable's name of type vector");
	gTokenMap.InsertToken("VID_STRN",			L"variable's name of type string");
	gTokenMap.InsertToken("VID_CLASS",			L"class's name of user defined type");
	gTokenMap.InsertToken("VID_USER",			L"member's name of a class");
	gTokenMap.InsertToken("VID_FILE",			L"variable's name of type file");

	gTokenMap.InsertToken("LID_INTG",			L"literal's name of type integer");
	gTokenMap.InsertToken("LID_STRN",			L"literal's name of type string");
	gTokenMap.InsertToken("LID_REAL",			L"literal's name of type real");
	gTokenMap.InsertToken("LID_CPLX",			L"literal's name of type complex");
	gTokenMap.InsertToken("LID_VCTR",			L"literal's name of type vector");

	gTokenMap.InsertToken("STMT_LABEL",			L"label statement");
	gTokenMap.InsertToken("GOTO_TARGET",		L"goto target label");

	gTokenMap.InsertToken("FNC_BOOL",			L"function returning bool");
	gTokenMap.InsertToken("FNC_VOID",			L"function returning no value");
	gTokenMap.InsertToken("FNC_INTG",			L"function returning integer");
	gTokenMap.InsertToken("FNC_STRN",			L"function returning string");
	gTokenMap.InsertToken("FNC_REAL",			L"function returning real");
	gTokenMap.InsertToken("FNC_CPLX",			L"function returning complex");
	gTokenMap.InsertToken("FNC_VCTR",			L"function returning vector");

	gTokenMap.InsertToken("FNC_BPTR",			L"function returning bool pointer");
	gTokenMap.InsertToken("FNC_SPTR",			L"function returning string pointer");
	gTokenMap.InsertToken("FNC_IPTR",			L"function returning integer pointer");
	gTokenMap.InsertToken("FNC_RPTR",			L"function returning real number pointer");
	gTokenMap.InsertToken("FNC_CPTR",			L"function returning complex number pointer");
	gTokenMap.InsertToken("FNC_VPTR",			L"function returning vector pointer");

	gTokenMap.InsertToken("FNC_USER",			L"function returning user defined data type");

	gTokenMap.InsertToken("FNC_AI1D",			L"function returning 1D integer array");
	gTokenMap.InsertToken("FNC_AI2D",			L"function returning 2D integer array");
	gTokenMap.InsertToken("FNC_AI3D",			L"function returning 3D integer array");
	gTokenMap.InsertToken("FNC_AI4D",			L"function returning 4D integer array");
	gTokenMap.InsertToken("FNC_AI5D",			L"function returning 5D integer array");
	
	gTokenMap.InsertToken("FNC_AR1D",			L"function returning 1D real array");
	gTokenMap.InsertToken("FNC_AR2D",			L"function returning 2D real array");
	gTokenMap.InsertToken("FNC_AR3D",			L"function returning 3D real array");
	gTokenMap.InsertToken("FNC_AR4D",			L"function returning 4D real array");
	gTokenMap.InsertToken("FNC_AR5D",			L"function returning 5D real array");
	
	gTokenMap.InsertToken("FNC_AC1D",			L"function returning 1D complex array");
	gTokenMap.InsertToken("FNC_AC2D",			L"function returning 2D complex array");
	gTokenMap.InsertToken("FNC_AC3D",			L"function returning 3D complex array");
	gTokenMap.InsertToken("FNC_AC4D",			L"function returning 4D complex array");
	gTokenMap.InsertToken("FNC_AC5D",			L"function returning 5D complex array");

	gTokenMap.InsertToken("FNC_AV1D",			L"function returning 1D vector array");
	gTokenMap.InsertToken("FNC_AV2D",			L"function returning 2D vector array");
	gTokenMap.InsertToken("FNC_AV3D",			L"function returning 3D vector array");
	gTokenMap.InsertToken("FNC_AV4D",			L"function returning 4D vector array");
	gTokenMap.InsertToken("FNC_AV5D",			L"function returning 5D vector array");
	
	gTokenMap.InsertToken("FN_INTG",			L"global function returning integer with no arguments");
	gTokenMap.InsertToken("FN_REAL",			L"global function returning real with no arguments");
	gTokenMap.InsertToken("FN_C_C",				L"global function returning complex with a complex argument");
	gTokenMap.InsertToken("FN_I_II",			L"global function returning integer with two integer arguments");
	gTokenMap.InsertToken("FN_R_RR",			L"global function returning real with two real arguments");
	gTokenMap.InsertToken("FN_R_RRR",			L"global function returning real with three real arguments");
	gTokenMap.InsertToken("FN_R_RRRR",			L"global function returning real with four real arguments");
	gTokenMap.InsertToken("FN_R_C",				L"global function returning real with one complex argument");
	gTokenMap.InsertToken("FN_R_V",				L"global function returning real with one vector argument");
	gTokenMap.InsertToken("FN_C_CC",			L"global function returning complex with two complex arguments");
	gTokenMap.InsertToken("FN_C_CCCC",			L"global function returning complex with four complex arguments");
	gTokenMap.InsertToken("FN_C_RR",			L"global function returning complex with two real arguments");
	gTokenMap.InsertToken("FN_R_I",				L"global function returning real with one integer argument");
	gTokenMap.InsertToken("FN_R_II",			L"global function returning real with two integer arguments");
	gTokenMap.InsertToken("FN_V_V",				L"global function returning vector with one vector argument");
	gTokenMap.InsertToken("FN_V_RRR",			L"global function returning vector with threee real arguments");

	gTokenMap.InsertToken("FN_VO_MR_MR",		L"global function with two 2D real arrays as arguments");
	gTokenMap.InsertToken("FN_VO_MAKEUNIT",		L"global function that makes unit matrix with one 2D real array as argument");

	gTokenMap.InsertToken("FN_B_S",				L"gobal function returning bool with one string argument");
	
	gTokenMap.InsertToken("TO_CONSOLE",			L"output to console");
	gTokenMap.InsertToken("FROM_CONSOLE",		L"input from console");
	
	gTokenMap.InsertToken("STMT_ELSEFALSE",		L"else_false clause");
	gTokenMap.InsertToken("STMT_ELSETRUE",		L"else_true clause");
	gTokenMap.InsertToken("STMT_ELSE",			L"else clause");

	gTokenMap.InsertToken("OPR_Z",				L"x component of vector");
	gTokenMap.InsertToken("OPR_Y",				L"y component of vector");
	gTokenMap.InsertToken("OPR_X",				L"z component of vector");

	gTokenMap.InsertToken("CPLX_OPR_IMAG",		L"imaginary part of complex");
	gTokenMap.InsertToken("CPLX_OPR_REAL",		L"real part of complex");

	gTokenMap.InsertToken("ARY_OPEN",			L"'{:'");
	gTokenMap.InsertToken("ARY_CLOSE",			L"':}'");
}

void KCH_EXP puterrormsg(const char* const msg[], int count)
{
	std::vector<std::wstring> tokens;
	for(int i=0; i<count; ++i)
	{
		if(i==0)
		{
			extern std::wstring gLastToken;
			
			std::wstring token = _T("\'")+gLastToken+_T("\'");
			std::wstring transtoken = gTokenMap.TranslateToken(msg[0]);
			
			if(STR_EQL_ASCII(msg[0],"TYPE_UNDF") || STR_EQL_ASCII(msg[0],"MEM_USER") ) 
				tokens.push_back(transtoken + _T(" ") + token);
			else tokens.push_back(transtoken);
		}
		else
			tokens.push_back(gTokenMap.TranslateToken(msg[i]));
	}

	std::wostringstream os;
	
	switch(count)
	{
	case 1: os<<_T("Unexpecting: ")<<tokens[0]; break;
		
	case 2: os<<_T("Unexpecting: ")<<tokens[0]<<_T(", Expecting: ")
			<<tokens[1]; break;
		
	case 3: os<<_T("Unexpecting: ")<<tokens[0]<<_T(", Expecting: ")
			<<tokens[1]<<_T(" or ")<<tokens[2]; break;
		
	case 4: os<<_T("Unexpecting: ")<<tokens[0]<<_T(", Expecting: ")
			<<tokens[1]<<_T(" or ")<<tokens[2]<<_T(" or ")<<tokens[3]; break;
		
	case 5: os<<_T("Unexpecting: ")<<tokens[0]<<_T(", Expecting: ")
			<<tokens[1]<<_T(" or ")<<tokens[2]<<_T(" or ")<<tokens[3]
			<<_T(" or ")<<tokens[4]; break;
	
	default: os<<_T("Unknown error");
	}
	
	os<<_T("\r\n"); HandleSyntaxError(os.str());
}

void TransKeywords::InsertKeyword(const wchar_t* keyword, KeywordID keyid)
{
	this->keywordmap.insert(KeywordPair(std::wstring(keyword), keyid));
}

KeywordID TransKeywords::GetKeywordID(const wchar_t* keyword)
{
	KeywordMapItr itr = this->keywordmap.find(std::wstring(keyword));
	
	if(itr==this->keywordmap.end()) // not found
		return keyword_null;
	else return itr->second;
}

KeywordID TransKeywords::GetKeywordID(const std::wstring& keyword)
{
	KeywordMapItr itr = this->keywordmap.find(keyword);
	
	if(itr==this->keywordmap.end()) // not found
		return keyword_null;
	else return itr->second;
}

const TransKeywords::KeywordMap& TransKeywords::GetKeywordMap() const
{
	return this->keywordmap;
}

extern "C" const TransKeywords::KeywordMap& GetKeywordMap()
{
	return gTransKeyword.GetKeywordMap();
}

extern "C" const TransKeywords::KeywordMap& GetMemberMap()
{
	return gTransOprType.GetKeywordMap();
}


KCH_EXP StrArray GetFileList(const CString& thePath, const CString& theFilter)
{
	StrArray filelist;
	HANDLE hFind;
	WIN32_FIND_DATA fd;
	CString strPath, fullPath;
	std::wstring fileName;

	if(thePath.Right(1) != _T("\\"))
		strPath = thePath + _T("\\");
	else strPath = thePath;

	fullPath = strPath + theFilter;
	
	if((hFind =::FindFirstFile(fullPath, &fd))!= INVALID_HANDLE_VALUE)
	{
		if(!(fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
		{
			fileName = std::wstring(strPath.GetString())+fd.cFileName;
			filelist.push_back(fileName);
		}

		while(::FindNextFile(hFind, &fd))
		{
			if(!(fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
			{
				fileName = std::wstring(strPath.GetString())+fd.cFileName;
				filelist.push_back(fileName);
			}
		}

		::FindClose(hFind);
	}
	return filelist;
}

extern  bool PrepareKeywordLexer(char* scanbuffer, size_t charcount);
extern  void DeleteKeywordBuffer(); extern void ResetKeywordLexer();
extern  int keywordparse();

extern "C" void InitializeKeywords(CEdit* pEditor, const CString& thePath)
{
	StrArray filelist = GetFileList(thePath, _T("*.keywords.yal"));
	int count = int(filelist.size());
	for(int i=0; i<count; ++i)
	{
		std::wstring contents; ResetKeywordLexer(); 
		if(LoadFileContents(filelist[i], contents))
		{
			long scanbuffer_size = long(contents.size()*2+2);
			AutoDeleteArray<char> scanbuffer(scanbuffer_size);
			::memcpy(scanbuffer.DataPtr, contents.c_str(), scanbuffer_size-2);
			
			scanbuffer.DataPtr[scanbuffer_size-2] = 
				scanbuffer.DataPtr[scanbuffer_size-1]='\0';

			if(PrepareKeywordLexer(scanbuffer.DataPtr, scanbuffer_size))
			{
				try
				{
					keywordparse(); 
				}
				catch(...)
				{
					; /* suppress any error messages */
				}
				
				DeleteKeywordBuffer();
			}
		}
	}
}

CString gLibPath, gLangPath, gAppPath, gWorkPath;

extern "C" void SetPathSettings(const CString& AppPath, const CString& LibPath, 
							 const CString &LangPath, const CString& WorkPath)
{
	gAppPath = AppPath; gLibPath = LibPath;
	gLangPath = LangPath; gWorkPath = WorkPath;
}
