#pragma once

class GarbageCollector
{
protected:
	typedef std::set<BaseType*> GarColType;
	typedef std::set<BaseType*>::iterator GarColItr;
	typedef std::pair<GarColItr, bool> LastElePair;


	LastElePair lastele;

	GarColType garcolglb;
	GarColType garcolstd;
	GarColType garcoltmp;
	GarColType *pGarcol;

	//bool which_one;
	
public:
		
	GCType which_one;

	GarbageCollector(): garcolglb(), garcolstd(), garcoltmp() 
	{
		which_one = GarColGbl; 
		pGarcol = &garcolglb;
		lastele.second = false;
	}

	void RemoveLastElement() 
	{ 
		if(lastele.second==false) return;
		pGarcol->erase(lastele.first);
		delete *(lastele.first);
	}
	
	void Erase(BaseType* ptr)
	{
		pGarcol->erase(ptr); delete ptr;
	}

	void ClearAll()
	{
		ClearGarcolGlb(); ClearGarcolStd(); ClearGarcolTmp(); 
	}

	GCType GetGarcolType() const { return which_one; }

	GCType SelectGarcol(GCType newtype)
	{
		GCType oldtype = which_one;
		if(oldtype == newtype) return oldtype;
		lastele.second = false;
		
		which_one = newtype;
		switch(newtype)
		{
		case GarColGbl:
			pGarcol = &garcolglb; break;
		case GarColStd:
			pGarcol = &garcolstd; break;
		case GarColTmp:
			pGarcol = &garcoltmp; break;
		}
		return oldtype;
	}

	// 1. Copies elements from source to target.
	// 2. Clears source elements.
	// 3. Selects target as current garcol.
	void InternalCopy(GCType target, GCType source)
	{
		if(target==source) return;

		GarColType *pSource, *pTarget;

		switch(target)
		{
		case GarColGbl: pTarget = &garcolglb; break;
		case GarColStd: pTarget = &garcolstd; break;
		case GarColTmp: pTarget = &garcoltmp; break;
		}

		switch(source)
		{
		case GarColGbl: pSource = &garcolglb; break;
		case GarColStd: pSource = &garcolstd; break;
		case GarColTmp: pSource = &garcoltmp; break;
		}

		for(GarColItr itr=pSource->begin(); 
			itr!=pSource->end(); ++itr) pTarget->insert(*itr);
		
		pSource->clear();

		pGarcol = pTarget; 
		which_one = target;
	}
	
	void ClearGarcolStd()
	{
		for(GarColItr itr = garcolstd.begin(); 
			itr != garcolstd.end(); ++itr)
			delete *itr;
		
		garcolstd.clear();
	}

	void ClearGarcolGlb()
	{
		for(GarColItr itr = garcolglb.begin(); 
			itr != garcolglb.end(); ++itr) delete *itr;
		garcolglb.clear();
	}

	void ClearGarcolTmp()
	{
		for(GarColItr itr = garcoltmp.begin(); 
			itr != garcoltmp.end(); ++itr) delete *itr;
		garcoltmp.clear();
	}

	void SelectGarcolGlb()
	{
		this->which_one = GarColGbl;
		this->pGarcol = &garcolglb;
		lastele.second = false;
	}

	void SelectGarcolStd()
	{
		this->which_one = GarColStd;
		this->pGarcol = &garcolstd;
		lastele.second = false;
	}

	void SelectGarcolTmp()
	{
		this->which_one = GarColTmp;
		this->pGarcol = &garcoltmp;
		lastele.second = false;
	}

	~GarbageCollector() { ClearAll(); }
	
	void InsertObject(BaseType *ptr)
	{
		lastele = pGarcol->insert(ptr); 
	}

	Literal<integer>* GetLiteral(integer v)
	{
		Literal<integer>* ptr = new Literal<integer>(v);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	Literal<string>* GetLiteral(const wchar_t* v)
	{
		Literal<string>* ptr = new Literal<string>(v);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	Literal<realnum>* GetLiteral(double v)
	{
		Literal<realnum>* ptr = new Literal<realnum>(v);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	Range* GetRange(ExpIntg* xpr1=NULL, ExpIntg* xpr2=NULL)
	{
		Range *ptr = new Range(xpr1, xpr2);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename rtn, typename vec>
	Expression<rtn>* GetSolveCramer(Expression<vec>* xpr)
	{
		SolveCramer<T, rtn, vec> *ptr = new SolveCramer<T, rtn, vec>(xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename vec>
	Expression<vec>* GetSolveGaussJordan(Expression<vec>* xpr)
	{
		SolveGaussJordan<T, vec> *ptr = new SolveGaussJordan<T, vec>(xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}
	
	StmtOutput* GetStmtOutput(Expression<stream&> *pstrm=NULL)
	{
		StmtOutput* ptr = new StmtOutput(pstrm);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	StmtInput* GetStmtInput(Expression<stream&> *pinpt=NULL)
	{
		StmtInput* ptr = new StmtInput(pinpt);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename vec, typename ary>
	Expression<vec>* GetSolveLUDecompo(Expression<vec>* xpr, VarType vt)
	{
		if(vt.dPtr)
		{
			ary* pAry = dynamic_cast<ary*>(vt.dPtr);

			if(pAry)
			{
				SolveLUDecompo<T, vec, ary> *ptr = new SolveLUDecompo<T, vec, ary>(xpr, pAry);
				lastele = pGarcol->insert(ptr); return ptr;
			}
			
			HandleSyntaxError(_T("Array type mismatch.\r\n"));
		}
		
		HandleSyntaxError(_T("It should not happened.\r\n"));
		
		return NULL; // to suppress warning
	}

	template<typename T, typename vec>
	Expression<vec>* GetSolveInverse(Expression<vec>* xpr)
	{
		SolveInverse<T, vec> *ptr = new SolveInverse<T, vec>(xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename rtn, typename vec>
	Expression<rtn>* GetSolveLUMethod(Expression<vec>* xpr)
	{
		SolveLUMethod<T, rtn, vec> *ptr = new SolveLUMethod<T, rtn, vec>(xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename vec>
	Expression<vec>* GetClsComplement(Expression<vec>* xpr, 
		ExpIntg* index1)
	{
		ClsComplement1<T, vec> *ptr = new ClsComplement1<T, vec>(xpr, index1);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename vec>
	Expression<vec>* GetClsComplement(Expression<vec>* xpr, 
		ExpIntg* index1, ExpIntg* index2)
	{
		ClsComplement2<T, vec> *ptr = new ClsComplement2<T, vec>(xpr, index1, index2);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename vec>
	Expression<vec>* GetClsComplement(Expression<vec>* xpr, 
		ExpIntg* index1, ExpIntg* index2, ExpIntg* index3)
	{
		ClsComplement3<T, vec> *ptr = new ClsComplement3<T, vec>(xpr, index1, index2, index3);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename vec>
	Expression<vec>* GetClsComplement(Expression<vec>* xpr, 
		ExpIntg* index1, ExpIntg* index2,
		ExpIntg* index3, ExpIntg* index4)
	{
		ClsComplement4<T, vec> *ptr = new ClsComplement4<T, vec>(xpr, index1, index2, index3, index4);
		lastele = pGarcol->insert(ptr); return ptr;
	}

		template<typename T, typename vec>
	Expression<vec>* GetClsComplement(Expression<vec>* xpr, 
		ExpIntg* index1, ExpIntg* index2,
		ExpIntg* index3, ExpIntg* index4, ExpIntg* index5)
	{
		ClsComplement5<T, vec> *ptr = new ClsComplement5<T, vec>(xpr, index1, index2, index3, index4, index5);
		lastele = pGarcol->insert(ptr); return ptr;
	}
	
	template<typename T, typename vec>
	Expression<vec>* GetClsSubArray(Expression<vec>* xpr, Range *rng1, bool comp)
	{
		ClsSubArray1<T, vec> *ptr = new ClsSubArray1<T, vec>(xpr, rng1, comp);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename vec>
	Expression<vec>* GetClsSubArray(Expression<vec>* xpr, Range *rng1, Range *rng2, bool comp)
	{
		ClsSubArray2<T, vec> *ptr = new ClsSubArray2<T, vec>(xpr, rng1, rng2, comp);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename vec>
	Expression<vec>* GetClsSubArray(Expression<vec>* xpr, 
		Range *rng1, Range *rng2, Range *rng3, bool comp)
	{
		ClsSubArray3<T, vec> *ptr = new ClsSubArray3<T, vec>(xpr, rng1, rng2, rng3, comp);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename vec>
	Expression<vec>* GetClsSubArray(Expression<vec>* xpr, 
		Range *rng1, Range *rng2, Range *rng3, Range *rng4, bool comp)
	{
		ClsSubArray4<T, vec> *ptr = new ClsSubArray4<T, vec>(xpr, rng1, rng2, rng3, rng4, comp);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename vec>
	Expression<vec>* GetClsSubArray(Expression<vec>* xpr, 
		Range *rng1, Range *rng2, Range *rng3, Range *rng4, Range *rng5, bool comp)
	{
		ClsSubArray5<T, vec> *ptr = new ClsSubArray5<T, vec>(xpr, rng1, rng2, rng3, rng4, rng5, comp);
		lastele = pGarcol->insert(ptr); return ptr;
	}
	
	Literal<bool>* GetLiteral(bool v)
	{
		Literal<bool>* ptr = new Literal<bool>(v);
		lastele = pGarcol->insert(ptr); return ptr;
	}


	Literal<Complex>* GetLiteral(ExpReal* l, ExpReal* r)
	{
		Literal<Complex>* ptr = new Literal<Complex>(l, r);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	Literal<Vector>* GetLiteral(ExpReal* i,
		ExpReal* j, ExpReal* k)
	{
		Literal<Vector>* ptr = new Literal<Vector>(i, j, k);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename vec, typename ary>
	LeftHandSide<T>* GetVarAry(LeftHandSide<vec>* pLHS, ExpIntg* index)
	{
		VaArray<vec, ary>* vaPtr = dynamic_cast<VaArray<vec, ary>*>(pLHS);

		if(vaPtr)
		{
			Array1D<T> *aPtr = vaPtr->GetDataPtr();
			VarAry1D<T>* ptr = new VarAry1D<T>(index, aPtr);
			lastele = pGarcol->insert(ptr); return ptr;	
		}
		else
			HandleSyntaxError(L"Invalid array type.\r\n");
		
		return NULL; // to suppress warning
	}

	template<typename T, typename vec, typename ary>
	LeftHandSide<T>* GetVarAry(LeftHandSide<vec>* pLHS,
		ExpIntg* index1, ExpIntg* index2)
	{
		VaArray<vec, ary>* vaPtr = dynamic_cast<VaArray<vec, ary>*>(pLHS);

		if(vaPtr)
		{
			Array2D<T> *aPtr = vaPtr->GetDataPtr();
			VarAry2D<T>* ptr = new VarAry2D<T>(index1, index2, aPtr);
			lastele = pGarcol->insert(ptr); return ptr;	
		}
		else
			HandleSyntaxError(L"Invalid array type.\r\n");
		
		return NULL; // to suppress warning
	}
	
	template<typename T, typename vec, typename ary>
	LeftHandSide<T>* GetVarAry(LeftHandSide<vec>* pLHS,
		ExpIntg* index1, 
		ExpIntg* index2, ExpIntg* index3)
	{
		VaArray<vec, ary>* vaPtr = dynamic_cast<VaArray<vec, ary>*>(pLHS);

		if(vaPtr)
		{
			Array3D<T> *aPtr = vaPtr->GetDataPtr();
			VarAry3D<T>* ptr = new VarAry3D<T>(index1, index2, index3, aPtr);
			lastele = pGarcol->insert(ptr); return ptr;	
		}
		else
			HandleSyntaxError(L"Invalid array type.\r\n");
		
		return NULL; // to suppress warning
	}

	template<typename T, typename vec, typename ary>
	LeftHandSide<T>* GetVarAry(LeftHandSide<vec>* pLHS,
		ExpIntg* index1, ExpIntg* index2, 
		ExpIntg* index3, ExpIntg* index4)
	{
		VaArray<vec, ary>* vaPtr = dynamic_cast<VaArray<vec, ary>*>(pLHS);

		if(vaPtr)
		{
			Array4D<T> *aPtr = vaPtr->GetDataPtr();
			VarAry4D<T>* ptr = new VarAry4D<T>(index1, index2, index3, index4, aPtr);
			lastele = pGarcol->insert(ptr); return ptr;	
		}
		else
			HandleSyntaxError(L"Invalid array type.\r\n");
		
		return NULL; // to suppress warning
	}

	template<typename T, typename vec, typename ary>
	LeftHandSide<T>* GetVarAry(LeftHandSide<vec>* pLHS,
		ExpIntg* index1, ExpIntg* index2, 
		ExpIntg* index3, ExpIntg* index4, ExpIntg* index5)
	{
		VaArray<vec, ary>* vaPtr = dynamic_cast<VaArray<vec, ary>*>(pLHS);

		if(vaPtr)
		{
			Array5D<T> *aPtr = vaPtr->GetDataPtr();
			VarAry5D<T>* ptr = new VarAry5D<T>(index1, index2, index3, index4, index5, aPtr);
			lastele = pGarcol->insert(ptr); return ptr;	
		}
		else
			HandleSyntaxError(L"Invalid array type.\r\n");
		
		return NULL; // to suppress warning
	}

	template<typename T, typename vec>
	LeftHandSide<T>* GetVarAryEle(LeftHandSide<vec>* pLHS, ExpIntg* idx1)
	{
		VarAryEle1D<T, vec>* ptr = new VarAryEle1D<T, vec>(pLHS, idx1);
		lastele = pGarcol->insert(ptr); return ptr;	
	}

	template<typename T, typename vec>
	LeftHandSide<T>* GetVarAryEle(LeftHandSide<vec>* pLHS, ExpIntg* idx1, ExpIntg* idx2)
	{
		VarAryEle2D<T, vec>* ptr = new VarAryEle2D<T, vec>(pLHS, idx1, idx2);
		lastele = pGarcol->insert(ptr); return ptr;	
	}

	template<typename T, typename vec>
	LeftHandSide<T>* GetVarAryEle(LeftHandSide<vec>* pLHS, ExpIntg* idx1, ExpIntg* idx2, ExpIntg* idx3)
	{
		VarAryEle3D<T, vec>* ptr = new VarAryEle3D<T, vec>(pLHS, idx1, idx2, idx3);
		lastele = pGarcol->insert(ptr); return ptr;	
	}

	template<typename T, typename vec>
	LeftHandSide<T>* GetVarAryEle(LeftHandSide<vec>* pLHS, ExpIntg* idx1, ExpIntg* idx2, ExpIntg* idx3, ExpIntg* idx4)
	{
		VarAryEle4D<T, vec>* ptr = new VarAryEle4D<T, vec>(pLHS, idx1, idx2, idx3, idx4);
		lastele = pGarcol->insert(ptr); return ptr;	
	}

	template<typename T, typename vec>
	LeftHandSide<T>* GetVarAryEle(LeftHandSide<vec>* pLHS, ExpIntg* idx1, ExpIntg* idx2, ExpIntg* idx3, ExpIntg* idx4, ExpIntg* idx5)
	{
		VarAryEle5D<T, vec>* ptr = new VarAryEle5D<T, vec>(pLHS, idx1, idx2, idx3, idx4, idx5);
		lastele = pGarcol->insert(ptr); return ptr;	
	}

	template<typename T>
	VarAryUdt1D<T>* GetVarAryUdt(VarUser<ArrayU1D<T> > *aptr, ExpIntg* index)
	{
		VarAryUdt1D<T> *ptr = new VarAryUdt1D<T>(aptr, index);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	VarAryUdt2D<T>* GetVarAryUdt(VarUser<ArrayU2D<T> > *aptr, 
		ExpIntg *index1, ExpIntg *index2)
	{
		VarAryUdt2D<T> *ptr = new VarAryUdt2D<T>(aptr, index1, index2);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	VarAryUdt3D<T>* GetVarAryUdt(VarUser<ArrayU3D<T> > *aptr, 
		ExpIntg *index1, ExpIntg *index2, ExpIntg *index3)
	{
		VarAryUdt3D<T> *ptr = new VarAryUdt3D<T>(aptr, index1, index2, index3);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	VarAryUdt4D<T>* GetVarAryUdt(VarUser<ArrayU4D<T> > *aptr, 
		ExpIntg *index1, ExpIntg *index2, 
		ExpIntg *index3, ExpIntg *index4)
	{
		VarAryUdt4D<T> *ptr = new VarAryUdt4D<T>(aptr, index1, index2, index3, index4);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	VarAryUdt5D<T>* GetVarAryUdt(VarUser<ArrayU5D<T> > *aptr, 
		ExpIntg *index1, ExpIntg *index2, 
		ExpIntg *index3, ExpIntg *index4, ExpIntg *index5)
	{
		VarAryUdt5D<T> *ptr = new VarAryUdt5D<T>(aptr, index1, index2, index3, index4, index5);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	VarAryVub1D<T>* GetVarAryVub(VarMem<ArrayU1D<T> > *aptr, ExpIntg* index)
	{
		VarAryVub1D<T> *ptr = new VarAryVub1D<T>(aptr, index);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	VarAryVub2D<T>* GetVarAryVub(VarMem<ArrayU2D<T> > *aptr, 
		ExpIntg *index1, ExpIntg *index2)
	{
		VarAryVub2D<T> *ptr = new VarAryVub2D<T>(aptr, index1, index2);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	VarAryVub3D<T>* GetVarAryVub(VarMem<ArrayU3D<T> > *aptr, 
		ExpIntg *index1, ExpIntg *index2, ExpIntg *index3)
	{
		VarAryVub3D<T> *ptr = new VarAryVub3D<T>(aptr, index1, index2, index3);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	VarAryVub4D<T>* GetVarAryVub(VarMem<ArrayU4D<T> > *aptr, 
		ExpIntg *index1, ExpIntg *index2, 
		ExpIntg *index3, ExpIntg *index4)
	{
		VarAryVub4D<T> *ptr = new VarAryVub4D<T>(aptr, index1, index2, index3, index4);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	VarAryVub5D<T>* GetVarAryVub(VarMem<ArrayU5D<T> > *aptr, 
		ExpIntg *index1, ExpIntg *index2, 
		ExpIntg *index3, ExpIntg *index4, ExpIntg *index5)
	{
		VarAryVub5D<T> *ptr = new VarAryVub5D<T>(aptr, index1, index2, index3, index4, index5);
		lastele = pGarcol->insert(ptr); return ptr;
	}


	template<typename vec, typename ary>
	VaArray<vec, ary>* GetVaArray(VarType vt)
	{
		if(vt.dPtr)
		{
			ary* ptr = dynamic_cast<ary*>(vt.dPtr);

			if(ptr)
			{
				VaArray<vec, ary>* vPtr = new VaArray<vec, ary>(ptr);
				lastele = pGarcol->insert(vPtr); return vPtr;
			}
			else
			{
				std::wostringstream os;
				os<<_T("Array type mismatch. Expecting array of type ")<<typeid(ary).name()<<_T("\r\n");
				HandleSyntaxError(os.str());
			}
		}
		
		HandleSyntaxError(_T("It should not happened.\r\n"));

		return NULL; // to suppress warning
	}

	template<typename vec, typename ary>
	VaArray<vec, ary>* GetVaArray(ary *pArray)
	{
		VaArray<vec, ary>* ptr = new VaArray<vec, ary>(pArray);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename ary, typename vec>
	Expression<vec>* GetCallArrayExpr1D(VarType vt, Statement *pStmt=NULL)
	{
		if(vt.dPtr)
		{
			ary* ptr = dynamic_cast<ary*>(vt.dPtr);

			if(ptr)
			{
				CallArrayExpr1D<T, VarAry1D<T>, ary, vec>* vPtr = 
					new CallArrayExpr1D<T, VarAry1D<T>, ary, vec>(vt.bufr, ptr, pStmt);

				lastele = pGarcol->insert(vPtr); return vPtr;
			}
			else
			{
				std::wostringstream os;
				os<<_T("Array type mismatch. Expecting array of type ")<<typeid(ary).name()<<_T("\r\n");
				HandleSyntaxError(os.str());

				return NULL; // to suppress warning
			}
		}
		
		HandleSyntaxError(_T("It should not happened."));

		return NULL; // to suppress warning
	}

	template<typename T, typename ary, typename vec>
	Expression<vec>* GetCallArrayExpr2D(VarType vt, Statement *pStmt=NULL)
	{
		if(vt.dPtr)
		{
			ary* ptr = dynamic_cast<ary*>(vt.dPtr);

			if(ptr)
			{
				CallArrayExpr2D<T, VarAry2D<T>, ary, vec>* vPtr = 
					new CallArrayExpr2D<T, VarAry2D<T>, ary, vec>(vt.bufr, ptr, pStmt);

				lastele = pGarcol->insert(vPtr); return vPtr;
			}
			else
			{
				std::wostringstream os;
				os<<_T("Array type mismatch. Expecting array of type ")<<typeid(ary).name()<<_T("\r\n");
				HandleSyntaxError(os.str());
				return NULL; // to suppress warning
			}
		}
		
		HandleSyntaxError(_T("It should not happened.\r\n"));

		return NULL; // to suppress warning
	}

	template<typename T, typename ary, typename vec>
	Expression<vec>* GetCallArrayExpr3D(VarType vt, Statement *pStmt=NULL)
	{
		if(vt.dPtr)
		{
			ary* ptr = dynamic_cast<ary*>(vt.dPtr);

			if(ptr)
			{
				CallArrayExpr3D<T, VarAry3D<T>, ary, vec>* vPtr = 
					new CallArrayExpr3D<T, VarAry3D<T>, ary, vec>(vt.bufr, ptr, pStmt);

				lastele = pGarcol->insert(vPtr); return vPtr;
			}
			else
			{
				std::wostringstream os;
				os<<_T("Array type mismatch. Expecting array of type ")<<typeid(ary).name()<<_T("\r\n");
				HandleSyntaxError(os.str());

				return NULL; // to suppress warning
			}
		}
		else HandleSyntaxError(_T("It should not happened.\r\n"));

		return NULL; // to suppress warning
	}

	template<typename T, typename ary, typename vec>
	Expression<vec>* GetCallArrayExpr4D(VarType vt, Statement *pStmt=NULL)
	{
		if(vt.dPtr)
		{
			ary* ptr = dynamic_cast<ary*>(vt.dPtr);

			if(ptr)
			{
				CallArrayExpr4D<T, VarAry4D<T>, ary, vec>* vPtr = 
					new CallArrayExpr4D<T, VarAry4D<T>, ary, vec>(vt.bufr, ptr, pStmt);

				lastele = pGarcol->insert(vPtr); return vPtr;
			}
			else
			{
				std::wostringstream os;
				os<<_T("Array type mismatch. Expecting array of type ")<<typeid(ary).name()<<_T("\r\n");
				HandleSyntaxError(os.str());
				return NULL; // to suppress warning
			}
		}
		else HandleSyntaxError(_T("It should not happened.\r\n"));

		return NULL; // to suppress warning
	}

	template<typename T, typename ary, typename vec>
	Expression<vec>* GetCallArrayExpr5D(VarType vt, Statement *pStmt=NULL)
	{
		if(vt.dPtr)
		{
			ary* ptr = dynamic_cast<ary*>(vt.dPtr);

			if(ptr)
			{
				CallArrayExpr5D<T, VarAry5D<T>, ary, vec>* vPtr = 
					new CallArrayExpr5D<T, VarAry5D<T>, ary, vec>(vt.bufr, ptr, pStmt);

				lastele = pGarcol->insert(vPtr); return vPtr;
			}
			else
			{
				std::wostringstream os;
				os<<_T("Array type mismatch. Expecting array of type ")<<typeid(ary).name()<<_T("\r\n");
				HandleSyntaxError(os.str());

				return NULL; // to suppress warning
			}
		}
		else HandleSyntaxError(_T("It should not happened.\r\n"));

		return NULL; // to suppress warning.
	}
	
	template<typename T, typename vec>
	Expression<vec>* GetVaAryBin(Expression<vec> *vXpr, Expression<T> *eXpr, char opr)
	{
		VaAryBin<T, vec>* ptr = new VaAryBin<T, vec>(vXpr, eXpr, opr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename vec>
	Expression<vec>* GetVaAryBin(Expression<T> *eXpr, Expression<vec> *vXpr, char opr)
	{
		
		VaAryBin<T, vec>* ptr = new VaAryBin<T, vec>(eXpr, vXpr, opr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename vec>
	Expression<vec>* GetVaVRBin(ExpReal *eXpr, Expression<vec> *vXpr, char opr)
	{
		VaVRBin<vec>* ptr = new VaVRBin<vec>(eXpr, vXpr, opr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename vec>
	Expression<vec>* GetVaVRBin(Expression<vec> *vXpr, ExpReal *eXpr, char opr)
	{
		VaVRBin<vec>* ptr = new VaVRBin<vec>(vXpr, eXpr, opr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename vec>
	Expression<vec>* GetArrayBin(Expression<vec> *xpr1, Expression<vec> *xpr2, char opr)
	{
		ArrayBin<vec>* ptr = new ArrayBin<vec>(xpr1, xpr2, opr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename tgt, typename src>
	Expression<tgt>* GetDotProduct(Expression<src> *xpr1, Expression<src> *xpr2)
	{
		DotProduct<tgt, src>* ptr = new DotProduct<tgt, src>(xpr1, xpr2);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename tgt, typename src>
	Expression<tgt>* GetSDotProduct(Expression<src> *vaPtr, ExpVctr *xPtr)
	{
		SDotProduct<tgt, src>* ptr = new SDotProduct<tgt, src>(vaPtr, xPtr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename tgt, typename src>
	Expression<tgt>* GetSDotProduct(ExpVctr *xPtr, Expression<src> *vaPtr)
	{
		SDotProduct<tgt, src>* ptr = new SDotProduct<tgt, src>(xPtr, vaPtr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename vec>
	Expression<vec>* GetArrayBinVec(Expression<vec> *xpr1, Expression<vec> *xpr2, char opr)
	{
		ArrayBinVec<vec>* ptr = new ArrayBinVec<vec>(xpr1, xpr2, opr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename vec>
	Expression<vec>* GetMatrixPro(Expression<vec> *xpr1, Expression<vec> *xpr2)
	{
		MatrixPro<T, vec>* ptr = new MatrixPro<T, vec>(xpr1, xpr2);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	DecAry1D<T>* GetDecAry1D(const wchar_t* vrid, ExpIntg* size=NULL)
	{
		DecAry1D<T>* ptr = new DecAry1D<T>(vrid, size, static_set);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	DecAry2D<T>* GetDecAry2D(const wchar_t* vrid, 
		ExpIntg* size1=NULL, ExpIntg* size2=NULL)
	{
		DecAry2D<T>* ptr = new DecAry2D<T>(vrid, size1, size2, static_set);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	DecAry3D<T>* GetDecAry3D(const wchar_t* vrid, ExpIntg* size1 = NULL,
		ExpIntg* size2 = NULL, ExpIntg* size3 = NULL)
	{
		DecAry3D<T>* ptr = new DecAry3D<T>(vrid, size1, size2, size3, static_set);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	DecAry4D<T>* GetDecAry4D(const wchar_t* vrid, 
		ExpIntg* size1 = NULL, ExpIntg* size2 = NULL, 
		ExpIntg* size3 = NULL, ExpIntg* size4 = NULL)
	{
		DecAry4D<T>* ptr = new DecAry4D<T>(vrid, size1, size2, size3, size4, static_set);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	DecAry5D<T>* GetDecAry5D(const wchar_t* vrid, ExpIntg* size1 = NULL,
		ExpIntg* size2 = NULL, ExpIntg* size3 = NULL,
		ExpIntg* size4 = NULL, ExpIntg* size5 = NULL)
	{
		DecAry5D<T>* ptr = new DecAry5D<T>(vrid, size1, size2, size3, size4, size5, static_set);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	LeftHandSide<T>* GetVariable(VarType vt)
	{
		Variable<T>* ptr = new Variable<T>(vt);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	LeftHandSide<T>* GetVariable(Declare<T> *pDecl)
	{
		Variable<T>* ptr = new Variable<T>(pDecl->GetVarPtr());
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	LeftHandSide<T>* GetPtrToVar(LeftHandSide<T*>* xpr)
	{
		PtrToVar<T>* ptr = new PtrToVar<T>(xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	LeftHandSide<T>* GetPtrExpToVar(Expression<T*>* xpr)
	{
		PtrExpToVar<T>* ptr = new PtrExpToVar<T>(xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	LeftHandSide<UserData>* GetPtrExpToVarUserData(UdtPtrBase *xpr)
	{
		PtrExpToVar<UserData>* ptr = new PtrExpToVar<UserData>(xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T> 
	AssignVarToPtr<T>* GetAssignVarToPtr(LeftHandSide<T*> *pLHS, LeftHandSide<T> *pRHS)
	{
		extern bool assign_set;  assign_set = false;
		AssignVarToPtr<T>* ptr = new AssignVarToPtr<T>(pLHS, pRHS);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	AssignVarToPtr<UserData>* GetAssignVarToPtrUserData(UdtPtrBase *upb, LeftHandSide<UserData> *pRHS)
	{
		extern bool assign_set;  assign_set = false;
		extern Stack<UserDefinedType*> gUdtFactoryStack;
		AssignVarToPtr<UserData>* ptr = new AssignVarToPtr<UserData>(upb, pRHS);
		lastele = pGarcol->insert(ptr); gUdtFactoryStack.clear(); return ptr;
	}

	template<typename T> 
	AssignExpPtrToPtr<T>* GetAssignExpPtrToPtr(LeftHandSide<T*> *pLHS, Expression<T*> *pRHS)
	{
		extern bool assign_set;  assign_set = false;
		
		AssignExpPtrToPtr<T>* ptr = new AssignExpPtrToPtr<T>(pLHS, pRHS);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	AssignExpPtrToPtr<UserData>* GetAssignExpPtrToPtrUserData(UdtPtrBase *pLHS, Expression<UserData*> *pRHS)
	{
		extern bool assign_set;  assign_set = false;
		extern Stack<UserDefinedType*> gUdtFactoryStack;
		AssignExpPtrToPtr<UserData>* ptr = new AssignExpPtrToPtr<UserData>(pLHS, pRHS);
		lastele = pGarcol->insert(ptr); gUdtFactoryStack.clear(); return ptr;
	}

	template<typename T>
	LeftHandSide<T*>* GetPtrVariable(PtrData<T>* pData)
	{
		PtrVariable<T>* ptr = new PtrVariable<T>(pData);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<>
	LeftHandSide<stream&>* GetVariable<stream&>(VarType vt)
	{
		Variable<stream>* ptr = new Variable<stream>(vt);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	LeftHandSide<realnum>* GetCplxLHSReal(LeftHandSide<Complex>* pLHS)
	{
		CplxLHSReal *ptr = new CplxLHSReal(pLHS);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	LeftHandSide<realnum>* GetCplxLHSImag(LeftHandSide<Complex>* pLHS)
	{
		CplxLHSImag *ptr = new CplxLHSImag(pLHS);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	ExpReal* GetCplxRHSReal(ExpCplx* pXpr)
	{
		CplxRHSReal *ptr = new CplxRHSReal(pXpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	ExpReal* GetCplxRHSImag(ExpCplx* pXpr)
	{
		CplxRHSImag *ptr = new CplxRHSImag(pXpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	LeftHandSide<realnum>* GetVctrLHSX(LeftHandSide<Vector>* pLHS)
	{
		VctrLHSX *ptr = new VctrLHSX(pLHS);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	LeftHandSide<realnum>* GetVctrLHSY(LeftHandSide<Vector>* pLHS)
	{
		VctrLHSY *ptr = new VctrLHSY(pLHS);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	LeftHandSide<realnum>* GetVctrLHSZ(LeftHandSide<Vector>* pLHS)
	{
		VctrLHSZ *ptr = new VctrLHSZ(pLHS);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	ExpReal* GetVctrRHSX(ExpVctr* pXpr)
	{
		VctrRHSX *ptr = new VctrRHSX(pXpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	ExpReal* GetVctrRHSY(ExpVctr* pXpr)
	{
		VctrRHSY *ptr = new VctrRHSY(pXpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	ExpReal* GetVctrRHSZ(ExpVctr* pXpr)
	{
		VctrRHSZ *ptr = new VctrRHSZ(pXpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	VarUser<T>* GetVarUser(VarUserBase *vub)
	{
		extern Stack<UserDefinedType*> gUdtFactoryStack;
		VarUser<T>* ptr = new VarUser<T>(vub);
		lastele = pGarcol->insert(ptr); 
		gUdtFactoryStack.clear();  return ptr;
	}

	template<typename T>
	VarUserPtr<T>* GetVarUserPtr(UdtPtrBase *vub)
	{
		extern Stack<UserDefinedType*> gUdtFactoryStack;
		VarUserPtr<T>* ptr = new VarUserPtr<T>(vub);
		lastele = pGarcol->insert(ptr); 
		gUdtFactoryStack.clear();  return ptr;
	}

	template<typename T>
	VarMem<T>* GetVarMem(VarMemBase *vmb)
	{
		extern Stack<UserDefinedType*> gFactoryStack;
		extern Stack<ReturnSig> gReturnStack;

		if(gFactoryStack.empty()) 
	  		HandleSyntaxError(_T("Non literal member can not be accessed in this context.\r\n")); 

		if(!gReturnStack.empty() && gReturnStack.top().fptr->NotUseMember)
			HandleSyntaxError(L"Non literal member CAN NOT be accessed in the static member funcion.\r\n");
		
		VarMem<T>* ptr = new VarMem<T>(vmb);
		lastele = pGarcol->insert(ptr); 
		gUdtFactoryStack.clear();  return ptr;
	}

	LeftHandSide<UserData>* GetVarMemExp(VarMemBase *vmb)
	{

		extern Stack<ReturnSig> gReturnStack;
		extern Stack<UserDefinedType*> gFactoryStack;

		if(gFactoryStack.empty()) 
	  		HandleSyntaxError(_T("Non literal member can not be accessed in this context.\r\n")); 

		if(!gReturnStack.empty() && gReturnStack.top().fptr->NotUseMember)
			HandleSyntaxError(L"Non literal member CAN NOT be accessed in the static member funcion.\r\n");

		VarMemExp* ptr = new VarMemExp(vmb);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	LeftHandSide<UserData>* GetVarUserExp(VarUserBase *vub)
	{
		VarUserExp* ptr = new VarUserExp(vub);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	LeftHandSide<UserData>* GetUdtPtrExp(UdtPtrBase *vub)
	{
		UdtPtrExp* ptr = new UdtPtrExp(vub);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename ary>
	VarMemAry<T, ary>* GetVarMemAry(VarMem<ary> *vptr)
	{
		VarMemAry<T, ary>* ptr = new VarMemAry<T, ary>(vptr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename ary>
	VarUserAry<T, ary>* GetVarUserAry(VarUser<ary> *vptr)
	{
		VarUserAry<T, ary>* ptr = new VarUserAry<T, ary>(vptr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	VarUserBase* GetVarUserBase(VarType vt)
	{
		UserData *udptr = dynamic_cast<UserData*>(vt.dPtr);
		if(udptr==0)
			HandleSyntaxError(L"Invalid user defined variable type.\r\n");

		VarUserBase* ptr = new VarUserBase(udptr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	VarUserBase* GetVarUserBase(UserData *udptr)
	{
		VarUserBase* ptr = new VarUserBase(udptr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	UdtPtrBase* GetUdtPtrBase(PtrData<UserData>* uptr)
	{
		UdtPtrBase* ptr = new UdtPtrBase(uptr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	UdtPtrBase* GetVarMemUPtrData(VarMem<UPtrData> *uptr)
	{
		VarMemUPtrData* ptr = new VarMemUPtrData(uptr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	UdtPtrBase* GetUserMemUPtrData(VarUser<UPtrData> *uptr)
	{
		UserMemUPtrData* ptr = new UserMemUPtrData(uptr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	UdtPtrBase* GetMemUPtrData(UdtPtrBase *uptr)
	{
		MemUPtrData* ptr = new MemUPtrData(uptr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	VarMemBase* GetVarMemBase(UserDefinedType *udtptr, Function *fPtr=NULL, VarUserBase *vub=NULL)
	{
		VarMemBase* ptr = new VarMemBase(udtptr, fPtr, vub);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename U>
	Expression<T>* GetIncDecVar(VarType vt, bool binc)
	{
		IncDecVar<T,U>* ptr = new IncDecVar<T,U>(vt, binc);
		lastele = pGarcol->insert(ptr); return ptr;
	}
	

	template<typename U>
	U*  GetPrimitive(std::string id="")
	{
		U* ptr = new U(id);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename U>
	Data* GetSimpleData(std::string id, bool varstatic=false)
	{
		U* ptr = new U(id, varstatic);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	Data* GetArrayData(std::string id, int dim, bool varstatic=false)
	{
		switch(dim)
		{
		case 1:
			Array1D<T>* ptr = new Array1D<T>(id, varstatic); break;
		case 2:
			Array2D<T>* ptr = new Array3D<T>(id, varstatic); break;
		case 3:
			Array3D<T>* ptr = new Array3D<T>(id, varstatic); break;
		default:
			HandleSyntaxError(_T("Unsupported Array Dimension.\r\n"));
		}

		lastele = pGarcol->insert(ptr); return ptr;
	}
	
	Factorial* GetFactorial(ExpIntg* xpr)
	{
		Factorial *ptr = new Factorial(xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	Absolute* GetAbs(ExpIntg* xpr)
	{
		Absolute *ptr = new Absolute(xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	Permutation* GetPermutation(ExpIntg* n, ExpIntg* r)
	{
		Permutation *ptr = new Permutation(n, r);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	Combination* GetCombination(ExpIntg* n, ExpIntg* r)
	{
		Combination *ptr = new Combination(n, r);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	Gamma* GetGamma(ExpReal* xpr)
	{
		Gamma *ptr = new Gamma(xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	IntToReal* GetI2R(ExpIntg* xpr)
	{
		IntToReal* ptr = new IntToReal(xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	IntToCplx* GetI2C(ExpIntg* xpr)
	{
		IntToCplx* ptr = new IntToCplx(xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	RealToInt* GetR2I(ExpReal* xpr)
	{
		RealToInt* ptr = new RealToInt(xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	RealToCplx* GetR2C(ExpReal* xpr)
	{
		RealToCplx* ptr = new RealToCplx(xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	RealToVctr* GetR2V(ExpReal* xpr)
	{
		RealToVctr* ptr = new RealToVctr(xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	IntgToVctr* GetI2V(ExpIntg* xpr)
	{
		IntgToVctr* ptr = new IntgToVctr(xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	CallFunc<T>* GetFunCall(Function* fptr, VarUserBase *vptr=NULL, VarMemBase *uptr=NULL, UdtPtrBase *bptr=NULL)
	{
		extern bool arg_set;
		extern Stack<ReturnSig> gReturnStack;
		extern Stack<UserDefinedType*> gFactoryStack;
		extern Stack<ScanFile> gScanFileStack;

		if(gFactoryStack.empty()) // call a function outside of a member function
		{
			if(uptr && fptr->NotUseMember==false) // not callable
				HandleSyntaxError(L"Can not call non static member fuction.\r\n");
		}
		else // call a function in a member function
		{
			if(gReturnStack.top().fptr->NotUseMember 
				&& fptr->NotUseMember==false)
			{
				if(vptr==NULL)
					HandleSyntaxError(L"Static member fuction CAN NOT call non static member function.\r\n");
			}
		}
		
		CallFunc<T>* ptr = 
			new CallFunc<T>(fptr->fname.c_str(), fptr, vptr, uptr, bptr);
		ptr->scanfile = gScanFileStack.top(); 
		
		lastele = pGarcol->insert(ptr); arg_set = true; return ptr;
	}

	template<typename T, typename U>
	Expression<T>* GetCallExpr(VarType vt, Statement* stmt)
	{
		extern Statement* FindAssignment(const std::wstring& id);

		Statement* pAsn = FindAssignment(vt.bufr);

		Assignment<T, U>* asgn;
		if(pAsn) asgn = dynamic_cast<Assignment<T, U>*>(pAsn);
		else HandleSyntaxError(_T("Assignment not found.\r\n"));
		
		if(asgn)
		{
			CallExpr<T, U>* ptr = new CallExpr<T, U>(vt, asgn, stmt);
			lastele = pGarcol->insert(ptr); return ptr;
		}
		else
			HandleSyntaxError(_T("Assignment statement not found.\r\n"));

		return NULL; // to suppress warning
	}

	template<typename T>
	Expression<T>* GetCallExpression(LeftHandSide<T> *lhs, Statement* stmt=NULL)
	{
		extern Stack<ScanFile> gScanFileStack;
		
		CallExpression<T>* ptr = new CallExpression<T>(lhs, stmt);
		ptr->scanfile = gScanFileStack.top(); 
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename vec>
	Expression<vec>* GetCallExprVec(LeftHandSideArray<vec> *lhs, Statement* stmt=NULL)
	{
		extern Stack<ScanFile> gScanFileStack;
		
		CallExprVec<vec>* ptr = new CallExprVec<vec>(lhs, stmt);
		ptr->scanfile = gScanFileStack.top(); 
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	Expression<T>* GetAssignExpression(LeftHandSide<T> *lhs, Statement* stmt=NULL)
	{
		extern Stack<ScanFile> gScanFileStack;
		
		AssignExpression<T>* ptr = new AssignExpression<T>(lhs, stmt);
		ptr->scanfile = gScanFileStack.top(); 
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename U>
	Expression<T>* GetAssignmentExpr(VarType vt)
	{
		extern Statement* FindAssignment(const std::wstring& id);
		Statement* pAsn = FindAssignment(vt.bufr);

		if(pAsn)
		{
			Assignment<T, U>* asgn = dynamic_cast<Assignment<T, U>*>(pAsn);

			if(asgn) return asgn->GetExpression();
			else HandleSyntaxError(_T("Assignment Expression type mismatch.\r\n"));
		}
		else HandleSyntaxError(_T("Assignment not found.\r\n"));

		return NULL; // to suppress warning
	}

	template<typename T, typename varary>
	CallExprArray<T, varary>* GetCallExprArray(varary *vptr, Statement* stmt)
	{
		CallExprArray<T, varary>* ptr = new CallExprArray<T, varary>(vptr, stmt);
		lastele = pGarcol->insert(ptr); return ptr;
	}
		
	template<typename T, typename varary>
	AssignmentExpr<T, varary>* GetAssignmentExpr(varary *vptr)
	{
		AssignmentExpr<T, varary>* ptr = new AssignmentExpr<T, varary>(vptr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	Expression<T>* GetAssignmentExpr(const wchar_t* vid, ExpIntg* index)
	{
		AssignmentExpr1D<T, VarAry1D<T> >* ptr = new AssignmentExpr1D<T, VarAry1D<T> >(vid, index);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	Expression<T>* GetAssignmentExpr(const wchar_t* vid, 
		ExpIntg* index1, ExpIntg* index2)
	{
		AssignmentExpr2D<T, VarAry2D<T> >* ptr = new AssignmentExpr2D<T, VarAry2D<T> >(vid, index1, index2);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	Expression<T>* GetAssignmentExpr(const wchar_t* vid, 
		ExpIntg* index1, ExpIntg* index2, ExpIntg* index3)
	{
		AssignmentExpr3D<T, VarAry3D<T> >* ptr = new AssignmentExpr3D<T, VarAry3D<T> >(vid, index1, index2, index3);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	Expression<T>* GetAssignmentExpr(const wchar_t* vid, 
		ExpIntg* index1, ExpIntg* index2, ExpIntg* index3, ExpIntg* index4)
	{
		AssignmentExpr4D<T, VarAry4D<T> >* ptr = new AssignmentExpr4D<T, VarAry4D<T> >(vid, index1, index2, index3, index4);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	Expression<T>* GetAssignmentExpr(const wchar_t* vid, 
		ExpIntg* index1, ExpIntg* index2, ExpIntg* index3,
		ExpIntg* index4, ExpIntg* index5)
	{
		AssignmentExpr5D<T, VarAry5D<T> >* ptr = new AssignmentExpr5D<T, VarAry5D<T> >(vid, 
			index1, index2, index3, index4, index5);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	VVDotOpr* GetVVDotOpr(ExpVctr* l, ExpVctr* r)
	{
		VVDotOpr* ptr = new VVDotOpr(l, r);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	ModOpr* GetModOpr(ExpIntg* l, ExpIntg* r)
	{
		ModOpr* ptr = new ModOpr(l, r);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	BinOpr<T>* GetBinOpr(int opr, Expression<T>* l, Expression<T>* r)
	{
		BinOpr<T>* ptr = new BinOpr<T>(opr, l, r);
		lastele = pGarcol->insert(ptr); return ptr;
	}
	
	BinOpr<Complex>* GetBinOpr(int opr, ExpReal* l, ExpCplx* r)
	{
		Literal<Complex>* cptr = GetLiteral(l, NULL);

		BinOpr<Complex>* ptr = new BinOpr<Complex>(opr, cptr, r);
		lastele = pGarcol->insert(ptr); return ptr;
	}
	
	BinOpr<Complex>* GetBinOpr(int opr, ExpCplx* l, ExpReal* r)
	{
		Literal<Complex>* cptr = GetLiteral(r, NULL);
		
		BinOpr<Complex>* ptr = new BinOpr<Complex>(opr, l, cptr);
		lastele = pGarcol->insert(ptr); return ptr;
	}
				
	RVBinOpr* GetRVBinOpr(int opr, 
		ExpReal* scalar, ExpVctr* vctr)
	{
		RVBinOpr* ptr = new RVBinOpr(opr, scalar, vctr);
		lastele = pGarcol->insert(ptr); return ptr;
	}
	
	RVBinOpr* GetRVBinOpr(int opr, 
		ExpVctr* vctr, ExpReal* scalar)
	{
		RVBinOpr* ptr = new RVBinOpr(opr, vctr, scalar);
		lastele = pGarcol->insert(ptr); return ptr;
	}
	
	template<typename T>
	Expression<T>* GetExprMaxMin(Expression<T>* l, Expression<T>* r, bool bmax)
	{
		ExprMaxMin<T>* ptr = new ExprMaxMin<T>(l, r, bmax);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	Expression<T>* GetExprWhich(ExpBool *pBool, Expression<T> *pXpr1, Expression<T> *pXpr2)
	{
		ExprWhich<T>* ptr = new ExprWhich<T>(pBool, pXpr1, pXpr2);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	ExpBool* GetBinBoolOpr(int opr, 
		Expression<T>* l, Expression<T>* r)
	{
		BinBoolOpr<T>* ptr = new BinBoolOpr<T>(opr, l, r);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	Expression<T*>* GetDynamicAllocation(Expression<T> *xpr=NULL)
	{
		DynamicAllocation<T>* ptr = new DynamicAllocation<T>(xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	Expression<UserData*>* GetAllocateUserData(UserDefinedType *udtPtr, 
		Expression<UserData> *xpr=NULL)
	{
		DynamicAllocation<UserData>* ptr = new DynamicAllocation<UserData>(udtPtr, xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	Statement* GetStmtDelete(LeftHandSide<T*> *lhs)
	{
		StmtDelete<T>* ptr = new StmtDelete<T>(lhs);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	Statement* GetStmtDeleteUserData(UdtPtrBase *lhs)
	{
		StmtDelete<UserData>* ptr = new StmtDelete<UserData>(lhs);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename ptrdata>
	Expression<T*>* GetPtrDataToPtr(ptrdata *pdata)
	{
		PtrDataToPtr<T, ptrdata>* ptr = new PtrDataToPtr<T, ptrdata>(pdata);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	ExpBool* GetRelation(int opr, 
		ExpBool* l, ExpBool* r)
	{
		Relation* ptr = new Relation(opr, l, r);
		lastele = pGarcol->insert(ptr); return ptr;
	}
	
	ExpEndLine* GetExpEndLine()
	{
		ExpEndLine* ptr = new ExpEndLine();
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename vec>
	Expression<T>* GetExpDet(Expression<vec>* xpr)
	{
		ExpDet<T, vec>* ptr = new ExpDet<T, vec>(xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	ExprType<T>* GetExpr(Expression<T>* xpr)
	{
		ExprType<T>* ptr = new ExprType<T>(xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	LeftHandSide<T>* GetExprPtr(PtrData<T>* dptr)
	{
		ExprPtr<T>* ptr = new ExprPtr<T>(dptr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename vec>
	LeftHandSideArray<vec>* GetExprAptr(AptrData<vec>* aptr)
	{
		ExprAptr<vec>* ptr = new ExprAptr<vec>(aptr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename vec>
	Expression<T>* GetMinMaxEle(Expression<vec>* vPtr, const char optr)
	{
		MinMaxEle<T, vec> *ptr = new MinMaxEle<T, vec>(vPtr, optr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename vec>
	Statement* GetAssignAptrN(AptrData<vec>* dptr, ExpIntg* idx1, Expression<T>* xpr)
	{
		AssignAptr1<T,vec>* ptr = new AssignAptr1<T,vec>(dptr, idx1, xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename vec>
	Statement* GetAssignAptrN(AptrData<vec>* dptr, ExpIntg* idx1, 
		ExpIntg* idx2, Expression<T>* xpr)
	{
		AssignAptr2<T, vec>* ptr = new AssignAptr2<T, vec>(dptr, idx1, idx2, xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename vec>
	Statement* GetAssignAptrN(AptrData<vec>* dptr, ExpIntg* idx1, 
		ExpIntg* idx2, ExpIntg* idx3, Expression<T>* xpr)
	{
		AssignAptr3<T,vec>* ptr = new AssignAptr3<T, vec>(dptr, idx1, idx2, idx3, xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename vec>
	Statement* GetAssignAptrN(AptrData<vec>* dptr, ExpIntg* idx1, ExpIntg* idx2, 
		ExpIntg* idx3, ExpIntg* idx4, Expression<T>* xpr)
	{
		AssignAptr4<T,vec>* ptr = new AssignAptr4<T,vec>(dptr, idx1, idx2, idx3, idx4, xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename vec>
	Statement* GetAssignAptrN(AptrData<vec>* dptr, ExpIntg* idx1, ExpIntg* idx2, 
		ExpIntg* idx3, ExpIntg* idx4, ExpIntg* idx5, Expression<T>* xpr)
	{
		AssignAptr5<T,vec>* ptr = new AssignAptr5<T,vec>(dptr, idx1, idx2, idx3, idx4, idx5, xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename vec>
	LeftHandSide<T>* GetExprAptr(AptrData<vec>* dptr, ExpIntg* idx1)
	{
		ExprAptr1<T,vec>* ptr = new ExprAptr1<T,vec>(dptr, idx1);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename ary>
	LeftHandSide<T>* GetExprAptr(AptrData<ary>* dptr, ExpIntg* idx1, 
		ExpIntg* idx2)
	{
		ExprAptr2<T,ary>* ptr = new ExprAptr2<T,ary>(dptr, idx1, idx2);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename ary>
	LeftHandSide<T>* GetExprAptr(AptrData<ary>* dptr, ExpIntg* idx1, 
		ExpIntg* idx2, ExpIntg* idx3)
	{
		ExprAptr3<T,ary>* ptr = new ExprAptr3<T,ary>(dptr, idx1, idx2, idx3);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename ary>
	LeftHandSide<T>* GetExprAptr(AptrData<ary>* dptr, ExpIntg* idx1, 
		ExpIntg* idx2, ExpIntg* idx3, ExpIntg* idx4)
	{
		ExprAptr4<T,ary>* ptr = new ExprAptr4<T,ary>(dptr, idx1, idx2, idx3, idx4);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename ary>
	LeftHandSide<T>* GetExprAptr(AptrData<ary>* dptr, ExpIntg* idx1, 
		ExpIntg* idx2, ExpIntg* idx3, ExpIntg* idx4, ExpIntg* idx5)
	{
		ExprAptr5<T,ary>* ptr = new ExprAptr5<T,ary>(dptr, idx1, idx2, idx3, idx4, idx5);
		lastele = pGarcol->insert(ptr); return ptr;
	}
	
	StmtIf* GetStmtIf(ExpBool* xpr, Statement* stmt, bool kind, Statement *pelse=NULL)
	{
		StmtIf* ptr = new StmtIf(xpr, stmt, kind, pelse);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	StmtIfElse* GetStmtIfElse(ExpBool* xpr, Statement* stmt1, Statement* stmt2)
	{
		StmtIfElse* ptr = new StmtIfElse(xpr, stmt1, stmt2);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	StmtSelect<T>* GetStmtSelect(Expression<T>* xpr)
	{
		StmtSelect<T>* ptr = new StmtSelect<T>(xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	ExpIntg* GetVOFunSize(FunName fname, VarType ary_vt, 
		VarType vt0, VarType vt1, VarType vt2, VarType vt3, VarType vt4)
	{
		
		Array* aptr = dynamic_cast<Array*>(ary_vt.dPtr);

		if(!aptr)
			HandleSyntaxError(_T("Type is not array.\r\n"));
		
		IntgData *pid0, *pid1, *pid2, *pid3, *pid4;
		pid0 = pid1 = pid2 = pid3 = pid4 = NULL;

		if(vt0.dPtr) pid0 = static_cast<IntgData*>(vt0.dPtr);
		if(vt1.dPtr) pid1 = static_cast<IntgData*>(vt1.dPtr);
		if(vt2.dPtr) pid2 = static_cast<IntgData*>(vt2.dPtr);
		if(vt3.dPtr) pid3 = static_cast<IntgData*>(vt3.dPtr);
		if(vt4.dPtr) pid4 = static_cast<IntgData*>(vt4.dPtr);
				
		VOFunSize* ptr = new VOFunSize(fname, aptr, pid0, pid1, pid2, pid3, pid4);

		lastele = pGarcol->insert(ptr); return ptr;
	}

	
	template<typename T, typename vec>
	ExpIntg* GetVOFunSizePtr(Data* dptr, 
		VarType vt0, VarType vt1, VarType vt2, VarType vt3, VarType vt4)
	{
		AptrData<vec>* aptr = dynamic_cast<AptrData<vec>*>(dptr);

		if(!aptr) HandleSyntaxError(_T("Invalid Array Pointer.\r\n"));
		
		IntgData *pid0, *pid1, *pid2, *pid3, *pid4;
		pid0 = pid1 = pid2 = pid3 = pid4 = NULL;

		if(vt0.dPtr) pid0 = static_cast<IntgData*>(vt0.dPtr);
		if(vt1.dPtr) pid1 = static_cast<IntgData*>(vt1.dPtr);
		if(vt2.dPtr) pid2 = static_cast<IntgData*>(vt2.dPtr);
		if(vt3.dPtr) pid3 = static_cast<IntgData*>(vt3.dPtr);
		if(vt4.dPtr) pid4 = static_cast<IntgData*>(vt4.dPtr);
				
		VOFunSizePtr5<T, vec>* ptr = 
			new VOFunSizePtr5<T, vec>(aptr, pid0, pid1, pid2, pid3, pid4);

		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename vec>
	ExpIntg* GetVOFunSizePtr(Data* dptr, 
		VarType vt0, VarType vt1, VarType vt2, VarType vt3)
	{
		AptrData<vec>* aptr = dynamic_cast<AptrData<vec>*>(dptr);

		if(!aptr) HandleSyntaxError(_T("Invalid Array Pointer.\r\n"));
		
		IntgData *pid0, *pid1, *pid2, *pid3;
		pid0 = pid1 = pid2 = pid3 = NULL;

		if(vt0.dPtr) pid0 = static_cast<IntgData*>(vt0.dPtr);
		if(vt1.dPtr) pid1 = static_cast<IntgData*>(vt1.dPtr);
		if(vt2.dPtr) pid2 = static_cast<IntgData*>(vt2.dPtr);
		if(vt3.dPtr) pid3 = static_cast<IntgData*>(vt3.dPtr);
						
		VOFunSizePtr4<T, vec>* ptr = 
			new VOFunSizePtr4<T, vec>(aptr, pid0, pid1, pid2, pid3);

		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename vec>
	ExpIntg* GetVOFunSizePtr(Data* dptr, 
		VarType vt0, VarType vt1, VarType vt2)
	{
		AptrData<vec>* aptr = dynamic_cast<AptrData<vec>*>(dptr);

		if(!aptr) HandleSyntaxError(_T("Invalid Array Pointer.\r\n"));
		
		IntgData *pid0, *pid1, *pid2;
		pid0 = pid1 = pid2 = NULL;

		if(vt0.dPtr) pid0 = static_cast<IntgData*>(vt0.dPtr);
		if(vt1.dPtr) pid1 = static_cast<IntgData*>(vt1.dPtr);
		if(vt2.dPtr) pid2 = static_cast<IntgData*>(vt2.dPtr);
								
		VOFunSizePtr3<T, vec>* ptr = 
			new VOFunSizePtr3<T, vec>(aptr, pid0, pid1, pid2);

		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename vec>
	ExpIntg* GetVOFunSizePtr(Data* dptr, VarType vt0, VarType vt1)
	{
		AptrData<vec>* aptr = dynamic_cast<AptrData<vec>*>(dptr);

		if(!aptr) HandleSyntaxError(_T("Invalid Array Pointer.\r\n"));
		
		IntgData *pid0, *pid1;
		pid0 = pid1 = NULL;

		if(vt0.dPtr) pid0 = static_cast<IntgData*>(vt0.dPtr);
		if(vt1.dPtr) pid1 = static_cast<IntgData*>(vt1.dPtr);
										
		VOFunSizePtr2<T, vec>* ptr = 
			new VOFunSizePtr2<T, vec>(aptr, pid0, pid1);

		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename vec>
	ExpIntg* GetVOFunSizePtr(Data* dptr, VarType vt0)
	{
		AptrData<vec>* aptr = dynamic_cast<AptrData<vec>*>(dptr);

		if(!aptr) HandleSyntaxError(_T("Invalid Array Pointer.\r\n"));
		
		IntgData *pid0;
		pid0 = NULL;

		if(vt0.dPtr) pid0 = static_cast<IntgData*>(vt0.dPtr);
												
		VOFunSizePtr1<T, vec>* ptr = 
			new VOFunSizePtr1<T, vec>(aptr, pid0);

		lastele = pGarcol->insert(ptr); return ptr;
	}

	ExpIntg* GetVOFunSizePtr(int dim)
	{
		VOFunSizePtr* ptr =  new VOFunSizePtr(dim);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	ExpIntg* GetIFunIntg(FunName fname)
	{
		IFunIntg* ptr = new IFunIntg(fname);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	ExpIntg* GetIFunII(FunName fname, ExpIntg *xpr1, ExpIntg *xpr2)
	{
		IFunII* ptr = new IFunII(fname, xpr1, xpr2);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	ExpBool* GetBFunS(FunName fname, ExpStrn *xpr)
	{
		BFunS* ptr = new BFunS(fname, xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}
	template<typename T>
	ExpStrn* GetExprStrn(Expression<T>* xpr)
	{
		ExprStrn<T> *ptr = new ExprStrn<T>(xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	Statement* GetVOFunResize(FunName fname, VarType ary_vt, 
		ExpIntg* pid0=NULL, ExpIntg* pid1=NULL, 
		ExpIntg* pid2=NULL, ExpIntg* pid3=NULL,
		ExpIntg* pid4=NULL)
	{
		
		Array* aptr = dynamic_cast<Array*>(ary_vt.dPtr);

		if(!aptr)
			HandleSyntaxError(_T("Type is not array.\r\n"));
		
		VOFunResize* ptr = new VOFunResize(fname, aptr, pid0, pid1, pid2, pid3, pid4);

		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	Statement* GetVOFunUdtResize(FunName fname, VarUser<T> *ary, 
		ExpIntg* pid0=NULL, ExpIntg* pid1=NULL, 
		ExpIntg* pid2=NULL, ExpIntg* pid3=NULL,
		ExpIntg* pid4=NULL)
	{
		
		VOFunUdtResize<T>* ptr = new VOFunUdtResize<T>(fname, ary, pid0, pid1, pid2, pid3, pid4);

		lastele = pGarcol->insert(ptr); return ptr;
	}

	Statement* GetVOFunMakeUnit(FunName fname, VarType ary_vt, ExpIntg* pid=NULL)
	{
		Array* aptr = dynamic_cast<Array*>(ary_vt.dPtr);

		if(!aptr)
			HandleSyntaxError(_T("Type is not array.\r\n"));
		
		VOFunMakeUnit* ptr = new VOFunMakeUnit(fname, aptr, pid);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	Statement* GetStmtFor(Statement* init, 
		ExpBool* xpr, Statement* update, Statement* stmt)
	{
		StmtFor* ptr = new StmtFor(init, xpr, update, stmt);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	Statement* GetStmtGrpx1D(const wchar_t* bname, VarType vt,
		ExpReal *xpr1, ExpReal *xpr2, ExpReal *xpr3)
	{
		RealData *uPtr = dynamic_cast<RealData*>(vt.dPtr);
		if(uPtr)
		{
			StmtGraph1D* ptr = new StmtGraph1D(bname, uPtr, xpr1, xpr2, xpr3);
			lastele = pGarcol->insert(ptr); return ptr;
		}
		else
			HandleSyntaxError(_T("Type mismatch at GetStmtGrpx1D()\r\n"));
		
		return NULL; // to suppress warning
	}

	Expression<RealVec1D>* GetSolveEquation(LeftHandSide<realnum> *pLHS,
		ExpReal *xpr1, ExpReal *xpr2, ExpReal *xpr3)
	{
		SolveEquation* ptr = new SolveEquation(pLHS, xpr1, xpr2, xpr3);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	Expression<RealVec1D>* GetSolveEquation(LeftHandSide<realnum> *pLHS,
		ExpReal *xpr1, ExpReal *xpr2, ExpReal *xpr3, ExpIntg* pCnt)
	{
		SolveEquation* ptr = new SolveEquation(pLHS, xpr1, xpr2, xpr3, pCnt);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	Expression<RealVec1D>* GetSolveEquation(LeftHandSide<realnum> *pLHS,
		ExpReal *xpr1, ExpReal *xpr2, ExpReal *xpr3, ExpReal* pEps)
	{
		SolveEquation* ptr = new SolveEquation(pLHS, xpr1, xpr2, xpr3, pEps);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	Expression<RealVec1D>* GetSolveEquation(LeftHandSide<realnum> *pLHS,
		ExpReal *xpr1, ExpReal *xpr2, ExpReal *xpr3, 
		ExpIntg* pCnt, ExpReal* pEps)
	{
		SolveEquation* ptr = new SolveEquation(pLHS, xpr1, xpr2, xpr3, pCnt, pEps);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	Expression<RealVec1D>* GetSolveEquation(LeftHandSide<realnum> *pLHS,
		ExpReal *xpr1, ExpReal *xpr2, ExpReal *xpr3, 
		ExpReal* pEps, ExpIntg* pCnt)
	{
		SolveEquation* ptr = new SolveEquation(pLHS, xpr1, xpr2, xpr3, pCnt, pEps);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	Expression<RealVec1D>* GetSolveExtrema(LeftHandSide<realnum> *pLHS,
		ExpReal *xpr1, ExpReal *xpr2, ExpReal *xpr3)
	{
		SolveExtrema* ptr = new SolveExtrema(pLHS, xpr1, xpr2, xpr3);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	Expression<RealVec1D>* GetSolveExtrema(LeftHandSide<realnum> *pLHS,
		ExpReal *xpr1, ExpReal *xpr2, ExpReal *xpr3, ExpIntg* pCnt)
	{
		SolveExtrema* ptr = new SolveExtrema(pLHS, xpr1, xpr2, xpr3, pCnt);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	Expression<RealVec1D>* GetSolveExtrema(LeftHandSide<realnum> *pLHS,
		ExpReal *xpr1, ExpReal *xpr2, ExpReal *xpr3, ExpReal* pEps)
	{
		SolveExtrema* ptr = new SolveExtrema(pLHS, xpr1, xpr2, xpr3, pEps);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	Expression<RealVec1D>* GetSolveExtrema(LeftHandSide<realnum> *pLHS,
		ExpReal *xpr1, ExpReal *xpr2, ExpReal *xpr3, 
		ExpIntg* pCnt, ExpReal* pEps)
	{
		SolveExtrema* ptr = new SolveExtrema(pLHS, xpr1, xpr2, xpr3, pCnt, pEps);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	Expression<RealVec1D>* GetSolveExtrema(LeftHandSide<realnum> *pLHS,
		ExpReal *xpr1, ExpReal *xpr2, ExpReal *xpr3, 
		ExpReal* pEps, ExpIntg* pCnt)
	{
		SolveExtrema* ptr = new SolveExtrema(pLHS, xpr1, xpr2, xpr3, pCnt, pEps);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	Statement* GetStmtWhile(ExpBool* xpr, Statement* stmt)
	{
		StmtWhile* ptr = new StmtWhile(xpr, stmt);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	Statement* GetStmtDo(ExpBool* xpr, Statement* stmt)
	{
		StmtDo* ptr = new StmtDo(xpr, stmt);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	Statement* GetStmtThrowBase()
	{
		extern Stack<bool> gCatchBlockStack;
		
		if(!gCatchBlockStack.empty() && gCatchBlockStack.top())
		{
			StmtThrowBase *ptr = new StmtThrowBase();
			lastele = pGarcol->insert(ptr); return ptr;
		}
		HandleSyntaxError(L"throw statement without argument is only allowed within catch block.\r\n");
		
		return NULL; // to suppress warning
	}
	
	template<typename T>
	Statement* GetStmtThrow(Expression<T>* xpr)
	{
		StmtThrow<T> *ptr = new StmtThrow<T>(xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	Statement* GetStmtThrowExp(LeftHandSide<UserData> *xpr)
	{
		StmtThrowExp *ptr = new StmtThrowExp(xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	Statement* GetStmtTryCatchBase(Statement *pStmtTry, StatementList *pStmtCatch)
	{
		StmtTryCatchBase *ptr = 
			new StmtTryCatchBase(pStmtTry, pStmtCatch);

		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	Statement* GetStmtTryCatch(Statement *pStmtTry, StatementList *pStmtCatch)
	{
		StmtTryCatch<T> *ptr = 
			new StmtTryCatch<T>(pStmtTry, pStmtCatch);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	Statement* GetStmtTryCatchPtr(Statement *pStmtTry, StatementList *pStmtCatch)
	{
		StmtTryCatchPtr<T> *ptr = 
			new StmtTryCatchPtr<T>(pStmtTry, pStmtCatch);
		lastele = pGarcol->insert(ptr); return ptr;
	}
	
	template<typename U>
	Statement* GetStmtSwap(VarType vt1, VarType vt2)
	{
		U* lhs = dynamic_cast<U*>(vt1.dPtr);
		U* rhs = dynamic_cast<U*>(vt2.dPtr);

		if(lhs && rhs)
		{
			StmtSwap<U>* ptr = new StmtSwap<U>(lhs, rhs);
			lastele = pGarcol->insert(ptr); return ptr;
		}
		
		HandleSyntaxError(_T("Swapping not supported for two different data types.\r\n"));
		
		return NULL; // to suppress warning
	}

	Statement* GetStmtSwapVUB(VarUserBase *lptr, VarUserBase *rptr)
	{
		StmtSwapVUB* ptr = new StmtSwapVUB(lptr, rptr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	Statement* GetStmtSwapLHS(LeftHandSide<T>* xpr1, LeftHandSide<T> *xpr2)
	{
		StmtSwapLHS<T>* ptr = new StmtSwapLHS<T>(xpr1, xpr2);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	Statement* GetStmtSwapArrayLHS(LeftHandSideArray<T>* xpr1, LeftHandSideArray<T> *xpr2)
	{
		StmtSwapArrayLHS<T>* ptr = new StmtSwapArrayLHS<T>(xpr1, xpr2);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename U>
	Statement* GetStmtSwapExpr(Expression<T> *xpr1, Expression<T> *xpr2)
	{
		StmtSwapExpr<T, U>* ptr = new StmtSwapExpr<T, U>(xpr1, xpr2);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename U>
	Statement* GetStmtPtrSwap(U* lhs, U* rhs)
	{
		StmtSwap<U>* ptr = new StmtSwap<U>(lhs, rhs);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	
	Statement* GetStmtLabel(const wchar_t* label)
	{
		Label* ptr = new Label(label);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	StmtInitArray* GetStmtInitArray()
	{
		StmtInitArray* ptr = new StmtInitArray();
		lastele = pGarcol->insert(ptr); return ptr;
	}

	Statement* GetStmtBreak()
	{
		Break* ptr = new Break();
		lastele = pGarcol->insert(ptr); return ptr;
	}

	Statement* GetStmtSkip()
	{
		Skip* ptr = new Skip();
		lastele = pGarcol->insert(ptr); return ptr;
	}

	Statement* GetStmtGoTo(const wchar_t* label)
	{
		GoTo* ptr = new GoTo(label);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	Statement* GetStmtAbort(ExpStrn *xpr)
	{
		StmtAbort* ptr = new StmtAbort(xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	Statement* GetStmtClear()
	{
		StmtClear* ptr = new StmtClear();
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename vec, typename ary>
	Statement* GetAssignArray(VarType vt1, Expression<vec>* xpr, char optr= '=')
	{
		ary* lhs = dynamic_cast<ary*>(vt1.dPtr);
		if(lhs == NULL) 
			HandleSyntaxError(_T("Left hand side is not array.\r\n"));
		
		AssignArray<vec, ary>* ptr = new AssignArray<vec, ary>(lhs, xpr, optr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename ary>
	Statement* GetInitializeArray(VarType vt1, Expression<T>* xpr)
	{
		ary* lhs = dynamic_cast<ary*>(vt1.dPtr);
		if(lhs == NULL) 
			HandleSyntaxError(_T("Left hand side is not array.\r\n"));
		
		InitializeArray<T, ary>* ptr = new InitializeArray<T, ary>(lhs, xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename vec, typename ary>
	Statement* GetAssignArrayVec(VarType vt1, Expression<vec>* xpr, char optr= '=')
	{
		ary* lhs = dynamic_cast<ary*>(vt1.dPtr);
		if(lhs == NULL) 
			HandleSyntaxError(_T("Left hand side is not array.\r\n"));
		
		AssignArrayVec<vec, ary>* ptr = new AssignArrayVec<vec, ary>(lhs, xpr, optr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename vec>
	ArrayResize<vec>* GetArrayResize(LeftHandSideArray<vec>* pLHS,
		ExpIntg *pSize1=NULL, ExpIntg *pSize2=NULL, 
		ExpIntg *pSize3=NULL, ExpIntg *pSize4=NULL, ExpIntg *pSize5=NULL)
	{
		ArrayResize<vec>* ptr = 
			new ArrayResize<vec>(pLHS, pSize1, pSize2, pSize3, pSize4, pSize5);

		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename vec>
	Statement* GetArraySize(LeftHandSideArray<vec>* pLHS, LeftHandSide<integer>* pSize1, 
		LeftHandSide<integer>* pSize2=NULL, LeftHandSide<integer>* pSize3=NULL,
		LeftHandSide<integer>* pSize4=NULL, LeftHandSide<integer>* pSize5=NULL)
	{
		ArraySize<vec> *ptr = 
			new ArraySize<vec>(pLHS, pSize1, pSize2, pSize3, pSize4, pSize5);

		lastele = pGarcol->insert(ptr); return ptr;
	}
	
	template<typename vec>
	Expression<integer>* GetArraySizeExp(LeftHandSideArray<vec>* pLHS, 
		LeftHandSide<integer>* pSize1=NULL, LeftHandSide<integer>* pSize2=NULL,
		LeftHandSide<integer>* pSize3=NULL, LeftHandSide<integer>* pSize4=NULL)
	{
		ArraySizeExp<vec> *ptr = new ArraySizeExp<vec>(pLHS, pSize1, pSize2, pSize3, pSize4);
		lastele = pGarcol->insert(ptr); return ptr;
	}
		
	template<typename T, typename ary>
	Statement* GetAssignVarMemAry(VarMemAry<T, ary> *vptr, Expression<T> *xpr)
	{
		AssignVarMemAry<T, ary>* ptr = new AssignVarMemAry<T, ary>(vptr, xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename ary>
	Statement* GetAssignVarUserAry(VarUserAry<T, ary> *vptr, Expression<T> *xpr)
	{
		AssignVarUserAry<T, ary>* ptr = new AssignVarUserAry<T, ary>(vptr, xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	AssignmentLHS<T>* GetAssignmentLHS(LeftHandSide<T> *lhs, Expression<T>* rhs, const char optr= '=')
	{
		extern bool assign_set; extern ArrayInit gArrayInit;
		if(gArrayInit.pStmtInit==NULL ) assign_set = false;

		AssignmentLHS<T>* ptr = new AssignmentLHS<T>(lhs, rhs, optr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	AssignmentLHS<bool>* GetAssignmentLHSBool(LeftHandSide<bool> *lhs, ExpBool* rhs)
	{
		extern bool assign_set; extern ArrayInit gArrayInit;
		if(gArrayInit.pStmtInit==NULL ) assign_set = false;

		AssignmentLHS<bool>* ptr = new AssignmentLHS<bool>(lhs, rhs);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename vec>
	AssignmentArrayLHS<vec>* GetAssignmentArrayLHS(LeftHandSide<vec> *lhs, Expression<vec>* rhs, const char optr= '=')
	{
		extern bool assign_set; extern ArrayInit gArrayInit;
		if(gArrayInit.pStmtInit==NULL ) assign_set = false;

		AssignmentArrayLHS<vec>* ptr = new AssignmentArrayLHS<vec>(lhs, rhs, optr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename vec>
	AssignmentArrayVctrLHS<vec>* GetAssignmentArrayVctrLHS(LeftHandSide<vec> *lhs, Expression<vec>* rhs, const char optr= '=')
	{
		extern bool assign_set; extern ArrayInit gArrayInit;
		if(gArrayInit.pStmtInit==NULL ) assign_set = false;

		AssignmentArrayVctrLHS<vec>* ptr = new AssignmentArrayVctrLHS<vec>(lhs, rhs, optr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename U>
	Statement* GetAssignment(VarType vt, Expression<T>* xpr, const char optr= '=')
	{
		extern bool assign_set; assign_set = false;

		U *uPtr = dynamic_cast<U*>(vt.dPtr);
		if(uPtr)
		{
			Assignment<T,U>* sPtr = new Assignment<T,U>(uPtr, xpr, optr);
			lastele = pGarcol->insert(sPtr); return sPtr;
		}
		
		HandleSyntaxError(_T("Type mismatch at assignment.\r\n"));
		
		return NULL; // to suppress warning
	}

	template<typename T>
	Statement* GetAssignment(VarUser<T> *vuPtr, Expression<T>* xpr, const char optr= '=')
	{
		Assignment<T,VarUser<T> >* ptr = new Assignment<T,VarUser<T> >(vuPtr, xpr, optr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	Statement* GetAssignmentVarMem(VarMem<T> *vmptr, Expression<T>* xpr, const char optr= '=')
	{
		AssignmentVarMem<T>* ptr = new AssignmentVarMem<T>(vmptr, xpr, optr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	Statement* GetAssignVarMemCplxReal(VarMem<Complex> *vmptr, ExpReal* xpr)
	{
		AssignVarMemCplxReal* ptr = new AssignVarMemCplxReal(vmptr, xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	Statement* GetAssignVarMemCplxImag(VarMem<Complex> *vmptr, ExpReal* xpr)
	{
		AssignVarMemCplxImag* ptr = new AssignVarMemCplxImag(vmptr, xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	Statement* GetAssignVarMemVctrX(VarMem<Vector> *vmptr, ExpReal* xpr)
	{
		AssignVarMemVctrX* ptr = new AssignVarMemVctrX(vmptr, xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	Statement* GetAssignVarMemVctrY(VarMem<Vector> *vmptr, ExpReal* xpr)
	{
		AssignVarMemVctrY* ptr = new AssignVarMemVctrY(vmptr, xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	Statement* GetAssignVarMemVctrZ(VarMem<Vector> *vmptr, ExpReal* xpr)
	{
		AssignVarMemVctrZ* ptr = new AssignVarMemVctrZ(vmptr, xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}
	
	Statement* GetAssignExpUser(LeftHandSide<UserData>* lhs, ExpUserPtr rhs)
	{
		AssignExpUser* ptr = new AssignExpUser(lhs, rhs);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	Statement* GetAssignUT2ExpUser(LeftHandSide<UserData>* lhs, CallFunc<UserData> *rhs)
	{
		AssignUT2ExpUser* ptr = new AssignUT2ExpUser(lhs, rhs);
		lastele = pGarcol->insert(ptr); return ptr;
	}
	
	Statement* GetAssignCplxReal(VarType vt, ExpReal* xpr)
	{
		CplxData *uPtr = dynamic_cast<CplxData*>(vt.dPtr);
		if(uPtr)
		{
			AssignCplxReal* sPtr = new AssignCplxReal(uPtr, xpr);
			lastele = pGarcol->insert(sPtr); return sPtr;
		}
		
		HandleSyntaxError(_T("Type mismatch at assignment.\r\n"));

		return NULL; // to suppress warning
	}

	Statement* GetAssignCplxImag(VarType vt, ExpReal* xpr)
	{
		CplxData *uPtr = dynamic_cast<CplxData*>(vt.dPtr);
		if(uPtr)
		{
			AssignCplxImag* sPtr = new AssignCplxImag(uPtr, xpr);
			lastele = pGarcol->insert(sPtr); return sPtr;
		}
		
		HandleSyntaxError(_T("Type mismatch at assignment.\r\n"));

		return NULL; // to suppress warning
	}

	Statement* GetAssignVctrX(VarType vt, ExpReal* xpr)
	{
		VctrData *uPtr = dynamic_cast<VctrData*>(vt.dPtr);
		if(uPtr)
		{
			AssignVctrX* sPtr = new AssignVctrX(uPtr, xpr);
			lastele = pGarcol->insert(sPtr); return sPtr;
		}
		
		HandleSyntaxError(_T("Type mismatch at assignment.\r\n"));

		return NULL; // to suppress warning
	}

	Statement* GetAssignVctrY(VarType vt, ExpReal* xpr)
	{
		VctrData *uPtr = dynamic_cast<VctrData*>(vt.dPtr);
		if(uPtr)
		{
			AssignVctrY* sPtr = new AssignVctrY(uPtr, xpr);
			lastele = pGarcol->insert(sPtr); return sPtr;
		}
		
		HandleSyntaxError(_T("Type mismatch at assignment.\r\n"));

		return NULL; // to suppress warning
	}

	Statement* GetAssignVctrZ(VarType vt, ExpReal* xpr)
	{
		VctrData *uPtr = dynamic_cast<VctrData*>(vt.dPtr);
		if(uPtr)
		{
			AssignVctrZ* sPtr = new AssignVctrZ(uPtr, xpr);
			lastele = pGarcol->insert(sPtr); return sPtr;
		}
		
		HandleSyntaxError(_T("Type mismatch at assignment.\r\n"));

		return NULL; // to suppress warning
	}

	template<typename ltype>
	Statement* GetAssignVctr(ltype *pLHS, ExpReal* xpr, const char optr)
	{
		AssignVctr<ltype>* ptr = new AssignVctr<ltype>(pLHS, xpr, optr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	Statement* GetAssignCplx(LeftHandSide<T> *pLHS, ExpReal* xpr, const char optr)
	{
		AssignCplx<T>* ptr = new AssignCplx<T>(pLHS, xpr, optr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	Return<T>* GetReturn(ExpTypeless* ptr)
	{
		extern Stack<ReturnSig> gReturnStack;

		if(typeid(T)==typeid(voidtype))
		{
			if(ptr) // function of type void can not return a value.
			{
				std::wostringstream os;
				os<<_T("Function \'")<<gReturnStack.top().fname<<_T("\' of type void cannot return a value.\r\n");
				HandleSyntaxError(os.str());
			}
			else
			{
				Return<T> *rtn = new Return<T>(&(gReturnStack.top().fptr->rtnStack), 
					NULL, gReturnStack.top().fptr->pStmtList);
				lastele = pGarcol->insert(rtn); return rtn;
			}
		}
		else if(ptr==NULL) // T != voidtype, and so ptr should not be NULL
		{
			std::wostringstream os;
			os<<_T("Function \'")<<gReturnStack.top().fname<<_T("\' should return a value.\r\n");
			HandleSyntaxError(os.str());
		}
		else // T != voidtype || ptr != NULL 
		{
			Expression<T>* xpr = dynamic_cast<Expression<T>*>(ptr);
			if(xpr)
			{
				Return<T> *rtn = new Return<T>(&(gReturnStack.top().fptr->rtnStack), 
					xpr, gReturnStack.top().fptr->pStmtList);
				lastele = pGarcol->insert(rtn); return rtn;
			}
			else
			{
				std::wostringstream os;
				os<<_T("Function \'")<<gReturnStack.top().fname
					<<_T("\' return type mismatch..\r\n");
				HandleSyntaxError(os.str());
			}
		}

		return NULL; // to suppress warning
	}

	ReturnUdt* GetReturnUdt(LeftHandSide<UserData>* vubPtr)
	{
		extern Stack<ReturnSig> gReturnStack;

		ReturnUdt *ptr = new ReturnUdt(&(gReturnStack.top().fptr->rtnStack), 
			vubPtr, gReturnStack.top().fptr->pStmtList);

		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	UnaryOpr<T>* GetUnaryOpr(int opr, Expression<T>* xpr)
	{
		UnaryOpr<T>* ptr = new UnaryOpr<T>(opr, xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	UnaryBoolOpr* GetUnaryBoolOpr(int opr, ExpBool* xpr)
	{
		UnaryBoolOpr* ptr = new UnaryBoolOpr(opr, xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}
	
	template<typename T, typename varary>
	AssignAryEle<T, varary>* GetAssignAryEle(varary *vptr, Expression<T>* xpr, char opr='=')
	{
		AssignAryEle<T, varary> *ptr = new AssignAryEle<T, varary>(vptr, xpr, opr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename varary>
	AssignAryEle<T, varary>* GetAssignAryEle(const std::wstring& id, varary *vptr, Expression<T>* xpr, char opr='=')
	{
		AssignAryEle<T, varary> *ptr = new AssignAryEle<T, varary>(id, vptr, xpr, opr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename varary>
	AssignAryEleCplxReal<varary>* GetAssignAryEleCplxReal(varary *vptr, ExpReal* xpr)
	{
		AssignAryEleCplxReal<varary> *ptr = new AssignAryEleCplxReal<varary>(vptr, xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename varary>
	AssignAryEleCplxImag<varary>* GetAssignAryEleCplxImag(varary *vptr, ExpReal* xpr)
	{
		AssignAryEleCplxImag<varary> *ptr = new AssignAryEleCplxImag<varary>(vptr, xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename varary>
	AssignAryEleVctrX<varary>* GetAssignAryEleVctrX(varary *vptr, ExpReal* xpr)
	{
		AssignAryEleVctrX<varary> *ptr = new AssignAryEleVctrX<varary>(vptr, xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename varary>
	AssignAryEleVctrY<varary>* GetAssignAryEleVctrY(varary *vptr, ExpReal* xpr)
	{
		AssignAryEleVctrY<varary> *ptr = new AssignAryEleVctrY<varary>(vptr, xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}
	
	template<typename varary>
	AssignAryEleVctrZ<varary>* GetAssignAryEleVctrZ(varary *vptr, ExpReal* xpr)
	{
		AssignAryEleVctrZ<varary> *ptr = new AssignAryEleVctrZ<varary>(vptr, xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename U>
	Statement* GetAssignPtr(PtrData<T>* ptr, VarType vt)
	{
		Statement* sPtr;
		U* uPtr = dynamic_cast<U*>(vt.dPtr);
		if(uPtr)
		{
			sPtr = new AssignPtr<T, U>(ptr, uPtr);
			lastele = pGarcol->insert(sPtr); return sPtr;
		}
		
		HandleSyntaxError(_T("Unsported data type.\r\n"));

		return NULL; // to suppress warning
	}

	template<typename T>
	Statement* GetAssignmentPtr(PtrData<T>* pData, LeftHandSide<T> *pRHS)
	{
		AssignmentPtr<T>* ptr = new AssignmentPtr<T>(pData, pRHS);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename vec>
	Statement* GetAssignmentAptr(AptrData<vec>* pData, LeftHandSide<vec> *pRHS)
	{
		AssignmentAptr<vec>* ptr = new AssignmentAptr<vec>(pData, pRHS);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T, typename U>
	Statement* GetAssignPtrPtr(PtrData<T>* ptr1, PtrData<T>* ptr2)
	{
		AssignPtrPtr<T, U>* ptr = new AssignPtrPtr<T, U>(ptr1, ptr2);
		lastele = pGarcol->insert(ptr); return ptr;
	}
	
	template<typename T, typename U>
	Statement* GetAssignPtrValue(PtrData<T>* ptr, Expression<T>* xpr, const char opr = '=')
	{
		Statement* sPtr;
		sPtr = new AssignPtrValue<T, U>(ptr, xpr, opr);
		lastele = pGarcol->insert(sPtr); return sPtr;
	}

	Statement* GetAssignPtrValueCplx(PtrData<Complex>* ptr, 
		ExpReal* xpr, const char opr)
	{
		Statement* sPtr;
		sPtr = new AssignPtrValueCplx(ptr, xpr, opr);
		lastele = pGarcol->insert(sPtr); return sPtr;
	}

	Statement* GetAssignPtrValueVctr(PtrData<Vector>* ptr, 
		ExpReal* xpr, const char opr)
	{
		Statement* sPtr;
		sPtr = new AssignPtrValueVctr(ptr, xpr, opr);
		lastele = pGarcol->insert(sPtr); return sPtr;
	}
	
	template<typename T, typename U>
	Statement* GetAssignPtrAryEle(PtrData<T>* ptr, VarType vt, ExpIntg* index)
	{
		VarAry1D<T>* vPtr = GetVarAry<T>(vt, index);
		Statement* sPtr = new AssignPtrAryEle<T, U, VarAry1D<T> >(ptr, vPtr);
		lastele = pGarcol->insert(sPtr); return sPtr;
	}

	template<typename T, typename U>
	Statement* GetAssignPtrAryEle(PtrData<T>* ptr, VarType vt, 
		ExpIntg* index1, ExpIntg* index2)
	{
		VarAry2D<T>* vPtr = GetVarAry<T>(vt, index1, index2);
		Statement* sPtr = new AssignPtrAryEle<T, U, VarAry2D<T> >(ptr, vPtr);
		lastele = pGarcol->insert(sPtr); return sPtr;
	}

	template<typename T, typename U>
	Statement* GetAssignPtrAryEle(PtrData<T>* ptr, VarType vt, ExpIntg* index1, 
		ExpIntg* index2, ExpIntg* index3)
	{
		VarAry3D<T>* vPtr = GetVarAry<T>(vt, index1, index2, index3);
		Statement* sPtr = new AssignPtrAryEle<T, U, VarAry3D<T> >(ptr, vPtr);
		lastele = pGarcol->insert(sPtr); return sPtr;
	}

	template<typename T, typename U>
	Statement* GetAssignPtrAryEle(PtrData<T>* ptr, VarType vt,
		ExpIntg* index1, ExpIntg* index2,
		ExpIntg* index3, ExpIntg* index4)
	{
		VarAry4D<T>* vPtr = GetVarAry<T>(vt, index1, index2, index3, index4);
		Statement* sPtr = new AssignPtrAryEle<T, U, VarAry4D<T> >(ptr, vPtr);
		lastele = pGarcol->insert(sPtr); return sPtr;
	}

	template<typename T, typename U>
	Statement* GetAssignPtrAryEle(PtrData<T>* ptr, VarType vt, ExpIntg* index1,
		ExpIntg* index2, ExpIntg* index3, 
		ExpIntg* index4, ExpIntg* index5)
	{
		VarAry5D<T>* vPtr = GetVarAry<T>(vt, index1, index2, index3, index4, index5);
		Statement* sPtr = new AssignPtrAryEle<T, U, VarAry5D<T> >(ptr, vPtr);
		lastele = pGarcol->insert(sPtr); return sPtr;
	}

	template<typename vec, typename ary>
	Statement* GetAssignAptr(AptrData<vec>* ptr, VarType vt)
	{
		Statement* sPtr;
		ary* uPtr = dynamic_cast<ary*>(vt.dPtr);
		if(uPtr)
		{	
			sPtr = new AssignAptr<vec, ary>(ptr, uPtr);
			lastele = pGarcol->insert(sPtr); 
			return sPtr;
		}
		
		HandleSyntaxError(_T("Unsported data type.\r\n"));

		return NULL; // to suppress warning
	}

	template<typename vec>
	Statement* GetAssignAptrAptr(AptrData<vec>* ptr1, AptrData<vec>* ptr2)
	{
		AssignAptrAptr<vec> *ptr = new AssignAptrAptr<vec>(ptr1, ptr2);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	DeclUdt* GetDeclUdt(const wchar_t* vid, UserDefinedType *pUdt, ExpUserPtr xpr=NULL)
	{
		DeclUdt* ptr= new DeclUdt(vid, pUdt, xpr, static_set);
		lastele = pGarcol->insert(ptr); return ptr;
	}
	
	template<typename T>
	Declare<T>* GetDeclare(const wchar_t* vid, Expression<T>* xpr=NULL)
	{
		Declare<T>* sPtr = new Declare<T>(vid, xpr, static_set);
		lastele = pGarcol->insert(sPtr); return sPtr;
	}

	template<typename T>
	DeclareLiteral<T>* GetDeclareLiteral(const wchar_t* vid, Expression<T>* xpr)
	{
		DeclareLiteral<T>* ptr = new DeclareLiteral<T>(vid, xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	LiteralData<T>* GetLiteralData(const wchar_t* vid, Expression<T>* xpr)
	{
		LiteralData<T>* ptr = new LiteralData<T>(vid, xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	DeclarePtr<T>* GetDeclarePtr(const wchar_t* vrid, LeftHandSide<T>* pLHS=NULL)
	{
		DeclarePtr<T>* ptr = new DeclarePtr<T>(vrid, pLHS, static_set);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	DeclarePtr<UserData>* GetDeclUdtPtr(const wchar_t* vrid, UserDefinedType *ftry, LeftHandSide<UserData>* pLHS=NULL)
	{
		DeclarePtr<UserData>* ptr = new DeclarePtr<UserData>(vrid, ftry, pLHS, static_set);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename T>
	DeclarePtr<T>* GetDeclarePtr(const wchar_t* vrid, Expression<T*>* xpr)
	{
		DeclarePtr<T>* ptr = new DeclarePtr<T>(vrid, NULL, static_set);
		ptr->pExpr = xpr; lastele = pGarcol->insert(ptr); return ptr;
	}

	template<typename vec>
	DeclareAptr<vec>* GetDeclareAptr(const wchar_t* vrid, LeftHandSideArray<vec>* pLHS=NULL)
	{
		DeclareAptr<vec>* ptr = new DeclareAptr<vec>(vrid, pLHS, static_set);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	StatementList* GetStmtList()
	{
		StatementList* ptr = new StatementList();
		lastele = pGarcol->insert(ptr); return ptr;
	}

	Function* GetFunction(DataType rtn, const std::wstring& name)
	{
		extern Stack<ScanFile> gScanFileStack;
		Function* ptr = new Function(rtn, name);
		lastele = pGarcol->insert(ptr); 
		ptr->scanfile = gScanFileStack.top(); return ptr;
	}

	template<typename T>
	ArgExpr<T>* GetArgExpr(Expression<T>* xpr, bool cbr=false)
	{
		ArgExpr<T>* ptr = new ArgExpr<T>(xpr, cbr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	ExpReal* GetFunSimple(const FunName& fname, ExpReal* xpr)
	{
		RFunR* ptr = new RFunR(fname, xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	ExpReal* GetFunSimple(const FunName& fname, ExpIntg* xpr)
	{
		RFunI* ptr = new RFunI(fname, xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	ExpCplx* GetFunSimple(const FunName& fname, ExpCplx* xpr)
	{
		CFunC* ptr = new CFunC(fname, xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	ExpVctr* GetFunSimple(const FunName& fname, ExpVctr* xpr)
	{
		VFunV* ptr = new VFunV(fname, xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	ExpReal* GetFunSimple(const FunName& fname, 
		ExpReal* xpr1, ExpReal* xpr2)
	{
		RFunRR* ptr = new RFunRR(fname, xpr1, xpr2);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	ExpReal* GetFunSimple(const FunName& fname, 
		ExpIntg* xpr1, ExpIntg* xpr2)
	{
		RFunII* ptr = new RFunII(fname, xpr1, xpr2);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	ExpReal* GetFunSimple(const FunName& fname, 
		ExpReal* xpr1, ExpReal* xpr2, ExpReal* xpr3)
	{
		RFunRRR* ptr = new RFunRRR(fname, xpr1, xpr2, xpr3);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	ExpReal* GetFunSimple(const FunName& fname, 
		ExpReal* xpr1, ExpReal* xpr2,
		ExpReal* xpr3, ExpReal* xpr4)
	{
		RFunRRRR* ptr = new RFunRRRR(fname, xpr1, xpr2, xpr3, xpr4);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	ExpCplx* GetFunSimple(const FunName& fname, 
		ExpCplx* xpr1, ExpCplx* xpr2,
		ExpCplx* xpr3, ExpCplx* xpr4)
	{
		CFunCCCC* ptr = new CFunCCCC(fname, xpr1, xpr2, xpr3, xpr4);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	ExpCplx* GetFunSimple(const FunName& fname, 
		ExpCplx* xpr1, ExpCplx* xpr2)
	{
		CFunCC* ptr = new CFunCC(fname, xpr1, xpr2);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	ExpCplx* GetCFunRR(const FunName& fname, 
		ExpReal* xpr1, ExpReal* xpr2)
	{
		CFunRR* ptr = new CFunRR(fname, xpr1, xpr2);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	ExpReal* GetRFunC(const FunName& fname, ExpCplx* xpr)
	{
		RFunC* ptr = new RFunC(fname, xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	ExpReal* GetRFunV(const FunName& fname, ExpVctr* xpr)
	{
		RFunV* ptr = new RFunV(fname, xpr);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	ExpVctr* GetVFunRRR(const FunName& fname, 
		ExpReal* xpr1, ExpReal* xpr2, ExpReal* xpr3)
	{
		VFunRRR* ptr = new VFunRRR(fname, xpr1, xpr2, xpr3);
		lastele = pGarcol->insert(ptr); return ptr;
	}

	Expression<voidtype>* GetVOFunMRMR(const FunName& fname, VarType v1, VarType v2)
	{
		Array2D<realnum>* ptr1 = dynamic_cast<Array2D<realnum>*>(v1.dPtr);
		Array2D<realnum>* ptr2 = dynamic_cast<Array2D<realnum>*>(v2.dPtr);

		if(ptr1 && ptr2)
		{
			VOFunMRMR* ptr = new VOFunMRMR(fname, ptr1, ptr2);
			lastele = pGarcol->insert(ptr); return ptr;
		}
		
		HandleSyntaxError(_T("Invalid Argument.\r\n"));

		return NULL; // to suppress warning
	}

	UserDefinedType * GetUserDefinedType(const std::wstring& tagname)
	{
		extern Stack<bool> gBlockTypeStack;
		extern Stack<UserDefinedType*> gFactoryStack;

		UserDefinedType* ptr = new UserDefinedType(tagname);
		lastele = pGarcol->insert(ptr); 
		
		if(!gFactoryStack.empty() && 
			gBlockTypeStack.size()>1 && 
			gBlockTypeStack.getoffset(1))
		{
			ptr->pEnclosingUDT = gFactoryStack.top();
		}
		
		return ptr;
	}
};

