#include "stdafx.h"

#include <kch_flex.h>
#include <kch_path.h>
#include <string>
#include <sstream>

#include "lexer_util.h"

YY_BUFFER_STATE include_stack[MAX_INCLUDE_DEPTH];

int include_stack_ptr = 0;

bool gInput;
char *scanbuffer;
int eof_seen, send_token;
int scanbuffer_size;
int scanbuffer_fsize;

std::wstring gPrevToken, gLastToken;
std::set<CString> gIncludeSet, gIncludeSetGlobal;

extern ArrayInit gArrayInit;

#ifdef _DEBUG

Stack<ScanFile> gPuncCurly(_T("{}"));
Stack<ScanFile> gPuncParan(_T("()"));

#else

Stack<ScanFile> gPuncCurly;
Stack<ScanFile> gPuncParan;

#endif

bool PrepareLexer()
{
	ResetLexer();
	
	gPuncCurly.clear();
	gPuncParan.clear();

	gBlockTypeStack.clear();
	gCatchBlockStack.clear();

	gArrayInit=MakeAI();

	gLastToken=gPrevToken=L"";
	
	gLastUdt = NULL;
	 
	eof_seen = 0;
		
	vartype = TypeNull;
				
	include_stack_ptr = 0;
	
	send_token = -1;						
	
	unitvctr_set = false;
	unitcplx_set = false;
		
	static_set = false; 
	goto_set = false;
	class_set = false;
	assign_set = false;
	literal_set = false;
	arg_set = false;
	point_set = false;
	sol_set = false;

	if(yy_scan_buffer(scanbuffer, scanbuffer_fsize))
	{
		gScanFileStack.clear(); gIncludeSet.clear();
		
		if(gStmtListGlobal) gIncludeSet = gIncludeSetGlobal;
		gIncludeSet.insert(gDocTitle);

		gScanFileStack.push(ScanFile(gDocTitle)); 
		if(!gScanFileStack.empty())
			gScanFile = gScanFileStack.top();
		
		return true;
	}
	else return false;

}

bool InitializeLexer()
{
	initial_set = 0;

	fmt_clear = true;
	fmt_precision = 7;
	fmt_showpoint = false;
	fmt_scientific = false;
		
	unitvctr = false; 
	unitcplx = false; 
	
	return PrepareLexer();
}

bool InitializeLexerForInput()
{
	return PrepareLexer();
}

void yycleanup()
{
	while(!gScanBuffer.empty()) 
	{
		delete [] gScanBuffer.top(); gScanBuffer.pop();
	}
	
	while(include_stack_ptr>0)
	{
		yy_delete_buffer(include_stack[--include_stack_ptr]);
	}
	
	extern void DeleteCurrentBuffer();

	DeleteCurrentBuffer(); include_stack_ptr = 0;
}


bool PrepareNewFileScan(const CString& fname)
{
	CString contents; int buf_size; 
	
	if(LoadFileContents(fname, contents))
	{
		buf_size = contents.GetLength()*2+2;
		char* pScanBuffer = new char[buf_size];
		PrepareScanBuffer(contents, pScanBuffer);
		
		include_stack[include_stack_ptr++] = GetCurrentBuffer();
		yy_scan_buffer(pScanBuffer, buf_size);

		gScanBuffer.push(pScanBuffer);
		gScanFileStack.push(ScanFile(fname));
					
		return true;
	}

	return false;
}

void StrCopy(wchar_t* tgt, const wchar_t* src)
{
	if(::lstrlen(src) > BUF_LENGTH -1)
	{
		CString msg, fmt;
		
		fmt.LoadStringW(IDS_ERROR_ID_TOOLONG);
		msg.FormatMessageW(fmt, src); PutString(msg);
		
		fmt.LoadStringW(IDS_MSG_MAXID_LENGTH); 
		msg.FormatMessage(fmt, BUF_LENGTH-1);
		HandleSyntaxError(msg.GetString());
	}
	else ::lstrcpy(tgt, src);
}

void StrCopy(wchar_t* tgt, const std::wstring& src)
{
	StrCopy(tgt, src.c_str());
}

int GetFunType(FunName& fn, YYSTYPE *lvalp)
{
	std::wstring id(fn.fn);

	gflist.find(fn);
	
	switch(fn.fsig.type)
	{
		case Fun_VO_MAKEUNIT: return FN_VO_MAKEUNIT;
		case Fun_VO_MR_MR:	return FN_VO_MR_MR;
		case Fun_I_VO:		return FN_INTG;
		case Fun_I_II:		return FN_I_II;
		case Fun_R_R:       return FN_REAL;
		case Fun_R_I:		return FN_R_I;
		case Fun_R_II:		return FN_R_II;
		case Fun_R_RR:      return FN_R_RR;
		case Fun_R_RRR:     return FN_R_RRR;
		case Fun_R_RRRR:    return FN_R_RRRR;
		case Fun_R_C:       return FN_R_C;
		case Fun_R_V:   	return FN_R_V;
		case Fun_C_C:		return FN_C_C;
		case Fun_C_RR: 		return FN_C_RR;
		case Fun_C_CC: 		return FN_C_CC;
		case Fun_C_CCCC: 	return FN_C_CCCC;
		case Fun_V_V:   	return FN_V_V;
		case Fun_V_RRR:  	return FN_V_RRR;
		case Fun_B_S:		return FN_B_S;
						
		default: // FunNull
			
			StrCopy(lvalp->bufr, id.c_str());
			//if(class_set) 
			//{	
			//	return VID_CLASS;
			//}
			//else
			//{
				if(gFactoryStack.empty())
				{
					return TYPE_UNDF;
				}
				else 
				{
					/////////////////////////////////////
					DataType type  = gFactoryStack.top()->GetElementType(id);
					
					switch(type)
					{
					
					case TypeIntgL:
						{
							LiteralData<integer>* ptr = 
								static_cast<LiteralData<integer>*>(gFactoryStack.top()->GetData(id));
							lvalp->eint = ptr->GetExpression(); vartype=TypeNull; return MEM_LTL_INTG;
						}

					case TypeRealL:
						{
							LiteralData<realnum>* ptr = 
								static_cast<LiteralData<realnum>*>(gFactoryStack.top()->GetData(id));
							lvalp->real = ptr->GetExpression(); vartype=TypeNull; return MEM_LTL_REAL;
						}

					case TypeCplxL:
						{
							LiteralData<Complex>* ptr = 
								static_cast<LiteralData<Complex>*>(gFactoryStack.top()->GetData(id));
							lvalp->cplx = ptr->GetExpression(); vartype=TypeNull; return MEM_LTL_CPLX;
						}

					case TypeVctrL:
						{
							LiteralData<Vector>* ptr = 
								static_cast<LiteralData<Vector>*>(gFactoryStack.top()->GetData(id));
							lvalp->vctr = ptr->GetExpression(); vartype=TypeNull; return MEM_LTL_VCTR;
						}

					case TypeStrn:
						{
							LiteralData<string>* ptr = 
								static_cast<LiteralData<string>*>(gFactoryStack.top()->GetData(id));
							lvalp->strn = ptr->GetExpression(); vartype=TypeNull; return MEM_LTL_STRN;
						}

					default:

						////////////////////////////////////////
						StrCopy(lvalp->bufr, id.c_str()); 
						return TYPE_UNDF;
					}
			//}
		}
	}
}

bool ConflictTest(const wchar_t* id)
{
	// return true, if conflict
	// else return false
	
	extern bool unitvctr;
	extern bool unitcplx;
	
	CString msg, fmt, prt, id_str(id);
		
	if(unitvctr)
	{
		if(STR_EQL_UNICODE(id, vctr_xcom_opr)) // conflict with vctr_x
		{
			fmt.LoadStringW(IDS_VEC_VAR_CONFLICT);
			prt.LoadStringW(IDS_VEC_FIRST);
			msg.FormatMessageW(fmt, id_str, prt);			
			PutString(msg); return true;
		}
		
		if(STR_EQL_UNICODE(id, vctr_ycom_opr)) // conflict with vctr_y
		{	
			fmt.LoadStringW(IDS_VEC_VAR_CONFLICT);
			prt.LoadStringW(IDS_VEC_SECOND);
			msg.FormatMessageW(fmt, id_str, prt);			
			PutString(msg); return true;
		}
		
		if(STR_EQL_UNICODE(id, vctr_zcom_opr)) // conflict with vctr_z
		{	
			fmt.LoadStringW(IDS_VEC_VAR_CONFLICT);
			prt.LoadStringW(IDS_VEC_THIRD);
			msg.FormatMessageW(fmt, id_str, prt);			
			PutString(msg); return true;
		}
	}
			
	if(unitcplx)
	{
		if(STR_EQL_UNICODE(id, cplx_imag_opr)) 
		{	
			fmt.LoadStringW(IDS_IMG_VAR_CONFLICT);
			msg.FormatMessageW(fmt, id_str);			
			PutString(msg); return true;
		}
		
		if(STR_EQL_UNICODE(id, cplx_real_opr))
		{	
			fmt.LoadStringW(IDS_REL_VAR_CONFLICT);
			msg.FormatMessageW(fmt, id_str);			
			PutString(msg); return true;
		}
	}
	
	return false;
}

int GetAptrType(Data* dPtr, YYSTYPE *lvalp)
{
	if(lvalp->ap1i = dynamic_cast<AptrData<IntgVec1D>*>(dPtr))
		return PTR_AP1I;
	else if(lvalp->ap1r = dynamic_cast<AptrData<RealVec1D>*>(dPtr))
		return PTR_AP1R;
	else if(lvalp->ap1c = dynamic_cast<AptrData<CplxVec1D>*>(dPtr))
		return PTR_AP1C;
	else if(lvalp->ap1v = dynamic_cast<AptrData<VctrVec1D>*>(dPtr))
		return PTR_AP1V;
		
	else if(lvalp->ap2i = dynamic_cast<AptrData<IntgVec2D>*>(dPtr))
		return PTR_AP2I;
	else if(lvalp->ap2r = dynamic_cast<AptrData<RealVec2D>*>(dPtr))
		return PTR_AP2R;
	else if(lvalp->ap2c = dynamic_cast<AptrData<CplxVec2D>*>(dPtr))
		return PTR_AP2C;
	else if(lvalp->ap2v = dynamic_cast<AptrData<VctrVec2D>*>(dPtr))
		return PTR_AP2V;
		
	else if(lvalp->ap3i = dynamic_cast<AptrData<IntgVec3D>*>(dPtr))
		return PTR_AP3I;
	else if(lvalp->ap3r = dynamic_cast<AptrData<RealVec3D>*>(dPtr))
		return PTR_AP3R;
	else if(lvalp->ap3c = dynamic_cast<AptrData<CplxVec3D>*>(dPtr))
		return PTR_AP3C;
	else if(lvalp->ap3v = dynamic_cast<AptrData<VctrVec3D>*>(dPtr))
		return PTR_AP3V;
	
	else if(lvalp->ap4i = dynamic_cast<AptrData<IntgVec4D>*>(dPtr))
		return PTR_AP4I;
	else if(lvalp->ap4r = dynamic_cast<AptrData<RealVec4D>*>(dPtr))
		return PTR_AP4R;
	else if(lvalp->ap4c = dynamic_cast<AptrData<CplxVec4D>*>(dPtr))
		return PTR_AP4C;
	else if(lvalp->ap4v = dynamic_cast<AptrData<VctrVec4D>*>(dPtr))
		return PTR_AP4V;
		
	else if(lvalp->ap5i = dynamic_cast<AptrData<IntgVec5D>*>(dPtr))
		return PTR_AP5I;
	else if(lvalp->ap5r = dynamic_cast<AptrData<RealVec5D>*>(dPtr))
		return PTR_AP5R;
	else if(lvalp->ap5c = dynamic_cast<AptrData<CplxVec5D>*>(dPtr))
		return PTR_AP5C;
	else if(lvalp->ap5v = dynamic_cast<AptrData<VctrVec5D>*>(dPtr))
		return PTR_AP5V;
	else
		return HandleSyntaxError(_T("Pointer to Array, type not supported.\r\n"));
}

int GetPtrType(Data *dPtr, YYSTYPE *lvalp)
{
	if(lvalp->bptr=dynamic_cast<PtrData<bool>*>(dPtr))
		return PTR_BOOL;
	else if(lvalp->iptr=dynamic_cast<PtrData<integer>*>(dPtr))
		return PTR_INTG;
	else if(lvalp->rptr=dynamic_cast<PtrData<realnum>*>(dPtr))
		return PTR_REAL;
	else if(lvalp->cptr=dynamic_cast<PtrData<Complex>*>(dPtr))
		return PTR_CPLX;
	else if(lvalp->vptr=dynamic_cast<PtrData<Vector>*>(dPtr))
		return PTR_VCTR;
	else if(lvalp->sptr=dynamic_cast<PtrData<string>*>(dPtr))
		return PTR_STRN;
	else if(lvalp->uptr=dynamic_cast<PtrData<UserData>*>(dPtr))
	{
		gUdtFactoryStack.clear(); 
		gUdtFactoryStack.push(lvalp->uptr->factory);
		return PTR_USER;
	}
	else
	{
		HandleSyntaxError(_T("Unsported data pointer.\r\n"));
		return 0; // to suppress warning message;
	}
}

int GetArrayType(Data *dPtr)
{
	if(dynamic_cast<Array1D<integer>*>(dPtr))      return ARY_INTG1D;
	else if(dynamic_cast<Array1D<realnum>*>(dPtr)) return ARY_REAL1D;
	else if(dynamic_cast<Array1D<Complex>*>(dPtr)) return ARY_CPLX1D;
	else if(dynamic_cast<Array1D<Vector>*>(dPtr))  return ARY_VCTR1D;
	
	else if(dynamic_cast<Array2D<integer>*>(dPtr)) return ARY_INTG2D;
	else if(dynamic_cast<Array2D<realnum>*>(dPtr)) return ARY_REAL2D;
	else if(dynamic_cast<Array2D<Complex>*>(dPtr)) return ARY_CPLX2D;
	else if(dynamic_cast<Array2D<Vector>*>(dPtr))  return ARY_VCTR2D;
	
	else if(dynamic_cast<Array3D<integer>*>(dPtr)) return ARY_INTG3D;
	else if(dynamic_cast<Array3D<realnum>*>(dPtr)) return ARY_REAL3D;
	else if(dynamic_cast<Array3D<Complex>*>(dPtr)) return ARY_CPLX3D;
	else if(dynamic_cast<Array3D<Vector>*>(dPtr))  return ARY_VCTR3D;
		
	else if(dynamic_cast<Array4D<integer>*>(dPtr)) return ARY_INTG4D;
	else if(dynamic_cast<Array4D<realnum>*>(dPtr)) return ARY_REAL4D;
	else if(dynamic_cast<Array4D<Complex>*>(dPtr)) return ARY_CPLX4D;
	else if(dynamic_cast<Array4D<Vector>*>(dPtr))  return ARY_VCTR4D;
		
	else if(dynamic_cast<Array5D<integer>*>(dPtr)) return ARY_INTG5D;
	else if(dynamic_cast<Array5D<realnum>*>(dPtr)) return ARY_REAL5D;
	else if(dynamic_cast<Array5D<Complex>*>(dPtr)) return ARY_CPLX5D;
	else if(dynamic_cast<Array5D<Vector>*>(dPtr))  return ARY_VCTR5D;
	else 
	{
		HandleSyntaxError(_T("Array type unclassified.\r\n"));
		return 0; // to suppress warning message
	}
}

int GetIdType(const wchar_t* id, YYSTYPE *lvalp)
{
	extern bool unitvctr, unitcplx;
	
	if(unitcplx)
	{
		if(STR_EQL_UNICODE(id, cplx_imag_opr))       return CPLX_OPR_IMAG;
		else if(STR_EQL_UNICODE(id, cplx_real_opr))  return CPLX_OPR_REAL;
	}
	
	if(unitvctr)
	{
		if(STR_EQL_UNICODE(id, vctr_xcom_opr)) 		 return OPR_X;
		else if(STR_EQL_UNICODE(id, vctr_ycom_opr))  return OPR_Y;
		else if(STR_EQL_UNICODE(id, vctr_zcom_opr))  return OPR_Z;
	}
		
	DataType type = LookupDataAndType(id, &lvalp->vart.dPtr);
		
	if(type!=TypeNull && ConflictTest(id)) 
		return HandleSyntaxError(_T("ID conflicts.\r\n"));
	
	switch(type)
	{	
	case TypeVoid: return VAR_VOID;
	case TypeBool: StrCopy(lvalp->vart.bufr, id); return VAR_BOOL;
	case TypeIntg: StrCopy(lvalp->vart.bufr, id); return VAR_INTG;
	case TypeReal: StrCopy(lvalp->vart.bufr, id); return VAR_REAL;
	case TypeCplx: StrCopy(lvalp->vart.bufr, id); return VAR_CPLX;
	case TypeVctr: StrCopy(lvalp->vart.bufr, id); return VAR_VCTR;
	case TypeStrn: StrCopy(lvalp->vart.bufr, id); return VAR_STRN;
	
	case TypeIntgL:
		{
			LiteralData<integer>* ptr = 
				static_cast<LiteralData<integer>*>(lvalp->vart.dPtr);
			lvalp->eint = ptr->GetExpression(); return LTL_INTG;
		}
	case TypeRealL:
		{
			LiteralData<realnum>* ptr = 
				static_cast<LiteralData<realnum>*>(lvalp->vart.dPtr);
			lvalp->real = ptr->GetExpression(); return LTL_REAL;
		}
	case TypeCplxL:
		{
			LiteralData<Complex>* ptr = 
				static_cast<LiteralData<Complex>*>(lvalp->vart.dPtr);
			lvalp->cplx = ptr->GetExpression(); return LTL_CPLX;
		}
	case TypeVctrL:
		{
			LiteralData<Vector>* ptr = 
				static_cast<LiteralData<Vector>*>(lvalp->vart.dPtr);
			lvalp->vctr = ptr->GetExpression(); return LTL_VCTR;
		}
	case TypeStrnL:
		{
			LiteralData<string>* ptr = 
				static_cast<LiteralData<string>*>(lvalp->vart.dPtr);
			lvalp->strn = ptr->GetExpression(); return LTL_STRN;
		}
	case TypeUser:
	{	
		gUdtFactoryStack.clear(); 
		gUdtFactoryStack.push(static_cast<UserData*>(lvalp->vart.dPtr)->factory);
		return VAR_USER;
	}
	case TypeArray:
		StrCopy(lvalp->vart.bufr, id); return GetArrayType(lvalp->vart.dPtr);
	case TypePtr:
		StrCopy(lvalp->vart.bufr, id); return GetPtrType(lvalp->vart.dPtr, lvalp);
	case TypeAptr:
		StrCopy(lvalp->vart.bufr, id); return GetAptrType(lvalp->vart.dPtr, lvalp);
	
	default:
		{
			Function* fPtr = FindFunction(id);
			lvalp->pfunc = fPtr;
			
			if(fPtr)
			{
				switch(fPtr->rtntype)
				{	
				case TypeVoid: return FNC_VOID;
				case TypeBool: return FNC_BOOL;
				case TypeIntg: return FNC_INTG;
				case TypeReal: return FNC_REAL;
				case TypeCplx: return FNC_CPLX;
				case TypeVctr: return FNC_VCTR;
				case TypeStrn: return FNC_STRN;

				case TypeUser: return FNC_USER;

				case TypeBPtr: return FNC_BPTR;
				case TypeSPtr: return FNC_SPTR;
				case TypeIPtr: return FNC_IPTR;
				case TypeRPtr: return FNC_RPTR;
				case TypeCPtr: return FNC_CPTR;
				case TypeVPtr: return FNC_VPTR;
				
				case TypeAi1D: return FNC_AI1D;
				case TypeAr1D: return FNC_AR1D;
				case TypeAc1D: return FNC_AC1D;
				case TypeAv1D: return FNC_AV1D;
				
				case TypeAi2D: return FNC_AI2D;
				case TypeAr2D: return FNC_AR2D;
				case TypeAc2D: return FNC_AC2D;
				case TypeAv2D: return FNC_AV2D;
					
				case TypeAi3D: return FNC_AI3D;
				case TypeAr3D: return FNC_AR3D;
				case TypeAc3D: return FNC_AC3D;
				case TypeAv3D: return FNC_AV3D;
								
				case TypeAi4D: return FNC_AI4D;
				case TypeAr4D: return FNC_AR4D;
				case TypeAc4D: return FNC_AC4D;
				case TypeAv4D: return FNC_AV4D;
									
				case TypeAi5D: return FNC_AI5D;
				case TypeAr5D: return FNC_AR5D;
				case TypeAc5D: return FNC_AC5D;
				case TypeAv5D: return FNC_AV5D;
					
				default:
					return HandleSyntaxError(_T("Function Type Unclassified.\r\n"));	
				}
			}
			else
			{
				UserDefinedType *udt = LookupUdt(id);
				if(udt)
				{
					gUdtFactoryStack.clear(); gUdtFactoryStack.push(udt); 
					vartype = TypeUser; return TYPE_UDT;
				}
				else
				{
					StrCopy(lvalp->fname.fn, id);
					return GetFunType(lvalp->fname, lvalp);
				}
			}
		}
	}
}

int ReturnMemType(int rtn)
{
	vartype = TypeNull; return rtn;
}

// This is called when point_set == true
int GetEleType(const std::wstring& id, YYSTYPE *lvalp)
{
	if(!gUdtFactoryStack.empty())
	{
		
		DataType type  = gUdtFactoryStack.top()->GetElementType(id);
		StrCopy(lvalp->bufr, id.c_str()); 
		switch(type)
		{
		
		case TypeBool: return ReturnMemType(MEM_BOOL);  
		case TypeStrn: return ReturnMemType(MEM_STRN);
		case TypeIntg: return ReturnMemType(MEM_INTG);
		case TypeReal: return ReturnMemType(MEM_REAL);  
		case TypeCplx: return ReturnMemType(MEM_CPLX);
		case TypeVctr: return ReturnMemType(MEM_VCTR);

		case TypeBPtr: return ReturnMemType(MEM_BPTR);  
		case TypeSPtr: return ReturnMemType(MEM_SPTR);
		case TypeIPtr: return ReturnMemType(MEM_IPTR);
		case TypeRPtr: return ReturnMemType(MEM_RPTR);  
		case TypeCPtr: return ReturnMemType(MEM_CPTR);
		case TypeVPtr: return ReturnMemType(MEM_VPTR);
					
		case TypeAi1D: return ReturnMemType(MEM_AI1D); 
		case TypeAr1D: return ReturnMemType(MEM_AR1D);
		case TypeAc1D: return ReturnMemType(MEM_AC1D); 
		case TypeAv1D: return ReturnMemType(MEM_AV1D);
		
		case TypeAi2D: return ReturnMemType(MEM_AI2D); 
		case TypeAr2D: return ReturnMemType(MEM_AR2D);
		case TypeAc2D: return ReturnMemType(MEM_AC2D); 
		case TypeAv2D: return ReturnMemType(MEM_AV2D);
		
		case TypeAi3D: return ReturnMemType(MEM_AI3D); 
		case TypeAr3D: return ReturnMemType(MEM_AR3D);
		case TypeAc3D: return ReturnMemType(MEM_AC3D); 
		case TypeAv3D: return ReturnMemType(MEM_AV3D);
		
		case TypeAi4D: return ReturnMemType(MEM_AI4D); 
		case TypeAr4D: return ReturnMemType(MEM_AR4D);
		case TypeAc4D: return ReturnMemType(MEM_AC4D); 
		case TypeAv4D: return ReturnMemType(MEM_AV4D);
		
		case TypeAi5D: return ReturnMemType(MEM_AI5D); 
		case TypeAr5D: return ReturnMemType(MEM_AR5D);
		case TypeAc5D: return ReturnMemType(MEM_AC5D); 
		case TypeAv5D: return ReturnMemType(MEM_AV5D);
		
		case TypeIntgL:
			{
				LiteralData<integer>* ptr = 
					static_cast<LiteralData<integer>*>(gUdtFactoryStack.top()->GetData(id));
				lvalp->eint = ptr->GetExpression(); vartype=TypeNull; return MEM_LTL_INTG;
			}

		case TypeRealL:
			{
				LiteralData<realnum>* ptr = 
					static_cast<LiteralData<realnum>*>(gUdtFactoryStack.top()->GetData(id));
				lvalp->real = ptr->GetExpression(); vartype=TypeNull; return MEM_LTL_REAL;
			}

		case TypeCplxL:
			{
				LiteralData<Complex>* ptr = 
					static_cast<LiteralData<Complex>*>(gUdtFactoryStack.top()->GetData(id));
				lvalp->cplx = ptr->GetExpression(); vartype=TypeNull; return MEM_LTL_CPLX;
			}

		case TypeVctrL:
			{
				LiteralData<Vector>* ptr = 
					static_cast<LiteralData<Vector>*>(gUdtFactoryStack.top()->GetData(id));
				lvalp->vctr = ptr->GetExpression(); vartype=TypeNull; return MEM_LTL_VCTR;
			}

		case TypeStrnL:
			{
				LiteralData<string>* ptr = 
					static_cast<LiteralData<string>*>(gUdtFactoryStack.top()->GetData(id));
				lvalp->strn = ptr->GetExpression(); vartype=TypeNull; return MEM_LTL_STRN;
			}

		case TypeUser: 
				gUdtFactoryStack.push(gUdtFactoryStack.top()->GetEleUdt(id)); 
				return MEM_USER;

		case TypeUPtr: 
				gUdtFactoryStack.push(gUdtFactoryStack.top()->GetEleUdt(id)); 
				return MEM_UPTR;

		default:
			{
				Function *fPtr = gUdtFactoryStack.top()->FindFunction(id);

				if(fPtr)
				{
					lvalp->pfunc = fPtr;
					gUdtFactoryStack.clear(); 
					vartype = TypeNull;

					switch(fPtr->rtntype)
					{	
					case TypeVoid: return FNC_VOID;
					case TypeBool: return FNC_BOOL;
					case TypeIntg: return FNC_INTG;
					case TypeReal: return FNC_REAL;
					case TypeCplx: return FNC_CPLX;
					case TypeVctr: return FNC_VCTR;
					case TypeStrn: return FNC_STRN;

					case TypeUser: return FNC_USER;

					case TypeBPtr: return FNC_BPTR;
					case TypeSPtr: return FNC_SPTR;
					case TypeIPtr: return FNC_IPTR;
					case TypeRPtr: return FNC_RPTR;
					case TypeCPtr: return FNC_CPTR;
					case TypeVPtr: return FNC_VPTR;
					
					case TypeAi1D: return FNC_AI1D;
					case TypeAr1D: return FNC_AR1D;
					case TypeAc1D: return FNC_AC1D;
					case TypeAv1D: return FNC_AV1D;
					
					case TypeAi2D: return FNC_AI2D;
					case TypeAr2D: return FNC_AR2D;
					case TypeAc2D: return FNC_AC2D;
					case TypeAv2D: return FNC_AV2D;
						
					case TypeAi3D: return FNC_AI3D;
					case TypeAr3D: return FNC_AR3D;
					case TypeAc3D: return FNC_AC3D;
					case TypeAv3D: return FNC_AV3D;
									
					case TypeAi4D: return FNC_AI4D;
					case TypeAr4D: return FNC_AR4D;
					case TypeAc4D: return FNC_AC4D;
					case TypeAv4D: return FNC_AV4D;
										
					case TypeAi5D: return FNC_AI5D;
					case TypeAr5D: return FNC_AR5D;
					case TypeAc5D: return FNC_AC5D;
					case TypeAv5D: return FNC_AV5D;
						
					default:
						return HandleSyntaxError(_T("Member function Type Unclassified.\r\n"));	
					}
				}
			}
		}
	}

	return 0;
}

// This is called when point_set == true
int GetUdtType(const std::wstring& id, YYSTYPE *lvalp)
{
	if(!gUdtFactoryStack.empty())
	{
		
		UserDefinedType *pUdt = gUdtFactoryStack.top()->GetUdt(id);
		UserDefinedType *pEleUdt = gUdtFactoryStack.top()->GetEleUdt(id);
		
		if(pUdt)
		{
			gUdtFactoryStack.clear(); gUdtFactoryStack.push(pUdt); 
			vartype = TypeUser; return TYPE_UDT;
		}
		else if(pEleUdt)
		{
			gUdtFactoryStack.clear(); gUdtFactoryStack.push(pEleUdt); 
			vartype = TypeUser; return TYPE_UDT;
		}

	}

	if(!gFactoryStack.empty())
	{
		
		UserDefinedType *pUdt = gFactoryStack.top()->GetUdt(id);
		UserDefinedType *pEleUdt = gFactoryStack.top()->GetEleUdt(id);
		
		if(pUdt)
		{
			gUdtFactoryStack.clear(); gUdtFactoryStack.push(pUdt); 
			vartype = TypeUser; return TYPE_UDT;
		}
		else if(pEleUdt)
		{
			gUdtFactoryStack.clear(); gUdtFactoryStack.push(pEleUdt); 
			vartype = TypeUser; return TYPE_UDT;
		}
	}

	return 0;
}

int GetMemType(const std::wstring& id, YYSTYPE *lvalp)
{
	if(unitcplx)
	{
		if(id==cplx_real_opr)
		{
			point_set = false; return CPLX_R;
		}
		else if(id==cplx_imag_opr)
		{
			point_set = false; return CPLX_I;
		}
	}
	
	if(unitvctr)
	{
		if(id==vctr_xcom_opr)
		{
			point_set = false; return VCTR_X;
		}
		else if(id==vctr_ycom_opr)
		{
			point_set = false; return VCTR_Y;
		}
		else if(id==vctr_zcom_opr)
		{
			point_set = false; return VCTR_Z;
		}
	}
	
	switch(gTransOprType.GetKeywordID(id.c_str()))
	{
		case opr_size: point_set = false; return OPR_SIZE;
		case opr_resize: point_set = false; return OPR_RESIZE;
		case opr_dimension: point_set = false; return OPR_DIMENSION ;
		
		case opr_real: point_set = false; return CPLX_R;
		case opr_imag: point_set = false; return CPLX_I;
		
		case opr_xcom: point_set = false; return VCTR_X;
		case opr_ycom: point_set = false; return VCTR_Y;
		case opr_zcom: point_set = false; return VCTR_Z;
	}
	
	int rlt;
	if((rlt=GetEleType(id, lvalp))!=0) 
	{
		point_set = false; return rlt;
	}
	if((rlt=GetUdtType(id, lvalp))!=0) 
	{
		point_set = false; return rlt;
	}
	
	CString msg, fmt; fmt.LoadStringW(IDS_ERROR_MEM_NOTFOUND);
	msg.FormatMessageW(fmt, (LPCSTR)id.c_str()); 
	return HandleSyntaxError(msg.GetString()); 
}

int GetSolveMethod(const std::wstring& id)
{
	sol_set = false;
	
	if(id == std::wstring(_T("detcramer"))) return SOL_DETCRAMER;
	else if(id == std::wstring(_T("solvecramer"))) return SOL_SOLVECRAMER;
	else if(id == std::wstring(_T("gaussjordan"))) return SOL_GAUSSJORDAN;
	else if(id == std::wstring(_T("ludecompo"))) return SOL_LUDECOMPO;
	else if(id == std::wstring(_T("lumethod"))) return SOL_LUMETHOD;
	else if(id == std::wstring(_T("inverse"))) return SOL_INVERSE;
	else
	{
		std::wostringstream os;
		os<<_T("Solve ?? operator's method, ")<<id<<_T(", is not defined\r\n");
		HandleSyntaxError(os.str());
		return 0 ; // to suppress warning
	}
}
int GetVarType(const wchar_t* id, YYSTYPE *lvalp)
{
	UserDefinedType *udt = NULL;
	
	if(goto_set) { goto_set = false; return GOTO_TARGET; }
	else if(sol_set) { return GetSolveMethod(id); }
	else if(point_set) return GetMemType(id, lvalp);
	else if(assign_set) return GetIdType(id, lvalp);
	else if(unitvctr_set) return VCTR_UNIT;
	else if(unitcplx_set) return CPLX_UNIT;
	else if(class_set)
	{
		StrCopy(lvalp->bufr, id); 
		
		if(!gFactoryStack.empty() && 
			gFactoryStack.top()->GetTypeTag()==id)
		{
			std::wostringstream os;
			os<<L"Class name \'"<<id<<L"\' conflicts with its enclosing class name.\r\n";
			HandleSyntaxError(os.str());
		}

		return VID_CLASS;
	}
	else if(vartype != TypeNull)
	{
		if(ConflictTest(id)) return HandleSyntaxError(_T("ID conflicts.\r\n"));
		
		switch(vartype)
		{
		
		case TypeVoid: return VID_VOID;
		case TypeBool: return VID_BOOL;
							
		case TypeIntg: 
			{
				if(literal_set) return LID_INTG;
				else return VID_INTG;
			}
		case TypeReal: 
			{
				if(literal_set) return LID_REAL;
				else return VID_REAL;
			}
		case TypeCplx: 
			{
				if(literal_set) return LID_CPLX;
				else return VID_CPLX;
			}
		case TypeVctr: 
			{
				if(literal_set) return LID_VCTR;
				else return VID_VCTR;
			}

		case TypeStrn: 
			{
				if(literal_set) return LID_STRN;
				else return VID_STRN;
			}
		case TypeUser: return VID_USER;
		case TypeFile: return VID_FILE;

		}
	}

	return GetIdType(id, lvalp);
}

int HandleReturn(DataType type)
{
	switch(type)
	{
		case TypeVoid: return RTN_VOID; case TypeBool: return RTN_BOOL;
		case TypeIntg: return RTN_INTG; case TypeReal: return RTN_REAL;
		case TypeCplx: return RTN_CPLX; case TypeVctr: return RTN_VCTR;
		
		case TypeAi1D: return RTN_AI1D; case TypeAr1D: return RTN_AR1D;
		case TypeAc1D: return RTN_AC1D; case TypeAv1D: return RTN_AV1D;
		
		case TypeAi2D: return RTN_AI2D; case TypeAr2D: return RTN_AR2D;
		case TypeAc2D: return RTN_AC2D; case TypeAv2D: return RTN_AV2D;
			
		case TypeAi3D: return RTN_AI3D; case TypeAr3D: return RTN_AR3D;
		case TypeAc3D: return RTN_AC3D; case TypeAv3D: return RTN_AV3D;
			
		case TypeAi4D: return RTN_AI4D; case TypeAr4D: return RTN_AR4D;
		case TypeAc4D: return RTN_AC4D; case TypeAv4D: return RTN_AV4D;
			
		case TypeAi5D: return RTN_AI5D; case TypeAr5D: return RTN_AR5D;
		case TypeAc5D: return RTN_AC5D; case TypeAv5D: return RTN_AV5D;

		case TypeBPtr: return RTN_BPTR; case TypeSPtr: return RTN_SPTR;
		case TypeIPtr: return RTN_IPTR; case TypeRPtr: return RTN_RPTR;
		case TypeCPtr: return RTN_CPTR; case TypeVPtr: return RTN_VPTR;

		case TypeUser: return RTN_VUB; case TypeStrn: return RTN_STRN;
		
		default:
			//PutString(IDS_ERROR_RTN_NOTALLOWED);
			HandleSyntaxError(_T("Return statement is not allowed.\r\n"));
			return 0 ; // to suppress warning
	}
}

int HandleScanID(std::wstring token, YYSTYPE *lvalp)
{
	switch(gTransKeyword.GetKeywordID(token.c_str()))
	{
		case keyword_fmt_clear:
		{
			fmt_clear = true; return 0;
		}
		case keyword_fmt_precision:
		{
			fmt_clear = false; return SET_PREC;
		}
		case keyword_fmt_scientific:
		{
			fmt_clear = false; fmt_scientific=true; return 0;
		}
		case keyword_fmt_fixed:
		{
			fmt_clear=false; fmt_scientific=false; return 0;
		}
		case keyword_fmt_showpoint:
		{
			fmt_clear=false; fmt_showpoint=true; return 0;
		}
		case keyword_fmt_hidepoint:
		{
			fmt_clear=false; fmt_showpoint=false; return 0;
		}
		case keyword_fmt_showthousand:
		{
			fmt_thousand=true; return 0;
		}
		case keyword_fmt_hidethousand:
		{
			fmt_thousand=false; return 0;
		}
		case keyword_include:
		{
			extern void ModeToInclude();
			ModeToInclude(); return 0;
		}
		
		case keyword_new: return TKN_NEW; 

		case keyword_integer:
		{
			vartype=TypeIntg; return TYPE_INTG; 
		}
		case keyword_file:
		{
			vartype=TypeFile; return TYPE_FILE;
		}
		case keyword_cout: return TO_CONSOLE;
		case keyword_cin: return FROM_CONSOLE; 
		case keyword_true:
		{
			lvalp->booln = true; return BOOLN;
		}
		case keyword_false:
		{
			lvalp->booln = false; return BOOLN;
		}
		
		case keyword_clear: return STMT_CLEAR;
		
		case keyword_endl: return END_LINE; 
		case keyword_uvctr:
		{
			unitvctr_set = true; return DEF_VUNIT;
		}
		case keyword_ucplx:
		{
			unitcplx_set = true; return DEF_CUNIT;
		}
		case keyword_static:
		{
			static_set = true; return DEF_STATIC;
		}
		case keyword_literal:
		{
			literal_set = true; return DEF_LITERAL;
		}
		case keyword_always:
			if(gSelectStack.empty())
				HandleSyntaxError(_T("Label always is allowed inside of select statement.\r\n"));
			else return LABEL_ALWAYS;

		case keyword_delete: return TKN_DELETE;

		case keyword_try:
			{
				if(gStmtListStack.top()->GetStmtSize()==0)
				{
					send_token = TKN_TRY; return ';';
				}
				else return TKN_TRY; 
			}
		case keyword_catch:		return TKN_CATCH;
		case keyword_throw:		return TKN_THROW;

		case keyword_declare:		return DECLARE; 	
		case keyword_select:		return STMT_SELECT; 
		case keyword_if:
			{
				if(gStmtListStack.top()->GetStmtSize()==0)
				{
					send_token = STMT_IF; return ';';
				}
				else return STMT_IF; 
			}
		case keyword_if_true:		
			{
				if(gStmtListStack.top()->GetStmtSize()==0)
				{
					send_token = STMT_IFTRUE; return ';';
				}
				else return STMT_IFTRUE; 
			}
		case keyword_if_false:		
			{
				if(gStmtListStack.top()->GetStmtSize()==0)
				{
					send_token = STMT_IFFALSE; return ';';
				}
				else return STMT_IFFALSE;
			}
		case keyword_else:			return STMT_ELSE; 
		case keyword_else_true:		return STMT_ELSETRUE; 
		case keyword_else_false:	return STMT_ELSEFALSE; 

		case keyword_for:	
			{
				gStmtListStack.push(gGC.GetStmtList());
				gLoopStack.push(StmtLoop); 
				return STMT_FOR;
			}
		case keyword_do:
			{
				gStmtListStack.push(gGC.GetStmtList());
				gLoopStack.push(StmtLoop); 
				return STMT_DO;
			}

		case keyword_while:	
			gLoopStack.push(StmtLoop); return STMT_WHILE;	

		case keyword_goto:
		{
			goto_set = true; return STMT_GOTO;
		}
		case keyword_break: return STMT_BREAK; 
		case keyword_continue: return STMT_SKIP; 
		case keyword_void:
		{
			vartype=TypeVoid; return TYPE_VOID;
		}
		case keyword_bool:
		{
			vartype=TypeBool; return TYPE_BOOL;
		}
		case keyword_real:
		{
			vartype=TypeReal; return TYPE_REAL;
		}
		case keyword_complex:
		{
			vartype=TypeCplx; return TYPE_CPLX;
		}
		case keyword_vector:
		{
			vartype=TypeVctr; return TYPE_VCTR;
		}
		case keyword_strn:
		{
			vartype=TypeStrn; return TYPE_STRN;
		}
				
		case keyword_choice: 
			{
				if(gSelectStack.empty())
					return HandleSyntaxError(_T("Label choice is allowed inside of select statement.\r\n"));
				else return LABEL_CHOICE; 	
			}
		
		case keyword_exit: return STMT_EXIT;

		case keyword_class:
		{
			class_set = true;  return TYPE_CLASS;
		}

		case keyword_extends: return TKN_EXTENDS; 
		
		case keyword_return:
		{
			if(gReturnStack.empty()) 
				return HandleSyntaxError(_T("Statement \'return\' is allowed only within function body.\r\n")); 
			else 
				return HandleReturn(gReturnStack.top().type);
		}
		
		default:
		{
			StrCopy(lvalp->bufr, token.c_str());
			return GetVarType(token.c_str(), lvalp);
		}
	}
}

void InitializeKeywordTable()
{
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_fmt_clear"),		keyword_fmt_clear);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_fmt_precision"),	keyword_fmt_precision);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_fmt_scientific"),	keyword_fmt_scientific);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_fmt_fixed"),		keyword_fmt_fixed);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_fmt_showpoint"),	keyword_fmt_showpoint);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_fmt_hidepoint"),	keyword_fmt_hidepoint);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_fmt_showthousand"), keyword_fmt_showthousand);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_fmt_hidethousand"), keyword_fmt_hidethousand);
	
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_new"),				keyword_new);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_delete"),			keyword_delete);

	gTransKeywordTemplate.InsertKeyword(_T("@keyword_try"),				keyword_try);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_catch"),			keyword_catch);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_throw"),			keyword_throw);

	gTransKeywordTemplate.InsertKeyword(_T("@keyword_declare"),			keyword_declare);

	gTransKeywordTemplate.InsertKeyword(_T("@keyword_include"),			keyword_include);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_exit"),			keyword_exit);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_class"),			keyword_class);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_extends"),			keyword_extends);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_file"),			keyword_file);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_cout"),			keyword_cout);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_cin"),				keyword_cin);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_true"),			keyword_true);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_false"),			keyword_false);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_clear"),			keyword_clear);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_endl"),			keyword_endl);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_uvctr"),			keyword_uvctr);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_ucplx"),			keyword_ucplx);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_static"),			keyword_static);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_share"),			keyword_share);
	
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_select"),			keyword_select);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_choice"),			keyword_choice);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_always"),			keyword_always);
	
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_if"),				keyword_if);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_if_true"),			keyword_if_true);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_if_false"),		keyword_if_false);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_else"),			keyword_else);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_else_true"),		keyword_else_true);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_else_false"),		keyword_else_false);
	
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_do"),				keyword_do);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_for"),				keyword_for);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_while"),			keyword_while);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_goto"),			keyword_goto);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_break"),			keyword_break);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_continue"),		keyword_continue);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_literal"),			keyword_literal);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_void"),			keyword_void);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_bool"),			keyword_bool);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_integer"),			keyword_integer);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_real"),			keyword_real);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_complex"),			keyword_complex);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_vector"),			keyword_vector);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_return"),			keyword_return);
	gTransKeywordTemplate.InsertKeyword(_T("@keyword_strn"),			keyword_strn);
		
	///////////////////////////////////////////////////////////////
	
	gTransOprTypeTemplate.InsertKeyword(_T("@opr_real"),			opr_real);
	gTransOprTypeTemplate.InsertKeyword(_T("@opr_imag"),			opr_imag);
	
	gTransOprTypeTemplate.InsertKeyword(_T("@opr_xcom"),			opr_xcom);
	gTransOprTypeTemplate.InsertKeyword(_T("@opr_ycom"),			opr_ycom);
	gTransOprTypeTemplate.InsertKeyword(_T("@opr_zcom"),			opr_zcom);
		
	gTransOprTypeTemplate.InsertKeyword(_T("@opr_dimension"),		opr_dimension);
	gTransOprTypeTemplate.InsertKeyword(_T("@opr_size"),			opr_size);
	gTransOprTypeTemplate.InsertKeyword(_T("@opr_resize"),			opr_resize);
	
	gTransOprTypeTemplate.InsertKeyword(_T("@opr_detcramer"),		opr_detcramer);
	gTransOprTypeTemplate.InsertKeyword(_T("@opr_solvecramer"),		opr_solvecramer);
	gTransOprTypeTemplate.InsertKeyword(_T("@opr_ludecompo"),		opr_ludecompo);
	gTransOprTypeTemplate.InsertKeyword(_T("@opr_lumethod"),		opr_lumethod);
	gTransOprTypeTemplate.InsertKeyword(_T("@opr_inverse"),			opr_inverse);
	gTransOprTypeTemplate.InsertKeyword(_T("@opr_gaussjordan"),		opr_gaussjordan);
}

int InvalidFileName(const CString& filename)
{
	// The zero-based index of the first character in
	// this string that is also in pszCharSet; ? if there is no match.
	// Finds the first occurrence of any of the characters in pszCharSet.
	return filename.FindOneOf(_T("/:*?\"<>|"));
}

std::wstring GetPath(const CString& fullpath)
{
	PathInfo pinfo;
	_wsplitpath_s(fullpath.GetString(), pinfo.drive, _MAX_DRIVE, 
		pinfo.dir, _MAX_DIR, pinfo.fname, _MAX_FNAME, pinfo.ext, _MAX_EXT);

	std::wostringstream os;
	os<<pinfo.drive<<pinfo.dir;
	return os.str();
}