#pragma once

#include "nrc\nr.h"

using namespace NR;

double gamma(double xx);
double frl(double xx); // factorial
double frln(double xx); // ln factorial
double px2v(double x, double v); // Chi-Square P(x2 | v)
double qx2v(double x, double v); // Chi-Square Q(x2 | v)
double expnt(double n, double x); // Exponential integeral En(x)
double stdp(double t, double v); // Student's Distribution Probability, A(t|v)

// F-Distribution Probability Function
// Q(F | v1, v2) = I(v2/2, v1/2, v2/(v2 + v1*F)), NRC P.234 (6.4.11)
double fdp(double v1, double v2, double f);

// Cumulative Binomial Probability Distribution
// P(k, n, p), NRC. P. 234 (6. 4. 12)
double cbp(double k, double n, double p);

// Bessel function Jn(x)
double bessjn(double n, double x);

// Bessel function Yn(x), x>0
double bessyn(double n, double x);

// Modified Bessel function In(x), for any real x
double bessin(double n, double x);

// Modified Bessel function Kn(x), for positive x
double besskn(double n, double x); 

// Bessel function with factional order, Jv(x)
double bessjnu(double nu, double x);

// Bessel function with factional order, Yv(x)
double bessynu(double nu, double x);

// Bessel function with factional order, Jv(x)'
double bessjnup(double nu, double x);

// Bessel function with factional order, Yv(x)'
double bessynup(double nu, double x);

// Modified Bessel function, Iv(x)
double bessinu(double nu, double x);

// Modified Bessel function, Kv(x)
double bessknu(double nu, double x);

// Modified Bessel function, derivative of Iv(x)
double bessinup(double nu, double x);

// Modified Bessel function, derivative of Kv(x)
double bessknup(double nu, double x);

// Gamma by Chebyshev expansion, P 250
double gamma1 (double x);

// Gamma by Chebyshev expansion, P 250
double gamma2 (double x);

// Gamma by Chebyshev expansion, P 250
double gampl (double x);

// Gamma by Chebyshev expansion, P 250, 1/gamma(1+x)
double gammi (double x);

// Airy function, Ai(x)
double ai(double x);

// Airy function, Bi(x)
double bi(double x);

// Airy function, derivative of Ai(x)
double aip(double x);

// Airy function, derivative of Bi(x)
double bip(double x);

// Spherical Bessel Functions, jn(x)
double spjn(double n, double x);

// Spherical Bessel Functions, yn(x)
double spyn(double n, double x);

// Spherical Bessel Functions, derivative of jn(x)
double spjnp(double n, double x);

// Spherical Bessel Functions, derivative of yn(x)
double spynp(double n, double x);

// Legendre Polynomial, P. 258
double legp(double l, double m, double x);

// Frenel integral C(x)
double frec(double x);

// Frenel integral S(x)
double fres(double x);

// Cosine integeral, Ci(x)
double ci(double x);

// Sine integeral, Si(x)
double si(double x);

// Jacobian elliptic fucntions sn(u, kc)
double sn(double u, double kc);

// Jacobian elliptic fucntions cn(u, kc)
double cn(double u, double kc);

// Jacobian elliptic fucntions dn(u, kc)
double dn(double u, double kc);

// Hypergeometirc series 2F1, P. 276
Complex hypsr (const Complex& a, const Complex& b,
			   const Complex& c, const Complex& z);

Complex hypsrd (const Complex& a, const Complex& b,
			   const Complex& c, const Complex& z);




