#include "stdafx.h"
#include <kch_theparser_type.h>
#include <kch_theparser_type2.h>

#include "garbagecollector.h"

ProgStatus gProgStatus;

VoidData::VoidData(std::wstring varid, bool varstatic): Data(varid, TypeVoid) { }
voidtype& VoidData::GetData() { return data; }
VoidData::~VoidData() { }

VarList::VarList(): pushcount(0) {};

int VarList::Push() { varlist.Push(); return ++pushcount;  }
int VarList::Pop() { varlist.Pop(); return --pushcount;}
int VarList::GetStackDepth() { return pushcount; }
int VarList::size() { return int(varlist.size()); }
int VarList::insert(Data* pData)
{
	IndexItr itr = varindex.find(pData->GetId());
	int index;

	if(itr != varindex.end()) 
	{
		std::wostringstream os; 
		os<<_T("\'")<<pData->GetId()<<_T("\' ")<<LoadUnicodeMsg(IDS_ALREADY_DECLARED);

		delete pData; HandleSyntaxError(os.str());
	}
	else
	{
		varlist.push_back(pData);
		index = int(varlist.size()) - 1;
		varindex.insert(IndexPair(pData->GetId(), index));
	}

	return index;
}

void VarList::put(Data* pData) { varlist.push_back(pData); }
Data* VarList::get(int index) { return varlist[index]; }

Data* VarList::operator[](const std::wstring& id)
{
	IndexItr itr = varindex.find(id);
	if(itr != varindex.end()) 
		return varlist[itr->second];
	else
	{			
		throw UnicodeException(LoadUnicodeMsg(IDS_ID_OUT_OF_RANGE).c_str());
	}
}

Data* VarList::at(const std::wstring& id)
{
	IndexItr itr = varindex.find(id);
	if(itr != varindex.end()) 
		return varlist[itr->second];
	else
	{			
		throw UnicodeException(LoadUnicodeMsg(IDS_ID_OUT_OF_RANGE).c_str());
	}
}

DataTypeVec VarList::GetDataTypeVec() const
{
	DataTypeVec typevec; 
	int count = int(varlist.size());
	for(int i=0; i<count; ++i)
		typevec.push_back(varlist[i]->GetType());
	
	return typevec;
}

DataPairVec VarList::GetDataPairVec() const
{
	DataPairVec argvec; 
	int count = int(varlist.size());
	for(int i=0; i<count; ++i)
		argvec.push_back(varlist[i]->GetDataSig());
	
	return argvec;
}

DataType VarList::GetType(const std::wstring& id)
{
	IndexItr itr = varindex.find(id);
	if(itr != varindex.end()) 
	{
		return varlist[itr->second]->GetType();
	}
	else
		return TypeNull;
}

DataType VarList::GetType(const std::wstring& id, Data** pData)
{
	IndexItr itr = varindex.find(id);
	if(itr != varindex.end()) 
	{
		*pData = varlist[itr->second];
		return (*pData)->GetType();
	}
	else
	{
		*pData = NULL;
		return TypeNull;
	}
}
	
Data* VarList::at(int index)
{
	if(index>=0 && index<int(varlist.size()))
		return varlist[index];
	else
		throw UnicodeException(LoadUnicodeMsg(IDS_INDEX_OUT_OF_RANGE).c_str());
}

void VarList::Delete() { varindex.clear(); }

void VarList::PrintElement()
{
	std::wostringstream os;

	for(int i=0; i<int(varlist.size()); ++i)
	{
		os<<varlist[i]->GetId()<<_T(" ");
	}
	os<<std::endl;
	
	PutString(os.str());
}

SolveEquation::SolveEquation(LeftHandSide<realnum> *var, ExpReal* xpr1, 
	ExpReal* xpr2, ExpReal* xpr3):
  pVar(var), pExprCnt(NULL), pExprEps(NULL), pExpr1(xpr1), pExpr2(xpr2), pExpr3(xpr3) { }

SolveEquation::SolveEquation(LeftHandSide<realnum> *var, ExpReal* xpr1, 
	ExpReal* xpr2, ExpReal* xpr3, ExpIntg* pCnt):
  pVar(var), pExprCnt(pCnt), pExprEps(NULL), pExpr1(xpr1), pExpr2(xpr2), pExpr3(xpr3) { }

SolveEquation::SolveEquation(LeftHandSide<realnum> *var, ExpReal* xpr1, 
	ExpReal* xpr2, ExpReal* xpr3, ExpReal* pEps):
  pVar(var), pExprCnt(NULL), pExprEps(pEps), pExpr1(xpr1), pExpr2(xpr2), pExpr3(xpr3) { }

SolveEquation::SolveEquation(LeftHandSide<realnum> *var, ExpReal* xpr1, 
	ExpReal* xpr2, ExpReal* xpr3, ExpIntg* pCnt, ExpReal* pEps):
  pVar(var), pExprCnt(pCnt), pExprEps(pEps), pExpr1(xpr1), pExpr2(xpr2), pExpr3(xpr3) { }
  	
RealVec1D SolveEquation::Eval()
{
	SolSet solset, extrema;

	realnum oldvalue = pVar->GetData();
	
	int count=SOL_SECTIONS; 
	if(pExprCnt)
	{
		count = int(pExprCnt->Eval());
		if(count<1) 
			throw UnicodeException(_T("Number of sections should be greater than zero(0)\r\n"));
	}

	realnum epsilon=1E-6;
	if(pExprEps)
	{
		epsilon = fabs(pExprEps->Eval());

		if(epsilon>1E-2) 
			throw UnicodeException(_T("Epsilon should be less than 1E-2 (=0.01) .\r\n"));
	}

	realnum x1 = pExpr1->Eval(), x2 = pExpr2->Eval();

	realnum delta = fabs((x1 - x2)/(10.0*count));
	
	realnum maxx = GetMax<realnum>(x1, x2)+delta;
	realnum minx = GetMin<realnum>(x1, x2)-delta;

	realnum dx = (maxx - minx)/realnum(count);

	for(realnum x = minx; x<maxx; x += dx)
	{			
		SET_USER_ABORT;

		if(SolveNewton(solset,   pExpr3, pVar, x, x+dx, epsilon))
			MaxMinBisection(extrema, pExpr3, pVar, x, x+dx, epsilon);
	}

	for(SolSetItr itr = extrema.begin(); itr != extrema.end(); ++itr)
	{
		SET_USER_ABORT;

		EVAL_XPR(pVar, *itr, dx, pExpr3);

		if(fabs(dx)<1E-5) solset.insert(*itr);
	}

	RealVec1D vec(int(solset.size())); count = 0;
	for(SolSetItr itr = solset.begin(); itr != solset.end(); ++itr)
	{
		SET_USER_ABORT;
		vec[count++] = *itr;
	}

	pVar->GetData() = oldvalue;
	
	return vec;
}

SolveEquation::~SolveEquation() {}

SolveExtrema::SolveExtrema(LeftHandSide<realnum> *var, ExpReal* xpr1, 
	ExpReal* xpr2, ExpReal* xpr3):
  pVar(var), pExprCnt(NULL), pExprEps(NULL), pExpr1(xpr1), pExpr2(xpr2), pExpr3(xpr3) { }

SolveExtrema::SolveExtrema(LeftHandSide<realnum> *var, ExpReal* xpr1, 
	ExpReal* xpr2, ExpReal* xpr3, ExpIntg* pCnt):
  pVar(var), pExprCnt(pCnt), pExprEps(NULL), pExpr1(xpr1), pExpr2(xpr2), pExpr3(xpr3) { }

SolveExtrema::SolveExtrema(LeftHandSide<realnum> *var, ExpReal* xpr1, 
	ExpReal* xpr2, ExpReal* xpr3, ExpReal* pEps):
  pVar(var), pExprCnt(NULL), pExprEps(pEps), pExpr1(xpr1), pExpr2(xpr2), pExpr3(xpr3) { }

SolveExtrema::SolveExtrema(LeftHandSide<realnum> *var, ExpReal* xpr1, 
	ExpReal* xpr2, ExpReal* xpr3, 
	ExpIntg* pCnt, ExpReal* pEps):
  pVar(var), pExprCnt(pCnt), pExprEps(pEps), pExpr1(xpr1), pExpr2(xpr2), pExpr3(xpr3) { }

RealVec1D SolveExtrema::Eval()
{
	SolSet extrema;

	realnum oldvalue = pVar->GetData();
			
	int count=SOL_SECTIONS;
	if(pExprCnt)
	{
		count = int(pExprCnt->Eval());
		if(count<1) 
			throw UnicodeException(_T("Number of sections should be greater than zero(0).\r\n"));
	}

	realnum epsilon = 1E-6;
	if(pExprEps)
	{
		epsilon = fabs(pExprEps->Eval());
		if(epsilon>1E-2) 
			throw UnicodeException(_T("Epsilon should be less than 1E-2 (=0.01) .\r\n"));
	}

	realnum x1 = pExpr1->Eval(), x2 = pExpr2->Eval();
	
	realnum delta = fabs((x1 - x2)/(10.0*count));

	realnum maxx = GetMax<realnum>(x1, x2)+delta;
	realnum minx = GetMin<realnum>(x1, x2)-delta;
	
	realnum dx = (maxx - minx)/realnum(count);

	for(realnum x = minx; x<maxx; x += dx)
	{			
		SET_USER_ABORT;
		MaxMinBisection(extrema, pExpr3, pVar, x, x+dx, epsilon);
	}

	RealVec1D vec(int(extrema.size())); count = 0;

	for(SolSetItr itr=extrema.begin(); itr!=extrema.end(); ++itr)
	{
		SET_USER_ABORT;
		vec[count++] = *itr;
	}
	
	pVar->GetData() = oldvalue;

	return vec;
}

SolveExtrema::~SolveExtrema() {}

void IntToReal::cache()
{
	if(expr && expr->bLiteral)
	{
		rtn = realnum(expr->Eval()); bLiteral = true;
	}
}

IntToReal::IntToReal(ExpIntg* xpr): expr(xpr) { cache(); }

realnum IntToReal::Eval() 
{ 
	if(bLiteral) return rtn;
	else return realnum(expr->Eval());
}

std::wstring IntToReal::Display() { return expr->Display(); }
IntToReal::~IntToReal() { }

void IntToCplx::cache()
{
	if(expr && expr->bLiteral)
	{
		rtn = Complex(realnum(expr->Eval()), 0.0);
		bLiteral = true;
	}
}

IntToCplx::IntToCplx(ExpIntg* xpr): expr(xpr) { cache(); }

Complex IntToCplx::Eval() 
{
	if(bLiteral) return rtn;
	else return Complex(realnum(expr->Eval()), 0.0);
}

std::wstring IntToCplx::Display()
{
	return expr->Display();
}

IntToCplx::~IntToCplx() { }

void RealToInt::cache()
{
	if(expr && expr->bLiteral)
	{
		rtn = ToInteger(expr->Eval()); bLiteral = true;
	}
}

RealToInt::RealToInt(ExpReal* xpr): expr(xpr) { cache(); }

integer RealToInt::Eval() 
{ 
	if(bLiteral) return rtn;
	return ToInteger(expr->Eval());
}

std::wstring RealToInt::Display() { return expr->Display(); }
RealToInt::~RealToInt() { }

void RealToCplx::cache()
{
	if(expr && expr->bLiteral)
	{
		rtn = Complex(expr->Eval(), 0.0); bLiteral=true;
	}
}

RealToCplx::RealToCplx(ExpReal* xpr): expr(xpr) { cache(); }
Complex RealToCplx::Eval() 
{ 
	if(bLiteral) return rtn;
	else return Complex(expr->Eval(), 0.0);
}
std::wstring RealToCplx::Display() { return expr->Display(); }
RealToCplx::~RealToCplx() { }

void RealToVctr::cache()
{
	if(expr && expr->bLiteral)
	{
		rtn = Vector(expr->Eval(), 0.0); bLiteral=true;
	}
}

RealToVctr::RealToVctr(ExpReal* xpr): expr(xpr) { cache(); }

Vector RealToVctr::Eval() 
{ 
	if(bLiteral) return rtn;
	else return Vector(expr->Eval(), 0.0, 0.0);
}

std::wstring RealToVctr::Display() { return expr->Display(); }
RealToVctr::~RealToVctr() { }

void IntgToVctr::cache()
{
	if(expr && expr->bLiteral)
	{
		rtn = Vector(realnum(expr->Eval()), 0.0, 0.0); bLiteral=true;
	}
}

IntgToVctr::IntgToVctr(ExpIntg* xpr): expr(xpr) { cache(); }

Vector IntgToVctr::Eval() 
{ 
	if(bLiteral) return rtn;
	else return Vector(realnum(expr->Eval()), 0.0, 0.0);
}

std::wstring IntgToVctr::Display() { return expr->Display(); }

IntgToVctr::~IntgToVctr() { }

usertype* VarUserBase::getusertype()
{
	usertype *ptr = data->GetPtr(); 
	
	if(memary.empty()) return ptr; 

	UdtPair* thePair = ptr->GetElement(memary[0]);
	int size = int(memary.size()), depth=1;

	while(depth<size)
	{
		if(TypeUser == thePair->first)
			thePair = static_cast<usertype*>(thePair->second)->GetElement(memary[depth++]);
		else if(TypeUPtr == thePair->first)
			thePair = static_cast<UPtrData*>(thePair->second)->GetData()->GetData().GetElement(memary[depth++]);
		else break;
	}

	if(depth == size)
	{
		return static_cast<usertype*>(thePair->second);
	}
	else throw UnicodeException(_T("Element typemismatch or not found.\r\n"));
	
}

VarUserBase::VarUserBase(UserData *udptr): 
data(udptr), UdtVarBase(NULL) { }

UserDefinedType* VarUserBase::GetUdt() const { return udtptr; }

usertype* VarUserBase::GetUserType() { return getusertype(); }

UserData *VarUserBase::GetUserData() { return this->data;  }

bool VarUserBase::IsCompatible(UserDefinedType *ptr)
{	return this->udtptr == ptr; }

bool VarUserBase::IsCompatible(VarUserBase* ptr)
{	return this->udtptr == ptr->udtptr; }

bool VarUserBase::IsCompatible(VarMemBase* ptr)
{	return this->udtptr == ptr->udtptr; }

bool VarUserBase::IsCompatible(LeftHandSide<UserData> *ptr)
{	
	UserDefinedType *pUdPtr = ptr->GetUdt();

	if(this->udtptr && pUdPtr && this->udtptr==pUdPtr) return true;
		else return false;
}

void VarUserBase::InsertMemberID(const std::wstring& memid) 
	{ memary.push_back(memid); }

std::wstring VarUserBase::GetFullName() const
{
	std::wostringstream os;
	int count = int(this->memary.size());
	
	for(int i=0; i<count; ++i)
		os<<_T(".")<<this->memary[i];

	return os.str();
}

VarUserBase::~VarUserBase() {}

//////////////////////////

void VarMemBase::SetDataPtr() const
{
	if(this->data)
	{
		if(*(this->data)) return;
		else throw UnicodeException(_T("UserData *data points to NULL0.\r\n"));
	}
	else
	{
		if(this->ptrFunc) // member function
		{
			if(this->ptrFunc->pUdt)
			{
				this->data = &this->ptrFunc->pUdt; return;
			}

			throw UnicodeException(_T("UserData *data points to NULL1.\r\n")); 
		}
		
		throw UnicodeException(_T("UserData *data points to NULL2\r\n")); 
	}
}

usertype* VarMemBase::getusertype()
{
	SetDataPtr(); usertype *ptr = (*data)->GetPtr(); 

	if(memary.empty()) return ptr;
	
	UdtPair* thePair = ptr->GetElement(memary[0]);
	int size = int(memary.size()), depth=1;

	while(depth<size)
	{
		if(TypeUser == thePair->first)
			thePair = static_cast<usertype*>(thePair->second)->GetElement(memary[depth++]);
		else if(TypeUPtr == thePair->first)
			thePair = static_cast<UPtrData*>(thePair->second)->GetData()->GetData().GetElement(memary[depth++]);
		else break;

	}
	
	if(depth == size)
	{
		return static_cast<usertype*>(thePair->second);
	}
	else throw UnicodeException(_T("Element type mismatch or not found.\r\n"));
}

VarMemBase::VarMemBase(UserDefinedType *uptr, Function *fptr, VarUserBase *vub) 
	: data(NULL), ptrFunc(fptr), UdtVarBase(uptr)
{
	if(vub) this->memary = vub->memary;
}

UserDefinedType* VarMemBase::GetUdt() const 
{
	if(udtptr) return udtptr; 
	else 
		HandleSyntaxError(L"VarMemBase::GetUdt(), UserDefinedType pointer is null.\r\n");
	
	return NULL; // to suppress warning
}

usertype* VarMemBase::GetUserType() { return getusertype(); }

UserData* VarMemBase::GetUserData() 
{ SetDataPtr(); return *this->data;  }

bool VarMemBase::IsCompatible(VarMemBase* ptr)
{	
	if(this->udtptr && ptr->udtptr 
	   && this->udtptr==ptr->udtptr) return true;
	else return false;
}

bool VarMemBase::IsCompatible(VarUserBase* ptr)
{	
	UserDefinedType *pUdPtr = ptr->GetUdt();

	if(this->udtptr && pUdPtr && this->udtptr==pUdPtr) return true;
	else return false;
}

bool VarMemBase::IsCompatible(UserDefinedType *ptr)
{	
	if(this->udtptr && this->udtptr==ptr) return true;
	else return false;
}

bool VarMemBase::IsCompatible(LeftHandSide<UserData> *ptr)
{	
	UserDefinedType *pUdPtr = ptr->GetUdt();

	if(this->udtptr && pUdPtr && this->udtptr==pUdPtr) return true;
		else return false;
}

void VarMemBase::InsertMemberID(const std::wstring& memid) 
	{ memary.push_back(memid); }

std::wstring VarMemBase::GetFullName() const
{
	std::wostringstream os;
	int count = int(this->memary.size());
	
	for(int i=0; i<count; ++i)
		os<<_T(".")<<this->memary[i];

	return os.str();
}

VarMemBase::~VarMemBase() {}

//////////////////////////


UdtPtrBase::UdtPtrBase(PtrData<UserData> *vptr):
	pData(vptr), UdtVarBase(NULL) {  }

UserDefinedType* UdtPtrBase::GetUdt() const { return udtptr; }

usertype* UdtPtrBase::GetUserType() 
{ 
	if(pData->GetData()==NULL)
		HandleSyntaxError(L"User Defined Type Pointer is not yet initialized.\r\n");
  
	return this->GetUserData()->GetPtr();
}

UserData* UdtPtrBase::GetUserData() 
{
	if(pData->GetData()==NULL)
		HandleSyntaxError(L"User Defined Type Pointer is not yet initialized.\r\n");
	
	return this->pData->GetData();
}

bool UdtPtrBase::IsCompatible(UserDefinedType *ptr) 
	{ return udtptr==ptr; }
bool UdtPtrBase::IsCompatible(LeftHandSide<UserData> *ptr) 
	{ return ptr->GetUdt()==udtptr;}

void UdtPtrBase::InsertMemberID(const std::wstring& memid) 
{
	memary.push_back(memid);
}

void UdtPtrBase::SetDataPtr(UserData* udptr)
{
	*pData->GetPtr() = udptr;
}
std::wstring UdtPtrBase::GetFullName() const 
{
	std::wostringstream os;
	int count = int(this->memary.size());
	
	for(int i=0; i<count; ++i)
		os<<_T(".")<<this->memary[i];

	return os.str();
}

UdtPtrBase::~UdtPtrBase() {}

//////////////////////////////

VarMemUPtrData::VarMemUPtrData(VarMem<UPtrData>* vptr):
	pData(vptr), UdtPtrBase(NULL) {  }

UserDefinedType* VarMemUPtrData::GetUdt() const 
{ return udtptr; }

usertype* VarMemUPtrData::GetUserType() 
{ 
	if(pData->GetData().GetData()==NULL)
		HandleSyntaxError(L"User Defined Type Pointer is not yet initialized.\r\n");
  
	return pData->GetData().GetData()->GetPtr();
}

UserData* VarMemUPtrData::GetUserData() 
{ 
	if(pData->GetData().GetData()==NULL)
		HandleSyntaxError(L"User Defined Type Pointer is not yet initialized.\r\n");
	return pData->GetData().GetData(); 
}

bool VarMemUPtrData::IsCompatible(UserDefinedType *ptr) 
	{ return udtptr==ptr; }
bool VarMemUPtrData::IsCompatible(LeftHandSide<UserData> *ptr) 
	{ return ptr->GetUdt()==udtptr;}

void VarMemUPtrData::InsertMemberID(const std::wstring& memid) 
{
	pData->vubPtr->InsertMemberID(memid);
}

void VarMemUPtrData::SetDataPtr(UserData* udptr)
{
	*pData->GetData().GetPtr() = udptr;
}
std::wstring VarMemUPtrData::GetFullName() const 
{
	std::wostringstream os;
	int count = int(this->memary.size());
	
	for(int i=0; i<count; ++i)
		os<<_T(".")<<this->memary[i];

	return os.str();
}

VarMemUPtrData::~VarMemUPtrData() {}

/////////////////////////////////////////


UserMemUPtrData::UserMemUPtrData(VarUser<UPtrData>* vptr):
	pData(vptr), UdtPtrBase(NULL) {  }

UserDefinedType* UserMemUPtrData::GetUdt() const 
{ return udtptr; }

usertype* UserMemUPtrData::GetUserType() 
{ 
	if(pData->GetData().GetData()==NULL)
		HandleSyntaxError(L"User Defined Type Pointer is not yet initialized.\r\n");
  
	return pData->GetData().GetData()->GetPtr();
}

UserData* UserMemUPtrData::GetUserData() 
{ 
	if(pData->GetData().GetData()==NULL)
		HandleSyntaxError(L"User Defined Type Pointer is not yet initialized.\r\n");

	return pData->GetData().GetData(); 
}

bool UserMemUPtrData::IsCompatible(UserDefinedType *ptr) 
	{ return udtptr==ptr; }
bool UserMemUPtrData::IsCompatible(LeftHandSide<UserData> *ptr) 
	{ return ptr->GetUdt()==udtptr;}

void UserMemUPtrData::InsertMemberID(const std::wstring& memid) 
{
	pData->vubPtr->InsertMemberID(memid);
}

void UserMemUPtrData::SetDataPtr(UserData* udptr)
{
	*pData->GetData().GetPtr() = udptr;
}
std::wstring UserMemUPtrData::GetFullName() const 
{
	std::wostringstream os;
	int count = int(this->memary.size());
	
	for(int i=0; i<count; ++i)
		os<<_T(".")<<this->memary[i];

	return os.str();
}

UserMemUPtrData::~UserMemUPtrData() {}

//////////////////////////////////////


MemUPtrData::MemUPtrData(UdtPtrBase* vptr):
	pData(vptr), UdtPtrBase(NULL) {  }

UserDefinedType* MemUPtrData::GetUdt() const { return udtptr; }

usertype* MemUPtrData::GetUserType() 
{ 
	if(pData->GetUserData()==NULL)
		HandleSyntaxError(L"User Defined Type Pointer is not yet initialized.\r\n");
  
	return pData->GetUserData()->GetPtr();
}

UserData* MemUPtrData::GetUserData() 
{ 
	if(pData->GetUserData()==NULL)
		HandleSyntaxError(L"User Defined Type Pointer is not yet initialized.\r\n");
	
	return pData->GetUserData(); 
}

bool MemUPtrData::IsCompatible(UserDefinedType *ptr) { return udtptr==ptr; }
bool MemUPtrData::IsCompatible(LeftHandSide<UserData> *ptr) 
	{ return ptr->GetUdt()==udtptr;}

void MemUPtrData::InsertMemberID(const std::wstring& memid) 
{
	pData->InsertMemberID(memid);
}

void MemUPtrData::SetDataPtr(UserData* udptr)
{
	pData->SetDataPtr(udptr);
}
std::wstring MemUPtrData::GetFullName() const 
{
	std::wostringstream os;
	int count = int(this->memary.size());
	
	for(int i=0; i<count; ++i)
		os<<_T(".")<<this->memary[i];

	return os.str();
}

MemUPtrData::~MemUPtrData() {}

//////////////////////////////
void Factorial::cache()
{
	if(expr->bLiteral)
	{
		rtn = this->Eval(); bLiteral = true;
	}
}

Factorial::Factorial(ExpIntg* xpr): expr(xpr) { cache(); }

integer Factorial::Eval()
{
	if(bLiteral) return rtn;

	integer n = expr->Eval();

	extern integer factorial(integer n);
	integer rlt = factorial(n);

	if(rlt<=0) throw UnicodeException(LoadUnicodeMsg(IDS_FACTORIAL_OVERFLOW));
	else return rlt;
}

std::wstring Factorial::Display() 
{
	std::wostringstream ostr;
	ostr<<expr->Display()<<_T('!');
	return ostr.str();
}

Factorial::~Factorial() { }

Absolute::Absolute(ExpIntg* xpr): expr(xpr) {}

integer Absolute::Eval()
{
	integer n = expr->Eval();
	integer rlt = (n>0 ? n : -n);
	
	return rlt;
}

std::wstring Absolute::Display() 
{
	std::wostringstream ostr;
	ostr<<'|'<<expr->Display()<<'|';
	return ostr.str();
}

Absolute::~Absolute() { }

void Permutation::cache()
{
	if(nXpr->bLiteral && rXpr->bLiteral)
	{
		rtn = this->Eval(); bLiteral = true;
	}
}

Permutation::Permutation(ExpIntg* n, ExpIntg* r): 
  nXpr(n), rXpr(r) { cache(); }

integer Permutation::Eval()
{
	if(bLiteral) return rtn;

	integer n = nXpr->Eval();
	integer r = rXpr->Eval();

	extern integer permutation(integer n, integer r);
	integer rlt = permutation(n, r);

	if(rlt<=0)
		throw UnicodeException(LoadUnicodeMsg(IDS_PERMUTATION_OVERFLOW).c_str());
	else
		return rlt;
}

std::wstring Permutation::Display() 
{
	std::wostringstream ostr;
	ostr<<'['<<nXpr->Display()<<'|'<<rXpr->Display()<<']';
	return ostr.str();
}

Permutation::~Permutation() { }

void Combination::cache()
{
	if(nXpr->bLiteral && rXpr->bLiteral)
	{
		rtn = this->Eval(); bLiteral = true;
	}
}

Combination::Combination(ExpIntg* n, ExpIntg* r): 
  nXpr(n), rXpr(r) { cache(); }

integer Combination::Eval()
{
	if(bLiteral) return rtn;

	integer n = nXpr->Eval();
	integer r = rXpr->Eval();

	extern integer combination(integer n, integer r);
	integer rlt = combination(n, r);
	
	if(rlt<=0)
		throw UnicodeException(LoadUnicodeMsg(IDS_COMBI_OVERFLOW).c_str());
	else
		return rlt;
}

std::wstring Combination::Display() 
{
	std::wostringstream ostr;
	ostr<<'('<<nXpr->Display()<<_T("|")<<rXpr->Display()<<')';
	return ostr.str();
}

Combination::~Combination() { }

Gamma::Gamma(ExpReal* xpr): 
	  expr(xpr) { }

realnum Gamma::Eval()
{
	realnum xx = expr->Eval();
	
	extern realnum gammaln(realnum xx);
	realnum x = gammaln(xx);

	return pow(M_E, x);
}

std::wstring Gamma::Display() 
{
	std::wostringstream ostr;
	ostr<<_T("gamma(")<<expr->Display()<<')';
	return ostr.str();
}

Gamma::~Gamma() { }

RVBinOpr::RVBinOpr(int opr, ExpReal* scalar, ExpVctr* vctr)
	: optr(opr), s(scalar), lhs(NULL), rhs(vctr) { }
RVBinOpr::RVBinOpr(int opr, ExpVctr* vctr, ExpReal* scalar)
	: optr(opr), s(scalar), lhs(vctr), rhs(NULL) { }
 
Vector RVBinOpr::Eval()
{
	if(rhs) // lhs = NULL, scalar op vctr
	{
		if(optr == '*') return s->Eval() * rhs->Eval();
	}
	else // rhs = NULL, vctr op scalar
	{
		switch(optr)
		{
		case '*': return lhs->Eval() * s->Eval();
		case '/': 
			{
				realnum r = s->Eval();
				if(r==0.0) throw ExpDivisionByZero();
				else return lhs->Eval() / r;
			}
		}
	}
	std::wostringstream os;
	os<<_T("Binary scalar and vector operation with operator '")<<(char)optr<<_T("' is not supported.\r\n");
	throw UnicodeException(os.str());
}

std::wstring RVBinOpr::Display() 
{
	std::wostringstream ostr;
	UniChar unicode;
	unicode.byte.low = optr; unicode.byte.high = '\0';
	if(lhs)
		ostr<<'('<<lhs->Display()<<unicode.wchar<<s->Display()<<_T(')');
	else if(rhs)
		ostr<<'('<<s->Display()<<unicode.wchar<<rhs->Display()<<_T(')');
	else
		throw UnicodeException(_T("This can not happen.\r\n"));

	return ostr.str();
}

RVBinOpr::~RVBinOpr() { }

void VVDotOpr::cache()
{
	if(lhs->bLiteral && rhs->bLiteral)
	{
		rtn = this->Eval(); bLiteral = true;
	}
}

VVDotOpr::VVDotOpr(ExpVctr* left, ExpVctr* right):
  lhs(left), rhs(right){ cache(); }

 realnum VVDotOpr::Eval()
{  
	if(bLiteral) return rtn;
	else return lhs->Eval() * rhs->Eval(); 
}

std::wstring VVDotOpr::Display() 
{
	std::wostringstream ostr;
	ostr<<_T('(')<<lhs->Display()<<_T('*')<<rhs->Display()<<_T(')');
	return ostr.str();
}

VVDotOpr::~VVDotOpr() { }


AssignExpUser::AssignExpUser(LeftHandSide<UserData> *lptr, ExpUserPtr rptr): 
  lhs(lptr), rhs(rptr), BaseAssign(_T("User Type")) { }

  void AssignExpUser::Run() 
{ 
	try
	{
		CallFunc<UserData> *pCallFunc 
			= dynamic_cast<CallFunc<UserData>*>(rhs);
		
		LeftHandSide<UserData> *pRHS = 
			dynamic_cast<LeftHandSide<UserData>*>(rhs);

		if(pCallFunc)
		{
			pCallFunc->FastEval(&lhs->GetData());
		}
		else if(pRHS)
		{
			lhs->GetData().Copy(pRHS->GetData());
		}
		else 
			lhs->GetData().GetData() = rhs->Eval();

	}catch(UnicodeException& e)
	{
		PutString(e.GetMessage());
	}
}

AssignExpUser::~AssignExpUser() { }


AssignUT2ExpUser::AssignUT2ExpUser(LeftHandSide<UserData> *lptr, CallFunc<UserData> *rptr): 
  lhs(lptr), rhs(rptr), BaseAssign(_T("User Type")) { }

void AssignUT2ExpUser::Run() 
{ 
	try
	{
		rhs->FastEval(&lhs->GetData());

	}catch(UnicodeException& e)
	{
		PutString(e.GetMessage());
	}
}

AssignUT2ExpUser::~AssignUT2ExpUser() { }













StmtSwapVUB::StmtSwapVUB(VarUserBase *lptr, VarUserBase *rptr): lhs(lptr), rhs(rptr) { }

void StmtSwapVUB::Run() 
{
	if(lhs->memary.empty() && rhs->memary.empty())
		lhs->data->Swap(rhs->data);
	else
		lhs->GetUserType()->Swap(rhs->GetUserType());
}

StmtSwapVUB::~StmtSwapVUB() { }

Function::Function(DataType rtn, const std::wstring& name): 
	fname(name), rtntype(rtn), pUdt(NULL), 
		pStmtList(NULL), Statement(StmtFunc), 
		pUDTFactory(NULL), NotUseMember(false)

#ifdef _DEBUG
		, rtnStack(name)
#endif
		{	}

void Function::InsertParam(Statement *pParam, int index)
{
	// function already declared or defined
	extern FunctionState gFuncState;
	extern GarbageCollector gGC;

	if(gFuncState != FS_NewFunction)
	{
		pStmtList->CheckIdentical(pParam, index);
		gGC.Erase(pParam); 
	}
	else pStmtList->Add(pParam);
}

void Function::CheckArgs(BaseType *xpr, int index)
{
	VarList *varptr = this->pStmtList->GetVarList();
	std::wostringstream os;
	if(index>=varptr->size())
	{
		os<<_T("Function \'")<<this->fname
			<<_T("\' does not take ")<<index+1<<_T(" argument(s).\r\n");
		HandleSyntaxError(os.str());
	}
	os<<_T("Argument #")<<index+1<<_T(", type mismatch.\r\n");

	switch(varptr->at(index)->GetType())
	{
		case TypeBool:
			if(dynamic_cast<ExpBool*>(xpr)) return;
			else
			{	os<<L"Boolean expression is required.\r\n";
				HandleSyntaxError(os.str());
			}
		case TypeIntg:
			if(dynamic_cast<ExpIntg*>(xpr) || 
				dynamic_cast<ExpReal*>(xpr)) return;
			else
			{	os<<L"Integer expression is required.\r\n";
				HandleSyntaxError(os.str());
			}
		case TypeReal:
			if(dynamic_cast<ExpReal*>(xpr) || 
				dynamic_cast<ExpIntg*>(xpr)) return;
			else 
			{	os<<L"Real number expression is required.\r\n";
				HandleSyntaxError(os.str());
			}
		case TypeCplx:
			if(dynamic_cast<ExpCplx*>(xpr)) return;
			else 
			{	os<<L"Complex expression is required.\r\n";
				HandleSyntaxError(os.str());
			}
		case TypeVctr:
			if(dynamic_cast<ExpVctr*>(xpr)) return;
			else 
			{	os<<L"Vector expression is required.\r\n";
				HandleSyntaxError(os.str());
			}
		case TypeStrn:
			if(dynamic_cast<ExpStrn*>(xpr)) return;
			else
			{	os<<L"String expression is required.\r\n";
				HandleSyntaxError(os.str());
			}
		case TypeUser:
			{
				extern Stack<UserDefinedType*> gUdtFactoryStack;

				if(! gUdtFactoryStack.empty() && static_cast<DeclUdt*>
						(pStmtList->GetStatement(index))->GetFactory()
						==gUdtFactoryStack.top()) return;
				else 
					os<<L"User defined data type mistmach.\r\n";
					
				os<<L"User defined type expression is required.\r\n";
				HandleSyntaxError(os.str());
			}

		case TypeAptr:
		{
			AptrDataBase *aptr = static_cast<AptrDataBase*>(varptr->at(index));
			switch(aptr->eletype)
			{
			case TypePi1D:
				if(dynamic_cast<AptrData<IntgVec1D>*>(xpr) || 
					dynamic_cast<LeftHandSideArray<IntgVec1D>*>(xpr)) 
					return;
				else HandleSyntaxError(os.str());

			case TypePi2D:
				if(dynamic_cast<AptrData<IntgVec2D>*>(xpr) || 
					dynamic_cast<LeftHandSideArray<IntgVec2D>*>(xpr)) return;
				else HandleSyntaxError(os.str());

			case TypePi3D:
				if(dynamic_cast<AptrData<IntgVec3D>*>(xpr) || 
					dynamic_cast<LeftHandSideArray<IntgVec3D>*>(xpr)) return;
				else HandleSyntaxError(os.str());

			case TypePi4D:
				if(dynamic_cast<AptrData<IntgVec4D>*>(xpr) || 
					dynamic_cast<LeftHandSideArray<IntgVec4D>*>(xpr)) return;
				else HandleSyntaxError(os.str());

			case TypePi5D:
				if(dynamic_cast<AptrData<IntgVec5D>*>(xpr) || 
					dynamic_cast<LeftHandSideArray<IntgVec5D>*>(xpr)) return;
				else HandleSyntaxError(os.str());

			case TypePr1D:
				if(dynamic_cast<AptrData<RealVec1D>*>(xpr) || 
					dynamic_cast<LeftHandSideArray<RealVec1D>*>(xpr)) return;
				else HandleSyntaxError(os.str());

			case TypePr2D:
				if(dynamic_cast<AptrData<RealVec2D>*>(xpr) || 
					dynamic_cast<LeftHandSideArray<RealVec2D>*>(xpr)) return;
				else HandleSyntaxError(os.str());

			case TypePr3D:
				if(dynamic_cast<AptrData<RealVec3D>*>(xpr) || 
					dynamic_cast<LeftHandSideArray<RealVec3D>*>(xpr)) return;
				else HandleSyntaxError(os.str());

			case TypePr4D:
				if(dynamic_cast<AptrData<RealVec4D>*>(xpr) || 
					dynamic_cast<LeftHandSideArray<RealVec4D>*>(xpr)) return;
				else HandleSyntaxError(os.str());

			case TypePr5D:
				if(dynamic_cast<AptrData<RealVec5D>*>(xpr) || 
					dynamic_cast<LeftHandSideArray<RealVec5D>*>(xpr)) return;
				else HandleSyntaxError(os.str());

			case TypePc1D:
				if(dynamic_cast<AptrData<CplxVec1D>*>(xpr) || 
					dynamic_cast<LeftHandSideArray<CplxVec1D>*>(xpr)) return;
				else HandleSyntaxError(os.str());

			case TypePc2D:
				if(dynamic_cast<AptrData<CplxVec2D>*>(xpr) || 
					dynamic_cast<LeftHandSideArray<CplxVec2D>*>(xpr)) return;
				else HandleSyntaxError(os.str());

			case TypePc3D:
				if(dynamic_cast<AptrData<CplxVec3D>*>(xpr) || 
					dynamic_cast<LeftHandSideArray<CplxVec3D>*>(xpr)) return;
				else HandleSyntaxError(os.str());

			case TypePc4D:
				if(dynamic_cast<AptrData<CplxVec4D>*>(xpr) || 
					dynamic_cast<LeftHandSideArray<CplxVec4D>*>(xpr)) return;
				else HandleSyntaxError(os.str());

			case TypePc5D:
				if(dynamic_cast<AptrData<CplxVec5D>*>(xpr) || 
					dynamic_cast<LeftHandSideArray<CplxVec5D>*>(xpr)) return;
				else HandleSyntaxError(os.str());

			case TypePv1D:
				if(dynamic_cast<AptrData<VctrVec1D>*>(xpr) || 
					dynamic_cast<LeftHandSideArray<VctrVec1D>*>(xpr)) return;
				else HandleSyntaxError(os.str());

			case TypePv2D:
				if(dynamic_cast<AptrData<VctrVec2D>*>(xpr) || 
					dynamic_cast<LeftHandSideArray<VctrVec2D>*>(xpr)) return;
				else HandleSyntaxError(os.str());

			case TypePv3D:
				if(dynamic_cast<AptrData<VctrVec3D>*>(xpr) || 
					dynamic_cast<LeftHandSideArray<VctrVec3D>*>(xpr)) return;
				else HandleSyntaxError(os.str());

			case TypePv4D:
				if(dynamic_cast<AptrData<VctrVec4D>*>(xpr) || 
					dynamic_cast<LeftHandSideArray<VctrVec4D>*>(xpr)) return;
				else HandleSyntaxError(os.str());

			case TypePv5D:
				if(dynamic_cast<AptrData<VctrVec5D>*>(xpr) || 
					dynamic_cast<LeftHandSideArray<VctrVec5D>*>(xpr)) return;
				else HandleSyntaxError(os.str());
			}
		}
		case TypeArray:
		{
			Array *pAry = static_cast<Array*>(varptr->at(index));
			switch(pAry->GetArrayType())
			{
			case TypeAi1D:
				if(dynamic_cast<Expression<IntgVec1D>*>(xpr)) return;
				else HandleSyntaxError(os.str());

			case TypeAi2D:
				if(dynamic_cast<Expression<IntgVec2D>*>(xpr)) return;
				else HandleSyntaxError(os.str());

			case TypeAi3D:
				if(dynamic_cast<Expression<IntgVec3D>*>(xpr)) return;
				else HandleSyntaxError(os.str());

			case TypeAi4D:
				if(dynamic_cast<Expression<IntgVec4D>*>(xpr)) return;
				else HandleSyntaxError(os.str());

			case TypeAi5D:
				if(dynamic_cast<Expression<IntgVec5D>*>(xpr)) return;
				else HandleSyntaxError(os.str());

			case TypeAr1D:
				if(dynamic_cast<Expression<RealVec1D>*>(xpr)) return;
				else HandleSyntaxError(os.str());

			case TypeAr2D:
				if(dynamic_cast<Expression<RealVec2D>*>(xpr)) return;
				else HandleSyntaxError(os.str());

			case TypeAr3D:
				if(dynamic_cast<Expression<RealVec3D>*>(xpr)) return;
				else HandleSyntaxError(os.str());

			case TypeAr4D:
				if(dynamic_cast<Expression<RealVec4D>*>(xpr)) return;
				else HandleSyntaxError(os.str());

			case TypeAr5D:
				if(dynamic_cast<Expression<RealVec5D>*>(xpr)) return;
				else HandleSyntaxError(os.str());

			case TypeAc1D:
				if(dynamic_cast<Expression<CplxVec1D>*>(xpr)) return;
				else HandleSyntaxError(os.str());

			case TypeAc2D:
				if(dynamic_cast<Expression<CplxVec2D>*>(xpr)) return;
				else HandleSyntaxError(os.str());

			case TypeAc3D:
				if(dynamic_cast<Expression<CplxVec3D>*>(xpr)) return;
				else HandleSyntaxError(os.str());

			case TypeAc4D:
				if(dynamic_cast<Expression<CplxVec4D>*>(xpr)) return;
				else HandleSyntaxError(os.str());

			case TypeAc5D:
				if(dynamic_cast<Expression<CplxVec5D>*>(xpr)) return;
				else HandleSyntaxError(os.str());

			case TypeAv1D:
				if(dynamic_cast<Expression<VctrVec1D>*>(xpr)) return;
				else HandleSyntaxError(os.str());

			case TypeAv2D:
				if(dynamic_cast<Expression<VctrVec2D>*>(xpr)) return;
				else HandleSyntaxError(os.str());

			case TypeAv3D:
				if(dynamic_cast<Expression<VctrVec3D>*>(xpr)) return;
				else HandleSyntaxError(os.str());

			case TypeAv4D:
				if(dynamic_cast<Expression<VctrVec4D>*>(xpr)) return;
				else HandleSyntaxError(os.str());

			case TypeAv5D:
				if(dynamic_cast<Expression<VctrVec5D>*>(xpr)) return;
				else HandleSyntaxError(os.str());
			}
		}

		case TypePtr:
		{
			PtrDataBase *ptr = static_cast<PtrDataBase*>(varptr->at(index));
			switch(ptr->GetEleType())
			{
			case TypeBool:
				{	if( dynamic_cast<PtrData<bool>*>(xpr)  ||
						dynamic_cast<LeftHandSide<bool>*>(xpr)) return;
					else
						HandleSyntaxError(L"Either boolean variable or pointer to boolean variable is required.\r\n");
				}
			case TypeStrn:
				{	if( dynamic_cast<PtrData<string>*>(xpr)  ||
						dynamic_cast<LeftHandSide<string>*>(xpr)) return;
					else
						HandleSyntaxError(L"Either string variable or pointer to string variable is required.\r\n");
				}

			case TypeIntg:
				{	if( dynamic_cast<PtrData<integer>*>(xpr)  ||
						dynamic_cast<LeftHandSide<integer>*>(xpr)) return;
					else
						HandleSyntaxError(L"Either integer variable or pointer to integer variable is required.\r\n");
				}
			case TypeReal:
				{	if( dynamic_cast<PtrData<realnum>*>(xpr)  ||
						dynamic_cast<LeftHandSide<realnum>*>(xpr)) return;
					else
						HandleSyntaxError(L"Either real number variable or pointer to real number variable is required.\r\n");
				}
			case TypeCplx:
				{	if( dynamic_cast<PtrData<Complex>*>(xpr)  ||
						dynamic_cast<LeftHandSide<Complex>*>(xpr)) return;
					else
						HandleSyntaxError(L"Either complex variable or pointer to complex variable is required.\r\n");
				}

			case TypeVctr:
				{	if( dynamic_cast<PtrData<Vector>*>(xpr)  ||
						dynamic_cast<LeftHandSide<Vector>*>(xpr)) return;
					else
						HandleSyntaxError(L"Either vector variable or pointer to vector variable is required.\r\n");
				}

			case TypeUPtr:
				{
					LeftHandSide<UserData> *pLHSUD; UdtPtrBase *ptrData;

					if(pLHSUD = dynamic_cast<LeftHandSide<UserData>*>(xpr))
					{
						if(pLHSUD->IsCompatible(static_cast<DeclarePtr<UserData>*>
							(pStmtList->GetStatement(index))->factory)) return;
						else
							HandleSyntaxError(L"User Defined Data Type mismatch.\r\n");
					}
					else if(ptrData = dynamic_cast<UdtPtrBase*>(xpr))
					{
						if(ptrData->pData->factory==NULL)
							HandleSyntaxError(L"Unitialized user defined type pointer variable is passed as argument.\r\n");
						
						if(static_cast<DeclarePtr<UserData>*>
							(pStmtList->GetStatement(index))->factory
							==ptrData->pData->factory) return;
						else
							HandleSyntaxError(L"User Defined Data Type mismatch.\r\n");
					}
					else
						HandleSyntaxError(L"Either user defined data variable or pointer to user defined data variable is required.\r\n");
				}
			}
		}
		default:
			HandleSyntaxError(_T("Unsupported argument type.\r\n"));
	}
}

UserData* Function::SetUdtFromParent()
{
	if(this->pParen) 
		return this->pUdt = this->pParen->pUdt != NULL ? 
			this->pParen->pUdt : this->pParen->SetUdtFromParent();
	else return this->pUdt;
}

bool Function::IsReturn() const
{
	if(pStmtList) return pStmtList->IsReturn();
	else return false;
}

void Function::Run()
{
	SET_USER_ABORT;

	extern Stack<StatementList*> gStmtListStack;
	StatementList *pStmt = gStmtListStack.top();
	try
	{
		if(pStmtList)
		{
			pStmtList->Run();
		}
	}
	catch(GotoException& e)
	{

		while(pStmt!=gStmtListStack.top())
		{	
			pSlist=gStmtListStack.pop();
			if(!pSlist->funcstmt) pSlist->CleanupStack();
		}

		std::wostringstream os;
		os<<_T("Can not goto ")<<e.what()<<_T(", it is out of function block.\r\n");

		throw UnicodeException(os.str());

	}
	catch(ReturnException&)
	{
		while(pStmt!=gStmtListStack.top()) 
		{			
			pSlist=gStmtListStack.pop();
			if(!pSlist->funcstmt) pSlist->CleanupStack();
		}
		return;
	}
	catch(StmtThrowBase&)
	{
		while(pStmt!=gStmtListStack.top())
		{	
			pSlist=gStmtListStack.pop();
			if(!pSlist->funcstmt) pSlist->CleanupStack();
		}

		throw;
	}

		
	if(rtntype != TypeVoid)
	{
		while(pStmt!=gStmtListStack.top()) 
		{			
			pSlist=gStmtListStack.pop();
			if(!pSlist->funcstmt) pSlist->CleanupStack();
		}

		if(!gBreak)
		{
			std::wostringstream os;
			os<<_T("Function \'")<<fname<<_T("\' should return a value.\r\n");
			throw RunTimeException(os.str());
		}
	}
}

std::wstring Function::GetName() { return fname; }

Function::~Function() { }

StmtListBase::StmtListBase(){}

NameUdtMap* StmtListBase::GetFactoryMap() { return &udtmap; }

void StmtListBase::AddFactory(UserDefinedType *udt) 
{  
	if(!udtmap.insert(NameUdtPair(udt->GetTypeTag(), udt)).second)
	{
		std::wostringstream os; 
		os<<_T("\'")<<udt->GetTypeTag()<<_T("\' ")<<LoadUnicodeMsg(IDS_ALREADY_DECLARED);

		HandleSyntaxError(os.str());
	}
}

Function* StmtListBase::FindFunction(const std::wstring& id)
{
	return this->funclist.FindFunction(id);
}

#ifdef new
#define OLD_NEW new
#undef new 
#endif

StmtOutput::StmtOutput(Expression<stream&> *pstrm): 
pStream(pstrm), loc(std::locale(), new MyPunck),
os(), initialized(false) { }


#ifdef OLD_NEW
#define new OLD_NEW
#undef OLD_NEW
#endif


void StmtOutput::SetStream()
{
	if(initialized) return;
	initialized = true;

	extern int  fmt_precision;
	extern bool fmt_clear;
	extern bool fmt_scientific;
	extern bool fmt_showpoint;
	extern bool fmt_thousand;

	if(pStream)
	{
		if(fmt_thousand) 
			pStream->Eval().imbue(loc);
		
		if(!fmt_clear)
		{
			pStream->Eval()<<std::setprecision(fmt_precision);
			
			if(fmt_scientific)  pStream->Eval()<<std::scientific;
			else pStream->Eval()<<std::fixed;

			if(fmt_showpoint) pStream->Eval()<<std::showpoint;
			else pStream->Eval()<<std::noshowpoint;
		}
		
		pStream->Eval()<<std::boolalpha;
	}
	else 
	{
		if(fmt_thousand) os.imbue(loc);
		if(!fmt_clear)
		{
			os<<std::setprecision(fmt_precision);
			
			if(fmt_scientific)  os<<std::scientific;
			else os<<std::fixed;

			if(fmt_showpoint) os<<std::showpoint;
			else os<<std::noshowpoint;
		}

		os<<std::boolalpha;
	}
}
void StmtOutput::InsertExpr(ExpTypeless* expr)
{
	exprvec.push_back(expr);
}

void StmtOutput::Run()
{  
	ExpEndLine*			 endline;
	ExpBool*    bexpr;
	ExpIntg* iexpr;
	ExpReal* rexpr;
	ExpCplx* cexpr;
	ExpVctr*  vexpr;
	ExpStrn*  strpr;
	Expression<stream&>* stmpr;

	SetStream();

	if(pStream)
	{
		for(ItrType itr = exprvec.begin(); itr!=exprvec.end(); ++itr)
		{
			SET_USER_ABORT;

			bexpr = dynamic_cast<ExpBool*>(*itr);
			if(bexpr) { pStream->Eval()<<bexpr->Eval(); continue; }

			iexpr = dynamic_cast<ExpIntg*>(*itr);
			if(iexpr) {  pStream->Eval()<<iexpr->Eval(); continue; }

			rexpr = dynamic_cast<ExpReal*>(*itr);
			if(rexpr) { pStream->Eval()<<rexpr->Eval(); continue; }

			cexpr = dynamic_cast<ExpCplx*>(*itr);
			if(cexpr) {	pStream->Eval()<<cexpr->Eval(); continue; }

			vexpr = dynamic_cast<ExpVctr*>(*itr);
			if(vexpr) { pStream->Eval()<<vexpr->Eval(); continue; }

			strpr = dynamic_cast<ExpStrn*>(*itr);
			if(strpr) { pStream->Eval()<<strpr->Eval(); continue; }
			
			stmpr = dynamic_cast<Expression<stream&>*>(*itr);
			if(stmpr) { pStream->Eval()<<stmpr->Eval().str(); continue; }

			endline = dynamic_cast<ExpEndLine*>(*itr);
			if(endline) { pStream->Eval()<<_T("\r\n"); continue; }
		}
	}
	else
	{
		for(ItrType itr = exprvec.begin(); itr!=exprvec.end(); ++itr)
		{
			SET_USER_ABORT;

			bexpr = dynamic_cast<ExpBool*>(*itr);
			if(bexpr) { os<<bexpr->Eval(); continue; }

			iexpr = dynamic_cast<ExpIntg*>(*itr);
			if(iexpr) {  os<<iexpr->Eval(); continue; }

			rexpr = dynamic_cast<ExpReal*>(*itr);
			if(rexpr) { os<<rexpr->Eval(); continue; }

			cexpr = dynamic_cast<ExpCplx*>(*itr);
			if(cexpr) {	os<<cexpr->Eval(); continue; }

			vexpr = dynamic_cast<ExpVctr*>(*itr);
			if(vexpr) { os<<vexpr->Eval(); continue; }

			strpr = dynamic_cast<ExpStrn*>(*itr);
			if(strpr) { os<<strpr->Eval(); continue; }

			stmpr = dynamic_cast<Expression<stream&>*>(*itr);
			if(stmpr) { os<<stmpr->Eval().str(); continue; }

			endline = dynamic_cast<ExpEndLine*>(*itr);
			if(endline) { os<<_T("\r\n"); continue; }
		}

		PutString(os.str());
		os.str(L""); os.clear();
	}
}

StmtOutput::~StmtOutput() { }

StmtInput::StmtInput(Expression<stream&> *pstrm): 
	pStream(pstrm) { }

void StmtInput::InsertPrompt(const wchar_t* prompt, ExpTypeless* pData)
{
	sdvec.push_back(StrDataPair(CString(prompt), pData));
}

void StmtInput::Run()
{  
	SET_USER_ABORT;

	extern ProgStatus gProgStatus;
	
	dialog.InitializePrompt(sdvec);
	
	if(dialog.DoModal() == 2) SetBreak();
	
	gProgStatus = PS_RunTime;
}

StmtInput::~StmtInput() { }

StmtFor::StmtFor(Statement* st1, ExpBool* xpr, Statement* st2, Statement* st):
	stmt1(st1), expr(xpr), stmt2(st2), stmt(st), Statement(StmtLoop) { }

bool StmtFor::IsReturn() const
{
	if(stmt) return stmt->IsReturn();
	else return false;
}

void StmtFor::Run()
{
	extern Stack<StatementList*> gStmtListStack;
	StatementList* pStackTop = gStmtListStack.top();

	if(stmt1) stmt1->Run();
	for( ; expr ? expr->Eval(): true ; )
	{
		SET_USER_ABORT;

		try
		{
			if(stmt) stmt->Run();
		}
		catch(BreakException&)
		{
			break;
		}
		catch(SkipException&)
		{
			while(pStackTop != gStmtListStack.top())
				gStmtListStack.pop();
		}

		if(stmt2) stmt2->Run();
	}

	while(pStackTop != gStmtListStack.top())
		gStmtListStack.pop();
}

StmtFor::~StmtFor() { }
	
StmtWhile::StmtWhile(ExpBool* expr, Statement* stmt): 
  pExpr(expr), pStmt(stmt), Statement(StmtLoop) { }

bool StmtWhile::IsReturn() const 
{
	if(pStmt) return pStmt->IsReturn();
	else return false;
}

void StmtWhile::Run()
{
	extern Stack<StatementList*> gStmtListStack;
	StatementList* pStackTop = gStmtListStack.top();

	while(pExpr ? pExpr->Eval(): true)
	{
		SET_USER_ABORT;

		try
		{
			if(pStmt) pStmt->Run();
		}
		catch(BreakException&)
		{
			break;
		}
		catch(SkipException&)
		{
			while(pStackTop != gStmtListStack.top())
				gStmtListStack.pop();
		}
		
	}

	while(pStackTop != gStmtListStack.top())
		gStmtListStack.pop();
}

StmtWhile::~StmtWhile() { }	


StmtDo::StmtDo(ExpBool* expr, Statement* stmt): 
  pExpr(expr), pStmt(stmt), pUpdate(NULL), Statement(StmtLoop) { }

bool StmtDo::IsReturn() const
{
	if(pStmt) return pStmt->IsReturn();
	else return false;
}

void StmtDo::Run()
{
	extern Stack<StatementList*> gStmtListStack;
	StatementList* pStackTop = gStmtListStack.top();

	do
	{
		SET_USER_ABORT;

		try
		{
			if(pUpdate) pUpdate->Run();

			if(pStmt) pStmt->Run();
		}
		catch(BreakException&)
		{
			break;
		}
		catch(SkipException&)
		{
			while(pStackTop != gStmtListStack.top())
				gStmtListStack.pop();
		}
	} while(pExpr ? pExpr->Eval(): true);

	while(pStackTop != gStmtListStack.top())
		gStmtListStack.pop();
}

StmtDo::~StmtDo() { }	

void FuncList::insert(const std::wstring& name, const FunSig& fs)
{
	Itr itr = funclist.find(name);
	if(itr != funclist.end())
	{
		std::wostringstream os;
		os<<_T("FuncList::insert(): \'")<<name<<_T("\' ")<<LoadUnicodeMsg(IDS_FUNC_EXISTS);
		throw UnicodeException(os.str());
	}
	else
		funclist.insert(Pair(name, fs));
}

void FuncList::find(FunName& fn)
{
	Itr itr = funclist.find(fn.fn);
	
	if(itr == funclist.end())
		fn.fsig.type = FunNull;
	else
		fn.fsig = itr->second;
}
void FuncList::clear() { funclist.clear(); }

StmtListBase::~StmtListBase() {}

int StatementList::PrepareStack() { return varlist.Push(); }
int StatementList::CleanupStack() { return varlist.Pop(); }

StatementList::StatementList()
: stmtindex(0), funcstmt(false), Statement(StmtList) {  }

VarList* StatementList::GetVarList() { return &varlist; }
	
int StatementList::Add(Statement* stmt)
{
	if(!stmt) return -1;
	int stmtsize = int(stmtlist.size());

	if(stmtsize>0 && stmt->stmttype!=StmtLabel)
	{
		if(stmtlist[stmtsize-1]->stmttype==StmtExit)
			return HandleSyntaxError(_T("Code not reachable after exit statement.\r\n"));
		else if(stmtlist[stmtsize-1]->stmttype==StmtGoto)
			return HandleSyntaxError(_T("Code not reachable after goto statement.\r\n"));
		else if(stmtlist[stmtsize-1]->IsReturn())
			return HandleSyntaxError(_T("Code not reachable after return statement.\r\n"));
		else if(stmtlist[stmtsize-1]->stmttype==StmtSkip)
			return HandleSyntaxError(_T("Code not reachable after continue statement.\r\n"));
		else if(stmtlist[stmtsize-1]->stmttype==StmtBreak)
			return HandleSyntaxError(_T("Code not reachable after break statement.\r\n"));
		else if(stmtlist[stmtsize-1]->stmttype==StmtThrw)
			return HandleSyntaxError(_T("Code not reachable after throw statement.\r\n"));
	}

	++stmtindex;

	stmtlist.push_back(stmt);
	int index = int(stmtlist.size()-1);
	stmt->Initialize(varlist);
	
	return index;
}

int StatementList::AddToHead(Statement* stmt)
{
	if(!stmt) return -1;
	int stmtsize = int(stmtlist.size());

	++stmtindex;  stmtlist.push_front(stmt);
	int index = int(stmtlist.size()-1);
	stmt->Initialize(varlist);
	
	return index;
}

StmtVector& StatementList::GetStmtList() { return stmtlist; }

// if fails, returns -1, otherwise returns stmtlist index
int StatementList::FindLabel(std::wstring& id)
{
	int size = int(stmtlist.size());

	for(int i=0; i<size; ++i)
	{
		if(stmtlist[i]->IsLabel(id)) return i;
	}

	return -1;
}

void StmtListBase::AddUDFunction(const wchar_t* id, Function* fptr)
{
	this->funclist.InsertFunction(id, fptr);
}

void StmtListBase::CheckImplemented() const
{
	funclist.CheckImplemented();
	for(NameUdtMapCItr itr=udtmap.begin();
		itr != udtmap.end(); ++itr)
	{
		itr->second->CheckImplemented();
	}
}

// if fails, returns -1, otherwise returns stmtlist index
bool StatementList::FindLoop()
{
	std::vector<Statement*>::size_type size = stmtlist.size();
	for(std::vector<Statement*>::size_type i=0; i<size; ++i)
	{
		SET_USER_ABORT;
		if(stmtlist[i]->stmttype == StmtLoop) return true;
	}
	return false;
}

void StatementList::CheckImplemented() const
{
	StmtListBase::CheckImplemented();
	int count = int(stmtlist.size());
	StatementList *ptr;

	for(int i=0; i<count; ++i)
	{
		ptr = dynamic_cast<StatementList*>(stmtlist[i]);
		
		if(ptr) ptr->CheckImplemented();
	}
}

void StatementList::Run()
{
	int size = int(stmtlist.size());
	extern Stack<StatementList*> gStmtListStack;
	extern __declspec(align(32)) volatile LONG gBreak;
	
	StatementList* pStmt=NULL, *pSlist;
	
	gStmtListStack.push(this);
	pStmt = gStmtListStack.top();
	if(!pStmt->funcstmt) pStmt->PrepareStack();

	for(int i=0; i<size; ++i)
	{
		SET_USER_ABORT;

		stmtindex = i;
						
		try { stmtlist[i]->Run(); }
		catch(GotoException& id)
		{
			int index = FindLabel(std::wstring(id.GetMessage()));
			
			if(index >= 0) 
			{
				i = index;

				while(pStmt!=gStmtListStack.top()) gStmtListStack.pop();
				
				continue;
			}
			else
			{
				while(pStmt!=gStmtListStack.top()) gStmtListStack.pop();
				gStmtListStack.pop(); throw;		
			}
		}
		catch(BreakException&)
		{
			while(pStmt!=gStmtListStack.top()) gStmtListStack.pop();
			gStmtListStack.pop(); throw;
		}
		catch(StmtThrowBase&)
		{
			while(pStmt!=gStmtListStack.top()) 
			{
				pSlist=gStmtListStack.pop();
				if(!pSlist->funcstmt) pSlist->CleanupStack();
			}
			
			pSlist=gStmtListStack.pop(); 
			if(!pSlist->funcstmt) pSlist->CleanupStack();

			throw;
		}
				
		while(pStmt!=gStmtListStack.top())
		{
			pSlist=gStmtListStack.pop();
			if(!pSlist->funcstmt) pSlist->CleanupStack();
		}
	}
	
	while(pStmt!=gStmtListStack.top())
	{
		pSlist=gStmtListStack.pop();
		if(!pSlist->funcstmt) pSlist->CleanupStack();
	}

	pSlist=gStmtListStack.pop();
	if(!pSlist->funcstmt) pSlist->CleanupStack();
}


Statement* StatementList::FindDeclare(const std::wstring& id)
{
	std::vector<Statement*>::size_type size = stmtlist.size();

	for(std::vector<Statement*>::size_type i=0; i<size; ++i)
	{
		SET_USER_ABORT;

		if(stmtlist[i]) 
		{
			if(stmtlist[i]->stmttype == StmtDecl)
			{
				if(stmtlist[i]->GetVarName() == id)
					return stmtlist[i];
			}
		}
	}

	throw UnicodeException(LoadUnicodeMsg(IDS_FIND_DECLARE_FAILED));
}

bool StatementList::IsReturn() const
{
	int count = int(this->stmtlist.size());

	if(count>0) return this->stmtlist[count-1]->IsReturn();
	else return false;
}

Statement* StatementList::FindAssignment(const std::wstring& id)
{
	for(int i=stmtindex-1; i>=0; --i)
	{
		SET_USER_ABORT;

		if(stmtlist[i]) 
		{
			if(stmtlist[i]->stmttype==StmtExpr) 
			{
				BaseAssign* asg_ptr = 
					dynamic_cast<BaseAssign*>(stmtlist[i]);

				if(asg_ptr)
				{
					if(asg_ptr->vrid == id)
						return stmtlist[i];
					else
						continue;
				}
				else
					throw UnicodeException(LoadUnicodeMsg(IDS_ASSGN_FIND_TYPE_MISMATCH).c_str());
			}
			
			if(stmtlist[i]->stmttype == StmtExpr)
			{
				StatementList* pStmtList = dynamic_cast<StatementList*>(stmtlist[i]);

				if(pStmtList)
				{
					Statement* pStmt = pStmtList->FindAssignment(id);
					if(pStmt) return pStmt;
				}
			}
			
		}
	}
	return NULL;
}

StatementList::~StatementList() { }

StmtIf::StmtIf(ExpBool* xpr, Statement *pstmt, bool b, Statement *pelse): 
	  pExpr(xpr), pStmt(pstmt), pElse(pelse), kind(b) { }
	
void StmtIf::Run()
{
	SET_USER_ABORT;

	if(pExpr->Eval()==kind) 
	{
		if(pStmt) pStmt->Run();
	}
	else
	{
		if(pElse) pElse->Run();
	}
}

bool StmtIf::IsReturn() const
{
	if(pElse && pStmt->IsReturn() && pElse->IsReturn()) return true;
	else return false;
}

StmtIfElse::StmtIfElse(ExpBool* xpr, Statement* stmt1, Statement* stmt2): 
	  pExpr(xpr), pStmt1(stmt1), pStmt2(stmt2) { }
	
void StmtIfElse::Run()
{
	if(pExpr->Eval()) 
	{
		if(pStmt1) pStmt1->Run();
	}
	else 
	{
		if(pStmt2) pStmt2->Run();
	}
}

bool StmtIfElse::IsReturn() const
{
	if(pStmt2 && pStmt1->IsReturn() && pStmt2->IsReturn()) return true;
	else return false;
}

AptrDataBase::AptrDataBase(const type_info& type, std::wstring vrid, bool vstatic): Data(vrid, TypeAptr, vstatic)
{
	if(type == typeid(IntgVec1D))		eletype = TypePi1D;
	else if(type == typeid(IntgVec2D))	eletype = TypePi2D;
	else if(type == typeid(IntgVec3D))	eletype = TypePi3D;
	else if(type == typeid(IntgVec4D))	eletype = TypePi4D;
	else if(type == typeid(IntgVec5D))	eletype = TypePi5D;
	else if(type == typeid(RealVec1D))	eletype = TypePr1D;
	else if(type == typeid(RealVec2D))	eletype = TypePr2D;
	else if(type == typeid(RealVec3D))	eletype = TypePr3D;
	else if(type == typeid(RealVec4D))	eletype = TypePr4D;
	else if(type == typeid(RealVec5D))	eletype = TypePr5D;
	else if(type == typeid(CplxVec1D))	eletype = TypePc1D;
	else if(type == typeid(CplxVec2D))	eletype = TypePc2D;
	else if(type == typeid(CplxVec3D))	eletype = TypePc3D;
	else if(type == typeid(CplxVec4D))	eletype = TypePc4D;
	else if(type == typeid(CplxVec5D))	eletype = TypePc5D;
	else if(type == typeid(VctrVec1D))	eletype = TypePv1D;
	else if(type == typeid(VctrVec2D))	eletype = TypePv2D;
	else if(type == typeid(VctrVec3D))	eletype = TypePv3D;
	else if(type == typeid(VctrVec4D))	eletype = TypePv4D;
	else if(type == typeid(VctrVec5D))	eletype = TypePv5D;
	else eletype = TypeNull;
}

void UDFuncList::CheckImplemented() const
{
	for(FuncMapCItr itr=theMap.begin(); itr != theMap.end(); ++itr)
	{
		if(!itr->second->pStmtList->IsLastStmtList())
		{
			extern int gLastLine, gLastCharCount;
			std::wostringstream os;
			os<<_T("Function \'")<<itr->first<<_T("\' is declared but NOT defined.\r\n");
			gLastLine = itr->second->scanfile.lineno;
			gLastCharCount = itr->second->scanfile.charcount;
			HandleSyntaxError(os.str());
		}
		else
			itr->second->pStmtList->CheckImplemented();
	}
}

void ReturnUdt::Run()
{
	extern Stack<StatementList*> gStmtListStack;
	void *ptr = rtnStack->top();

	if(ptr && pExp) ((UserData*)ptr)->Copy(pExp->GetData());
		
	int i=1;
	StatementList *pSL = gStmtListStack.getoffset(i++);
	while(pStmtList != pSL)
	{
		pSL->CleanupStack();
		pSL = gStmtListStack.getoffset(i++);
	}
	
	throw ReturnException();
}

