#include "StdAfx.h"
#include "ArtText.h"

CArtText::CArtText(): bInitial(true), theWnd(NULL)
{

}

CArtText::~CArtText(void)
{
	if(bInitial==false)
	{
		dcMem.Detach();
		dcMem.DeleteDC();
		theBitmap.DeleteObject();
	}
}

void CArtText::DrawOutlineText(int y, UINT rcid, 
						int fontsize, const CString& fontname, 
						COLORREF textColor, COLORREF outlineColor, int depth)
{
	CString msg; msg.LoadString(rcid);
	DrawOutlineText(y, msg, fontsize, fontname, textColor, outlineColor, depth);
}


void CArtText::DrawOutlineText(int y, const CString& text, 
						int fontsize, const CString& fontname, 
						COLORREF textColor, COLORREF outlineColor, int depth)
{
	ASSERT(theWnd);
	int x;

	if(bInitial)
	{
		bInitial = false;
		CClientDC dcScreen(theWnd); 
		
		CFont theFont, *pOldFont;
		theFont.CreatePointFont(fontsize, fontname, &dcScreen);
		pOldFont = (CFont*)dcScreen.SelectObject(&theFont);

		CRect winRect; theWnd->GetClientRect(&winRect);
		dcScreen.DPtoLP(&winRect);
		CSize size = dcScreen.GetTextExtent(text);  
		x = (winRect.Width()-size.cx)/2;
		oldX = x-depth; oldY = y-depth; 
		
		size.cx += depth<<1; size.cy += depth<<1;
		oldSize = size; 

		theBitmap.CreateCompatibleBitmap(&dcScreen, size.cx, size.cy);
		dcMem.CreateCompatibleDC(&dcScreen);
		dcMem.SelectObject(&theBitmap);
		dcMem.BitBlt(0, 0, size.cx, size.cy, &dcScreen, x-depth, y-depth, SRCCOPY);
		
		dcScreen.SetBkMode(TRANSPARENT);

		COLORREF oldColor = dcScreen.GetTextColor();
		dcScreen.SetTextColor(outlineColor);
		for(int i=-depth; i<=depth; ++i)
			for(int j=-depth; j<=depth; ++j)
				dcScreen.TextOut(x+i, y+j, text);

		dcScreen.SetTextColor(textColor);
		dcScreen.TextOut(x, y, text);
		dcScreen.SetTextColor(oldColor);
		dcScreen.SelectObject(pOldFont);
	}
	else
	{
		CClientDC dcScreen(theWnd); 
		dcScreen.BitBlt(oldX, oldY, oldSize.cx, oldSize.cy, &dcMem, 0, 0, SRCCOPY);
		dcMem.Detach();
		dcMem.DeleteDC();
		theBitmap.DeleteObject();

		CFont theFont, *pOldFont;
		theFont.CreatePointFont(fontsize, fontname, &dcScreen);
		pOldFont = (CFont*)dcScreen.SelectObject(&theFont);
		
		CRect winRect; theWnd->GetClientRect(&winRect);
		dcScreen.DPtoLP(&winRect);
		CSize size = dcScreen.GetTextExtent(text);  
		x = (winRect.Width()-size.cx)/2;
		oldX = x-depth; oldY = y-depth; 
		
		size.cx += depth<<1; size.cy += depth<<1;
		oldSize = size;

		theBitmap.CreateCompatibleBitmap(&dcScreen, size.cx, size.cy);
		dcMem.CreateCompatibleDC(&dcScreen);
		dcMem.SelectObject(&theBitmap);
		dcMem.BitBlt(0, 0, size.cx, size.cy, &dcScreen, x-depth, y-depth, SRCCOPY);
		
		dcScreen.SetBkMode(TRANSPARENT);
		
		COLORREF oldColor = dcScreen.GetTextColor();
		dcScreen.SetTextColor(outlineColor);
		for(int i=-depth; i<=depth; ++i)
			for(int j=-depth; j<=depth; ++j)
				dcScreen.TextOut(x+i, y+j, text);

		dcScreen.SetTextColor(textColor);
		dcScreen.TextOut(x, y, text);
		dcScreen.SetTextColor(oldColor);
		dcScreen.SelectObject(pOldFont);
	}
}

void CArtText::DrawTitleText(int y, UINT rcid, 
						int fontsize, const CString& fontname, 
						COLORREF textColor, COLORREF outlineColor, int depth)
{
	CString msg; msg.LoadString(rcid);
	DrawTitleText(y, msg, fontsize, fontname, textColor, outlineColor, depth);
}

void CArtText::DrawTitleText(int y, const CString& text, 
						int fontsize, const CString& fontname, 
						COLORREF textColor, COLORREF outlineColor, int depth)
{
	ASSERT(theWnd);
	int x; 

	CClientDC dcScreen(theWnd); 
	CFont theFont, *pOldFont;
	theFont.CreatePointFont(fontsize, fontname, &dcScreen);
	pOldFont = (CFont*)dcScreen.SelectObject(&theFont);

	CSize size = dcScreen.GetTextExtent(text);  
	CRect winRect; theWnd->GetClientRect(&winRect);
	dcScreen.DPtoLP(&winRect);
	x = (winRect.Width()-size.cx)/2;
	
	size.cx += depth<<1; size.cy += depth<<1;
	oldSize = size; 

	dcScreen.SetBkMode(TRANSPARENT);

	COLORREF oldColor = dcScreen.GetTextColor();
	dcScreen.SetTextColor(outlineColor);
	
	for(int i=-depth; i<=depth; ++i)
		for(int j=-depth; j<=depth; ++j)
			dcScreen.TextOut(x+i, y+j, text);

	dcScreen.SetTextColor(textColor);
	dcScreen.TextOut(x, y, text);
	dcScreen.SetTextColor(oldColor);
	dcScreen.SelectObject(pOldFont);
}