// CodeEditor.cpp : implementation file
//

#include "stdafx.h"
#include "CodeEditor.h"
#include "YalDoc.h"
#include "Yal.h"

struct TipString
{
	LPCTSTR lpszTip;
	LPCTSTR lpszContent;
};

static TipString g_aTipText[] = 
{
	{_T("CMyClass"),_T("class CMyClass")},
	{_T("ind"),_T("int ind = -1")},
	{_T("memset"),_T("void * memset(void *, int, size_t)")},
	{_T("m_lf"),_T("LOGFONT CMyClass::m_lf")},
	{_T("m_bTrace"),_T("BOOL CMyClass::m_bTrace")},
	{_T("OnDestroy"),_T("void CMyClass::OnDestroy ()")},
	{_T("DoMain"),_T("int CMyClass::DoMain ()")},
	{_T("printf"),_T("int printf (const char *, ...)")}, 
	{_T("GetAppName"),_T("const char* CMyClass::GetAppName()")},
	{_T("EnableTrace"),_T("void CMyClass::EnableTrace(BOOL bTrace = TRUE)")},
	{_T("bTrace"),_T("BOOL bTrace = TRUE")},
	{_T("DO_TRACE"),_T("#define DO_TRACE(bTrace, lpszText)  { if (bTrace) { printf(\"%s\\n\",lpszText); } }")},
	{_T("main"),_T("int main(int argc, char* argv[])")},
	{_T("pMyClass"),_T("CMyClass* pMyClass = new CMyClass(argc, argv)")},
	{_T("nRet"),_T("int nRet = pMyClass->DoMain()")}
};

struct StringIndex
{
	LPCTSTR lpszName;
	int		nIndex;
};

static StringIndex g_aStringIndex1[] =
{
	{_T("CMyClass"), 3},
	{_T("DoMain"), 3},
	{_T("EnableTrace"), 3},
	{_T("GetAppName"), 3},
	{_T("m_argc"), 5},
	{_T("m_argv"), 5},
	{_T("m_bTrace"), 4},
	{_T("m_lf"), 4},
	{_T("OnDestroy"), 6},
	{ NULL }
};

static StringIndex g_aStringIndex2[] =
{
	{_T("lfHeight"), 7},
	{_T("lfWidth"), 7},
	{_T("lfEscapement"), 7},
	{_T("lfOrientation"), 7},
	{_T("lfWeight"), 7},
	{_T("lfItalic"), 7},
	{_T("lfUnderline"), 7},
	{_T("lfStrikeOut"), 7},
	{_T("lfCharSet"), 7},
	{_T("lfOutPrecision"), 7},
	{_T("lfClipPrecision"), 7},
	{_T("lfQuality"), 7},
	{_T("lfPitchAndFamily"), 7},
	{_T("lfFaceName"), 7},
	{ NULL }
};

struct IntelliStringArray
{
	LPCTSTR lpszName;
	StringIndex* ptr;
};

static IntelliStringArray g_aIntelliStringArray[] =
{
	{_T("this->"), g_aStringIndex1},
	{_T("pMyClass->"), g_aStringIndex1},
	{_T("CMyClass::"), g_aStringIndex1},
	{_T("this->m_lf."), g_aStringIndex2},
	{_T("pMyClass->m_lf."), g_aStringIndex2},
	{_T("CMyClass::m_lf."), g_aStringIndex1},
	{_T ("this->m_lf.lfFaceName->"), NULL},
	{_T ("pMyClass->m_lf.lfFaceName->"), NULL},
	{_T ("this->m_argv->"), NULL},
	{_T ("pMyClass->m_argv->"), NULL},
};

LANGFLAG LangFlag;

// Define code pages 
int nCodePage[5] = {
	0,		    // DEFAULT 
	950,		// TRADITIONAL CHINESE
	932,		// JAPANESE
	949,		// KOREAN 
	936			// SIMPLIFIED CHINESE
};

// Define charset
BYTE nCharSet[5] = {
	DEFAULT_CHARSET,		// Default 
	CHINESEBIG5_CHARSET,	// TRADITIONAL CHINESE
	SHIFTJIS_CHARSET,		// JAPANESE
	HANGUL_CHARSET,			// KOREAN
	GB2312_CHARSET			// SIMPLIFIED CHINESE
};

// Define Default font
char szDefaultFontName[5][19] = {
	"\x54\x00\x61\x00\x68\x00\x6F\x00\x6D\x00\x61\x00\x00\x00",					// Default font
	"\xB0\x65\x30\x7D\x0E\x66\xD4\x9A\x00\x00",									// TRADITIONAL_CHINESE font
	"\x2D\xFF\x33\xFF\x20\x00\x30\xFF\xB4\x30\xB7\x30\xC3\x30\xAF\x30\x00\x00",	// JAPANESE font
	"\x74\xAD\xBC\xB9\x00\x00",													// KOREAN font
	"\xB0\x65\x8B\x5B\x53\x4F\x00\x00"											// SIMPLIFIED_CHINESE font
};

TCHAR szEnglishFontName[5][11] = {
	_T("Tahoma"),		_T("MingLiU"),		_T("MS PGothic"), 
	_T("Gulim"),		_T("NSimSun") };


// CCodeEditor

IMPLEMENT_DYNCREATE(CCodeEditor, CBCGPEditCtrl)

CCodeEditor::CCodeEditor() 
{ 
	SetLanguage((HKL)0); 
	
	m_bEnableBreakpoints = FALSE;
	m_bCheckColorTags = FALSE;
	m_bCopyRTFToClipboard = TRUE;

	PrepareIntelliSense();
	EnableGradientMarkers (TRUE);
}

CCodeEditor::~CCodeEditor() { }


BEGIN_MESSAGE_MAP(CCodeEditor, CBCGPEditCtrl)
	ON_WM_LBUTTONDBLCLK()
	ON_WM_CREATE()
	ON_WM_MOUSEWHEEL()
END_MESSAGE_MAP()

// CCodeEditor message handlers

const CString& CCodeEditor::GetText() const
{
	return m_strBuffer;
}

void CCodeEditor::StartIMEComposition() 
{
	HIMC hIMC; CPoint point;
    COMPOSITIONFORM Composition;
	hIMC = ImmGetContext(this->m_hWnd);

	SyncIMEFont();

	if(hIMC) 
	{
		// Set composition window position near caret position
		point = CBCGPEditCtrl::GetCaretPos();

		Composition.dwStyle = CFS_POINT;
		Composition.ptCurrentPos.x = point.x;
		Composition.ptCurrentPos.y = point.y;
		
		ImmSetCompositionWindow(hIMC, &Composition);
		ImmReleaseContext(this->m_hWnd,hIMC);
		
		if(this->m_nLanguage == KOREAN)
		{
			m_CaretSize = CBCGPEditCtrl::GetCaretSize();
			::DestroyCaret(); 
			CreateSolidCaret(m_CaretSize.cy-1, m_CaretSize.cy-1); ShowCaret();
		}
	}
}

void CCodeEditor::SyncIMEFont()
{
	HIMC hIMC = ImmGetContext(this->m_hWnd);

	if(hIMC) 
	{
		LOGFONT lf; m_Font.GetLogFont(&lf);
		switch(this->m_nLanguage)
		{
		case KOREAN:
			{				
				ImmSetCompositionFont(hIMC, &lf);
				ImmReleaseContext(this->m_hWnd,hIMC);	
				return;	
			}

		case JAPANESE:
			{				
				ImmSetCompositionFont(hIMC, &lf);
				ImmReleaseContext(this->m_hWnd,hIMC);	
				return;	
			}

		case SIMPLIFIED_CHINESE:
			{				
				ImmSetCompositionFont(hIMC, &lf);
				ImmReleaseContext(this->m_hWnd,hIMC);	
				return;	
			}

		case TRADITIONAL_CHINESE:
			{				
				ImmSetCompositionFont(hIMC, &lf);
				ImmReleaseContext(this->m_hWnd,hIMC);	
				return;	
			}
		}
	}
}

void CCodeEditor::InitializeKeywords()
{
	const TransKeywords::KeywordMap &theMap = GetKeywordMap();
	const TransKeywords::KeywordMap &theMbr = GetMemberMap();

	for(TransKeywords::KeywordMapCItr itr=theMap.begin(); 
		itr != theMap.end(); ++itr)
	{
		switch(itr->second)
		{
		case keyword_fmt_clear:
		case keyword_fmt_precision:
		case keyword_fmt_scientific:
		case keyword_fmt_fixed:
		case keyword_fmt_showpoint:
		case keyword_fmt_hidepoint:
		case keyword_fmt_showthousand:
		case keyword_fmt_hidethousand:
			this->SetWordColor(itr->first.c_str(), RGB(0xFF, 0, 0)); break;
		default:
			this->SetWordColor(itr->first.c_str(), RGB(0, 0, 0xFF)); break;
		}
	}

	for(TransKeywords::KeywordMapCItr itr=theMbr.begin(); 
		itr != theMbr.end(); ++itr)
	{
		this->SetWordColor(itr->first.c_str(), RGB(0, 0xFF, 0)); 
	}
}

void CCodeEditor::OnImeComposition(WPARAM, LPARAM lParam)
{
	HIMC hIMC = ImmGetContext(this->m_hWnd);
	if(hIMC == NULL) return;

	LONG ComSize; TCHAR ComStr[80]; 
	int buf_size = 80*sizeof(TCHAR);

	CDC *pDC = CBCGPEditCtrl::GetDC();
	CPoint pt = CBCGPEditCtrl::GetCaretPos();
	
	CFont *pOldFont = CBCGPEditCtrl::SelectFont(pDC);
		
	if (lParam & GCS_RESULTSTR)
		ComSize = ImmGetCompositionStringW(hIMC, GCS_RESULTSTR, (LPVOID)ComStr, buf_size);
	else if (lParam & GCS_COMPSTR)
		ComSize = ImmGetCompositionStringW(hIMC, GCS_COMPSTR, (LPVOID)ComStr, buf_size);
	else if (lParam & GCS_COMPREADSTR)
		ComSize = ImmGetCompositionStringW(hIMC, GCS_COMPREADSTR, (LPVOID)ComStr, buf_size);
	else if (lParam & GCS_RESULTREADSTR)
		ComSize = ImmGetCompositionStringW(hIMC, GCS_RESULTREADSTR, (LPVOID)ComStr, buf_size);
	else ComSize = 0;

	if(ComSize>0) pDC->TextOutW(pt.x, pt.y, ComStr, 1);
		
	pDC->SelectObject(pOldFont);
	ImmReleaseContext(this->m_hWnd, hIMC);
}

void CCodeEditor::EndIMEComposition() 
{
	if(this->m_nLanguage==KOREAN)
	{
		::DestroyCaret(); 
		CreateSolidCaret(m_CaretSize.cx, m_CaretSize.cy); 
		ShowCaret();
	}
}

void CCodeEditor::SetLanguage(HKL hKeyboardLayout)
{
		// Check keyboard layout
		if (hKeyboardLayout == 0)
			hKeyboardLayout = GetKeyboardLayout(0);

		switch (LOWORD(hKeyboardLayout))
		{
		case LID_TRADITIONAL_CHINESE: // Traditional Chinese 
				m_nLanguage = TRADITIONAL_CHINESE; break;

		case LID_JAPANESE: // Japanese
				m_nLanguage = JAPANESE; break;

		case LID_KOREAN: // Korean
				m_nLanguage = KOREAN; break;

		case LID_SIMPLIFIED_CHINESE: // Simplified Chinese
				m_nLanguage = SIMPLIFIED_CHINESE; break;

		default:
			{
				m_nCodePage = GetACP();
				
				switch(m_nCodePage)
				{
					case 950: m_nLanguage = TRADITIONAL_CHINESE; break;
					case 932: m_nLanguage = JAPANESE; break;
					case 949: m_nLanguage = KOREAN; break;
					case 936: m_nLanguage = SIMPLIFIED_CHINESE; break;
					default:  m_nLanguage = DEFAULT; 
				}			
			}
		}
		
		// Set code page and charset
		if (m_nLanguage) m_nCodePage = nCodePage[m_nLanguage];
		else m_nCodePage = GetACP();

		m_nCharSet = nCharSet[m_nLanguage];		

		// Get IME property
		m_dwProperty = ImmGetProperty(hKeyboardLayout, IGP_PROPERTY);
}


LRESULT CCodeEditor::WindowProc(UINT message, WPARAM wParam, LPARAM lParam)
{
	// TODO: Add your specialized code here and/or call the base class
	
	switch (message)
	{
		case WM_INPUTLANGCHANGE: // SetLanguage
			SetLanguage((HKL)lParam);  break;
		
		case WM_IME_STARTCOMPOSITION: // Start composition
			StartIMEComposition(); 
			if(m_nLanguage == KOREAN) return TRUE;
			else break; 
		
		case WM_IME_COMPOSITION: // Composition in progress
			if(m_nLanguage == KOREAN) 
			{
				OnImeComposition(wParam, lParam); 
				return DefWindowProc(message, wParam, lParam);
			}
			else break;
			
		case WM_IME_ENDCOMPOSITION: // Composition completed
			EndIMEComposition(); break;

		case WM_IME_SETCONTEXT:  SyncIMEFont(); break;
	}		

	return CBCGPEditCtrl::WindowProc(message, wParam, lParam);
}

void CCodeEditor::AddBlockType(LPCTSTR lpszOpen, LPCTSTR lpszClose, 
							   LPCTSTR lpszReplace, BOOL bNested, BOOL bIgnore, CStringList* pKeywordsList)
{
	this->m_pOutlineParser->AddBlockType(lpszOpen, lpszClose, lpszReplace, bNested, bIgnore, pKeywordsList);
}

/////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////

void CCodeEditor::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	if (IsEnableBreakpoints() &&
		point.x < m_nLeftMarginWidth)
	{
		ToggleBreakpoint();
	}
	
	CBCGPEditCtrl::OnLButtonDblClk(nFlags, point);
}

void CCodeEditor::OnGetCharColor (TCHAR ch, int nOffset, COLORREF& clrText, COLORREF& clrBk)
{
	if (m_bCheckColorTags)
	{
		
		TCHAR chOpen = _T ('<');
		TCHAR chClose = _T ('>');
		
		if (ch == chOpen || ch == chClose || ch == _T ('/'))
		{
			clrText = RGB (0, 0, 255);
		}
		else 
		{
			COLORREF clrDefaultBack = GetDefaultBackColor ();
			COLORREF clrDefaultTxt = GetDefaultTextColor ();
			int nBlockStart, nBlockEnd;
			if (!IsInBlock (nOffset, chOpen, chClose, nBlockStart, nBlockEnd))
			{
				clrText = clrDefaultTxt;
				clrBk = clrDefaultBack;
			}
			else if (GetCharAt (nBlockStart + 1) == _T ('%') && 
					 GetCharAt (nBlockEnd - 1) == _T ('%'))
			{

			}
			else if (clrText == clrDefaultTxt)
			{
				if (ch == _T ('='))
				{
					clrText = RGB (0, 0, 255);
				}
				else
				{
					clrText = RGB (255, 0, 0);
				}
			}
		}
	}
}

void CCodeEditor::OnDrawMarker (CDC* pDC, CRect rectMarker, const CBCGPEditMarker* pMarker)
{
	if (pMarker->m_dwMarkerType & g_dwBreakPointType)
	{
		CBrush br (RGB (127, 0, 0));
		CPen pen (PS_SOLID, 1, RGB (127, 0, 0));
		CBrush* pBrOld = pDC->SelectObject (&br);
		CPen* pOldPen = pDC->SelectObject (&pen);

		rectMarker.DeflateRect (1, 1);
		int nD = min (rectMarker.Width (), rectMarker.Height());
		int nR = nD / 2;
		CPoint ptCenter = rectMarker.CenterPoint ();
		rectMarker.SetRect(	ptCenter.x - nR, ptCenter.y - nR,
							ptCenter.x + nR, ptCenter.y + nR);
		pDC->Ellipse (rectMarker);
		pDC->SelectObject (pBrOld);
		pDC->SelectObject (pOldPen);
	}
	else
	{
		CBCGPEditCtrl::OnDrawMarker (pDC, rectMarker, pMarker);
	}
}

BOOL CCodeEditor::OnGetTipText (CString& strTipString)
{
	if (!IsIntelliSenseEnabled())
	{
		return FALSE;
	}
	
	CString strTipContent;
	BOOL bIsHyperlink = m_bEnableHyperlinkSupport && GetHyperlinkToolTip (strTipContent);
	
	if (bIsHyperlink && strTipString == strTipContent)
	{
		return TRUE; 
	}
	else if (m_mTipString.Lookup(strTipString,strTipString))
	{
		return TRUE; 
	}

	return FALSE;
}

BOOL CCodeEditor::EnableBreakpoints(BOOL bFl /* = TRUE */)
{
	const BOOL bEnableBreakpoints = m_bEnableBreakpoints;
	m_bEnableBreakpoints = bFl;

	return bEnableBreakpoints;
}

BOOL CCodeEditor::ToggleBreakpoint(int nCurrRow /* = -1 */)
{
	if (nCurrRow == -1)
	{
		nCurrRow = GetCurRow();
	}

	BOOL bMarkerSet = ToggleMarker (nCurrRow, g_dwBreakPointType, NULL, FALSE);

	if (bMarkerSet)
	{
		SetLineColorMarker (nCurrRow, RGB (255, 255, 255), 
			                RGB (127, 0, 0), TRUE, 0, g_dwColorBreakPointType);
	}
	else
	{
		DeleteMarker (nCurrRow, g_dwColorBreakPointType);
	}

	return bMarkerSet;
}

void CCodeEditor::RemoveAllBreakpoints()
{
	DeleteAllMarkers(g_dwBreakPointType | g_dwColorBreakPointType);
}

/////////////////////////////////////////////////////////////////////////////
// IntelliSense Support

void CCodeEditor::PrepareIntelliSense()
{
	int ind = 0;
	for (ind = sizeof(g_aIntelliStringArray) / (sizeof(IntelliStringArray));
		 ind-- != 0;
		 m_mIntelliString.SetAt(g_aIntelliStringArray[ind].lpszName,g_aIntelliStringArray[ind].ptr));

	for (ind = sizeof(g_aTipText) / sizeof(TipString);
		 ind-- != 0;
		 m_mTipString.SetAt(g_aTipText[ind].lpszTip,g_aTipText[ind].lpszContent));
}

BOOL CCodeEditor::IsIntelliSenceWord(const CString& strWord) const
{
	if (!IsIntelliSenseEnabled())
	{
		return FALSE;
	}
	
	StringIndex* pStringIndex;
	
	if (m_mIntelliString.Lookup(strWord, (void*&)pStringIndex))
	{
		return TRUE; 
	}

	return FALSE;
}

BOOL CCodeEditor::FillIntelliSenseList (CObList& lstIntelliSenseData,
											LPCTSTR lpszIntelliSense /* = NULL */) const
{
	if (lpszIntelliSense == NULL)
	{
		BOOL bRet;
		CString strIntelliSence;
		int nCurrOffset = m_nCurrOffset;
		
		if (!OnBeforeInvokeIntelliSense(m_strBuffer, nCurrOffset, strIntelliSence))
		{
			return FALSE;
		}

		bRet = FillIntelliSenseList(lstIntelliSenseData, strIntelliSence.GetBuffer(0));
		ASSERT(bRet);

		return bRet;
	}

	if (!IsIntelliSenseEnabled() ||
		!IsIntelliSenceWord(lpszIntelliSense))
	{
		return FALSE;
	}
	
	StringIndex* pStringIndex;

	if (!m_mIntelliString.Lookup(lpszIntelliSense, (void*&)pStringIndex))
	{
		return FALSE;
	}
	
	CBCGPIntelliSenseData* pData;
	lstIntelliSenseData.RemoveAll();
	
	for (int ind = 0; pStringIndex[ind].lpszName != NULL; ind++)
	{
		pData = new CBCGPIntelliSenseData;
		pData->m_strItemName = pStringIndex[ind].lpszName;
		pData->m_nImageListIndex = pStringIndex[ind].nIndex;
		lstIntelliSenseData.AddTail (pData);
	}

	return TRUE;
}

BOOL CCodeEditor::IntelliSenseCharUpdate(const CString& strBuffer, int nCurrOffset, TCHAR nChar, CString& strIntelliSense)
{
	if (!IsIntelliSenseEnabled())
	{
		return FALSE;
	}
	
	if (nChar == _T('.'))
	{
		if ((nCurrOffset > 0) && (nCurrOffset <= strBuffer.GetLength()) &&
			strBuffer.GetAt(nCurrOffset - 1) == nChar)
		{
			CString strWord;
			int nOffset = nCurrOffset;

			if (!OnBeforeInvokeIntelliSense(strBuffer, nOffset, strWord))
			{
				nOffset = strWord.GetLength();
				if ((nOffset > 1) && (strWord.GetAt(nOffset - 1) == nChar))
				{
					strWord.SetAt(nOffset - 1, _T('-'));
					strWord += _T('>');
					nOffset++;

					if (IsIntelliSenceWord(strWord))
					{
						strIntelliSense = _T("->");
						return TRUE;
					}
				}
			}
		}
		else
		{
			ASSERT(FALSE);
		}
	}

	return FALSE;
}

BOOL CCodeEditor::OnBeforeInvokeIntelliSense (const CString& strBuffer, int& nCurrOffset, CString& strIntelliSence) const
{
	if (!IsIntelliSenseEnabled())
	{
		return FALSE;
	}
	
	strIntelliSence.Empty();

	int nOffset = nCurrOffset;

	if (nOffset > strBuffer.GetLength())
	{
		nOffset = strBuffer.GetLength();
	}

	if (nOffset >= 0)
	{
		for (--nOffset;
			 nOffset >= 0 &&
			 m_strIntelliSenseChars.Find(strBuffer.GetAt(nOffset)) == -1 &&
			 (m_strNonSelectableChars.Find(strBuffer.GetAt(nOffset)) >= 0 ||
			  m_strWordDelimeters.Find(strBuffer.GetAt(nOffset)) == -1);
			 nOffset--);

		if (nOffset >= 0 &&
			FillIntelliSenceWord(strBuffer,nOffset,strIntelliSence) &&
			IsIntelliSenceWord(strIntelliSence))
		{
			nCurrOffset = nOffset + 1;
			return TRUE;
		}
	}
	else
	{
		ASSERT(FALSE);
	}

	return FALSE;
}

BOOL CCodeEditor::FillIntelliSenceWord(const CString& strBuffer, int nOffset, CString& strIntelliSence) const
{
	CString strISWord, strWordSuffix;

	if (!CheckIntelliMark(strBuffer,nOffset,strWordSuffix) || nOffset >= 0)
	{
		do
		{
			int nStartOffset = -1,
				nEndOffset = -1;
			
			FindWordStartFinish (nOffset, strBuffer, nStartOffset, nEndOffset);
			
			if ((nStartOffset < nEndOffset) && (nStartOffset >= 0))
			{
				const CString& strWord = strBuffer.Mid(nOffset = nStartOffset, nEndOffset - nStartOffset);
				strISWord = strWord + strWordSuffix + strISWord;
			}
			else
			{
				strISWord = strWordSuffix + strISWord;
				break;
			}

			strWordSuffix.Empty();
		}
		while (GetNextPos(strBuffer, m_strNonSelectableChars, nOffset, FALSE) >= 0 &&
			   CheckIntelliMark(strBuffer,nOffset,strWordSuffix) &&
			   nOffset >= 0);
	}

	if ((strISWord = strWordSuffix + strISWord).IsEmpty())
	{
		return FALSE;
	}

	strIntelliSence = strISWord;
	return TRUE;
}

void CCodeEditor::ReleaseIntelliSenseList(CObList& lstIntelliSenseData)
{
	for (POSITION pos = lstIntelliSenseData.GetHeadPosition();
		 pos != NULL;
		 delete lstIntelliSenseData.GetNext(pos));

	lstIntelliSenseData.RemoveAll();
}

BOOL CCodeEditor::CheckIntelliMark(const CString& strBuffer, int& nOffset, CString& strWordSuffix) const
{
	BOOL bIntelliMark = (strBuffer.GetAt (nOffset) == _T ('.'));

	if (bIntelliMark)
	{
		strWordSuffix = _T('.');
	}
	else if (strBuffer.GetAt (nOffset) == _T ('>'))
	{
		int nTempOffset = nOffset;
		
		if (GetNextPos(strBuffer, m_strNonSelectableChars, nTempOffset, FALSE) >= 0)
		{
			if (strBuffer.GetAt (nTempOffset) == _T ('-'))
			{
				strWordSuffix = _T("->");
				nOffset = nTempOffset;
				bIntelliMark = TRUE;
			}
		}
	}
	else if (strBuffer.GetAt (nOffset) == _T (':'))
	{
		int nTempOffset = nOffset;
		
		if (GetNextPos(strBuffer, m_strNonSelectableChars, nTempOffset, FALSE) >= 0)
		{
			if (strBuffer.GetAt (nTempOffset) == _T (':'))
			{
				strWordSuffix = _T("::");
				nOffset = nTempOffset;
				bIntelliMark = TRUE;
			}
		}
	}

	if (bIntelliMark)
	{
		nOffset--;
	}

	return bIntelliMark;
}

int CCodeEditor::GetNextPos(const CString& strBuffer, const CString& strSkipChars, int& nPos, BOOL bForward)
{
	if (bForward)
	{
		for (int nLen = strBuffer.GetLength();
			 nPos < nLen &&
			 strSkipChars.Find(strBuffer.GetAt(nPos)) >= 0;
			 nPos++);
	}
	else
	{
		for (--nPos;
			 nPos >= 0 &&
			 strSkipChars.Find(strBuffer.GetAt(nPos)) >= 0;
			 nPos--);
	}

	return nPos;
}

BOOL CCodeEditor::CopyAsHTML ()
{
	CString strBuffer;
	ExportToHTML (strBuffer);
	
	return CopyTextToClipboard (strBuffer, strBuffer.GetLength ());
}

void CCodeEditor::ResetDefaultFont()
{
	HFONT hFont = (HFONT)::GetStockObject(DEFAULT_GUI_FONT);
	CFont* pFont = CFont::FromHandle(hFont);
	
	LOGFONT lf; pFont->GetLogFont(&lf);
	m_Font.CreateFontIndirect(&lf);
	this->SetFont(&m_Font);
	::DeleteObject(hFont);
}

int CCodeEditor::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CBCGPEditCtrl::OnCreate(lpCreateStruct) == -1)
		return -1;

	// Enabling tooltips, creating image lists, setting fonts 
	this->EnableToolTips();
	ResetDefaultFont();
	
	return 0;
}

void CCodeEditor::OnDraw(CDC* /*pDC*/)
{
	// TODO: Add your specialized code here and/or call the base class
	//this->RedrawText(0);
}

BOOL CCodeEditor::OnMouseWheel(UINT nFlags, short zDelta, CPoint pt)
{
	// TODO: Add your message handler code here and/or call default

	BOOL rlt = CBCGPEditCtrl::OnMouseWheel(nFlags, zDelta, pt);
	this->RedrawText(0, this->GetLineCount());
	return rlt;
}
