#pragma once

// CCodeEditor

// Define language ID
#define LID_TRADITIONAL_CHINESE	0x0404
#define LID_JAPANESE			0x0411
#define LID_KOREAN				0x0412
#define LID_SIMPLIFIED_CHINESE	0x0804

static const DWORD g_dwBreakPointType = g_dwBCGPEdit_FirstUserDefinedMarker;
static const DWORD g_dwColorBreakPointType = g_dwBCGPEdit_FirstUserDefinedMarker << 1;
static const DWORD g_dwColorHeaderType = g_dwColorBreakPointType << 1;


// Declare enum type Languages
enum LANGFLAG           
{
	DEFAULT,				
	TRADITIONAL_CHINESE,	
	JAPANESE,
	KOREAN,
	SIMPLIFIED_CHINESE

}; 

extern LANGFLAG LangFlag;

// Define code pages 
extern int nCodePage[5];

// Define charset
extern BYTE nCharSet[5];

// Define Default font
extern char szDefaultFontName[5][19];

extern TCHAR szEnglishFontName[5][11];

class  CCodeEditor : public CBCGPEditCtrl
{
	DECLARE_DYNCREATE(CCodeEditor)
	
public:
	CCodeEditor();
	virtual ~CCodeEditor();
	//virtual void Serialize(CArchive& ar);
	void InitializeKeywords();
	void AddBlockType(LPCTSTR lpszOpen,LPCTSTR lpszClose,LPCTSTR lpszReplace=NULL,BOOL bNested=TRUE,BOOL bIgnore=FALSE,CStringList*pKeywordsList=NULL);

protected:
	CFont	m_Font;
	CSize   m_CaretSize;				// English Caret Size
	DWORD	m_dwProperty;				// Property of IME
	UINT	m_nCodePage;				// Code page
	BYTE	m_nCharSet;					// Character set
	int		m_nLanguage;				// Internal index of language

	void StartIMEComposition();
	void OnImeComposition(WPARAM, LPARAM lParam);
	void EndIMEComposition();
	void SyncIMEFont();
	void SetLanguage(HKL hKeyboardLayout);
	virtual LRESULT WindowProc(UINT message, WPARAM wParam, LPARAM lParam);

public:

	BOOL	m_bCheckColorTags;

public:

	BOOL ToggleBreakpoint(int nCurrRow = -1);
	void RemoveAllBreakpoints();
	BOOL IsEnableBreakpoints() const {  return m_bEnableBreakpoints; }
	BOOL EnableBreakpoints(BOOL bFl);
	BOOL CopyAsHTML ();

	void ResetDefaultFont();


	virtual const CString& GetText() const;

protected:
	DECLARE_MESSAGE_MAP()

public:

	virtual void OnGetCharColor (TCHAR ch, int nOffset, COLORREF& clrText, COLORREF& clrBk);

	// Marker Support
	virtual void OnDrawMarker(CDC* pDC, CRect rectMarker, const CBCGPEditMarker* pMarker);

	// Tooltip Support
	virtual BOOL OnGetTipText (CString& strTipString);

	virtual BOOL CanUpdateMarker (CBCGPEditMarker* pMarker) const
	{
		if (pMarker->m_dwMarkerType & g_dwColorHeaderType)
		{
			return FALSE;
		}

		return TRUE;
	}

	virtual BOOL CanRemoveMarker (CBCGPEditMarker* pMarker) const  
	{
		if (pMarker->m_dwMarkerType & g_dwColorHeaderType)
		{
			return FALSE;
		}

		return TRUE;
	}

	// IntelliSense Support
	virtual BOOL FillIntelliSenseList(CObList& lstIntelliSenseData,	LPCTSTR lpszIntelliSense = NULL) const;
	virtual void ReleaseIntelliSenseList(CObList& lstIntelliSenseData);

protected:
	// IntelliSense Support
	virtual BOOL OnBeforeInvokeIntelliSense (const CString& strBuffer, int& nCurrOffset, CString& strIntelliSence) const;
	virtual BOOL IntelliSenseCharUpdate(const CString& strBuffer, int nCurrOffset, TCHAR nChar, CString& strIntelliSense);
	
// Implementation

	// Generated message map functions
protected:
	//{{AFX_MSG(CCustomEditCtrl)
	afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
	//}}AFX_MSG

private:
	BOOL	m_bEnableBreakpoints;
	CMapStringToPtr m_mIntelliString;
	CMapStringToString m_mTipString;

	void PrepareIntelliSense();
	BOOL IsIntelliSenceWord(const CString& strWord) const;
	BOOL FillIntelliSenceWord(const CString& strBuffer, int nOffset, CString& strIntelliSence) const;
	BOOL CheckIntelliMark(const CString& strBuffer, int& nOffset, CString& strWordSuffix) const;
	inline static int GetNextPos(const CString& strBuffer, const CString& strSkipChars, int& nPos, BOOL bForward);
public:
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
protected:
	virtual void OnDraw(CDC* /*pDC*/);
public:
	afx_msg BOOL OnMouseWheel(UINT nFlags, short zDelta, CPoint pt);
};


