// MainFrame.h : interface of the CMainFrame class
//

#pragma once

#include "WorkspaceBar.h"
#include "WorkspaceBar2.h"
#include "OutputBar.h"
#include "ExplorerBar.h"
#include "SystemTray.h"

class CMainFrame : public CBCGPMDIFrameWnd
{
	DECLARE_DYNAMIC(CMainFrame)
public:
	CMainFrame();

	int CallCount;

	CSystemTray m_TrayIcon;
	BOOL		m_Option1Enabled;

	CBCGPRibbonCategory *pCategoryYal;
	CBCGPRibbonCategory *pCategoryWeb;
	CBCGPRibbonComboBox *pAddressBox;
	CBCGPRibbonEdit		*pNewAddress;

	CBCGPAnimCtrl	m_wndAnimate;
	CBitmap			m_bmpBack;
	CBitmap			m_bmpStatus;
	CImageList		m_imlStatusAnimation;
	CExplorerBar	m_wndExplorerBar;

	CMenu			m_menuFavotites;
	UINT			mFontSize;
	
// Attributes
public:
	BOOL IsPrintPreviewMode ();
	void ShowProgress (BOOL bShow = TRUE);
	BOOL CreateStatusBar();
	void StartAnimation();
	void SetAddress(LPCTSTR lpszUrl);
	void SetProgress (long nCurr, long nTotal);
	
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	virtual BOOL LoadFrame(UINT nIDResource, 
		DWORD dwDefaultStyle = WS_OVERLAPPEDWINDOW | FWS_ADDTOTITLE,
		CWnd* pParentWnd = NULL, CCreateContext* pContext = NULL);

	virtual BOOL OnShowPopupMenu (CBCGPPopupMenu* pMenuPopup);

	BOOL IsFavoritesMenu (const CBCGPToolbarMenuButton* pMenuButton);

	virtual BOOL OnDrawMenuImage (CDC* pDC,
									const CBCGPToolbarMenuButton* pMenuButton, 
									const CRect& rectImage);

	virtual BOOL OnMenuButtonToolHitTest (CBCGPToolbarButton* pButton, TOOLINFO* pTI);

	virtual ~CMainFrame();

#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

public:  // control bar embedded members
	CBCGPRibbonFontComboBox	*pFontCombo;
	CBCGPRibbonComboBox		*pFontSizeCombo;

	CBCGPRibbonFontComboBox	*pFontOutputCombo;
	CBCGPRibbonComboBox		*pFontOutputSizeCombo;
	
	//CBCGPRibbonProgressBar	*pProgressBar;
	CBCGPRibbonButton		*pBtnRun;
	

	CBCGPRibbonBar			m_wndRibbonBar;
	CBCGPRibbonMainButton	m_MainButton;	
	
	CBCGPStatusBar			m_wndStatusBar;

	//CWorkSpaceBar			m_wndWorkSpace;
	//CWorkSpaceBar2			m_wndWorkSpace2;
	CBCGPToolBarImages		m_PanelImages;
	CBCGPToolBarImages		m_LinkImages;
	CBCGPToolBarImages		m_WebImages;
	COutputBar				m_wndOutputBar;

	UINT		m_nAppLook;
	int			m_nProgressValue;
	BOOL		m_bInfiniteProgressMode;

// Generated message map functions
protected:
	afx_msg void OnNewAddress();
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnWindowManager();
	afx_msg void OnUpdateWindowManager(CCmdUI* pCmdUI);
	afx_msg void OnAppLook(UINT id);
	afx_msg void OnUpdateAppLook(CCmdUI* pCmdUI);
	afx_msg void OnMdiMoveToNextGroup();
	afx_msg void OnMdiMoveToPrevGroup();
	afx_msg void OnMdiNewHorzTabGroup();
	afx_msg void OnMdiNewVertGroup();
	afx_msg void OnMdiCancel();
	afx_msg void OnToolsOptions();
	afx_msg void OnViewWorkspace();
	afx_msg void OnUpdateViewWorkspace(CCmdUI* pCmdUI);
	afx_msg void OnViewWorkspace2();
	afx_msg void OnUpdateViewWorkspace2(CCmdUI* pCmdUI);
	afx_msg void OnViewOutput();
	afx_msg void OnUpdateViewOutput(CCmdUI* pCmdUI);
	afx_msg LRESULT OnHelpCustomizeToolbars(WPARAM wp, LPARAM lp);
	DECLARE_MESSAGE_MAP()

	virtual BOOL OnShowMDITabContextMenu (CPoint point, DWORD dwAllowedItems, BOOL bDrop);
	virtual CBCGPMDIChildWnd* CreateDocumentWindow (LPCTSTR lpcszDocName, CObject* /*pObj*/);
	void InitRibbon ();

	afx_msg void OnViewExplorerbar();
	afx_msg void OnUpdateViewExplorerbar(CCmdUI* pCmdUI);

public:
	afx_msg void OnFavorite(UINT nID);
	afx_msg void OnHistory(UINT nID);
	afx_msg void OnEditClearoutput();
	afx_msg void OnUpdateEditClearoutput(CCmdUI *pCmdUI);
	afx_msg void OnTimer(UINT_PTR nIDEvent);
	afx_msg void OnViewLinks();
	afx_msg void OnUpdateViewLinks(CCmdUI *pCmdUI);
	afx_msg void OnClose();

	afx_msg void OnPopupOption1();
	afx_msg void OnUpdatePopupOption1(CCmdUI* pCmdUI);
	afx_msg void OnPopupOption2();
	void ShowOptions (int nPage);
	afx_msg void OnViewNewWindow();
	afx_msg void OnTutorial();
	afx_msg void OnYalHelp();
	afx_msg void OnYalhome();
	void OnWindowCode(void);
	afx_msg void OnFileSaveAll();
	afx_msg void OnUpdateFileSaveAll(CCmdUI *pCmdUI);
public:
	afx_msg void OnCommandReload();
public:
	afx_msg void OnBestInfoPdr();
public:
	afx_msg void OnLinkDocdoc();
public:
	afx_msg void OnLinkIop();
public:
	afx_msg void OnLinkSciam();
public:
	afx_msg void OnLinkSciwolrd();
};
