// YalView.cpp : implementation of the CYalView class
//

#include "stdafx.h"
#include "Yal.h"
#include "MainFrame.h"

#include "YalFrame.h"

#include "YalDoc.h"
#include "YalView.h"
#include "CodeEditor.h"
#include "StartupDlg.h"

static const CString g_strEOL = _T ("\n");
static const TCHAR g_chEOL = _T ('\n');
static const CString g_strEOLExport = _T ("\r\n");

#include "resource.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


#define WPD_CHARFORMAT CHARFORMAT2
#define WPD_PARAFORMAT PARAFORMAT2

struct CCharFormat : public WPD_CHARFORMAT
{
	CCharFormat() {cbSize = sizeof(WPD_CHARFORMAT);}
	BOOL operator==(CCharFormat& cf);
};

int GetTwipSize (CString str)
{
	LPCTSTR lpszText = str;

	while (*lpszText == _T(' ') || *lpszText == _T('\t'))
		lpszText++;

	if (lpszText[0] == NULL)
		return -1; // no text in control

	double d = _tcstod(lpszText, (LPTSTR*)&lpszText);
	while (*lpszText == _T(' ') || *lpszText == _T('\t'))
		lpszText++;

	if (*lpszText != NULL)
		return -2;   // not terminated properly

	return (d<0.) ? 0 : (int)(d*20.);
}


static UINT WM_FINDREPLACE = ::RegisterWindowMessage(FINDMSGSTRING);

// CYalView

IMPLEMENT_DYNCREATE(CYalView, CCodeEditor)

BEGIN_MESSAGE_MAP(CYalView, CCodeEditor)
	
	ON_WM_KILLFOCUS()
	ON_WM_SETFOCUS()
	ON_WM_CREATE()
	ON_WM_ERASEBKGND()
	ON_WM_SETFOCUS()
	ON_WM_DESTROY()
	
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CCodeEditor::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, OnFilePrintPreview)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CCodeEditor::OnFilePrint)
	ON_UPDATE_COMMAND_UI(ID_FILE_PRINT_PREVIEW, &CYalView::OnUpdateFilePrintPreview)
	ON_UPDATE_COMMAND_UI(ID_FILE_PRINT, &CYalView::OnUpdateFilePrint)
	//ON_UPDATE_COMMAND_UI(ID_FILE_SAVE, &CYalView::OnUpdateFileSave)

	ON_MESSAGE(WM_MOVECURSOR, &CYalView::OnMoveCursor)
	ON_COMMAND(ID_EDIT_CUT, OnEditCut)
	ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
	ON_COMMAND(ID_EDIT_PASTE, OnEditPaste)
	ON_UPDATE_COMMAND_UI(ID_EDIT_CUT, OnUpdateEditCut)
	ON_UPDATE_COMMAND_UI(ID_EDIT_COPY, OnUpdateEditCopy)
	ON_UPDATE_COMMAND_UI(ID_EDIT_PASTE, OnUpdateEditPaste)
	ON_COMMAND(ID_EDIT_SELECT_ALL, OnEditSelectAll)
	ON_COMMAND(ID_EDIT_FIND, OnEditFind)
	ON_COMMAND(ID_EDIT_REPLACE, OnEditReplace)
	
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, OnFilePrintPreview)
	ON_UPDATE_COMMAND_UI(ID_FILE_PRINT_PREVIEW, OnUpdateFilePrintPreview)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, OnFilePrint)
	ON_UPDATE_COMMAND_UI(ID_FILE_PRINT_DIRECT, OnUpdateFilePrint)
	
	// Undo/redo
	ON_COMMAND(ID_EDIT_REDO, OnEditRedo)
	ON_UPDATE_COMMAND_UI(ID_EDIT_REDO, OnUpdateEditRedo)
	ON_COMMAND(ID_EDIT_UNDO, OnEditUndo)
	ON_UPDATE_COMMAND_UI(ID_EDIT_UNDO, OnUpdateEditUndo)
	ON_REGISTERED_MESSAGE(WM_FINDREPLACE, OnFindReplace)
	ON_REGISTERED_MESSAGE(BCGM_ON_EDITCHANGE, OnEditChange)

	ON_COMMAND(ID_COMMAND_RUN, OnCommandRun)
	ON_UPDATE_COMMAND_UI(ID_COMMAND_RUN, OnUpdateCommandRun)
	ON_UPDATE_COMMAND_UI(ID_EDIT_FIND, &CYalView::OnUpdateEditFind)
	ON_UPDATE_COMMAND_UI(ID_EDIT_SELECT_ALL, &CYalView::OnUpdateEditSelectAll)
	ON_UPDATE_COMMAND_UI(ID_EDIT_REPLACE, &CYalView::OnUpdateEditReplace)
	ON_COMMAND(ID_WINDOW_NEW, &CYalView::OnWindowNew)

	ON_COMMAND(ID_EDIT_TOGGLEBOOKMARK, OnEditTogglebookmark)
	ON_COMMAND(ID_EDIT_NEXTBOOKMARK, OnEditNextbookmark)
	ON_COMMAND(ID_EDIT_PREVIOUSBOOKMARK, OnEditPreviousbookmark)
	ON_COMMAND(ID_EDIT_TOGGLEBREAKPOINT, OnEditTogglebreakpoint)
	ON_COMMAND(ID_EDIT_REMOVE_ALL_BREAKPOINTS, OnEditRemoveAllBreakpoints)

	ON_COMMAND(ID_FONT_FONT, OnFontname)
	ON_COMMAND(ID_FONT_FONTSIZE, OnFontsize)
		
END_MESSAGE_MAP()

CEditFindDlg::~CEditFindDlg ()
{
	if (m_pParent != NULL)
	{
		ASSERT_VALID (m_pParent);

		m_pParent->m_pFindDlg = NULL;
		AfxGetMainWnd()->ModifyStyle(WS_DISABLED,0);
		m_pParent->SetFocus();
	}
}

// CYalView construction/destruction

CYalView::CYalView(): 
	CCodeEditor(), 
		m_pFindDlg(NULL), m_pThread(NULL), m_bJustCreated(true) { }

CYalView::~CYalView() { }

int CYalView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CCodeEditor::OnCreate(lpCreateStruct) == -1)
		return -1;

	else return 0;
}

/////////////////////////////////////////////////////////////////////////////
// CBCGPEditView

DWORD		CYalView::m_dwFindMask = FR_DOWN;
CString		CYalView::m_strFindText;
CString		CYalView::m_strReplaceText;
BOOL		CYalView::m_bUpdateFindString = TRUE;


BOOL CYalView::OnEraseBkgnd(CDC* /*pDC*/) 
{
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CBCGPEditView printing

BOOL CYalView::OnPreparePrinting(CPrintInfo* pInfo)
{
	return DoPreparePrinting(pInfo);
}

void CYalView::OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo)
{
	CCodeEditor::OnBeginPrinting(pDC, pInfo);
}

void CYalView::OnAfterTextChanged(int /*nOffsetFrom*/ , const CString& /* strText*/, BOOL /* bInserted */)
{
	if(m_bJustCreated) 
	{
		m_bJustCreated = false; return;
	}

	GetDocument()->SetModifiedFlag();
}

void CYalView::OnEditCut() { CCodeEditor::Cut(); }


void CYalView::OnEditCopy() { CCodeEditor::Copy(); }


void CYalView::OnEditPaste() { CCodeEditor::Paste(); }


void CYalView::OnEditSelectAll() { MakeSelection(CBCGPEditCtrl::ST_ALL_TEXT); }


void CYalView::OnUpdateEditCut(CCmdUI* pCmdUI) { pCmdUI->Enable (IsCutEnabled());
}

void CYalView::OnUpdateEditCopy(CCmdUI* pCmdUI) { pCmdUI->Enable (IsCopyEnabled()); }

void CYalView::OnUpdateEditPaste(CCmdUI* pCmdUI) { pCmdUI->Enable (IsPasteEnabled()); }


void CYalView::OnDestroy() 
{
	if(this->m_pThread && 
		theApp.m_pThread == this->m_pThread)
	{
		SetBreak();
	}

	if (m_pFindDlg != NULL)
	{
		m_pFindDlg->PostMessage(WM_CLOSE);
	}

	CCodeEditor::OnDestroy();
}

void CYalView::OnEditRedo() { CCodeEditor::OnRedo(); }
	
void CYalView::OnUpdateEditRedo(CCmdUI* pCmdUI) { pCmdUI->Enable (CanRedo()); }

void CYalView::OnEditUndo() { CCodeEditor::OnUndo();	}

void CYalView::OnUpdateEditUndo(CCmdUI* pCmdUI) { pCmdUI->Enable (CanUndo()); }

void CYalView::OnEditFind() 
{
	if (m_pFindDlg != NULL)
	{
		m_pFindDlg->SetFocus ();
		return;
	}

	m_pFindDlg = new CEditFindDlg;
	m_pFindDlg->m_pParent = this;
	
	OnPrepareFindString (m_strFindText);

	if (m_pFindDlg->Create (TRUE, m_strFindText, NULL, m_dwFindMask, this))
	{
		AfxGetMainWnd()->ModifyStyle(0,WS_DISABLED);
	}
	else
	{
		delete m_pFindDlg;
	}
}


void CYalView::OnEditReplace()
{
	if (m_pFindDlg != NULL)
	{
		m_pFindDlg->SetFocus (); return;
	}

	m_pFindDlg = new CEditFindDlg;
	m_pFindDlg->m_pParent = this;
	
	OnPrepareFindString (m_strFindText);

	if (m_pFindDlg->Create (FALSE, m_strFindText, m_strReplaceText, m_dwFindMask, this))
	{
		AfxGetMainWnd()->ModifyStyle(0,WS_DISABLED);
	}
	else delete m_pFindDlg; 
}

LRESULT CYalView::OnFindReplace(WPARAM /*wParam*/, LPARAM /*lParam*/)
{
	if (m_pFindDlg == NULL)
	{
		if (!FindText(m_strFindText, m_dwFindMask))
		{
			OnTextNotFound(m_strFindText);
		}

		SetFocus();
	}
	else if (!m_pFindDlg->IsTerminating())
	{
		m_strFindText = m_pFindDlg->GetFindString();
		m_strReplaceText = m_pFindDlg->GetReplaceString();
		m_dwFindMask = m_pFindDlg->m_fr.Flags;

		if (!FindText(m_strFindText,m_dwFindMask))
		{
			OnTextNotFound(m_strFindText);
		}

		if (m_pFindDlg != NULL)
		{
			m_pFindDlg->SetFocus ();
		}
	}

	return 0;
}

BOOL CYalView::FindText (const CString& strFind, DWORD dwFindMask)
{
	if ((dwFindMask & FR_REPLACE) != 0)
	{
		return ReplaceText(strFind, m_strReplaceText,
							(dwFindMask & FR_DOWN) != 0,
							(dwFindMask & FR_MATCHCASE) != 0,
							(dwFindMask & FR_WHOLEWORD) != 0);
	}
	else if ((dwFindMask & FR_REPLACEALL) != 0)
	{
		return ReplaceAll(strFind, m_strReplaceText,
							(dwFindMask & FR_DOWN) != 0,
							(dwFindMask & FR_MATCHCASE) != 0,
							(dwFindMask & FR_WHOLEWORD) != 0) > 0;
	}
	else
	{
		return CCodeEditor::FindText(strFind,
							(dwFindMask & FR_DOWN) != 0,
							(dwFindMask & FR_MATCHCASE) != 0,
							(dwFindMask & FR_WHOLEWORD) != 0);
	}
}

LRESULT CYalView::OnEditChange(WPARAM, LPARAM)
{
	ASSERT_VALID(this);

	if (GetDocument() != NULL)
	{
		GetDocument()->SetModifiedFlag();
		ASSERT_VALID(this);
	}

	return 0;
}

void CYalView::OnPrepareFindString (CString& strFind)
{
	if (m_bUpdateFindString)
	{
		int i1, i2;
		GetSel (i1, i2);
		int iStartSel = min (i1, i2);
		int iEndSel = max (i1, i2);

		if (iStartSel < 0 || iEndSel <= iStartSel)
		{
			int iCurOffset = GetCurOffset ();
			GetWordFromOffset(iCurOffset, strFind);
		}
		else
		{
			const CString& strBuffer = GetText ();
			CString strSel = strBuffer.Mid (iStartSel, iEndSel - iStartSel);
			if (strSel.Find (_T('\n')) == -1)
			{
				strSel.Replace (_T('\t'), _T(' '));
				strFind = strSel;
			}
		}
	}
}

/////////////////////////////////////////////////////////

BOOL CYalView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	BOOL bPreCreated = CCodeEditor::PreCreateWindow(cs);
	cs.style &= ~(ES_AUTOHSCROLL|WS_HSCROLL);	// Enable word-wrapping

	return bPreCreated;
}


// CYalView printing

void CYalView::OnFilePrintPreview()
{  BCGPPrintPreview (this); }


// CYalView diagnostics

#ifdef _DEBUG
void CYalView::AssertValid() const
{
	CCodeEditor::AssertValid();
}

void CYalView::Dump(CDumpContext& dc) const
{
	CCodeEditor::Dump(dc);
}

CYalDoc* CYalView::GetDocument() const // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CYalDoc)));
	return (CYalDoc*)m_pDocument;
}
#endif //_DEBUG


// CYalView message handlers

void CYalView::OnUpdateFilePrintPreview(CCmdUI *pCmdUI)
{
	// TODO: Add your command update UI handler code here
	pCmdUI->Enable(this->GetText().GetLength()>0);
}

void CYalView::OnUpdateFilePrint(CCmdUI *pCmdUI)
{
	// TODO: Add your command update UI handler code here
	CYalDoc *pDoc = GetDocument();
	pCmdUI->Enable(pDoc->pActiveBuffer->GetLength()>2);
}

/*
void CYalView::OnUpdateFileSave(CCmdUI *pCmdUI)
{
	// TODO: Add your command update UI handler code here
	//pCmdUI->Enable(this->IsModified());
}
*/

void CYalView::OnViewLinenumber()
{
	// TODO: Add your command handler code here
		
	if(this->IsLineNumbersMarginVisible())
		this->SetLineNumbersMargin(FALSE);
	else this->SetLineNumbersMargin();
}

void CYalView::OnUpdateViewLinenumber(CCmdUI *pCmdUI)
{
	// TODO: Add your command update UI handler code here
	pCmdUI->SetCheck(this->IsLineNumbersMarginVisible());
}

void CYalView::EnableOutline()
{
	this->EnableOutlining();
	this->EnableAutoOutlining();
	this->UpdateAutoOutlining();
}

void CYalView::OnInitialUpdate()
{
	CCodeEditor::OnInitialUpdate();
	
	CMainFrame *pMainFrame = (CMainFrame*)::AfxGetMainWnd();
	CYalDoc* pDoc = GetDocument();

	CSize sizeTotal;
	// TODO: calculate the total size of this view
	sizeTotal.cx = sizeTotal.cy = 100;
	SetScrollSizes(MM_TEXT, sizeTotal);

	CString outline;
	outline = theApp.theLangPath + _T("\\outline.xml");
		
	LoadXMLSettings(outline);
	this->SetLineNumbersMargin();
	this->SetOutlineMargin();
		
	InitializeKeywords();

	this->AddBlockType(_T("<%"), _T("%>"), _T("<% ... %>"));
	
	this->EnableOutlining();
	this->EnableAutoOutlining();
	this->UpdateAutoOutlining();
	
	CEdit &theOutput = 
		((CMainFrame*)::AfxGetMainWnd())->m_wndOutputBar.m_wndOutput;
	
	CFont *theOutputFont = 
		CFont::FromHandle((HFONT)::GetStockObject(DEFAULT_GUI_FONT));
	theOutput.SetFont(theOutputFont);

	if(pDoc->GetPathName().GetLength() != 0)
	{
		m_bJustCreated = true;
		CCodeEditor::InsertText(*pDoc->pActiveBuffer);
		CCodeEditor::SetCaret(0);
		this->UpdateAutoOutlining();
	}
}

void CYalView::OnCommandRun()
{
	// TODO: Add your command handler code here
	//extern LONG IsExecuting();
	
	CMainFrame* pMainFrame = (CMainFrame*)::AfxGetMainWnd();
	if(!pMainFrame->m_wndOutputBar.IsVisible())
		pMainFrame->SendMessage(ID_VIEW_OUTPUT);

	if(IsExecuting())
	{
		CString msg; msg.LoadStringW(IDS_WANT_ABORT);

		CString abrt_msg, wait_msg;
		abrt_msg.LoadStringW(IDS_ABORTED);
		wait_msg.LoadStringW(IDS_STACK_UNWINDING);

		int rtn=MessageBox(msg, _T("Yet Another Language"), MB_YESNO|MB_ICONQUESTION);

		if(rtn == IDYES && IsExecuting())
		{
			//PutStringCString(abrt_msg); PutStringCString(wait_msg);
			SetBreak(); this->m_pThread = NULL;  
		}
	}
	else
	{
		// TODO: Add your command handler code here
		
		CYalDoc * pDoc = this->GetDocument();
						
		SetOutputEditorHandle(pMainFrame->m_wndOutputBar.m_wndOutput.GetSafeHwnd());
		SetInputCodeEditor(this->GetSafeHwnd()); 
		SetStatusBar(pMainFrame->m_wndStatusBar.GetSafeHwnd());
			
		if(pDoc->pActiveBuffer->GetLength()<1)
		{
			PutStringUINT(IDS_TEXTEMPTY); return;
		}
		
		theApp.SaveAllFiles();
		
		SetDocTitle(pDoc->GetPathName());

		CString running_msg;
		running_msg.LoadStringW(IDS_IS_RUNNING);

		pMainFrame->m_wndStatusBar.SetWindowTextW(running_msg);
		pMainFrame->m_bInfiniteProgressMode = TRUE;
		
		this->m_pThread = ::AfxBeginThread(ParseCode, (LPVOID)pDoc->pActiveBuffer);
		theApp.m_pThread = this->m_pThread;

		pMainFrame->ShowProgress(TRUE);
	}
}

void CYalView::OnCommandBreak()
{
	// TODO: Add your command handler code here
	SetBreak();
}

void CYalView::GoToErrorLine(int nStartSel, int nEndSel)
{
	this->SetSel2(nStartSel, nEndSel);
}
LRESULT CYalView::OnMoveCursor(WPARAM wParam, LPARAM lParam)
{
	GoToErrorLine((int)wParam, (int)lParam); return 0;
}

void CYalView::OnPrepareDC(CDC* pDC, CPrintInfo* pInfo)
{
	// TODO: Add your specialized code here and/or call the base class

	if (!pDC->IsPrinting ())
	{
		CCodeEditor::OnPrepareDC(pDC, pInfo);
	}
}

void CYalView::OnKillFocus(CWnd* pNewWnd)
{
	CCodeEditor::OnKillFocus(pNewWnd);
}

void CYalView::OnSetFocus(CWnd* pOldWnd)
{
	CMainFrame *pMainFrame = (CMainFrame*)::AfxGetMainWnd();
	if(!pMainFrame->m_wndRibbonBar.IsMinimized())
		pMainFrame->m_wndRibbonBar.SetActiveCategory(pMainFrame->pCategoryYal);

	EnableOutline();
	
	theApp.pYalView = this;
	CCodeEditor::OnSetFocus(pOldWnd);
	// TODO: Add your message handler code here
	CYalDoc *pDoc = GetDocument();  
	pDoc->pActiveYalView = this;
	
	if(*pDoc->pActiveBuffer != this->GetText())
	{
		SetSel(0, -1);  ReplaceSel(*pDoc->pActiveBuffer);
	}
	
	this->UpdateAutoOutlining();
	
	pDoc->pActiveBuffer = &this->m_strBuffer;
	extern std::wstring GetPath(const CString& fullpath);
	
	if(pDoc->GetPathName().GetLength()!=0) 
		::SetCurrentDirectory(GetPath(pDoc->GetPathName()).c_str());
}

void CYalView::OnEndPrintPreview(CDC* pDC, CPrintInfo* pInfo, POINT point, CPreviewView* pView)
{
	// TODO: Add your specialized code here and/or call the base class

	CCodeEditor::OnEndPrintPreview(pDC, pInfo, point, pView);
	((CYalFrame*)GetParentFrame())->AfterPrintPreview();
}

void CYalView::OnPrint(CDC* pDC, CPrintInfo* pInfo) 
{
	CCodeEditor::OnPrint(pDC, pInfo);
}


void CYalView::OnUpdateCommandRun(CCmdUI *pCmdUI)
{
	// TODO: Add your command update UI handler code here
	CYalDoc *pDoc = GetDocument();
	pCmdUI->Enable(pDoc->pActiveBuffer->GetLength()>2 
		|| theApp.m_pThread != NULL);
}

void CYalView::OnUpdateEditFind(CCmdUI *pCmdUI)
{
	// TODO: Add your command update UI handler code here
	CYalDoc *pDoc = GetDocument();
	pCmdUI->Enable(pDoc->pActiveBuffer->GetLength()>2);
}

void CYalView::OnUpdateEditSelectAll(CCmdUI *pCmdUI)
{
	// TODO: Add your command update UI handler code here
	CYalDoc *pDoc = GetDocument();
	pCmdUI->Enable(pDoc->pActiveBuffer->GetLength()>2);
}

void CYalView::OnUpdateEditReplace(CCmdUI *pCmdUI)
{
	// TODO: Add your command update UI handler code here
	CYalDoc *pDoc = GetDocument();
	pCmdUI->Enable(pDoc->pActiveBuffer->GetLength()>2);
}

void CYalView::OnWindowNew()
{
	// TODO: Add your command handler code here
	CAboutDlg aboutdlg; aboutdlg.DoModal();
}


//////////////////////////////////////////

void CYalView::OnEditTogglebookmark() 
{
	this->ToggleMarker(this->GetCurRow (), g_dwBCGPEdit_BookMark);
}

void CYalView::OnEditNextbookmark() 
{
	this->GoToNextMarker(g_dwBCGPEdit_BookMark, TRUE);
}

void CYalView::OnEditPreviousbookmark() 
{
	this->GoToNextMarker(g_dwBCGPEdit_BookMark,FALSE);
}

void CYalView::OnEditTogglebreakpoint() 
{
	CCodeEditor::ToggleBreakpoint();
}

void CYalView::OnEditRemoveAllBreakpoints() 
{
	this->RemoveAllBreakpoints();
}


void CYalView::OnFontname() 
{
	USES_CONVERSION;

	CBCGPRibbonBar* pRibbon = ((CMainFrame*) GetTopLevelFrame ())->GetRibbonBar ();
	ASSERT_VALID (pRibbon);

	CBCGPRibbonFontComboBox* pFontCombo = DYNAMIC_DOWNCAST (
		CBCGPRibbonFontComboBox, pRibbon->FindByID (ID_FONT_FONT));
	if (pFontCombo == NULL)
	{
		return;
	}

	CCharFormat cf;
	cf.szFaceName[0] = NULL;
	cf.dwMask = CFM_FACE | CFM_CHARSET;

	const CBCGPFontDesc* pDesc = pFontCombo->GetFontDesc ();
	ASSERT_VALID (pDesc);
	ASSERT(pDesc->m_strName.GetLength() < LF_FACESIZE);

	lstrcpyn(cf.szFaceName, pDesc->m_strName, LF_FACESIZE);

	cf.bCharSet = pDesc->m_nCharSet;
	cf.bPitchAndFamily = pDesc->m_nPitchAndFamily;

	LOGFONT lf;
	this->GetFont()->GetLogFont(&lf);
	::lstrcpy(lf.lfFaceName, cf.szFaceName);

	m_Font.DeleteObject();
	m_Font.CreateFontIndirect(&lf);
	CCodeEditor::SetFont(&m_Font);
	//CCodeEditor::SetPrinterFont(&m_PrintFont);
}

void CYalView::OnFontsize ()
{
	CBCGPRibbonBar* pRibbon = ((CMainFrame*) GetTopLevelFrame ())->GetRibbonBar ();
	ASSERT_VALID (pRibbon);

	CBCGPRibbonComboBox* pSizeCombo = DYNAMIC_DOWNCAST (
		CBCGPRibbonComboBox, pRibbon->FindByID (ID_FONT_FONTSIZE));
	
	if (pSizeCombo == NULL) return;

	std::wstringstream os;

	os<<pSizeCombo->GetItem(-1);
	int pointsize; os>>pointsize;
	
	LOGFONT logfont; m_Font.GetLogFont(&logfont);
	m_Font.DeleteObject();
	m_Font.CreatePointFont(pointsize*10, logfont.lfFaceName);
	CCodeEditor::SetFont(&m_Font);
	//CCodeEditor::SetPrinterFont(&m_Font);
}