#pragma once

#include <vector>
#include <iostream>

class Array: public Data
{
public:
	int dim;
	DataType eletype;
public:
	
	Array(std::string vrid, int d, DataType elet, bool vstatic):
	  dim(d), eletype(elet), Data(vrid, TypeArray, vstatic) {}
	
	virtual DataType GetEleType() const { return eletype; }
	virtual int get_dim() const { return dim; }
	virtual void setzero() = 0;

	virtual ~Array() { /* empty */ }
};

template<typename T> class Array1D: public Array
{
protected:

	PtrStack<std::vector<T> > theStack;
	
public:
	
	Array1D(std::string vrid="", DataType elet=TypeArray, bool vstatic=false): 
	 theStack(vrid.c_str(), true), Array(vrid, 1, elet, vstatic)
	 { 
		if(vstatic) Push();
	 }
				
	void size(int sz)
	{
		if(sz<= 0)
			throw std::exception("Array size should be greater than 0.");

		theStack.top()->resize(sz);
	}

	void resize(int sz) { this->size(sz); }


	void copy(Array1D<T>& M)
	{
		M.size(size());
		for(int i=0; i < size(); ++i)
			M[i] = theStack.top()->operator [](i);
	}
	
	int getsize()
	{
		return int(theStack.top()->size());
	}

	virtual void setzero()
	{
		for(int i=0; i < getsize(); ++i)
			theStack.top()->operator [](i) = T();
	}
	
	T& operator[](int index)
	{
		return theStack.top()->operator [](index);
	}
	
	virtual void Push() { theStack.push(new std::vector<T>()); }
	virtual void Pop()  { delete theStack.pop(); }
	
	virtual ~Array1D() 
	{	while(!theStack.empty()) 
			delete theStack.pop(); }

};

template<typename T>
std::ostream& operator<<(std::ostream& os, Array1D<T>& M)
{
	for(int i=M.size()-1; i>=0; --i)
		if(i == 0)
			os<<MakeRational(M[i]);
		else
			os<<MakeRational(M[i])<<", ";
	return os;
}


template<typename T> class Array2D: public Array
{
protected:
	
	PtrStack<std::vector<std::vector<T> > > theStack;

public:
	
	Array2D(std::string vrid="", DataType elet=TypeArray, bool vstatic=false): 
	theStack(vrid.c_str(), true), Array(vrid, 2, elet, vstatic)
	{  if(vstatic) Push(); }

	void size(int size1, int size2)
	{
		if(size1<= 0)
			throw std::exception("Array size should be greater than 0.");
		
		if(size2<= 0)
			throw std::exception("Array size should be greater than 0.");

		std::vector<std::vector<T> > *pVec = theStack.top();
		
		pVec->resize(size1);
			
		for(int i=0; i<size1; ++i)
		{
			::Sleep(0);

			pVec->operator[](i).resize(size2);
		}
	}

	void resize(int size1, int size2)
	{
		std::vector<std::vector<T> > *pVec = theStack.top();
		
		if(size1>0)
			theStack.top()->resize(size1);
		else
			size1 = int(pVec->size());
							
		for(int i=0; i<size1; ++i)
		{
			::Sleep(0);

			if(size2>0)
				pVec->operator [](i).resize(size2);
			else break;
		}
	}

	virtual void setzero()
	{
		int size1, size2;
		getsize(size1, size2);

		for(int i=0; i<size1; ++i)
		{
			::Sleep(0);
			for(int j=0; j<size2; ++j)
			{
				::Sleep(0);
				theStack.top()->operator [](i)[j]=T();
			}
		}
	}
	
	void getsize(int& size1, int& size2)
	{
		std::vector<std::vector<T> >
			*pAry = theStack.top();
		
		size1 = int(pAry->size());
		size2 = int(pAry->operator [](0).size());
	}

	std::vector<T>& operator[](int index)
	{
		return theStack.top()->operator [](index);
	}

	virtual void Push() 
	{ 
		theStack.push(new std::vector<std::vector<T> > ()); 
	}

	virtual void Pop()  { delete theStack.pop(); }

	virtual ~Array2D() 
	{	while(!theStack.empty()) delete theStack.pop();  }
};

template<typename T> class Array3D: public Array
{
protected:
	
	PtrStack< std::vector<std::vector<std::vector<T> > > > theStack;

public:
	Array3D(std::string vrid="", DataType elet=TypeArray, bool vstatic=false): 
	 theStack(vrid.c_str(), true), Array(vrid, 3, elet, vstatic)  
	  { if(vstatic) Push(); }
	
	void size(int size1, int size2, int size3)
	{
		if(size1<= 0)
			throw std::exception("Array size should be greater than 0.");
		
		if(size2<= 0)
			throw std::exception("Array size should be greater than 0.");
		
		if(size3<= 0)
			throw std::exception("Array size should be greater than 0.");

		std::vector<std::vector<std::vector<T> > > *pVec = theStack.top();
		
		pVec->resize(size1);

		for(int i=0; i<size1; ++i)
		{
			::Sleep(0);
			pVec->operator [](i).resize(size2);
			for(int j=0; j<size2; ++j)
			{
				::Sleep(0);
				pVec->operator [](i)[j].resize(size3);
			}
		}
	}

	void resize(int size1, int size2, int size3)
	{
		std::vector<std::vector<std::vector<T> > > *pVec = theStack.top();
		
		if(size1>0)
			pVec->resize(size1);
		else
			size1 = int(pVec->size());

		for(int i=0; i<size1; ++i)
		{
			::Sleep(0);

			if(size2>0)
				pVec->operator [](i).resize(size2);
			else
				size2 = int(pVec->size());
			
			for(int j=0; j<size2; ++j)
			{
				::Sleep(0);
				if(size3>0)
					pVec->operator [](i)[j].resize(size3);
				else break;
			}
		}
	}
	
	virtual void setzero()
	{
		int size1, size2, size3;
		getsize(size1, size2, size3);
		std::vector<std::vector<std::vector<T> > > 
			*pAry = theStack.top();

		for(int i=0; i<size1; ++i)
		{
			::Sleep(0);
			for(int j=0; j<size2; ++j)
			{
				::Sleep(0);
				for(int k=0; k<size3; ++k)
				{
					::Sleep(0);
					pAry->operator[](i)[j][k] = T();
				}
			}
		}
	}

	void getsize(int& size1, int& size2, int& size3)
	{
		std::vector<std::vector<std::vector<T> > > 
			*pAry = theStack.top();
		
		size1 = int(pAry->size());
		size2 = int(pAry->operator [](0).size());
		size3 = int(pAry->operator [](0)[0].size());
	}

	std::vector<std::vector<T> >& operator[](int index)
	{
		return theStack.top()->operator [](index);
	}

	virtual void Push() 
	{ 
		theStack.push(new std::vector<std::vector<std::vector<T> > > () );
	}

	virtual void Pop()  { delete theStack.pop(); }

	virtual ~Array3D() 
	{	while(!theStack.empty())  delete theStack.pop(); }
};

template<typename T>
void Add_MN(Array1D<T>& M, Array1D<T>& N, Array1D<T>& R)
{
	int min = std::min<int>(M.size(), N.size());

	if(M.size()>N.size())
	{
		R = M;	
		for(int i = 0; i<min; ++i) R[i] += N[i];
	}
	else
	{
		R = N;
		for(int i = 0; i<min; ++i) R[i] += M[i];
	}
}

template<typename T>
void Sub_MN(Array1D<T>& M, Array1D<T>& N, Array1D<T>& R)
{
	int min = std::min<int>(M.size(), N.size());
	
	if(M.size()>=N.size())
	{
		R = M;
		for(int i = 0; i<min; ++i) R[i] -= N[i];
	}
	else
	{
		R.size(N.size());
		for(int i = 0; i<N.size(); ++i) R[i] = -N[i];
		for(int i = 0; i<min; ++i) R[i] += M[i];
	}
}

template<typename T>
void Mul_SM(T& scalar, Array1D<T>& M, Array1D<T>& R)
{
	for(int i = 0; i<M.size(); ++i)
		R[i] = scalar*M[i];
}

template<typename T>
void Div_MS(T& scalar, Array1D<T>& M, Array1D<T>& R)
{
	for(int i = 0; i<M.size(); ++i)
		R[i] = M[i]/scalar;
}

template<typename T>
void Mul_MN(Array1D<T>&M, Array1D<T>& N, Array1D<T>& R)
{
	R.size(M.size()+N.size()-1);
	for(int m=0; m<M.size(); ++m)
		for(int n=0; n<N.size(); ++n)
			R[m+n] += M[m]*N[n];
}
template<typename T> // M/N
void Div_mn(Array1D<T>&M, Array1D<T>& N, Array1D<T>& Q, Array1D<T>& R)
{
	if(M.size()<N.size()) return;
	else
	{
		Array1D<double> MM = M;

		T ab = MM[MM.size()-1] / N[N.size()-1];
		int mn = MM.size()-N.size();
		Q[mn] = ab;

		for(int i = 0; i<MM.size()-1; ++i)
			R[i] = MM[i];

		for(int i= 0; i<N.size()-1; ++i)
			R[i+mn] -= ab*N[i];
		
		if(R.size()<N.size()) return;
		
		MM = R;	R.size(MM.size()-1);
		Div_mn(MM, N, Q, R);
	}
}

template<typename T> // M/N
void Div_MN(Array1D<T>&M, Array1D<T>& N, Array1D<T>& Q, Array1D<T>& R)
{
	if(M.size()<N.size()) return;
	
	Q.size(M.size() - N.size() + 1);
	R.size(M.size() - 1);

	Div_mn(M, N, Q, R);
}

template<typename T> bool operator == (Array1D<T>& M, T v)
{
	for(int i=0; i<M.size(); ++i)
		if(M[i] != v) return false;
	return true;
}

template<typename T> bool operator != (Array1D<T>& M, T v)
{
	return !operator==(M, v);
}

// Greatest Common Divisor
template<typename T>
void Gcd(Array1D<T> M, Array1D<T> N, Array1D<T>& G)
{ 
	// declaration of remainder
	Array1D<T> Q, R;
	
	while (N != 0.0) 
	{ 	
		Div_MN(M, N, Q, R); M = N; N = R; 
	}

	G = M;
}

