#pragma once

#include <vector>
#include <iostream>
#include <kch_basetype.h>



template<typename T> void GetVecSize(int &size, typename const VctrType<T>::Vec1D& v)
{
	size = v.size();
}

template<typename T> void GetVecSize(int &size1, int &size2,
									 typename const VctrType<T>::Vec2D& v)
{
	size1 = v.size();

	if(size1>0) GetVecSize<T>(size2, v[0]);
	else size2 = 0;
}

template<typename T> void GetVecSize(int &size1, int &size2,
									 int &size3, typename const VctrType<T>::Vec3D& v)
{
	size1 = v.size();

	if(size1>0)  GetVecSize<T>(size2, size3, v[0]);
	else size2 = size3 = 0;
}

template<typename T> void GetVecSize(int &size1, int &size2,
									 int &size3, int &size4, typename const VctrType<T>::Vec4D& v)
{
	size1 = v.size();

	if(size1>0)  GetVecSize<T>(size2, size3, size4, v[0]);
	else size2 = size3 = size4 = 0;
}

template<typename T> void GetVecSize(int &size1, int &size2,
									 int &size3, int &size4, int &size5, typename const VctrType<T>::Vec5D& v)
{
	size1 = v.size();

	if(size1>0)  GetVecSize<T>(size2, size3, size4, size5, v[0]);
	else size2 = size3 = size4 = size5 = 0;
}

template<typename T> bool IsCongruent(typename const VctrType<T>::Vec1D& v1, typename const VctrType<T>::Vec1D& v2)
{
	if(v1.size() == v2.size()) return true;
	else return false;
}

template<typename T> bool IsCongruent(typename const VctrType<T>::Vec2D& v1, typename const VctrType<T>::Vec2D& v2)
{
	if(v1.size() == v2.size() && v1.size() > 0) 
		return IsCongruent<T>(v1[0], v2[0]);
	else return false;
}

template<typename T> bool IsCongruent(typename const VctrType<T>::Vec3D& v1, typename const VctrType<T>::Vec3D& v2)
{
	if(v1.size() == v2.size() && v1.size() > 0) 
		return IsCongruent<T>(v1[0], v2[0]);
	else return false;
}

template<typename T> bool IsCongruent(typename const VctrType<T>::Vec4D& v1, typename const VctrType<T>::Vec4D& v2)
{
	if(v1.size() == v2.size() && v1.size() > 0) 
		return IsCongruent<T>(v1[0], v2[0]);
	else return false;
}

template<typename T> bool IsCongruent(typename const VctrType<T>::Vec5D& v1, typename const VctrType<T>::Vec5D& v2)
{
	if(v1.size() == v2.size() && v1.size() > 0) 
		return IsCongruent<T>(v1[0], v2[0]);
	else return false;
}

template<typename T> bool SetVecSize(int size, typename VctrType<T>::Vec1D& v)
{
	if(size>0)
	{
		v.resize(size); return true;
	}
	else return false;
}

template<typename T> bool SetVecSize(int size1, int size2, typename VctrType<T>::Vec2D& v)
{
	if(size1>0)
	{
		v.resize(size1);
		for(int i=0; i<size1; ++i)
		{
			if(SetVecSize<T>(size2, v[i])) continue;
			else return false;
		}

		return true;
	}
	else return false;
}

template<typename T> bool SetVecSize(int size1, int size2, int size3, typename VctrType<T>::Vec3D& v)
{
	if(size1>0)
	{
		v.resize(size1);
		for(int i=0; i<size1; ++i)
		{
			if(SetVecSize<T>(size2, size3, v[i])) continue;
			else return false;
		}

		return true;
	}
	else return false;
}

template<typename T> bool SetVecSize(int size1, int size2, int size3, int size4, typename VctrType<T>::Vec4D& v)
{
	if(size1>0)
	{
		v.resize(size1);
		for(int i=0; i<size1; ++i)
		{
			if(SetVecSize<T>(size2, size3, size4, v[i])) continue;
			else return false;
		}

		return true;
	}
	else return false;
}

template<typename T> bool SetVecSize(int size1, int size2, int size3, int size4, int size5, typename VctrType<T>::Vec5D& v)
{
	if(size1>0)
	{
		v.resize(size1);
		for(int i=0; i<size1; ++i)
		{
			if(SetVecSize<T>(size2, size3, size4, size5, v[i])) continue;
			else return false;
		}

		return true;
	}
	else return false;
}

template<typename T> void
MatPro(typename VctrType<T>::Vec2D& v, 
	   typename const VctrType<T>::Vec2D& v1, typename const VctrType<T>::Vec2D& v2)
{
	int v1m, v1n; GetVecSize<T>(v1m, v1n, v1);
	int v2m, v2n; GetVecSize<T>(v2m, v2n, v2);

	if(v1n != v2m) 
		throw UnicodeException(_T("Matrices are not congruent for multiplication"));

	if(SetVecSize<T>(v1m, v2n, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<v1m; ++i)
		for(int j=0; j<v2n; ++j)
			for(int k=0; k<v1n; ++k)
				v[i][j] += v1[i][k]*v2[k][j];
}

template<typename vec> bool SetSize(int size, vec& v)
{
	if(size>0)
	{
		v.resize(size); return true;
	}
	else return false;
}

inline void MulScalar(VctrVec1D& v, const VctrVec1D& v1, const realnum& value)
{
	int size=v1.size();
		
	if(SetSize<VctrVec1D>(size, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size; ++i) v[i] = v1[i] * value;
}

inline void DivScalar(VctrVec1D& v, const VctrVec1D& v1, const realnum& value)
{
	if(value == realnum()) throw UnicodeException(_T("Division By Zero"));

	int size = v1.size();
	
	if(SetSize<VctrVec1D>(size, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size; ++i) v[i] = v1[i] / value;
}

inline void MulScalar(VctrVec2D& v, const VctrVec2D& v1, const realnum& value)
{
	int size = v1.size();
	
	if(SetSize<VctrVec2D>(size, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size; ++i) MulScalar(v[i], v1[i], value);
}

inline void DivScalar(VctrVec2D& v, const VctrVec2D& v1, const realnum& value)
{
	if(value == realnum()) throw UnicodeException(_T("Division By Zero"));

	int size = v1.size();
	
	if(SetSize<VctrVec2D>(size, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size; ++i) DivScalar(v[i], v1[i], value);
}

inline void MulScalar(VctrVec3D& v, const VctrVec3D& v1, const realnum& value)
{
	int size = v1.size();
	
	if(SetSize<VctrVec3D>(size, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size; ++i) MulScalar(v[i], v1[i], value);
}

inline void DivScalar(VctrVec3D& v, const VctrVec3D& v1, const realnum& value)
{
	if(value == realnum()) throw UnicodeException(_T("Division By Zero"));

	int size = v1.size();
	
	if(SetSize<VctrVec3D>(size, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size; ++i) DivScalar(v[i], v1[i], value);
}

inline void MulScalar(VctrVec4D& v, const VctrVec4D& v1, const realnum& value)
{
	int size = v1.size();
	
	if(SetSize<VctrVec4D>(size, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size; ++i) MulScalar(v[i], v1[i], value);
}

inline void DivScalar(VctrVec4D& v, const VctrVec4D& v1, const realnum& value)
{
	if(value == realnum()) throw UnicodeException(_T("Division By Zero"));

	int size = v1.size();
	
	if(SetSize<VctrVec4D>(size, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size; ++i) DivScalar(v[i], v1[i], value);
}

inline void MulScalar(VctrVec5D& v, const VctrVec5D& v1, const realnum& value)
{
	int size=v1.size();
	
	if(SetSize<VctrVec5D>(size, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size; ++i) MulScalar(v[i], v1[i], value);
}

inline void DivScalar(VctrVec5D& v, const VctrVec5D& v1, const realnum& value)
{
	if(value == realnum()) throw UnicodeException(_T("Division By Zero"));

	int size = v1.size();
	
	if(SetSize<VctrVec5D>(size, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size; ++i) DivScalar(v[i], v1[i], value);
}

inline void DotPro(RealVec1D& v, const VctrVec1D& v1, const VctrVec1D& v2)
{
	if(IsCongruent<Vector>(v1, v2)==false)
		throw UnicodeException(_T("Vector Array Mismatch"));

	v.resize(v1.size());
	
	dotproduct(v, v1, v2);
}

inline void DotPro(RealVec2D& v, const VctrVec2D& v1, const VctrVec2D& v2)
{
	if(IsCongruent<Vector>(v1, v2)==false)
		throw UnicodeException(_T("Vector Array Mismatch"));

	int size=v1.size(); v.resize(size);
	
	for(int i=0; i<size; ++i) DotPro(v[i], v1[i], v2[i]);
}

inline void DotPro(RealVec3D& v, const VctrVec3D& v1, const VctrVec3D& v2)
{
	if(IsCongruent<Vector>(v1, v2)==false)
		throw UnicodeException(_T("Vector Array Mismatch"));

	int size=v1.size(); v.resize(size);
	
	for(int i=0; i<size; ++i) DotPro(v[i], v1[i], v2[i]);
}

inline void DotPro(RealVec4D& v, const VctrVec4D& v1, const VctrVec4D& v2)
{
	if(IsCongruent<Vector>(v1, v2)==false)
		throw UnicodeException(_T("Vector Array Mismatch"));

	int size=v1.size(); v.resize(size);
	
	for(int i=0; i<size; ++i) DotPro(v[i], v1[i], v2[i]);
}

inline void DotPro(RealVec5D& v, const VctrVec5D& v1, const VctrVec5D& v2)
{
	if(IsCongruent<Vector>(v1, v2)==false)
		throw UnicodeException(_T("Vector Array Mismatch"));

	int size=v1.size(); v.resize(size);
	
	for(int i=0; i<size; ++i) DotPro(v[i], v1[i], v2[i]);
}

inline void CrossPro(VctrVec1D& v, const VctrVec1D& v1, const Vector& value)
{
	int size=v1.size(); v.resize(size);
	v = v1 % value;
}

inline void CrossPro(VctrVec1D& v, const Vector& value, const VctrVec1D& v1)
{
	int size=v1.size(); v.resize(size);
	v = value % v1;
}

inline void CrossPro(VctrVec2D& v, const VctrVec2D& v1, const Vector& value)
{
	int size=v1.size(); v.resize(size);
	for(int i=0; i<size; ++i) CrossPro(v[i], v1[i], value);
}

inline void CrossPro(VctrVec2D& v, const Vector& value, const VctrVec2D& v1)
{
	int size=v1.size(); v.resize(size);
	for(int i=0; i<size; ++i) CrossPro(v[i], value, v1[i]);
}

inline void CrossPro(VctrVec3D& v, const VctrVec3D& v1, const Vector& value)
{
	int size=v1.size(); v.resize(size);
	for(int i=0; i<size; ++i) CrossPro(v[i], v1[i], value);
}

inline void CrossPro(VctrVec3D& v, const Vector& value, const VctrVec3D& v1)
{
	int size=v1.size(); v.resize(size);
	for(int i=0; i<size; ++i) CrossPro(v[i], value, v1[i]);
}

inline void CrossPro(VctrVec4D& v, const VctrVec4D& v1, const Vector& value)
{
	int size=v1.size(); v.resize(size);
	for(int i=0; i<size; ++i) CrossPro(v[i], v1[i], value);
}

inline void CrossPro(VctrVec4D& v, const Vector& value, const VctrVec4D& v1)
{
	int size=v1.size(); v.resize(size);
	for(int i=0; i<size; ++i) CrossPro(v[i], value, v1[i]);
}

inline void CrossPro(VctrVec5D& v, const VctrVec5D& v1, const Vector& value)
{
	int size=v1.size(); v.resize(size);
	for(int i=0; i<size; ++i) CrossPro(v[i], v1[i], value);
}

inline void CrossPro(VctrVec5D& v, const Vector& value, const VctrVec5D& v1)
{
	int size=v1.size(); v.resize(size);
	for(int i=0; i<size; ++i) CrossPro(v[i], value, v1[i]);
}

inline void SDotPro(RealVec1D& r, const VctrVec1D& va, const Vector& v)
{	dotpro(r, va, v);  }
inline void SDotPro(RealVec1D& r, const Vector& v, const VctrVec1D& va)
{	dotpro(r, v, va);  }

inline void SDotPro(RealVec2D& r, const VctrVec2D& va, const Vector& v)
{	
	int size = va.size(); r.resize(size);

	for(int i=0; i<size; ++i) SDotPro(r[i], va[i], v);  
}
inline void SDotPro(RealVec2D& r, const Vector& v, const VctrVec2D& va)
{	
	int size = va.size(); r.resize(size);

	for(int i=0; i<size; ++i) SDotPro(r[i], v, va[i]);  
}

inline void SDotPro(RealVec3D& r, const VctrVec3D& va, const Vector& v)
{	
	int size = va.size(); r.resize(size);

	for(int i=0; i<size; ++i) SDotPro(r[i], va[i], v);  
}
inline void SDotPro(RealVec3D& r, const Vector& v, const VctrVec3D& va)
{	
	int size = va.size(); r.resize(size);

	for(int i=0; i<size; ++i) SDotPro(r[i], v, va[i]);  
}

inline void SDotPro(RealVec4D& r, const VctrVec4D& va, const Vector& v)
{	
	int size = va.size(); r.resize(size);

	for(int i=0; i<size; ++i) SDotPro(r[i], va[i], v);  
}
inline void SDotPro(RealVec4D& r, const Vector& v, const VctrVec4D& va)
{	
	int size = va.size(); r.resize(size);

	for(int i=0; i<size; ++i) SDotPro(r[i], v, va[i]);  
}

inline void SDotPro(RealVec5D& r, const VctrVec5D& va, const Vector& v)
{	
	int size = va.size(); r.resize(size);

	for(int i=0; i<size; ++i) SDotPro(r[i], va[i], v);  
}
inline void SDotPro(RealVec5D& r, const Vector& v, const VctrVec5D& va)
{	
	int size = va.size(); r.resize(size);

	for(int i=0; i<size; ++i) SDotPro(r[i], v, va[i]);  
}

///////////////////////////////////////////

template<typename T> typename VctrType<T>::Vec2D 
operator + (const T& s, typename const VctrType<T>::Vec2D& vec)
{
	int size1, size2; 
	GetVecSize<T>(size1, size2, vec);
	typename VctrType<T>::Vec2D v;

	if(SetVecSize<T>(size1, size2, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size1; ++i) v[i] = s + vec[i];
	
	return v;
}

template<typename T> typename VctrType<T>::Vec2D 
operator + (typename const VctrType<T>::Vec2D& vec, const T& s)
{
	int size1, size2; 
	GetVecSize<T>(size1, size2, vec);
	typename VctrType<T>::Vec2D v;

	if(SetVecSize<T>(size1, size2, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size1; ++i) v[i] = vec[i] + s;
	
	return v;
}

template<typename T> typename VctrType<T>::Vec2D 
operator - (const T& s, typename const VctrType<T>::Vec2D& vec)
{
	int size1, size2; 
	GetVecSize<T>(size1, size2, vec);
	typename VctrType<T>::Vec2D v;

	if(SetVecSize<T>(size1, size2, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size1; ++i) v[i] = s - vec[i];
	
	return v;
}

template<typename T> typename VctrType<T>::Vec2D 
operator - (typename const VctrType<T>::Vec2D& vec, const T& s)
{
	int size1, size2; 
	GetVecSize<T>(size1, size2, vec);
	typename VctrType<T>::Vec2D v;

	if(SetVecSize<T>(size1, size2, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size1; ++i) v[i] = vec[i] - s;
	
	return v;
}

template<typename T> typename VctrType<T>::Vec2D 
operator * (const T& s, typename const VctrType<T>::Vec2D& vec)
{
	int size1, size2; 
	GetVecSize<T>(size1, size2, vec);
	typename VctrType<T>::Vec2D v;

	if(SetVecSize<T>(size1, size2, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size1; ++i) v[i] = s * vec[i];
	
	return v;
}

template<typename T> typename VctrType<T>::Vec2D 
operator * (typename const VctrType<T>::Vec2D& vec, const T& s)
{
	int size1, size2; 
	GetVecSize<T>(size1, size2, vec);
	typename VctrType<T>::Vec2D v;

	if(SetVecSize<T>(size1, size2, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size1; ++i) v[i] = vec[i] * s;
	
	return v;
}

template<typename T> typename VctrType<T>::Vec2D 
operator / (const T& s, typename const VctrType<T>::Vec2D& vec)
{
	int size1, size2; 
	GetVecSize<T>(size1, size2, vec);
	typename VctrType<T>::Vec2D v;

	if(SetVecSize<T>(size1, size2, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size1; ++i) v[i] = s / vec[i];
	
	return v;
}

template<typename T> typename VctrType<T>::Vec2D 
operator / (typename const VctrType<T>::Vec2D& vec, const T& s)
{
	int size1, size2; 
	GetVecSize<T>(size1, size2, vec);
	typename VctrType<T>::Vec2D v;

	if(SetVecSize<T>(size1, size2, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size1; ++i) v[i] = vec[i] / s;
	
	return v;
}

template<typename T> typename VctrType<T>::Vec3D 
operator + (const T& s, typename const VctrType<T>::Vec3D& vec)
{
	int size1, size2, size3; 
	GetVecSize<T>(size1, size2, size3,vec);
	typename VctrType<T>::Vec3D v;

	if(SetVecSize<T>(size1, size2, size3, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size1; ++i) v[i] = s + vec[i];
	
	return v;
}

template<typename T> typename VctrType<T>::Vec3D 
operator + (typename const VctrType<T>::Vec3D& vec, const T& s)
{
	int size1, size2, size3; 
	GetVecSize<T>(size1, size2, size3, vec);
	typename VctrType<T>::Vec3D v;

	if(SetVecSize<T>(size1, size2, size3, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size1; ++i) v[i] = vec[i] + s;
	
	return v;
}

template<typename T> typename VctrType<T>::Vec3D 
operator - (const T& s, typename const VctrType<T>::Vec3D& vec)
{
	int size1, size2, size3; 
	GetVecSize<T>(size1, size2, size3, vec);
	typename VctrType<T>::Vec3D v;

	if(SetVecSize<T>(size1, size2, size3, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size1; ++i) v[i] = s - vec[i];
	
	return v;
}

template<typename T> typename VctrType<T>::Vec3D 
operator - (typename const VctrType<T>::Vec3D& vec, const T& s)
{
	int size1, size2, size3; 
	GetVecSize<T>(size1, size2, size3, vec);
	typename VctrType<T>::Vec3D v;

	if(SetVecSize<T>(size1, size2, size3, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size1; ++i) v[i] = vec[i] - s;
	
	return v;
}

template<typename T> typename VctrType<T>::Vec3D 
operator * (const T& s, typename const VctrType<T>::Vec3D& vec)
{
	int size1, size2, size3; 
	GetVecSize<T>(size1, size2, size3, vec);
	typename VctrType<T>::Vec3D v;

	if(SetVecSize<T>(size1, size2, size3, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size1; ++i) v[i] = s * vec[i];
	
	return v;
}

template<typename T> typename VctrType<T>::Vec3D 
operator * (typename const VctrType<T>::Vec3D& vec, const T& s)
{
	int size1, size2, size3; 
	GetVecSize<T>(size1, size2, size3, vec);
	typename VctrType<T>::Vec3D v;

	if(SetVecSize<T>(size1, size2, size3, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size1; ++i) v[i] = vec[i] * s;
	
	return v;
}

template<typename T> typename VctrType<T>::Vec3D 
operator / (const T& s, typename const VctrType<T>::Vec3D& vec)
{
	int size1, size2, size3; 
	GetVecSize<T>(size1, size2, size3, vec);
	typename VctrType<T>::Vec3D v;

	if(SetVecSize<T>(size1, size2, size3, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size1; ++i) v[i] = s / vec[i];
	
	return v;
}

template<typename T> typename VctrType<T>::Vec3D 
operator / (typename const VctrType<T>::Vec3D& vec, const T& s)
{
	int size1, size2, size3; 
	GetVecSize<T>(size1, size2, size3, vec);
	typename VctrType<T>::Vec3D v;

	if(SetVecSize<T>(size1, size2, size3, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size1; ++i) v[i] = vec[i] / s;
	
	return v;
}

template<typename T> typename VctrType<T>::Vec4D 
operator + (const T& s, typename const VctrType<T>::Vec4D& vec)
{
	int size1, size2, size3, size4; 
	GetVecSize<T>(size1, size2, size3, size4, vec);
	typename VctrType<T>::Vec4D v;

	if(SetVecSize<T>(size1, size2, size3, size4, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size1; ++i) v[i] = s + vec[i];
	
	return v;
}

template<typename T> typename VctrType<T>::Vec4D 
operator + (typename const VctrType<T>::Vec4D& vec, const T& s)
{
	int size1, size2, size3, size4; 
	GetVecSize<T>(size1, size2, size3, size4, vec);
	typename VctrType<T>::Vec4D v;

	if(SetVecSize<T>(size1, size2, size3, size4, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size1; ++i) v[i] = vec[i] + s;
	
	return v;
}

template<typename T> typename VctrType<T>::Vec4D 
operator - (const T& s, typename const VctrType<T>::Vec4D& vec)
{
	int size1, size2, size3, size4; 
	GetVecSize<T>(size1, size2, size3, size4, vec);
	typename VctrType<T>::Vec4D v;

	if(SetVecSize<T>(size1, size2, size3, size4, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size1; ++i) v[i] = s - vec[i];
	
	return v;
}

template<typename T> typename VctrType<T>::Vec4D 
operator - (typename const VctrType<T>::Vec4D& vec, const T& s)
{
	int size1, size2, size3, size4; 
	GetVecSize<T>(size1, size2, size3, size4, vec);
	typename VctrType<T>::Vec4D v;

	if(SetVecSize<T>(size1, size2, size3, size4, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size1; ++i) v[i] = vec[i] - s;
	
	return v;
}

template<typename T> typename VctrType<T>::Vec4D 
operator * (const T& s, typename const VctrType<T>::Vec4D& vec)
{
	int size1, size2, size3, size4; 
	GetVecSize<T>(size1, size2, size3, size4, vec);
	typename VctrType<T>::Vec4D v;

	if(SetVecSize<T>(size1, size2, size3, size4, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size1; ++i) v[i] = s * vec[i];
	
	return v;
}

template<typename T> typename VctrType<T>::Vec4D 
operator * (typename const VctrType<T>::Vec4D& vec, const T& s)
{
	int size1, size2, size3, size4; 
	GetVecSize<T>(size1, size2, size3, size4, vec);
	typename VctrType<T>::Vec4D v;

	if(SetVecSize<T>(size1, size2, size3, size4, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size1; ++i) v[i] = vec[i] * s;
	
	return v;
}

template<typename T> typename VctrType<T>::Vec4D 
operator / (const T& s, typename const VctrType<T>::Vec4D& vec)
{
	int size1, size2, size3, size4; 
	GetVecSize<T>(size1, size2, size3, size4, vec);
	typename VctrType<T>::Vec4D v;

	if(SetVecSize<T>(size1, size2, size3, size4, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size1; ++i) v[i] = s / vec[i];
	
	return v;
}

template<typename T> typename VctrType<T>::Vec4D 
operator / (typename const VctrType<T>::Vec4D& vec, const T& s)
{
	int size1, size2, size3, size4; 
	GetVecSize<T>(size1, size2, size3, size4, vec);
	typename VctrType<T>::Vec4D v;

	if(SetVecSize<T>(size1, size2, size3, size4, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size1; ++i) v[i] = vec[i] / s;
	
	return v;
}

template<typename T> typename VctrType<T>::Vec5D 
operator + (const T& s, typename const VctrType<T>::Vec5D& vec)
{
	int size1, size2, size3, size4, size5; 
	GetVecSize<T>(size1, size2, size3, size4, size5, vec);
	typename VctrType<T>::Vec5D v;

	if(SetVecSize<T>(size1, size2, size3, size4, size5, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size1; ++i) v[i] = s + vec[i];
	
	return v;
}

template<typename T> typename VctrType<T>::Vec5D 
operator + (typename const VctrType<T>::Vec5D& vec, const T& s)
{
	int size1, size2, size3, size4, size5; 
	GetVecSize<T>(size1, size2, size3, size4, size5, vec);
	typename VctrType<T>::Vec5D v;

	if(SetVecSize<T>(size1, size2, size3, size4, size5, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size1; ++i) v[i] = vec[i] + s;
	
	return v;
}

template<typename T> typename VctrType<T>::Vec5D 
operator - (const T& s, typename const VctrType<T>::Vec5D& vec)
{
	int size1, size2, size3, size4, size5; 
	GetVecSize<T>(size1, size2, size3, size4, size5, vec);
	typename VctrType<T>::Vec5D v;

	if(SetVecSize<T>(size1, size2, size3, size4, size5, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size1; ++i) v[i] = s - vec[i];
	
	return v;
}

template<typename T> typename VctrType<T>::Vec5D 
operator - (typename const VctrType<T>::Vec5D& vec, const T& s)
{
	int size1, size2, size3, size4, size5; 
	GetVecSize<T>(size1, size2, size3, size4, size5, vec);
	typename VctrType<T>::Vec5D v;

	if(SetVecSize<T>(size1, size2, size3, size4, size5, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size1; ++i) v[i] = vec[i] - s;
	
	return v;
}

template<typename T> typename VctrType<T>::Vec5D 
operator * (const T& s, typename const VctrType<T>::Vec5D& vec)
{
	int size1, size2, size3, size4, size5; 
	GetVecSize<T>(size1, size2, size3, size4, size5, vec);
	typename VctrType<T>::Vec5D v;

	if(SetVecSize<T>(size1, size2, size3, size4, size5, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size1; ++i) v[i] = s * vec[i];
	
	return v;
}

template<typename T> typename VctrType<T>::Vec5D 
operator * (typename const VctrType<T>::Vec5D& vec, const T& s)
{
	int size1, size2, size3, size4, size5; 
	GetVecSize<T>(size1, size2, size3, size4, size5, vec);
	typename VctrType<T>::Vec5D v;

	if(SetVecSize<T>(size1, size2, size3, size4, size5, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size1; ++i) v[i] = vec[i] * s;
	
	return v;
}

template<typename T> typename VctrType<T>::Vec5D 
operator / (const T& s, typename const VctrType<T>::Vec5D& vec)
{
	int size1, size2, size3, size4, size5; 
	GetVecSize<T>(size1, size2, size3, size4, size5, vec);
	typename VctrType<T>::Vec5D v;

	if(SetVecSize<T>(size1, size2, size3, size4, size5, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size1; ++i) v[i] = s / vec[i];
	
	return v;
}

template<typename T> typename VctrType<T>::Vec5D 
operator / (typename const VctrType<T>::Vec5D& vec, const T& s)
{
	int size1, size2, size3, size4, size5; 
	GetVecSize<T>(size1, size2, size3, size4, size5, vec);
	typename VctrType<T>::Vec5D v;

	if(SetVecSize<T>(size1, size2, size3, size4, size5, v)==false)
		throw UnicodeException(_T("Arrays are not conguent for the operation"));

	for(int i=0; i<size1; ++i) v[i] = vec[i] / s;
	
	return v;
}

template<typename T> T GetMaxEle(typename const VctrType<T>::Vec1D& vec)
{
	if(vec.size()==0) 
		throw UnicodeException(_T("Number of elements is zero(0). GetMaxEle() failed.\r\n"));

	return vec.max();
}

template<typename T> T GetMinEle(typename const VctrType<T>::Vec1D& vec)
{
	if(vec.size()==0) 
		throw UnicodeException(_T("Number of elements is zero(0). GetMinEle() failed.\r\n"));

	return vec.min();
}

template<typename T> T GetMaxEle(typename const VctrType<T>::Vec2D& vec)
{
	int size = vec.size(); 

	if(size ==0) 
		throw UnicodeException(_T("Number of elements is zero(0). GetMaxEle() failed.\r\n"));

	T rlt, tmp;
	
	for(int i=0; i<size; ++i) 
	{
		tmp = GetMaxEle<T>(vec[i]);
		if(i==0) rlt = tmp;
		else
		{  if(rlt<tmp) rlt = tmp; }
	}

	return rlt;
}

template<typename T> T GetMinEle(typename const VctrType<T>::Vec2D& vec)
{
	int size = vec.size(); 
	
	if(size ==0) 
		throw UnicodeException(_T("Number of elements is zero(0). GetMinEle() failed.\r\n"));

	T rlt, tmp;
	
	for(int i=0; i<size; ++i) 
	{
		tmp = GetMinEle<T>(vec[i]);
		if(i==0) rlt = tmp;
		else
		{ if(rlt>tmp) rlt = tmp; }
	}

	return rlt;
}

template<typename T> T GetMaxEle(typename const VctrType<T>::Vec3D& vec)
{
	int size = vec.size(); 
	
	if(size ==0) 
		throw UnicodeException(_T("Number of elements is zero(0). GetMaxEle() failed.\r\n"));

	T rlt, tmp;
	
	for(int i=0; i<size; ++i) 
	{
		tmp = GetMaxEle<T>(vec[i]);
		if(i==0) rlt = tmp;
		else
		{  if(rlt<tmp) rlt = tmp; }
	}

	return rlt;
}

template<typename T> T GetMinEle(typename const VctrType<T>::Vec3D& vec)
{
	int size = vec.size(); 
	
	if(size ==0) 
		throw UnicodeException(_T("Number of elements is zero(0). GetMinEle() failed.\r\n"));

	T rlt, tmp;
	
	for(int i=0; i<size; ++i) 
	{
		tmp = GetMinEle<T>(vec[i]);
		if(i==0) rlt = tmp;
		else
		{ if(rlt>tmp) rlt = tmp; }
	}

	return rlt;
}

template<typename T> T GetMaxEle(typename const VctrType<T>::Vec4D& vec)
{
	int size = vec.size(); 
	
	if(size ==0) 
		throw UnicodeException(_T("Number of elements is zero(0). GetMaxEle() failed.\r\n"));

	T rlt, tmp;
	
	for(int i=0; i<size; ++i) 
	{
		tmp = GetMaxEle<T>(vec[i]);
		if(i==0) rlt = tmp;
		else
		{  if(rlt<tmp) rlt = tmp; }
	}

	return rlt;
}

template<typename T> T GetMinEle(typename const VctrType<T>::Vec4D& vec)
{
	int size = vec.size(); 
	
	if(size ==0) 
		throw UnicodeException(_T("Number of elements is zero(0). GetMinEle() failed.\r\n"));

	T rlt, tmp;
	
	for(int i=0; i<size; ++i) 
	{
		tmp = GetMinEle<T>(vec[i]);
		if(i==0) rlt = tmp;
		else
		{ if(rlt>tmp) rlt = tmp; }
	}

	return rlt;
}

template<typename T> T GetMaxEle(typename const VctrType<T>::Vec5D& vec)
{
	int size = vec.size(); 
	
	if(size ==0) 
		throw UnicodeException(_T("Number of elements is zero(0). GetMaxEle() failed.\r\n"));

	T rlt, tmp;
	
	for(int i=0; i<size; ++i) 
	{
		tmp = GetMaxEle<T>(vec[i]);
		if(i==0) rlt = tmp;
		else
		{  if(rlt<tmp) rlt = tmp; }
	}

	return rlt;
}

template<typename T> T GetMinEle(typename const VctrType<T>::Vec5D& vec)
{
	int size = vec.size(); 
	
	if(size ==0) 
		throw UnicodeException(_T("Number of elements is zero(0). GetMinEle() failed.\r\n"));

	T rlt, tmp;
	
	for(int i=0; i<size; ++i) 
	{
		tmp = GetMinEle<T>(vec[i]);
		if(i==0) rlt = tmp;
		else
		{ if(rlt>tmp) rlt = tmp; }
	}

	return rlt;
}

template<typename T> typename VctrType<T>::Vec1D
GetComplement(typename VctrType<T>::Vec1D& v, integer index)
{	return v.Complement(index);  }

template<typename T> typename VctrType<T>::Vec2D
GetComplement(typename VctrType<T>::Vec2D& v, integer index1, integer index2)
{  
	VctrType<T>::Vec2D vec = v.Complement(index1); 
	int size = vec.size();
	for(int i=0; i<size; ++i) 
		vec[i] = GetComplement<T>(vec[i], index2);
	
	return vec;
}

template<typename T> typename VctrType<T>::Vec3D 
GetComplement(typename VctrType<T>::Vec3D& v,
			  integer index1, integer index2, integer index3)
{  
	VctrType<T>::Vec3D vec = v.Complement(index1); 
	int size = vec.size();
	for(int i=0; i<size; ++i) 
		vec[i] = GetComplement<T>(vec[i], index2, index3);
	
	return vec;
}

template<typename T> typename VctrType<T>::Vec4D 
GetComplement(typename VctrType<T>::Vec4D& v,
			  integer index1, integer index2, integer index3, integer index4)
{  
	VctrType<T>::Vec4D vec = v.Complement(index1); 
	int size = vec.size();
	for(int i=0; i<size; ++i) 
		vec[i] = GetComplement<T>(vec[i], index2, index3, index4);

	return vec;
}

template<typename T> typename VctrType<T>::Vec5D 
GetComplement(typename VctrType<T>::Vec5D& v,
			  integer index1, integer index2, integer index3, integer index4, integer index5)
{  
	VctrType<T>::Vec5D vec = v.Complement(index1); 
	int size = vec.size();
	for(int i=0; i<size; ++i) 
		vec[i] = GetComplement<T>(vec[i], index2, index3, index4, index5);
	
	return vec;
}


template<typename T> typename VctrType<T>::Vec1D
GetSubArray(typename VctrType<T>::Vec1D& v, bool cmp, Range *rng1)
{	return v.SubArray(cmp, rng1);   }

template<typename T> typename VctrType<T>::Vec2D
GetSubArray(typename VctrType<T>::Vec2D& v,
			bool cmp, Range *rng1, Range *rng2)
{
	typename VctrType<T>::Vec2D vec = v.SubArray(cmp, rng1);
	int size = vec.size();
	for(int i=0; i<size; ++i) 
		vec[i] = GetSubArray<T>(vec[i], cmp, rng2);
	
	return vec;
}

template<typename T> typename VctrType<T>::Vec3D
GetSubArray(typename VctrType<T>::Vec3D& v,
			bool cmp, Range *rng1, Range *rng2, Range *rng3)
{
	typename VctrType<T>::Vec3D vec = v.SubArray(cmp, rng1);
	int size = vec.size();
	for(int i=0; i<size; ++i) 
		vec[i] = GetSubArray<T>(vec[i], cmp, rng2, rng3);

	return vec;
}

template<typename T> typename VctrType<T>::Vec4D
GetSubArray(typename VctrType<T>::Vec4D& v,
			bool cmp, Range *rng1, Range *rng2, Range *rng3, Range *rng4)
{
	typename VctrType<T>::Vec4D vec = v.SubArray(cmp, rng1);
	int size = vec.size();
	for(int i=0; i<size; ++i) 
		vec[i] = GetSubArray<T>(vec[i], cmp, rng2, rng3, rng4);

	return vec;
}

template<typename T> typename VctrType<T>::Vec5D
GetSubArray(typename VctrType<T>::Vec5D& v,
			bool cmp, Range *rng1, Range *rng2, Range *rng3, Range *rng4, Range *rng5)
{
	typename VctrType<T>::Vec5D vec = v.SubArray(cmp, rng1);
	int size = vec.size();
	for(int i=0; i<size; ++i) 
		vec[i] = GetSubArray<T>(vec[i], cmp, rng2, rng3, rng4, rng5);

	return vec;
}

class Array: public Data
{
protected:
	int dim; DataType eletype, arytype;
	
	void copy(const Array& array)
	{
		this->dim = array.dim;
		this->eletype = array.eletype;
	}
	const Array& get_array() const { return *this; }

public:
	
	Array(std::wstring vrid, int d, bool vstatic=false):
	  dim(d), Data(vrid, TypeArray, vstatic) {  }
	  
	  DataType GetArrayType() const { return arytype;}
	Array(const Array& array): 
	  Data(array.get_data()) { this->copy(array); }

	Array& operator=(const Array& array)
	{
		Data::copy(array.get_data());
		this->copy(array); return *this;
	}

	template<typename T> void SetEleType()
	{
		const type_info& type = typeid(T);

		if(type==typeid(integer))
		{
			eletype = TypeIntg;
			switch(dim)
			{
			case 1: arytype = TypeAi1D; return;
			case 2: arytype = TypeAi2D; return;
			case 3: arytype = TypeAi3D; return;
			case 4: arytype = TypeAi4D; return;
			case 5: arytype = TypeAi5D; return;
			}
		}
		else if(type==typeid(realnum))
		{
			eletype = TypeReal;
			switch(dim)
			{
			case 1: arytype = TypeAr1D; return;
			case 2: arytype = TypeAr2D; return;
			case 3: arytype = TypeAr3D; return;
			case 4: arytype = TypeAr4D; return;
			case 5: arytype = TypeAr5D; return;
			}
		}
		else if(type==typeid(Complex))
		{
			eletype = TypeCplx;
			switch(dim)
			{
			case 1: arytype = TypeAc1D; return;
			case 2: arytype = TypeAc2D; return;
			case 3: arytype = TypeAc3D; return;
			case 4: arytype = TypeAc4D; return;
			case 5: arytype = TypeAc5D; return;
			}
		}
		else if(type==typeid(Vector))
		{
			eletype = TypeVctr;
			switch(dim)
			{
			case 1: arytype = TypeAv1D; return;
			case 2: arytype = TypeAv2D; return;
			case 3: arytype = TypeAv3D; return;
			case 4: arytype = TypeAv4D; return;
			case 5: arytype = TypeAv5D; return;
			}
		}
		
		HandleSyntaxError(_T("Unsupported Array Element Type.\r\n"));
	}
	
	DataType GetEleType() const { return eletype; }
	int GetDim() const { return dim; }

	virtual void resize(int size1) 
		{ throw UnicodeException(_T("Array 1 dimension mismatch in resize().\r\n")); }
	virtual void resize(int size1, int size2) 
		{ throw UnicodeException(_T("Array 2 dimension mismatch in resize().\r\n")); }
	virtual void resize(int size1, int size2, int size3) 
		{ throw UnicodeException(_T("Array 3 dimension mismatch in resize().\r\n")); }
	virtual void resize(int size1, int size2, int size3, int size4) 
		{ throw UnicodeException(_T("Array 4 dimension mismatch in resize().\r\n")); }
	virtual void resize(int size1, int size2, int size3, int size4, int size5) 
		{ throw UnicodeException(_T("Array 5 dimension mismatch in resize().\r\n")); }

	virtual int getsize() const 
		{ throw UnicodeException(_T("Array 1 dimension mismatch in getsize().\r\n")); }
	virtual void getsize(int& size1, int& size2)
		{ throw UnicodeException(_T("Array 2 dimension mismatch in getsize().\r\n")); }
	virtual void getsize(int& size1, int& size2, int& size3) 
		{ throw UnicodeException(_T("Array 3 dimension mismatch in getsize().\r\n")); }
	virtual void getsize(int& size1, int& size2, int& size3, int& size4) 
		{ throw UnicodeException(_T("Array 4 dimension mismatch in getsize().\r\n")); }
	virtual void getsize(int& size1, int& size2, int& size3, int& size4, int& size5) 
		{ throw UnicodeException(_T("Array 5 dimension mismatch in getsize().\r\n")); }
	
	virtual ~Array() { /* empty */ }
};

template<typename T> class Expression;

typedef Expression<bool>	ExpBool, *ExpBoolPtr;
typedef Expression<integer> ExpIntg, *ExpIntgPtr;
typedef Expression<realnum> ExpReal, *ExpRealPtr;
typedef Expression<Complex> ExpCplx, *ExpCplxPtr;
typedef Expression<Vector>  ExpVctr, *ExpVctrPtr;
typedef Expression<string>  ExpStrn, *ExpStrnPtr;

class Range: public BaseType
{
protected:
	ExpIntg *pStart, *pEnd;
public:
	Range(ExpIntg *xpr1, ExpIntg *xpr2):
	pStart(xpr1), pEnd(xpr2) {  }

	bool CheckRange() { return GetStart() <= GetEnd() ? true : false; }

	integer GetStart();
	integer GetEnd();
	
	virtual ~Range() { }
};

template<typename T> ValArray<T> ValArray<T>::SubArray(bool cmp, Range *pRange)
{
	if(!pRange->CheckRange()) 
		throw UnicodeException(_T("Invalid range: begin is greater than end"));
	
	int size = int(this->size());
	int min = int(pRange->GetStart()); min = (min==INT_MIN) ? 0      : min ;
	int max = int(pRange->GetEnd());   max = (max==INT_MAX) ? size-1 : max ;
	
	if(min<0 || min>size-1 || max<0 || max>size-1)
		throw UnicodeException(_T("Invalid range"));
			
	ValArray vec;

	if(cmp) vec.resize(size - (max-min+1));
	else vec.resize(max-min+1);

	if(cmp)
	{
		int j = 0;
		for(int i=0; i<min; ++i) vec[j++] = this->operator [](i); 
	
		for(int i=max+1; i<size; ++i) vec[j++] = this->operator [](i); 
	}
	else
	{
		int j = 0;
		for(int i=min; i<= max; ++i) vec[j++] = this->operator [](i); 
	}

	return vec;
}

inline ValArray<Vector> ValArray<Vector>::SubArray(bool cmp, Range *pRange)
{
	if(!pRange->CheckRange()) 
		throw UnicodeException(_T("Invalid range: begin is greater than end"));
	
	int size = int(this->size());
	int min = int(pRange->GetStart()); min = (min==INT_MIN) ? 0      : min ;
	int max = int(pRange->GetEnd());   max = (max==INT_MAX) ? size-1 : max ;
	
	if(min<0 || min>size-1 || max<0 || max>size-1)
		throw UnicodeException(_T("Invalid range"));
			
	ValArray<Vector> vec;

	if(cmp) vec.resize(size - (max-min+1));
	else vec.resize(max-min+1);

	if(cmp)
	{
		int j = 0;
		for(int i=0; i<min; ++i) vec[j++] = this->operator [](i); 
		for(int i=max+1; i<size; ++i) vec[j++] = this->operator [](i);
	}
	else
	{
		int j = 0;
		for(int i=min; i<= max; ++i) vec[j++] = this->operator [](i);
	}

	return vec;
}

template<typename T> class Array1D: public Array
{

protected:
	
	PtrStack<typename VctrType<T>::Vec1D> theStack;

	const Array1D& get_array1d() const { return *this; }
	
	void copy(const Array1D& ary)
	{
		if(theStack.empty()) this->Push();
		
		typename VctrType<T>::Vec1D *pThis = theStack.top();
		typename const VctrType<T>::Vec1D *pThat = ary.theStack.top();
		
		*pThis = *pThat;
	}

	void size(int sz)
	{
		if(sz<= 0)
			throw UnicodeException(_T("Array size should be greater than 0.\r\n"));

		if(theStack.empty())
			throw UnicodeException(_T("Array1D stack empty")); 

		theStack.top()->resize(sz);
	}
public:
	
	Array1D(std::wstring vrid=_T(""), bool vstatic=false): 
	 Array(vrid, 1, vstatic),

#ifdef _DEBUG
		 theStack(vrid)
#else
		 theStack()
#endif
	 {  
		 if(vstatic) Push(); 
		 Array::SetEleType<T>();
	 }
	
	Array1D(const Array1D& ary): 
		Array(ary.get_array()) {  this->copy(ary); }

    Array1D& operator= (const Array1D& ary)
	{
		this->copy(ary); return *this;
	}

	typename VctrType<T>::Vec1D* GetTop()
	{
		return theStack.top();
	}

	void Initialize(const T& value)
	{
		VctrType<T>::Vec1D& v = *theStack.top();
		
		if(v.size() == 0)
			throw UnicodeException(_T("Array size is not yet defined.\r\n"));
		v = value;
	}

	void Swap(Array1D* pArray)
	{  
		theStack.swaptop(&pArray->theStack);
		//pArray->theStack.replace(this->theStack.replace(pArray->theStack.top())); 
	}

	virtual void resize(int sz) { this->size(sz); }
	
	virtual int getsize() const
	{
		if(theStack.empty())
			throw UnicodeException(_T("Array1D stack empty"));

		return int(theStack.top()->size());
	}

	T& at(int index) { return theStack.top()->at(index); }

	const T& at(int index) const { return theStack.top()->at(index); }

	virtual void Push() { theStack.push(new VctrType<T>::Vec1D); }
	
	virtual void Pop() { delete theStack.pop(); }

	virtual ~Array1D() 
	{	while(!theStack.empty()) delete theStack.pop(); }

};

inline realnum norm(realnum r) { return fabs(r); }

template<typename T> class Array2D: public Array
{
protected:
	
	PtrStack<Matrix<T> > theStack;
	
	const Array2D& get_array2d() const { return *this; }
	
	void copy(const Array2D& ary)
	{
		if(theStack.empty()) this->Push();
		
		typename VctrType<T>::Vec2D *pThis=theStack.top();
		typename const VctrType<T>::Vec2D *pThat=ary.theStack.top();
		
		*pThis = *pThat;
	}

	void size(int size1, int size2)
	{
		if(size1<=0 || size2<=0)
			throw UnicodeException(_T("Array size should be greater than 0.\r\n"));
		
		if(theStack.empty())
			throw UnicodeException(_T("Array2D stack empty\r\n")); 

		typename VctrType<T>::Vec2D *pVec = theStack.top();
		pVec->resize(size1);
			
		for(int i=0; i<size1; ++i)
		{
			::Sleep(0);
			pVec->operator [](i).resize(size2);
		}
	}

public:
	
	Array2D(std::wstring vrid=_T(""), bool vstatic=false): 
	Array(vrid, 2, vstatic),

#ifdef _DEBUG
		theStack(vrid)
#else
		theStack()
#endif

	{  
		if(vstatic) Push(); 
		Array::SetEleType<T>();
	}

	Array2D& operator = (const Array2D& ary)
	{ this->copy(ary); return *this; }

	Array2D(const Array2D& ary):
		Array(ary.get_array()) { this->copy(ary); }

	typename VctrType<T>::Vec2D* GetTop()
	{
		return theStack.top();
	}

	typename Matrix<T>* GetTopMatrix()
	{
		return theStack.top();
	}

	void Initialize(const T& value)
	{
		typename VctrType<T>::Vec2D& vec = *theStack.top();
		int size = vec.size();
		
		if(size == 0)
			throw UnicodeException(_T("Array size is not yet defined\r\n"));
		
		for(int i=0; i<size; ++i) vec[i] = value;
	}

	void Swap(Array2D* pArray)
	{  pArray->theStack.replace(this->theStack.replace(pArray->theStack.top())); }

	virtual void resize(int size1, int size2)
	{ this->size(size1, size2); }

	virtual void getsize(int& size1, int& size2) 
	{
		if(theStack.empty())
			throw UnicodeException(_T("Array2D stack empty")); 

		typename VctrType<T>::Vec2D *pAry = theStack.top();
		
		size1 = size2 = 0;

		size1 = int(pAry->size());
		if(size1>0) size2 = int(pAry->operator [](0).size());
	}

	typename VctrType<T>::Vec1D& at(int index)
	{	return theStack.top()->at(index); }
	
	typename const VctrType<T>::Vec1D& at(int index) const
	{	return theStack.top()->at(index); }

	T& at(int index1, int index2)
	{	return theStack.top()->at(index1).at(index2); }

	const T& at(int index1, int index2) const
	{	return theStack.top()->at(index1).at(index2); }

	virtual void Push() { theStack.push(new Matrix<T>()); }

	virtual void Pop() { delete theStack.pop(); }
	
	virtual ~Array2D() 
	{	while(!theStack.empty()) delete theStack.pop(); }
};

template<typename T> class Array3D: public Array
{

protected:
	
	PtrStack<Matrix3D<T> > theStack;

	const Array3D& get_array3d() const { return *this; }
	void copy(const Array3D& ary)
	{
		if(theStack.empty()) this->Push();

		typename VctrType<T>::Vec3D *pThis=theStack.top();
		typename const VctrType<T>::Vec3D *pThat=ary.theStack.top();
		
		*pThis = *pThat;
	}

	void size(int size1, int size2, int size3)
	{
		if(theStack.empty())
			throw UnicodeException(_T("Array3D stack empty")); 

		if(size1<=0 || size2<=0 || size3<=0)
			throw UnicodeException(_T("Array size should be greater than 0."));
		
		typename VctrType<T>::Vec3D *pVec = theStack.top();
		
		pVec->resize(size1);
		for(int i=0; i<size1; ++i)
		{
			::Sleep(0);
			pVec->operator[](i).resize(size2);
			for(int j=0; j<size2; ++j)
			{
				::Sleep(0);
				pVec->operator[](i)[j].resize(size3);
			}
		}
	}

public:
	Array3D(std::wstring vrid=_T(""), bool vstatic=false): 
	 Array(vrid, 3, vstatic),
#ifdef _DEBUG
		theStack(vrid) 
#else
		theStack()
#endif
	{  
		if(vstatic) Push(); 
		Array::SetEleType<T>();
	}

	 Array3D(const Array3D& ary): 
		Array(ary.get_array()) { this->copy(ary); }

	Array3D& operator = (const Array3D& ary)
	{
		this->copy(ary); return *this;
	}

	typename VctrType<T>::Vec3D* GetTop()
	{
		return theStack.top();
	}

	typename Matrix3D<T>* GetTopMatrix()
	{
		return theStack.top();
	}

	void Initialize(const T& value)
	{
		typename VctrType<T>::Vec3D& vec = *theStack.top();

		int size1, size2, size3; 
		this->getsize(size1, size2, size3);

		if(size1 == 0) 
			throw UnicodeException(_T("Array size is not yet defined.\r\n"));
		
		for(int i=0; i<size1; ++i) 
			for(int j=0; j<size2; ++j)
				vec[i][j] = value;
	}

	void Swap(Array3D* pArray)
	{  pArray->theStack.replace(this->theStack.replace(pArray->theStack.top())); }
	
	virtual void resize(int size1, int size2, int size3)
	{ this->size(size1, size2, size3); }

	virtual void getsize(int& size1, int& size2, int& size3) 
	{
		if(theStack.empty())
			throw UnicodeException(_T("Array3D stack empty")); 

		typename VctrType<T>::Vec3D *pAry = theStack.top();
		
		size1 = size2 = size3 = 0;
		
		size1 = int(pAry->size());
		if(size1>0)
		{
			size2 = int(pAry->operator[](0).size());
			if(size2>0)
				size3 = int(pAry->operator[](0)[0].size());
		}
	}

	typename VctrType<T>::Vec2D& at(int index)
	{	return theStack.top()->at(index); }

	typename const VctrType<T>::Vec2D& at(int index) const 
	{	return theStack.top()->at(index); }

	T& at(int index1, int index2, int index3)
	{	return theStack.top()->at(index1).at(index2).at(index3); }

	const T& at(int index1, int index2, int index3) const
	{	return theStack.top()->at(index1).at(index2).at(index3); }

	virtual void Push() { theStack.push(new Matrix3D<T> ); }

	virtual void Pop() { delete theStack.pop(); }

	virtual ~Array3D() 
	{	while(!theStack.empty())  delete theStack.pop(); }
};


template<typename T> class Array4D: public Array
{
protected:
	
	PtrStack<typename VctrType<T>::Vec4D> theStack;

	const Array4D& get_array4d() const { return *this; }
	void copy(const Array4D& ary)
	{
		if(theStack.empty()) this->Push();
		
		typename VctrType<T>::Vec4D* pThis=theStack.top();
		typename const VctrType<T>::Vec4D* pThat=ary.theStack.top();
		
		*pThis = *pThat;
	}

	void size(int size1, int size2, int size3, int size4)
	{
		if(theStack.empty())
			throw UnicodeException(_T("Array3D stack empty")); 

		if(size1<= 0 || size2<=0 || size3<=0 || size4<=0)
			throw UnicodeException(_T("Array size should be greater than 0."));
		
		typename VctrType<T>::Vec4D *pVec = theStack.top();
		
		pVec->resize(size1);
		for(int i=0; i<size1; ++i)
		{
			::Sleep(0);
			pVec->operator[](i).resize(size2);
			for(int j=0; j<size2; ++j)
			{
				::Sleep(0);
				pVec->operator[](i)[j].resize(size3);
				for(int k=0; k<size3; ++k)
				{
					::Sleep(0);
					pVec->operator[](i)[j][k].resize(size4);
				}
			}
		}
	}

public:
	Array4D(std::wstring vrid=_T(""), bool vstatic=false): 
	 Array(vrid, 4, vstatic),

#ifdef _DEBUG
		 theStack(vrid)
#else
		 theStack()
#endif
	{  
		if(vstatic) Push(); 
		Array::SetEleType<T>();
	}

	 Array4D(const Array4D& ary): 
		Array(ary.get_array()) { this->copy(ary); }

	Array4D& operator = (const Array4D& ary)
	{
		this->copy(ary); return *this;
	}

	typename VctrType<T>::Vec4D* GetTop()
	{
		return theStack.top();
	}
	
	void Initialize(const T& value)
	{
		typename VctrType<T>::Vec4D& vec = *theStack.top();

		int size1, size2, size3, size4; 
		this->getsize(size1, size2, size3, size4);

		if(size1 == 0) 
			throw UnicodeException(_T("Array size is not yet defined"));
		
		for(int i=0; i<size1; ++i) 
			for(int j=0; j<size2; ++j)
				for(int k=0; k<size3; ++k)
					vec[i][j][k] = value;
	}

	void Swap(Array4D* pArray)
	{  pArray->theStack.replace(this->theStack.replace(pArray->theStack.top())); }

	virtual void resize(int size1, int size2, int size3, int size4)
	{	this->size(size1, size2, size3, size4); }

	virtual void getsize(int& size1, int& size2, int& size3, int& size4) 
	{
		if(theStack.empty())
			throw UnicodeException(_T("Array3D stack empty")); 

		VctrType<T>::Vec4D *pAry = theStack.top();

		size1 = size2 = size3 = size4 = 0;
		
		size1 = int(pAry->size());
		if(size1>0)
		{
			size2 = int(pAry->operator[](0).size());
			if(size2>0)
			{
				size3 = int(pAry->operator[](0)[0].size());
				if(size3>0)
					size4 =int(pAry->operator[](0)[0][0].size());
			}
		}
	}

	typename VctrType<T>::Vec3D& at(int index)
	{	return theStack.top()->at(index); }

	typename const VctrType<T>::Vec3D& at(int index) const
	{	return theStack.top()->at(index); }

	T& at(int index1, int index2, int index3, int index4)
	{ return theStack.top()->at(index1).at(index2).at(index3).at(index4); }

	const T& at(int index1, int index2, int index3, int index4) const
	{	return theStack.top()->at(index1).at(index2).at(index3).at(index4); }

	virtual void Push() { theStack.push(new typename VctrType<T>::Vec4D); }

	virtual void Pop() { delete theStack.pop(); }

	virtual ~Array4D() 
	{	while(!theStack.empty())  delete theStack.pop(); }
};


template<typename T> class Array5D: public Array
{
protected:
	
	PtrStack<typename VctrType<T>::Vec5D> theStack;

	const Array5D& get_array5d() const { return *this; }
	void copy(const Array5D& ary)
	{
		if(theStack.empty()) this->Push();
		
		typename VctrType<T>::Vec5D* pThis=theStack.top();
		typename const VctrType<T>::Vec5D* pThat=ary.theStack.top();
		
		*pThis = *pThat;
	}

	void size(int size1, int size2, int size3, int size4, int size5)
	{
		if(theStack.empty())
			throw UnicodeException(_T("Array5D stack empty")); 

		if(size1<= 0 || size2<=0 || size3<=0 || size4<=0 || size5<=0)
			throw UnicodeException(_T("Array size should be greater than 0."));
		
		typename VctrType<T>::Vec5D *pVec = theStack.top();
		
		pVec->resize(size1);
		for(int i=0; i<size1; ++i)
		{
			::Sleep(0);
			pVec->operator[](i).resize(size2);
			for(int j=0; j<size2; ++j)
			{
				::Sleep(0);
				pVec->operator[](i)[j].resize(size3);
				for(int k=0; k<size3; ++k)
				{
					::Sleep(0);
					pVec->operator[](i)[j][k].resize(size4);
					for(int l=0; l<size4; ++l)
					{
						::Sleep(0);
						pVec->operator[](i)[j][k][l].resize(size5);
					}
				}
			}
		}
	}

public:
	Array5D(std::wstring vrid=_T(""), bool vstatic=false): 
	 Array(vrid, 5, vstatic),

#ifdef _DEBUG
		theStack(vrid)
#else
		 theStack()
#endif

	{  
		if(vstatic) Push(); 
		Array::SetEleType<T>();
	}

	 Array5D(const Array5D& ary): 
		Array(ary.get_array()) { this->copy(ary); }

	Array5D& operator = (const Array5D& ary)
	{
		this->copy(ary); return *this;
	}

	typename VctrType<T>::Vec5D* GetTop()
	{
		return theStack.top();
	}

	void Initialize(const T& value)
	{
		typename VctrType<T>::Vec5D& vec = *theStack.top();

		int size1, size2, size3, size4, size5; 
		this->getsize(size1, size2, size3, size4, size5);
		
		if(size1 == 0) 
			throw UnicodeException(_T("Array size is not yet defined"));
		
		for(int i=0; i<size1; ++i) 
			for(int j=0; j<size2; ++j)
				for(int k=0; k<size3; ++k)
					for(int l=0; l<size4; ++l)
						vec[i][j][k][l] = value;
	}

	void Swap(Array5D* pArray)
	{  pArray->theStack.replace(this->theStack.replace(pArray->theStack.top())); }

	virtual void resize(int size1, int size2, int size3, int size4, int size5)
	{	this->size(size1, size2, size3, size4, size5); }

	virtual void getsize(int& size1, int& size2, int& size3, int& size4, int& size5) 
	{
		if(theStack.empty())
			throw UnicodeException(_T("Array3D stack empty")); 

		VctrType<T>::Vec5D *pAry = theStack.top();
		
		size1 = size2 = size3 = size4 = size5;

		size1 = int(pAry->size());
		if(size1>0)
		{
			size2 = int(pAry->operator[](0).size());
			if(size2>0)
			{
				size3 = int(pAry->operator[](0)[0].size());
				if(size3>0)
				{
					size4 =int(pAry->operator[](0)[0][0].size());
					if(size4>0)
						size5 = int(pAry->operator[](0)[0][0][0].size());
				}

			}
		}
	}

	typename VctrType<T>::Vec4D& at(int index)
	{	return theStack.top()->at(index); }

	typename const VctrType<T>::Vec4D& at(int index) const
	{	return theStack.top()->at(index); }

	T& at(int index1, int index2, int index3, int index4, int index5)
	{	return theStack.top()->at(index1).at(index2).at(index3).at(index4).at(index5); }

	const T& at(int index1, int index2, int index3, int index4, int index5) const
	{	return theStack.top()->at(index1).at(index2).at(index3).at(index4).at(index5); }

	virtual void Push() { theStack.push(new typename VctrType<T>::Vec5D); }

	virtual void Pop()  { delete theStack.pop(); }

	virtual ~Array5D() 
	{	while(!theStack.empty())  delete theStack.pop(); }
};

template<typename T> class SqrArray
{
protected:
	T* ptr; 

	void copy(const SqrArray& A)
	{
		if(size!=A.size || ptr==NULL)
		{
			if(ptr) delete[] ptr;
			size = A.size; ptr = new T[size*size];
		}

		for(int i=0; i<size; ++i)
			for(int j=0; j<size; ++j)
				*(ptr + size * i + j) = A(i,j);
	}
		
public:
	int size;

	SqrArray(int count): size(count)
	{
		if(size<=0) ptr = NULL;
		else  ptr = new T[size*size];
	}

	SqrArray(const typename VctrType<T>::Vec2D& vec)
	{
		int rows, cols;
		GetVecSize<T>(rows, cols, vec);

		if(rows<=cols && rows>1)
		{
			size = rows;
			ptr = new T[size*size];

			for(int i=0; i<size; ++i)
				for(int j=0; j<size; ++j)
					*(ptr + size * i + j) = vec[i][j];
		}
		else
		{
			ptr=NULL; size=0;
			throw UnicodeException(_T("Invalid matrix in the SqrArray contructor\r\n"));
		}
	}

	SqrArray(const SqrArray& A) { ptr=NULL; size=0; copy(A); }
	SqrArray& operator=(const SqrArray& A) { copy(A); return *this; }

	void Resize(int count)
	{
		if(ptr) delete[] ptr;
		size = count;
		if(size>0) ptr = new T[size*size];
	}

	T& operator () (int row, int column) 
	{
		return *(ptr + size * row + column); 
	}

	T operator () (int row, int column) const
	{
		return *(ptr + size * row + column); 
	}

	void Complement(SqrArray& A, int row, int col) const
	{
		for(int i=0, i1=0; i<size; ++i)
		{
			if(i==row) continue;

			for(int j=0, j1=0; j<size; ++j)
			{
				if(j==col) continue;
				A(i1, j1)= *(ptr + size * i + j); j1++; 
			}

			i1++;
		}
	}

	friend std::ostream& operator<<(std::ostream& os, const SqrArray& A);

	~SqrArray()
	{
		if(ptr) delete[] ptr;
	}
};

template<typename T>
std::ostream& operator<<(std::ostream& os, const SqrArray<T>& A)
{
	for(int i=0; i<A.size; ++i)
	{
		for(int j=0; j<A.size; ++j)
		{
			os<<A(i, j)<<_T(" ");
		}
		os<<std::endl;
	}

	return os;
}

template<typename T> T Determinant(const SqrArray<T>& A)
{
	int rowzeros=0, colzeros=0;

	for(int k=0; k<A.size; ++k)
	{
		if(A(k, 0)==T()) ++rowzeros;
		if(A(0, k)==T()) ++colzeros;
	}
	
	if((rowzeros==A.size)||(colzeros==A.size)) return T();
	
	if(A.size<3)
		return A(0, 0)*A(1, 1) - A(0, 1)*A(1, 0);
	else
	{
		SqrArray<T> B(A.size-1); 
		T total=T(), zero=T(), coef;

		if(rowzeros<colzeros)
		{
			for(int j=0; j<A.size; ++j)
			{
				coef = A(0, j);
				if(coef == zero) continue;

				A.Complement(B, 0, j); 

				if(j%2==0)
					total += coef * Determinant(B);
				else
					total -= coef * Determinant(B);
			}
			return total;
		}
		else
		{
			for(int i=0; i<A.size; ++i)
			{
				coef = A(i, 0);
				if(coef == zero) continue;

				A.Complement(B, i, 0); 

				if(i%2==0)
					total += coef * Determinant(B);
				else
					total -= coef * Determinant(B);
			}
			return total;
		}
	}
}

template<typename T>
void Add_MN(Array1D<T>& M, Array1D<T>& N, Array1D<T>& R)
{
	int min = std::min<int>(M.size(), N.size());

	if(M.size()>N.size())
	{
		R = M;	
		for(int i = 0; i<min; ++i) R[i] += N[i];
	}
	else
	{
		R = N;
		for(int i = 0; i<min; ++i) R[i] += M[i];
	}
}

template<typename T>
void Sub_MN(Array1D<T>& M, Array1D<T>& N, Array1D<T>& R)
{
	int min = std::min<int>(M.size(), N.size());
	
	if(M.size()>=N.size())
	{
		R = M;
		for(int i = 0; i<min; ++i) R[i] -= N[i];
	}
	else
	{
		R.size(N.size());
		for(int i = 0; i<N.size(); ++i) R[i] = -N[i];
		for(int i = 0; i<min; ++i) R[i] += M[i];
	}
}

template<typename T>
void Mul_SM(T& scalar, Array1D<T>& M, Array1D<T>& R)
{
	for(int i = 0; i<M.size(); ++i)
		R[i] = scalar*M[i];
}

template<typename T>
void Div_MS(T& scalar, Array1D<T>& M, Array1D<T>& R)
{
	for(int i = 0; i<M.size(); ++i)
		R[i] = M[i]/scalar;
}

template<typename T>
void Mul_MN(Array1D<T>&M, Array1D<T>& N, Array1D<T>& R)
{
	R.size(M.size()+N.size()-1);
	for(int m=0; m<M.size(); ++m)
		for(int n=0; n<N.size(); ++n)
			R[m+n] += M[m]*N[n];
}
template<typename T> // M/N
void Div_mn(Array1D<T>&M, Array1D<T>& N, Array1D<T>& Q, Array1D<T>& R)
{
	if(M.size()<N.size()) return;
	else
	{
		Array1D<realnum> MM = M;

		T ab = MM[MM.size()-1] / N[N.size()-1];
		int mn = MM.size()-N.size();
		Q[mn] = ab;

		for(int i = 0; i<MM.size()-1; ++i)
			R[i] = MM[i];

		for(int i= 0; i<N.size()-1; ++i)
			R[i+mn] -= ab*N[i];
		
		if(R.size()<N.size()) return;
		
		MM = R;	R.size(MM.size()-1);
		Div_mn(MM, N, Q, R);
	}
}

template<typename T> // M/N
void Div_MN(Array1D<T>&M, Array1D<T>& N, Array1D<T>& Q, Array1D<T>& R)
{
	if(M.size()<N.size()) return;
	
	Q.size(M.size() - N.size() + 1);
	R.size(M.size() - 1);

	Div_mn(M, N, Q, R);
}

template<typename T> bool operator == (Array1D<T>& M, T v)
{
	for(int i=0; i<M.size(); ++i)
		if(M[i] != v) return false;
	return true;
}

template<typename T> bool operator != (Array1D<T>& M, T v)
{
	return !operator==(M, v);
}

// Greatest Common Divisor
template<typename T>
void Gcd(Array1D<T> M, Array1D<T> N, Array1D<T>& G)
{ 
	// declaration of remainder
	Array1D<T> Q, R;
	
	while (N != 0.0) 
	{ 	
		Div_MN(M, N, Q, R); M = N; N = R; 
	}
	G = M;
}

template<typename T> class ArrayU1D: public Array
{
protected:
	
	typename VctrType<T>::Vec1D *vec;

	void copy(const ArrayU1D& ary)
	{
		if(vec)
		{
			if(this->vec->size() != ary.vec->size())
				this->vec->resize(ary.vec->size());
			
			*this->vec = *ary.vec;
		}
		else vec = new VctrType<T>::Vec1D(*ary.vec);
	}

public:
	
	ArrayU1D(std::wstring vrid=_T("")): vec(NULL), Array(vrid, 1)
	 {  
		 vec = new typename VctrType<T>::Vec1D();
		 Array::SetEleType<T>();
	 }
	
	ArrayU1D(const ArrayU1D& ary): 
		Array(ary.get_array()) {  vec = NULL; this->copy(ary); }

    ArrayU1D& operator= (const ArrayU1D& ary)
	{
		this->copy(ary); return *this;
	}

	typename VctrType<T>::Vec1D* GetVec() { return vec; }

	void Initialize(const T& value)
	{
		if(vec->size() == 0)
			throw UnicodeException(_T("Array size is not yet defined.\r\n"));
		*vec = value;
	}

	void Swap(ArrayU1D* pArray)
	{  
		typename VctrType<T>::Vec1D *ptr = vec;
		vec = pArray->vec; pArray->vec = ptr;
	}

	virtual void resize(int size) 
	{ 
		if(size<=0)
			throw UnicodeException(_T("Array size should be greater than zero(0).\r\n"));
		vec->resize(size);
	}
	
	virtual int getsize() const { return vec->size(); }

	T& at(int index) { return vec->at(index); }
	const T& at(int index) const { return vec->at(index); }
	
	virtual ~ArrayU1D() {	if(vec) delete vec;  }
};

template<typename T> class ArrayU2D: public Array
{
protected:
	
	typename VctrType<T>::Vec2D *vec;

	void copy(const ArrayU2D& ary)
	{
		if(vec)
		{
			if(!IsCongruent<T>(*this->vec, *ary.vec))
			{
				int size1, size2; 
				GetVecSize<T>(size1, size2, *ary.vec);
				SetVecSize<T>(size1, size2, *this->vec);
			}
			*this->vec = *ary.vec;
		}
		else vec = new VctrType<T>::Vec2D(*ary.vec);
	}

public:
	
	ArrayU2D(std::wstring vrid=_T("")): vec(NULL), Array(vrid, 2)
	 {  
		 vec = new typename VctrType<T>::Vec2D();
		 Array::SetEleType<T>();
	 }
	
	ArrayU2D(const ArrayU2D& ary): 
		Array(ary.get_array()) {  vec = NULL; this->copy(ary); }

    ArrayU2D& operator= (const ArrayU2D& ary)
	{
		this->copy(ary); return *this;
	}

	typename VctrType<T>::Vec2D* GetVec() { return vec; }

	void Initialize(const T& value)
	{
		if(vec->size() == 0)
			throw UnicodeException(_T("Array size is not yet defined.\r\n"));
		*vec = value;
	}

	void Swap(ArrayU2D* pArray)
	{  
		typename VctrType<T>::Vec2D *ptr = vec;
		vec = pArray->vec; pArray->vec = ptr;
	}

	virtual void resize(int size1, int size2) 
	{ 
		if(size1<=0 || size2<0)
			throw UnicodeException(_T("Array size should be greater than zero(0).\r\n"));
		SetVecSize<T>(size1, size2, *this->vec);
	}
	
	virtual void getsize(int& size1, int& size2)
	{ GetVecSize<T>(size1, size2, *this->vec); }

	T& at(int index1, int index2) 
		{ return vec->at(index1).at(index2); }
	const T& at(int index1, int index2) const 
		{ return vec->at(index1).at(index2); }
	
	virtual ~ArrayU2D() {	if(vec) delete vec;  }
};

template<typename T> class ArrayU3D: public Array
{
protected:
	
	typename VctrType<T>::Vec3D *vec;

	void copy(const ArrayU3D& ary)
	{
		if(vec)
		{
			if(!IsCongruent<T>(*this->vec, *ary.vec))
			{
				int size1, size2, size3; 
				GetVecSize<T>(size1, size2, size3, *ary.vec);
				SetVecSize<T>(size1, size2, size3, *this->vec);
			}
			*this->vec = *ary.vec;
		}
		else vec = new VctrType<T>::Vec3D(*ary.vec);
	}

public:
	
	ArrayU3D(std::wstring vrid=_T("")): vec(NULL), Array(vrid, 3)
	 {  
		 vec = new typename VctrType<T>::Vec3D();
		 Array::SetEleType<T>();
	 }
	
	ArrayU3D(const ArrayU3D& ary): 
		Array(ary.get_array()) {  vec = NULL; this->copy(ary); }

    ArrayU3D& operator= (const ArrayU3D& ary)
	{
		this->copy(ary); return *this;
	}

	typename VctrType<T>::Vec3D* GetVec() { return vec; }

	void Initialize(const T& value)
	{
		if(vec->size() == 0)
			throw UnicodeException(_T("Array size is not yet defined.\r\n"));
		*vec = value;
	}

	void Swap(ArrayU3D* pArray)
	{  
		typename VctrType<T>::Vec3D *ptr = vec;
		vec = pArray->vec; pArray->vec = ptr;
	}

	virtual void resize(int size1, int size2, int size3) 
	{ 
		if(size1<=0 || size2<=0 || size3<=0)
			throw UnicodeException(_T("Array size should be greater than zero(0).\r\n"));
		SetVecSize<T>(size1, size2, size3, *this->vec);
	}
	
	virtual void getsize(int& size1, int& size2, int& size3)
	{ GetVecSize<T>(size1, size2, size3, *this->vec); }

	T& at(int index1, int index2, int index3) 
		{ return vec->at(index1).at(index2).at(index3); }
	const T& at(int index1, int index2, int index3) const 
		{ return vec->at(index1).at(index2).at(index3); }
	
	virtual ~ArrayU3D() {	if(vec) delete vec;  }
};

template<typename T> class ArrayU4D: public Array
{
protected:
	
	typename VctrType<T>::Vec4D *vec;

	void copy(const ArrayU4D& ary)
	{
		if(vec)
		{
			if(!IsCongruent<T>(*this->vec, *ary.vec))
			{
				int size1, size2, size3, size4; 
				GetVecSize<T>(size1, size2, size3, size4, *ary.vec);
				SetVecSize<T>(size1, size2, size3, size4, *this->vec);
			}
			*this->vec = *ary.vec;
		}
		else vec = new VctrType<T>::Vec4D(*ary.vec);
	}

public:
	
	ArrayU4D(std::wstring vrid=_T("")): vec(NULL), Array(vrid, 4)
	 {  
		 vec = new typename VctrType<T>::Vec4D();
		 Array::SetEleType<T>();
	 }
	
	ArrayU4D(const ArrayU4D& ary): 
		Array(ary.get_array()) {  vec = NULL; this->copy(ary); }

    ArrayU4D& operator= (const ArrayU4D& ary)
	{
		this->copy(ary); return *this;
	}

	typename VctrType<T>::Vec4D* GetVec() { return vec; }

	void Initialize(const T& value)
	{
		if(vec->size() == 0)
			throw UnicodeException(_T("Array size is not yet defined.\r\n"));
		*vec = value;
	}

	void Swap(ArrayU4D* pArray)
	{  
		typename VctrType<T>::Vec4D *ptr = vec;
		vec = pArray->vec; pArray->vec = ptr;
	}

	virtual void resize(int size1, int size2, int size3, int size4) 
	{ 
		if(size1<=0 || size2<=0 || size3<=0 || size4<=0)
			throw UnicodeException(_T("Array size should be greater than zero(0).\r\n"));
		SetVecSize<T>(size1, size2, size3, size4, *this->vec);
	}
	
	virtual void getsize(int& size1, int& size2, int& size3, int& size4)
	{ GetVecSize<T>(size1, size2, size3, size4, *this->vec); }

	T& at(int index1, int index2, int index3, int index4) 
		{ return vec->at(index1).at(index2).at(index3).at(index4); }
	const T& at(int index1, int index2, int index3, int index4) const 
		{ return vec->at(index1).at(index2).at(index3).at(index4); }
	
	virtual ~ArrayU4D() {	if(vec) delete vec;  }
};

template<typename T> class ArrayU5D: public Array
{
protected:
	
	typename VctrType<T>::Vec5D *vec;

	void copy(const ArrayU5D& ary)
	{
		if(vec)
		{
			if(!IsCongruent<T>(*this->vec, *ary.vec))
			{
				int size1, size2, size3, size4, size5; 
				GetVecSize<T>(size1, size2, size3, size4, size5, *ary.vec);
				SetVecSize<T>(size1, size2, size3, size4, size5, *this->vec);
			}
			*this->vec = *ary.vec;
		}
		else vec = new VctrType<T>::Vec5D(*ary.vec);
	}

public:
	
	ArrayU5D(std::wstring vrid=_T("")): vec(NULL), Array(vrid, 5)
	 {  
		 vec = new typename VctrType<T>::Vec5D();
		 Array::SetEleType<T>();
	 }
	
	ArrayU5D(const ArrayU5D& ary): 
		Array(ary.get_array()) {  vec = NULL; this->copy(ary); }

    ArrayU5D& operator= (const ArrayU5D& ary)
	{
		this->copy(ary); return *this;
	}

	typename VctrType<T>::Vec5D* GetVec() { return vec; }

	void Initialize(const T& value)
	{
		if(vec->size() == 0)
			throw UnicodeException(_T("Array size is not yet defined.\r\n"));
		*vec = value;
	}

	void Swap(ArrayU5D* pArray)
	{  
		typename VctrType<T>::Vec5D *ptr = vec;
		vec = pArray->vec; pArray->vec = ptr;
	}

	virtual void resize(int size1, int size2, int size3, int size4, int size5) 
	{ 
		if(size1<=0 || size2<=0 || size3<=0 || size4<=0 || size5<=0)
			throw UnicodeException(_T("Array size should be greater than zero(0).\r\n"));
		SetVecSize<T>(size1, size2, size3, size4, size5, *this->vec);
	}
	
	virtual void getsize(int& size1, int& size2, int& size3, int& size4, int& size5)
	{ GetVecSize<T>(size1, size2, size3, size4, size5, *this->vec); }

	T& at(int index1, int index2, int index3, int index4, int index5) 
		{ return vec->at(index1).at(index2).at(index3).at(index4).at(index5); }
	const T& at(int index1, int index2, int index3, int index4, int index5) const 
		{ return vec->at(index1).at(index2).at(index3).at(index4).at(index5); }
	
	virtual ~ArrayU5D() { if(vec) delete vec;  }
};

typedef ArrayU1D<integer> IntgAry1D;
typedef ArrayU1D<realnum> RealAry1D;
typedef ArrayU1D<Complex> CplxAry1D;
typedef ArrayU1D<Vector>  VctrAry1D;

typedef ArrayU2D<integer> IntgAry2D;
typedef ArrayU2D<realnum> RealAry2D;
typedef ArrayU2D<Complex> CplxAry2D;
typedef ArrayU2D<Vector>  VctrAry2D;

typedef ArrayU3D<integer> IntgAry3D;
typedef ArrayU3D<realnum> RealAry3D;
typedef ArrayU3D<Complex> CplxAry3D;
typedef ArrayU3D<Vector>  VctrAry3D;

typedef ArrayU4D<integer> IntgAry4D;
typedef ArrayU4D<realnum> RealAry4D;
typedef ArrayU4D<Complex> CplxAry4D;
typedef ArrayU4D<Vector>  VctrAry4D;

typedef ArrayU5D<integer> IntgAry5D;
typedef ArrayU5D<realnum> RealAry5D;
typedef ArrayU5D<Complex> CplxAry5D;
typedef ArrayU5D<Vector>  VctrAry5D;
