#pragma once

#include <new>
#include <deque>
#include <iostream>
#include <utility>
#include <hash_map>
#include <string>
#include <valarray>
#include <limits>
#include <sstream>
#include <fstream>
#include <cfloat>
#include <vector>
#include <cstdio>
#include <set>
#include <kch_autodelete.h>
#include <kch_afx.h>
#include <kch_complex.h>
#include <kch_tmpvec.h>
#include <kch_exceptions.h>

#ifndef UNICODE
#define UNICODE
#endif

#ifndef _UNICODE
#define _UNICODE
#endif

#ifdef max
#undef max
#endif

#ifdef min
#undef min
#endif

#ifndef BUF_LENGTH
#define BUF_LENGTH 512
#endif

#ifndef MAX_PATH
#define MAX_PATH 260
#endif

#ifndef MAX_MODE
#define MAX_MODE 80
#endif

typedef unsigned int TypeCntr;

typedef __int64 integer;
typedef double  realnum;
typedef std::wstring string;
typedef std::wstringstream stream;

enum GCType { GarColGbl=0, GarColTmp, GarColStd };

enum FunctionState { FS_NewFunction, FS_Declared, FS_Implemented };

union UniChar
{
	wchar_t wchar;	
	struct { char low, high; } byte;
	short unsigned code; 
};

enum ProgStatus { PS_IdleTime=0, PS_StartUpTime, PS_CompileTime, PS_RunTime, PS_InputTime };
extern ProgStatus gProgStatus;

enum ExceptionType { ET_NoException=0, ET_CompileTime, ET_RunTime, ET_DivisionByZero };

enum DataType { TypeNull, TypeVoid, TypeBool, TypeFile, TypeAptr,  TypePtr,
				TypeIntg, TypeReal, TypeCplx, TypeVctr, TypeArray, TypeUary, 
				TypeUser, TypeUPtr,
				
				TypeIntgL, TypeRealL, TypeCplxL, TypeVctrL, TypeStrnL,
				
				TypeBPtr, TypeSPtr, TypeIPtr, TypeRPtr, TypeCPtr, TypeVPtr,

				TypeStrn, TypeStrnP1, TypeStrnP2, TypeStrnP3, 
				TypeStrm, TypeStrmP1, TypeStrmP2, TypeStrmP3, 

				TypePi1D, TypePi2D, TypePi3D, TypePi4D, TypePi5D,
				TypePr1D, TypePr2D, TypePr3D, TypePr4D, TypePr5D,
				TypePc1D, TypePc2D, TypePc3D, TypePc4D, TypePc5D,
				TypePv1D, TypePv2D, TypePv3D, TypePv4D, TypePv5D,

				TypeAi1D, TypeAi2D, TypeAi3D, TypeAi4D, TypeAi5D,
				TypeAr1D, TypeAr2D, TypeAr3D, TypeAr4D, TypeAr5D,
				TypeAc1D, TypeAc2D, TypeAc3D, TypeAc4D, TypeAc5D,
				TypeAv1D, TypeAv2D, TypeAv3D, TypeAv4D, TypeAv5D  

				};

enum StmtType { StmtNull, StmtCase, StmtLabel, StmtLoop, StmtFile, 
				StmtDecl, StmtFunc, StmtAsgn,  StmtExpr, StmtAslt,
				StmtSkip, StmtBreak, StmtRtn,  StmtList, StmtGoto, 
				StmtExit, StmtSele, StmtThrw,  StmtCath, StmtGr1D };

typedef std::pair<DataType, std::wstring> DataPair;
typedef std::vector<DataType> DataTypeVec;
typedef std::vector<DataPair> DataPairVec;

class UserDefinedType; class Data;

typedef std::vector<std::wstring> StrArray;
typedef StrArray::iterator StrArrayItr;
typedef StrArray::const_iterator StrArrayCItr;

typedef std::pair<std::wstring, UserDefinedType*> NameUdtPair;
typedef stdext::hash_map<std::wstring, UserDefinedType*> NameUdtMap;
typedef NameUdtMap::iterator NameUdtMapItr;
typedef NameUdtMap::const_iterator NameUdtMapCItr;

typedef std::pair<std::wstring, Data*> NameDataPair;
typedef stdext::hash_map<std::wstring, Data*> NameDataMap;
typedef NameDataMap::iterator NameDataMapItr;
typedef NameDataMap::const_iterator NameDataMapCItr;

typedef std::pair<std::wstring, DataType> TypePair;
typedef stdext::hash_map<std::wstring, DataType> TypeMap;
typedef TypeMap::iterator TypeMapItr;
typedef TypeMap::const_iterator TypeMapCItr;

typedef std::pair<DataType, void*> UdtPair;
typedef std::pair<std::wstring, UdtPair>UsrtPair;
typedef stdext::hash_map<std::wstring, UdtPair> UsrtMap;
typedef UsrtMap::iterator UsrtMapItr;
typedef UsrtMap::const_iterator UsrtMapCItr;

typedef std::pair<void*, DataType> MemPair;
typedef stdext::hash_map<void*, DataType> MemMap;
typedef MemMap::iterator MemMapItr;
typedef MemMap::const_iterator MemMapCItr;

enum XpreType { ExprNull, ExprLtrl, ExprVari, ExprBnop, ExprBool };

extern int HandleSyntaxError(UINT rcid, ExceptionType etype=ET_NoException);
extern int HandleSyntaxError(std::wstring msg, ExceptionType etype=ET_NoException);

extern void ReleaseMemory(void *ptr);

#define VWS_GETFULLNAME	virtual std::wstring GetFullName()
class VariableBase
{
public:
	VariableBase() {}
	
	virtual const std::wstring& GetId() const = 0;
	VWS_GETFULLNAME = 0;
	
	virtual ~VariableBase() {}
};

inline integer ToInteger(realnum value)
{
	char buffer[256];
	sprintf_s(buffer, 255, "%15.0lf", value);
	integer rtn; sscanf_s(buffer, "%I64d", &rtn);

	return rtn;
}

inline integer ToInteger(const char* buffer)
{
	char buf[256]; int size = int(strlen(buffer));

	int j=0;
	for(int i=0; i<size; ++i)
	{
		extern char decimal_point;
		extern char thousand_point;
		extern bool fmt_thousand;

		if(buffer[i]==thousand_point) continue;
		
		buf[j++] = buffer[i];
	}

	buf[j]='\0';
	
	integer rtn; sscanf_s(buf, "%I64d", &rtn);

	return rtn;
}

inline realnum ToReal(const char* buffer)
{
	char buf[256]; int size = int(strlen(buffer));

	int j=0;
	for(int i=0; i<size; ++i)
	{
		extern char decimal_point;
		extern char thousand_point;

		if(buffer[i]==thousand_point) continue;
		if(buffer[i]==decimal_point)
		{
			buf[j++] = '.'; continue;
		}

		buf[j++] = buffer[i];
	}

	buf[j]='\0';
	
	realnum rtn; sscanf_s(buf, "%lf", &rtn);

	return rtn;
}

inline realnum ToReal(integer value)
{
	char buffer[256];
	sprintf_s(buffer, 255, "%I64d", value);
	realnum rtn; sscanf_s(buffer, "%If", &rtn);

	return rtn;
}

struct voidtype { voidtype() {}  } ;

template<typename T> T GetMin(T a, T b) { return a < b ? a : b; }

template<typename T> T GetMax(T a, T b) { return a < b ? b : a; }

typedef std::set<realnum> SolSet;
typedef std::set<realnum>::iterator SolSetItr;

class Data; class Statement;

typedef std::set<Data*> DataSet;
typedef DataSet::iterator DataSetItr;

typedef std::deque<Statement*> StmtVector;
typedef StmtVector::iterator StmtVectorItr;

union RealUnion { integer i; realnum  r; };

#define CompareSignBit(a, b) int((a.i^b.i)>>63)

extern realnum RoundReal(realnum value);

extern realnum Magnitude(realnum r);
extern realnum Magnitude(Complex c);

namespace NR 
{
	inline void nrerror(std::wstring error_text)
	// Numerical Recipes standard error handler
	{
		std::wostringstream os;
		os<<_T("Runtime Error: ")<<error_text;
		throw RunTimeException(os.str());
	}
}

typedef double DP;

template<class T>
inline const T SQR(const T a) {return a*a;}

template<class T>
inline const T MAX(const T &a, const T &b)
        {return b > a ? (b) : (a);}

inline float MAX(const double &a, const float &b)
        {return b > a ? (b) : float(a);}

inline float MAX(const float &a, const double &b)
        {return b > a ? float(b) : (a);}

template<class T>
inline const T MIN(const T &a, const T &b)
        {return b < a ? (b) : (a);}

inline float MIN(const double &a, const float &b)
        {return b < a ? (b) : float(a);}

inline float MIN(const float &a, const double &b)
        {return b < a ? float(b) : (a);}

template<class T>
inline const T SIGN(const T &a, const T &b)
	{return b >= 0 ? (a >= 0 ? a : -a) : (a >= 0 ? -a : a);}

inline float SIGN(const float &a, const double &b)
	{return b >= 0 ? (a >= 0 ? a : -a) : (a >= 0 ? -a : a);}

inline float SIGN(const double &a, const float &b)
	{return float(b >= 0 ? (a >= 0 ? a : -a) : (a >= 0 ? -a : a));}

template<class T>
inline void SWAP(T &a, T &b) {T dum=a; a=b; b=dum;}

class Range;

template<typename T>
class ValArray: public std::valarray<T>
{
public:
	ValArray(){ }
	ValArray(const ValArray& va) { *this = va; }
	ValArray(const std::valarray<T>& va): valarray<T>(va) {  }
	virtual ~ValArray(){ std::valarray<T>::~valarray(); } 

	operator std::valarray<T>() const { return *this; }

	T& at(int index)
	{
		if( index < 0 || index >= size())
			throw RunTimeException(_T("Subscript out of range.\r\n"));
		
		return std::valarray<T>::operator[](index);
	}
	
	
	T at(int index) const
	{
		if( index < 0 || index >= size())
			throw RunTimeException(_T("Subscript out of range.\r\n"));
		
		return std::valarray<T>::operator[](index);
	}

	T& operator[] (int index) { return this->at(index); }

	T operator[](int index) const { return this->at(index); }
	
	int nrows() { return int(this->size()); }
	
	ValArray SubArray(bool cmp, Range *pRange);

	ValArray Complement(integer index)
	{
		int size = this->size();
		if(index<0 || index >= size) 
			throw RunTimeException(_T("Index out of range.\r\n"));
		
		ValArray vec(size-1); int j=0;

		for(int i=0; i<int(index); ++i)
			vec[j++] = this->operator[](i);

		for(int i=int(index)+1; i<size; ++i)
			vec[j++] = this->operator[](i);
		
		return vec;
	}
	
	// Zero-based array of n elements
	explicit ValArray(int n): std::valarray<T>(n) { }
	
	//initialize to constant value
	ValArray(const T &a, int n): std::valarray<T>(a, n) { } 
		
	// Initialize to array
	ValArray(const T *a, int n): std::valarray<T>(a, n) { }	
		
	//assign a to every element
	ValArray& operator=(const T &a) 
	{ 
		std::valarray<T> *ptr = dynamic_cast<std::valarray<T>*>(this);
		*ptr = a; return *this; 
	} 
	
	friend ValArray operator/(const T& value, const ValArray& ary)
	{
		int size = ary.size();
		for(int i=0; i<size; ++i)
			if(ary[i] == T()) throw RunTimeException(_T("Division By Zero.\r\n"));
		
		return std::operator/(value, ary);
	}

	friend ValArray operator/(const ValArray& ary, const T& value)
	{
		if(value == T()) throw RunTimeException(_T("Division By Zero.\r\n"));
		
		return std::operator/(ary, value);
	}

	friend ValArray operator/(const ValArray& ary1, const ValArray& ary2)
	{
		if(ary1.size() != ary2.size()) 
			throw RunTimeException(_T("Array size mismatch.\r\n"));

		ValArray v;

		v = std::operator/(ary1, ary2); return v;
	}

	friend ValArray operator + (const ValArray& ary1, const ValArray& ary2)
	{
		if(ary1.size() != ary2.size()) 
			throw RunTimeException(_T("Array size mismatch.\r\n"));

		return std::operator+(ary1, ary2);;
	}

	friend ValArray operator - (const ValArray& ary1, const ValArray& ary2)
	{
		if(ary1.size() != ary2.size()) 
			throw RunTimeException(_T("Array size mismatch.\r\n"));

		return std::operator-(ary1, ary2);;
	}

	friend ValArray operator * (const ValArray& ary1, const ValArray& ary2)
	{
		if(ary1.size() != ary2.size()) 
			throw RunTimeException(_T("Array size mismatch.\r\n"));

		return std::operator*(ary1, ary2);;
	}
		
	int size() const { return int(std::valarray<T>::size()); };
};

template<>
class ValArray<Vector>: public std::valarray<Vector>
{
public:
	ValArray() { }
	explicit ValArray(int size): std::valarray<Vector>(size) {}
	ValArray(const ValArray& va) { *this = va; }
	ValArray(std::valarray<Vector>& va): valarray<Vector>(va) { }

	virtual ~ValArray(){ std::valarray<Vector>::~valarray(); }

	Vector& at(int index)
	{
		if(index<0 || index>=size())
			throw RunTimeException(_T("Subscript out of range.\r\n"));

		return std::valarray<Vector>::operator[](index);
	}

	Vector at(int index) const
	{
		if(index<0 || index>=size())
			throw RunTimeException(_T("Subscript out of range.\r\n"));

		return std::valarray<Vector>::operator[](index);
	}

	Vector& operator[](int index) { return this->at(index); }

	Vector operator[](int index) const { return this->at(index); }

	//assign a to every element
	ValArray& operator=(const Vector &a) 
	{	
		std::valarray<Vector> *ptr = dynamic_cast<std::valarray<Vector>*>(this);
		*ptr = a; return *this; 
	} 

	ValArray SubArray(bool cmp, Range *pRange);

	ValArray Complement(integer index)
	{
		int size = this->size();

		if(index<0 || index >= size) 
			throw RunTimeException(_T("Index out of range.\r\n"));

		ValArray vec(size-1); int j=0;

		for(int i= 0; i<index; ++i)
			vec[j++] = this->operator[](i);

		for(int i=int(index)+1; i<size; ++i)
			vec[j++] = this->operator[](i);
		
		return vec;
	}

	friend ValArray operator/(const ValArray& ary1, const ValArray& ary2)
	{
		throw RunTimeException(_T("Operation not defined.\r\n"));
	}

	friend ValArray operator + (const ValArray& ary1, const ValArray& ary2)
	{
		if(ary1.size() != ary2.size()) 
			throw RunTimeException(_T("Array size mismatch.\r\n"));

		return std::operator+(ary1, ary2);;
	}

	friend ValArray operator - (const ValArray& ary1, const ValArray& ary2)
	{
		if(ary1.size() != ary2.size()) 
			throw RunTimeException(_T("Array size mismatch.\r\n"));

		return std::operator-(ary1, ary2);;
	}

	friend ValArray operator * (const ValArray& ary1, const ValArray& ary2)
	{
		throw RunTimeException(_T("Operation not defined.\r\n"));
	}

	friend void dotproduct(ValArray<realnum>& vec, const ValArray& ary1, const ValArray& ary2)
	{
		int size = ary1.size();
		if(size != ary2.size()) throw RunTimeException(_T("Array size mismatch.\r\n"));
		
		vec.resize(int(size));

		for(int i=0; i<size; ++i) vec[i] = ary1[i] * ary2[i];
	}

	friend ValArray operator % (const ValArray& ary1, const ValArray& ary2)
	{
		if(ary1.size() != ary2.size()) 
			throw RunTimeException(_T("Array size mismatch.\r\n"));

		return std::operator%(ary1, ary2);;
	}
	
	friend ValArray operator *(const Vector&, const ValArray&)
	{
		throw RunTimeException(_T("Operation not defined.\r\n"));
	}

	friend void dotpro(ValArray<realnum>& r, const ValArray& va, const Vector& v)
	{
		int size = va.size(); r.resize(size);
		for(int i=0; i<size; ++i) r[i] = va[i] * v;
		
	}

	friend void dotpro(ValArray<realnum>& r, const Vector& v, const ValArray& va)
	{
		int size = va.size(); r.resize(size);
		for(int i=0; i<size; ++i) r[i] = v * va[i];
	}

	friend ValArray operator %(const Vector& v, const ValArray& va)
	{
		int size = int(va.size()); ValArray valary;
		valary.resize(size);

		for(int i=0; i<size; ++i) valary[i] = v % va[i];

		return valary;
	}

	friend ValArray operator %(const ValArray& va, const Vector& v)
	{
		int size = int(va.size()); ValArray valary;
		valary.resize(size);

		for(int i=0; i<size; ++i) valary[i] = va[i] % v;

		return valary;
	}

	friend ValArray operator +(const Vector& v, const ValArray& va)
	{
		int size = int(va.size()); ValArray valary;
		valary.resize(size);

		for(int i=0; i<size; ++i) valary[i] = v + va[i];

		return valary;
	}

	friend ValArray operator +(const ValArray& va, const Vector& v)
	{
		int size = int(va.size()); ValArray valary;
		valary.resize(size);

		for(int i=0; i<size; ++i) valary[i] = v + va[i];

		return valary;
	}

	friend ValArray operator -(const Vector& v, const ValArray& va)
	{
		int size = int(va.size()); ValArray valary;
		valary.resize(size);

		for(int i=0; i<size; ++i) valary[i] = v - va[i];

		return valary;
	}

	friend ValArray operator -(const ValArray& va, const Vector& v)
	{
		int size = int(va.size()); ValArray valary;
		valary.resize(size);

		for(int i=0; i<size; ++i) valary[i] = va[i] - v;

		return valary;
	}

	friend ValArray operator /(const Vector& v, const ValArray& va)
	{
		throw RunTimeException(_T("Operation not defined.\r\n"));
	}

	friend ValArray operator /(const ValArray& va, const Vector& v)
	{
		throw RunTimeException(_T("Operation not defined.\r\n"));
	}

	friend ValArray operator *(const realnum& value, const ValArray& va)
	{
		int size = int(va.size()); ValArray valary;
		valary.resize(size);

		for(int i=0; i<size; ++i) valary[i] = value * va[i]; 
		
		return valary;
	}

	friend ValArray operator *(const ValArray& va, const realnum& value)
	{
		int size = int(va.size()); ValArray valary;
		valary.resize(size);

		for(int i=0; i<size; ++i) valary[i] = va[i] * value; 

		return valary;
	}

	friend ValArray operator /(double value, const ValArray& va)
	{
		throw RunTimeException(_T("Operation not defined.\r\n"));
	}
	
	friend ValArray operator /(const ValArray& va, double value)
	{
		if(value == 0.0) throw RunTimeException(_T("Division By Zero.\r\n"));

		int size = int(va.size()); ValArray valary;
		valary.resize(size);

		for(int i=0; i<size; ++i) valary[i] = va[i]/ value;

		return valary;
	}

	friend ValArray operator + (double value, const ValArray& va)
	{
		throw RunTimeException(_T("Operation not defined.\r\n"));
	}

	friend ValArray operator + (const ValArray& va, double value)
	{
		throw RunTimeException(_T("Operation not defined.\r\n"));
	}

	friend ValArray operator - (double value, const ValArray& va)
	{
		throw RunTimeException(_T("Operation not defined.\r\n"));
	}

	friend ValArray operator - (const ValArray& va, double value)
	{
		throw RunTimeException(_T("Operation not defined.\r\n"));
	}

	int size() const { return int(std::valarray<Vector>::size()); };
};

template<typename T> struct VctrType
{ 
	typedef ValArray<T>     Vec1D;
	typedef ValArray<Vec1D> Vec2D;
	typedef ValArray<Vec2D> Vec3D;
	typedef ValArray<Vec3D> Vec4D;
	typedef ValArray<Vec4D> Vec5D;

};

typedef VctrType<int>::Vec1D     SintVec1D;
typedef VctrType<int>::Vec2D     SintVec2D;
typedef VctrType<int>::Vec3D     SintVec3D;
typedef VctrType<int>::Vec4D     SintVec4D;
typedef VctrType<int>::Vec5D     SintVec5D;

typedef VctrType<integer>::Vec1D IntgVec1D;
typedef VctrType<integer>::Vec2D IntgVec2D;
typedef VctrType<integer>::Vec3D IntgVec3D;
typedef VctrType<integer>::Vec4D IntgVec4D;
typedef VctrType<integer>::Vec5D IntgVec5D;

typedef VctrType<realnum>::Vec1D RealVec1D;
typedef VctrType<realnum>::Vec2D RealVec2D;
typedef VctrType<realnum>::Vec3D RealVec3D;
typedef VctrType<realnum>::Vec4D RealVec4D;
typedef VctrType<realnum>::Vec5D RealVec5D;

typedef VctrType<Complex>::Vec1D CplxVec1D;
typedef VctrType<Complex>::Vec2D CplxVec2D;
typedef VctrType<Complex>::Vec3D CplxVec3D;
typedef VctrType<Complex>::Vec4D CplxVec4D;
typedef VctrType<Complex>::Vec5D CplxVec5D;

typedef VctrType<Vector>::Vec1D VctrVec1D;
typedef VctrType<Vector>::Vec2D VctrVec2D;
typedef VctrType<Vector>::Vec3D VctrVec3D;
typedef VctrType<Vector>::Vec4D VctrVec4D;
typedef VctrType<Vector>::Vec5D VctrVec5D;

template <typename T>
class Matrix: public VctrType<T>::Vec2D
{
public:
	Matrix() {}
	
	// Zero-based array
	Matrix(int size1, int size2) 
	{
		if(size1<=0 || size2<=0)
			throw RunTimeException(_T("Matrix size should be greater than 0.\r\n"));
		
		this->resize(size1);
			
		for(int i=0; i<size1; ++i)
		{
			::Sleep(0); this->operator [](i).resize(size2);
		}
	}
	
	//Initialize to constant
	Matrix(const T &a, int size1, int size2)
	{
		if(size1<=0 || size2<=0)
			throw RunTimeException(_T("Matrix size should be greater than 0.\r\n"));
		
		this->resize(size1);
			
		for(int i=0; i<size1; ++i)
		{
			::Sleep(0);
			this->operator [](i).resize(size2);
			this->operator [](i) = a;

		}
	}

	// Initialize to array
	Matrix(const T *a, int size1, int size2)
	{
		if(size1<=0 || size2<=0)
			throw RunTimeException(_T("Matrix size should be greater than 0.\r\n"));
		
		this->resize(size1);
			
		for(int i=0; i<size1; ++i)
		{
			::Sleep(0);
			this->operator [](i).resize(size2);
			this->operator [](i) = *a++;

		}
	}

	// Copy constructor
	Matrix(const Matrix &rhs) { *this = rhs; }

	Matrix & operator=(const Matrix &rhs)	
	{   
		if(this != &rhs) 
			VctrType<T>::Vec2D::operator=(rhs); 

		return *this; 
	}

	//assign a to every element
	Matrix & operator=(const T &a)		
	{
		int count = nrows();

		for(int i=0; i<count; i++) this->operator[](i) = a;
		
		return *this;
	}
	
	int nrows() const { return this->size(); }
	int ncols() const { return this->operator[](0).size(); }
	
	virtual ~Matrix(){}
};

template <typename T>
class Matrix3D: public VctrType<T>::Vec3D
{
public:
	Matrix3D(){ }
		
	Matrix3D(int size1, int size2, int size3)
	{
		if(size1<=0 || size2<=0 || size3<=0)
			throw RunTimeException(_T("Matrix3D size should be greater than 0.\r\n"));
		
		this->resize(size1);
		for(int i=0; i<size1; ++i)
		{
			::Sleep(0);
			this->operator[](i).resize(size2);
			for(int j=0; j<size2; ++j)
			{
				::Sleep(0);
				this->operator[](i)[j].resize(size3);
			}
		}
	}

	int dim1() const { return this->size(); }
	int dim2() const { return this->operator[](0).size(); }
	int dim3() const { return this->operator[](0)(0).size(); }
	
	virtual ~Matrix3D() {}
};

//The next 3 classes are used in artihmetic coding, Huffman coding, and
//wavelet transforms respectively. This is as good a place as any to put them!

class arithcode {
private:
	ValArray<unsigned long> *ilob_p,*iupb_p,*ncumfq_p;
public:
	ValArray<unsigned long> &ilob,&iupb,&ncumfq;
	unsigned long jdif,nc,minint,nch,ncum,nrad;
	arithcode(unsigned long n1, unsigned long n2, unsigned long n3)
		: ilob_p(new ValArray<unsigned long>(n1)),
		iupb_p(new ValArray<unsigned long>(n2)),
		ncumfq_p(new ValArray<unsigned long>(n3)),
		ilob(*ilob_p),iupb(*iupb_p),ncumfq(*ncumfq_p) {}
	~arithcode() {
		if (ilob_p != 0) delete ilob_p;
		if (iupb_p != 0) delete iupb_p;
		if (ncumfq_p != 0) delete ncumfq_p;
	}
};

class huffcode {
private:
	ValArray<unsigned long> *icod_p,*ncod_p,*left_p,*right_p;
public:
	ValArray<unsigned long> &icod,&ncod,&left,&right;
	int nch,nodemax;
	huffcode(unsigned long n1, unsigned long n2, unsigned long n3,
		unsigned long n4) :
		icod_p(new ValArray<unsigned long>(n1)),
		ncod_p(new ValArray<unsigned long>(n2)),
		left_p(new ValArray<unsigned long>(n3)),
		right_p(new ValArray<unsigned long>(n4)),
		icod(*icod_p),ncod(*ncod_p),left(*left_p),right(*right_p) {}
	~huffcode() {
		if (icod_p != 0) delete icod_p;
		if (ncod_p != 0) delete ncod_p;
		if (left_p != 0) delete left_p;
		if (right_p != 0) delete right_p;
	}
};

class wavefilt {
private:
	ValArray<DP> *cc_p,*cr_p;
public:
	int ncof,ioff,joff;
	ValArray<DP> &cc,&cr;
	wavefilt() : cc(*cc_p),cr(*cr_p) {}
	wavefilt(const DP *a, const int n) :  //initialize to array
		cc_p(new ValArray<DP>(n)),cr_p(new ValArray<DP>(n)),
		ncof(n),ioff(-(n >> 1)),joff(-(n >> 1)),cc(*cc_p),cr(*cr_p) {
			int i;
			for (i=0; i<n; i++)
				cc[i] = *a++;
			DP sig = -1.0;
			for (i=0; i<n; i++) {
				cr[n-1-i]=sig*cc[i];
				sig = -sig;
			}
	}
	~wavefilt() {
		if (cc_p != 0) delete cc_p;
		if (cr_p != 0) delete cr_p;
	}
};

typedef const ValArray<bool> Vec_I_BOOL;
typedef ValArray<bool> Vec_BOOL, Vec_O_BOOL, Vec_IO_BOOL;

typedef const ValArray<char> Vec_I_CHR;
typedef ValArray<char> Vec_CHR, Vec_O_CHR, Vec_IO_CHR;

typedef const ValArray<unsigned char> Vec_I_UCHR;
typedef ValArray<unsigned char> Vec_UCHR, Vec_O_UCHR, Vec_IO_UCHR;

typedef const ValArray<int> Vec_I_INT;
typedef ValArray<int> Vec_INT, Vec_O_INT, Vec_IO_INT;

typedef const ValArray<unsigned int> Vec_I_UINT;
typedef ValArray<unsigned int> Vec_UINT, Vec_O_UINT, Vec_IO_UINT;

typedef const ValArray<long> Vec_I_LNG;
typedef ValArray<long> Vec_LNG, Vec_O_LNG, Vec_IO_LNG;

typedef const ValArray<unsigned long> Vec_I_ULNG;
typedef ValArray<unsigned long> Vec_ULNG, Vec_O_ULNG, Vec_IO_ULNG;

typedef const ValArray<float> Vec_I_SP;
typedef ValArray<float> Vec_SP, Vec_O_SP, Vec_IO_SP;

typedef const ValArray<DP> Vec_I_DP;
typedef ValArray<DP> Vec_DP, Vec_O_DP, Vec_IO_DP;

typedef const ValArray<std::complex<float> > Vec_I_CPLX_SP;
typedef ValArray<std::complex<float> > Vec_CPLX_SP, Vec_O_CPLX_SP, Vec_IO_CPLX_SP;

typedef const ValArray<Complex > Vec_I_CPLX_DP;
typedef ValArray<Complex > Vec_CPLX_DP, Vec_O_CPLX_DP, Vec_IO_CPLX_DP;

typedef const Matrix<bool> Mat_I_BOOL;
typedef Matrix<bool> Mat_BOOL, Mat_O_BOOL, Mat_IO_BOOL;

typedef const Matrix<char> Mat_I_CHR;
typedef Matrix<char> Mat_CHR, Mat_O_CHR, Mat_IO_CHR;

typedef const Matrix<unsigned char> Mat_I_UCHR;
typedef Matrix<unsigned char> Mat_UCHR, Mat_O_UCHR, Mat_IO_UCHR;

typedef const Matrix<int> Mat_I_INT;
typedef Matrix<int> Mat_INT, Mat_O_INT, Mat_IO_INT;

typedef const Matrix<unsigned int> Mat_I_UINT;
typedef Matrix<unsigned int> Mat_UINT, Mat_O_UINT, Mat_IO_UINT;

typedef const Matrix<long> Mat_I_LNG;
typedef Matrix<long> Mat_LNG, Mat_O_LNG, Mat_IO_LNG;

typedef const Matrix<unsigned long> Mat_I_ULNG;
typedef Matrix<unsigned long> Mat_ULNG, Mat_O_ULNG, Mat_IO_ULNG;

typedef const Matrix<float> Mat_I_SP;
typedef Matrix<float> Mat_SP, Mat_O_SP, Mat_IO_SP;

typedef const Matrix<DP> Mat_I_DP;
typedef Matrix<DP> Mat_DP, Mat_O_DP, Mat_IO_DP;

typedef const Matrix<std::complex<float> > Mat_I_CPLX_SP;
typedef Matrix<std::complex<float> > Mat_CPLX_SP, Mat_O_CPLX_SP, Mat_IO_CPLX_SP;

typedef const Matrix<Complex > Mat_I_CPLX_DP;
typedef Matrix<Complex > Mat_CPLX_DP, Mat_O_CPLX_DP, Mat_IO_CPLX_DP;

typedef const Matrix3D<DP> Mat3D_I_DP;
typedef Matrix3D<DP> Mat3D_DP, Mat3D_O_DP, Mat3D_IO_DP;

typedef ValArray<unsigned long *> Vec_ULNG_p;
typedef ValArray<Matrix<DP> *>    Vec_Mat_DP_p;
typedef ValArray<std::fstream *>  Vec_FSTREAM_p;

namespace NR 
{
	void addint(Mat_O_DP &uf, Mat_I_DP &uc, Mat_O_DP &res);
	void airy(const DP x, DP &ai, DP &bi, DP &aip, DP &bip);
	void amebsa(Mat_IO_DP &p, Vec_IO_DP &y, Vec_O_DP &pb, DP &yb, const DP ftol,
		DP funk(Vec_I_DP &), int &iter, const DP temptr);
	void amoeba(Mat_IO_DP &p, Vec_IO_DP &y, const DP ftol, DP funk(Vec_I_DP &),
		int &nfunk);
	DP amotry(Mat_IO_DP &p, Vec_O_DP &y, Vec_IO_DP &psum, DP funk(Vec_I_DP &),
		const int ihi, const DP fac);
	DP amotsa(Mat_IO_DP &p, Vec_O_DP &y, Vec_IO_DP &psum, Vec_O_DP &pb, DP &yb,
		DP funk(Vec_I_DP &), const int ihi, DP &yhi, const DP fac);
	void anneal(Vec_I_DP &x, Vec_I_DP &y, Vec_IO_INT &iorder);
	DP anorm2(Mat_I_DP &a);
	void arcmak(Vec_I_ULNG &nfreq, unsigned long nchh, unsigned long nradd,
		arithcode &acode);
	void arcode(unsigned long &ich, std::string &code, unsigned long &lcd,
		const int isign, arithcode &acode);
	void arcsum(Vec_I_ULNG &iin, Vec_O_ULNG &iout, unsigned long ja,
		const int nwk, const unsigned long nrad, const unsigned long nc);
	void asolve(Vec_I_DP &b, Vec_O_DP &x, const int itrnsp);
	void atimes(Vec_I_DP &x, Vec_O_DP &r, const int itrnsp);
	void avevar(Vec_I_DP &data, DP &ave, DP &var);
	void balanc(Mat_IO_DP &a);
	void banbks(Mat_I_DP &a, const int m1, const int m2, Mat_I_DP &al,
		Vec_I_INT &indx, Vec_IO_DP &b);
	void bandec(Mat_IO_DP &a, const int m1, const int m2, Mat_O_DP &al,
		Vec_O_INT &indx, DP &d);
	void banmul(Mat_I_DP &a, const int m1, const int m2, Vec_I_DP &x,
		Vec_O_DP &b);
	void bcucof(Vec_I_DP &y, Vec_I_DP &y1, Vec_I_DP &y2, Vec_I_DP &y12,
		const DP d1, const DP d2, Mat_O_DP &c);
	void bcuint(Vec_I_DP &y, Vec_I_DP &y1, Vec_I_DP &y2, Vec_I_DP &y12,
		const DP x1l, const DP x1u, const DP x2l, const DP x2u,
		const DP x1, const DP x2, DP &ansy, DP &ansy1, DP &ansy2);
	void beschb(const DP x, DP &gam1, DP &gam2, DP &gampl, DP &gammi);
	DP bessi(const int n, const DP x);
	DP bessi0(const DP x);
	DP bessi1(const DP x);
	void bessik(const DP x, const DP xnu, DP &ri, DP &rk, DP &rip, DP &rkp);
	DP bessj(const int n, const DP x);
	DP bessj0(const DP x);
	DP bessj1(const DP x);
	void bessjy(const DP x, const DP xnu, DP &rj, DP &ry, DP &rjp, DP &ryp);
	DP bessk(const int n, const DP x);
	DP bessk0(const DP x);
	DP bessk1(const DP x);
	DP bessy(const int n, const DP x);
	DP bessy0(const DP x);
	DP bessy1(const DP x);
	DP beta(const DP z, const DP w);
	DP betacf(const DP a, const DP b, const DP x);
	DP betai(const DP a, const DP b, const DP x);
	DP bico(const int n, const int k);
	void bksub(const int ne, const int nb, const int jf, const int k1,
		const int k2, Mat3D_IO_DP &c);
	DP bnldev(const DP pp, const int n, int &idum);
	DP brent(const DP ax, const DP bx, const DP cx, DP f(const DP),
		const DP tol, DP &xmin);
	void broydn(Vec_IO_DP &x, bool &check, void vecfunc(Vec_I_DP &, Vec_O_DP &));
	void bsstep(Vec_IO_DP &y, Vec_IO_DP &dydx, DP &xx, const DP htry,
		const DP eps, Vec_I_DP &yscal, DP &hdid, DP &hnext,
		void derivs(const DP, Vec_I_DP &, Vec_O_DP &));
	void caldat(const int julian, int &mm, int &id, int &iyyy);
	void chder(const DP a, const DP b, Vec_I_DP &c, Vec_O_DP &cder, const int n);
	DP chebev(const DP a, const DP b, Vec_I_DP &c, const int m, const DP x);
	void chebft(const DP a, const DP b, Vec_O_DP &c, DP func(const DP));
	void chebpc(Vec_I_DP &c, Vec_O_DP &d);
	void chint(const DP a, const DP b, Vec_I_DP &c, Vec_O_DP &cint, const int n);
	DP chixy(const DP bang);
	void choldc(Mat_IO_DP &a, Vec_O_DP &p);
	void cholsl(Mat_I_DP &a, Vec_I_DP &p, Vec_I_DP &b, Vec_O_DP &x);
	void chsone(Vec_I_DP &bins, Vec_I_DP &ebins, const int knstrn, DP &df,
		DP &chsq, DP &prob);
	void chstwo(Vec_I_DP &bins1, Vec_I_DP &bins2, const int knstrn, DP &df,
		DP &chsq, DP &prob);
	void cisi(const DP x, Complex &cs);
	void cntab1(Mat_I_INT &nn, DP &chisq, DP &df, DP &prob, DP &cramrv, DP &ccc);
	void cntab2(Mat_I_INT &nn, DP &h, DP &hx, DP &hy, DP &hygx, DP &hxgy,
		DP &uygx, DP &uxgy, DP &uxy);
	void convlv(Vec_I_DP &data, Vec_I_DP &respns, const int isign,
		Vec_O_DP &ans);
	void copy(Mat_O_DP &aout, Mat_I_DP &ain);
	void correl(Vec_I_DP &data1, Vec_I_DP &data2, Vec_O_DP &ans);
	void cosft1(Vec_IO_DP &y);
	void cosft2(Vec_IO_DP &y, const int isign);
	void covsrt(Mat_IO_DP &covar, Vec_I_BOOL &ia, const int mfit);
	void crank(Vec_IO_DP &w, DP &s);
	void cyclic(Vec_I_DP &a, Vec_I_DP &b, Vec_I_DP &c, const DP alpha,
		const DP beta, Vec_I_DP &r, Vec_O_DP &x);
	void daub4(Vec_IO_DP &a, const int n, const int isign);
	DP dawson(const DP x);
	DP dbrent(const DP ax, const DP bx, const DP cx, DP f(const DP),
		DP df(const DP), const DP tol, DP &xmin);
	void ddpoly(Vec_I_DP &c, const DP x, Vec_O_DP &pd);
	bool decchk(std::string str, char &ch);
	void derivs_s(const DP x, Vec_I_DP &y, Vec_O_DP &dydx);
	DP df1dim(const DP x);
	void dfpmin(Vec_IO_DP &p, const DP gtol, int &iter, DP &fret,
		DP func(Vec_I_DP &), void dfunc(Vec_I_DP &, Vec_O_DP &));
	DP dfridr(DP func(const DP), const DP x, const DP h, DP &err);
	void dftcor(const DP w, const DP delta, const DP a, const DP b,
		Vec_I_DP &endpts, DP &corre, DP &corim, DP &corfac);
	void dftint(DP func(const DP), const DP a, const DP b, const DP w,
		DP &cosint, DP &sinint);
	void difeq(const int k, const int k1, const int k2, const int jsf,
		const int is1, const int isf, Vec_I_INT &indexv, Mat_O_DP &s,
		Mat_I_DP &y);
	void dlinmin(Vec_IO_DP &p, Vec_IO_DP &xi, DP &fret, DP func(Vec_I_DP &),
		void dfunc(Vec_I_DP &, Vec_O_DP &));
	void eclass(Vec_O_INT &nf, Vec_I_INT &lista, Vec_I_INT &listb);
	void eclazz(Vec_O_INT &nf, bool equiv(const int, const int));
	DP ei(const DP x);
	void eigsrt(Vec_IO_DP &d, Mat_IO_DP &v);
	DP elle(const DP phi, const DP ak);
	DP ellf(const DP phi, const DP ak);
	DP ellpi(const DP phi, const DP en, const DP ak);
	void elmhes(Mat_IO_DP &a);
	DP erfcc(const DP x);
	DP erff(const DP x);
	DP erffc(const DP x);
	void eulsum(DP &sum, const DP term, const int jterm, Vec_IO_DP &wksp);
	DP evlmem(const DP fdt, Vec_I_DP &d, const DP xms);
	DP expdev(int &idum);
	DP expint(const int n, const DP x);
	DP f1dim(const DP x);
	DP factln(const int n);
	DP factrl(const int n);
	void fasper(Vec_I_DP &x, Vec_I_DP &y, const DP ofac, const DP hifac,
		Vec_O_DP &wk1, Vec_O_DP &wk2, int &nout, int &jmax, DP &prob);
	void fdjac(Vec_IO_DP &x, Vec_I_DP &fvec, Mat_O_DP &df,
		void vecfunc(Vec_I_DP &, Vec_O_DP &));
	void fgauss(const DP x, Vec_I_DP &a, DP &y, Vec_O_DP &dyda);
	void fit(Vec_I_DP &x, Vec_I_DP &y, Vec_I_DP &sig, const bool mwt, DP &a,
		DP &b, DP &siga, DP &sigb, DP &chi2, DP &q);
	void fitexy(Vec_I_DP &x, Vec_I_DP &y, Vec_I_DP &sigx, Vec_I_DP &sigy,
		DP &a, DP &b, DP &siga, DP &sigb, DP &chi2, DP &q);
	void fixrts(Vec_IO_DP &d);
	void fleg(const DP x, Vec_O_DP &pl);
	void flmoon(const int n, const int nph, int &jd, DP &frac);
	DP fmin(Vec_I_DP &x);
	void four1(Vec_IO_DP &data, const int isign);
	void fourew(Vec_FSTREAM_p &file, int &na, int &nb, int &nc, int &nd);
	void fourfs(Vec_FSTREAM_p &file, Vec_I_INT &nn, const int isign);
	void fourn(Vec_IO_DP &data, Vec_I_INT &nn, const int isign);
	void fpoly(const DP x, Vec_O_DP &p);
	void fred2(const DP a, const DP b, Vec_O_DP &t, Vec_O_DP &f, Vec_O_DP &w,
		DP g(const DP), DP ak(const DP, const DP));
	DP fredin(const DP x, const DP a, const DP b, Vec_I_DP &t, Vec_I_DP &f,
		Vec_I_DP &w, DP g(const DP), DP ak(const DP, const DP));
	void frenel(const DP x, Complex &cs);
	void frprmn(Vec_IO_DP &p, const DP ftol, int &iter, DP &fret,
		DP func(Vec_I_DP &), void dfunc(Vec_I_DP &, Vec_O_DP &));
	void ftest(Vec_I_DP &data1, Vec_I_DP &data2, DP &f, DP &prob);
	DP gamdev(const int ia, int &idum);
	DP gammln(const DP xx);
	DP gammp(const DP a, const DP x);
	DP gammq(const DP a, const DP x);
	DP gasdev(int &idum);
	void gaucof(Vec_IO_DP &a, Vec_IO_DP &b, const DP amu0, Vec_O_DP &x,
		Vec_O_DP &w);
	void gauher(Vec_O_DP &x, Vec_O_DP &w);
	void gaujac(Vec_O_DP &x, Vec_O_DP &w, const DP alf, const DP bet);
	void gaulag(Vec_O_DP &x, Vec_O_DP &w, const DP alf);
	void gauleg(const DP x1, const DP x2, Vec_O_DP &x, Vec_O_DP &w);
	void gaussj(Mat_IO_DP &a, Mat_IO_DP &b);
	void gcf(DP &gammcf, const DP a, const DP x, DP &gln);
	DP golden(const DP ax, const DP bx, const DP cx, DP f(const DP),
		const DP tol, DP &xmin);
	void gser(DP &gamser, const DP a, const DP x, DP &gln);
	void hpsel(Vec_I_DP &arr, Vec_O_DP &heap);
	void hpsort(Vec_IO_DP &ra);
	void hqr(Mat_IO_DP &a, Vec_O_CPLX_DP &wri);
	void hufapp(Vec_IO_ULNG &index, Vec_I_ULNG &nprob, const unsigned long n,
		const unsigned long m);
	void hufdec(unsigned long &ich, std::string &code, const unsigned long lcode,
		unsigned long &nb, huffcode &hcode);
	void hufenc(const unsigned long ich, std::string &code, unsigned long &nb,
		huffcode &hcode);
	void hufmak(Vec_I_ULNG &nfreq, const unsigned long nchin,
		unsigned long &ilong, unsigned long &nlong, huffcode &hcode);
	void hunt(Vec_I_DP &xx, const DP x, int &jlo);
	void hypdrv(const DP s, Vec_I_DP &yy, Vec_O_DP &dyyds);
	Complex hypgeo(const Complex &a, const Complex &b,
		const Complex &c, const Complex &z);
	void hypser(const Complex &a, const Complex &b,
		const Complex &c, const Complex &z,
		Complex &series, Complex &deriv);
	unsigned short icrc(const unsigned short crc, const std::string &bufptr,
		const short jinit, const int jrev);
	unsigned short icrc1(const unsigned short crc, const unsigned char onech);
	unsigned long igray(const unsigned long n, const int is);
	void indexx(Vec_I_DP &arr, Vec_O_INT &indx);
	void indexx(Vec_I_INT &arr, Vec_O_INT &indx);
	void interp(Mat_O_DP &uf, Mat_I_DP &uc);
	int irbit1(unsigned long &iseed);
	int irbit2(unsigned long &iseed);
	void jacobi(Mat_IO_DP &a, Vec_O_DP &d, Mat_O_DP &v, int &nrot);
	void jacobn_s(const DP x, Vec_I_DP &y, Vec_O_DP &dfdx, Mat_O_DP &dfdy);
	int julday(const int mm, const int id, const int iyyy);
	void kendl1(Vec_I_DP &data1, Vec_I_DP &data2, DP &tau, DP &z, DP &prob);
	void kendl2(Mat_I_DP &tab, DP &tau, DP &z, DP &prob);
	void kermom(Vec_O_DP &w, const DP y);
	void ks2d1s(Vec_I_DP &x1, Vec_I_DP &y1, void quadvl(const DP, const DP,
		DP &, DP &, DP &, DP &), DP &d1, DP &prob);
	void ks2d2s(Vec_I_DP &x1, Vec_I_DP &y1, Vec_I_DP &x2, Vec_I_DP &y2, DP &d,
		DP &prob);
	void ksone(Vec_IO_DP &data, DP func(const DP), DP &d, DP &prob);
	void kstwo(Vec_IO_DP &data1, Vec_IO_DP &data2, DP &d, DP &prob);
	void laguer(Vec_I_CPLX_DP &a, Complex &x, int &its);
	void lfit(Vec_I_DP &x, Vec_I_DP &y, Vec_I_DP &sig, Vec_IO_DP &a,
		Vec_I_BOOL &ia, Mat_O_DP &covar, DP &chisq,
		void funcs(const DP, Vec_O_DP &));
	void linbcg(Vec_I_DP &b, Vec_IO_DP &x, const int itol, const DP tol,
		const int itmax, int &iter, DP &err);
	void linmin(Vec_IO_DP &p, Vec_IO_DP &xi, DP &fret, DP func(Vec_I_DP &));
	void lnsrch(Vec_I_DP &xold, const DP fold, Vec_I_DP &g, Vec_IO_DP &p,
		Vec_O_DP &x, DP &f, const DP stpmax, bool &check, DP func(Vec_I_DP &));
	void locate(Vec_I_DP &xx, const DP x, int &j);
	void lop(Mat_O_DP &out, Mat_I_DP &u);
	void lubksb(Mat_I_DP &a, Vec_I_INT &indx, Vec_IO_DP &b);
	void ludcmp(Mat_IO_DP &a, Vec_O_INT &indx, DP &d);
	void machar(int &ibeta, int &it, int &irnd, int &ngrd, int &machep,
		int &negep, int &iexp, int &minexp, int &maxexp, DP &eps, DP &epsneg,
		DP &xmin, DP &xmax);
	void matadd(Mat_I_DP &a, Mat_I_DP &b, Mat_O_DP &c);
	void matsub(Mat_I_DP &a, Mat_I_DP &b, Mat_O_DP &c);
	void medfit(Vec_I_DP &x, Vec_I_DP &y, DP &a, DP &b, DP &abdev);
	void memcof(Vec_I_DP &data, DP &xms, Vec_O_DP &d);
	bool metrop(const DP de, const DP t);
	void mgfas(Mat_IO_DP &u, const int maxcyc);
	void mglin(Mat_IO_DP &u, const int ncycle);
	DP midexp(DP funk(const DP), const DP aa, const DP bb, const int n);
	DP midinf(DP funk(const DP), const DP aa, const DP bb, const int n);
	DP midpnt(DP func(const DP), const DP a, const DP b, const int n);
	DP midsql(DP funk(const DP), const DP aa, const DP bb, const int n);
	DP midsqu(DP funk(const DP), const DP aa, const DP bb, const int n);
	void miser(DP func(Vec_I_DP &), Vec_I_DP &regn, const int npts,
		const DP dith, DP &ave, DP &var);
	void mmid(Vec_I_DP &y, Vec_I_DP &dydx, const DP xs, const DP htot,
		const int nstep, Vec_O_DP &yout,
		void derivs(const DP, Vec_I_DP &, Vec_O_DP &));
	void mnbrak(DP &ax, DP &bx, DP &cx, DP &fa, DP &fb, DP &fc,
		DP func(const DP));
	void mnewt(const int ntrial, Vec_IO_DP &x, const DP tolx, const DP tolf);
	void moment(Vec_I_DP &data, DP &ave, DP &adev, DP &sdev, DP &var, DP &skew,
		DP &curt);
	void mp2dfr(Vec_IO_UCHR &a, std::string &s);
	void mpadd(Vec_O_UCHR &w, Vec_I_UCHR &u, Vec_I_UCHR &v);
	void mpdiv(Vec_O_UCHR &q, Vec_O_UCHR &r, Vec_I_UCHR &u, Vec_I_UCHR &v);
	void mpinv(Vec_O_UCHR &u, Vec_I_UCHR &v);
	void mplsh(Vec_IO_UCHR &u);
	void mpmov(Vec_O_UCHR &u, Vec_I_UCHR &v);
	void mpmul(Vec_O_UCHR &w, Vec_I_UCHR &u, Vec_I_UCHR &v);
	void mpneg(Vec_IO_UCHR &u);
	void mppi(const int np);
	void mprove(Mat_I_DP &a, Mat_I_DP &alud, Vec_I_INT &indx, Vec_I_DP &b,
		Vec_IO_DP &x);
	void mpsad(Vec_O_UCHR &w, Vec_I_UCHR &u, const int iv);
	void mpsdv(Vec_O_UCHR &w, Vec_I_UCHR &u, const int iv, int &ir);
	void mpsmu(Vec_O_UCHR &w, Vec_I_UCHR &u, const int iv);
	void mpsqrt(Vec_O_UCHR &w, Vec_O_UCHR &u, Vec_I_UCHR &v);
	void mpsub(int &is, Vec_O_UCHR &w, Vec_I_UCHR &u, Vec_I_UCHR &v);
	void mrqcof(Vec_I_DP &x, Vec_I_DP &y, Vec_I_DP &sig, Vec_I_DP &a,
		Vec_I_BOOL &ia, Mat_O_DP &alpha, Vec_O_DP &beta, DP &chisq,
		void funcs(const DP, Vec_I_DP &,DP &, Vec_O_DP &));
	void mrqmin(Vec_I_DP &x, Vec_I_DP &y, Vec_I_DP &sig, Vec_IO_DP &a,
		Vec_I_BOOL &ia, Mat_O_DP &covar, Mat_O_DP &alpha, DP &chisq,
		void funcs(const DP, Vec_I_DP &, DP &, Vec_O_DP &), DP &alamda);
	void newt(Vec_IO_DP &x, bool &check, void vecfunc(Vec_I_DP &, Vec_O_DP &));
	void odeint(Vec_IO_DP &ystart, const DP x1, const DP x2, const DP eps,
		const DP h1, const DP hmin, int &nok, int &nbad,
		void derivs(const DP, Vec_I_DP &, Vec_O_DP &),
		void rkqs(Vec_IO_DP &, Vec_IO_DP &, DP &, const DP, const DP,
		Vec_I_DP &, DP &, DP &, void (*)(const DP, Vec_I_DP &, Vec_O_DP &)));
	void orthog(Vec_I_DP &anu, Vec_I_DP &alpha, Vec_I_DP &beta, Vec_O_DP &a,
		Vec_O_DP &b);
	void pade(Vec_IO_DP &cof, DP &resid);
	void pccheb(Vec_I_DP &d, Vec_O_DP &c);
	void pcshft(const DP a, const DP b, Vec_IO_DP &d);
	void pearsn(Vec_I_DP &x, Vec_I_DP &y, DP &r, DP &prob, DP &z);
	void period(Vec_I_DP &x, Vec_I_DP &y, const DP ofac, const DP hifac,
		Vec_O_DP &px, Vec_O_DP &py, int &nout, int &jmax, DP &prob);
	void piksr2(Vec_IO_DP &arr, Vec_IO_DP &brr);
	void piksrt(Vec_IO_DP &arr);
	void pinvs(const int ie1, const int ie2, const int je1, const int jsf,
		const int jc1, const int k, Mat3D_O_DP &c, Mat_IO_DP &s);
	DP plgndr(const int l, const int m, const DP x);
	DP poidev(const DP xm, int &idum);
	void polcoe(Vec_I_DP &x, Vec_I_DP &y, Vec_O_DP &cof);
	void polcof(Vec_I_DP &xa, Vec_I_DP &ya, Vec_O_DP &cof);
	void poldiv(Vec_I_DP &u, Vec_I_DP &v, Vec_O_DP &q, Vec_O_DP &r);
	void polin2(Vec_I_DP &x1a, Vec_I_DP &x2a, Mat_I_DP &ya, const DP x1,
		const DP x2, DP &y, DP &dy);
	void polint(Vec_I_DP &xa, Vec_I_DP &ya, const DP x, DP &y, DP &dy);
	void powell(Vec_IO_DP &p, Mat_IO_DP &xi, const DP ftol, int &iter,
		DP &fret, DP func(Vec_I_DP &));
	void predic(Vec_I_DP &data, Vec_I_DP &d, Vec_O_DP &future);
	DP probks(const DP alam);
	void psdes(unsigned long &lword, unsigned long &irword);
	void pwt(Vec_IO_DP &a, const int n, const int isign);
	void pwtset(const int n);
	DP pythag(const DP a, const DP b);
	void pzextr(const int iest, const DP xest, Vec_I_DP &yest, Vec_O_DP &yz,
		Vec_O_DP &dy);
	DP qgaus(DP func(const DP), const DP a, const DP b);
	void qrdcmp(Mat_IO_DP &a, Vec_O_DP &c, Vec_O_DP &d, bool &sing);
	DP qromb(DP func(const DP), DP a, DP b);
	DP qromo(DP func(const DP), const DP a, const DP b,
		DP choose(DP (*)(const DP), const DP, const DP, const int));
	void qroot(Vec_I_DP &p, DP &b, DP &c, const DP eps);
	void qrsolv(Mat_I_DP &a, Vec_I_DP &c, Vec_I_DP &d, Vec_IO_DP &b);
	void qrupdt(Mat_IO_DP &r, Mat_IO_DP &qt, Vec_IO_DP &u, Vec_I_DP &v);
	DP qsimp(DP func(const DP), const DP a, const DP b);
	DP qtrap(DP func(const DP), const DP a, const DP b);
	DP quad3d(DP func(const DP, const DP, const DP), const DP x1, const DP x2);
	void quadct(const DP x, const DP y, Vec_I_DP &xx, Vec_I_DP &yy, DP &fa,
		DP &fb, DP &fc, DP &fd);
	void quadmx(Mat_O_DP &a);
	void quadvl(const DP x, const DP y, DP &fa, DP &fb, DP &fc, DP &fd);
	DP ran0(int &idum);
	DP ran1(int &idum);
	DP ran2(int &idum);
	DP ran3(int &idum);
	DP ran4(int &idum);
	void rank(Vec_I_INT &indx, Vec_O_INT &irank);
	void ranpt(Vec_O_DP &pt, Vec_I_DP &regn);
	void ratint(Vec_I_DP &xa, Vec_I_DP &ya, const DP x, DP &y, DP &dy);
	void ratlsq(DP fn(const DP), const DP a, const DP b, const int mm,
		const int kk, Vec_O_DP &cof, DP &dev);
	DP ratval(const DP x, Vec_I_DP &cof, const int mm, const int kk);
	DP rc(const DP x, const DP y);
	DP rd(const DP x, const DP y, const DP z);
	void realft(Vec_IO_DP &data, const int isign);
	void rebin(const DP rc, const int nd, Vec_I_DP &r, Vec_O_DP &xin,
		Mat_IO_DP &xi, const int j);
	void red(const int iz1, const int iz2, const int jz1, const int jz2,
		const int jm1, const int jm2, const int jmf, const int ic1,
		const int jc1, const int jcf, const int kc, Mat3D_I_DP &c,
		Mat_IO_DP &s);
	void relax(Mat_IO_DP &u, Mat_I_DP &rhs);
	void relax2(Mat_IO_DP &u, Mat_I_DP &rhs);
	void resid(Mat_O_DP &res, Mat_I_DP &u, Mat_I_DP &rhs);
	DP revcst(Vec_I_DP &x, Vec_I_DP &y, Vec_I_INT &iorder, Vec_IO_INT &n);
	void reverse(Vec_IO_INT &iorder, Vec_I_INT &n);
	DP rf(const DP x, const DP y, const DP z);
	DP rj(const DP x, const DP y, const DP z, const DP p);
	void rk4(Vec_I_DP &y, Vec_I_DP &dydx, const DP x, const DP h,
		Vec_O_DP &yout, void derivs(const DP, Vec_I_DP &, Vec_O_DP &));
	void rkck(Vec_I_DP &y, Vec_I_DP &dydx, const DP x,
		const DP h, Vec_O_DP &yout, Vec_O_DP &yerr,
		void derivs(const DP, Vec_I_DP &, Vec_O_DP &));
	void rkdumb(Vec_I_DP &vstart, const DP x1, const DP x2,
		void derivs(const DP, Vec_I_DP &, Vec_O_DP &));
	void rkqs(Vec_IO_DP &y, Vec_IO_DP &dydx, DP &x, const DP htry,
		const DP eps, Vec_I_DP &yscal, DP &hdid, DP &hnext,
		void derivs(const DP, Vec_I_DP &, Vec_O_DP &));
	void rlft3(Mat3D_IO_DP &data, Mat_IO_DP &speq, const int isign);
	DP rofunc(const DP b);
	void rotate(Mat_IO_DP &r, Mat_IO_DP &qt, const int i, const DP a,
		const DP b);
	void rsolv(Mat_I_DP &a, Vec_I_DP &d, Vec_IO_DP &b);
	void rstrct(Mat_O_DP &uc, Mat_I_DP &uf);
	DP rtbis(DP func(const DP), const DP x1, const DP x2, const DP xacc);
	DP rtflsp(DP func(const DP), const DP x1, const DP x2, const DP xacc);
	DP rtnewt(void funcd(const DP, DP &, DP &), const DP x1, const DP x2,
		const DP xacc);
	DP rtsafe(void funcd(const DP, DP &, DP &), const DP x1, const DP x2,
		const DP xacc);
	DP rtsec(DP func(const DP), const DP x1, const DP x2, const DP xacc);
	void rzextr(const int iest, const DP xest, Vec_I_DP &yest, Vec_O_DP &yz,
		Vec_O_DP &dy);
	void savgol(Vec_O_DP &c, const int np, const int nl, const int nr,
		const int ld, const int m);
	void scrsho(DP fx(const DP));
	DP select(const int k, Vec_IO_DP &arr);
	DP selip(const int k, Vec_I_DP &arr);
	void shell(const int n, Vec_IO_DP &a);
	void shoot(Vec_I_DP &v, Vec_O_DP &f);
	void shootf(Vec_I_DP &v, Vec_O_DP &f);
	void simp1(Mat_I_DP &a, const int mm, Vec_I_INT &ll, const int nll,
		const int iabf, int &kp, DP &bmax);
	void simp2(Mat_I_DP &a, const int m, const int n, int &ip, const int kp);
	void simp3(Mat_IO_DP &a, const int i1, const int k1, const int ip,
		const int kp);
	void simplx(Mat_IO_DP &a, const int m1, const int m2, const int m3,
		int &icase, Vec_O_INT &izrov, Vec_O_INT &iposv);
	void simpr(Vec_I_DP &y, Vec_I_DP &dydx, Vec_I_DP &dfdx, Mat_I_DP &dfdy,
		const DP xs, const DP htot, const int nstep, Vec_O_DP &yout,
		void derivs(const DP, Vec_I_DP &, Vec_O_DP &));
	void sinft(Vec_IO_DP &y);
	void slvsm2(Mat_O_DP &u, Mat_I_DP &rhs);
	void slvsml(Mat_O_DP &u, Mat_I_DP &rhs);
	void sncndn(const DP uu, const DP emmc, DP &sn, DP &cn, DP &dn);
	DP snrm(Vec_I_DP &sx, const int itol);
	void sobseq(const int n, Vec_O_DP &x);
	void solvde(const int itmax, const DP conv, const DP slowc,
		Vec_I_DP &scalv, Vec_I_INT &indexv, const int nb, Mat_IO_DP &y);
	void sor(Mat_I_DP &a, Mat_I_DP &b, Mat_I_DP &c, Mat_I_DP &d, Mat_I_DP &e,
		Mat_I_DP &f, Mat_IO_DP &u, const DP rjac);
	void sort(Vec_IO_DP &arr);
	void sort2(Vec_IO_DP &arr, Vec_IO_DP &brr);
	void sort3(Vec_IO_DP &ra, Vec_IO_DP &rb, Vec_IO_DP &rc);
	void spctrm(std::ifstream &fp, Vec_O_DP &p, const int k, const bool ovrlap);
	void spear(Vec_I_DP &data1, Vec_I_DP &data2, DP &d, DP &zd, DP &probd,
		DP &rs, DP &probrs);
	void sphbes(const int n, const DP x, DP &sj, DP &sy, DP &sjp, DP &syp);
	void splie2(Vec_I_DP &x1a, Vec_I_DP &x2a, Mat_I_DP &ya, Mat_O_DP &y2a);
	void splin2(Vec_I_DP &x1a, Vec_I_DP &x2a, Mat_I_DP &ya, Mat_I_DP &y2a,
		const DP x1, const DP x2, DP &y);
	void spline(Vec_I_DP &x, Vec_I_DP &y, const DP yp1, const DP ypn,
		Vec_O_DP &y2);
	void splint(Vec_I_DP &xa, Vec_I_DP &ya, Vec_I_DP &y2a, const DP x, DP &y);
	void spread(const DP y, Vec_IO_DP &yy, const DP x, const int m);
	void sprsax(Vec_I_DP &sa, Vec_I_INT &ija, Vec_I_DP &x, Vec_O_DP &b);
	void sprsin(Mat_I_DP &a, const DP thresh, Vec_O_DP &sa, Vec_O_INT &ija);
	void sprspm(Vec_I_DP &sa, Vec_I_INT &ija, Vec_I_DP &sb, Vec_I_INT &ijb,
		Vec_O_DP &sc, Vec_I_INT &ijc);
	void sprstm(Vec_I_DP &sa, Vec_I_INT &ija, Vec_I_DP &sb, Vec_I_INT &ijb,
		const DP thresh, Vec_O_DP &sc, Vec_O_INT &ijc);
	void sprstp(Vec_I_DP &sa, Vec_I_INT &ija, Vec_O_DP &sb, Vec_O_INT &ijb);
	void sprstx(Vec_I_DP &sa, Vec_I_INT &ija, Vec_I_DP &x, Vec_O_DP &b);
	void stifbs(Vec_IO_DP &y, Vec_IO_DP &dydx, DP &xx, const DP htry,
		const DP eps, Vec_I_DP &yscal, DP &hdid, DP &hnext,
		void derivs(const DP, Vec_I_DP &, Vec_O_DP &));
	void stiff(Vec_IO_DP &y, Vec_IO_DP &dydx, DP &x, const DP htry,
		const DP eps, Vec_I_DP &yscal, DP &hdid, DP &hnext,
		void derivs(const DP, Vec_I_DP &, Vec_O_DP &));
	void stoerm(Vec_I_DP &y, Vec_I_DP &d2y, const DP xs,
		const DP htot, const int nstep, Vec_O_DP &yout,
		void derivs(const DP, Vec_I_DP &, Vec_O_DP &));
	void svbksb(Mat_I_DP &u, Vec_I_DP &w, Mat_I_DP &v, Vec_I_DP &b, Vec_O_DP &x);
	void svdcmp(Mat_IO_DP &a, Vec_O_DP &w, Mat_O_DP &v);
	void svdfit(Vec_I_DP &x, Vec_I_DP &y, Vec_I_DP &sig, Vec_O_DP &a,
		Mat_O_DP &u, Mat_O_DP &v, Vec_O_DP &w, DP &chisq,
		void funcs(const DP, Vec_O_DP &));
	void svdvar(Mat_I_DP &v, Vec_I_DP &w, Mat_O_DP &cvm);
	void toeplz(Vec_I_DP &r, Vec_O_DP &x, Vec_I_DP &y);
	void tptest(Vec_I_DP &data1, Vec_I_DP &data2, DP &t, DP &prob);
	void tqli(Vec_IO_DP &d, Vec_IO_DP &e, Mat_IO_DP &z);
	DP trapzd(DP func(const DP), const DP a, const DP b, const int n);
	void tred2(Mat_IO_DP &a, Vec_O_DP &d, Vec_O_DP &e);
	void tridag(Vec_I_DP &a, Vec_I_DP &b, Vec_I_DP &c, Vec_I_DP &r, Vec_O_DP &u);
	DP trncst(Vec_I_DP &x, Vec_I_DP &y, Vec_I_INT &iorder, Vec_IO_INT &n);
	void trnspt(Vec_IO_INT &iorder, Vec_I_INT &n);
	void ttest(Vec_I_DP &data1, Vec_I_DP &data2, DP &t, DP &prob);
	void tutest(Vec_I_DP &data1, Vec_I_DP &data2, DP &t, DP &prob);
	void twofft(Vec_I_DP &data1, Vec_I_DP &data2, Vec_O_DP &fft1,
		Vec_O_DP &fft2);
	void vander(Vec_I_DP &x, Vec_O_DP &w, Vec_I_DP &q);
	void vegas(Vec_I_DP &regn, DP fxn(Vec_I_DP &, const DP), const int init,
		const int ncall, const int itmx, const int nprn, DP &tgral, DP &sd,
		DP &chi2a);
	void voltra(const DP t0, const DP h, Vec_O_DP &t, Mat_O_DP &f,
		DP g(const int, const DP),
		DP ak(const int, const int, const DP, const DP));
	void wt1(Vec_IO_DP &a, const int isign,
		void wtstep(Vec_IO_DP &, const int, const int));
	void wtn(Vec_IO_DP &a, Vec_I_INT &nn, const int isign,
		void wtstep(Vec_IO_DP &, const int, const int));
	void wwghts(Vec_O_DP &wghts, const DP h,
		void kermom(Vec_O_DP &w, const DP y));
	bool zbrac(DP func(const DP), DP &x1, DP &x2);
	void zbrak(DP fx(const DP), const DP x1, const DP x2, const int n,
		Vec_O_DP &xb1, Vec_O_DP &xb2, int &nroot);
	DP zbrent(DP func(const DP), const DP x1, const DP x2, const DP tol);
	void zrhqr(Vec_I_DP &a, Vec_O_CPLX_DP &rt);
	DP zriddr(DP func(const DP), const DP x1, const DP x2, const DP xacc);
	void zroots(Vec_I_CPLX_DP &a, Vec_O_CPLX_DP &roots, const bool &polish);

} // namespace NR

typedef struct { char * pScanBuffer; int BufferSize; } ScanBuffer;
struct ScanFile 
	{ 
		CString filename; 
		int charcount; 
		int lineno; 
		
		ScanFile(CString fname=CString("")): 
		 filename(fname), charcount(0), lineno(0) {}
	} ;

class LineCharPair 
{	
public:

	int linecount, charcount; 
	
	LineCharPair(int lc=0, int cc=0): 
		linecount(lc), charcount(cc) {}
};

inline std::ostream& operator<<(std::ostream& os, const voidtype& )
{
		return os;
}

class Array; // forward declaration
class Data;  // forward declaration

struct VarType { wchar_t bufr[BUF_LENGTH]; Data* dPtr;  };

inline VarType MakeVarType(const wchar_t *str=NULL, Data *ptr=NULL)
{
	VarType vt;

	extern void StrCopy(wchar_t* tgt, const wchar_t* src);
	if(str)  StrCopy(vt.bufr, str);
	else vt.bufr[0]=L'\0';
	
	vt.dPtr = ptr;
	
	return vt;
}

inline void CopyVarType(VarType& tgt, const VarType& src)
 {
  extern void StrCopy(wchar_t* tgt, const wchar_t* src);

  StrCopy(tgt.bufr, src.bufr);
  tgt.dPtr = src.dPtr;
}

class DataReturn
{
public:

	DataType type;
	bool	 booln;
	integer  intg;
	realnum  dble;
	Complex  cplx;
	Vector	 vctr;

	DataReturn(): 
		type(TypeNull), booln(false), intg(0), dble(0.0),
		cplx(Complex(0, 0)), vctr(){}

	DataReturn(const DataReturn& dc)
	{
		type = dc.type;
		booln = dc.booln;
		intg = dc.intg;
		dble = dc.dble;
		cplx = dc.cplx;
		vctr = dc.vctr;
	}
} ;

#ifndef MAX_DIMENSION
#define MAX_DIMENSION 5
#endif 

/*
typedef struct
{
	int index[MAX_DIMENSION];
	int maxidx[MAX_DIMENSION];
	int maxdim;

	int i, j, k, dim;

	VarType vt;

} ArrayInit;

inline ArrayInit MakeAI()
{
	ArrayInit ai;

	ai.maxdim = 0;
	ai.i = ai.j = ai.k = ai.dim = 0;

	ai.vt.bufr[0]='\0'; ai.vt.dPtr = NULL;

	for(int i=0; i<MAX_DIMENSION; ++i)
		ai.index[i] = ai.maxidx[i] = 0;
	
	return ai;
}
*/

class BaseType
{
public:
	
	class BaseException: public UnicodeException
	{
		UINT rcid;
	public:
		BaseException(UINT nID): rcid(nID) { }

		virtual void PutMsg() const
		{
			CString msg; msg.LoadString(rcid);
			extern void PutString(const CString&);
			PutString(msg);
		}
		virtual ~BaseException() {}
	};

	BaseType() { }
	virtual ~BaseType() { }
};

class Data: public BaseType
{
protected:
	//DataType dtype; std::wstring id;
	DataPair datasig; bool static_data;
	
	void copy(const Data& data)
	{
		this->datasig = data.datasig;
		this->static_data = data.static_data;
	}
	
	const Data& get_data() const { return *this; }

public:
	
	Data(std::wstring varid=_T(""), DataType type = TypeNull, bool is_static=false):
	  datasig(type, varid), static_data(is_static) { }

	Data(const Data& data) { this->copy(data); }
	Data& operator = (const Data& data) { this->copy(data); return *this; }

	virtual void Push() { ; }
	virtual void Pop() { ; }

	bool IsStatic() const { return static_data; }
	void SetStatic(bool bstatic=true) { static_data = bstatic; }
    const DataType& GetType() const { return datasig.first; }
	const std::wstring& GetId() const { return datasig.second; }
	const DataPair& GetDataSig() const { return this->datasig; }

	virtual ~Data() { }
};

template<typename T>
class ptrvector: public std::vector<T*>
{
protected:
	bool deleted;
	
	typename std::vector<T*>::size_type preserve;

	void clearall()
	{
		if(deleted==false)
		{
			for(std::vector<T*>::size_type i = this->preserve; i < this->size(); ++i)
					delete this->at(i);
			
			this->clear();	deleted = true;
		}
	}
public:

	ptrvector(T* tptr=NULL): deleted(false)
	{
		this->preserve = 0;

		if(tptr != NULL) this->push_back(tptr);
	}

	void SetPreserve() { this->preserve = this->size(); }

	void push_back(T* pVal)
	{
		if(pVal)
		{
			std::vector<T*>::push_back(pVal); deleted=false;
		}
	}

	void Push()
	{
		std::vector<T*>::iterator itr;
		for(itr = std::vector<T*>::begin(); 
			itr != std::vector<T*>::end(); ++itr)
			if(!(*itr)->IsStatic()) (*itr)->Push();
	}

	void Pop()
	{
		std::vector<T*>::iterator itr;
		for(itr = std::vector<T*>::begin(); 
			itr != std::vector<T*>::end(); ++itr)
			if(!(*itr)->IsStatic()) (*itr)->Pop();
	}

	void Delete() { clearall(); }
	~ptrvector() { clearall(); }
};


#pragma warning(disable: 4996)


template<typename T> class PtrStack
{
protected:
	// container for the elements
	std::deque<T*> c; 
	mutable int count;

#ifdef _DEBUG
	std::wstring name;
#endif

public:
	/* exception class for pop() and top() with empty stack */
#ifdef _DEBUG
	PtrStack(const std::wstring label=L"No Name"): count(0), name(label) { }
#else
	PtrStack(): count(0) { }
#endif
	
	// number of elements
	const int& size() const { return count; }

	// is stack empty?
	bool empty() const { return count==0; }

	// push element into the stack
	void push(T* elem) 
	{ 
		c.push_back(elem); ++count;

#ifdef _DEBUG
		//TRACE(_T("PtrStack: %S, push, count: %u\r\n"), name.c_str(), c.size());
		std::wostringstream os;
		os<<_T("PtrStack[")<<name<<_T("]:  push count: ")<<c.size()<<_T("\r\n");
		PutString(os.str());
#endif
	}

	T* replace(T* elem)
	{
		T* ptr = c.back(); c.pop_back(); c.push_back(elem);
		return ptr;
	}

	void swaptop(PtrStack* stack)
	{
		this->replace(stack->replace(this->top()));
	}

	// pop element out of the stack and return its value
	T* pop()
	{
		if(count==0)
			//throw PtrStackException(_T("PtrStack::pop(), statck empty.\r\n"));
			HandleSyntaxError(L"Stack overflow. Possibly Syntax Error.\r\n");
		
		T* elem(c.back()); c.pop_back(); --count;

#ifdef _DEBUG
		//TRACE(_T("PtrStack: %S, pop, count: %u\r\n"), name.c_str(), c.size());
		std::wostringstream os;
		os<<_T("PtrStack[")<<name<<_T("]:  pop count: ")<<c.size()<<_T("\r\n");
		PutString(os.str());
#endif

		return elem;
	}

	T* bottom()
	{
		if(count>0) return c[0];
		else
			//throw PtrStackException(_T("PtrStack::bottom(), Reached bottom of the stack.\r\n"));
			HandleSyntaxError(L"Stack Overflow. Possibly Syntax Error.\r\n");
	}

	std::deque<T*>& GetDeque() { return c;}

	void clear() 
	{
		for(int i=0; i<count; ++i) delete c[i];
		c.clear(); count=0;
	}
	
	T* getoffset(int offset)
	{
		int index = count-offset-1;

		if(index>=0 && index<count) return c[index];
		else 
			//throw PtrStackException(_T("PtrStack::getoffset(), offset out of range.\r\n"));
			HandleSyntaxError(L"Stack Overflow. Possibly Syntax Error.\r\n");
	}

	// return the value of next element
	
	T* top()
	{
		if(count>0) return c.back();
		else 
			//throw PtrStackException(_T("PtrStack::top(), stack empty.\r\n"));
			HandleSyntaxError(L"Stack Overflow. Possibly Syntax Error.\r\n");
		
		return c.back() ; // to suppress warning
	}

	// return the value of next element
	const T* top() const
	{
		if(count>0) return c.back();
		else 
			//throw PtrStackException(_T("PtrStack::top()const, stack empty.\r\n"));
			HandleSyntaxError(L"Stack Overflow. Possibly Syntax Error.\r\n");
		
		return c.back();	// to suppress warning
	}

	~PtrStack() { clear(); }
};

template<typename T>
class Stack
{
protected:
	// container for the elements
	std::deque<T> c; 
	mutable int count;

#ifdef _DEBUG
	std::wstring name;
#endif

public:
	/* exception class for pop() and top() with empty stack */
#ifdef _DEBUG
	Stack(const std::wstring& label): count(0), name(label) { }
#else
	Stack(): count(0) { }
#endif
	
	// number of elements
	const int& size() const { return count; }

	// is stack empty?
	bool empty() const { return count==0; }

	// push element into the stack
	void push(const T& elem) 
	{ 
		c.push_back(elem); ++count;
#ifdef _DEBUG
		//TRACE("Stack: %S, push, count: %u\r\n", name.c_str(), c.size());
		std::wostringstream os;
		os<<_T("Stack[")<<name<<_T("]:  push count: ")<<c.size()<<_T("\r\n");
		PutString(os.str());
#endif
	}

	// pop element out of the stack and return its value
	T pop()
	{
		if(count==0)
			//throw StackException(_T("Stack::pop(), stack empty.\r\n"));
			HandleSyntaxError(L"Stack Overflow. Possibly Syntax Error.\r\n");
		
		T elem(c.back()); c.pop_back(); --count;

#ifdef _DEBUG
		//TRACE(_T("Stack: %S, pop, count: %u\r\n"), name.c_str(), c.size());
		std::wostringstream os;
		os<<_T("Stack[")<<name<<_T("]:  pop count: ")<<c.size()<<_T("\r\n");
		PutString(os.str());
#endif
	
		return elem;
	}

	T bottom()
	{
		if(count>0) return c[0];
		else
			//throw StackException(_T("Stack::bottom(), reach bottom of the stack.\r\n"));
			HandleSyntaxError(L"Stack Overflow. Possibly Syntax Error.\r\n");
	}

	void clear() { c.clear(); count=0; }
	
	T getoffset(int offset)
	{
		int index = count-offset-1;

		if(index>=0 && index<count) return c[index];
		else //throw StackException(_T("Stack::getoffset(), offset out of range.\r\n"));
			HandleSyntaxError(L"Stack Overflow. Possibly Syntax Error.\r\n");

		return c[index];		// to suppress warning
	}

	// return the value of next element
	T& top()
	{
		if(count>0) return c.back();
		else //throw StackException(_T("Stack::top(), stack empty.\r\n"));
			HandleSyntaxError(L"Stack Overflow. Possibly Syntax Error.\r\n");
		
		return c.back();	// to suppress warning
	}
	~Stack() { clear(); }
};

template<typename T>
class DataTemp: public Data
{
protected:
	PtrStack<T> theStack;
public:
	DataTemp(std::wstring varid, bool vstatic)

#ifdef _DEBUG
		:theStack(varid)
#else
		:theStack()
#endif
	  {	
		  datasig.second = varid;

		  const type_info& type = typeid(T);
		  
		  if(type == typeid(bool))					datasig.first = TypeBool;
		  else if(type == typeid(integer))			datasig.first = TypeIntg;
		  else if(type == typeid(realnum))			datasig.first = TypeReal;
		  else if(type == typeid(Complex))			datasig.first = TypeCplx;
		  else if(type == typeid(Vector))			datasig.first = TypeVctr;
		  else if(type == typeid(string))			datasig.first = TypeStrn;
		  
		 // else if(type == typeid(bool*))			datasig.first = TypeBoolP1;
		  else if(type == typeid(integer*))			datasig.first = TypeIPtr;
		  else if(type == typeid(realnum*))			datasig.first = TypeRPtr;
		  else if(type == typeid(Complex*))			datasig.first = TypeCPtr;
		  else if(type == typeid(Vector*))			datasig.first = TypeVPtr;
		  //else if(type == typeid(string*))			datasig.first = TypeStrnP1;
		  //else if(type == typeid(stream*))			datasig.first = TypeStrmP1;

		  else										datasig.first = TypeNull;

		   Data::static_data = vstatic;

		  if(vstatic) Push(); 
	  }

	void Swap(DataTemp<T>* pData)
	{  
		theStack.swaptop(&pData->theStack);
	}

	T& GetData() { return *theStack.top(); }
	T* GetPtr() { return  theStack.top(); }
	operator T&() { return GetData(); }

	T& operator=(const T& rhs) { return *theStack.top() = rhs; }
	DataTemp& operator=(const DataTemp& rhs) 
	{ *theStack.top() = *rhs.theStack.top(); return *this; }

	virtual void Push() { theStack.push(new T()); }
	virtual void Pop() { delete theStack.pop(); }

	virtual ~DataTemp() 
	{ 
		
#ifdef _DEBUG 
		if(!theStack.empty() && !Data::static_data)
		{
			std::wostringstream os;
			os<<L"DataTemp Stack for \'"<<datasig.second
				<<L"\' is not properly cleaned. Stack Depth: "<<int(theStack.size())<<L"\r\n";
			PutString(os.str());
			
		}
#endif
		theStack.clear();
	}
};

typedef DataTemp<bool>			BoolData;
typedef DataTemp<integer>		IntgData;
typedef DataTemp<realnum>		RealData;
typedef DataTemp<Complex>		CplxData;
typedef DataTemp<Vector>		VctrData;
typedef DataTemp<string>		StrnData;
typedef DataTemp<stream>		StrmData;

typedef DataTemp<bool*>			BoolDataP1;
typedef DataTemp<integer*>		IntgDataP1;
typedef DataTemp<realnum*>		RealDataP1;
typedef DataTemp<Complex*>		CplxDataP1;
typedef DataTemp<Vector*>		VctrDataP1;
typedef DataTemp<string*>		StrnDataP1;
typedef DataTemp<stream*>		StrmDataP1;

typedef DataTemp<bool**>		BoolDataP2;
typedef DataTemp<integer**>		IntgDataP2;
typedef DataTemp<realnum**>		RealDataP2;
typedef DataTemp<Complex**>		CplxDataP2;
typedef DataTemp<Vector**>		VctrDataP2;
typedef DataTemp<string**>		StrnDataP2;
typedef DataTemp<stream**>		StrmDataP2;

typedef DataTemp<bool***>		BoolDataP3;
typedef DataTemp<integer***>	IntgDataP3;
typedef DataTemp<realnum***>	RealDataP3;
typedef DataTemp<Complex***>	CplxDataP3;
typedef DataTemp<Vector***>		VctrDataP3;
typedef DataTemp<string***>		StrnDataP3;
typedef DataTemp<stream***>		StrmDataP3;

class Function;

class UDFuncList
{
public:
	
	typedef std::pair<std::wstring, Function*> FuncPair;
	typedef stdext::hash_map<std::wstring, Function*> FuncMap;
	typedef FuncMap::iterator FuncMapItr;
	typedef FuncMap::const_iterator FuncMapCItr;
	
	FuncMap theMap;
	
	void copy(const UDFuncList& udf) { this->theMap = udf.theMap; }

public:
	
	UDFuncList() {}

	void InsertFunction(const wchar_t* name, Function* fptr)
	{
		if(!theMap.insert(FuncPair(std::wstring(name), fptr)).second)
		{
			std::wostringstream os;
			os<<L"Function \'"<<name<<L"\' is already defined.\r\n";
			HandleSyntaxError(os.str());
		}
	}
	
	Function* FindFunction(const std::wstring& name)
	{
		FuncMapItr itr = theMap.find(name);
		if(itr != theMap.end()) return itr->second;
		else return NULL;
	}

	void CheckImplemented() const;

	UDFuncList(const UDFuncList& udf) { copy(udf); }

	UDFuncList& operator=(const UDFuncList& udf) { copy(udf); return *this; }
};

extern Function* FindFunction(const wchar_t* id);

class PtrDataBase: public Data
{
protected:
	DataType eletype;
public:
	PtrDataBase(std::wstring vrid=L"Temporary PtrData", bool vstatic=false):
	  Data(vrid, TypePtr, vstatic) { }

	DataType GetEleType() const { return eletype; }
	virtual ~PtrDataBase() {}
};

template<typename T>
class PtrData: public PtrDataBase
{
protected:
	PtrStack<T*> theStack;

	void copy(const PtrData& data)
	{	
		if(theStack.empty()) Push();
		*theStack.top() = data.GetData();
	}

	void SetElementType()
	{
		const type_info& type=typeid(T);

		if(type==typeid(bool))			eletype = TypeBool;
		else if(type==typeid(integer))	eletype = TypeIntg;
		else if(type==typeid(realnum))	eletype = TypeReal;
		else if(type==typeid(Complex))	eletype = TypeCplx;
		else if(type==typeid(Vector))	eletype = TypeVctr;
		else if(type==typeid(string))	eletype = TypeStrn;
		else HandleSyntaxError(_T("Unsupported pointer data type.\r\n"));
	}

public:
	
	PtrData(std::wstring vrid=L"Temporary PtrData", bool vstatic=false): 
	  PtrDataBase(vrid, vstatic),

#ifdef _DEBUG
	  theStack(vrid)
#else
	  theStack()
#endif
	  { 
		  if(vstatic) Push(); 
		  SetElementType();
	  }

	 PtrData(const PtrData& data) 
	 { 
		 if(const_cast<PtrData*>(&data)->GetDataPtr())
		 {
			Data::datasig.first = L"Temporary PtrData";
			Data::datasig.second = TypePtr;

			SetElementType(); copy(data); 
		 }
	 }

   void Swap(PtrData* pData)
	{
		T** lhs = this->theStack.top();
		T** rhs = pData->theStack.top();
		T* value = *lhs; *lhs = *rhs; *rhs = value;
	}

	PtrData& operator = (const PtrData& data)
	{
		if(this != &data) copy(data);
		return *this;
	}

	T* GetData() 
	{ 		
		if(!theStack.empty() && *theStack.top())
			return *theStack.top();
		else
		{
			std::wostringstream os;
			os<<L"Pointer type variable \'"
				<<Data::GetId()<<L"\' is not yet initialized.\r\n";
			
			HandleSyntaxError(os.str());

			return NULL; // to suppress warning
		}
	}

	T* GetDataPtr() 
	{ 		
		if(!theStack.empty() && *theStack.top())
			return *theStack.top();

		else return NULL;
	}
	
	T** GetPtr() 
	{ 
		if(!theStack.empty())
			return theStack.top();
		else
		{
			std::wostringstream os;
			os<<L"Pointer variable \'"
				<<Data::GetId()<<L"\' is not yet initialized.\r\n";
			
			HandleSyntaxError(os.str());

			return NULL; // to suppress warning
		}
	}
	
	virtual void Push() 
	{ theStack.push(new T*); *theStack.top() = (T*)NULL; }
	virtual void Pop() 
	{ 
		T** pptr = theStack.pop(); 
		delete pptr; 
	}
	
	virtual ~PtrData() 
	{  

#ifdef _DEBUG 
		
		if(!theStack.empty() && !Data::static_data)
		{
			std::wostringstream os;
			os<<L"PtData Stack for \'"<<datasig.second
				<<L"\' is not properly cleaned. Stack Depth: "<<int(theStack.size())<<L"\r\n";
			PutString(os.str());
		}
#endif
		theStack.clear();
	}
};