#pragma once

#include <kch_objects.h>
#include <kch_figures.h>

#include "resource.h"

#define OFFSET_X 15
#define OFFSET_Y 15

///////////////////////////////////////////
//
//	class KSheet: public CObject
//
///////////////////////////////////////////
class AFX_EXT_CLASS KSheet: public CObject
{
	// methods
public:
	DECLARE_COUNT

	KSheet(int sheet=1, const CRect& r=CRect(0, 0, 0, 0));
	virtual ~KSheet();
	virtual void Serialize(CArchive& ar);

	void DrawDisplay(CDC* pDC, float zoom, BOOL current);
	void DrawPrinter(CDC* pDC, float zoom);
	
	void DrawTail(CDC* pDC, float zoom);
	
	// removes tail element, and return bounding rectangle at rect
	// figures' zoom factor zoom if success, return value is TRUE
	// else FALSE
	BOOL KSheet::RemoveTail(KFigures &figure);

	// sets sheet bounding rect, m_Bound
	void SetBound(int sheet, const CRect& r);

	// draws current sheet bound
	void DrawCurrentSheet(CDC* pDC, BOOL flag, float zoom) const;
	BOOL PtInRect(const CPoint& pt) const;
	KPoint GetRefPt() const { return KPoint(m_Bound.left, m_Bound.top); }

	KFigures* KSheet::InsertFigure(FigureType figure, 
		float zoom, int thick, const KColor& color, BOOL antialias);

	//attributes
public:
	int m_Number; // sheet number
	CRect m_Bound; // sheet bound
protected:
	KTypedPtrList<CObList, KFigures*> m_Figures;
		
	DECLARE_SERIAL(KSheet)
};

///////////////////////////////////////////
//
//	class KChapter: public CObject
//
///////////////////////////////////////////
class AFX_EXT_CLASS KChapter: public CObject
{
	// methods
public:
	DECLARE_COUNT
	
	KChapter();
	virtual ~KChapter();
	virtual void Serialize(CArchive& ar);

	// print draw
	void DrawPrinter(CDC* pDC, int sheetNo, BOOL preview);
		
	// onscreen draw
	void DrawDisplay(CDC* pDC, const CRect& clipRect, float zoom);

	int GetSheetCount() const;

	CRect GetUpdateRect(CDC* pDC);

	KSheet* GetSheet();
	KSheet* GetSheet(int index);
	KSheet* GetSheet(const CPoint& point, BOOL& inside, float zoom);
	KSheet* AppendSheet();
	KSheet* InsertSheet(int index);
	KSheet* RemoveSheet(int index, int count=1);
	int SetCurrentSheet(const CPoint& point, BOOL& inside, CDC* pDC, float zoom);
	int WhichSheet(const CPoint& point, BOOL& inside, float zoom) const;

	CSize GetViewBound(float zoom) const;
	
	void SwapWithNext();
	void SwapWithPrevious();
	
	// attributes
public:
	//float m_Zoom; // zoom factor

	CRect m_Bound; // sheet size
	int m_Index; // current sheet index
	HWND m_hWnd; // client window handle
	
	//UINT m_HorzPos; // horizontal scroll position
	//UINT m_VertPos; // vertical scroll position

	BOOL partialUpdate;
	CDocument* m_pDoc;

	///////////////////////////
	KPaperMetric m_ScreenPaper;
	KPaperMetric m_PrinterPaper;
	
	// Screen-to-Printer horizontal dpi ratio
	float GetSPRatioX() const; 
	// Screen-to-Printer verical dpi ratio
	float GetSPRatioY() const;

	// Printer-to-Screen horizontal dpi ratio
	float GetPSRatioX() const; 
	// Printer-to-Screen verical dpi ratio
	float GetPSRatioY() const;

protected:
	KTypedPtrArray<CObArray, KSheet*> m_Sheets;

protected:
	DECLARE_SERIAL(KChapter)
};