#pragma once

#if !defined(WINVER)
#define WINVER 0x0501
#endif

#include <afxwin.h>
#include <afxtempl.h>
#include <afxcoll.h>
#include <tchar.h>
#include <gdiplus.h>

#if defined(_DEBUG)
	#define DECLARE_COUNT	static unsigned long element_count;
	#define IMPLEMENT_COUNT(classname)	unsigned long classname::element_count=0;
	#define INCREMENT_COUNT	++element_count;
	#define	DECREMENT_COUNT	--element_count;
#else
	#define DECLARE_COUNT
	#define IMPLEMENT_COUNT(classname)
	#define INCREMENT_COUNT
	#define	DECREMENT_COUNT
#endif

/*
#ifdef _DEBUG
#define new DEBUG_NEW
#endif
*/

namespace siliners
{
	enum FigureType { FigureFreehand, FigureStraight, 
		              FigurePolyline, FigurePolygon };

	// GDI Primary Colors
	enum PrimaryColor { Black =			RGB(0,   0,   0), 
						Blue =			RGB(0,   0,   192),
						Green =			RGB(0,   192, 0),
						Cyan =			RGB(0,   192, 192),
						Red =			RGB(192, 0,   0),
						Magenta =		RGB(192, 0,   192),
						Yellow =		RGB(192, 192, 0),
						DarkGray =		RGB(128, 128, 128),
						LightGray =		RGB(192, 192, 192),
						BrightBlue =	RGB(0,   0,   255),
						BrightGreen =	RGB(0,   255, 0),
						BrightCyan =	RGB(0,   255, 255),
						BrightRed =		RGB(255, 0,   0),
						BrightMagenta = RGB(255, 0,   255),
						BrightYellow =	RGB(255, 255, 0),
						White =			RGB(255, 255, 255) };
	// GDI Primary Colors
	static unsigned long PrimaryColors[]= 
					  { /* Black */ RGB(0,   0,   0), 
						/* Blue */ RGB(0,   0,   192),
						/* Green */ RGB(0,   192, 0),
						/* Cyan */ RGB(0,   192, 192),
						/* Red */ RGB(192, 0,   0),
						/* Magenta */ RGB(192, 0,   192),
						/* Yellow */ RGB(192, 192, 0),
						/* DarkGray */ RGB(128, 128, 128),
						/* LightGray */ RGB(192, 192, 192),
						/* BrightBlue */ RGB(0,   0,   255),
						/* BrightGreen */ RGB(0,   255, 0),
						/* BrightCyan */ RGB(0,   255, 255),
						/* BrightRed */ RGB(255, 0,   0),
						/* BrightMagenta */ RGB(255, 0,   255),
						/* BrightYellow */ RGB(255, 255, 0),
						/* White */ RGB(255, 255, 255) };


	// Used to split array or list
	enum IndexPosition {IndexTail, // index elemnet at the tail
						IndexHead, // index element at the head
						IndexRemove	/* remove index element */ };

	/*
		typename TYPE: class *N*O*T* derived from CObject
		typename BASE_CLASS: CPtrList

		Ex: KTypedPtrList<CPtrList, MyPoint*> myptList;
		
		typename TYPE: class derived from CObject
		typename BASE_CLASS: CObList
        
		Ex: KTypedPtrList<CObList, ObPoint*> obptList;

		void RemoveAll() deletes all element and the list itself

		The destructor automatically calls RemoveAll() method
	*/
	template<typename BASE_CLASS, typename TYPE> class KTypedPtrList: 
			public CTypedPtrList<BASE_CLASS, TYPE>
	{
	public:
		KTypedPtrList() { /* empty */}
		
		void RemoveAll() 
		{
			POSITION pos = this->GetHeadPosition();
			while(pos) delete this->GetNext(pos);
			CTypedPtrList<BASE_CLASS, TYPE>::RemoveAll();
		}

		void EmptyAll() 
		{
			CTypedPtrList<BASE_CLASS, TYPE>::RemoveAll();
		}

		void Copy(const KTypedPtrList& theList)
		{
			this->RemoveAll();

			POSITION pos = theList.GetHeadPosition();

			while(pos)
				this->AddTail(theList.GetNext(pos)->Clone());
		}

		void PutDown(TYPE pObject)
		{
			POSITION posPrev, posHead, posInsert, pos;
			posHead = pos = this->GetHeadPosition();

			TYPE pObj, pObjPrev=NULL; 

			while(pos)
			{
				posPrev = pos;
				
				pObj = this->GetNext(pos);

				if(pObj == pObject)
				{
					if(posPrev != posHead)
					{
						this->SetAt(posPrev, pObjPrev);
						this->SetAt(posInsert, pObject);
					}

					return;
				}
				
				pObjPrev = pObj;  posInsert = posPrev;
			}
		}

		void PutUp(TYPE pObject)
		{
			POSITION posNext, posTail, posInsert, pos;
			posTail = pos = this->GetTailPosition();

			TYPE pObj, pObjNext; 

			while(pos)
			{
				posNext = pos;
				
				pObj = this->GetPrev(pos);

				if(pObj == pObject)
				{
					if(posNext != posTail)
					{
						this->SetAt(posNext, pObjNext);
						this->SetAt(posInsert, pObject);
					}

					return;
				}
				
				pObjNext = pObj;  posInsert = posNext;
			}
		}

		
		virtual ~KTypedPtrList() { RemoveAll(); }
	};

	/*
		typename TYPE: class *N*O*T* derived from CObject
		typename BASE_CLASS: CPtrArray

		Ex: KTypedPtrArray<CPtrArray, MyPoint*> myptArray;
		
		typename TYPE: class derived from CObject
		typename BASE_CLASS: CObArray
        
		Ex: KTypedPtrArray<CObArray, ObPoint*> obptArray;
	*/
	template<typename BASE_CLASS, typename TYPE> class KTypedPtrArray: 
			public CTypedPtrArray<BASE_CLASS, TYPE>
	{
	public:
		KTypedPtrArray() { /* empty */}
		
		void RemoveAll() 
		{
			for(int i=0; i<GetSize(); i++) 
			{
				delete this->GetAt(i);
			}

            CTypedPtrArray<BASE_CLASS, TYPE>::RemoveAll();
		}

		void EmptyAll() 
		{
			CTypedPtrArray<BASE_CLASS, TYPE>::RemoveAll();
		}

		
		void Copy(const KTypedPtrArray& theArray)
		{
			this->RemoveAll();

			for(int i=0; i<GetSize(); ++i)
			{
				this->Add(theArray.GetAt(i)->Clone());
			}
		}
		
		virtual ~KTypedPtrArray() { RemoveAll(); } 
	};

	/* WARNIG: Default ARG_TYPE is TYPE, NOT const TYPE& */
	template<typename TYPE, typename ARG_TYPE=TYPE> class KArray: 
		public CArray<TYPE, ARG_TYPE>
	{
	public:
		KArray() { /* empty */ }

		void Remove(const TYPE& arg)
		{
			for(__w64 int i=GetSize()-1; i>=0; --i)
			{
				if(GetAt(i) == arg) RemoveAt(i);
			}
			FreeExtra();
		}
		
		/* Split array into list
		   When TYPE is derived from CObject,
		   BASE_CLASS is CObArray, otherwise CPtrArray
		   
		   Return:
		   if index is out of range, returns NULL

		   if successfull, KTypedPtrArray<BASE_CLASS, KArray*>* is returend   */
		
		template<typename BASE_CLASS>
		KTypedPtrArray<BASE_CLASS, KArray*>* SplitToArray(int index, IndexPosition pos=IndexTail)
		{
			// return NULL, if split index is
			// greater than or equal to GetSize()
			if(index < 0 || index >= GetSize()) return NULL;
			
			KArray<TYPE, ARG_TYPE> *lArray, *rArray;
			
			lArray = new KArray<TYPE, ARG_TYPE>;
			rArray = new KArray<TYPE, ARG_TYPE>;

			int i;
			switch(pos)
			{
			case IndexHead: // index element at the head

				for(i = 0; i<index; i++) lArray->Add(GetAt(i));
				for(i=index; i<GetSize(); i++) rArray->Add(GetAt(i));
				break;
			
			default: // IndexTail & IndexRemove
				for(i = 0; i<=index; i++) lArray->Add(GetAt(i));
				for(i=index+1; i<GetSize(); i++) rArray->Add(GetAt(i));
			}

			KTypedPtrArray<BASE_CLASS, KArray*> *array = new KTypedPtrArray<BASE_CLASS, KArray*>;

			if(lArray->GetSize()>0)
			{
				lArray->FreeExtra(); array->Add(lArray); 
			}
			if(rArray->GetSize()>0)
			{
				rArray->FreeExtra(); array->Add(rArray); 
			}

			array->FreeExtra();
			return array;
		}
		
		// Returns index for element ele
		// if fails, returns -1
		int FindIndex(TYPE ele, int start) const
		{
			for(int i=start; i<GetSize(); i++)
			{
				if(ele == this->GetAt(i)) return i;
			}
			return -1; // Failed
		}

		CUIntArray* FindIndex(TYPE ele) const
		{
			CUIntArray* array=new CUIntArray;
			int start;
            
            start = FindIndex(ele, 0);
			while(start != -1)
			{
				array->Add(start);
				start = FindIndex(ele, start+1);
			}

			return array;
		}

		/* Split array into list
		   When TYPE is derived from CObject,
		   BASE_CLASS is CObArray, otherwise CPtrArray
		   
		   Return:
		   if element ele is not found, returns NULL

		   if successfull, KTypedPtrArray<BASE_CLASS, KArray*>* is returend   */

		template<typename BASE_CLASS>
		KTypedPtrArray<BASE_CLASS, KArray*>* SplitToArray(TYPE ele, IndexPosition pos=IndexTail)
		{
			KTypedPtrArray<BASE_CLASS, KArray*> *array, *tmpArray;
			KArray *tmpType;
            
			int start = FindIndex(ele, 0);

			// If element ele does not exist, return NULL
			if(start == -1) return NULL; 
	
			array = new KTypedPtrArray<BASE_CLASS, KArray*>;
            
			if((ele==GetAt(0))&&(pos==IndexHead))
                tmpArray = SplitToArray<BASE_CLASS>(start+1, pos);
			else
				tmpArray = SplitToArray<BASE_CLASS>(start, pos);
			
			while(tmpArray)
			{
				array->Add(tmpArray->GetAt(0));
				
				if(tmpArray->GetSize()==1)
					break;
				else
					tmpType=tmpArray->GetAt(1);

				switch(pos)
				{
				case IndexHead:
					start = tmpType->FindIndex(ele, 1); 
					break;
				
				default: // IndexTail & IndexRemove
				    start = tmpType->FindIndex(ele, 0); 
				}
						
				if(start == -1) 
				{
					array->Add(tmpType);
					break;
				}
				else
				{
					tmpArray = tmpType->SplitToArray<BASE_CLASS>(start, pos);
					delete tmpType;
				}
			}

			if(pos == IndexRemove)
			{
				for(__w64 int i=array->GetSize()-1; i>=0; --i)
				{
					array->GetAt(i)->Remove(ele);
					if(array->GetAt(i)->GetSize() == 0)
						array->RemoveAt(i);
				}
			}
			array->FreeExtra();
			return array;
		}
		
		template<typename BASE_CLASS>
		KTypedPtrList<BASE_CLASS, KArray*>* SplitToList(int index, IndexPosition pos=IndexTail)
		{
			// return NULL, if split index is
			// greater than or equal to GetSize()
			if(index < 0 || index >= GetSize()) return NULL;
			
			KArray<TYPE, ARG_TYPE> *lArray, *rArray;
			
			lArray = new KArray<TYPE, ARG_TYPE>;
			rArray = new KArray<TYPE, ARG_TYPE>;

			int i;
			switch(pos)
			{
			case IndexHead: // index element at the head

				for(i = 0; i<index; i++) lArray->Add(GetAt(i));
				for(i=index; i<GetSize(); i++) rArray->Add(GetAt(i));
				break;
			
			default: // IndexTail & IndexRemove
				for(i = 0; i<=index; i++) lArray->Add(GetAt(i));
				for(i=index+1; i<GetSize(); i++) rArray->Add(GetAt(i));
			}

			KTypedPtrList<BASE_CLASS, KArray*> *list = new KTypedPtrList<BASE_CLASS, KArray*>;

			if(lArray->GetSize()>0)
			{
				lArray->FreeExtra(); list->AddTail(lArray); 
			}
			if(rArray->GetSize()>0)
			{
				rArray->FreeExtra(); list->AddTail(rArray); 
			}

			return list;
		}

		template<typename BASE_CLASS>
		KTypedPtrList<BASE_CLASS, KArray*>* SplitToList(TYPE ele, IndexPosition pos=IndexTail)
		{
			KTypedPtrList<BASE_CLASS, KArray*> *list, *tmpList;
			KArray *tmpType, *tmpType2;
            
			int start = FindIndex(ele, 0);

			// If element ele does not exist, return NULL
			if(start == -1) return NULL; 
	
			list = new KTypedPtrList<BASE_CLASS, KArray*>;
            
			if((ele==GetAt(0))&&(pos==IndexHead))
                tmpList = SplitToList<BASE_CLASS>(start+1, pos);
			else
				tmpList = SplitToList<BASE_CLASS>(start, pos);
			
			POSITION posTmpList = tmpList->GetHeadPosition();
            while(posTmpList)
			{
				list->AddTail(tmpList->GetNext(posTmpList));
				
				if(tmpList->GetSize()==1)
					break;
				else
					tmpType=tmpList->GetNext(posTmpList);

				switch(pos)
				{
				case IndexHead:
					start = tmpType->FindIndex(ele, 1); 
					break;
				
				default: // IndexTail & IndexRemove
				    start = tmpType->FindIndex(ele, 0); 
				}
						
				if(start == -1) 
				{
					list->AddTail(tmpType);
					break;
				}
				else
				{
					tmpList = tmpType->SplitToList<BASE_CLASS>(start, pos);
					posTmpList = tmpList->GetHeadPosition();
					delete tmpType;
				}
			}

			if(pos == IndexRemove)
			{
				POSITION pos1, pos2 = list->GetHeadPosition();
				while(pos2) list->GetNext(pos2)->Remove(ele);
				
				pos1 = list->GetHeadPosition();
				while(pos2=pos1) // (pos2=pos1) != NULL
				{
					if(list->GetNext(pos1)->GetSize() == 0)
					{
						// Save the old pointer for deletion
						tmpType2 = list->GetAt(pos2);
						list->RemoveAt(pos2);
						
						// Deletion avoids memory leak
						delete tmpType2;
					}
				}
			}
			
			return list;
		}
		virtual ~KArray()
		{
			this->RemoveAll();
		}
	};

	template<typename TYPE, typename ARG_TYPE=TYPE> class CCArray: 
		public CArray<TYPE, ARG_TYPE>
	{
	public:
		CCArray() { /* empty */ }

		~CCArray()
		{
			ASSERT_VALID(this);

			if (m_pData != NULL)
			{
				try
				{
					for( int i = 0; i < m_nSize; i++ )
						(m_pData + i)->~TYPE();
					delete[] (BYTE*)m_pData;
				}
				catch(...)
				{
					::AfxMessageBox(_T("Here ...\r\n"));

					delete[] (BYTE*)m_pData;
				}
			}
		}

		virtual void Serialize(CArchive& ar)
		{
			if(ar.IsStoring())
			{
				int size = int(this->GetSize()); ar<<size;

				for(int i=0; i<size; ++i)
					this->m_pData->Serialize(ar);
			}
			else
			{
				this->RemoveAll(); int size;  ar>>size;
				TYPE data;
				for(int i=0; i<size; ++i)
				{
					data.Serialize(ar); this->Add(data);
				}
			}
		}
	};

	/* WARNIG: Default ARG_TYPE is TYPE, NOT const TYPE& */
	template<typename TYPE, typename ARG_TYPE=TYPE> class KList: 
		public CList<TYPE, ARG_TYPE>
	{
	public:
		KList() { /* empty */ }
	};
}