#pragma once

#include <iostream>

#include <complex>
#include <kch_cmath.h>

typedef double  realnum;

//typedef std::complex<realnum> Complex;

class Complex: virtual public std::complex<realnum>
{
protected:
	void copy(const Complex& c) 
	{ this->real(c.real()); this->imag(c.imag()); }

public:
	Complex(realnum r=0.0, realnum i=0.0)
		: std::complex<realnum>(r, i) { }
	
	Complex(const std::complex<realnum>& c) 
		{ this->real(c.real()); this->imag(c.imag()); }
	
	Complex(const Complex& c) { copy(c); }
	
	Complex& operator=(const Complex& c) 
		{ if(this != &c) copy(c); return *this;} 
	
	operator std::complex<realnum>() { return *this; }

	realnum& get_real() { return _Val[0]; }
	realnum& get_imag() { return _Val[1]; }
};

extern wchar_t cplx_real_opr[];
extern wchar_t cplx_imag_opr[];
extern bool unitcplx;
extern bool fmt_scientific;

inline std::wostream& operator<<(std::wostream& os, const Complex& c)
{
	if(unitcplx)
	{
		if(c.real() != 0.0) os<<c.real()<<cplx_real_opr;
		
		if(c.imag() != 0.0)
		{
			if(c.real() != 0.0) os<<std::showpos;
			os<<c.imag()<<cplx_imag_opr;
			os<<std::noshowpos;
		}

		if(c.real() == 0 && c.imag() == 0)
			os<<L'0';
	}
	else
		os<<L"( "<<c.real()<<L", "<<c.imag()<<L" )";

	return os;
}

inline bool operator > (const Complex& c1, const Complex& c2)
{
	double d1 = norm(c1), d2 = norm(c2);

	if(d1 == d2)
		return c1.real() > c2.real();
	else
		return d1>d2;
}

inline bool operator < (const Complex& c1, const Complex& c2)
{
	double d1 = norm(c1), d2 = norm(c2);

	if(d1 == d2)
		return c1.real() < c2.real();
	else
		return d1<d2;
}

inline bool operator >= (const Complex& c1, const Complex& c2)
{
	if(c1 == c2)
		return true;
	else
		return norm(c1)>norm(c2);
}

inline bool operator <= (const Complex& c1, const Complex& c2)
{
	if(c1 == c2)
		return true;
	else
		return norm(c1)<norm(c2);
}

inline Complex operator ^ (const Complex& c1, const Complex& c2)
{
	return pow(c1, c2);
}


inline double realc(const Complex& c)
{
	return c.real();
}

inline double imagc(const Complex& c)
{
	return c.imag();
}

inline double absc(const Complex& c)
{
	return abs(c);
}

inline double argc(const Complex& c)
{
	double arg;
	double x = c.real();
	double y = c.imag();

	if(x == 0.0)
	{
		if(y==0.0)
			arg = 0.0;
		else if (y>0.0)
			arg = M_PI_2;
		else
			arg = M_PI + M_PI - M_PI_2;
	}
	else if(x>0.0)
	{
		if(y == 0.0)
			arg = 0.0;
		else if(y>0.0) // 1st quadrant
			arg = atan(y/x);
		else // 4th quadrant
			arg = M_PI+M_PI+ atan(y/x);
	}
	else // x<0
	{
		if(y == 0.0)
			arg = M_PI;
		else
			arg = M_PI + atan(y/x);
	}

	return arg;
}

inline double argdc(const Complex& c)
{
	extern double deg(double);
	return deg(argc(c));
}

inline double normc(const Complex& c)
{
	return norm(c);
}

inline Complex cosc(const Complex& c)
{
	return cos(c);
}

inline Complex sinc(const Complex& c)
{
	return sin(c);
}

inline Complex tanc(const Complex& c)
{
	return tan(c);
}

inline Complex coshc(const Complex& c)
{
	return cosh(c);
}

inline Complex sinhc(const Complex& c)
{
	return sinh(c);
}

inline Complex tanhc(const Complex& c)
{
	return tanh(c);
}

inline Complex sqrtc(const Complex& c)
{
	return sqrt(c);
}

inline Complex conjc(const Complex& c)
{
	return conj(c);
}

inline Complex expc(const Complex& c)
{
	return exp(c);
}

inline Complex lnc(const Complex& c)
{
	return log(c);
}

inline Complex log10c(const Complex& c)
{
	return log10(c);
}

inline Complex powc(const Complex& c1, const Complex& c2)
{
	return pow(c1, c2);
}

// a is radian 
inline Complex polarc(double r, double a)
{
	return std::polar<realnum>(r, a);
}

// a is degree
inline Complex polardc(double r, double a)
{
	extern double rad(double);
	return polarc(r, rad(a));
}

//Overloaded complex operations to handle mixed float and double
//This takes care of e.g. 1.0/z, z complex<float>

inline std::complex<float> operator+(const double &a,
	const std::complex<float> &b) { return float(a)+b; }

inline std::complex<float> operator+(const std::complex<float> &a,
	const double &b) { return a+float(b); }

inline std::complex<float> operator-(const double &a,
	const std::complex<float> &b) { return float(a)-b; }

inline std::complex<float> operator-(const std::complex<float> &a,
	const double &b) { return a-float(b); }

inline std::complex<float> operator*(const double &a,
	const std::complex<float> &b) { return float(a)*b; }

inline std::complex<float> operator*(const std::complex<float> &a,
	const double &b) { return a*float(b); }

inline std::complex<float> operator/(const double &a,
	const std::complex<float> &b) { return float(a)/b; }

inline std::complex<float> operator/(const std::complex<float> &a,
	const double &b) { return a/float(b); }

