#pragma once

#include <iostream>

typedef double Real;
typedef struct { double re, im; } Complex;
typedef struct { double x, y, z; } Vector;

inline std::ostream& operator<<(std::ostream& os, const Complex& c)
{
	os<<"("<<c.re<<", "<<c.im<<")";
	return os;
}

// complex constructor
inline Complex MakeComplex(double re=0, double im=0)
{
	Complex c;
	c.re = re; c.im = im;
	return c;
}

// complex conjugate
inline Complex operator ~(const Complex& c)
{
	return MakeComplex(c.re, -c.im);
}

inline Complex operator + (const Complex& c1, const Complex& c2)
{
	return MakeComplex(c1.re+c2.re, c1.im+c2.im);
}

inline Complex operator - (const Complex& c1, const Complex& c2)
{
	return MakeComplex(c1.re-c2.re, c1.im-c2.im);
}

inline Complex operator * (const Complex& c1, const Complex& c2)
{
	return MakeComplex(c1.re*c2.re-c1.im*c2.im, 
		c1.re*c2.im+c1.im*c2.re);
}

inline Complex operator / (const Complex& c1, const Complex& c2)
{
	double c22 = c2.re*c2.re + c2.im*c2.im;

	return MakeComplex((c1.re*c2.re + c1.im*c2.im)/c22,
		(c1.im*c2.re - c1.re*c2.im)/c22);
}

inline Complex operator + (double d, const Complex& c)
{
	return MakeComplex(d) + c;
}

inline Complex operator + (const Complex& c, double d)
{
	return MakeComplex(d) + c;
}

inline Complex operator - (double d, const Complex& c)
{
	return MakeComplex(d)-c;
}

inline Complex operator - (const Complex& c, double d)
{
	return c - MakeComplex(d);
}

inline Complex operator * (double d, const Complex& c)
{
	return MakeComplex(d)*c;
}

inline Complex operator * (const Complex& c, double d)
{
	return c*MakeComplex(d);
}

inline Complex operator / (double d, const Complex& c)
{
	return MakeComplex(d) / c;
}

inline Complex operator / (const Complex& c, double d)
{
	return c / MakeComplex(d);
}