#pragma once

#include <exception>
#include <string>

class UnicodeException: public std::exception
{
protected:
	std::wstring msg;
public:
	UnicodeException(const wchar_t *pMsg=NULL)
	{
		if(pMsg) msg = std::wstring(pMsg);
		else msg=L"";
	}

	UnicodeException(const std::wstring& str): msg(str) {}
	const std::wstring& GetMessage() const { return msg;}

	virtual ~UnicodeException() { }
};

class RunTimeException: public UnicodeException
{
public:
	RunTimeException(const wchar_t *pMsg=NULL): 
	  UnicodeException(pMsg) { }

	RunTimeException(const std::wstring& str): 
	  UnicodeException(str) {}

	virtual ~RunTimeException() {}
};

class CompileTimeException: public UnicodeException
{
public:
	CompileTimeException(const wchar_t *pMsg=NULL): 
	  UnicodeException(pMsg) { }

	CompileTimeException(const std::wstring& str): 
	  UnicodeException(str) {}

	virtual ~CompileTimeException() {}
};

class UserAbortException: public UnicodeException
{
public:
	UserAbortException(const wchar_t *pMsg=NULL): 
	  UnicodeException(pMsg) { }

	UserAbortException(const std::wstring& str): 
	  UnicodeException(str) {}

	virtual ~UserAbortException() {}
};

struct ReturnException: public UnicodeException 
{
	ReturnException(const wchar_t* msg=L"Return Exception")
		: UnicodeException(msg) {}

	virtual ~ReturnException() {}
} ;

struct BreakException: public UnicodeException 
{ 
	BreakException(const wchar_t* msg=L"Break Exception")
		: UnicodeException(msg) {}
	virtual ~BreakException() {}
} ;

struct GotoException: public UnicodeException 
{ 
	GotoException(const wchar_t* msg=L"Goto Exception")
		: UnicodeException(msg) {}
	virtual ~GotoException() {}
} ;

struct SkipException: public UnicodeException 
{ 
	SkipException(const wchar_t* msg=L"Continue Exception")
		: UnicodeException(msg) {}
	virtual ~SkipException() {}
} ;

struct StackException: public UnicodeException 
{ 
	StackException(const wchar_t* msg=L"Stack Exception")
		: UnicodeException(msg) {}
	virtual ~StackException() {}
} ;

struct PtrStackException: public UnicodeException 
{ 
	PtrStackException(const wchar_t* msg=L"PtrStack Exception")
		: UnicodeException(msg) {}
	virtual ~PtrStackException() {}
} ;

class ExceptionParsing: public UnicodeException
{
public:
	ExceptionParsing(const wchar_t *msg=L"Parsing Error\r\n")
		: UnicodeException(msg) {}

	  virtual ~ExceptionParsing(){}
};

class ExceptionMath: public UnicodeException
{
public:
	ExceptionMath(const wchar_t *msg=L"Math Exception\r\n")
		: UnicodeException(msg) {}
	virtual ~ExceptionMath() {}
};

class ExpDivisionByZero: public ExceptionMath
{
public:
	ExpDivisionByZero(const wchar_t *msg = L"Division By Zero\r\n")
		: ExceptionMath(msg) {}
	virtual ~ExpDivisionByZero() {}
};

class ExpOutOfDomain: public ExceptionMath
{
public:
	ExpOutOfDomain(const wchar_t *msg = L"Domain Out of Range\r\n")
		: ExceptionMath(msg) {}
	virtual ~ExpOutOfDomain() {}

};

class ExpArgSingularity: public ExceptionMath
{
public:

	ExpArgSingularity(const wchar_t *msg = L"Argument Singularity\r\n")
		: ExceptionMath(msg) {}
	virtual ~ExpArgSingularity() {}
};

class ExpOverflow: public ExceptionMath
{
	public:

	ExpOverflow(const wchar_t *msg = L"Overflow range error\r\n")
		: ExceptionMath(msg) {}
	virtual ~ExpOverflow() {}

};

class ExpPartialLoss: ExceptionMath
{
	public:

	ExpPartialLoss(const wchar_t *msg = L"Partial loss of significance\r\n")
		: ExceptionMath(msg) {}
	virtual ~ExpPartialLoss() {}

};

class ExpTotalLoss: ExceptionMath
{
	public:

	ExpTotalLoss(const wchar_t *msg = L"Total loss of significance\r\n")
		: ExceptionMath(msg) {}
	virtual ~ExpTotalLoss() {}

};