#pragma once

#include <kch_collections.h>
#include <kch_objects.h>

using namespace siliners;

class AFX_EXT_CLASS KFigures: public CObject
{
public:
	DECLARE_COUNT

	KFigures();
	virtual ~KFigures();

	KFigures& operator=(const KFigures& figure)
	{
		this->m_Bound = figure.m_Bound;
		this->m_Zoom = figure.m_Zoom;
		return *this;
	}

	virtual void Serialize(CArchive& ar);

	virtual void Draw(CDC* pDC, float zoom,
		const KPoint& kpt, const CRect& clip) const;

	// normalize and set m_Bound
	// kpt is shift point for the sheet
	virtual KRect Finish(const KPoint& kpt);
	
	// attributes
public:
	CRect m_Bound; // bounding rectangle of the figure
	float m_Zoom;
	int m_Thick; // thickness of the stroke
	KColor m_Color;
	BOOL m_Antialias;
	
protected:
	DECLARE_SERIAL(KFigures)
};

class AFX_EXT_CLASS KFreehand: public KFigures
{
public:
	DECLARE_COUNT

	KFreehand();
	virtual ~KFreehand();

	virtual void Serialize(CArchive& ar);

	virtual void Draw(CDC* pDC, float zoom,
		const KPoint& kpt, const CRect& clip) const;

	virtual KRect Finish(const KPoint& kpt);

	void Add(const KPoint& kpt);

	// attributes
public:
	// KPoint array
	KArray<KPoint, KPoint> m_Points;
			
protected:
	DECLARE_SERIAL(KFreehand)
};

class AFX_EXT_CLASS KStraight: public KFigures
{
public:
	DECLARE_COUNT

	KStraight();
	virtual ~KStraight();

	virtual void Serialize(CArchive& ar);

	virtual void Draw(CDC* pDC, float zoom,
		const KPoint& kpt, const CRect& clip) const;

	virtual KRect Finish(const KPoint& kpt);

	void NotXorDraw(CDC* pDC, float zoom,
		const KPoint& kpt, const CRect& clip) const;
	
	// attributes
public:
	KPoint m_Start, m_End;
			
protected:
	DECLARE_SERIAL(KStraight)
};

class AFX_EXT_CLASS KEllipse: public KFigures
{
public:
	DECLARE_COUNT

	KEllipse();
	virtual ~KEllipse();

	virtual void Serialize(CArchive& ar);

	virtual void Draw(CDC* pDC, float zoom,
		const KPoint& kpt, const CRect& clip) const;

	virtual KRect Finish(const KPoint& kpt);

	void NotXorDraw(CDC* pDC, float zoom,
		const KPoint& kpt, const CRect& clip) const;
	
	// attributes
public:
	KPoint m_Start, m_End;
			
protected:
	DECLARE_SERIAL(KEllipse)
};

class AFX_EXT_CLASS KPolyline: public KFreehand
{
public:
	DECLARE_COUNT

	KPolyline();
	virtual ~KPolyline();

	virtual void Serialize(CArchive& ar);

	virtual void Draw(CDC* pDC, float zoom,
		const KPoint& kpt, const CRect& clip) const;

	virtual KRect Finish(const KPoint& kpt);
	
	// attributes
public:
	float m_Tension;
		
protected:
	DECLARE_SERIAL(KPolyline)
};

class AFX_EXT_CLASS KPolygon: public KPolyline
{
public:
	DECLARE_COUNT

	KPolygon();
	virtual ~KPolygon();

	virtual void Serialize(CArchive& ar);

	virtual void Draw(CDC* pDC, float zoom,
		const KPoint& kpt, const CRect& clip) const;
		
protected:
	DECLARE_SERIAL(KPolygon)
};