#pragma once

#include <kch_afx.h>
#include <exception>
#include <limits>

#ifndef _USE_MATH_DEFINES
#define _USE_MATH_DEFINES
#endif
#include <cmath>

typedef __int64 integer;

class AFX_EXT_CLASS MathException: public std::exception
{
protected:
	UINT rcid;
public:
	MathException(UINT nID): rcid(nID) { }

	virtual CString PutMsg() const;
	virtual ~MathException() {}
};


/////////// chap06-special-functions.cpp, 
// NRC P. 219
KCH_EXP double gammln(double xx);

// NCR P . 220
// Returns ln(n!)
KCH_EXP double factln(integer n);

// NRC P. 219
KCH_EXP double factrl(integer n);

// NRC P. 221
// Returns the value of beta function B(z, w)
KCH_EXP double beta(double z, double w);

// NRC P. 223
// Returns the incomplete gamma function P(a, x)
KCH_EXP double gammp(double a, double x);

// NRC P. 223
// Returns the incomplete gamma function 
// Q(a, x) = 1 - P(a, x)
KCH_EXP double gammq(double a, double x);

// NRC P. 225
// Returns the error function erf(x)
KCH_EXP double erff(double x);

// NRC P. 225
// Returns the complementary error function erfc(x)
KCH_EXP double erffc(double x);

// NRC P. 226
// Returns the complementary error function erfc(x)
// with fractional error everywhere less than 1.2e-7
KCH_EXP double erfcc(double x);

// NRC P. 228
// Evaluates the exponential integral En(x)
KCH_EXP double expint(int n, double x);

// NRC P. 230
// Computes the exponential integral Ei(x) for x>0
KCH_EXP double ei(double x);

// NRC P. 232
// Returns the incomplete beta function Ix(a, b)
KCH_EXP double betai(double a, double b, double x);

//Bessel functions of the first kind: orders 0, Jo(x)
KCH_EXP double bessj0(double x);

//Bessel functions of the second kind: orders 0, Yo(x)
KCH_EXP double bessy0(double x);

//Bessel functions of the first kind: orders 1, J1(x)
KCH_EXP double bessj1(double x);

//Bessel functions of the second kind: orders 1, Y1(x)
KCH_EXP double bessy1(double x);

//Bessel functions of the first kind: orders n, Jn(x)
KCH_EXP double bessj(int n, double x);

//Bessel functions of the second kind: orders n, Yn(x)
KCH_EXP double bessy(int n, double x);

