#pragma once

#include <cmath>
#include <kch_lexerparser.h>

class RealFunc: public Expression<double>
{
protected:
	Expression<double>* pArg;
public:
	RealFunc(Expression<double>* arg): pArg(arg) {}
	
	virtual ~RealFunc() {delete pArg; }
};

class Sin: public RealFunc
{
public:
	Sin(Expression<double>* arg): RealFunc(arg) { }
	virtual double Eval() { return sin(pArg->Eval()); }
	virtual std::string Display() 
	{
		std::ostringstream os;
		os<<"sin("<<pArg->Display()<<")"; 
		return os.str();
	}

	virtual ~Sin() { }
};

class Cos: public RealFunc
{
public:
	Cos(Expression<double>* arg): RealFunc(arg) { }
	virtual double Eval() { return cos(pArg->Eval()); }
	virtual std::string Display() 
	{
		std::ostringstream os;
		os<<"cos("<<pArg->Display()<<")";
		return os.str();
	}

	virtual ~Cos() { }
};

class Sqrt: public RealFunc
{
public:
	Sqrt(Expression<double>* arg): RealFunc(arg) { }
	virtual double Eval() { return sqrt(pArg->Eval()); }
	virtual std::string Display() 
	{
		std::ostringstream os;
		os<<"sqrt("<<pArg->Display()<<")";
		return os.str();
	}

	virtual ~Sqrt() { }
};

class Pow: public RealFunc
{
protected:
	Expression<double>* pBase;
public:
	Pow(Expression<double>* base, Expression<double>* arg): pBase(base), RealFunc(arg) { }
	virtual double Eval() { return pow(pBase->Eval(), pArg->Eval()); }
	virtual std::string Display() 
	{
		std::ostringstream os;
		os<<"pow("<<pBase->Display()<<","<<pArg->Display()<<")";
		return os.str();
	}

	virtual ~Pow() { delete pBase; }
};



