#pragma once

#ifdef LIBBUILD
#define KCH_EXP __declspec(dllexport)
#else
#define KCH_EXP __declspec(dllimport)
#endif


#include "stdafx.h"

#include <kch_collections.h>
#include <iostream>
#include <gdiplus.h>

using namespace Gdiplus;
using namespace siliners;

class AFX_EXT_CLASS KPaperMetric: public CObject
{
	void copy(const KPaperMetric& pm); 
public:
	KPaperMetric(BOOL printer=FALSE);
	KPaperMetric(const KPaperMetric& pm);

	int m_HorzRes; // Width, in pixels, of the screen
	int m_VertRes; // Height, in raster lines, of the screen
	
	// Number of pixels per logical inch along the screen width
	int m_LogPixelSX; 
	// Number of pixels per logical inch along the screen height
	int m_LogPixelSY; 
	
	KPaperMetric& operator = (const KPaperMetric& pm);
	BOOL operator == (const KPaperMetric& pm);

	void FillMembers(CDC* pDC);
	virtual void Serialize(CArchive& ar);

protected:
	DECLARE_SERIAL(KPaperMetric)
};

class KRect;

// Forward declaration
class KPointF;
class KPoint;

class AFX_EXT_CLASS KException
{
public:
	UINT m_errorcode;

	KException(UINT errorcode = 0): m_errorcode(errorcode)
	{ /* empty */ }
};

/////////////////// KPoint /////////////////////////////
class AFX_EXT_CLASS KPoint: public CObject, public Point
{
public:
	DECLARE_COUNT
	KPoint(INT x=0, INT y=0);
	KPoint(const KPoint& pt);
	KPoint(const CPoint& pt);

	virtual ~KPoint();
		
public:
	
	operator CString() const; 
	// Conversion to KPointF
	operator KPointF() const;
	// Conversion to CPoint
	operator CPoint() const;
		
	virtual void Serialize(CArchive& ar);
	
	BOOL operator==(const KPoint& pt) const;	
	BOOL operator != (const KPoint& pt) const;

	KPoint operator + (const KPoint& pt) const
	{	return KPoint(X+pt.X, Y+pt.Y); }

	KPoint operator - (const KPoint& pt) const
	{	return KPoint(X-pt.X, Y-pt.Y); }

	KPoint operator * (double f) const
	{	return KPoint(INT(X*f), INT(Y*f)); }
	
	KPoint operator / (double f) const
	{	return KPoint(INT(X/f), INT(Y/f)); }

	KPoint& KPoint::operator=(const KPoint& pt)
	{	X = pt.X; Y = pt.Y; return *this; }

protected:
	DECLARE_SERIAL(KPoint)
};

/////////////////// KPointF /////////////////////////////
class AFX_EXT_CLASS KPointF: public CObject, public PointF
{
public:
	DECLARE_COUNT
	KPointF(REAL x=0, REAL y=0);
	KPointF(const KPointF& pt);
	virtual ~KPointF();
		
public:
	KPointF& operator=(const KPointF& pt);

	operator CString() const; 
	// Conversion to KPoint
	operator KPoint() const;

	virtual void Serialize(CArchive& ar);
			
	BOOL operator ==(const KPointF& pt) const;	

protected:
	DECLARE_SERIAL(KPointF)
};

// Forward declaration
class KSizeF;

/////////////////////// KSize //////////////////////////
class AFX_EXT_CLASS KSize: public CObject, public Size
{
public:
	DECLARE_COUNT
	KSize(INT width=0, INT height=0);
	KSize(const KSize& sz);
	virtual ~KSize();
		
public:
	KSize& operator=(const KSize& sz);
	operator KSizeF() const;

	operator CString() const;
	virtual void Serialize(CArchive& ar);

	BOOL operator==(const KSize& pt) const;	

protected:
	DECLARE_SERIAL(KSize)
};

/////////////////////// KSizeF //////////////////////////
class AFX_EXT_CLASS KSizeF: public CObject, public SizeF
{
public:
	DECLARE_COUNT
	KSizeF(REAL width=0, REAL height=0);
	KSizeF(const KSizeF& sz);
	virtual ~KSizeF();
		
public:
	KSizeF& operator=(const KSizeF& sz);
	operator KSize() const;

	operator CString() const;
	virtual void Serialize(CArchive& ar);

	BOOL operator==(const KSizeF& pt) const;	

protected:
	DECLARE_SERIAL(KSizeF)
};

// Forward declaration
class KRectF;

/////////////////////// KRect ///////////////////////////
class AFX_EXT_CLASS KRect: public CObject, public Rect
{
public:
	DECLARE_COUNT
	KRect(INT x=0, INT y=0, INT width=0, INT height=0);
	KRect(const KPoint& location, const KSize& size);

	KRect(const KRect& rect);
	KRect(const KRectF& rect);
	KRect(const CRect& rect);

	virtual ~KRect();
		
public:
	KRect& operator=(const KRect& rect);
	KRect& operator=(const CRect& rect);
	
	KRect& operator+(const KPoint& pt);
	KRect& operator-(const KPoint& pt);

	operator KRectF() const;
	operator RectF() const;
	operator CRect() const;

	operator CString() const;
	virtual void Serialize(CArchive& ar);

	BOOL operator ==(const KRect& rect) const;	

protected:
	DECLARE_SERIAL(KRect)
};

/////////////////////// KRectF ///////////////////////////
class AFX_EXT_CLASS KRectF: public CObject, public RectF
{
public:
	DECLARE_COUNT
	KRectF(REAL x=0, REAL y=0, REAL width=0, REAL height=0);
	KRectF(const KPointF& location, const KSizeF& size);

	KRectF(const KRectF& rect);
	virtual ~KRectF();
		
public:
	KRectF& operator=(const KRectF& rect);
	operator KRect() const;

	operator CString() const;
	virtual void Serialize(CArchive& ar);

	BOOL operator ==(const KRectF& rect) const;	

protected:
	DECLARE_SERIAL(KRectF)
};


////////////////////// KColor ///////////////////////////
class AFX_EXT_CLASS KColor: public CObject, public Color
{
public:
	DECLARE_COUNT

	// Color(BYTE a, BYTE r, BYTE g, BYTE b);
	KColor(BYTE alpha, BYTE red, BYTE green, BYTE blue);

	// Color(BYTE r, BYTE g, BYTE b)
	KColor(BYTE r, BYTE g, BYTE b);

	// Color()
	KColor();

	// Color(ARGB)
	KColor(ARGB argb);

	// Copy constructor
	KColor(const KColor& color);

	KColor(UINT color);
	KColor(BYTE alpha, UINT color);

	// Assignment operator
	KColor& operator= (const KColor& color);

	operator COLORREF() const;

	// Equal operator
	BOOL operator==(const KColor& color) const;
	
	operator CString() const;

	virtual void Serialize(CArchive& ar);

protected:
	DECLARE_SERIAL(KColor)
};

class AFX_EXT_CLASS KObject: public CObject
{
public:
	virtual KRect GetBoundingRect() const = 0;

	virtual void Draw(CDC *pDC, float zoom, const KPoint& kpt) const = 0;
	
	virtual void Finish(float zoom, const KPoint& kpt) = 0;
			
	virtual ~KObject() { /* empty */ }
};


////////////////////////////////////////////////////////
//
//		class KText: public KObject
//
////////////////////////////////////////////////////////

class AFX_EXT_CLASS KText: public KObject
{
public:
	DECLARE_COUNT

	KText(LPCTSTR lpsz=NULL);

	virtual void Draw(CDC* pDC, float zoom, const KPoint& kpt) const;
	virtual KRect GetBoundingRect() const;
	
	virtual void Finish(float zoom, const KPoint& kpt);
	virtual void Serialize(CArchive& ar);

	virtual ~KText();

public:
	CString m_Text;

protected:
	DECLARE_SERIAL(KText)
};


/////////////////////// KObjectList /////////////
class AFX_EXT_CLASS KObjectList: public CObject
{
public:
	DECLARE_COUNT
	KObjectList();
	virtual ~KObjectList();
	
	virtual void Serialize(CArchive& ar);

	void Draw(CDC *pDC, float zoom, const KPoint& kpt) const;
	void Draw(CDC *pDC, float zoom, const KPoint& kpt,const KRect& rect) const;
			
	KTypedPtrList<CObList, KObject*> m_Objects;

protected:
	DECLARE_SERIAL(KObjectList)
};

KCH_EXP void DrawCardinal(Graphics* pGrfx, Pen* pen, 
						  const KPoint* pts, int n, float tension, BOOL closed=FALSE);

KCH_EXP void DrawCardinal(CDC* pDC, const KPoint* pts, int n, float tension, BOOL closed=FALSE);

KCH_EXP void DrawCardiSeg(CDC* pDC, float tension,
								 const KPoint& pt0,
								 const KPoint& pt1,
								 const KPoint& pt2, 
								 const KPoint& pt3);

KCH_EXP void DrawCardiSeg(Graphics* pGrfx, Pen* pen,
								 float tension,
								 const KPoint& pt0,
								 const KPoint& pt1,
								 const KPoint& pt2, 
								 const KPoint& pt3);

KCH_EXP BOOL GetDefaultPrinter(int &nHorzDPI, // Horizontal Dots per Inch
							   int &nVertDPI, // Vertical Dots per Inch
							   long &nHorzDots, // Horizontal Dots
							   long &nVertDots); // Vertical Dots

KCH_EXP CRect operator *(const CRect& r, double d);
KCH_EXP CRect operator /(const CRect& r, double d);
KCH_EXP CRect operator +(const CRect& r, double d);
KCH_EXP CRect operator -(const CRect& r, double d);

KCH_EXP CPoint operator *(const CPoint& r, double d);
KCH_EXP CPoint operator /(const CPoint& r, double d);
KCH_EXP CPoint operator +(const CPoint& r, double d);
KCH_EXP CPoint operator -(const CPoint& r, double d);

KCH_EXP LineCap GetLineCap(int index);
KCH_EXP DashStyle GetDashStyle(int index);
KCH_EXP void NotXorLine(const KPoint& start,
						const KPoint& end,
						CDC* pDC, int thick,
						COLORREF color);

