#pragma once

#define _USE_MATH_DEFINES
#include <kch_exception.h>
#include <cmath>
#include <iostream>
#include <iomanip>
#include <sstream>
#include <cstdlib>


typedef long long Integer;

typedef struct { Integer n, p, q; } Rational;

using namespace std;

inline std::ostream& operator<<(std::ostream& os, const Rational& r)
{
	if(r.n==0)
	{
		if(r.p == 0)
			return os<<"0";
		else // n==0, p != 0
			return os<<r.p<<"/"<<r.q;
	}
	else // n != 0
	{
		if(r.p == 0) 
			return os<<r.n;
		else
			return os<<r.n<<std::showpos<<r.p<<"/"<<std::noshowpos<<r.q;
	}
}

inline Integer abs(Integer n) 
{
	if(n>0) return n;
	else return -n;
}

// Greatest Common Divisor
inline Integer gcd(Integer m, Integer n)
{ 
	// declaration of remainder
	Integer r; 
	while (n != 0) 
	{ 	r = m % n; m = n; n = r; }
	return m; // exit gcd with value m
}

// Least Common Multiple
inline Integer lcm(Integer m, Integer n)
{ 
	// lcm * gcd = m * n
	return (m*n)/gcd(m, n);
}

inline Rational MakeRational(Integer p, Integer q)
{
	Rational r;

	if(q == 1)
	{
		r.n = p; r.p = 0; r.q = 1; return r;
	}
	else if (p == q)
	{
		r.n = 1; r.p = 0; r.q = 1; return r;
	}
	else if (p == 0)
	{
		r.n = 0; r.p = 0; r.q = 1; return r;
	}
	else if (q == 0)
		throw bad_arg("invalid argument");
		
	// q != 1, q != 0, p != q, p != 0
	
	Integer g = gcd(p, q);
	p /= g; q /= g;

	if(q<0) { p = -p; q = -q; }

	// q != 1, q != 0, q > 0, p != q

	if(abs(p) < q)
	{
		r.n = 0; r.p = p; r.q = q;
		return r;
	}
	else //  -q < p < q
	{
		r.q = q;
		r.p = p%r.q; // remainder
		r.n = (p-r.p)/r.q;
		return r;
	}
}

inline void normalize(Rational& r)
{
	Integer n, p, q;

	if(r.q<0) { p = -r.p; q=-r.q; }
	else { p = r.p; q=r.q; }

	Rational r1 = MakeRational(p, q);
	
	if(r.n>0 && r1.n>0)
	{
		r.n = r.n + r1.n; r.p = r1.p; r.q = r1.q; return;
	}

	if(r.n<0 && r1.n<0)
	{
		r.n = r.n + r1.n; r.p = r1.p; r.q = r1.q; return;
	}

	n = r.n + r1.n;

	if(n == 0)
	{
		r.n = 0; r.p = r1.p; r.q = r1.q; return;
	}
	if(n>0)
	{
		if(p>0)
		{
			r.n = n; r.p = r1.p; r.q = r1.q; return;
		}
		else //n>0, p<0
		{
			r.n = n-1; r.p = r1.q + r1.p; r.q = r1.q; return;
		}
	}
	else // n<0
	{
		if(p<0)
		{
			r.n = n; r.p = r1.p; r.q = r1.q; return;
		}
		else // n<0, p>0
		{
			r.n = n+1; r.p = r1.p-r1.q; r.q = r1.q; return;
		}
	}
}

inline int find_repeat(const std::string& num)
{
	int i, j, k;
	int len = int(num.size());
	int len2 = std::min<int>(std::max<int>(len-2, 2), len);
	std::string str1, str2, str3;

	for(k=1; k <= len2; ++k)
	{
		str1 = num.substr(0, k);
		
		for(j=k; j<len; j+=k)
		{
			str2 = num.substr(j, k);
			
			if((i=int(str2.size()))<k)
			{
				char c[] = {str1[i], '\0'};

				str1 = str1.substr(0, i);
		
				if(c[0]>='5')
				{
					c[0]=str2[i-1]-1;
					str2.replace(i-1, 1, c);
				}
			}
			
			if(str2 != str1) break;
		}
		
		if(j>=len) return k;
	}

	return 0;
}

inline bool find_pattern(const std::string& num, int& lead, int& repeat)
{
	int len = int(num.size());
	int len2 = std::min<int>(std::max<int>(len-2, 2), len); 
	std::string str1;
	
	for(lead=0; lead<len2; ++lead)
	{
		str1 = num.substr(lead, std::string::npos);
		repeat = find_repeat(str1) + lead;

		if(repeat != lead) return true;
	}

	return false;
}

inline Rational MakeRational(double x)
{
	double intptr, remainder = modf(x, &intptr);
	const int precision = 14;
		
	Rational r;
	std::string strnum, str;;
	std::ostringstream ostrm;
	
	r.n = Integer(intptr);	
	if(fabs(remainder)<1E-14)
	{
		r.p = 0; r.q = 1; return r;
	}

	ostrm<<std::fixed<<std::setprecision(precision)<<std::showpos<<remainder;
	std::istringstream istrm(ostrm.str());
	
	char c, sign;
	istrm>>sign>>c>>c>>strnum;	
	int i, j, k;

	if(find_pattern(strnum, i, j))
	{		
		if(i) // i != 0, lead non - repeat
		{
			r.p = atol(strnum.substr(0, j).c_str()) 
				- atol(strnum.substr(0, i).c_str());
			if(sign == '-') r.p = -r.p;
			
			for(k=0; k<(j-i); ++k) str.push_back('9');
			for(k=0; k<i;++k) str.push_back('0');

			r.q = ::atol(str.c_str());
		}
		else // 
		{
			r.p = atol(strnum.substr(0, j).c_str());
			if(sign == '-') r.p=-r.p;
			for(k=0; k<j; ++k) str.push_back('9');
			r.q = ::atol(str.c_str());
		}
	}
	else
	{
		// find trailing zero
		for(i=precision-1; i>-1; --i) 
			if(strnum[i] != '0') break;
		
		str = sign + strnum.substr(0, i+1);
		for(j=0, r.q=1; j<=i; ++j) r.q *= 10;
		std::istringstream istrm1(str.c_str());
		istrm1>>r.p;
	}
		
	normalize(r);
	return r;
}

inline Rational operator + (const Rational& r1, const Rational& r2)
{
	Rational r;

	r.n = r1.n + r2.n;
	r.p = r1.p*r2.q + r1.q*r2.p;
	r.q = r1.q * r2.q;

	normalize(r);
	return r;
}

inline Rational operator - (const Rational& r1, const Rational& r2)
{
	Rational r;

	r.n = r1.n - r2.n;
	r.p = r1.p*r2.q - r1.q*r2.p;
	r.q = r1.q * r2.q;

	normalize(r);
	return r;
}

inline Rational operator * (const Rational& r1, const Rational& r2)
{
	return MakeRational(r1.n * r2.n, 0) +
	       MakeRational(r1.n * r2.p, r2.q) +
		   MakeRational(r2.n * r1.p, r1.q) +
		   MakeRational(r1.p * r2.p, r1.q*r2.q);
}

inline Rational reverse(const Rational& r)
{
	return MakeRational(r.q, r.n*r.q+r.p);
}

inline Rational operator / (const Rational& r1, const Rational& r2)
{
	return r1 * reverse(r2);
}

inline double mag(const Rational& r)
{
	return double(r.n) + double(r.p)/double(r.q);
}

inline Rational operator + (double d, const Rational& r)
{
	return MakeRational(d)+r;
}

inline Rational operator + (const Rational& r, double d)
{
	return r + MakeRational(d);
}

inline Rational operator - (double d, const Rational& r)
{
	return MakeRational(d) - r;
}

inline Rational operator - (const Rational& r, double d)
{
	return r - MakeRational(d);
}

inline Rational operator * (double d, const Rational& r)
{
	return MakeRational(d)*r;
}

inline Rational operator * (const Rational& r, double d)
{
	return r*MakeRational(d);
}

inline Rational operator / (double d, const Rational& r)
{
	return MakeRational(d) / r;
}

inline Rational operator / (const Rational& r, double d)
{
	return r / MakeRational(d);
}

inline bool operator > (const Rational& r1, const Rational& r2)
{
	return mag(r1)>mag(r2);
}

inline bool operator < (const Rational& r1, const Rational& r2)
{
	return mag(r1)<mag(r2);
}

inline bool operator == (const Rational& r1, const Rational& r2)
{
	return mag(r1)==mag(r2);
}

inline std::string express(const Rational& r, double value, const char* symbol)
{
	double m = mag(r), d1=m/value, d2=m*value;
	Rational r1, r2;
	std::ostringstream ostr;
		
	r1 = MakeRational(d1); r2 = MakeRational(d2);
	
	if(abs(r1.p) < abs(r2.p))
		ostr<<symbol<<"*"<<"("<<r1.n<<", "<<r1.p<<"/"<<r1.q<<")";
	else
		ostr<<"(1/"<<symbol<<")*"<<"("<<r2.n<<", "<<r2.p<<"/"<<r2.q<<")";

	return ostr.str();
}
