#pragma once

#include <map>
#include <kch_theparser_type.h>

#include "SyntaxAnalyzer.h"

#define UI_KOREAN

class BaseCUF
{
protected:
	// 0: call by value, 1: call by reference
	std::vector<int> callbyref; 
	VarUserBase *vubptr;
	VarMemBase  *udtptr;
	UdtPtrBase	*upbptr;

public:
	std::wstring fname; 
	Function *pFunc;
		
	virtual void insert(const std::wstring& id, BaseType* xpr) { }
	virtual void insert(BaseType* xpr, int cbr=0){ ;}

	//BaseCUF(const std::wstring& name, Function* fptr, UserData *ud=NULL): 
	//	fname(name), pFunc(fptr), udptr(ud) {}

	BaseCUF(const std::wstring& name, Function* fptr, 
		VarUserBase *vptr, VarMemBase *uptr, UdtPtrBase *bptr=NULL): 
		fname(name), pFunc(fptr), vubptr(vptr), udtptr(uptr), upbptr(bptr){ }
	virtual ~BaseCUF() {}
};

template<typename T>
class CallFunc: public BaseCUF, public Expression<T>
{
protected:
	//std::string fname;
	std::vector<BaseType*> argvec; 
	typedef std::vector<BaseType*>::iterator Itr;
	
	void ReplArgs();
	
	void Step1Array(Data *pParData, VarList* pArgData);
	void Step2Array(Data* pArgData, int i);
	void Step3Array(Data *pParData, Data* pArgData);

	void Step1Ptr(Data *pParData, VarList* pArgData);
	void Step2Ptr(Data* pArgData, int i);
	void Step3Ptr(Data *pParData, Data* pArgData);

	void Step1Aptr(Data *pParData, VarList* pArgData);
	void Step2Aptr(Data* pArgData, int i);
	void Step3Aptr(Data *pParData, Data* pArgData);

public:
		
	ScanFile scanfile;

	CallFunc(const std::wstring& name, Function *fptr, 
		VarUserBase *vptr=NULL, VarMemBase *uptr=NULL, UdtPtrBase *bptr=NULL):
		BaseCUF(name, fptr, vptr, uptr, bptr)
	{ 	}

	virtual void insert(BaseType* xpr, int cbr = 0)
	{
		int index = int(argvec.size());
		BaseCUF::pFunc->CheckArgs(xpr, index);

		argvec.push_back(xpr); 
		callbyref.push_back(cbr); 
	}
	
	bool ArgCountNotCompatible() 
	{
		VarList *varptr = pFunc->pStmtList->GetVarList();
		return varptr->size() != argvec.size();
	}
	
	Function* GetFunPtr() { return pFunc; }

	virtual T Eval()
	{	
		ReplArgs(); // set args

		T theValue;

		if(typeid(T) != typeid(VoidData))
			pFunc->rtnStack.push((void*)&theValue);
		else
			pFunc->rtnStack.push((void*)NULL);

		if(vubptr) pFunc->pUdt = vubptr->GetUserData();
		else if(udtptr)
		{
			if(pFunc->NotUseMember==false)
				pFunc->pUdt = udtptr->GetUserData();
		}
		else if(upbptr)
		{
			pFunc->pUdt = upbptr->GetUserData();
		}
		else if(pFunc->pParen) 
			pFunc->pUdt = pFunc->pParen->SetUdtFromParent();
		
		try
		{
			pFunc->Run(); // run function
		}
		catch(StmtThrowBase&)
		{
			pFunc->pStmtList->CleanupStack();
			pFunc->rtnStack.pop(); throw;
		}

		// restore local variables
		pFunc->pStmtList->CleanupStack();
		pFunc->rtnStack.pop();

		return theValue;
	}

	void FastEval(T* ptr)
	{	
		ReplArgs(); // set args

		if(typeid(T) != typeid(VoidData))
			pFunc->rtnStack.push((void*)ptr);
		else
			pFunc->rtnStack.push((void*)NULL);

		if(vubptr) pFunc->pUdt = vubptr->GetUserData();
		else if(udtptr)
		{
			if(pFunc->NotUseMember==false)
				pFunc->pUdt = udtptr->GetUserData();
		}
		else if(pFunc->pParen) 
			pFunc->pUdt = pFunc->pParen->SetUdtFromParent();
				
		try
		{
			pFunc->Run(); // run function
		}
		catch(StmtThrowBase&)
		{
			pFunc->pStmtList->CleanupStack();
			pFunc->rtnStack.pop(); throw;
		}

		// restore local variables
		pFunc->pStmtList->CleanupStack();
		pFunc->rtnStack.pop();

		//return theValue;
	}


	virtual std::wstring Display()
	{
		std::wostringstream os;

		if(typeid(T) != typeid(voidtype))
			os<<fname<<_T("()");

		return os.str();
	}

	virtual ~CallFunc() {}
};

template<typename T>
void CallFunc<T>::Step1Ptr(Data *pParData, VarList* pArgData)
{
	PtrDataBase *ptr = static_cast<PtrDataBase*>(pParData);
	std::wstring id = pParData->GetId();

	switch(ptr->GetEleType())
	{
	case TypeBool:
		{ pArgData->put(new PtrData<bool>(id, true)); return; }
	case TypeIntg:
		{ pArgData->put(new PtrData<integer>(id, true)); return; }
	case TypeReal:
		{ pArgData->put(new PtrData<realnum>(id, true)); return; }
	case TypeCplx:
		{ pArgData->put(new PtrData<Complex>(id, true)); return; }
	case TypeVctr:
		{ pArgData->put(new PtrData<Vector>(id, true)); return; }
	case TypeStrn:
		{ pArgData->put(new PtrData<string>(id, true)); return; }
	case TypeUPtr:
		{ pArgData->put(new PtrData<UserData>(id)); return; }
	}

	HandleSyntaxError(_T("Unsupported pointer type for function argument.\r\n"));
}

template<typename T>
void CallFunc<T>::Step2Ptr(Data *pArgData, int i)
{
	extern void SetErrorFile(CString& filename,
		CString& token, int errorline, int errorchar);

	PtrData<bool>		*bpPtr; PtrData<integer> *ipPtr; 
	PtrData<realnum>	*rpPtr; PtrData<Complex> *cpPtr; 
	PtrData<Vector>		*vpPtr; PtrData<string>  *spPtr; 
	PtrData<UserData>	*upPtr; std::wostringstream os;

	if(bpPtr=dynamic_cast<PtrData<bool>*>(pArgData))      
	{ 
		LeftHandSide<bool> *pLHS = dynamic_cast<LeftHandSide<bool>*>(argvec[i]);
		PtrData<bool> *ptrdata = dynamic_cast<PtrData<bool>*>(argvec[i]);

		if(pLHS) { *bpPtr->GetPtr() = &pLHS->GetData(); return; }
		else if(ptrdata) 
		{ 
			*bpPtr->GetPtr() = ptrdata->GetDataPtr(); 
			if(ptrdata->GetDataPtr()) return; 
			else
			{	os<<L"In the function call \'"
					<<this->fname<<L"\', pointer variable \'"<<ptrdata->GetId()<<L"\' is not initialized.\r\n";
				os<<L"File name: "<<scanfile.filename.GetString()<<L"\r\n";
				os<<L"Line number: "<<scanfile.lineno<<L"  Character count: "<<scanfile.charcount<<L"\r\n";
			}
		}
		else os<<L"Invalid boolean pointer type.\r\n";
	}
	else if(spPtr=dynamic_cast<PtrData<string>*>(pArgData))      
	{ 
		LeftHandSide<string> *pLHS = dynamic_cast<LeftHandSide<string>*>(argvec[i]);
		PtrData<string> *ptrdata = dynamic_cast<PtrData<string>*>(argvec[i]);

		if(pLHS) { *spPtr->GetPtr() = &pLHS->GetData(); return; }
		else if(ptrdata) 
		{ 
			*spPtr->GetPtr() = ptrdata->GetDataPtr(); 
			if(ptrdata->GetDataPtr()) return; 
			else
			{	os<<L"In the function call \'"
					<<this->fname<<L"\', pointer variable \'"<<ptrdata->GetId()<<L"\' is not initialized.\r\n";
				os<<L"File name: "<<scanfile.filename.GetString()<<L"\r\n";
				os<<L"Line number: "<<scanfile.lineno<<L"  Character count: "<<scanfile.charcount<<L"\r\n";
			}
		}
		else os<<L"Invalid string pointer type.\r\n";
	}
	else if(ipPtr=dynamic_cast<PtrData<integer>*>(pArgData)) 
	{ 
		LeftHandSide<integer> *pLHS = dynamic_cast<LeftHandSide<integer>*>(argvec[i]);
		PtrData<integer> *ptrdata = dynamic_cast<PtrData<integer>*>(argvec[i]);

		if(pLHS) { *ipPtr->GetPtr() = &pLHS->GetData(); return; }
		else if(ptrdata) 
		{ 
			*ipPtr->GetPtr() = ptrdata->GetDataPtr(); 
			if(ptrdata->GetDataPtr()) return; 
			else
			{	os<<L"In the function call \'"
					<<this->fname<<L"\', pointer variable \'"<<ptrdata->GetId()<<L"\' is not initialized.\r\n";
				os<<L"File name: "<<scanfile.filename.GetString()<<L"\r\n";
				os<<L"Line number: "<<scanfile.lineno<<L"  Character count: "<<scanfile.charcount<<L"\r\n";
			}
		}
		else os<<L"Invalid integer pointer type.\r\n";
	}
	else if(rpPtr=dynamic_cast<PtrData<realnum>*>(pArgData)) 				
	{ 
		LeftHandSide<realnum> *pLHS = dynamic_cast<LeftHandSide<realnum>*>(argvec[i]);
		PtrData<realnum> *ptrdata = dynamic_cast<PtrData<realnum>*>(argvec[i]);

		if(pLHS) { *rpPtr->GetPtr() = &pLHS->GetData(); return; }
		else if(ptrdata) 
		{ 
			*rpPtr->GetPtr() = ptrdata->GetDataPtr();
			if(ptrdata->GetDataPtr()) return; 
			else
			{	os<<L"In the function call \'"
					<<this->fname<<L"\', pointer variable \'"<<ptrdata->GetId()<<L"\' is not initialized.\r\n";
				os<<L"File name: "<<scanfile.filename.GetString()<<L"\r\n";
				os<<L"Line number: "<<scanfile.lineno<<L"  Character count: "<<scanfile.charcount<<L"\r\n";
			}
		}
		else os<<L"Invalid real number pointer type.\r\n";
	}
	else if(cpPtr=dynamic_cast<PtrData<Complex>*>(pArgData)) 
	{ 
		LeftHandSide<Complex> *pLHS = dynamic_cast<LeftHandSide<Complex>*>(argvec[i]);
		PtrData<Complex> *ptrdata = dynamic_cast<PtrData<Complex>*>(argvec[i]);

		if(pLHS) { *cpPtr->GetPtr() = &pLHS->GetData(); return; }
		else if(ptrdata) 
		{ 
			*cpPtr->GetPtr() = ptrdata->GetDataPtr(); 
			if(ptrdata->GetDataPtr()) return; 
			else
			{	os<<L"In the function call \'"
					<<this->fname<<L"\', pointer variable \'"<<ptrdata->GetId()<<L"\' is not initialized.\r\n";
				os<<L"File name: "<<scanfile.filename.GetString()<<L"\r\n";
				os<<L"Line number: "<<scanfile.lineno<<L"  Character count: "<<scanfile.charcount<<L"\r\n";
			}
		}
		else os<<L"Invalid complex pointer type.\r\n";
	}
	else if(vpPtr=dynamic_cast<PtrData<Vector>*>(pArgData)) 
	{ 
		LeftHandSide<Vector> *pLHS = dynamic_cast<LeftHandSide<Vector>*>(argvec[i]);
		PtrData<Vector> *ptrdata = dynamic_cast<PtrData<Vector>*>(argvec[i]);

		if(pLHS) { *vpPtr->GetPtr() = &pLHS->GetData(); return; }
		else if(ptrdata) 
		{ 
			*vpPtr->GetPtr() = ptrdata->GetDataPtr();
			if(ptrdata->GetDataPtr()) return; 
			else
			{	os<<L"In the function call \'"
					<<this->fname<<L"\', pointer variable \'"<<ptrdata->GetId()<<L"\' is not initialized.\r\n";
				os<<L"File name: "<<scanfile.filename.GetString()<<L"\r\n";
				os<<L"Line number: "<<scanfile.lineno<<L"  Character count: "<<scanfile.charcount<<L"\r\n";
			}
		}
		else os<<L"Invalid vector pointer type.\r\n";
	}
	else if(upPtr=dynamic_cast<PtrData<UserData>*>(pArgData)) 
	{ 
		LeftHandSide<UserData>* pLHSUD = dynamic_cast<LeftHandSide<UserData>*>(argvec[i]);
		UdtPtrBase *ptrdata = dynamic_cast<UdtPtrBase*>(argvec[i]);
		
		if(pLHSUD)
		{
			upPtr->factory = pLHSUD->GetUdt(); upPtr->Push(); 
			*upPtr->GetPtr() = pLHSUD->GetUserData(); return;
		}
		else if(ptrdata)
		{
			if(ptrdata->pData->GetDataPtr())
			{
				upPtr->factory = ptrdata->GetUdt(); upPtr->Push(); 
				*upPtr->GetPtr() = ptrdata->pData->GetDataPtr(); return; 
			}
			else
			{	os<<L"In the function call \'"
					<<this->fname<<L"\', pointer variable \'"<<ptrdata->pData->GetId()<<L"\' is not initialized.\r\n";
				os<<L"File name: "<<scanfile.filename.GetString()<<L"\r\n";
				os<<L"Line number: "<<scanfile.lineno<<L"  Character count: "<<scanfile.charcount<<L"\r\n";
			}
		}
		else os<<L"Invalid user defined pointer type.\r\n";
	}
	else
		os<<L"Unsupported pointer type.\r\n";
	
	SetErrorFile(scanfile.filename, CString(L""), scanfile.lineno, scanfile.charcount);
	HandleSyntaxError(os.str());
}

template<typename T>
void CallFunc<T>::Step2Aptr(Data *pArgData, int i)
{
	std::wostringstream os;

	AptrDataBase *aptr = static_cast<AptrDataBase*>(pArgData);
	switch(aptr->eletype)
	{
	case TypePi1D:
		{	
			AptrData<IntgVec1D> *srcPtr;  LeftHandSideArray<IntgVec1D> *pRHS;
			AptrData<IntgVec1D> *tgtPtr = static_cast<AptrData<IntgVec1D>*>(pArgData);
			
			if(srcPtr = dynamic_cast<AptrData<IntgVec1D>*>(argvec[i]))
			{
				*tgtPtr->GetPtr() = srcPtr->GetDataPtr();
				if(srcPtr->GetDataPtr()) return;

				os<<L"In the function call \'"
					<<this->fname<<L"\', pointer variable \'"<<srcPtr->GetId()<<L"\' is not initialized.\r\n";
				os<<L"File name: "<<scanfile.filename.GetString()<<L"\r\n";
				os<<L"Line number: "<<scanfile.lineno<<L"  Character count: "<<scanfile.charcount<<L"\r\n";
			}
			else if(pRHS = dynamic_cast<LeftHandSideArray<IntgVec1D>*>(argvec[i]))
			{
				*tgtPtr->GetPtr() = &pRHS->GetData(); return;
			}
			
			os<<L"Invalid 1D integer-type array pointer.\r\n"; break;
		}

	case TypePi2D:
		{	
			AptrData<IntgVec2D> *srcPtr;  LeftHandSideArray<IntgVec2D> *pRHS;
			AptrData<IntgVec2D> *tgtPtr = static_cast<AptrData<IntgVec2D>*>(pArgData);
			
			if(srcPtr = dynamic_cast<AptrData<IntgVec2D>*>(argvec[i]))
			{
				*tgtPtr->GetPtr() = srcPtr->GetDataPtr();
				if(srcPtr->GetDataPtr()) return;

				os<<L"In the function call \'"
					<<this->fname<<L"\', pointer variable \'"<<srcPtr->GetId()<<L"\' is not initialized.\r\n";
				os<<L"File name: "<<scanfile.filename.GetString()<<L"\r\n";
				os<<L"Line number: "<<scanfile.lineno<<L"  Character count: "<<scanfile.charcount<<L"\r\n";
			}
			else if(pRHS = dynamic_cast<LeftHandSideArray<IntgVec2D>*>(argvec[i]))
			{
				*tgtPtr->GetPtr() = &pRHS->GetData(); return;
			}
			
			os<<L"Invalid 2D integer-type array pointer.\r\n"; break;
		}

	case TypePi3D:
		{	
			AptrData<IntgVec3D> *srcPtr;  LeftHandSideArray<IntgVec3D> *pRHS;
			AptrData<IntgVec3D> *tgtPtr = static_cast<AptrData<IntgVec3D>*>(pArgData);
			
			if(srcPtr = dynamic_cast<AptrData<IntgVec3D>*>(argvec[i]))
			{
				*tgtPtr->GetPtr() = srcPtr->GetDataPtr();
				if(srcPtr->GetDataPtr()) return;

				os<<L"In the function call \'"
					<<this->fname<<L"\', pointer variable \'"<<srcPtr->GetId()<<L"\' is not initialized.\r\n";
				os<<L"File name: "<<scanfile.filename.GetString()<<L"\r\n";
				os<<L"Line number: "<<scanfile.lineno<<L"  Character count: "<<scanfile.charcount<<L"\r\n";
			}
			else if(pRHS = dynamic_cast<LeftHandSideArray<IntgVec3D>*>(argvec[i]))
			{
				*tgtPtr->GetPtr() = &pRHS->GetData(); return;
			}
			os<<L"Invalid 3D integer-type array pointer.\r\n"; break;
		}

	case TypePi4D:
		{	
			AptrData<IntgVec4D> *srcPtr;  LeftHandSideArray<IntgVec4D> *pRHS;
			AptrData<IntgVec4D> *tgtPtr = static_cast<AptrData<IntgVec4D>*>(pArgData);
			
			if(srcPtr = dynamic_cast<AptrData<IntgVec4D>*>(argvec[i]))
			{
				*tgtPtr->GetPtr() = srcPtr->GetDataPtr();
				if(srcPtr->GetDataPtr()) return;

				os<<L"In the function call \'"
					<<this->fname<<L"\', pointer variable \'"<<srcPtr->GetId()<<L"\' is not initialized.\r\n";
				os<<L"File name: "<<scanfile.filename.GetString()<<L"\r\n";
				os<<L"Line number: "<<scanfile.lineno<<L"  Character count: "<<scanfile.charcount<<L"\r\n";
			}
			else if(pRHS = dynamic_cast<LeftHandSideArray<IntgVec4D>*>(argvec[i]))
			{
				*tgtPtr->GetPtr() = &pRHS->GetData(); return;
			}
			os<<L"Invalid 4D integer-type array pointer.\r\n"; break;
		}

	case TypePi5D:
		{	
			AptrData<IntgVec5D> *srcPtr;  LeftHandSideArray<IntgVec5D> *pRHS;
			AptrData<IntgVec5D> *tgtPtr = static_cast<AptrData<IntgVec5D>*>(pArgData);
			
			if(srcPtr = dynamic_cast<AptrData<IntgVec5D>*>(argvec[i]))
			{
				*tgtPtr->GetPtr() = srcPtr->GetDataPtr();
				if(srcPtr->GetDataPtr()) return;

				os<<L"In the function call \'"
					<<this->fname<<L"\', pointer variable \'"<<srcPtr->GetId()<<L"\' is not initialized.\r\n";
				os<<L"File name: "<<scanfile.filename.GetString()<<L"\r\n";
				os<<L"Line number: "<<scanfile.lineno<<L"  Character count: "<<scanfile.charcount<<L"\r\n";
			}
			else if(pRHS = dynamic_cast<LeftHandSideArray<IntgVec5D>*>(argvec[i]))
			{
				*tgtPtr->GetPtr() = &pRHS->GetData(); return;
			}
			os<<L"Invalid 5D integer-type array pointer.\r\n"; break;
		}
	
	///////////////////////////////////////////////////////////////////////////////////////

	case TypePr1D:
		{	
			AptrData<RealVec1D> *srcPtr;  LeftHandSideArray<RealVec1D> *pRHS;
			AptrData<RealVec1D> *tgtPtr = static_cast<AptrData<RealVec1D>*>(pArgData);
			
			if(srcPtr = dynamic_cast<AptrData<RealVec1D>*>(argvec[i]))
			{
				*tgtPtr->GetPtr() = srcPtr->GetDataPtr();
				if(srcPtr->GetDataPtr()) return;

				os<<L"In the function call \'"
					<<this->fname<<L"\', pointer variable \'"<<srcPtr->GetId()<<L"\' is not initialized.\r\n";
				os<<L"File name: "<<scanfile.filename.GetString()<<L"\r\n";
				os<<L"Line number: "<<scanfile.lineno<<L"  Character count: "<<scanfile.charcount<<L"\r\n";
			}
			else if(pRHS = dynamic_cast<LeftHandSideArray<RealVec1D>*>(argvec[i]))
			{
				*tgtPtr->GetPtr() = &pRHS->GetData(); return;
			}
			os<<L"Invalid 1D real-type array pointer.\r\n"; break;
		}

	case TypePr2D:
		{	
			AptrData<RealVec2D> *srcPtr;  LeftHandSideArray<RealVec2D> *pRHS;
			AptrData<RealVec2D> *tgtPtr = static_cast<AptrData<RealVec2D>*>(pArgData);
			
			if(srcPtr = dynamic_cast<AptrData<RealVec2D>*>(argvec[i]))
			{
				*tgtPtr->GetPtr() = srcPtr->GetDataPtr();
				if(srcPtr->GetDataPtr()) return;

				os<<L"In the function call \'"
					<<this->fname<<L"\', pointer variable \'"<<srcPtr->GetId()<<L"\' is not initialized.\r\n";
				os<<L"File name: "<<scanfile.filename.GetString()<<L"\r\n";
				os<<L"Line number: "<<scanfile.lineno<<L"  Character count: "<<scanfile.charcount<<L"\r\n";
			}
			else if(pRHS = dynamic_cast<LeftHandSideArray<RealVec2D>*>(argvec[i]))
			{
				*tgtPtr->GetPtr() = &pRHS->GetData(); return;
			}
			os<<L"Invalid 2D real-type array pointer.\r\n"; break;
		}

	case TypePr3D:
		{	
			AptrData<RealVec3D> *srcPtr;  LeftHandSideArray<RealVec3D> *pRHS;
			AptrData<RealVec3D> *tgtPtr = static_cast<AptrData<RealVec3D>*>(pArgData);
			
			if(srcPtr = dynamic_cast<AptrData<RealVec3D>*>(argvec[i]))
			{
				*tgtPtr->GetPtr() = srcPtr->GetDataPtr();
				if(srcPtr->GetDataPtr()) return;

				os<<L"In the function call \'"
					<<this->fname<<L"\', pointer variable \'"<<srcPtr->GetId()<<L"\' is not initialized.\r\n";
				os<<L"File name: "<<scanfile.filename.GetString()<<L"\r\n";
				os<<L"Line number: "<<scanfile.lineno<<L"  Character count: "<<scanfile.charcount<<L"\r\n";
			}
			else if(pRHS = dynamic_cast<LeftHandSideArray<RealVec3D>*>(argvec[i]))
			{
				*tgtPtr->GetPtr() = &pRHS->GetData(); return;
			}
			os<<L"Invalid 3D real-type array pointer.\r\n"; break;
		}

	case TypePr4D:
		{	
			AptrData<RealVec4D> *srcPtr;  LeftHandSideArray<RealVec4D> *pRHS;
			AptrData<RealVec4D> *tgtPtr = static_cast<AptrData<RealVec4D>*>(pArgData);
			
			if(srcPtr = dynamic_cast<AptrData<RealVec4D>*>(argvec[i]))
			{
				*tgtPtr->GetPtr() = srcPtr->GetDataPtr();
				if(srcPtr->GetDataPtr()) return;

				os<<L"In the function call \'"
					<<this->fname<<L"\', pointer variable \'"<<srcPtr->GetId()<<L"\' is not initialized.\r\n";
				os<<L"File name: "<<scanfile.filename.GetString()<<L"\r\n";
				os<<L"Line number: "<<scanfile.lineno<<L"  Character count: "<<scanfile.charcount<<L"\r\n";
			}
			else if(pRHS = dynamic_cast<LeftHandSideArray<RealVec4D>*>(argvec[i]))
			{
				*tgtPtr->GetPtr() = &pRHS->GetData(); return;
			}
			os<<L"Invalid 4D real-type array pointer.\r\n"; break;
		}

	case TypePr5D:
		{	
			AptrData<RealVec5D> *srcPtr;  LeftHandSideArray<RealVec5D> *pRHS;
			AptrData<RealVec5D> *tgtPtr = static_cast<AptrData<RealVec5D>*>(pArgData);
			
			if(srcPtr = dynamic_cast<AptrData<RealVec5D>*>(argvec[i]))
			{
				*tgtPtr->GetPtr() = srcPtr->GetDataPtr();
				if(srcPtr->GetDataPtr()) return;

				os<<L"In the function call \'"
					<<this->fname<<L"\', pointer variable \'"<<srcPtr->GetId()<<L"\' is not initialized.\r\n";
				os<<L"File name: "<<scanfile.filename.GetString()<<L"\r\n";
				os<<L"Line number: "<<scanfile.lineno<<L"  Character count: "<<scanfile.charcount<<L"\r\n";
			}
			else if(pRHS = dynamic_cast<LeftHandSideArray<RealVec5D>*>(argvec[i]))
			{
				*tgtPtr->GetPtr() = &pRHS->GetData(); return;
			}
			os<<L"Invalid 5D real-type array pointer.\r\n"; break;
		}

	///////////////////////////////////////////////////////////////////////////////////////

	case TypePc1D:
		{	
			AptrData<CplxVec1D> *srcPtr;  LeftHandSideArray<CplxVec1D> *pRHS;
			AptrData<CplxVec1D> *tgtPtr = static_cast<AptrData<CplxVec1D>*>(pArgData);
			
			if(srcPtr = dynamic_cast<AptrData<CplxVec1D>*>(argvec[i]))
			{
				*tgtPtr->GetPtr() = srcPtr->GetDataPtr();
				if(srcPtr->GetDataPtr()) return;

				os<<L"In the function call \'"
					<<this->fname<<L"\', pointer variable \'"<<srcPtr->GetId()<<L"\' is not initialized.\r\n";
				os<<L"File name: "<<scanfile.filename.GetString()<<L"\r\n";
				os<<L"Line number: "<<scanfile.lineno<<L"  Character count: "<<scanfile.charcount<<L"\r\n";
			}
			else if(pRHS = dynamic_cast<LeftHandSideArray<CplxVec1D>*>(argvec[i]))
			{
				*tgtPtr->GetPtr() = &pRHS->GetData(); return;
			}
			os<<L"Invalid 1D complex-type array pointer.\r\n"; break;
		}

	case TypePc2D:
		{	
			AptrData<CplxVec2D> *srcPtr;  LeftHandSideArray<CplxVec2D> *pRHS;
			AptrData<CplxVec2D> *tgtPtr = static_cast<AptrData<CplxVec2D>*>(pArgData);
			
			if(srcPtr = dynamic_cast<AptrData<CplxVec2D>*>(argvec[i]))
			{
				*tgtPtr->GetPtr() = srcPtr->GetDataPtr();
				if(srcPtr->GetDataPtr()) return;

				os<<L"In the function call \'"
					<<this->fname<<L"\', pointer variable \'"<<srcPtr->GetId()<<L"\' is not initialized.\r\n";
				os<<L"File name: "<<scanfile.filename.GetString()<<L"\r\n";
				os<<L"Line number: "<<scanfile.lineno<<L"  Character count: "<<scanfile.charcount<<L"\r\n";
			}
			else if(pRHS = dynamic_cast<LeftHandSideArray<CplxVec2D>*>(argvec[i]))
			{
				*tgtPtr->GetPtr() = &pRHS->GetData(); return;
			}
			os<<L"Invalid 2D complex-type array pointer.\r\n"; break;
		}

	case TypePc3D:
		{	
			AptrData<CplxVec3D> *srcPtr;  LeftHandSideArray<CplxVec3D> *pRHS;
			AptrData<CplxVec3D> *tgtPtr = static_cast<AptrData<CplxVec3D>*>(pArgData);
			
			if(srcPtr = dynamic_cast<AptrData<CplxVec3D>*>(argvec[i]))
			{
				*tgtPtr->GetPtr() = srcPtr->GetDataPtr();
				if(srcPtr->GetDataPtr()) return;

				os<<L"In the function call \'"
					<<this->fname<<L"\', pointer variable \'"<<srcPtr->GetId()<<L"\' is not initialized.\r\n";
				os<<L"File name: "<<scanfile.filename.GetString()<<L"\r\n";
				os<<L"Line number: "<<scanfile.lineno<<L"  Character count: "<<scanfile.charcount<<L"\r\n";
			}
			else if(pRHS = dynamic_cast<LeftHandSideArray<CplxVec3D>*>(argvec[i]))
			{
				*tgtPtr->GetPtr() = &pRHS->GetData(); return;
			}
			os<<L"Invalid 3D complex-type array pointer.\r\n"; break;
		}

	case TypePc4D:
		{	
			AptrData<CplxVec4D> *srcPtr;  LeftHandSideArray<CplxVec4D> *pRHS;
			AptrData<CplxVec4D> *tgtPtr = static_cast<AptrData<CplxVec4D>*>(pArgData);
			
			if(srcPtr = dynamic_cast<AptrData<CplxVec4D>*>(argvec[i]))
			{
				*tgtPtr->GetPtr() = srcPtr->GetDataPtr();
				if(srcPtr->GetDataPtr()) return;

				os<<L"In the function call \'"
					<<this->fname<<L"\', pointer variable \'"<<srcPtr->GetId()<<L"\' is not initialized.\r\n";
				os<<L"File name: "<<scanfile.filename.GetString()<<L"\r\n";
				os<<L"Line number: "<<scanfile.lineno<<L"  Character count: "<<scanfile.charcount<<L"\r\n";
			}
			else if(pRHS = dynamic_cast<LeftHandSideArray<CplxVec4D>*>(argvec[i]))
			{
				*tgtPtr->GetPtr() = &pRHS->GetData(); return;
			}
			os<<L"Invalid 4D complex-type array pointer.\r\n"; break;
		}

	case TypePc5D:
		{	
			AptrData<CplxVec5D> *srcPtr;  LeftHandSideArray<CplxVec5D> *pRHS;
			AptrData<CplxVec5D> *tgtPtr = static_cast<AptrData<CplxVec5D>*>(pArgData);
			
			if(srcPtr = dynamic_cast<AptrData<CplxVec5D>*>(argvec[i]))
			{
				*tgtPtr->GetPtr() = srcPtr->GetDataPtr();
				if(srcPtr->GetDataPtr()) return;

				os<<L"In the function call \'"
					<<this->fname<<L"\', pointer variable \'"<<srcPtr->GetId()<<L"\' is not initialized.\r\n";
				os<<L"File name: "<<scanfile.filename.GetString()<<L"\r\n";
				os<<L"Line number: "<<scanfile.lineno<<L"  Character count: "<<scanfile.charcount<<L"\r\n";
			}
			else if(pRHS = dynamic_cast<LeftHandSideArray<CplxVec5D>*>(argvec[i]))
			{
				*tgtPtr->GetPtr() = &pRHS->GetData(); return;
			}
			os<<L"Invalid 5D complex-type array pointer.\r\n"; break;
		}

	///////////////////////////////////////////////////////////////////////////////////////

	case TypePv1D:
		{	
			AptrData<VctrVec1D> *srcPtr;  LeftHandSideArray<VctrVec1D> *pRHS;
			AptrData<VctrVec1D> *tgtPtr = static_cast<AptrData<VctrVec1D>*>(pArgData);
			
			if(srcPtr = dynamic_cast<AptrData<VctrVec1D>*>(argvec[i]))
			{
				*tgtPtr->GetPtr() = srcPtr->GetDataPtr();
				if(srcPtr->GetDataPtr()) return;

				os<<L"In the function call \'"
					<<this->fname<<L"\', pointer variable \'"<<srcPtr->GetId()<<L"\' is not initialized.\r\n";
				os<<L"File name: "<<scanfile.filename.GetString()<<L"\r\n";
				os<<L"Line number: "<<scanfile.lineno<<L"  Character count: "<<scanfile.charcount<<L"\r\n";
			}
			else if(pRHS = dynamic_cast<LeftHandSideArray<VctrVec1D>*>(argvec[i]))
			{
				*tgtPtr->GetPtr() = &pRHS->GetData(); return;
			}
			os<<L"Invalid 1D vector-type array pointer.\r\n"; break;
		}

	case TypePv2D:
		{	
			AptrData<VctrVec2D> *srcPtr;  LeftHandSideArray<VctrVec2D> *pRHS;
			AptrData<VctrVec2D> *tgtPtr = static_cast<AptrData<VctrVec2D>*>(pArgData);
			
			if(srcPtr = dynamic_cast<AptrData<VctrVec2D>*>(argvec[i]))
			{
				*tgtPtr->GetPtr() = srcPtr->GetDataPtr();
				if(srcPtr->GetDataPtr()) return;

				os<<L"In the function call \'"
					<<this->fname<<L"\', pointer variable \'"<<srcPtr->GetId()<<L"\' is not initialized.\r\n";
				os<<L"File name: "<<scanfile.filename.GetString()<<L"\r\n";
				os<<L"Line number: "<<scanfile.lineno<<L"  Character count: "<<scanfile.charcount<<L"\r\n";
			}
			else if(pRHS = dynamic_cast<LeftHandSideArray<VctrVec2D>*>(argvec[i]))
			{
				*tgtPtr->GetPtr() = &pRHS->GetData(); return;
			}
			os<<L"Invalid 2D vector-type array pointer.\r\n"; break;
		}

	case TypePv3D:
		{	
			AptrData<VctrVec3D> *srcPtr;  LeftHandSideArray<VctrVec3D> *pRHS;
			AptrData<VctrVec3D> *tgtPtr = static_cast<AptrData<VctrVec3D>*>(pArgData);
			
			if(srcPtr = dynamic_cast<AptrData<VctrVec3D>*>(argvec[i]))
			{
				*tgtPtr->GetPtr() = srcPtr->GetDataPtr();
				if(srcPtr->GetDataPtr()) return;

				os<<L"In the function call \'"
					<<this->fname<<L"\', pointer variable \'"<<srcPtr->GetId()<<L"\' is not initialized.\r\n";
				os<<L"File name: "<<scanfile.filename.GetString()<<L"\r\n";
				os<<L"Line number: "<<scanfile.lineno<<L"  Character count: "<<scanfile.charcount<<L"\r\n";
			}
			else if(pRHS = dynamic_cast<LeftHandSideArray<VctrVec3D>*>(argvec[i]))
			{
				*tgtPtr->GetPtr() = &pRHS->GetData(); return;
			}
			os<<L"Invalid 3D vector-type array pointer.\r\n"; break;
		}

	case TypePv4D:
		{	
			AptrData<VctrVec4D> *srcPtr;  LeftHandSideArray<VctrVec4D> *pRHS;
			AptrData<VctrVec4D> *tgtPtr = static_cast<AptrData<VctrVec4D>*>(pArgData);
			
			if(srcPtr = dynamic_cast<AptrData<VctrVec4D>*>(argvec[i]))
			{
				*tgtPtr->GetPtr() = srcPtr->GetDataPtr();
				if(srcPtr->GetDataPtr()) return;

				os<<L"In the function call \'"
					<<this->fname<<L"\', pointer variable \'"<<srcPtr->GetId()<<L"\' is not initialized.\r\n";
				os<<L"File name: "<<scanfile.filename.GetString()<<L"\r\n";
				os<<L"Line number: "<<scanfile.lineno<<L"  Character count: "<<scanfile.charcount<<L"\r\n";
			}
			else if(pRHS = dynamic_cast<LeftHandSideArray<VctrVec4D>*>(argvec[i]))
			{
				*tgtPtr->GetPtr() = &pRHS->GetData(); return;
			}
			os<<L"Invalid 4D vector-type array pointer.\r\n"; break;
		}

	case TypePv5D:
		{	
			AptrData<VctrVec5D> *srcPtr;  LeftHandSideArray<VctrVec5D> *pRHS;
			AptrData<VctrVec5D> *tgtPtr = static_cast<AptrData<VctrVec5D>*>(pArgData);
			
			if(srcPtr = dynamic_cast<AptrData<VctrVec5D>*>(argvec[i]))
			{
				*tgtPtr->GetPtr() = srcPtr->GetDataPtr();
				if(srcPtr->GetDataPtr()) return;

				os<<L"In the function call \'"
					<<this->fname<<L"\', pointer variable \'"<<srcPtr->GetId()<<L"\' is not initialized.\r\n";
				os<<L"File name: "<<scanfile.filename.GetString()<<L"\r\n";
				os<<L"Line number: "<<scanfile.lineno<<L"  Character count: "<<scanfile.charcount<<L"\r\n";
			}
			else if(pRHS = dynamic_cast<LeftHandSideArray<VctrVec5D>*>(argvec[i]))
			{
				*tgtPtr->GetPtr() = &pRHS->GetData(); return;
			}
			
			os<<L"Invalid 5D vector-type array pointer.\r\n"; break;
		}

	default: os<<L"Unsupported array pointer type.\r\n";

	}

	extern void SetErrorFile(CString& filename,
		CString& token, int errorline, int errorchar);

	SetErrorFile(scanfile.filename, CString(L""), scanfile.lineno, scanfile.charcount);
	HandleSyntaxError(os.str());
}

template<typename T>
void CallFunc<T>::Step3Aptr(Data *pParData, Data* pArgData)
{
	AptrData<IntgVec1D> *ai1ptr, *bi1ptr;
	AptrData<IntgVec2D> *ai2ptr, *bi2ptr;
	AptrData<IntgVec3D> *ai3ptr, *bi3ptr;
	AptrData<IntgVec4D> *ai4ptr, *bi4ptr;
	AptrData<IntgVec5D> *ai5ptr, *bi5ptr;

	AptrData<RealVec1D> *ar1ptr, *br1ptr;
	AptrData<RealVec2D> *ar2ptr, *br2ptr;
	AptrData<RealVec3D> *ar3ptr, *br3ptr;
	AptrData<RealVec4D> *ar4ptr, *br4ptr;
	AptrData<RealVec5D> *ar5ptr, *br5ptr;

	AptrData<CplxVec1D> *ac1ptr, *bc1ptr;
	AptrData<CplxVec2D> *ac2ptr, *bc2ptr;
	AptrData<CplxVec3D> *ac3ptr, *bc3ptr;
	AptrData<CplxVec4D> *ac4ptr, *bc4ptr;
	AptrData<CplxVec5D> *ac5ptr, *bc5ptr;

	AptrData<VctrVec1D> *av1ptr, *bv1ptr;
	AptrData<VctrVec2D> *av2ptr, *bv2ptr;
	AptrData<VctrVec3D> *av3ptr, *bv3ptr;
	AptrData<VctrVec4D> *av4ptr, *bv4ptr;
	AptrData<VctrVec5D> *av5ptr, *bv5ptr;

	if((ai1ptr=dynamic_cast<AptrData<IntgVec1D>*>(pParData)) &&
		(bi1ptr=dynamic_cast<AptrData<IntgVec1D>*>(pArgData)))  
		*ai1ptr->GetPtr() = bi1ptr->GetData();
	else if((ai2ptr=dynamic_cast<AptrData<IntgVec2D>*>(pParData)) &&
		(bi2ptr=dynamic_cast<AptrData<IntgVec2D>*>(pArgData)))  
		*ai2ptr->GetPtr() = bi2ptr->GetData();
	else if((ai3ptr=dynamic_cast<AptrData<IntgVec3D>*>(pParData)) &&
		(bi3ptr=dynamic_cast<AptrData<IntgVec3D>*>(pArgData)))  
		*ai3ptr->GetPtr() = bi3ptr->GetData();
	else if((ai4ptr=dynamic_cast<AptrData<IntgVec4D>*>(pParData)) &&
		(bi4ptr=dynamic_cast<AptrData<IntgVec4D>*>(pArgData)))
		*ai4ptr->GetPtr() = bi4ptr->GetData();
	else if((ai5ptr=dynamic_cast<AptrData<IntgVec5D>*>(pParData)) &&
		(bi5ptr=dynamic_cast<AptrData<IntgVec5D>*>(pArgData)))
		*ai5ptr->GetPtr() = bi5ptr->GetData();
	
	else if((ar1ptr=dynamic_cast<AptrData<RealVec1D>*>(pParData)) &&
		(br1ptr=dynamic_cast<AptrData<RealVec1D>*>(pArgData)))  
		*ar1ptr->GetPtr() = br1ptr->GetData();
	else if((ar2ptr=dynamic_cast<AptrData<RealVec2D>*>(pParData)) &&
		(br2ptr=dynamic_cast<AptrData<RealVec2D>*>(pArgData)))
		*ar2ptr->GetPtr() = br2ptr->GetData();
	else if((ar3ptr=dynamic_cast<AptrData<RealVec3D>*>(pParData)) &&
		(br3ptr=dynamic_cast<AptrData<RealVec3D>*>(pArgData)))
		*ar3ptr->GetPtr() = br3ptr->GetData();
	else if((ar4ptr=dynamic_cast<AptrData<RealVec4D>*>(pParData)) &&
		(br4ptr=dynamic_cast<AptrData<RealVec4D>*>(pArgData)))
		*ar4ptr->GetPtr() = br4ptr->GetData();
	else if((ar5ptr=dynamic_cast<AptrData<RealVec5D>*>(pParData)) &&
		(br5ptr=dynamic_cast<AptrData<RealVec5D>*>(pArgData)))
		*ar5ptr->GetPtr() = br5ptr->GetData();

	else if((ac1ptr=dynamic_cast<AptrData<CplxVec1D>*>(pParData)) &&
		(bc1ptr=dynamic_cast<AptrData<CplxVec1D>*>(pArgData)))
		*ac1ptr->GetPtr() = bc1ptr->GetData();
	else if((ac2ptr=dynamic_cast<AptrData<CplxVec2D>*>(pParData)) &&
		(bc2ptr=dynamic_cast<AptrData<CplxVec2D>*>(pArgData)))
		*ac2ptr->GetPtr() = bc2ptr->GetData();
	else if((ac3ptr=dynamic_cast<AptrData<CplxVec3D>*>(pParData)) &&
		(bc3ptr=dynamic_cast<AptrData<CplxVec3D>*>(pArgData)))
		*ac3ptr->GetPtr() = bc3ptr->GetData();
	else if((ac4ptr=dynamic_cast<AptrData<CplxVec4D>*>(pParData)) &&
		(bc4ptr=dynamic_cast<AptrData<CplxVec4D>*>(pArgData)))
		*ac4ptr->GetPtr() = bc4ptr->GetData();
	else if((ac5ptr=dynamic_cast<AptrData<CplxVec5D>*>(pParData)) &&
		(bc5ptr=dynamic_cast<AptrData<CplxVec5D>*>(pArgData)))
		*ac5ptr->GetPtr() = bc5ptr->GetData();

	else if((av1ptr=dynamic_cast<AptrData<VctrVec1D>*>(pParData)) &&
		(bv1ptr=dynamic_cast<AptrData<VctrVec1D>*>(pArgData)))
		*av1ptr->GetPtr() = bv1ptr->GetData();
	else if((av2ptr=dynamic_cast<AptrData<VctrVec2D>*>(pParData)) &&
		(bv2ptr=dynamic_cast<AptrData<VctrVec2D>*>(pArgData)))
		*av2ptr->GetPtr() = bv2ptr->GetData();
	else if((av3ptr=dynamic_cast<AptrData<VctrVec3D>*>(pParData)) &&
		(bv3ptr=dynamic_cast<AptrData<VctrVec3D>*>(pArgData)))
		*av3ptr->GetPtr() = bv3ptr->GetData();
	else if((av4ptr=dynamic_cast<AptrData<VctrVec4D>*>(pParData)) &&
		(bv4ptr=dynamic_cast<AptrData<VctrVec4D>*>(pArgData)))
		*av4ptr->GetPtr() = bv4ptr->GetData();
	else if((av5ptr=dynamic_cast<AptrData<VctrVec5D>*>(pParData)) &&
		(bv5ptr=dynamic_cast<AptrData<VctrVec5D>*>(pArgData)))
		*av5ptr->GetPtr() = bv5ptr->GetData();

	else 
		throw UnicodeException(_T("Pointer argument type mismatch"));
	
	return;
}

template<typename T>
void CallFunc<T>::Step3Ptr(Data *pParData, Data* pArgData)
{
	PtrData<bool>		*bpPtr, *baPtr;
	PtrData<string>		*spPtr, *saPtr;
	PtrData<integer>	*ipPtr, *iaPtr;
	PtrData<realnum>	*rpPtr, *raPtr;
	PtrData<Complex>	*cpPtr, *caPtr;
	PtrData<Vector>		*vpPtr, *vaPtr;
	PtrData<UserData>	*upPtr, *uaPtr;

	if((bpPtr=dynamic_cast<PtrData<bool>*>(pParData)) &&
		(baPtr=dynamic_cast<PtrData<bool>*>(pArgData)))     
		*bpPtr->GetPtr() = baPtr->GetData(); 
	else if((spPtr=dynamic_cast<PtrData<string>*>(pParData)) &&
		(saPtr=dynamic_cast<PtrData<string>*>(pArgData)))     
		*spPtr->GetPtr() = saPtr->GetData(); 
	else if((ipPtr=dynamic_cast<PtrData<integer>*>(pParData)) &&
		(iaPtr=dynamic_cast<PtrData<integer>*>(pArgData)))
		*ipPtr->GetPtr() = iaPtr->GetData();
	else if((rpPtr=dynamic_cast<PtrData<realnum>*>(pParData)) &&
		(raPtr=dynamic_cast<PtrData<realnum>*>(pArgData)))
		*rpPtr->GetPtr() = raPtr->GetData();
	else if((cpPtr=dynamic_cast<PtrData<Complex>*>(pParData)) &&
		(caPtr=dynamic_cast<PtrData<Complex>*>(pArgData)))
		*cpPtr->GetPtr() = caPtr->GetData();
	else if((vpPtr=dynamic_cast<PtrData<Vector>*>(pParData)) &&
		(vaPtr=dynamic_cast<PtrData<Vector>*>(pArgData)))
		*vpPtr->GetPtr() = vaPtr->GetData();
	else if((upPtr=dynamic_cast<PtrData<UserData>*>(pParData)) &&
		(uaPtr=dynamic_cast<PtrData<UserData>*>(pArgData)))
		*upPtr->GetPtr() = uaPtr->GetData();
	else 
		throw UnicodeException(_T("Pointer argument type mismatch"));
	
	return;
}

template<typename T>
void CallFunc<T>::Step1Aptr(Data *pParData, VarList* pArgData)
{
	AptrDataBase *aptr = static_cast<AptrDataBase*>(pParData);
	std::wstring id = pParData->GetId();

	switch(aptr->eletype)
	{
	case TypePi1D:
		{ pArgData->put(new AptrData<IntgVec1D>(id, true)); return; }
	case TypePi2D:
		{ pArgData->put(new AptrData<IntgVec2D>(id, true)); return; }
	case TypePi3D:
		{ pArgData->put(new AptrData<IntgVec3D>(id, true)); return; }
	case TypePi4D:
		{ pArgData->put(new AptrData<IntgVec4D>(id, true)); return; }
	case TypePi5D:
		{ pArgData->put(new AptrData<IntgVec5D>(id, true)); return; }

	case TypePr1D:
		{ pArgData->put(new AptrData<RealVec1D>(id, true)); return; }
	case TypePr2D:
		{ pArgData->put(new AptrData<RealVec2D>(id, true)); return; }
	case TypePr3D:
		{ pArgData->put(new AptrData<RealVec3D>(id, true)); return; }
	case TypePr4D:
		{ pArgData->put(new AptrData<RealVec4D>(id, true)); return; }
	case TypePr5D:
		{ pArgData->put(new AptrData<RealVec5D>(id, true)); return; }

	case TypePc1D:
		{ pArgData->put(new AptrData<CplxVec1D>(id, true)); return; }
	case TypePc2D:
		{ pArgData->put(new AptrData<CplxVec2D>(id, true)); return; }
	case TypePc3D:
		{ pArgData->put(new AptrData<CplxVec3D>(id, true)); return; }
	case TypePc4D:
		{ pArgData->put(new AptrData<CplxVec4D>(id, true)); return; }
	case TypePc5D:
		{ pArgData->put(new AptrData<CplxVec5D>(id, true)); return; }

	case TypePv1D:
		{ pArgData->put(new AptrData<VctrVec1D>(id, true)); return; }
	case TypePv2D:
		{ pArgData->put(new AptrData<VctrVec2D>(id, true)); return; }
	case TypePv3D:
		{ pArgData->put(new AptrData<VctrVec3D>(id, true)); return; }
	case TypePv4D:
		{ pArgData->put(new AptrData<VctrVec4D>(id, true)); return; }
	case TypePv5D:
		{ pArgData->put(new AptrData<VctrVec5D>(id, true)); return; }
	}
	
	HandleSyntaxError(_T("Unsupported pointer type for function argument.\r\n"));
}

template<typename T>
void CallFunc<T>::Step3Array(Data *pParData, Data* pArgData)
{
	Array *pAryL = static_cast<Array*>(pParData);

	switch(pAryL->GetDim())
	{
	case 1:
		switch(pAryL->GetEleType())
		{
		case TypeIntg:
			{
				Array1D<integer> *pLeft  = static_cast<Array1D<integer>*>(pParData);
				Array1D<integer> *pRight = static_cast<Array1D<integer>*>(pArgData);
				*pLeft = *pRight; return;
			}
		case TypeReal:
			{
				Array1D<realnum> *pLeft  = static_cast<Array1D<realnum>*>(pParData);
				Array1D<realnum> *pRight = static_cast<Array1D<realnum>*>(pArgData);
				*pLeft = *pRight; return;
			}
		case TypeCplx:
			{
				Array1D<Complex> *pLeft  = static_cast<Array1D<Complex>*>(pParData);
				Array1D<Complex> *pRight = static_cast<Array1D<Complex>*>(pArgData);
				*pLeft = *pRight; return;
			}
		case TypeVctr:
			{
				Array1D<Vector> *pLeft  = static_cast<Array1D<Vector>*>(pParData);
				Array1D<Vector> *pRight = static_cast<Array1D<Vector>*>(pArgData);
				*pLeft = *pRight; return;
			}
		default:
			throw RunTimeException(_T("Unsupported array element type.\r\n"));
		}
		break;
	case 2:
		switch(pAryL->GetEleType())
		{
		case TypeIntg:
			{
				Array2D<integer> *pLeft  = static_cast<Array2D<integer>*>(pParData);
				Array2D<integer> *pRight = static_cast<Array2D<integer>*>(pArgData);
				*pLeft = *pRight; return;
			}
		case TypeReal:
			{
				Array2D<realnum> *pLeft  = static_cast<Array2D<realnum>*>(pParData);
				Array2D<realnum> *pRight = static_cast<Array2D<realnum>*>(pArgData);
				*pLeft = *pRight; return;
			}
		case TypeCplx:
			{
				Array2D<Complex> *pLeft  = static_cast<Array2D<Complex>*>(pParData);
				Array2D<Complex> *pRight = static_cast<Array2D<Complex>*>(pArgData);
				*pLeft = *pRight; return;
			}
		case TypeVctr:
			{
				Array2D<Vector> *pLeft  = static_cast<Array2D<Vector>*>(pParData);
				Array2D<Vector> *pRight = static_cast<Array2D<Vector>*>(pArgData);
				*pLeft = *pRight; return;
			}
		default:
			throw RunTimeException(_T("Unsupported array element type.\r\n"));
		}
		return;
	case 3:
		switch(pAryL->GetEleType())
		{
		case TypeIntg:
			{
				Array3D<integer> *pLeft  = static_cast<Array3D<integer>*>(pParData);
				Array3D<integer> *pRight = static_cast<Array3D<integer>*>(pArgData);
				*pLeft = *pRight; return;
			}
		case TypeReal:
			{
				Array3D<realnum> *pLeft  = static_cast<Array3D<realnum>*>(pParData);
				Array3D<realnum> *pRight = static_cast<Array3D<realnum>*>(pArgData);
				*pLeft = *pRight; return;
			}
		case TypeCplx:
			{
				Array3D<Complex> *pLeft  = static_cast<Array3D<Complex>*>(pParData);
				Array3D<Complex> *pRight = static_cast<Array3D<Complex>*>(pArgData);
				*pLeft = *pRight; return;
			}
		case TypeVctr:
			{
				Array3D<Vector> *pLeft  = static_cast<Array3D<Vector>*>(pParData);
				Array3D<Vector> *pRight = static_cast<Array3D<Vector>*>(pArgData);
				*pLeft = *pRight; return;
			}
		default:
			throw RunTimeException(_T("Unsupported array element type.\r\n"));
		}
		return;
	case 4:
		switch(pAryL->GetEleType())
		{
		case TypeIntg:
			{
				Array4D<integer> *pLeft  = static_cast<Array4D<integer>*>(pParData);
				Array4D<integer> *pRight = static_cast<Array4D<integer>*>(pArgData);
				*pLeft = *pRight; return;
			}
		case TypeReal:
			{
				Array4D<realnum> *pLeft  = static_cast<Array4D<realnum>*>(pParData);
				Array4D<realnum> *pRight = static_cast<Array4D<realnum>*>(pArgData);
				*pLeft = *pRight; return;
			}
		case TypeCplx:
			{
				Array4D<Complex> *pLeft  = static_cast<Array4D<Complex>*>(pParData);
				Array4D<Complex> *pRight = static_cast<Array4D<Complex>*>(pArgData);
				*pLeft = *pRight; return;
			}
		case TypeVctr:
			{
				Array4D<Vector> *pLeft  = static_cast<Array4D<Vector>*>(pParData);
				Array4D<Vector> *pRight = static_cast<Array4D<Vector>*>(pArgData);
				*pLeft = *pRight; return;
			}
		default:
			throw RunTimeException(_T("Unsupported array element type.\r\n"));
		}
		break;
	case 5:
		switch(pAryL->GetEleType())
		{
		case TypeIntg:
			{
				Array5D<integer> *pLeft  = static_cast<Array5D<integer>*>(pParData);
				Array5D<integer> *pRight = static_cast<Array5D<integer>*>(pArgData);
				*pLeft = *pRight; return;
			}
		case TypeReal:
			{
				Array5D<realnum> *pLeft  = static_cast<Array5D<realnum>*>(pParData);
				Array5D<realnum> *pRight = static_cast<Array5D<realnum>*>(pArgData);
				*pLeft = *pRight; return;
			}
		case TypeCplx:
			{
				Array5D<Complex> *pLeft  = static_cast<Array5D<Complex>*>(pParData);
				Array5D<Complex> *pRight = static_cast<Array5D<Complex>*>(pArgData);
				*pLeft = *pRight; return;
			}
		case TypeVctr:
			{
				Array5D<Vector> *pLeft  = static_cast<Array5D<Vector>*>(pParData);
				Array5D<Vector> *pRight = static_cast<Array5D<Vector>*>(pArgData);
				*pLeft = *pRight; return;
			}
		default:
			throw RunTimeException(_T("Unsupported array element type.\r\n"));
		}
		break;
	default:
		throw RunTimeException(_T("Unsupported array dimension.\r\n"));
	}
}

template<typename T>
void CallFunc<T>::Step2Array(Data* pArgData, int i)
{
	Array *pAry = static_cast<Array*>(pArgData);
	switch(pAry->GetArrayType())
	{
	case TypeAi1D:
		{
			*static_cast<Array1D<integer>*>(pArgData)->GetTop() = 
				static_cast<Expression<IntgVec1D>*>(argvec[i])->Eval(); return;
		}
	case TypeAi2D:
		{
			*static_cast<Array2D<integer>*>(pArgData)->GetTop() = 
				static_cast<Expression<IntgVec2D>*>(argvec[i])->Eval(); return;
		}
	case TypeAi3D:
		{
			*static_cast<Array3D<integer>*>(pArgData)->GetTop() = 
				static_cast<Expression<IntgVec3D>*>(argvec[i])->Eval(); return;
		}
	case TypeAi4D:
		{
			*static_cast<Array4D<integer>*>(pArgData)->GetTop() = 
				static_cast<Expression<IntgVec4D>*>(argvec[i])->Eval(); return;
		}
	case TypeAi5D:
		{
			*static_cast<Array5D<integer>*>(pArgData)->GetTop() = 
				static_cast<Expression<IntgVec5D>*>(argvec[i])->Eval(); return;
		}

	///////////////////////////////////////////////////////////////////////////
	
	case TypeAr1D:
		{
			*static_cast<Array1D<realnum>*>(pArgData)->GetTop() = 
				static_cast<Expression<RealVec1D>*>(argvec[i])->Eval(); return;
		}
	case TypeAr2D:
		{
			*static_cast<Array2D<realnum>*>(pArgData)->GetTop() = 
				static_cast<Expression<RealVec2D>*>(argvec[i])->Eval(); return;
		}
	case TypeAr3D:
		{
			*static_cast<Array3D<realnum>*>(pArgData)->GetTop() = 
				static_cast<Expression<RealVec3D>*>(argvec[i])->Eval(); return;
		}
	case TypeAr4D:
		{
			*static_cast<Array4D<realnum>*>(pArgData)->GetTop() = 
				static_cast<Expression<RealVec4D>*>(argvec[i])->Eval(); return;
		}
	case TypeAr5D:
		{
			*static_cast<Array5D<realnum>*>(pArgData)->GetTop() = 
				static_cast<Expression<RealVec5D>*>(argvec[i])->Eval(); return;
		}

	///////////////////////////////////////////////////////////////////////////
	
	case TypeAc1D:
		{
			*static_cast<Array1D<Complex>*>(pArgData)->GetTop() = 
				static_cast<Expression<CplxVec1D>*>(argvec[i])->Eval(); return;
		}
	case TypeAc2D:
		{
			*static_cast<Array2D<Complex>*>(pArgData)->GetTop() = 
				static_cast<Expression<CplxVec2D>*>(argvec[i])->Eval(); return;
		}
	case TypeAc3D:
		{
			*static_cast<Array3D<Complex>*>(pArgData)->GetTop() = 
				static_cast<Expression<CplxVec3D>*>(argvec[i])->Eval(); return;
		}
	case TypeAc4D:
		{
			*static_cast<Array4D<Complex>*>(pArgData)->GetTop() = 
				static_cast<Expression<CplxVec4D>*>(argvec[i])->Eval(); return;
		}
	case TypeAc5D:
		{
			*static_cast<Array5D<Complex>*>(pArgData)->GetTop() = 
				static_cast<Expression<CplxVec5D>*>(argvec[i])->Eval(); return;
		}

	///////////////////////////////////////////////////////////////////////////
	
	case TypeAv1D:
		{
			*static_cast<Array1D<Vector>*>(pArgData)->GetTop() = 
				static_cast<Expression<VctrVec1D>*>(argvec[i])->Eval(); return;
		}
	case TypeAv2D:
		{
			*static_cast<Array2D<Vector>*>(pArgData)->GetTop() = 
				static_cast<Expression<VctrVec2D>*>(argvec[i])->Eval(); return;
		}
	case TypeAv3D:
		{
			*static_cast<Array3D<Vector>*>(pArgData)->GetTop() = 
				static_cast<Expression<VctrVec3D>*>(argvec[i])->Eval(); return;
		}
	case TypeAv4D:
		{
			*static_cast<Array4D<Vector>*>(pArgData)->GetTop() = 
				static_cast<Expression<VctrVec4D>*>(argvec[i])->Eval(); return;
		}
	case TypeAv5D:
		{
			*static_cast<Array5D<Vector>*>(pArgData)->GetTop() = 
				static_cast<Expression<VctrVec5D>*>(argvec[i])->Eval(); return;
		}
	}

	HandleSyntaxError(_T("Unsupported array type for function argument.\r\n"));
}

template<typename T>
void CallFunc<T>::Step1Array(Data *pParData, VarList* pArgData)
{
	Array *pAry = static_cast<Array*>(pParData); 
	std::wstring id = pParData->GetId();

	switch(pAry->GetArrayType())
	{
	case TypeAi1D:
		{ pArgData->put(new Array1D<integer>(id, true)); return; }
	case TypeAi2D:
		{ pArgData->put(new Array2D<integer>(id, true)); return; }
	case TypeAi3D:
		{ pArgData->put(new Array3D<integer>(id, true)); return; }
	case TypeAi4D:
		{ pArgData->put(new Array4D<integer>(id, true)); return; }
	case TypeAi5D:
		{ pArgData->put(new Array5D<integer>(id, true)); return; }

	case TypeAr1D:
		{ pArgData->put(new Array1D<realnum>(id, true)); return; }
	case TypeAr2D:
		{ pArgData->put(new Array2D<realnum>(id, true)); return; }
	case TypeAr3D:
		{ pArgData->put(new Array3D<realnum>(id, true)); return; }
	case TypeAr4D:
		{ pArgData->put(new Array4D<realnum>(id, true)); return; }
	case TypeAr5D:
		{ pArgData->put(new Array5D<realnum>(id, true)); return; }

	case TypeAc1D:
		{ pArgData->put(new Array1D<Complex>(id, true)); return; }
	case TypeAc2D:
		{ pArgData->put(new Array2D<Complex>(id, true)); return; }
	case TypeAc3D:
		{ pArgData->put(new Array3D<Complex>(id, true)); return; }
	case TypeAc4D:
		{ pArgData->put(new Array4D<Complex>(id, true)); return; }
	case TypeAc5D:
		{ pArgData->put(new Array5D<Complex>(id, true)); return; }

	case TypeAv1D:
		{ pArgData->put(new Array1D<Vector>(id, true)); return; }
	case TypeAv2D:
		{ pArgData->put(new Array2D<Vector>(id, true)); return; }
	case TypeAv3D:
		{ pArgData->put(new Array3D<Vector>(id, true)); return; }
	case TypeAv4D:
		{ pArgData->put(new Array4D<Vector>(id, true)); return; }
	case TypeAv5D:
		{ pArgData->put(new Array5D<Vector>(id, true)); return; }
	}
	
	HandleSyntaxError(_T("Unsupported array type for function argument.\r\n"));
}

template<typename T>
void CallFunc<T>::ReplArgs()
{
	int argcount = int(argvec.size());
	VarList *varptr = pFunc->pStmtList->GetVarList();
	int parcount = varptr->size();

	DataType partype; VarList ArgData; 
	Data *pArgData, *pParData; 
	
	// VarMemBase *udtPtr;  VarUserBase *vubPtr;

	ExpBool	*boolptr; ExpIntg *intgptr;
	ExpReal	*realptr; ExpCplx *cplxptr;
	ExpVctr	*vctrptr; ExpStrn *strnptr;
	ExpUser *userptr;
	
	// Step 1. ///////////////////////
	for(int i= 0; i<argcount; ++i)
	{
		pParData = varptr->at(i);
		partype = pParData->GetType();

		switch(partype)
		{
		case TypeBool:
			ArgData.put(new BoolData(pParData->GetId(), true)); break;
		case TypeIntg:
			ArgData.put(new IntgData(pParData->GetId(), true)); break;
		case TypeReal:
			ArgData.put(new RealData(pParData->GetId(), true)); break;
		case TypeCplx:
			ArgData.put(new CplxData(pParData->GetId(), true)); break;
		case TypeVctr:
			ArgData.put(new VctrData(pParData->GetId(), true)); break;
		case TypeStrn:
			ArgData.put(new StrnData(pParData->GetId(), true)); break;

		case TypeUser:
			{
				ArgData.put(new UserData(L"No Name", 
					dynamic_cast<DeclUdt*>(pFunc->pStmtList->GetStatement(i))->GetFactory(), true));
				break;
			}
		case TypeAptr:
			Step1Aptr(pParData, &ArgData); break; 
		case TypeArray:
			Step1Array(pParData, &ArgData); break; 
		case TypePtr:
			Step1Ptr(pParData, &ArgData); break;
		default:
			throw UnicodeException(_T("Step 1. Failed"));
		}
	}
		
	// Step. 2 //////////////////////////////////////
					
	for(int i= 0; i<argcount; ++i)
	{
		pArgData = ArgData.at(i);
		partype = varptr->at(i)->GetType();
		
		switch(partype)
		{
		case TypeBool:
			boolptr = dynamic_cast<ExpBool*>(argvec[i]);
			if(boolptr)
			{	static_cast<BoolData*>(pArgData)->GetData() = boolptr->Eval(); break; }
			else
				throw UnicodeException(_T("First arg, type mismatch."));

		case TypeIntg:
			if(intgptr = dynamic_cast<ExpIntg*>(argvec[i]))
			{ static_cast<IntgData*>(pArgData)->GetData() = intgptr->Eval(); break; }
			else if(realptr = dynamic_cast<ExpReal*>(argvec[i]))
			{ static_cast<IntgData*>(pArgData)->GetData() = ToInteger(realptr->Eval()); break; }
			else
				throw UnicodeException(_T("First arg, type mismatch."));

		case TypeReal:
			if(realptr=dynamic_cast<ExpReal*>(argvec[i]))
			{ static_cast<RealData*>(pArgData)->GetData() = realptr->Eval(); break; }
			else if(intgptr = dynamic_cast<ExpIntg*>(argvec[i]))
			{ static_cast<RealData*>(pArgData)->GetData() = realnum(intgptr->Eval()); break; }
			else 
				throw UnicodeException(_T("First arg, type mismatch."));

		case TypeCplx:
			cplxptr = dynamic_cast<ExpCplx*>(argvec[i]);
			if(cplxptr)
			{ static_cast<CplxData*>(pArgData)->GetData() = cplxptr->Eval(); break; }
			else
				throw UnicodeException(_T("First arg, type mismatch."));

		case TypeVctr:
			vctrptr = dynamic_cast<ExpVctr*>(argvec[i]);
			if(vctrptr)
			{ dynamic_cast<VctrData*>(pArgData)->GetData() = vctrptr->Eval(); break; }
			else
				throw UnicodeException(_T("Arg, type mismatch."));

		case TypeStrn:
			strnptr = dynamic_cast<ExpStrn*>(argvec[i]);
			if(strnptr)
			{ dynamic_cast<StrnData*>(pArgData)->GetData() = strnptr->Eval(); break; }
			else
				throw UnicodeException(_T("Arg, type mismatch."));

		case TypeUser:
			if(userptr = dynamic_cast<ExpUser*>(argvec[i]))
			{ 
				*dynamic_cast<UserData*>(pArgData) = userptr->Eval(); break; 
			}
			else
				throw UnicodeException(_T("Arg, type mismatch."));

		case TypeAptr:
			Step2Aptr(pArgData, i); break;

		case TypeArray:
			Step2Array(pArgData, i); break;

		case TypePtr:
			Step2Ptr(pArgData, i); break;

		default:
			throw UnicodeException(_T("Arg type unknown"));
		}
	}

	// Step 3. /////////////////////////////////////////////////
	pFunc->pStmtList->PrepareStack();

	for(int i= 0; i<argcount; ++i)
	{
		pArgData = ArgData.at(i); pParData = varptr->at(i);
		partype = pParData->GetType();

		switch(partype)
		{
		case TypeBool:
			static_cast<BoolData*>(pParData)->GetData() = 
				static_cast<BoolData*>(pArgData)->GetData();
			break;

		case TypeIntg:
			static_cast<IntgData*>(pParData)->GetData() = 
				static_cast<IntgData*>(pArgData)->GetData();
			
			//static_cast<IntgData*>(pParData)->GetData() = 
			//	dynamic_cast<ExpIntg*>(argvec[i])->Eval();
			break;

		case TypeReal:
			static_cast<RealData*>(pParData)->GetData() = 
				static_cast<RealData*>(pArgData)->GetData();
			break;

		case TypeCplx:
			static_cast<CplxData*>(pParData)->GetData() = 
				static_cast<CplxData*>(pArgData)->GetData();
			break;

		case TypeVctr:
			static_cast<VctrData*>(pParData)->GetData() = 
				static_cast<VctrData*>(pArgData)->GetData();
			break;

		case TypeStrn:
			static_cast<StrnData*>(pParData)->GetData() = 
				static_cast<StrnData*>(pArgData)->GetData();
			break;

		case TypeUser:

			*dynamic_cast<UserData*>(pParData)->GetPtr() = 
				*dynamic_cast<UserData*>(pArgData)->GetPtr();
			break;

		case TypeAptr:
			Step3Aptr(pParData, pArgData); break;

		case TypeArray:
			Step3Array(pParData, pArgData); break;
		
		case TypePtr:
			Step3Ptr(pParData, pArgData); break;
		
		default:
			throw UnicodeException(_T("This message never show"));
		}
	}
}

typedef int  (*VO_FUN_SIZE)    (Array*, int&, int&, int&, int&, int&);
typedef void  (*VO_FUN_RESIZE)  (Array*, int, int, int, int, int);
typedef void  (*VO_FUN_MAKEUNIT)  (Array*, int);

typedef bool    (*B_FUN_S)   (std::wstring&);

typedef integer (*I_FUN_I)   (integer);
typedef integer (*I_FUN_II)  (integer, integer);
typedef integer (*I_FUN_VO)  (void);

typedef realnum (*R_FUN_I)   (integer);
typedef realnum (*R_FUN_II)   (integer, integer);

typedef realnum (*R_FUN_C)   (const Complex&);
typedef realnum (*R_FUN_R)   (realnum);
typedef realnum (*R_FUN_MR)   (realnum&);
typedef realnum (*R_FUN_RR)  (realnum, realnum);
typedef realnum (*R_FUN_RRR)  (realnum, realnum, realnum);
typedef realnum (*R_FUN_RRRR)  (realnum, realnum, realnum, realnum);
typedef realnum (*R_FUN_V)   (const Vector&);
typedef Complex (*C_FUN_C)   (const Complex&);
typedef Complex (*C_FUN_CC)   (const Complex&, const Complex&);
typedef Complex (*C_FUN_CCCC)   (const Complex&, const Complex&, const Complex&, const Complex&);
typedef Complex (*C_FUN_RR)  (realnum, realnum);
typedef Vector  (*V_FUN_V)   (const Vector&);
typedef Vector	(*V_FUN_RRR) (realnum, realnum, realnum);

typedef void (*VO_FUN_MR_MR) (Mat_IO_DP&, Mat_IO_DP&);

enum FunType {	FunNull,
				
				Fun_VO_MAKEUNIT,

				Fun_VO_SIZE,
				Fun_VO_RESIZE,

				Fun_B_S,

				Fun_I_I,
				Fun_I_II,
				Fun_I_VO,

				Fun_R_I,
				Fun_R_II,

				Fun_R_C, 
				Fun_R_R, 
				Fun_R_MR,
				Fun_RRMRMRMRMR,
				Fun_R_RR, 
				Fun_R_RRR, 
				Fun_R_RRRR, 
				Fun_R_V,
				Fun_C_C, 
				Fun_C_CC, 
				Fun_C_CCCC, 
				Fun_C_RR,
				Fun_V_V, 
				Fun_V_RRR,

				Fun_VO_MR_MR

				};

typedef union
{		
		VO_FUN_MAKEUNIT vo_fun_makeunit;

		VO_FUN_SIZE   vo_fun_size;
		
		VO_FUN_RESIZE vo_fun_resize;

		B_FUN_S		b_fun_s;

		I_FUN_I		i_fun_i;
		I_FUN_II	i_fun_ii;
		I_FUN_VO	i_fun_vo;
		
		R_FUN_I		r_fun_i;
		R_FUN_II	r_fun_ii;

		R_FUN_C		r_fun_c;
		R_FUN_R		r_fun_r;
		R_FUN_MR	r_fun_mr;
		R_FUN_RR	r_fun_rr;
		R_FUN_RRR	r_fun_rrr;
		R_FUN_RRRR	r_fun_rrrr;
		R_FUN_V		r_fun_v;
		C_FUN_C		c_fun_c;
		C_FUN_CC	c_fun_cc;
		C_FUN_CCCC	c_fun_cccc;
		C_FUN_RR	c_fun_rr;
		V_FUN_V		v_fun_v;
		V_FUN_RRR	v_fun_rrr;

		VO_FUN_MR_MR vo_fun_mr_mr;
} FunPtr;


typedef struct { FunType type; FunPtr ptr; } FunSig;

typedef struct { FunSig fsig; wchar_t fn[BUF_LENGTH]; } FunName;

inline FunSig MakeFunSig(FunType t, FunPtr p)
{
	FunSig fun;
	fun.type = t; fun.ptr = p;
	return fun;
}

class BaseFunc
{
protected:
	FunSig fsig;
	std::wstring fname;

public:
	BaseFunc(FunSig fs, const std::wstring& name):
	  fsig(fs), fname(name) {}

	virtual ~BaseFunc() {}
};

class FuncList
{
protected:
	typedef stdext::hash_map<std::wstring, FunSig> FMap;
	typedef std::pair<std::wstring, FunSig> Pair;
	typedef FMap::iterator Itr;
	FMap funclist;

public:

	void insert(const std::wstring& name, const FunSig& fs);

	void find(FunName& fn);
	void clear();
};

class RFunR: public BaseFunc, public ExpReal
{
protected:
	ExpReal* arg; realnum rtn;

	void cache()
	{
		if(arg->bLiteral) 
		{
			this->Eval(); bLiteral=true;
		}
	}
public:
	RFunR(const FunName& fname, ExpReal* axpr):
	  BaseFunc(fname.fsig, fname.fn), arg(axpr) { cache(); }
	
	virtual double Eval()
	{
		if(bLiteral) return rtn;

		rtn = fsig.ptr.r_fun_r(arg->Eval());	
		errno_t err_; _get_errno(&err_);
		
		if(err_ != 0)
		{
			_set_errno(0); 	throw ExceptionMath();
		}

		return rtn;
	}

	virtual void GetVariables(DataSet& dataset)
	{
		if(bLiteral) return;

		arg->GetVariables(dataset);
	}

	virtual std::wstring Display()
	{
		std::wostringstream os;
		os<<fname<<'('<<arg->Display()<<')';
		return os.str();
	}

	virtual ~RFunR() {}
};

class RFunI: public BaseFunc, public ExpReal
{
protected:
	ExpIntg* arg; realnum rtn;

	void cache()
	{
		if(arg->bLiteral)
		{
			this->Eval(); bLiteral = true;
		}
	}
	
public:
	RFunI(const FunName& fname, ExpIntg* axpr):
	  BaseFunc(fname.fsig, fname.fn), arg(axpr) { cache(); }
	
	virtual realnum Eval()
	{
		if(bLiteral) return rtn;

		rtn = fsig.ptr.r_fun_i(arg->Eval());

		errno_t err_; _get_errno(&err_);
		
		if(err_ != 0)
		{
			_set_errno(0); throw ExceptionMath();
		}

		return rtn;
	}

	virtual void GetVariables(DataSet& dataset)
	{
		if(bLiteral) return;

		arg->GetVariables(dataset);
	}

	virtual std::wstring Display()
	{
		std::wostringstream os;
		os<<fname<<'('<<arg->Display()<<')';
		return os.str();
	}

	virtual ~RFunI() {}
};

class CFunC: public BaseFunc, public ExpCplx
{
protected:
	ExpCplx *arg; Complex rtn;
	
	void cache()
	{
		if(arg->bLiteral)
		{
			rtn = this->Eval();  this->bLiteral = true;
		}
	}
public:
	CFunC(const FunName& fname, ExpCplx* xpr):
	  BaseFunc(fname.fsig, fname.fn), arg(xpr) 
	  { cache(); }
	
	virtual Complex Eval()
	{
		if(this->bLiteral) return rtn;

		return fsig.ptr.c_fun_c(arg->Eval());
	}
	virtual std::wstring Display()
	{
		std::wostringstream os;
		os<<fname<<'('<<arg->Display()<<')';
		return os.str();
	}

	virtual ~CFunC() {}
};

class IFunIntg: public BaseFunc, public ExpIntg
{
public:
	IFunIntg(const FunName& fname): 
	  BaseFunc(fname.fsig, fname.fn) { }
	
	  virtual integer Eval()
	  {
		  return fsig.ptr.i_fun_vo();
	  }
	
	virtual std::wstring Display()
	{
		std::wostringstream os;
		os<<fname<<_T("()");
		return os.str();
	}

	virtual ~IFunIntg() {}
};

class IFunII: public BaseFunc, public ExpIntg
{
protected:
	ExpIntg *pXpr1, *pXpr2;
	integer rtn;
	void cache()
	{
		if(pXpr1->bLiteral && pXpr2->bLiteral && fname!=L"irandom")
		{
			rtn = Eval(); bLiteral = true;
		}
	}

public:
	IFunII(const FunName& fname, ExpIntg* xpr1, ExpIntg* xpr2): 
	  BaseFunc(fname.fsig, fname.fn), pXpr1(xpr1), pXpr2(xpr2) { cache(); }
	
	  virtual integer Eval()
	  {
		  if(bLiteral) return rtn;

		  integer arg1 = pXpr1->Eval();
		  integer arg2 = pXpr2->Eval();

		  return fsig.ptr.i_fun_ii(arg1, arg2);
	  }
	
	virtual std::wstring Display()
	{
		std::wostringstream os;
		os<<fname<<_T("()");
		return os.str();
	}

	virtual ~IFunII() {}
};

class VOFunSize: public BaseFunc, public ExpIntg
{
protected:
	Array* ptr; 
	IntgData *psize1, *psize2, *psize3, *psize4, *psize5;
public:
	VOFunSize(const FunName& fname, Array* aptr,
		IntgData* pid1=NULL, IntgData* pid2=NULL, 
		IntgData* pid3=NULL, IntgData* pid4=NULL, IntgData* pid5=NULL):
		BaseFunc(fname.fsig, fname.fn), ptr(aptr), 
		  psize1(pid1), psize2(pid2), psize3(pid3),
		  psize4(pid4), psize5(pid5) { }
	
	 virtual integer Eval()
	 {
		 int size1, size2, size3, size4, size5, dim;

		 dim = fsig.ptr.vo_fun_size(ptr, size1, size2, size3, size4, size5);
		 
		 if(psize1) psize1->GetData() = size1;
		 if(psize2) psize2->GetData() = size2;
		 if(psize3) psize3->GetData() = size3;
		 if(psize4) psize4->GetData() = size4;
		 if(psize5) psize5->GetData() = size5;

		 return dim;
	 }
	
	virtual std::wstring Display()
	{
		std::wostringstream os;
		os<<fname<<_T("()");
		return os.str();
	}

	virtual ~VOFunSize() {}
};

template<typename T, typename vec>
class VOFunSizePtr5: public ExpIntg
{
protected:
	AptrData<vec>* ptr; 
	IntgData *psize1, *psize2, *psize3, *psize4, *psize5;
public:
	VOFunSizePtr5(AptrData<vec>* aptr,
		IntgData* pid1, IntgData* pid2, IntgData* pid3, IntgData* pid4, IntgData* pid5):
		ptr(aptr), psize1(pid1), psize2(pid2), psize3(pid3),
		psize4(pid4), psize5(pid5) { }
	
	 virtual integer Eval()
	 {
		 int size1, size2, size3, size4, size5;
		 vec* v = *ptr->GetPtr();
		 if(!v) throw UnicodeException(_T("Array Pointer NOT Yet Initialized"));
		 GetVecSize<T>(size1, size2, size3, size4, size5, *v);
		 
		 psize1->GetData() = size1;
		 psize2->GetData() = size2;
		 psize3->GetData() = size3;
		 psize4->GetData() = size4;
		 psize5->GetData() = size5;

		 return integer(5);
	 }
	
	 virtual ~VOFunSizePtr5() {}
};

template<typename T, typename vec>
class VOFunSizePtr4: public ExpIntg
{
protected:
	AptrData<vec>* ptr; 
	IntgData *psize1, *psize2, *psize3, *psize4;
public:
	VOFunSizePtr4(AptrData<vec>* aptr,
		IntgData* pid1, IntgData* pid2, IntgData* pid3, IntgData* pid4):
		ptr(aptr), psize1(pid1), psize2(pid2), psize3(pid3), psize4(pid4) { }
	
	 virtual integer Eval()
	 {
		 int size1, size2, size3, size4;
		 vec* v = *ptr->GetPtr();
		 if(!v) throw UnicodeException(_T("Array Pointer NOT Yet Initialized"));
		 GetVecSize<T>(size1, size2, size3, size4, *v);
		 
		 psize1->GetData() = size1;
		 psize2->GetData() = size2;
		 psize3->GetData() = size3;
		 psize4->GetData() = size4;
		 
		 return integer(4);
	 }
	
	 virtual ~VOFunSizePtr4() {}
};

template<typename T, typename vec>
class VOFunSizePtr3: public ExpIntg
{
protected:
	AptrData<vec>* ptr; 
	IntgData *psize1, *psize2, *psize3;
public:
	VOFunSizePtr3(AptrData<vec>* aptr,
		IntgData* pid1, IntgData* pid2, IntgData* pid3):
		ptr(aptr), psize1(pid1), psize2(pid2), psize3(pid3) { }
	
	 virtual integer Eval()
	 {
		 int size1, size2, size3;
		 vec* v = *ptr->GetPtr();
		 if(!v) throw UnicodeException(_T("Array Pointer NOT Yet Initialized"));
		 GetVecSize<T>(size1, size2, size3, *v);
		 
		 psize1->GetData() = size1;
		 psize2->GetData() = size2;
		 psize3->GetData() = size3;
		 		 
		 return integer(3);
	 }
	
	 virtual ~VOFunSizePtr3() {}
};

template<typename T, typename vec>
class VOFunSizePtr2: public ExpIntg
{
protected:
	AptrData<vec>* ptr; 
	IntgData *psize1, *psize2;
public:
	VOFunSizePtr2(AptrData<vec>* aptr,
		IntgData* pid1, IntgData* pid2):
		ptr(aptr), psize1(pid1), psize2(pid2){ }
	
	 virtual integer Eval()
	 {
		 int size1, size2;
		 vec* v = *ptr->GetPtr();
		 if(!v) throw UnicodeException(_T("Array Pointer NOT Yet Initialized"));
		 GetVecSize<T>(size1, size2, *v);
		 
		 psize1->GetData() = size1;
		 psize2->GetData() = size2;
		 		 		 
		 return integer(2);
	 }
	
	 virtual ~VOFunSizePtr2() {}
};

template<typename T, typename vec>
class VOFunSizePtr1: public ExpIntg
{
protected:
	AptrData<vec>* ptr; 
	IntgData *psize1;
public:
	VOFunSizePtr1(AptrData<vec>* aptr,
		IntgData* pid1): ptr(aptr), psize1(pid1){ }
	
	 virtual integer Eval()
	 {
		 int size1;
		 vec* v = *ptr->GetPtr();
		 GetVecSize<T>(size1, *v);
		 
		 psize1->GetData() = size1;
		 		 		 		 
		 return integer(1);
	 }
	
	 virtual ~VOFunSizePtr1() {}
};

class VOFunSizePtr: public ExpIntg
{
protected:
	integer dimension;
public:
	VOFunSizePtr(int dim): dimension(dim) {}
	
	 virtual integer Eval()
	 {
		 return dimension;
	 }
	
	 virtual ~VOFunSizePtr() {}
};

class VOFunResize: public BaseFunc, public Statement
{
protected:
	Array* ptr; 
	ExpIntg *psize[MAX_DIMENSION];

public:
	VOFunResize(const FunName& fname, Array* aptr,
		ExpIntg* pid0=NULL, ExpIntg* pid1=NULL, 
		ExpIntg* pid2=NULL, ExpIntg* pid3=NULL,
		ExpIntg* pid4=NULL):
		BaseFunc(fname.fsig, fname.fn), ptr(aptr) 
		{
			psize[0] = pid0; psize[1] = pid1; psize[2] = pid2;
			psize[3] = pid3; psize[4] = pid4;
		}
	
	void SetExprSize(int index, ExpIntg* ps) { psize[index] = ps; }

	virtual void Run()
	{
		int size[MAX_DIMENSION];
		 
		for(int i=0; i<MAX_DIMENSION; ++i)
		{
			if(psize[i]) size[i] = int(psize[i]->Eval());
			else size[i] = 0;
		}

		fsig.ptr.vo_fun_resize(ptr, size[0], size[1], size[2], size[3], size[4]);
 	 }
	
	virtual std::wstring Display()
	{
		std::wostringstream os;
		os<<fname<<_T("()");
		return os.str();
	}

	virtual ~VOFunResize() {}
};

template<typename T>
class VOFunUdtResize: public BaseFunc, public Statement
{
protected:
	VarUser<T>* ptr; 
	ExpIntg *psize[MAX_DIMENSION];

public:
	VOFunUdtResize(const FunName& fname, VarUser<T>* aptr,
		ExpIntg* pid0=NULL, ExpIntg* pid1=NULL, 
		ExpIntg* pid2=NULL, ExpIntg* pid3=NULL,
		ExpIntg* pid4=NULL):
		BaseFunc(fname.fsig, fname.fn), ptr(aptr) 
		{
			psize[0] = pid0; psize[1] = pid1; psize[2] = pid2;
			psize[3] = pid3; psize[4] = pid4;
		}
	
	void SetExprSize(int index, ExpIntg* ps) { psize[index] = ps; }

	virtual void Run()
	{
		int size[MAX_DIMENSION];
		 
		for(int i=0; i<MAX_DIMENSION; ++i)
		{
			if(psize[i]) size[i] = int(psize[i]->Eval());
			else size[i] = 0;
		}
		
		Array* ary = &ptr->GetData();
		fsig.ptr.vo_fun_resize(ary, size[0], size[1], size[2], size[3], size[4]);
 	 }
	
	virtual std::wstring Display()
	{
		std::wostringstream os;
		os<<fname<<_T("()");
		return os.str();
	}

	virtual ~VOFunUdtResize() {}
};

class VOFunMakeUnit: public BaseFunc, public Statement
{
protected:
	Array* ptr; ExpIntg *psize;

public:
	VOFunMakeUnit(const FunName& fname, Array* aptr, ExpIntg* pid):
		BaseFunc(fname.fsig, fname.fn), ptr(aptr), psize(pid) { }
	
	virtual void Run()
	{
		int size=0;
		
		if(psize) size = int(psize->Eval());
				
		fsig.ptr.vo_fun_makeunit(ptr, size);
	}
	
	virtual std::wstring Display()
	{
		std::wostringstream os;
		os<<fname<<_T("()");
		return os.str();
	}

	virtual ~VOFunMakeUnit() {}
};


// gaussj
class VOFunMRMR: public BaseFunc, public Expression<voidtype>
{
protected:
	Array2D<realnum> *aptr1, *aptr2; 
public:
	VOFunMRMR(const FunName& fname, Array2D<realnum> *ptr1, Array2D<realnum> *ptr2):
	  aptr1(ptr1), aptr2(ptr2), BaseFunc(fname.fsig, fname.fn) { }
	
	 virtual voidtype Eval()
	 {
		 Matrix<DP> m1 = *aptr1->GetTopMatrix();
		 Matrix<DP> m2 = *aptr2->GetTopMatrix();

		 fsig.ptr.vo_fun_mr_mr(m1, m2);
		 
		 return voidtype();
	}
	
	virtual ~VOFunMRMR() {}
};

class CFunCC: public BaseFunc, public ExpCplx
{
protected:
	ExpCplx *arg1, *arg2;  Complex rtn;

	void cache()
	{
		if(arg1->bLiteral && arg2->bLiteral)
		{
			rtn = this->Eval(); this->bLiteral = true;
		}
	}
public:
	CFunCC(const FunName& fname, 
		ExpCplx* xpr1, ExpCplx* xpr2):
	  BaseFunc(fname.fsig, fname.fn), arg1(xpr1), arg2(xpr2)  
	  { cache(); }
	
	virtual Complex Eval()
	{
		if(this->bLiteral) return rtn;

		Complex d1 = arg1->Eval();
		Complex d2 = arg2->Eval();
		
		if(fsig.ptr.c_fun_cc==powc)
		{
			d2 = arg2->Eval();
			d1 = arg1->Eval();
		}
		else
		{
			d1 = arg1->Eval();
			d2 = arg2->Eval();
		}

		return fsig.ptr.c_fun_cc(d1, d2);
	}

	virtual std::wstring Display()
	{
		std::wostringstream os;
		if(fname == _T("powc"))
			os<<_T('(')<<arg1->Display()<<_T('^')<<arg2->Display()<<_T(')');
		else
			os<<fname<<_T('(')<<arg1->Display()<<_T(", ")<<arg2->Display()<<_T(')');
		return os.str();
	}

	virtual ~CFunCC() {}
};


class RFunC: public BaseFunc, public ExpReal
{
protected:
	ExpCplx *arg; realnum rtn;

	void cache()
	{
		if(arg->bLiteral)
		{
			rtn = this->Eval(); this->bLiteral = true;
		}
	}
public:
	RFunC(const FunName& fname, ExpCplx* xpr):
	  BaseFunc(fname.fsig, fname.fn), arg(xpr) 
	  { cache(); }
	
	virtual double Eval()
	{
		if(this->bLiteral) return rtn;

		return fsig.ptr.r_fun_c(arg->Eval());
	}

	virtual std::wstring Display()
	{
		std::wostringstream os;
		if(fname == _T("absc"))
			os<<_T('|')<<arg->Display()<<_T('|');
		else if(fname == _T(".r") || fname == _T(".i"))
			os<<arg->Display()<<fname;
		else
			os<<fname<<_T('(')<<arg->Display()<<_T(')');

		return os.str();
	}

	virtual ~RFunC() {}
};

class RFunV: public BaseFunc, public ExpReal
{
protected:
	ExpVctr *arg; realnum rtn;

	void cache()
	{
		if(arg->bLiteral)
		{
			rtn = this->Eval(); this->bLiteral = true;
		}
	}
public:
	RFunV(const FunName& fname, ExpVctr* xpr):
	  BaseFunc(fname.fsig, fname.fn), arg(xpr) { cache(); }
	
	virtual double Eval()
	{
		if(this->bLiteral) return rtn;

		return fsig.ptr.r_fun_v(arg->Eval());
	}

	virtual std::wstring Display()
	{
		std::wostringstream os;
		if(fname == _T("absv"))
			os<<_T('|')<<arg->Display()<<_T('|');
		else if(fname == _T(".x") || fname == _T(".y") || fname == _T(".z"))
			os<<arg->Display()<<fname;
		else
			os<<fname<<_T('(')<<arg->Display()<<_T(')');
		return os.str();
	}

	virtual ~RFunV() {}
};

class RFunRR: public BaseFunc, public ExpReal
{
protected:
	ExpReal *arg1, *arg2; realnum rtn;
	void cache()
	{
		if(arg1->bLiteral && arg2->bLiteral && fname!=L"drandom")
		{
			this->Eval(); bLiteral = true;
		}
	}
public:
	RFunRR(const FunName& fname,
		ExpReal* xpr1, ExpReal* xpr2):
	    BaseFunc(fname.fsig, fname.fn), arg1(xpr1), arg2(xpr2) { cache(); }
	
	virtual double Eval()
	{
		if(bLiteral) return rtn;

		realnum d1, d2;

		if(fsig.ptr.r_fun_rr == pow)
		{
			d2 = arg2->Eval(); d1 = arg1->Eval();
		}
		else
		{
			d1 = arg1->Eval(); d2 = arg2->Eval();
		}

		rtn = fsig.ptr.r_fun_rr(d1, d2);
		
		errno_t err_; _get_errno(&err_);
		if(err_ != 0)
		{
			_set_errno(0); throw ExceptionMath();
		}

		return rtn;
	}

	virtual void GetVariables(DataSet& dataset)
	{
		if(bLiteral) return;

		arg1->GetVariables(dataset);
		arg2->GetVariables(dataset);
	}

	virtual std::wstring Display()
	{
		std::wostringstream os;
		if(fname == _T("pow"))
			os<<_T('(')<<arg1->Display()<<_T('^')<<arg2->Display()<<_T(')');
		else
			os<<fname<<_T('(')<<arg1->Display()<<_T(", ")<<arg2->Display()<<_T(')');
		return os.str();
	}

	virtual ~RFunRR() {}
};

class RFunII: public BaseFunc, public ExpReal
{
protected:
	ExpIntg *arg1, *arg2; realnum rtn;

	void cache()
	{
		if(arg1->bLiteral && arg2->bLiteral)
		{
			this->Eval(); bLiteral=true;
		}
	}
public:
	RFunII(const FunName& fname,
		ExpIntg* xpr1, ExpIntg* xpr2):
	    BaseFunc(fname.fsig, fname.fn), arg1(xpr1), arg2(xpr2) { cache(); }
	
	virtual double Eval()
	{
		if(bLiteral) return rtn;
		
		integer i1 = arg1->Eval();
		integer i2 = arg2->Eval();

		rtn = fsig.ptr.r_fun_ii(i1, i2);

		errno_t err_; _get_errno(&err_);

		if(err_ != 0)
		{
			_set_errno(0); 	throw ExceptionMath();
		}

		return rtn;
	}

	virtual void GetVariables(DataSet& dataset)
	{
		if(bLiteral) return;

		arg1->GetVariables(dataset);
		arg2->GetVariables(dataset);
	}

	virtual std::wstring Display()
	{
		std::wostringstream os;
		os<<fname<<'('<<arg1->Display()<<_T(", ")<<arg2->Display()<<')';
		return os.str();
	}

	virtual ~RFunII() {}
};

class RFunRRR: public BaseFunc, public ExpReal
{
protected:
	ExpReal *arg1, *arg2, *arg3; realnum rtn;

	void cache()
	{
		if(arg1->bLiteral && arg2->bLiteral && arg3->bLiteral)
		{
			this->Eval(); this->bLiteral = true;
		}
	}
public:
	RFunRRR(const FunName& fname,
		ExpReal* xpr1, ExpReal* xpr2, ExpReal* xpr3):
	    BaseFunc(fname.fsig, fname.fn), arg1(xpr1), arg2(xpr2), arg3(xpr3) 
		{ cache(); }
	
	virtual double Eval()
	{
		if(this->bLiteral) return rtn;

		realnum d1 = arg1->Eval();
		realnum d2 = arg2->Eval();
		realnum d3 = arg3->Eval();
		
		rtn = fsig.ptr.r_fun_rrr(d1, d2, d3);

		errno_t err_; _get_errno(&err_);
		if(err_ != 0)
		{
			_set_errno(0);
			throw ExceptionMath();
		}

		return rtn;
	}
	
	virtual void GetVariables(DataSet& dataset)
	{
		if(bLiteral) return;

		arg1->GetVariables(dataset);
		arg2->GetVariables(dataset);
		arg3->GetVariables(dataset);
	}

	virtual std::wstring Display()
	{
		std::wostringstream os;
		os<<fname<<_T('(')<<arg1->Display()<<_T(", ")<<
			arg2->Display()<<_T(", ")<<arg3->Display()<<_T(')');
		return os.str();
	}

	virtual ~RFunRRR() {}
};

class RFunRRRR: public BaseFunc, public ExpReal
{
protected:
	ExpReal *arg1, *arg2, *arg3, *arg4;  realnum rtn;

	void cache()
	{
		if(arg1->bLiteral && arg2->bLiteral && arg3->bLiteral && arg4->bLiteral)
		{
			this->Eval(); this->bLiteral = true;
		}
	}
public:
	RFunRRRR(const FunName& fname,
		ExpReal* xpr1, ExpReal* xpr2,
		ExpReal* xpr3, ExpReal* xpr4):
	    BaseFunc(fname.fsig, fname.fn), arg1(xpr1), arg2(xpr2),
			arg3(xpr3), arg4(xpr4) { cache(); }
	
	virtual double Eval()
	{
		if(this->bLiteral) return rtn;

		realnum d1 = arg1->Eval();
		realnum d2 = arg2->Eval();
		realnum d3 = arg3->Eval();
		realnum d4 = arg4->Eval();

		rtn = fsig.ptr.r_fun_rrrr(d1, d2, d3, d4);
			
		errno_t err_; _get_errno(&err_);
		if(err_ != 0)
		{
			_set_errno(0); throw ExceptionMath();
		}

		return rtn;
	}
	virtual std::wstring Display()
	{
		std::wostringstream os;
		os<<fname<<_T('(')<<arg1->Display()<<_T(", ")<<
			arg2->Display()<<_T(", ")<<arg3->Display()
			<<_T(", ")<<arg4->Display()<<_T(')');
		return os.str();
	}

	virtual ~RFunRRRR() {}
};

class CFunCCCC: public BaseFunc, public ExpCplx
{
protected:
	ExpCplx *arg1, *arg2, *arg3, *arg4;
	Complex rtn;

	void cache()
	{
		if(arg1->bLiteral && arg2->bLiteral && arg3->bLiteral && arg4->bLiteral)
		{
			rtn = this->Eval(); this->bLiteral = true;
		}
	}
public:
	CFunCCCC(const FunName& fname,
		ExpCplx* xpr1, ExpCplx* xpr2,
		ExpCplx* xpr3, ExpCplx* xpr4):
	    BaseFunc(fname.fsig, fname.fn), arg1(xpr1), arg2(xpr2),
			arg3(xpr3), arg4(xpr4) { cache(); }
	
	virtual Complex Eval()
	{
		if(this->bLiteral) return rtn;

		Complex d1 = arg1->Eval();
		Complex d2 = arg2->Eval();
		Complex d3 = arg3->Eval();
		Complex d4 = arg4->Eval();

		return fsig.ptr.c_fun_cccc(d1, d2, d3, d4);
	}
	virtual std::wstring Display()
	{
		std::wostringstream os;
		os<<fname<<_T('(')<<arg1->Display()<<_T(", ")<<
			arg2->Display()<<_T(", ")<<arg3->Display()
			<<_T(", ")<<arg4->Display()<<_T(')');
		return os.str();
	}

	virtual ~CFunCCCC() {}
};



class CFunRR: public BaseFunc, public ExpCplx
{
protected:
	ExpReal *arg1, *arg2; Complex rtn;

	void cache()
	{
		if(arg1->bLiteral && arg2->bLiteral)
		{
			rtn = this->Eval(); this->bLiteral = true;
		}
	}
	
public:
	CFunRR(const FunName& fname,
		ExpReal* xpr1, ExpReal* xpr2):
	    BaseFunc(fname.fsig, fname.fn), arg1(xpr1), arg2(xpr2) 
		{	cache();  }
	
	virtual Complex Eval()
	{
		if(this->bLiteral) return rtn;

		realnum d1 = arg1->Eval();
		realnum d2 = arg2->Eval();

		return fsig.ptr.c_fun_rr(d1, d2);
	}
	virtual std::wstring Display()
	{
		std::wostringstream os;
		os<<fname<<'('<<arg1->Display()<<_T(", ")<<arg2->Display()<<')';
		return os.str();
	}

	virtual ~CFunRR() {}
};

class VFunV: public BaseFunc, public ExpVctr
{
protected:
	ExpVctr *arg; Vector rtn;

	void cache()
	{
		if(arg->bLiteral)
		{
			rtn = this->Eval(); this->bLiteral = true;
		}
	}
public:
	VFunV(const FunName& fname, ExpVctr* xpr):
	  BaseFunc(fname.fsig, fname.fn), arg(xpr) { cache(); }
	
	virtual Vector Eval()
	{
		if(this->bLiteral) return rtn;
		
		return fsig.ptr.v_fun_v(arg->Eval());
	}
	virtual std::wstring Display()
	{
		std::wostringstream os;
		os<<fname<<_T('(')<<arg->Display()<<_T(')');
		return os.str();
	}

	virtual ~VFunV() {}
};

class VFunRRR: public BaseFunc, public ExpVctr
{
protected:
	ExpReal *arg1, *arg2, *arg3; Vector rtn;

	void cache()
	{
		if(arg1->bLiteral && arg2->bLiteral && arg3->bLiteral)
		{
			rtn = this->Eval(); this->bLiteral = true;
		}
	}
public:
	VFunRRR(const FunName& fname,
		ExpReal* xpr1, 
		ExpReal* xpr2, ExpReal* xpr3):
	    arg1(xpr1), arg2(xpr2), arg3(xpr3),
			BaseFunc(fname.fsig, fname.fn) { cache(); }
	
	virtual Vector Eval()
	{
		if(this->bLiteral) return rtn;

		realnum d1 = arg1->Eval();
		realnum d2 = arg2->Eval();
		realnum d3 = arg3->Eval();

		return fsig.ptr.v_fun_rrr(d1, d2, d3);
	}
	virtual std::wstring Display()
	{
		std::wostringstream os;
		os<<fname<<_T('(')<<arg1->Display()<<_T(", ")
			<<arg2->Display()<<_T(", ")<<arg3->Display()<<_T(')');
		return os.str();
	}

	virtual ~VFunRRR() {}
};

template<typename U>
class StmtSwap: public Statement
{
protected:
	U *pLeft, *pRight;
public:
	StmtSwap(U *left, U *right): pLeft(left), pRight(right) { }
	virtual void Run()
	{
		pLeft->Swap(pRight);
	}

	virtual ~StmtSwap() {}
};

template<typename T>
class StmtSwapLHS: public Statement
{
protected:
	LeftHandSide<T> *pLeft, *pRight;
public:
	StmtSwapLHS(LeftHandSide<T> *left, LeftHandSide<T> *right)
		: pLeft(left), pRight(right) { }

	virtual void Run()
	{
		T value = pLeft->GetData();
		pLeft->GetData() = pRight->GetData();
		pRight->GetData() = value;
	}

	virtual ~StmtSwapLHS() {}
};

template<typename T>
class StmtSwapArrayLHS: public Statement
{
protected:
	LeftHandSideArray<T> *pLeft, *pRight;
public:
	StmtSwapArrayLHS(LeftHandSideArray<T> *left, LeftHandSideArray<T> *right)
		: pLeft(left), pRight(right) { }

	virtual void Run()
	{
		T value = pLeft->GetData();
		pLeft->GetData() = pRight->GetData();
		pRight->GetData() = value;
	}

	virtual ~StmtSwapArrayLHS() {}
};

template<typename T, typename U>
class StmtSwapExpr: public Statement
{
protected:
	Expression<T> *lhs, *rhs;

public:
	StmtSwapExpr(Expression<T> *xpr1, Expression<T>* xpr2):
	  lhs(xpr1), rhs(xpr2) { }

	virtual void Run()
	{
		Variable<T>	   *valhs, *varhs;
		VarAry1D<T>    *a1lhs, *a1rhs;
		VarAry2D<T>    *a2lhs, *a2rhs;
		VarAry3D<T>    *a3lhs, *a3rhs;
		VarAry4D<T>    *a4lhs, *a4rhs;
		VarAry5D<T>    *a5lhs, *a5rhs;
		VarUser<T>	   *vlhs,  *vrhs;	
		
		valhs = dynamic_cast<Variable<T>*>(lhs);
		varhs = dynamic_cast<Variable<T>*>(rhs);
		
		a1lhs = dynamic_cast<VarAry1D<T>*>(lhs);
		a1rhs = dynamic_cast<VarAry1D<T>*>(rhs);
		a2lhs = dynamic_cast<VarAry2D<T>*>(lhs);
		a2rhs = dynamic_cast<VarAry2D<T>*>(rhs);
		a3lhs = dynamic_cast<VarAry3D<T>*>(lhs);
		a3rhs = dynamic_cast<VarAry3D<T>*>(rhs);
		a4lhs = dynamic_cast<VarAry4D<T>*>(lhs);
		a4rhs = dynamic_cast<VarAry4D<T>*>(rhs);
		a5lhs = dynamic_cast<VarAry5D<T>*>(lhs);
		a5rhs = dynamic_cast<VarAry5D<T>*>(rhs);
		
		vlhs = dynamic_cast<VarUser<T>*>(lhs);
		vrhs = dynamic_cast<VarUser<T>*>(rhs);

		T *left=NULL, *right=NULL;

		if(valhs) left  = &valhs->GetData();
		if(varhs) right = &varhs->GetData();
		
		if(a1lhs) left  = &a1lhs->GetData();
		if(a1rhs) right = &a1rhs->GetData();
		
		if(a2lhs) left  = &a2lhs->GetData();
		if(a2rhs) right = &a2rhs->GetData();
		
		if(a3lhs) left  = &a3lhs->GetData();
		if(a3rhs) right = &a3rhs->GetData();
		
		if(a4lhs) left  = &a4lhs->GetData();
		if(a4rhs) right = &a4rhs->GetData();

		if(a5lhs) left  = &a5lhs->GetData();
		if(a5rhs) right = &a5rhs->GetData();

		if(vlhs) left  = &vlhs->GetData();
		if(vrhs) right = &vrhs->GetData();

		if(left && right)
		{
			T value = *left;
			*left = *right;
			*right = value;
		}
		else
			throw UnicodeException(_T("Invalid swapping..."));
	}

	virtual ~StmtSwapExpr() {}
};



class TransTokens
{
	typedef std::map<std::wstring, std::wstring> TokenMap;
	typedef TokenMap::iterator TokenMapItr;
	typedef std::pair<std::wstring, std::wstring> TokenPair;
	
	TokenMap tokenmap;

public:
	void InsertToken(const wchar_t* tname, UINT msgid);
	void InsertToken(const wchar_t* tname, const wchar_t* msgstr);
	const wchar_t* GetToken(const wchar_t* tname);
};

template<typename T, typename U>
class CallExpr: public Expression<T>
{
protected:
	std::wstring vrid;
	Statement* pStmt;
	Assignment<T, U>* pAsgn;
	U* data;
	
public:

	CallExpr(VarType vt, Assignment<T, U>* asmt, Statement* stmt): 
	  vrid(vt.bufr), pAsgn(asmt), pStmt(stmt) 
	  { 
		  data = dynamic_cast<U*>(vt.dPtr); 
		  if(data==NULL) 
			  throw UnicodeException(_T("Type mismatch in the functional expression."));
		  
		  DataSet dataset;
		  pAsgn->GetExpression()->GetVariables(dataset);
	  }

	virtual T Eval() 
	{
		if(pStmt) pStmt->Run();
		return data->GetData() = pAsgn->GetExpression()->Eval();
	}

	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		if(typeid(T)!=typeid(voidtype))
			ostr<<vrid;
		return ostr.str();
	}
	
	virtual ~CallExpr() { }
};


template<typename T>
class CallExpression: public Expression<T>
{
protected:
	Statement* pStmt; 
	LeftHandSide<T>* pLHS;

public:
	ScanFile scanfile;

	CallExpression(LeftHandSide<T>* lhs, Statement* stmt): 
	  pLHS(lhs), pStmt(stmt) {  }

	virtual T Eval() 
	{
		extern void SetErrorFile(CString& filename,
		CString& token, int errorline, int errorchar);

		AssignmentLHS<T>* pAssign = FindAssignment<T>(pLHS);

		if(pAssign==NULL)
		{
			SetErrorFile(scanfile.filename, CString(L""), scanfile.lineno, scanfile.charcount);
			std::wostringstream os;
			os<<L"Functional expression \'"<<pLHS->GetFullName()<<L"\' is not found.\r\n";
			HandleSyntaxError(os.str());
		}

		if(pStmt) pStmt->Run();

		return pLHS->GetData()=pAssign->GetExpression()->Eval();
	}

	virtual ~CallExpression() { }
};


template<typename vec>
class CallExprVec: public Expression<vec>
{
protected:
	Statement* pStmt; 
	LeftHandSideArray<vec>* pLHS;

public:
	ScanFile scanfile;

	CallExprVec(LeftHandSideArray<vec>* lhs, Statement* stmt): 
	  pLHS(lhs), pStmt(stmt) {  }

	virtual vec Eval() 
	{
		extern void SetErrorFile(CString& filename,
		CString& token, int errorline, int errorchar);

		StmtInitArray* pAssign = FindAssignment<vec>(pLHS);

		if(pAssign==NULL)
		{
			SetErrorFile(scanfile.filename, CString(L""), scanfile.lineno, scanfile.charcount);
			std::wostringstream os;
			os<<L"Functional expression \'"<<pLHS->GetFullName()<<L"\' is not found.\r\n";
			HandleSyntaxError(os.str());
		}

		if(pStmt) pStmt->Run();
		
		pAssign->Initialize(); return pLHS->GetData();
	}

	virtual ~CallExprVec() { }
};


template<typename T>
class AssignExpression: public Expression<T>
{
protected:
	Statement* pStmt; 
	LeftHandSide<T>* pLHS;

public:
	ScanFile scanfile;

	AssignExpression(LeftHandSide<T>* lhs, Statement* stmt): 
	  pLHS(lhs), pStmt(stmt) {  }

	virtual T Eval() 
	{
		extern void SetErrorFile(CString& filename,
		CString& token, int errorline, int errorchar);

		AssignmentLHS<T>* pAssign = FindAssignment<T>(pLHS);

		if(pAssign==NULL)
		{
			SetErrorFile(scanfile.filename, CString(L""), scanfile.lineno, scanfile.charcount);
			std::wostringstream os;
			os<<L"Functional expression \'"<<pLHS->GetFullName()<<L"\' is not found.\r\n";
			HandleSyntaxError(os.str());
		}

		if(pStmt) pStmt->Run();

		return pAssign->GetExpression()->Eval();
	}

	virtual ~AssignExpression() { }
};

template<typename T, typename VarAry, typename ary, typename vec>
class CallArrayExpr1D: public Expression<vec>
{
protected:
	std::wstring vrid; Statement* pStmt;
	AssignAryEle<T, VarAry >* pAsgn; T* data;
	ary *theArray;
	
	bool SetPtr(int idx1)
	{
		Statement* asgn = FindAssignment<T, VarAry>(vrid, idx1);
		if(asgn)
		{
			pAsgn = dynamic_cast<AssignAryEle<T, VarAry>*>(asgn);

			if(pAsgn == NULL) return false;
			
			data = &pAsgn->GetVarAry()->GetDataPtr()->at(idx1); 
			return true;
		}
		else return false;
	}

public:

	CallArrayExpr1D(const wchar_t* vid, ary *ptr, Statement* stmt): 
	  vrid(vid), theArray(ptr), pAsgn(NULL), pStmt(stmt) { }

	virtual vec Eval() 
	{
		if(pStmt) pStmt->Run(); 
		
		int size1 = theArray->getsize(); vec v(size1);
		for(int i=0; i<size1; ++i)
		{
			if(SetPtr(i))
				v[i] = *data = pAsgn->GetExpression()->Eval();
			else v[i] = theArray->at(i);
		}

		return v;
	}

	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		ostr<<vrid;
		return ostr.str();
	}
	
	virtual ~CallArrayExpr1D() { }
};

template<typename T, typename VarAry, typename ary, typename vec>
class CallArrayExpr2D: public Expression<vec>
{
protected:
	std::wstring vrid; Statement* pStmt;
	AssignAryEle<T, VarAry >* pAsgn; T* data;
	ary *theArray;
	
	bool SetPtr(int idx1, int idx2)
	{
		Statement* asgn = FindAssignment<T, VarAry>(vrid, idx1, idx2);
		if(asgn)
		{
			pAsgn = dynamic_cast<AssignAryEle<T, VarAry>*>(asgn);

			if(pAsgn == NULL) return false;
			
			data = &pAsgn->GetVarAry()->GetDataPtr()->at(idx1, idx2); 
			return true;
		}
		else return false;
	}

public:

	CallArrayExpr2D(const wchar_t* vid, ary *ptr, Statement* stmt): 
	  vrid(vid), theArray(ptr), pAsgn(NULL), pStmt(stmt) { }

	virtual vec Eval() 
	{
		if(pStmt) pStmt->Run(); 
		
		int size1, size2; 
		theArray->getsize(size1, size2); 
		vec v; ::SetVecSize<T>(size1, size2, v);

		for(int i=0; i<size1; ++i)
			for(int j=0; j<size2; ++j)
			{
				if(SetPtr(i, j))
					v[i][j] = *data = pAsgn->GetExpression()->Eval();
				else v[i][i] = theArray->at(i, j);
			}
		return v;
	}

	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		ostr<<vrid;
		return ostr.str();
	}
	
	virtual ~CallArrayExpr2D() { }
};

template<typename T, typename VarAry, typename ary, typename vec>
class CallArrayExpr3D: public Expression<vec>
{
protected:
	std::wstring vrid; Statement* pStmt;
	AssignAryEle<T, VarAry >* pAsgn; T* data;
	ary *theArray;
	
	bool SetPtr(int idx1, int idx2, int idx3)
	{
		Statement* asgn = FindAssignment<T, VarAry>(vrid, idx1, idx2, idx3);
		if(asgn)
		{
			pAsgn = dynamic_cast<AssignAryEle<T, VarAry>*>(asgn);

			if(pAsgn == NULL) return false;
			
			data = &pAsgn->GetVarAry()->GetDataPtr()->at(idx1, idx2, idx3); 
			return true;
		}
		else return false;
	}

public:

	CallArrayExpr3D(const wchar_t* vid, ary *ptr, Statement* stmt): 
	  vrid(vid), theArray(ptr), pAsgn(NULL), pStmt(stmt) { }

	virtual vec Eval() 
	{
		if(pStmt) pStmt->Run(); 
		
		int size1, size2, size3; 
		theArray->getsize(size1, size2, size3); 
		vec v; ::SetVecSize<T>(size1, size2, size3, v);

		for(int i=0; i<size1; ++i)
			for(int j=0; j<size2; ++j)
				for(int k=0; k<size3; ++k)
				{
					if(SetPtr(i, j, k))
						v[i][j][k] = *data = pAsgn->GetExpression()->Eval();
					else v[i][j][k] = theArray->at(i, j, k);
				}

		return v;
	}

	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		ostr<<vrid;
		return ostr.str();
	}
	
	virtual ~CallArrayExpr3D() { }
};

template<typename T, typename VarAry, typename ary, typename vec>
class CallArrayExpr4D: public Expression<vec>
{
protected:
	std::wstring vrid; Statement* pStmt;
	AssignAryEle<T, VarAry >* pAsgn; T* data;
	ary *theArray;
	
	bool SetPtr(int idx1, int idx2, int idx3, int idx4)
	{
		Statement* asgn = FindAssignment<T, VarAry>(vrid, idx1, idx2, idx3, idx4);
		if(asgn)
		{
			pAsgn = dynamic_cast<AssignAryEle<T, VarAry>*>(asgn);

			if(pAsgn == NULL) return false;
			
			data = &pAsgn->GetVarAry()->GetDataPtr()->at(idx1, idx2, idx3, idx4); 
			return true;
		}
		else return false;
	}

public:

	CallArrayExpr4D(const wchar_t* vid, ary *ptr, Statement* stmt): 
	  vrid(vid), theArray(ptr), pAsgn(NULL), pStmt(stmt) { }

	virtual vec Eval() 
	{
		if(pStmt) pStmt->Run(); 
		
		int size1, size2, size3, size4; 
		theArray->getsize(size1, size2, size3, size4); 
		vec v; ::SetVecSize<T>(size1, size2, size3, size4, v);

		for(int i=0; i<size1; ++i)
			for(int j=0; j<size2; ++j)
				for(int k=0; k<size3; ++k)
					for(int l=0; l<size4; ++l)
					{
						if(SetPtr(i, j, k, l))
							v[i][j][k][l] = *data = pAsgn->GetExpression()->Eval();
						else v[i][j][k][l] = theArray->at(i, j, k, l);
					}
		return v;
	}

	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		ostr<<vrid;
		return ostr.str();
	}
	
	virtual ~CallArrayExpr4D() { }
};

template<typename T, typename VarAry, typename ary, typename vec>
class CallArrayExpr5D: public Expression<vec>
{
protected:
	std::wstring vrid; Statement* pStmt;
	AssignAryEle<T, VarAry >* pAsgn; T* data;
	ary *theArray;
	
	bool SetPtr(int idx1, int idx2, int idx3, int idx4, int idx5)
	{
		Statement* asgn = FindAssignment<T, VarAry>(vrid, idx1, idx2, idx3, idx4, idx5);
		if(asgn)
		{
			pAsgn = dynamic_cast<AssignAryEle<T, VarAry>*>(asgn);

			if(pAsgn == NULL) return false;
			
			data = &pAsgn->GetVarAry()->GetDataPtr()->at(idx1, idx2, idx3, idx4, idx5); 
			return true;
		}
		else return false;
	}

public:

	CallArrayExpr5D(const wchar_t* vid, ary *ptr, Statement* stmt): 
	  vrid(vid), theArray(ptr), pAsgn(NULL), pStmt(stmt) { }

	virtual vec Eval() 
	{
		if(pStmt) pStmt->Run(); 
		
		int size1, size2, size3, size4, size5; 
		theArray->getsize(size1, size2, size3, size4, size5); 
		vec v; ::SetVecSize<T>(size1, size2, size3, size4, size5, v);

		for(int i=0; i<size1; ++i)
			for(int j=0; j<size2; ++j)
				for(int k=0; k<size3; ++k)
					for(int l=0; l<size4; ++l)
						for(int m=0; m<size5; ++m)
						{
							if(SetPtr(i, j, k, l, m))
								v[i][j][k][l][m] = *data = pAsgn->GetExpression()->Eval();
							else v[i][j][k][l][m] = theArray->at(i, j, k, l, m);
						}
		return v;
	}

	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		ostr<<vrid;
		return ostr.str();
	}
	
	virtual ~CallArrayExpr5D() { }
};

template<typename T, typename VarAry>
class AssignmentExpr: public Expression<T>
{
protected:
	VarAry *vptr;
	AssignAryEle<T, VarAry >* pAsgn; 

	void SetPtr()
	{
		pAsgn = FindAssignment<T, VarAry>(vptr);
		if(!pAsgn)
			throw UnicodeException(LoadUnicodeMsg(IDS_CALLEXPR1D_ASSGNARY1D_NOT_FOUND).c_str());
	}

public:

	AssignmentExpr(VarAry *ptr): vptr(ptr), pAsgn(NULL) { }

	virtual T Eval() 
	{
		SetPtr(); return pAsgn->GetExpression()->Eval();
	}

	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		ostr<<_T(" ");
		return ostr.str();
	}
	
	virtual ~AssignmentExpr() { }
};

template<typename T, typename VarAry>
class AssignmentExpr1D: public Expression<T>
{
protected:
	std::wstring vrid;
	ExpIntg* ixpr;
	AssignAryEle<T, VarAry >* pAsgn; 

	integer odx1;

	void SetPtr()
	{
		integer idx1 = ixpr->Eval();

		if(pAsgn && odx1==idx1) return;
		
		Statement* asgn = FindAssignment<T, VarAry>(vrid, idx1);
		if(asgn)
		{
			pAsgn = dynamic_cast<AssignAryEle<T, VarAry>*>(asgn);
			if(pAsgn == NULL)
				throw UnicodeException(LoadUnicodeMsg(IDS_CALLEXPR1D_TYPE_MISMATCH).c_str());		
		}
		else
			throw UnicodeException(LoadUnicodeMsg(IDS_CALLEXPR1D_ASSGNARY1D_NOT_FOUND).c_str());
		
		odx1 = idx1; if(ixpr->bLiteral) bLiteral=true;
	}

public:

	AssignmentExpr1D(const std::wstring& id, ExpIntg* xpr): 
	  vrid(id), ixpr(xpr), pAsgn(NULL) { }

	virtual T Eval() 
	{
		if(!bLiteral) SetPtr(); 

		return pAsgn->GetExpression()->Eval();
	}

	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		ostr<<vrid;
		return ostr.str();
	}
	
	virtual ~AssignmentExpr1D() { }
};


template<typename T, typename VarAry>
class AssignmentExpr2D: public Expression<T>
{
protected:
	std::wstring vrid;
	ExpIntg *ixpr1, *ixpr2;
	AssignAryEle<T, VarAry>* pAsgn;

	integer odx1, odx2;

	void SetPtr()
	{	
		integer idx1 = ixpr1->Eval(), idx2 = ixpr2->Eval();

		if(pAsgn && idx1==odx1 && idx2==odx2) return;

		Statement* asgn = FindAssignment<T, VarAry>(vrid, idx1, idx2);
		if(asgn)
		{
			pAsgn = dynamic_cast<AssignAryEle<T, VarAry>*>(asgn);
			if(pAsgn == NULL)
				throw UnicodeException(LoadUnicodeMsg(IDS_TYPE_MISMATCH_CALLEXPR1D).c_str());
		}
		else
			throw UnicodeException(LoadUnicodeMsg(IDS_CALLEXPR2D_ASSGNARY2D_NOT_FOUND).c_str());

		odx1 = idx1; odx2 = idx2; 
		if(ixpr1->bLiteral && ixpr2->bLiteral) bLiteral = true;
	}

public:

	AssignmentExpr2D(const std::wstring& id, 
		ExpIntg* xpr1, ExpIntg* xpr2): 
	  vrid(id), ixpr1(xpr1), ixpr2(xpr2), pAsgn(NULL){ }

	virtual T Eval() 
	{
		if(!bLiteral) SetPtr();
		return pAsgn->GetExpression()->Eval();
	}

	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		ostr<<vrid;
		return ostr.str();
	}
	
	virtual ~AssignmentExpr2D() { }
};

template<typename T, typename VarAry>
class AssignmentExpr3D: public Expression<T>
{
protected:
	std::wstring vrid;
	ExpIntg *ixpr1, *ixpr2, *ixpr3;
	AssignAryEle<T, VarAry>* pAsgn;

	integer odx1, odx2, odx3;

	void SetPtr()
	{	
		integer idx1 = ixpr1->Eval(), idx2 = ixpr2->Eval(), idx3= ixpr3->Eval();

		if(pAsgn && idx1==odx1 && idx2==odx2 && idx3==odx3) return;

		Statement* asgn = FindAssignment<T, VarAry>(vrid, idx1, idx2, idx3);
		if(asgn)
		{
			pAsgn = dynamic_cast<AssignAryEle<T, VarAry>*>(asgn);
			if(pAsgn == NULL)
				throw UnicodeException(LoadUnicodeMsg(IDS_TYPE_MISMATCH_CALLEXPR1D).c_str());
		}
		else
			throw UnicodeException(LoadUnicodeMsg(IDS_CALLEXPR2D_ASSGNARY2D_NOT_FOUND).c_str());
		
		odx1 = idx1; odx2 = idx2; odx3 = idx3;

		if(ixpr1->bLiteral && ixpr2->bLiteral && ixpr3->bLiteral) bLiteral = true;
	}

public:

	AssignmentExpr3D(const std::wstring& id, 
		ExpIntg* xpr1, ExpIntg* xpr2, ExpIntg* xpr3): 
	  vrid(id), ixpr1(xpr1), ixpr2(xpr2), ixpr3(xpr3), pAsgn(NULL){ }

	virtual T Eval() 
	{
		if(!bLiteral) SetPtr(); 
		return pAsgn->GetExpression()->Eval();
	}

	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		ostr<<vrid;
		return ostr.str();
	}
	
	virtual ~AssignmentExpr3D() { }
};

template<typename T, typename VarAry>
class AssignmentExpr4D: public Expression<T>
{
protected:
	std::wstring vrid;
	ExpIntg *ixpr1, *ixpr2, *ixpr3, *ixpr4;
	AssignAryEle<T, VarAry>* pAsgn;

	integer odx1, odx2, odx3, odx4;

	void SetPtr()
	{	
		integer idx1 = ixpr1->Eval(), idx2 = ixpr2->Eval(),
			    idx3 = ixpr3->Eval(), idx4 = ixpr4->Eval();

		if(pAsgn && idx1==odx1 && idx2==odx2 && idx3==odx3 && idx4==odx4) return;

		Statement* asgn = FindAssignment<T, VarAry>(vrid, idx1, idx2, idx3, idx4);
		if(asgn)
		{
			pAsgn = dynamic_cast<AssignAryEle<T, VarAry>*>(asgn);
			if(pAsgn == NULL)
				throw UnicodeException(LoadUnicodeMsg(IDS_TYPE_MISMATCH_CALLEXPR1D).c_str());
		}
		else
			throw UnicodeException(LoadUnicodeMsg(IDS_CALLEXPR2D_ASSGNARY2D_NOT_FOUND).c_str());
		
		odx1 = idx1; odx2 = idx2; odx3 = idx3; odx4 = idx4;

		if(ixpr1->bLiteral && ixpr2->bLiteral && 
			ixpr3->bLiteral && ixpr4->bLiteral) bLiteral = true;
	}

public:

	AssignmentExpr4D(const std::wstring& id, 
		ExpIntg* xpr1, ExpIntg* xpr2, 
		ExpIntg* xpr3, ExpIntg* xpr4): vrid(id), 
			ixpr1(xpr1), ixpr2(xpr2), ixpr3(xpr3), ixpr4(xpr4), pAsgn(NULL){ }

	virtual T Eval() 
	{
		if(!bLiteral) SetPtr(); 
		return pAsgn->GetExpression()->Eval();
	}

	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		ostr<<vrid;
		return ostr.str();
	}
	
	virtual ~AssignmentExpr4D() { }
};


template<typename T, typename VarAry>
class AssignmentExpr5D: public Expression<T>
{
protected:
	std::wstring vrid;
	ExpIntg *ixpr1, *ixpr2, *ixpr3, *ixpr4, *ixpr5;
	AssignAryEle<T, VarAry>* pAsgn;

	integer odx1, odx2, odx3, odx4, odx5;

	void SetPtr()
	{	
		integer idx1 = ixpr1->Eval(), idx2 = ixpr2->Eval(),
			    idx3 = ixpr3->Eval(), idx4 = ixpr4->Eval(), 
				idx5 = ixpr5->Eval();

		if(pAsgn && idx1==odx1 && idx2==odx2 && 
			idx3==odx3 && idx4==odx4 && idx5==odx5) return;

		Statement* asgn = FindAssignment<T, VarAry>(vrid, idx1, idx2, idx3, idx4, idx5);
		if(asgn)
		{
			pAsgn = dynamic_cast<AssignAryEle<T, VarAry>*>(asgn);
			if(pAsgn == NULL)
				throw UnicodeException(LoadUnicodeMsg(IDS_TYPE_MISMATCH_CALLEXPR1D).c_str());
		}
		else
			throw UnicodeException(LoadUnicodeMsg(IDS_CALLEXPR2D_ASSGNARY2D_NOT_FOUND).c_str());
		
		odx1 = idx1; odx2 = idx2; odx3 = idx3; odx4 = idx4; odx5 = idx5;

		if(ixpr1->bLiteral && ixpr2->bLiteral && 
			ixpr3->bLiteral && ixpr4->bLiteral && ixpr5->bLiteral) bLiteral = true;
	}

public:

	AssignmentExpr5D(const std::wstring& id, 
		ExpIntg* xpr1, ExpIntg* xpr2, 
		ExpIntg* xpr3, ExpIntg* xpr4, ExpIntg* xpr5): vrid(id), 
			ixpr1(xpr1), ixpr2(xpr2), ixpr3(xpr3), ixpr4(xpr4), ixpr5(xpr5), pAsgn(NULL){ }

	virtual T Eval() 
	{
		if(!bLiteral) SetPtr();

		return pAsgn->GetExpression()->Eval();
	}

	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		ostr<<vrid;
		return ostr.str();
	}
	
	virtual ~AssignmentExpr5D() { }
};

class BFunS: public BaseFunc, public ExpBool
{
protected:
	//std::wstring msg;
	ExpStrn* Expr; 
	
public:
	BFunS(const FunName& fname, ExpStrn* xpr): 
	  BaseFunc(fname.fsig, fname.fn), Expr(xpr) {   }
	
	  virtual bool Eval()
	  {
		  return fsig.ptr.b_fun_s(Expr->Eval());
	  }
	
	virtual std::wstring Display()
	{
		std::wostringstream os;
		os<<fname<<_T("(")<<Expr->Eval()<<_T(")");
		return os.str();
	}

	virtual ~BFunS() {}
};

template<typename T> 
class VarAryUdt1D: public VarAryBase, public LeftHandSide<T>
{
protected:
	VarUser<ArrayU1D<T> > *data;
	ExpIntg* index1;

public:

	VarAryUdt1D(VarUser<ArrayU1D<T> >* ptr, ExpIntg* i):
	  index1(i), data(ptr) { }

	virtual int GetIndex1() const { return int(index1->Eval()); }
	virtual const std::wstring& GetId() const { return data->GetId(); } 

	virtual std::wstring GetFullName() const 
	{ 
		std::wostringstream os;
		
		os<<data->GetFullName()<<_T("[")<<GetIndex1()<<_T("]");

		return os.str();
	}

	virtual bool IsCongruent(VarAryBase *vptr) const
	{
		if(this->GetId()==vptr->GetId() &&
			this->GetIndex1()==vptr->GetIndex1()) return true;
		else return false;
	}

	VarUser<ArrayU1D<T> >* GetDataPtr() { return data->GetData(); }

	virtual T Eval() { return GetData(); }
	virtual T& GetData() 
	{ 
		return data->GetData().at(int(index1->Eval())); 
	}

	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		ostr<<data->GetId()<<_T("[")<<int(index1->Eval())<<_T("]");
		return ostr.str();
	}
	
	virtual ~VarAryUdt1D() { }
};

template<typename T> 
class VarAryUdt2D: public VarAryBase, public LeftHandSide<T>
{
protected:
	VarUser<ArrayU2D<T> >* data; 
	ExpIntg *index1, *index2;
	
public:
	VarAryUdt2D(VarUser<ArrayU2D<T> >* ptr, ExpIntg* i1, ExpIntg* i2): 
	  index1(i1), index2(i2), data(ptr){ }

	virtual int GetIndex1() const {  return int(index1->Eval()); }
	virtual int GetIndex2() const {  return int(index2->Eval()); }
	virtual const std::wstring& GetId() const { return data->GetId(); } 
	
	virtual std::wstring GetFullName() const 
	{ 
		std::wostringstream os;
		os<<data->GetFullName()<<_T("[")<<GetIndex1()<<_T(", ")<<GetIndex2()<<_T("]");
		return os.str();
	}

	virtual bool IsCongruent(VarAryBase *vptr) const
	{
		if(this->GetId()==vptr->GetId() &&
			this->GetIndex1()==vptr->GetIndex1() &&
			this->GetIndex2()==vptr->GetIndex2() ) return true;
		else return false;
	}

	VarUser<ArrayU2D<T> >* GetDataPtr() { return data; }

	virtual T Eval() { return GetData(); }
	virtual T& GetData() 
	{ return data->GetData().at(int(index1->Eval()), int(index2->Eval())); }

	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		ostr<<data->GetId()<<_T("[")<<int(index1->Eval())<<_T(" , ")<<int(index2->Eval())<<_T("]");
		return ostr.str();
	}
	
	virtual ~VarAryUdt2D() { }
};

template<typename T> 
class VarAryUdt3D: public VarAryBase, public LeftHandSide<T>
{
protected:
	VarUser<ArrayU3D<T> >* data;
	ExpIntg *index1, *index2, *index3;

public:

	VarAryUdt3D(VarUser<ArrayU3D<T> >* ptr, ExpIntg* i1, ExpIntg* i2, 
		ExpIntg* i3): index1(i1), index2(i2), index3(i3), data(ptr) { }

	VarUser<ArrayU3D<T> >* GetDataPtr() { return data; }
	
	virtual const std::wstring& GetId() const { return data->GetId(); } 

	virtual std::wstring GetFullName() const 
	{ 
		std::wostringstream os;
		os<<data->GetFullName()<<_T("[")<<GetIndex1()<<_T(", ")<<GetIndex2()<<_T(", ")<<GetIndex3()<<_T("]");
		return os.str();
	}

	virtual int GetIndex1() const { return int(index1->Eval()); }
	virtual int GetIndex2() const { return int(index2->Eval()); }
	virtual int GetIndex3() const { return int(index3->Eval()); }
	virtual bool IsCongruent(VarAryBase *vptr) const
	{
		if(this->GetId()==vptr->GetId() &&
			this->GetIndex1()==vptr->GetIndex1() &&
			this->GetIndex2()==vptr->GetIndex2() &&
			this->GetIndex3()==vptr->GetIndex3() ) return true;
		else return false;
	}

	virtual T Eval() { return GetData(); }
	virtual T& GetData() 
	{ return data->GetData().at(int(index1->Eval()), int(index2->Eval()), int(index3->Eval())); }

	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		ostr<<data->GetId()<<_T("[")<<int(index1->Eval())<<_T(" , ")<<int(index2->Eval())<<_T(" , ")<<int(index3->Eval())<<_T("]");
		return ostr.str();
	}
	
	virtual ~VarAryUdt3D() { }
};

template<typename T> 
class VarAryUdt4D: public VarAryBase, public LeftHandSide<T>
{
protected:
	VarUser<ArrayU4D<T> >* data;
	ExpIntg *index1, *index2, *index3, *index4;

public:

	VarAryUdt4D(VarUser<ArrayU4D<T> >* ptr, ExpIntg* i1, 
		ExpIntg* i2, ExpIntg* i3, ExpIntg* i4):
	    index1(i1), index2(i2), index3(i3), index4(i4), data(ptr) { }

	VarUser<ArrayU4D<T> >* GetDataPtr() { return data; }
	
	virtual const std::wstring& GetId() const { return data->GetId(); } 

	virtual std::wstring GetFullName() const 
	{ 
		std::wostringstream os;
		os<<data->GetFullName()<<_T("[")<<GetIndex1()<<_T(", ")<<GetIndex2()<<_T(", ")
			<<GetIndex3()<<_T(", ")<<GetIndex4()<<_T("]");
		return os.str();
	}

	virtual int GetIndex1() const { return int(index1->Eval()); }
	virtual int GetIndex2() const { return int(index2->Eval()); }
	virtual int GetIndex3() const { return int(index3->Eval()); }
	virtual int GetIndex4() const { return int(index4->Eval()); }
	virtual bool IsCongruent(VarAryBase *vptr) const
	{
		if(this->GetId()==vptr->GetId() &&
			this->GetIndex1()==vptr->GetIndex1() &&
			this->GetIndex2()==vptr->GetIndex2() &&
			this->GetIndex3()==vptr->GetIndex3() &&
			this->GetIndex4()==vptr->GetIndex4() ) return true;
		else return false;
	}

	virtual T Eval() { return GetData(); }
	virtual T& GetData() 
	{
		return data->GetData().at(int(index1->Eval()), int(index2->Eval()),
			int(index3->Eval()), int(index4->Eval())); 
	}

	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		ostr<<data->GetId()<<_T("[")<<int(index1->Eval())<<_T(" , ")<<int(index2->Eval())<<_T(" , ")
					   <<int(index3->Eval())<<_T(" , ")<<int(index4->Eval())<<_T("]");
		return ostr.str();
	}
	
	virtual ~VarAryUdt4D() { }
};


template<typename T> 
class VarAryUdt5D: public VarAryBase, public LeftHandSide<T>
{
protected:
	VarUser<ArrayU5D<T> >* data;
	ExpIntg *index1, *index2, *index3, *index4, *index5;

public:

	VarAryUdt5D(VarUser<ArrayU5D<T> >* ptr, ExpIntg* i1, ExpIntg* i2, 
		ExpIntg* i3, ExpIntg* i4, ExpIntg* i5):
		index1(i1), index2(i2), index3(i3), index4(i4), index5(i5), data(ptr) { }

	VarUser<ArrayU5D<T> >* GetDataPtr() { return data; }
	virtual const std::wstring& GetId() const { return data->GetId(); } 

	virtual std::wstring GetFullName() const 
	{ 
		std::wostringstream os;
		os<<data->GetFullName()<<_T("[")<<GetIndex1()<<_T(", ")<<GetIndex2()<<
			GetIndex3()<<_T(", ")<<GetIndex4()<<_T(", ")<<GetIndex5()<<_T("]");
		return os.str();
	}

	virtual int GetIndex1() const { return int(index1->Eval()); }
	virtual int GetIndex2() const { return int(index2->Eval()); }
	virtual int GetIndex3() const { return int(index3->Eval()); }
	virtual int GetIndex4() const { return int(index4->Eval()); }
	virtual int GetIndex5() const { return int(index5->Eval()); }
	virtual bool IsCongruent(VarAryBase *vptr) const
	{
		if(this->GetId()==vptr->GetId() &&
			this->GetIndex1()==vptr->GetIndex1() &&
			this->GetIndex2()==vptr->GetIndex2() &&
			this->GetIndex3()==vptr->GetIndex3() &&
			this->GetIndex4()==vptr->GetIndex4() &&
			this->GetIndex5()==vptr->GetIndex5()) return true;
		else return false;
	}

	virtual T Eval() { return GetData();  }
	virtual T& GetData() 
	{
		return data->GetData().at(int(index1->Eval()), int(index2->Eval()),
			int(index3->Eval()), int(index4->Eval()), int(index5->Eval())); 
	}

	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		ostr<<data->GetId()<<_T("[")<<int(index1->Eval())<<_T(" , ")<<int(index2->Eval())<<_T(" , ")
					   <<int(index3->Eval())<<_T(" , ")<<int(index4->Eval())<<_T(" , ")
					   <<int(index5->Eval())<<_T("]");
		return ostr.str();
	}
	
	virtual ~VarAryUdt5D() { }
};


template<typename T> 
class VarAryVub1D: public VarAryBase, public LeftHandSide<T>
{
protected:
	VarMem<ArrayU1D<T> > *data;
	ExpIntg* index1;

public:

	VarAryVub1D(VarMem<ArrayU1D<T> >* ptr, ExpIntg* i):
	  index1(i), data(ptr) { }

	virtual int GetIndex1() const { return int(index1->Eval()); }
	virtual const std::wstring& GetId() const { return data->GetId(); } 

	virtual std::wstring GetFullName() const 
	{ 
		std::wostringstream os;
		
		os<<data->GetFullName()<<_T("[")<<GetIndex1()<<_T("]");

		return os.str();
	}

	virtual bool IsCongruent(VarAryBase *vptr) const
	{
		if(this->GetId()==vptr->GetId() &&
			this->GetIndex1()==vptr->GetIndex1()) return true;
		else return false;
	}

	VarMem<ArrayU1D<T> >* GetDataPtr() { return data->GetData(); }

	virtual T Eval() { return GetData(); }
	virtual T& GetData() 
	{ 
		return data->GetData().at(int(index1->Eval())); 
	}

	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		ostr<<data->GetId()<<_T("[")<<int(index1->Eval())<<_T("]");
		return ostr.str();
	}
	
	virtual ~VarAryVub1D() { }
};

template<typename T> 
class VarAryVub2D: public VarAryBase, public LeftHandSide<T>
{
protected:
	VarMem<ArrayU2D<T> >* data; 
	ExpIntg *index1, *index2;
	
public:
	VarAryVub2D(VarMem<ArrayU2D<T> >* ptr, ExpIntg* i1, ExpIntg* i2): 
	  index1(i1), index2(i2), data(ptr){ }

	virtual int GetIndex1() const {  return int(index1->Eval()); }
	virtual int GetIndex2() const {  return int(index2->Eval()); }
	virtual const std::wstring& GetId() const { return data->GetId(); } 
	
	virtual std::wstring GetFullName() const 
	{ 
		std::wostringstream os;
		os<<data->GetFullName()<<_T("[")<<GetIndex1()<<_T(", ")<<GetIndex2()<<_T("]");
		return os.str();
	}

	virtual bool IsCongruent(VarAryBase *vptr) const
	{
		if(this->GetId()==vptr->GetId() &&
			this->GetIndex1()==vptr->GetIndex1() &&
			this->GetIndex2()==vptr->GetIndex2() ) return true;
		else return false;
	}

	VarMem<ArrayU2D<T> >* GetDataPtr() { return data; }

	virtual T Eval() { return GetData(); }
	virtual T& GetData() 
	{ return data->GetData().at(int(index1->Eval()), int(index2->Eval())); }

	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		ostr<<data->GetId()<<_T("[")<<int(index1->Eval())<<_T(" , ")<<int(index2->Eval())<<_T("]");
		return ostr.str();
	}
	
	virtual ~VarAryVub2D() { }
};

template<typename T> 
class VarAryVub3D: public VarAryBase, public LeftHandSide<T>
{
protected:
	VarMem<ArrayU3D<T> >* data;
	ExpIntg *index1, *index2, *index3;

public:

	VarAryVub3D(VarMem<ArrayU3D<T> >* ptr, ExpIntg* i1, ExpIntg* i2, 
		ExpIntg* i3): index1(i1), index2(i2), index3(i3), data(ptr) { }

	VarMem<ArrayU3D<T> >* GetDataPtr() { return data; }
	
	virtual const std::wstring& GetId() const { return data->GetId(); } 

	virtual std::wstring GetFullName() const 
	{ 
		std::wostringstream os;
		os<<data->GetFullName()<<_T("[")<<GetIndex1()<<_T(", ")<<GetIndex2()<<_T(", ")<<GetIndex3()<<_T("]");
		return os.str();
	}

	virtual int GetIndex1() const { return int(index1->Eval()); }
	virtual int GetIndex2() const { return int(index2->Eval()); }
	virtual int GetIndex3() const { return int(index3->Eval()); }
	virtual bool IsCongruent(VarAryBase *vptr) const
	{
		if(this->GetId()==vptr->GetId() &&
			this->GetIndex1()==vptr->GetIndex1() &&
			this->GetIndex2()==vptr->GetIndex2() &&
			this->GetIndex3()==vptr->GetIndex3() ) return true;
		else return false;
	}

	virtual T Eval() { return GetData(); }
	virtual T& GetData() 
	{ return data->GetData().at(int(index1->Eval()), int(index2->Eval()), int(index3->Eval())); }

	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		ostr<<data->GetId()<<_T("[")<<int(index1->Eval())<<_T(" , ")<<int(index2->Eval())<<_T(" , ")<<int(index3->Eval())<<_T("]");
		return ostr.str();
	}
	
	virtual ~VarAryVub3D() { }
};

template<typename T> 
class VarAryVub4D: public VarAryBase, public LeftHandSide<T>
{
protected:
	VarMem<ArrayU4D<T> >* data;
	ExpIntg *index1, *index2, *index3, *index4;

public:

	VarAryVub4D(VarMem<ArrayU4D<T> >* ptr, ExpIntg* i1, 
		ExpIntg* i2, ExpIntg* i3, ExpIntg* i4):
	    index1(i1), index2(i2), index3(i3), index4(i4), data(ptr) { }

	VarMem<ArrayU4D<T> >* GetDataPtr() { return data; }
	
	virtual const std::wstring& GetId() const { return data->GetId(); } 

	virtual std::wstring GetFullName() const 
	{ 
		std::wostringstream os;
		os<<data->GetFullName()<<_T("[")<<GetIndex1()<<_T(", ")<<GetIndex2()<<_T(", ")
			<<GetIndex3()<<_T(", ")<<GetIndex4()<<_T("]");
		return os.str();
	}

	virtual int GetIndex1() const { return int(index1->Eval()); }
	virtual int GetIndex2() const { return int(index2->Eval()); }
	virtual int GetIndex3() const { return int(index3->Eval()); }
	virtual int GetIndex4() const { return int(index4->Eval()); }
	virtual bool IsCongruent(VarAryBase *vptr) const
	{
		if(this->GetId()==vptr->GetId() &&
			this->GetIndex1()==vptr->GetIndex1() &&
			this->GetIndex2()==vptr->GetIndex2() &&
			this->GetIndex3()==vptr->GetIndex3() &&
			this->GetIndex4()==vptr->GetIndex4() ) return true;
		else return false;
	}

	virtual T Eval() { return GetData(); }
	virtual T& GetData() 
	{
		return data->GetData().at(int(index1->Eval()), int(index2->Eval()),
			int(index3->Eval()), int(index4->Eval())); 
	}

	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		ostr<<data->GetId()<<_T("[")<<int(index1->Eval())<<_T(" , ")<<int(index2->Eval())<<_T(" , ")
					   <<int(index3->Eval())<<_T(" , ")<<int(index4->Eval())<<_T("]");
		return ostr.str();
	}
	
	virtual ~VarAryVub4D() { }
};


template<typename T> 
class VarAryVub5D: public VarAryBase, public LeftHandSide<T>
{
protected:
	VarMem<ArrayU5D<T> >* data;
	ExpIntg *index1, *index2, *index3, *index4, *index5;

public:

	VarAryVub5D(VarMem<ArrayU5D<T> >* ptr, ExpIntg* i1, ExpIntg* i2, 
		ExpIntg* i3, ExpIntg* i4, ExpIntg* i5):
		index1(i1), index2(i2), index3(i3), index4(i4), index5(i5), data(ptr) { }

	VarMem<ArrayU5D<T> >* GetDataPtr() { return data; }
	virtual const std::wstring& GetId() const { return data->GetId(); } 

	virtual std::wstring GetFullName() const 
	{ 
		std::wostringstream os;
		os<<data->GetFullName()<<_T("[")<<GetIndex1()<<_T(", ")<<GetIndex2()<<
			GetIndex3()<<_T(", ")<<GetIndex4()<<_T(", ")<<GetIndex5()<<_T("]");
		return os.str();
	}

	virtual int GetIndex1() const { return int(index1->Eval()); }
	virtual int GetIndex2() const { return int(index2->Eval()); }
	virtual int GetIndex3() const { return int(index3->Eval()); }
	virtual int GetIndex4() const { return int(index4->Eval()); }
	virtual int GetIndex5() const { return int(index5->Eval()); }
	virtual bool IsCongruent(VarAryBase *vptr) const
	{
		if(this->GetId()==vptr->GetId() &&
			this->GetIndex1()==vptr->GetIndex1() &&
			this->GetIndex2()==vptr->GetIndex2() &&
			this->GetIndex3()==vptr->GetIndex3() &&
			this->GetIndex4()==vptr->GetIndex4() &&
			this->GetIndex5()==vptr->GetIndex5()) return true;
		else return false;
	}

	virtual T Eval() { return GetData();  }
	virtual T& GetData() 
	{
		return data->GetData().at(int(index1->Eval()), int(index2->Eval()),
			int(index3->Eval()), int(index4->Eval()), int(index5->Eval())); 
	}

	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		ostr<<data->GetId()<<_T("[")<<int(index1->Eval())<<_T(" , ")<<int(index2->Eval())<<_T(" , ")
					   <<int(index3->Eval())<<_T(" , ")<<int(index4->Eval())<<_T(" , ")
					   <<int(index5->Eval())<<_T("]");
		return ostr.str();
	}
	
	virtual ~VarAryVub5D() { }
};


template<typename T, typename VarAry>
class CallExprArray: public Expression<T>
{
protected:
	VarAry *pVarAry; Statement* pStmt;
	AssignAryEle<T, VarAry >* pAsgn; T* data;

	void SetPtr()
	{
		pAsgn = FindAssignment<T, VarAry>(pVarAry);
		if(!pAsgn)
			throw UnicodeException(LoadUnicodeMsg(IDS_CALLEXPR1D_ASSGNARY1D_NOT_FOUND).c_str());
	}

public:

	CallExprArray(VarAry *ptr, Statement* stmt): 
	  pVarAry(ptr), pAsgn(NULL), pStmt(stmt) { }

	virtual T Eval() 
	{
		if(pStmt) pStmt->Run(); if(!bLiteral) SetPtr(); 
		return pAsgn->GetVarAry()->GetData() = 
			pAsgn->GetExpression()->Eval();
	}

	virtual std::wstring Display() 
	{
		std::wostringstream ostr; ostr<<_T("  ");
		return ostr.str();
	}
	
	virtual ~CallExprArray() { }
};

template<typename vec> 
class ArraySizeExp: public Expression<integer>
{
protected:
	LeftHandSideArray<vec> *pLHS;
	LeftHandSide<integer> *size1, *size2, *size3, *size4;

public:
	
	ArraySizeExp(LeftHandSideArray<vec> *ptr, LeftHandSide<integer> *sptr1=NULL, 
		LeftHandSide<integer> *sptr2=NULL, LeftHandSide<integer> *sptr3=NULL, 
		LeftHandSide<integer> *sptr4=NULL): pLHS(ptr), 
		size1(sptr1), size2(sptr2), size3(sptr3), size4(sptr4) { }

	virtual integer Eval() 
	{ 
		int sz1, sz2, sz3, sz4, sz5;
		sz1 = sz2 = sz3 = sz4 = sz5 = 0;

		vec *vPtr = &pLHS->GetData();

		IntgVec1D *pIntgVec1D = dynamic_cast<IntgVec1D*>(vPtr);
		IntgVec2D *pIntgVec2D = dynamic_cast<IntgVec2D*>(vPtr);
		IntgVec3D *pIntgVec3D = dynamic_cast<IntgVec3D*>(vPtr);
		IntgVec4D *pIntgVec4D = dynamic_cast<IntgVec4D*>(vPtr);
		IntgVec5D *pIntgVec5D = dynamic_cast<IntgVec5D*>(vPtr);

		RealVec1D *pRealVec1D = dynamic_cast<RealVec1D*>(vPtr);
		RealVec2D *pRealVec2D = dynamic_cast<RealVec2D*>(vPtr);
		RealVec3D *pRealVec3D = dynamic_cast<RealVec3D*>(vPtr);
		RealVec4D *pRealVec4D = dynamic_cast<RealVec4D*>(vPtr);
		RealVec5D *pRealVec5D = dynamic_cast<RealVec5D*>(vPtr);

		CplxVec1D *pCplxVec1D = dynamic_cast<CplxVec1D*>(vPtr);
		CplxVec2D *pCplxVec2D = dynamic_cast<CplxVec2D*>(vPtr);
		CplxVec3D *pCplxVec3D = dynamic_cast<CplxVec3D*>(vPtr);
		CplxVec4D *pCplxVec4D = dynamic_cast<CplxVec4D*>(vPtr);
		CplxVec5D *pCplxVec5D = dynamic_cast<CplxVec5D*>(vPtr);

		VctrVec1D *pVctrVec1D = dynamic_cast<VctrVec1D*>(vPtr);
		VctrVec2D *pVctrVec2D = dynamic_cast<VctrVec2D*>(vPtr);
		VctrVec3D *pVctrVec3D = dynamic_cast<VctrVec3D*>(vPtr);
		VctrVec4D *pVctrVec4D = dynamic_cast<VctrVec4D*>(vPtr);
		VctrVec5D *pVctrVec5D = dynamic_cast<VctrVec5D*>(vPtr);

		if(pIntgVec1D)
			GetVecSize<integer>(sz1, *pIntgVec1D);
		else if(pIntgVec2D)
			GetVecSize<integer>(sz1, sz2, *pIntgVec2D);
		else if(pIntgVec3D)
			GetVecSize<integer>(sz1, sz2, sz3, *pIntgVec3D);
		else if(pIntgVec4D)
			GetVecSize<integer>(sz1, sz2, sz3, sz4, *pIntgVec4D);
		else if(pIntgVec5D)
			GetVecSize<integer>(sz1, sz2, sz3, sz4, sz5, *pIntgVec5D);

		else if(pRealVec1D)
			GetVecSize<realnum>(sz1, *pRealVec1D);
		else if(pRealVec2D)
			GetVecSize<realnum>(sz1, sz2, *pRealVec2D);
		else if(pRealVec3D)
			GetVecSize<realnum>(sz1, sz2, sz3, *pRealVec3D);
		else if(pRealVec4D)
			GetVecSize<realnum>(sz1, sz2, sz3, sz4, *pRealVec4D);
		else if(pRealVec5D)
			GetVecSize<realnum>(sz1, sz2, sz3, sz4, sz5, *pRealVec5D);

		else if(pCplxVec1D)
			GetVecSize<Complex>(sz1, *pCplxVec1D);
		else if(pCplxVec2D)
			GetVecSize<Complex>(sz1, sz2, *pCplxVec2D);
		else if(pCplxVec3D)
			GetVecSize<Complex>(sz1, sz2, sz3, *pCplxVec3D);
		else if(pCplxVec4D)
			GetVecSize<Complex>(sz1, sz2, sz3, sz4, *pCplxVec4D);
		else if(pCplxVec5D)
			GetVecSize<Complex>(sz1, sz2, sz3, sz4, sz5, *pCplxVec5D);

		else if(pVctrVec1D)
			GetVecSize<Vector>(sz1, *pVctrVec1D);
		else if(pVctrVec2D)
			GetVecSize<Vector>(sz1, sz2, *pVctrVec2D);
		else if(pVctrVec3D)
			GetVecSize<Vector>(sz1, sz2, sz3, *pVctrVec3D);
		else if(pVctrVec4D)
			GetVecSize<Vector>(sz1, sz2, sz3, sz4, *pVctrVec4D);
		else if(pVctrVec5D)
			GetVecSize<Vector>(sz1, sz2, sz3, sz4, sz5, *pVctrVec5D);
		else
			throw RunTimeException(L"Can not get array size.\r\n");

		if(size1) size1->GetData() = integer(sz2); 
		if(size2) size2->GetData() = integer(sz3); 
		if(size3) size3->GetData() = integer(sz4); 
		if(size4) size4->GetData() = integer(sz5); 
		
		return integer(sz1); 
	}
	
	virtual ~ArraySizeExp() { }
};

template<typename vec> 
class ArraySize: public Statement
{
protected:
	LeftHandSideArray<vec> *pLHS;
	LeftHandSide<integer> *size1, *size2, *size3, *size4, *size5;

public:
	
	ArraySize(LeftHandSideArray<vec> *ptr, LeftHandSide<integer> *sptr1, 
		LeftHandSide<integer> *sptr2=NULL, LeftHandSide<integer> *sptr3=NULL, 
		LeftHandSide<integer> *sptr4=NULL, LeftHandSide<integer> *sptr5=NULL): pLHS(ptr), 
		size1(sptr1), size2(sptr2), size3(sptr3), size4(sptr4), size5(sptr5) { }

	virtual void Run() 
	{ 
		int sz1, sz2, sz3, sz4, sz5;
		sz1 = sz2 = sz3 = sz4 = sz5 = 0;

		vec *vPtr = &pLHS->GetData();

		IntgVec1D *pIntgVec1D = dynamic_cast<IntgVec1D*>(vPtr);
		IntgVec2D *pIntgVec2D = dynamic_cast<IntgVec2D*>(vPtr);
		IntgVec3D *pIntgVec3D = dynamic_cast<IntgVec3D*>(vPtr);
		IntgVec4D *pIntgVec4D = dynamic_cast<IntgVec4D*>(vPtr);
		IntgVec5D *pIntgVec5D = dynamic_cast<IntgVec5D*>(vPtr);

		RealVec1D *pRealVec1D = dynamic_cast<RealVec1D*>(vPtr);
		RealVec2D *pRealVec2D = dynamic_cast<RealVec2D*>(vPtr);
		RealVec3D *pRealVec3D = dynamic_cast<RealVec3D*>(vPtr);
		RealVec4D *pRealVec4D = dynamic_cast<RealVec4D*>(vPtr);
		RealVec5D *pRealVec5D = dynamic_cast<RealVec5D*>(vPtr);

		CplxVec1D *pCplxVec1D = dynamic_cast<CplxVec1D*>(vPtr);
		CplxVec2D *pCplxVec2D = dynamic_cast<CplxVec2D*>(vPtr);
		CplxVec3D *pCplxVec3D = dynamic_cast<CplxVec3D*>(vPtr);
		CplxVec4D *pCplxVec4D = dynamic_cast<CplxVec4D*>(vPtr);
		CplxVec5D *pCplxVec5D = dynamic_cast<CplxVec5D*>(vPtr);

		VctrVec1D *pVctrVec1D = dynamic_cast<VctrVec1D*>(vPtr);
		VctrVec2D *pVctrVec2D = dynamic_cast<VctrVec2D*>(vPtr);
		VctrVec3D *pVctrVec3D = dynamic_cast<VctrVec3D*>(vPtr);
		VctrVec4D *pVctrVec4D = dynamic_cast<VctrVec4D*>(vPtr);
		VctrVec5D *pVctrVec5D = dynamic_cast<VctrVec5D*>(vPtr);

		if(pIntgVec1D)
			GetVecSize<integer>(sz1, *pIntgVec1D);
		else if(pIntgVec2D)
			GetVecSize<integer>(sz1, sz2, *pIntgVec2D);
		else if(pIntgVec3D)
			GetVecSize<integer>(sz1, sz2, sz3, *pIntgVec3D);
		else if(pIntgVec4D)
			GetVecSize<integer>(sz1, sz2, sz3, sz4, *pIntgVec4D);
		else if(pIntgVec5D)
			GetVecSize<integer>(sz1, sz2, sz3, sz4, sz5, *pIntgVec5D);

		else if(pRealVec1D)
			GetVecSize<realnum>(sz1, *pRealVec1D);
		else if(pRealVec2D)
			GetVecSize<realnum>(sz1, sz2, *pRealVec2D);
		else if(pRealVec3D)
			GetVecSize<realnum>(sz1, sz2, sz3, *pRealVec3D);
		else if(pRealVec4D)
			GetVecSize<realnum>(sz1, sz2, sz3, sz4, *pRealVec4D);
		else if(pRealVec5D)
			GetVecSize<realnum>(sz1, sz2, sz3, sz4, sz5, *pRealVec5D);

		else if(pCplxVec1D)
			GetVecSize<Complex>(sz1, *pCplxVec1D);
		else if(pCplxVec2D)
			GetVecSize<Complex>(sz1, sz2, *pCplxVec2D);
		else if(pCplxVec3D)
			GetVecSize<Complex>(sz1, sz2, sz3, *pCplxVec3D);
		else if(pCplxVec4D)
			GetVecSize<Complex>(sz1, sz2, sz3, sz4, *pCplxVec4D);
		else if(pCplxVec5D)
			GetVecSize<Complex>(sz1, sz2, sz3, sz4, sz5, *pCplxVec5D);

		else if(pVctrVec1D)
			GetVecSize<Vector>(sz1, *pVctrVec1D);
		else if(pVctrVec2D)
			GetVecSize<Vector>(sz1, sz2, *pVctrVec2D);
		else if(pVctrVec3D)
			GetVecSize<Vector>(sz1, sz2, sz3, *pVctrVec3D);
		else if(pVctrVec4D)
			GetVecSize<Vector>(sz1, sz2, sz3, sz4, *pVctrVec4D);
		else if(pVctrVec5D)
			GetVecSize<Vector>(sz1, sz2, sz3, sz4, sz5, *pVctrVec5D);
		else
			throw RunTimeException(L"Can not resize array.\r\n");

		if(size1) size1->GetData() = integer(sz1); 
		if(size2) size2->GetData() = integer(sz2); 
		if(size3) size3->GetData() = integer(sz3); 
		if(size4) size4->GetData() = integer(sz4); 
		if(size5) size5->GetData() = integer(sz5); 
	}
	
	virtual ~ArraySize() { }
};

class ArrayResizeBase: public Statement
{
public:
	ArrayResizeBase() {}

	virtual void InsertSize(int index, ExpIntg *pExp) = 0;

	virtual ~ArrayResizeBase() {}
};

template<typename vec> 
class ArrayResize: public ArrayResizeBase
{
protected:
	LeftHandSideArray<vec>* vaptr; 
	ExpIntg *psize[MAX_DIMENSION];

public:
	
	ArrayResize(LeftHandSideArray<vec>* ptr, 
		ExpIntg *sp0, ExpIntg *sp1, ExpIntg *sp2, 
		ExpIntg *sp3, ExpIntg *sp4): vaptr(ptr)
	{
		psize[0] = sp0; psize[1] = sp1; psize[2] = sp2;
		psize[3] = sp3; psize[4] = sp4;
	}

	virtual void InsertSize(int index, ExpIntg *pExp) {	psize[index] = pExp; }

	virtual void Run() 
	{ 
		vec *vPtr = &vaptr->GetData();

		int sz1, sz2, sz3, sz4, sz5;
		sz1 = sz2 = sz3 = sz4 = sz5 = 0;
		
		if(psize[0]) sz1= int(psize[0]->Eval());
		if(psize[1]) sz2= int(psize[1]->Eval());
		if(psize[2]) sz3= int(psize[2]->Eval());
		if(psize[3]) sz4= int(psize[3]->Eval());
		if(psize[4]) sz5= int(psize[4]->Eval());
		
		IntgVec1D *pIntgVec1D = dynamic_cast<IntgVec1D*>(vPtr);
		IntgVec2D *pIntgVec2D = dynamic_cast<IntgVec2D*>(vPtr);
		IntgVec3D *pIntgVec3D = dynamic_cast<IntgVec3D*>(vPtr);
		IntgVec4D *pIntgVec4D = dynamic_cast<IntgVec4D*>(vPtr);
		IntgVec5D *pIntgVec5D = dynamic_cast<IntgVec5D*>(vPtr);

		RealVec1D *pRealVec1D = dynamic_cast<RealVec1D*>(vPtr);
		RealVec2D *pRealVec2D = dynamic_cast<RealVec2D*>(vPtr);
		RealVec3D *pRealVec3D = dynamic_cast<RealVec3D*>(vPtr);
		RealVec4D *pRealVec4D = dynamic_cast<RealVec4D*>(vPtr);
		RealVec5D *pRealVec5D = dynamic_cast<RealVec5D*>(vPtr);

		CplxVec1D *pCplxVec1D = dynamic_cast<CplxVec1D*>(vPtr);
		CplxVec2D *pCplxVec2D = dynamic_cast<CplxVec2D*>(vPtr);
		CplxVec3D *pCplxVec3D = dynamic_cast<CplxVec3D*>(vPtr);
		CplxVec4D *pCplxVec4D = dynamic_cast<CplxVec4D*>(vPtr);
		CplxVec5D *pCplxVec5D = dynamic_cast<CplxVec5D*>(vPtr);

		VctrVec1D *pVctrVec1D = dynamic_cast<VctrVec1D*>(vPtr);
		VctrVec2D *pVctrVec2D = dynamic_cast<VctrVec2D*>(vPtr);
		VctrVec3D *pVctrVec3D = dynamic_cast<VctrVec3D*>(vPtr);
		VctrVec4D *pVctrVec4D = dynamic_cast<VctrVec4D*>(vPtr);
		VctrVec5D *pVctrVec5D = dynamic_cast<VctrVec5D*>(vPtr);

		if(pIntgVec1D)
			SetVecSize<integer>(sz1, *pIntgVec1D);
		else if(pIntgVec2D)
			SetVecSize<integer>(sz1, sz2, *pIntgVec2D);
		else if(pIntgVec3D)
			SetVecSize<integer>(sz1, sz2, sz3, *pIntgVec3D);
		else if(pIntgVec4D)
			SetVecSize<integer>(sz1, sz2, sz3, sz4, *pIntgVec4D);
		else if(pIntgVec5D)
			SetVecSize<integer>(sz1, sz2, sz3, sz4, sz5, *pIntgVec5D);

		else if(pRealVec1D)
			SetVecSize<realnum>(sz1, *pRealVec1D);
		else if(pRealVec2D)
			SetVecSize<realnum>(sz1, sz2, *pRealVec2D);
		else if(pRealVec3D)
			SetVecSize<realnum>(sz1, sz2, sz3, *pRealVec3D);
		else if(pRealVec4D)
			SetVecSize<realnum>(sz1, sz2, sz3, sz4, *pRealVec4D);
		else if(pRealVec5D)
			SetVecSize<realnum>(sz1, sz2, sz3, sz4, sz5, *pRealVec5D);

		else if(pCplxVec1D)
			SetVecSize<Complex>(sz1, *pCplxVec1D);
		else if(pCplxVec2D)
			SetVecSize<Complex>(sz1, sz2, *pCplxVec2D);
		else if(pCplxVec3D)
			SetVecSize<Complex>(sz1, sz2, sz3, *pCplxVec3D);
		else if(pCplxVec4D)
			SetVecSize<Complex>(sz1, sz2, sz3, sz4, *pCplxVec4D);
		else if(pCplxVec5D)
			SetVecSize<Complex>(sz1, sz2, sz3, sz4, sz5, *pCplxVec5D);

		else if(pVctrVec1D)
			SetVecSize<Vector>(sz1, *pVctrVec1D);
		else if(pVctrVec2D)
			SetVecSize<Vector>(sz1, sz2, *pVctrVec2D);
		else if(pVctrVec3D)
			SetVecSize<Vector>(sz1, sz2, sz3, *pVctrVec3D);
		else if(pVctrVec4D)
			SetVecSize<Vector>(sz1, sz2, sz3, sz4, *pVctrVec4D);
		else if(pVctrVec5D)
			SetVecSize<Vector>(sz1, sz2, sz3, sz4, sz5, *pVctrVec5D);
		else
			throw RunTimeException(L"Can not resize array.\r\n");
	}
	
	virtual ~ArrayResize() { }
};

template<typename T>
std::wstring PrepareDataForEdit(ExpTypeless* data)
{
	/*
	Variable<T>* pVar = dynamic_cast<Variable<T>*>(data);
	VarUser<T>* pUdt = dynamic_cast<VarUser<T>*>(data);
	VarMem<T>*  pVub = dynamic_cast<VarMem<T>*>(data);
	
	VarAry1D<T>* pAry1D = dynamic_cast<VarAry1D<T>*>(data); 
	VarAry2D<T>* pAry2D = dynamic_cast<VarAry2D<T>*>(data); 
	VarAry3D<T>* pAry3D = dynamic_cast<VarAry3D<T>*>(data);
	VarAry4D<T>* pAry4D = dynamic_cast<VarAry4D<T>*>(data);
	VarAry5D<T>* pAry5D = dynamic_cast<VarAry5D<T>*>(data);

	VarAryUdt1D<T>* pAryUdt1D = dynamic_cast<VarAryUdt1D<T>*>(data); 
	VarAryUdt2D<T>* pAryUdt2D = dynamic_cast<VarAryUdt2D<T>*>(data); 
	VarAryUdt3D<T>* pAryUdt3D = dynamic_cast<VarAryUdt3D<T>*>(data);
	VarAryUdt4D<T>* pAryUdt4D = dynamic_cast<VarAryUdt4D<T>*>(data);
	VarAryUdt5D<T>* pAryUdt5D = dynamic_cast<VarAryUdt5D<T>*>(data);
	
	VarAryVub1D<T>* pAryVub1D = dynamic_cast<VarAryVub1D<T>*>(data); 
	VarAryVub2D<T>* pAryVub2D = dynamic_cast<VarAryVub2D<T>*>(data); 
	VarAryVub3D<T>* pAryVub3D = dynamic_cast<VarAryVub3D<T>*>(data);
	VarAryVub4D<T>* pAryVub4D = dynamic_cast<VarAryVub4D<T>*>(data);
	VarAryVub5D<T>* pAryVub5D = dynamic_cast<VarAryVub5D<T>*>(data);
	*/

	Expression<T> *pExpr = dynamic_cast<Expression<T>*>(data);
		
	#ifdef new
	#define OLD_NEW new
	#undef new 
	#endif

	std::locale loc(std::locale(), new MyPunck);
	
	#ifdef OLD_NEW
	#define new OLD_NEW
	#undef OLD_NEW
	#endif

	
	std::wostringstream os;
	extern bool fmt_thousand;
	if(fmt_thousand) os.imbue(loc);

	/*
	if(pVar)				os<<pVar->Eval();
	else if(pUdt)			os<<pUdt->Eval();
	else if(pAry1D)			os<<pAry1D->Eval();
	else if(pAry2D)			os<<pAry2D->Eval();
	else if(pAry3D)			os<<pAry3D->Eval();
	else if(pAry4D)			os<<pAry4D->Eval();
	else if(pAry5D)			os<<pAry5D->Eval();
	else if(pAryUdt1D)		os<<pAryUdt1D->Eval();
	else if(pAryUdt2D)		os<<pAryUdt2D->Eval();
	else if(pAryUdt3D)		os<<pAryUdt3D->Eval();
	else if(pAryUdt4D)		os<<pAryUdt4D->Eval();
	else if(pAryUdt5D)		os<<pAryUdt5D->Eval();

	else if(pVub)			os<<pVub->Eval();
	else if(pAryVub1D)		os<<pAryVub1D->Eval();
	else if(pAryVub2D)		os<<pAryVub2D->Eval();
	else if(pAryVub3D)		os<<pAryVub3D->Eval();
	else if(pAryVub4D)		os<<pAryVub4D->Eval();
	else if(pAryVub5D)		os<<pAryVub5D->Eval();

	else return std::wstring(L"");

	*/

	if(pExpr)
		os<<pExpr->Eval();
	else 
		return std::wstring(L"");

	return os.str();
}

template<typename T>
bool SetDataValue(ExpTypeless *data, T rlt)
{
	LeftHandSide<T> *pLHS = dynamic_cast<LeftHandSide<T>*>(data);
	if(pLHS)
	{
		pLHS->GetData() = rlt; return true;
	}
	else  return false;
}


template<typename T>
AssignmentLHS<T>* StatementList::FindAssignment(LeftHandSide<T> *pLHS)
{
	StatementList* pStmtList;
	AssignmentLHS<T> *pAssign;
	
	int count = this->stmtindex == 0 ? 
		int(this->stmtlist.size()) : this->stmtindex;

	for(int i = count-1; i>=0; --i)
	{
		SET_USER_ABORT;

		if(stmtlist[i]) 
		{
			if(stmtlist[i]->stmttype==StmtExpr) 
			{
				pAssign = dynamic_cast<AssignmentLHS<T>*>(stmtlist[i]);

				if(pAssign)
				{
					if(pAssign->IsLHSEqual(pLHS)) 
						return pAssign;
					else continue;
				}
			}
			
			pStmtList = dynamic_cast<StatementList*>(stmtlist[i]);

			if(pStmtList)
			{
				pAssign = pStmtList->FindAssignment<T>(pLHS);
				if(pAssign) return pAssign;
			}
		}
	}

	return NULL;
}

template<typename vec>
StmtInitArray* StatementList::FindAssignment(LeftHandSideArray<vec>* pLHS)
{
	StatementList *pStmtList;
	StmtInitArray *pAssign;
	
	int count = this->stmtindex == 0 ? 
		int(this->stmtlist.size()) : this->stmtindex;

	for(int i = count-1; i>=0; --i)
	{
		SET_USER_ABORT;

		if(stmtlist[i]) 
		{
			if(stmtlist[i]->stmttype==StmtExpr) 
			{
				pAssign = dynamic_cast<StmtInitArray*>(stmtlist[i]);

				if(pAssign)
				{
					if(pAssign->IsLHSEqual<vec>(pLHS)) 
						return pAssign;
					else continue;
				}
			}
			
			pStmtList = dynamic_cast<StatementList*>(stmtlist[i]);

			if(pStmtList)
			{
				pAssign = pStmtList->FindAssignment<vec>(pLHS);
				if(pAssign) return pAssign;
			}
		}
	}

	return NULL;
}

template<typename T>
AssignmentLHS<T>* FindAssignment(LeftHandSide<T> *lhs)
{
	extern Stack<StatementList*> gStmtListStack;
	StatementList *pStatementList; 
	AssignmentLHS<T> *pStmt;
	
	int count = gStmtListStack.size();
	for(int i=0; i<count; ++i)
	{
		SET_USER_ABORT;

		pStatementList = gStmtListStack.getoffset(i);
		pStmt = pStatementList->FindAssignment<T>(lhs);
		if(pStmt) return pStmt;
	} 
	return NULL;
}

template<typename vec>
StmtInitArray* FindAssignment(LeftHandSideArray<vec> *lhs)
{
	extern Stack<StatementList*> gStmtListStack;
	StatementList *pStatementList; 
	StmtInitArray *pStmt;
	
	int count = gStmtListStack.size();
	for(int i=0; i<count; ++i)
	{
		SET_USER_ABORT;

		pStatementList = gStmtListStack.getoffset(i);
		pStmt = pStatementList->FindAssignment<vec>(lhs);
		if(pStmt) return pStmt;
	} 
	return NULL;
}