#pragma once

#include <string>
#include <hash_map>

class UnicodeToken
{
public:
	typedef std::pair<std::string, std::wstring> TokenPair;
	typedef stdext::hash_map<std::string, std::wstring> TokenMap;
	typedef TokenMap::iterator TokenMapItr;

public:
	TokenMap theMap;

	UnicodeToken() {}
	virtual ~UnicodeToken() {}

	void InsertToken(const char* ascii, const wchar_t* unicode);
	std::wstring UnicodeToken::TranslateToken(const char* ascii);
};

enum KeywordID 
{
	keyword_null = 0,

	keyword_fmt_clear, 
	keyword_fmt_precision,
	keyword_fmt_scientific,
	keyword_fmt_fixed,
	keyword_fmt_showpoint, 
	keyword_fmt_hidepoint,
	keyword_fmt_showthousand,
	keyword_fmt_hidethousand,

	keyword_exit,
	keyword_class,
	keyword_extends,

	keyword_new,
	keyword_delete,

	keyword_try,
	keyword_catch,
	keyword_throw,

	keyword_include,
	keyword_file,
	keyword_cout,
	keyword_cin,
	keyword_true,
	keyword_false,
	keyword_clear,
	keyword_endl,
	keyword_uvctr,
	keyword_ucplx,
	keyword_static,
	keyword_share,
	keyword_declare,
	keyword_select,
	keyword_choice,
	keyword_always,

	keyword_if,
	keyword_if_true,
	keyword_if_false,
	keyword_else,
	keyword_else_true,
	keyword_else_false,

	keyword_do,
	keyword_for,
	keyword_while,
	keyword_goto,
	keyword_break,
	keyword_continue,
	
	keyword_literal,

	keyword_strn,

	keyword_void,
	keyword_bool,
	keyword_integer,
	keyword_real,
	keyword_complex,
	keyword_vector,
	keyword_return,
	
	keyword_end,

	opr_real,
	opr_imag,

	opr_xcom,
	opr_ycom,
	opr_zcom,

	opr_dimension,
	opr_size, 
	opr_resize,
	
	opr_detcramer,
	opr_solvecramer,
	opr_gaussjordan,
	opr_ludecompo,
	opr_lumethod,
	opr_inverse,

	opr_end
};

class TransKeywords
{
public: // type declaration

	typedef stdext::hash_map<std::wstring, KeywordID> KeywordMap;
	typedef KeywordMap::iterator KeywordMapItr;
	typedef KeywordMap::const_iterator KeywordMapCItr;
	typedef std::pair<std::wstring, KeywordID> KeywordPair;
	
protected:

	KeywordMap keywordmap;

public:
	void InsertKeyword(const wchar_t* keyword, KeywordID keyid);
	KeywordID GetKeywordID(const wchar_t* keyword);
	KeywordID GetKeywordID(const std::wstring& keyword);
	const KeywordMap& GetKeywordMap() const;
};
