#pragma once

#include <kch_basetype.h>

#include "resource.h"

class UserDefinedType;
class usertype;

template<> class DataTemp<usertype>;

template<> class PtrData<DataTemp<usertype> >;

class usertype: public BaseType
{
protected:
	
	std::wstring typetag;
	void clear();

	bool CheckAllocated(const usertype& ut)
	{
		if(udtPtr && udtPtr==ut.udtPtr)
			return elemap.size()==ut.elemap.size();
		
		return false;
	}
public:

	void Copy(const usertype& ut);
	
	usertype* AllocateCopy() const;

	std::deque<DataTemp<usertype>*> udtvec;

	UsrtMap elemap; UserDefinedType *udtPtr;
	
	std::wstring& GetTypeTag() { return typetag; }

	void SetID(const std::wstring& id) { typetag = id; }
	void FastCopy(const usertype* utptr);

	DataTemp<usertype>* FindUserData(const std::wstring& id);

	void InsertElement(UsrtPair ele);
	void ClearEleMap();
	
	usertype(std::wstring tag=L"No Name: usertype");
	usertype(const usertype& ut);
	usertype& operator=(const usertype& ut);
		
	UdtPair* GetElement(const std::wstring& id);
	void Swap(usertype *ptr);
	virtual ~usertype();
};

template<> class DataTemp<usertype>;

class UserDefinedType: public BaseType
{
protected:
	std::wstring typetag; // user defined class tag
	NameUdtMap udtmap; // for class
	NameUdtMap elemap;	// for element class
	UDFuncList funclist; 
	NameDataMap datamap; // for literal data type
	
public:
	UserDefinedType *pEnclosingUDT;

	TypeMap typemap; 
	UserDefinedType(const std::wstring& tag=L"User Defined Type");

	void CheckImplemented() const;
	
	Function* FindFunction(const std::wstring& id);
	void AddUDFunction(const wchar_t* id, Function* fptr);
	
	const std::wstring& GetTypeTag() const;
	void InsertElement(TypePair tpr);
	
	void AddFactory(UserDefinedType *udt);
	
	void InsertElement(UserDefinedType *udt);
	void InsertElement(UserDefinedType *udt, std::wstring name);
	void InsertElementUPtr(UserDefinedType *udt, std::wstring name);
	void InsertElement(Data* ptr, DataType type, std::wstring name);
	UserDefinedType* GetUdt(const std::wstring& name);
	UserDefinedType* GetEleUdt(const std::wstring& name);
	Data* GetData(const std::wstring& name);
	DataType GetElementType(const std::wstring& id);
	bool CheckRecursive(UserDefinedType *const ptr) const;
	
	bool IsLiteral();
	usertype* AllocateMemory(); 
	void AllocateMemory(usertype* const uptr);
	
	virtual ~UserDefinedType();
};

template<>
class DataTemp<usertype>: public Data
{
protected:
	PtrStack<usertype> theStack;
	
	void copy(const DataTemp& data)
	{
		if(factory==NULL)
		{
			factory = data.factory; 
			usertype *utptr = data.theStack.top()->AllocateCopy();
			utptr->SetID(Data::datasig.second); this->Push(utptr);
		}
		else
		{
			if(theStack.empty())
			{
				usertype *utptr = data.theStack.top()->AllocateCopy();
				utptr->SetID(Data::datasig.second); this->Push(utptr);
			}
			else 
				theStack.top()->FastCopy(data.theStack.top());
		}
	}

	void copy(const usertype& data)
	{
		if(factory==NULL)
		{
			factory = data.udtPtr; 
			usertype *utptr = data.AllocateCopy();
			utptr->SetID(Data::datasig.second); this->Push(utptr);
		}
		else
		{
			if(theStack.empty()) 
			{
				usertype *utptr = data.AllocateCopy();
				utptr->SetID(Data::datasig.second); this->Push(utptr);
			}
			else theStack.top()->FastCopy(&data);
		}
	}
public:
	UserDefinedType *factory;
	DataTemp(std::wstring varid=L"Temporary UserData", 
		UserDefinedType *ftry=NULL, bool vstatic=false): 
		factory(ftry),

#ifdef _DEBUG 
			theStack(varid)
#else
			theStack()
#endif
	  {	
		  Data::datasig.first = TypeUser;
		  Data::datasig.second = varid;
		    Data::static_data = vstatic;

		  if(vstatic) Push(); 
	  }

	void Swap(DataTemp<usertype>* pData)
	{
		theStack.swaptop(&pData->theStack);
	}

	DataTemp(const DataTemp& data) 
	{ 
		factory = NULL; Data::datasig.first = TypeUser;
		Data::datasig.second = L"Temporary UserData"; 
		copy(data); 
	}
	
	DataTemp& operator=(const DataTemp& data) 
	{ 
		if(this!=&data)
		{
		  if(factory && factory != data.factory)
			HandleSyntaxError(L"User defined data type mismatch. Can not copy.\r\n");
		  else
		  {
			  copy(data); return *this;
		  }
		}
		return *this;  
	}

	void Copy(const DataTemp& data) 
	{ 
		if(this!=&data)
		{
		  if(factory && factory != data.factory)
			HandleSyntaxError(L"User defined data type mismatch. Can not copy.\r\n");
		  else copy(data); 
		}
	}

	usertype& GetData() 
	{ 
		return *theStack.top(); 
	}
	usertype* GetPtr() 
	{ 
		return  theStack.top(); 
	}

	operator usertype& () { return GetData(); }
	
	usertype& operator=(const usertype& rhs) 
	{
		if(factory && factory != rhs.udtPtr)
			HandleSyntaxError(L"User defined data type mismatch. Can not copy.\r\n");
		else copy(rhs); 

		return *theStack.top(); 
	}

	//DataTemp& operator=(const DataTemp& rhs) 
	//{ *theStack.top() = *rhs.theStack.top(); return *this; }

	virtual void Push() 
	{ 
		usertype *ptr = factory->AllocateMemory();
		ptr->SetID(Data::datasig.second);
		theStack.push(ptr); 
	}

	virtual void Push(usertype* ptr) { theStack.push(ptr); }

	/*
	DataTemp* Clone(std::wstring id) const
	{
		if(factory==NULL)
			HandleSyntaxError(L"DataTemp<usertype>::Clone(), factory is null.\r\n");

		DataTemp *dtPtr = new DataTemp(id,factory);

		dtPtr->Push(theStack.top()->AllocateCopy());
				
		return dtPtr;
	} */

	virtual void Pop() { delete theStack.pop(); }

	virtual ~DataTemp() 
	{  

#ifdef _DEBUG 
		
		if(!theStack.empty() && !Data::static_data)
		{
			if(Data::datasig.second != std::wstring(L"Temporary UserData"))
			{
				std::wostringstream os;
				os<<L"DataTemp Stack for \'"<<datasig.second
					<<L"\' is not properly cleaned. Stack Depth: "<<int(theStack.size())<<L"\r\n";
				PutString(os.str());
			}
		}
#endif
		theStack.clear(); 
	}
};

typedef DataTemp<usertype> UserData;

template<>
class PtrData<UserData>: public PtrDataBase
{
protected:
	PtrStack<UserData*> theStack;
	
	void copy(const PtrData& rhs)
	{
		if(factory==NULL)
		{
			this->factory = rhs.factory; this->Push(); 
			*theStack.top() = rhs.GetData();
		}
		else
		{
			if(theStack.empty()) this->Push();
			*theStack.top() = rhs.GetData();
		}
	}

public:
	UserDefinedType *factory;
	
	PtrData(std::wstring vrid=L"Temporary PtrData<UserType>", 
		UserDefinedType *ftry=NULL, bool vstatic=false): 
	  PtrDataBase(vrid, vstatic), factory(ftry), 

#ifdef _DEBUG
	  theStack(vrid)
#else
	  theStack()
#endif
	  { 
		  if(vstatic) Push(); 
		  eletype = TypeUPtr;
	  }

   void Swap(PtrData* pData)
	{
		UserData** lhs = this->theStack.top();
		UserData** rhs = pData->theStack.top();
		UserData* value = *lhs; *lhs = *rhs; *rhs = value;
	}

    PtrData(const PtrData& rhs)
	{
		if(const_cast<PtrData*>(&rhs)->GetDataPtr())
		{
			this->factory = NULL; Data::datasig.first = TypePtr;
			Data::datasig.second = L"Temporary PtrData<UserData>";
			eletype = TypeUPtr; copy(rhs);
		}
	}

	PtrData& operator = (const PtrData& rhs)
	{
		if(this != &rhs)
		{
			if(factory && factory != rhs.factory)
				HandleSyntaxError(L"The pointer to user defined data is of type mismatch.\r\n");
			else
			{
				copy(rhs); return *this;
			}
		}
		
		return *this;
	}

	UserData* GetData() const
	{ 		
		if(!theStack.empty() && *theStack.top())
			return *theStack.top();
		else
		{
			std::wostringstream os;
			os<<L"Pointer type variable "
				<<Data::GetId()<<L" is not yet initialized.\r\n";

			HandleSyntaxError(os.str());

			return *theStack.top(); // to suppress warning
		}
	}

	UserData* GetDataPtr() 
	{ 		
		if(!theStack.empty() && *theStack.top())
			return *theStack.top();

		else return NULL;
	}

	UserData*& GetData() 
	{ 		
		if(!theStack.empty() && *theStack.top())
			return *theStack.top();
		else
		{
			std::wostringstream os;
			os<<L"Pointer type variable "
				<<Data::GetId()<<L" is not yet initialized.\r\n";
			
			HandleSyntaxError(os.str());

			return *theStack.top() ; // to suppress warning
		}
	}
	
	UserData** GetPtr() 
	{ 
		if(!theStack.empty())
			return theStack.top();
		else
		{
			CString msg, fmt, vid(Data::GetId().c_str());
			fmt.LoadStringW(IDS_PTR_NOT_INITIALIZED);
			msg.FormatMessageW(fmt, vid); 
			HandleSyntaxError(msg.GetString());

			return NULL; // to suppress warning;
		}
	}
	
	virtual void Push() 
	{ 
		UserData **pptr = new UserData*;
		theStack.push(pptr);
		*pptr = (UserData*)NULL;
	}

	virtual void Pop() 
	{ 
		UserData** pptr = theStack.pop(); 
		delete pptr; 
	}
	
	virtual ~PtrData() 
	{  

#ifdef _DEBUG 
		
		if(!theStack.empty() && !Data::static_data)
		{
			std::wostringstream os;
			os<<L"PtData Stack for \'"<<datasig.second
				<<L"\' is not properly cleaned. Stack Depth: "<<int(theStack.size())<<L"\r\n";
			PutString(os.str());
		}
#endif
		theStack.clear();
	}
};

typedef PtrData<UserData> UPtrData;

class MemoryManager
{
protected:

	MemMap memmap;

public:
	
	MemoryManager();

	void Clear();
		
	void Release(void* ptr);
	
	void* CreateMemory(DataType type);

	UserData* AllocateUserData(UserDefinedType *udtPtr);

	virtual ~MemoryManager();
};

extern MemoryManager gMM;

template<typename T> T* AllocateMemory()
{
	const type_info& type = typeid(T);

	if(type == typeid(bool))			return (T*) gMM.CreateMemory(TypeBool);
	else if(type == typeid(string))		return (T*) gMM.CreateMemory(TypeStrn);
	else if(type == typeid(integer))	return (T*) gMM.CreateMemory(TypeIntg);
	else if(type == typeid(realnum))	return (T*) gMM.CreateMemory(TypeReal);
	else if(type == typeid(Complex))	return (T*) gMM.CreateMemory(TypeCplx);
	else if(type == typeid(Vector))		return (T*) gMM.CreateMemory(TypeVctr);
	
	else if(type == typeid(IntgVec1D))	return (T*) gMM.CreateMemory(TypeAi1D);
	else if(type == typeid(RealVec1D))	return (T*) gMM.CreateMemory(TypeAr1D);
	else if(type == typeid(CplxVec1D))	return (T*) gMM.CreateMemory(TypeAc1D);
	else if(type == typeid(VctrVec1D))	return (T*) gMM.CreateMemory(TypeAv1D);

	else if(type == typeid(IntgVec2D))	return (T*) gMM.CreateMemory(TypeAi2D);
	else if(type == typeid(RealVec2D))	return (T*) gMM.CreateMemory(TypeAr2D);
	else if(type == typeid(CplxVec2D))	return (T*) gMM.CreateMemory(TypeAc2D);
	else if(type == typeid(VctrVec2D))	return (T*) gMM.CreateMemory(TypeAv2D);
	
	else 
		HandleSyntaxError(L"Unsupported dynamic allocation type.\r\n");

	return NULL; // to suppress warning
}

UserData* AllocateUserData(UserDefinedType *udtPtr);
