#pragma once

#include <afxwin.h>
#include <hash_map>
#include <set>
#include <vector>
#include <deque>
#include <exception>
#include <string>
#include <memory>
#include <iomanip>
#include <ios>
#include <eh.h>
#include <wchar.h>
#include <limits>
#include <iostream>
#include <sstream>

#include <kch_cmath.h>
#include <kch_basetype.h>
#include <kch_autodeletestr.h>
#include <kch_complex.h>
#include <kch_array.h>
#include <kch_theparser.h>
#include <kch_userdefinedtype.h>

#include "resource.h"
#include "EditInput.h"
#include "InputDialogBox.h"

#define MAX_ARGCOUNT  80
#define SOL_SECTIONS  111

extern bool static_set;
extern bool gRuntime;
extern Array *gArrayPtr;
extern Data* gDataPtr;

extern __declspec(align(32)) volatile LONG gBreak;
extern Stack<BaseType*> gArgStack;

//class RealData;

extern std::string TrimStr(const char* str);

template<typename T> class LeftHandSide;

extern bool SolveBisection(SolSet& solset, ExpReal* pExpr, 
					LeftHandSide<realnum> *pVar, realnum x1, realnum x2, realnum epsilon);
extern void MaxMinBisection(SolSet& solset, ExpReal *pExpr,
					LeftHandSide<realnum> *pVar, realnum x1, realnum x2, realnum epsilon);
extern bool SolveNewton(SolSet& solset, ExpReal* pExpr,
					LeftHandSide<realnum> *pVar, realnum x1, realnum x2, realnum epsilon);
extern void MaxMinNewton(SolSet& solset, ExpReal *pExpr,
					LeftHandSide<realnum> *pVar, realnum x1, realnum x2, realnum epsilon);

#define EVAL_XPR(pVar, x, rtn, pXpr)  pVar->GetData() = (x);  rtn = pXpr->Eval(); 
#define EVAL_EPS(pVar, x, rtn, pXpr)  { realnum y1_, y2_;  \
										pVar->GetData() = (x)-epsilon; \
										y1_ = pXpr->Eval(); \
										pVar->GetData() = (x)+epsilon; \
										y2_ = pXpr->Eval(); \
										if(fabs(y1_-y2_)<epsilon) rtn = (y1_+y2_)/2.0 ; \
										else return false; \
										}



class VoidData: public Data
{
protected:
	voidtype data;
public:
	VoidData(std::wstring varid=_T(""), bool varstatic=false);
	voidtype& GetData();
	virtual ~VoidData();
};

template<typename T> class LiteralData : public Data
{
protected:
	Expression<T> *ptr; 
	
public:
	LiteralData(std::wstring varid, Expression<T>* expr): ptr(expr), Data(varid)
	{
		const type_info& type = typeid(T);
		if(type == typeid(integer)) Data::datasig.first = TypeIntgL;
		else if(type == typeid(realnum)) Data::datasig.first = TypeRealL;
		else if(type == typeid(Complex)) Data::datasig.first = TypeCplxL;
		else if(type == typeid(Vector)) Data::datasig.first = TypeVctrL;
		else if(type == typeid(string)) Data::datasig.first = TypeStrnL;
		else HandleSyntaxError(_T("Unsupported literal type.\r\n"));

		if(expr->bLiteral)
		{
			expr->Eval();
		}
		else HandleSyntaxError(_T("Cannot initialize literal members with an expression with variable(s).\r\n"));
	}
	
	Expression<T>* GetExpression() { return ptr; }

	//const T& GetData() const { return value; }
	
	virtual ~LiteralData() { }
};

/*
class PtrDataBase: public Data
{
protected:
	DataType eletype;
public:
	PtrDataBase(std::wstring vrid, bool vstatic):
	  Data(vrid, TypePtr, vstatic) { }

	DataType GetEleType() const { return eletype; }
	virtual ~PtrDataBase() {}
};

template<typename T>
class PtrData: public PtrDataBase
{
protected:
	PtrStack<T*> theStack;
	
public:
	PtrData(std::wstring vrid, bool vstatic=false): 
	  PtrDataBase(vrid, vstatic),

#ifdef _DEBUG
	  theStack(vrid)
#else
	  theStack()
#endif
	  { 
		  if(vstatic) Push(); 
		  const type_info& type=typeid(T);

		  if(type==typeid(bool))		 eletype = TypeBool;
		  else if(type==typeid(integer)) eletype = TypeIntg;
		  else if(type==typeid(realnum)) eletype = TypeReal;
		  else if(type==typeid(Complex)) eletype = TypeCplx;
		  else if(type==typeid(Vector))  eletype = TypeVctr;
		  else if(type==typeid(string))  eletype = TypeStrn;
		  else HandleSyntaxError(_T("Unsupported pointer data type.\r\n"));
	  }

   void Swap(PtrData* pData)
	{
		T** lhs = this->theStack.top();
		T** rhs = pData->theStack.top();
		T* value = *lhs; *lhs = *rhs; *rhs = value;
	}

	PtrData& operator = (PtrData& ptr)
	{
		*this->theStack.top() = *ptr.theStack.top();
		return *this;
	}

	T* GetData() 
	{ 		
		if(*theStack.top())
			return *theStack.top();
		else
		{
			CString msg, fmt, vid(Data::GetId().c_str());
			fmt.LoadStringW(IDS_PTR_NOT_INITIALIZED);
			msg.FormatMessageW(fmt, vid);
			PutString(msg);
			throw UnicodeException(_T(""));
		}
	}
	
	T** GetPtr() 
	{ 
		if(!theStack.empty())
			return theStack.top();
		else
		{
			CString msg, fmt, vid(Data::GetId().c_str());
			fmt.LoadStringW(IDS_PTR_NOT_INITIALIZED);
			msg.FormatMessageW(fmt, vid);
			PutString(msg);
			throw UnicodeException(_T(""));
		}
	}
	
	virtual void Push() 
	{ theStack.push(new T*); *theStack.top() = (T*)NULL; }
	virtual void Pop() 
	{ T** pptr = theStack.pop(); delete pptr; }
	
	virtual ~PtrData() 
	{  

#ifdef _DEBUG 
		
		if(!theStack.empty() && !Data::static_data)
		{
			std::wostringstream os;
			os<<L"PtData Stack for \'"<<datasig.second
				<<L"\' is not properly cleaned. Stack Depth: "<<int(theStack.size())<<L"\r\n";
			PutString(os.str());
		}
#endif

		theStack.clear();
	}
};

*/

class AptrDataBase: public Data
{
public:
	DataType eletype;

	AptrDataBase(const type_info& type, std::wstring vrid, bool vstatic);
	virtual ~AptrDataBase(){}
};

template<typename vec>
class AptrData: public AptrDataBase
{
protected:
	PtrStack<vec*> theStack;

public:
	
	AptrData(std::wstring vrid, bool vstatic=false): 
	  AptrDataBase(typeid(vec), vrid, vstatic),

#ifdef _DEBUG
		 theStack(vrid) 
#else
		 theStack() 
#endif
	  { if(vstatic) Push(); }
	
    void Swap(AptrData* pData)
	{
		vec** lhs = this->theStack.top();
		vec** rhs = pData->theStack.top();
		vec* value = *lhs; *lhs = *rhs; *rhs = value;
	}

	vec* GetData() 
	{ 		
		if(!theStack.empty() && *theStack.top())
			return *theStack.top();
		else
		{
			CString msg, fmt, vid(Data::GetId().c_str());
			fmt.LoadStringW(IDS_PTR_NOT_INITIALIZED);
			msg.FormatMessageW(fmt, vid); HandleSyntaxError(msg.GetString()); 
			
			return NULL; // to suppress warning
		}
	}

	vec* GetData() const
	{ 		
		if(!theStack.empty() && *theStack.top())
			return *theStack.top();
		else
		{
			CString msg, fmt, vid(Data::GetId().c_str());
			fmt.LoadStringW(IDS_PTR_NOT_INITIALIZED);
			msg.FormatMessageW(fmt, vid); 
			
			HandleSyntaxError(msg.GetString()); 
			
			return NULL; // to suppress warning
		}
	}

	vec* GetDataPtr()
	{ 		
		if(!theStack.empty() && *theStack.top())
			return *theStack.top();
		else return NULL;
	}

	vec** GetPtr() 
	{
		if(!theStack.empty())
			return theStack.top(); 
		else
		{
			CString msg, fmt, vid(Data::GetId().c_str());
			fmt.LoadStringW(IDS_PTR_NOT_INITIALIZED);
			msg.FormatMessageW(fmt, vid); HandleSyntaxError(msg.GetString()); 
			
			return (vec**) NULL; // to suppress warning
		}
	}

	virtual void Push() 
	{  theStack.push(new vec*); *theStack.top() = (vec*)NULL; }
	virtual void Pop() 
	{  vec** pptr = theStack.pop(); delete pptr; }
	
	virtual ~AptrData() 
	{	while(!theStack.empty()) this->Pop(); }
};

extern DataType LookupDataType(const std::wstring id);
extern Data* LookupData(const std::wstring id);
extern bool LookupLoop();

class VarList
{
public:
	typedef std::pair<const std::wstring, int> IndexPair;
	typedef stdext::hash_map<const std::wstring, int>::iterator IndexItr;

protected:
	stdext::hash_map<const std::wstring, int> varindex;
	ptrvector<Data> varlist; int pushcount;
	void copy(const VarList& vl)
	{
		this->varindex = vl.varindex;
		this->varlist = vl.varlist;
	}
public:

	VarList();
	int Push();
	int Pop();
	int GetStackDepth();

	void PreserveData() { varlist.SetPreserve(); }
	int size();
	int insert(Data* pData);
	void put(Data* pData);
	Data* get(int index);
	
	DataTypeVec GetDataTypeVec() const;
	DataPairVec GetDataPairVec() const;

	Data* operator[](const std::wstring& id);
	Data* at(const std::wstring& id);
	DataType GetType(const std::wstring& id);
	DataType GetType(const std::wstring& id, Data** pData);
		
	Data* at(int index);
	void Delete();
	void PrintElement();

	VarList(const VarList& vl) { copy(vl); }

	VarList& operator=(const VarList& vl) { copy(vl); return *this; }
};

class ExpTypeless: public BaseType
{
protected:
	
public:
	bool bLiteral; DataType datatype;

	ExpTypeless(DataType type=TypeNull): datatype(type), bLiteral(false) {}
	
	virtual std::wstring Display() =0;
	virtual ~ExpTypeless() {}
};

class ExpEndLine: public ExpTypeless
{

public:
	
	ExpEndLine() { }

	//virtual bool EvalBool() {return false; }
	virtual std::wstring Display() { return std::wstring(); }
	virtual ~ExpEndLine() {}
};

template<typename T>
class Expression: public ExpTypeless
{

public:
	Expression() 
	{
		const type_info& type = typeid(T);

		if(type == typeid(bool))         ExpTypeless::datatype = TypeBool;
		else if(type == typeid(integer)) ExpTypeless::datatype = TypeIntg;
		else if(type == typeid(Complex)) ExpTypeless::datatype = TypeCplx;
		else if(type == typeid(Vector))  ExpTypeless::datatype = TypeVctr;
		else if(type == typeid(string))  ExpTypeless::datatype = TypeStrn;
		else if(type == typeid(usertype))  ExpTypeless::datatype = TypeUser;
		else ExpTypeless::datatype = TypeNull;
	}
	
	virtual T Eval() = 0;
			//{ throw BaseType::BaseException(IDS_EVAL_NOT_SUPPORTED); }
	
	virtual void GetVariables(DataSet& dataset) {  }
	
	virtual std::wstring Display() { return std::wstring(_T("")); };
	virtual ~Expression() { }
};

template<>
class Expression<realnum>: public ExpTypeless
{

public:
	Expression() { ExpTypeless::datatype = TypeReal; }
	
	virtual realnum Eval() = 0;
			//{ throw BaseType::BaseException(IDS_EVAL_NOT_SUPPORTED); }
	
	virtual void GetVariables(DataSet& dataset) {  }

	virtual realnum EvalPlus()  { return this->Eval()+1E6; }
	virtual realnum EvalMinus() { return this->Eval()-1E6; }

	virtual std::wstring Display() { return std::wstring(_T("")); };
	virtual ~Expression() { }
};

typedef Expression<voidtype> ExpVoid, *ExpVoidPtr;
typedef Expression<bool>	 ExpBool, *ExpBoolPtr;
typedef Expression<integer>  ExpIntg, *ExpIntgPtr;
typedef Expression<realnum>  ExpReal, *ExpRealPtr;
typedef Expression<Complex>  ExpCplx, *ExpCplxPtr;
typedef Expression<Vector>   ExpVctr, *ExpVctrPtr;
typedef Expression<string>   ExpStrn, *ExpStrnPtr;
typedef Expression<UserData> ExpUser, *ExpUserPtr;

typedef Expression<bool*>	  ExpPBool, *ExpPBoolPtr;
typedef Expression<integer*>  ExpPIntg, *ExpPIntgPtr;
typedef Expression<realnum*>  ExpPReal, *ExpPRealPtr;
typedef Expression<Complex*>  ExpPCplx, *ExpPCplxPtr;
typedef Expression<Vector*>   ExpPVctr, *ExpPVctrPtr;
typedef Expression<string*>   ExpPStrn, *ExpPStrnPtr;
typedef Expression<UserData*> ExpPUser, *ExpPUserPtr;

template<typename T>
class DynamicAllocation: public Expression<T*>
{
protected:
	Expression<T> *pExpr;
public:
	DynamicAllocation(Expression<T> *xpr=NULL): pExpr(xpr) { }
	virtual T* Eval() 
	{
		T* ptr = AllocateMemory<T>();
		
		if(pExpr) *ptr = pExpr->Eval();

		return ptr;
	}

	virtual ~DynamicAllocation() { }
};


template<>
class DynamicAllocation<UserData>: public Expression<UserData*>
{
protected:
	Expression<UserData> *pExpr; UserDefinedType *udtPtr;
public:
	DynamicAllocation(UserDefinedType *uptr, Expression<UserData> *xpr=NULL): 
	  udtPtr(uptr), pExpr(xpr) { }
	virtual UserData* Eval() 
	{
		UserData* ptr = AllocateUserData(udtPtr);
		
		if(pExpr) *ptr = pExpr->Eval();

		return ptr;
	}

	virtual ~DynamicAllocation() { }
};


template<typename T, typename ptrdata>
class PtrDataToPtr: public Expression<T*>
{
protected:
	ptrdata *pPtrData;

public:
	PtrDataToPtr(ptrdata *pdata): pPtrData(pdata) {}

	virtual T* Eval() { return *pPtrData->GetPtr(); }

	virtual std::wstring Display() 
		{ return std::wstring(_T("")); };

	virtual ~PtrDataToPtr() {}
};

template<typename T>
class LeftHandSide: public Expression<T>
{
public:
	LeftHandSide() {}

	virtual const std::wstring& GetId() const = 0;

	virtual T& GetData() = 0;
	
	virtual std::wstring GetFullName() const
	{
		throw UnicodeException(L"LeftHandSide<T>::GetFullName(), It should not be called.\r\n");	
	}

	virtual ~LeftHandSide() {}
};

template<>
class LeftHandSide<UserData>: public Expression<UserData>
{
public:
	LeftHandSide() { ExpTypeless::datatype = TypeUser; }

	virtual UserData& GetData() = 0;
	virtual UserData* GetUserData() = 0;
	virtual UserDefinedType* GetUdt() = 0;
	//virtual StrArray& GetMemArray() = 0;
	virtual void InsertMemberID(const std::wstring& memid) = 0;
	virtual std::wstring GetFullName() const = 0;
	virtual bool IsCompatible(LeftHandSide<UserData> *ptr) = 0;
	virtual bool IsCompatible(UserDefinedType *ptr) = 0;
	virtual std::wstring Display() { return std::wstring(L""); }

	virtual ~LeftHandSide() {}
};

template<>
class LeftHandSide<stream&>: public Expression<stream&>
{
public:
	LeftHandSide() {}

	virtual const std::wstring& GetId() const = 0;
	virtual stream& GetData() = 0;
	virtual std::wstring GetFullName() const 
	{ 
		throw UnicodeException(L"LeftHandSide<stream&>::GetFullName(), It should not be called.\r\n");
	}

	virtual ~LeftHandSide() {}
};

typedef LeftHandSide<bool>     LHSBool, *LHSBoolPtr;
typedef LeftHandSide<integer>  LHSIntg, *LHSIntgPtr;
typedef LeftHandSide<realnum>  LHSReal, *LHSRealPtr;
typedef LeftHandSide<Complex>  LHSCplx, *LHSCplxPtr;
typedef LeftHandSide<Vector>   LHSVctr, *LHSVctrPtr;
typedef LeftHandSide<string>   LHSStrn, *LHSStrnPtr;
typedef LeftHandSide<UserData> LHSUser, *LHSUserPtr;

typedef LeftHandSide<bool*>     LHSPBool, *LHSPBoolPtr;
typedef LeftHandSide<integer*>  LHSPIntg, *LHSPIntgPtr;
typedef LeftHandSide<realnum*>  LHSPReal, *LHSPRealPtr;
typedef LeftHandSide<Complex*>  LHSPCplx, *LHSPCplxPtr;
typedef LeftHandSide<Vector*>   LHSPVctr, *LHSPVctrPtr;
typedef LeftHandSide<string*>   LHSPStrn, *LHSPStrnPtr;
typedef LeftHandSide<UserData*> LHSPUser, *LHSPUserPtr;

template<typename T>
class Increment: public Expression<T>
{
protected:
	LeftHandSide<T> *pLHS;
public:

	Increment(LeftHandSide<T>* lhs): pLHS(lhs) { }
	
	virtual T Eval() 
		{	T value = pLHS->Eval(); return ++value; }

	virtual ~Increment() {}
};

class LHSArrayBase
{
public:
	LHSArrayBase() {}

	virtual int GetDimension(DataType& type)  = 0;
	
	virtual ~LHSArrayBase() {}
};

template <typename vec> class LeftHandSideArray
: public LHSArrayBase, public LeftHandSide<vec>
{
public:
	LeftHandSideArray() {}

	virtual int GetDimension(DataType& type) 
	{
		const type_info& typeinfo = typeid(vec);
		
		if(typeinfo == typeid(IntgVec1D))
		{
			type = TypeAi1D; return 1;
		}
		else if(typeinfo == typeid(RealVec1D))
		{
			type = TypeAr1D; return 1;
		}
		else if(typeinfo == typeid(CplxVec1D))
		{
			type = TypeAc1D; return 1;
		}
		else if(typeinfo == typeid(VctrVec1D))
		{
			type = TypeAv1D; return 1;
		}
		else if(typeinfo == typeid(IntgVec2D))
		{
			type = TypeAi2D; return 2;
		}
		else if(typeinfo == typeid(RealVec2D))
		{
			type = TypeAr2D; return 2;
		}
		else if(typeinfo == typeid(CplxVec2D))
		{
			type = TypeAc2D; return 2;
		}
		else if(typeinfo == typeid(VctrVec2D))
		{
			type = TypeAv2D; return 2;
		}
		else if(typeinfo == typeid(IntgVec3D))
		{
			type = TypeAi3D; return 3;
		}
		else if(typeinfo == typeid(RealVec3D))
		{
			type = TypeAr3D; return 3;
		}
		else if(typeinfo == typeid(CplxVec3D))
		{
			type = TypeAc3D; return 3;
		}
		else if(typeinfo == typeid(VctrVec3D))
		{
			type = TypeAv3D; return 3;
		}
		else if(typeinfo == typeid(IntgVec4D))
		{
			type = TypeAi4D; return 4;
		}
		else if(typeinfo == typeid(RealVec4D))
		{
			type = TypeAr4D; return 4;
		}
		else if(typeinfo == typeid(CplxVec4D))
		{
			type = TypeAc4D; return 4;
		}
		else if(typeinfo == typeid(VctrVec4D))
		{
			type = TypeAv4D; return 4;
		}
		else if(typeinfo == typeid(IntgVec5D))
		{
			type = TypeAi5D; return 5;
		}
		else if(typeinfo == typeid(RealVec5D))
		{
			type = TypeAr5D; return 5;
		}
		else if(typeinfo == typeid(CplxVec5D))
		{
			type = TypeAc5D; return 5;
		}
		else if(typeinfo == typeid(VctrVec5D))
		{
			type = TypeAv5D; return 5;
		}
		else
			throw UnicodeException(L"Invalid array type.\r\n");
	}
	
	virtual ~LeftHandSideArray(){}
};

class ArrayResizeBase;
class StmtInitArray;

typedef struct
{
	int index[MAX_DIMENSION];
	int maxidx[MAX_DIMENSION];
	int maxdim;

	int i, j, k, dim;

	int arraydim; DataType type;
	
	ArrayResizeBase *pArrayResizePtr;
	StmtInitArray *pStmtInit;

	union
	{
		void *VoidPtr;

		LeftHandSideArray<IntgVec1D>* pIntgVec1D;
		LeftHandSideArray<RealVec1D>* pRealVec1D;
		LeftHandSideArray<CplxVec1D>* pCplxVec1D;
		LeftHandSideArray<VctrVec1D>* pVctrVec1D;

		LeftHandSideArray<IntgVec2D>* pIntgVec2D;
		LeftHandSideArray<RealVec2D>* pRealVec2D;
		LeftHandSideArray<CplxVec2D>* pCplxVec2D;
		LeftHandSideArray<VctrVec2D>* pVctrVec2D;

		LeftHandSideArray<IntgVec3D>* pIntgVec3D;
		LeftHandSideArray<RealVec3D>* pRealVec3D;
		LeftHandSideArray<CplxVec3D>* pCplxVec3D;
		LeftHandSideArray<VctrVec3D>* pVctrVec3D;

		LeftHandSideArray<IntgVec4D>* pIntgVec4D;
		LeftHandSideArray<RealVec4D>* pRealVec4D;
		LeftHandSideArray<CplxVec4D>* pCplxVec4D;
		LeftHandSideArray<VctrVec4D>* pVctrVec4D;

		LeftHandSideArray<IntgVec5D>* pIntgVec5D;
		LeftHandSideArray<RealVec5D>* pRealVec5D;
		LeftHandSideArray<CplxVec5D>* pCplxVec5D;
		LeftHandSideArray<VctrVec5D>* pVctrVec5D;
	
	} lhsPtr;

} ArrayInit;

inline ArrayInit MakeAI()
{
	ArrayInit ai;

	ai.maxdim = 0;
	ai.i = ai.j = ai.k = ai.dim = 0;

	ai.arraydim = 0; ai.type = TypeNull;
	ai.lhsPtr.VoidPtr = NULL;
	
	ai.pArrayResizePtr = NULL;
	ai.pStmtInit = NULL;

	for(int i=0; i<MAX_DIMENSION; ++i)
		ai.index[i] = ai.maxidx[i] = 0;
	
	return ai;
}

class VctrLHSX: public LeftHandSide<realnum>
{
protected:
	LeftHandSide<Vector>* pLHS; 

public:
	
	VctrLHSX(LeftHandSide<Vector>* ptr): pLHS(ptr) { }

	virtual const std::wstring& GetId() const { return pLHS->GetId(); }

	virtual std::wstring GetFullName() const
	{
		std::wostringstream os;
		os<<pLHS->GetFullName()<<L".xcom"; return os.str();
	}
	
	virtual realnum& GetData() { return pLHS->GetData().x; }
	virtual realnum Eval() { return GetData(); }

	virtual ~VctrLHSX() {}
};

class VctrLHSY: public LeftHandSide<realnum>
{
protected:
	LeftHandSide<Vector>* pLHS; 

public:
	
	VctrLHSY(LeftHandSide<Vector>* ptr): pLHS(ptr) { }

	virtual const std::wstring& GetId() const { return pLHS->GetId(); }
	virtual std::wstring GetFullName() const
	{
		std::wostringstream os;
		os<<pLHS->GetFullName()<<L".ycom"; return os.str();
	}

	virtual realnum& GetData() { return pLHS->GetData().y; }
	virtual realnum Eval() { return GetData(); }

	virtual ~VctrLHSY() {}
};

class VctrLHSZ: public LeftHandSide<realnum>
{
protected:
	LeftHandSide<Vector>* pLHS; 

public:
	
	VctrLHSZ(LeftHandSide<Vector>* ptr): pLHS(ptr) { }

	virtual const std::wstring& GetId() const { return pLHS->GetId(); }
	virtual std::wstring GetFullName() const
	{
		std::wostringstream os;
		os<<pLHS->GetFullName()<<L".zcom"; return os.str();
	}

	virtual realnum& GetData() { return pLHS->GetData().z; }
	virtual realnum Eval() { return GetData(); }

	virtual ~VctrLHSZ() {}
};

class VctrRHSX: public ExpReal
{
protected:
	ExpVctr* pXpr; 

public:
	
	VctrRHSX(ExpVctr* ptr): pXpr(ptr) { }

	virtual realnum Eval() { return pXpr->Eval().x; }

	virtual ~VctrRHSX() {}
};

class VctrRHSY: public ExpReal
{
protected:
	ExpVctr* pXpr; 

public:
	
	VctrRHSY(ExpVctr* ptr): pXpr(ptr) { }

	virtual realnum Eval() { return pXpr->Eval().y; }

	virtual ~VctrRHSY() {}
};

class VctrRHSZ: public ExpReal
{
protected:
	ExpVctr* pXpr; 

public:
	
	VctrRHSZ(ExpVctr* ptr): pXpr(ptr) { }

	virtual realnum Eval() { return pXpr->Eval().z; }

	virtual ~VctrRHSZ() {}
};

class CplxLHSReal: public LeftHandSide<realnum>
{
protected:
	LeftHandSide<Complex>* pLHS; 
public:
	
	CplxLHSReal(LeftHandSide<Complex>* ptr): pLHS(ptr)  { }

	virtual const std::wstring& GetId() const  {  return pLHS->GetId();  }
	virtual std::wstring GetFullName() const
	{
		std::wostringstream os;
		os<<pLHS->GetFullName()<<L".rpart";  return os.str();
	}

	virtual realnum& GetData() { return pLHS->GetData().get_real(); }
	virtual realnum Eval() { return GetData(); }

	virtual ~CplxLHSReal() {}
};

class CplxLHSImag: public LeftHandSide<realnum>
{
protected:
	LeftHandSide<Complex>* pLHS; 

public:
	
	CplxLHSImag(LeftHandSide<Complex>* ptr): pLHS(ptr) { }

	virtual const std::wstring& GetId() const {  return pLHS->GetId(); }
	virtual std::wstring GetFullName() const
	{
		std::wostringstream os;
		os<<pLHS->GetFullName()<<L".ipart";  return os.str();
	}

	virtual realnum& GetData() { return pLHS->GetData().get_imag(); }
	virtual realnum Eval() { return GetData(); }

	virtual ~CplxLHSImag() {}
};

class CplxRHSReal: public ExpReal
{
protected:
	ExpCplx* pExp; 

public:
	
	CplxRHSReal(ExpCplx* ptr): pExp(ptr) { }
	
	virtual realnum Eval() { return pExp->Eval().real(); }

	virtual ~CplxRHSReal() {}
};

class CplxRHSImag: public ExpReal
{
protected:
	ExpCplx* pExp; 

public:
	
	CplxRHSImag(ExpCplx* ptr): pExp(ptr) { }
	
	virtual realnum Eval() { return pExp->Eval().imag(); }

	virtual ~CplxRHSImag() {}
};

template<typename T>
class ExprStrn: public ExpStrn
{
protected:
	Expression<T> *Expr;
	std::locale loc;
	stream os; 
public:
	ExprStrn(Expression<T> *xpr): 
	  Expr(xpr), loc(std::locale(), new MyPunck), os()
	  { 
		  extern int  fmt_precision;
		  extern bool fmt_clear;
		  extern bool fmt_scientific;
		  extern bool fmt_showpoint;
		  extern bool fmt_thousand;

		  if(fmt_thousand) os.imbue(loc);
		  if(!fmt_clear)
			{
				os<<std::setprecision(fmt_precision);
		
				if(fmt_scientific)  os<<std::scientific;
				else os<<std::fixed;

				if(fmt_showpoint) os<<std::showpoint;
				else os<<std::noshowpoint;
			}

		  os<<std::boolalpha;
	  }

	virtual string Eval() 
	{
		os.str(L""); os<<Expr->Eval(); return os.str();
	}
	virtual ~ExprStrn() {}
};

class SolveEquation: public Expression<RealVec1D>
{
protected:
	
	LeftHandSide<realnum> *pVar;

	ExpIntg *pExprCnt;
	ExpReal *pExprEps, *pExpr1, *pExpr2, *pExpr3;
	
public:

	SolveEquation(LeftHandSide<realnum> *var, ExpReal* xpr1, 
		ExpReal* xpr2, ExpReal* xpr3);
	SolveEquation(LeftHandSide<realnum> *var, ExpReal* xpr1, 
		ExpReal* xpr2, ExpReal* xpr3, ExpIntg* pCnt);
  	SolveEquation(LeftHandSide<realnum> *var, ExpReal* xpr1, 
		ExpReal* xpr2, ExpReal* xpr3, ExpReal* pEps);
	SolveEquation(LeftHandSide<realnum> *var, ExpReal* xpr1, 
		ExpReal* xpr2, ExpReal* xpr3, 
		ExpIntg* pCnt, ExpReal* pEps);
	  	
	virtual RealVec1D Eval();
	virtual ~SolveEquation();
};

class SolveExtrema: public Expression<RealVec1D>
{
protected:
	
	LeftHandSide<realnum> *pVar;

	ExpIntg *pExprCnt;  // section count
	ExpReal *pExprEps;  // epsilon

	ExpReal *pExpr1, *pExpr2, *pExpr3;
	
public:

	SolveExtrema(LeftHandSide<realnum> *var, ExpReal* xpr1, 
		ExpReal* xpr2, ExpReal* xpr3);
	SolveExtrema(LeftHandSide<realnum> *var, ExpReal* xpr1, 
		ExpReal* xpr2, ExpReal* xpr3, ExpIntg* pCnt);
	SolveExtrema(LeftHandSide<realnum> *var, ExpReal* xpr1, 
		ExpReal* xpr2, ExpReal* xpr3, ExpReal* pEps);
	SolveExtrema(LeftHandSide<realnum> *var, ExpReal* xpr1, 
		ExpReal* xpr2, ExpReal* xpr3, 
		ExpIntg* pCnt, ExpReal* pEps);

	virtual RealVec1D Eval();
	virtual ~SolveExtrema();
};

template<typename T, typename vec>
class ClsComplement1: public Expression<vec>
{
protected:
	Expression<vec>* vXpr; 
	ExpIntg* index1;

public:
	ClsComplement1(Expression<vec>* xpr, ExpIntg* iptr1):
	  vXpr(xpr), index1(iptr1){ }

	virtual vec Eval()
	{
		return GetComplement<T>(vXpr->Eval(), index1->Eval());
	}

	virtual ~ClsComplement1() {}
};

template<typename T, typename vec>
class ClsComplement2: public Expression<vec>
{
protected:
	Expression<vec>* vXpr; 
	ExpIntg *index1, *index2;

public:
	ClsComplement2(Expression<vec>* xpr, 
		ExpIntg* iptr1, ExpIntg* iptr2):
	  vXpr(xpr), index1(iptr1), index2(iptr2){ }

	virtual vec Eval()
	{
		return GetComplement<T>(vXpr->Eval(), index1->Eval(), index2->Eval());
	}

	virtual ~ClsComplement2() {}
};

template<typename T, typename vec>
class ClsComplement3: public Expression<vec>
{
protected:
	Expression<vec>* vXpr; 
	ExpIntg *index1, *index2, *index3;

public:
	ClsComplement3(Expression<vec>* xpr, 
		ExpIntg* iptr1, ExpIntg* iptr2, ExpIntg* iptr3):
	  vXpr(xpr), index1(iptr1), index2(iptr2), index3(iptr3){ }

	virtual vec Eval()
	{
		return GetComplement<T>(vXpr->Eval(), index1->Eval(), index2->Eval(), index3->Eval());
	}

	virtual ~ClsComplement3() {}
};

template<typename T, typename vec>
class ClsComplement4: public Expression<vec>
{
protected:
	Expression<vec>* vXpr; 
	ExpIntg *index1, *index2, *index3, *index4;

public:
	ClsComplement4(Expression<vec>* xpr, 
		ExpIntg* iptr1, ExpIntg* iptr2, ExpIntg* iptr3, ExpIntg* iptr4):
	  vXpr(xpr), index1(iptr1), index2(iptr2), index3(iptr3), index4(iptr4){ }

	virtual vec Eval()
	{
		return GetComplement<T>(vXpr->Eval(), index1->Eval(), index2->Eval(), index3->Eval(), index4->Eval());
	}

	virtual ~ClsComplement4() {}
};

template<typename T, typename vec>
class ClsComplement5: public Expression<vec>
{
protected:
	Expression<vec>* vXpr; 
	ExpIntg *index1, *index2, *index3, *index4, *index5;

public:
	ClsComplement5(Expression<vec>* xpr, 
		ExpIntg* iptr1, ExpIntg* iptr2, ExpIntg* iptr3, 
		ExpIntg* iptr4, ExpIntg* iptr5):
	  vXpr(xpr), index1(iptr1), index2(iptr2), index3(iptr3), index4(iptr4), index5(iptr5){ }

	virtual vec Eval()
	{
		return GetComplement<T>(vXpr->Eval(), index1->Eval(), index2->Eval(), 
			index3->Eval(), index4->Eval(), index5->Eval());
	}

	virtual ~ClsComplement5() {}
};

template<typename T, typename vec>
class ClsSubArray1: public Expression<vec>
{
protected:
	Expression<vec>* vXpr; Range *rng1; bool cmp;
public:
	ClsSubArray1(Expression<vec>* xpr, Range *rg1, bool comp):
	  vXpr(xpr), rng1(rg1), cmp(comp) { }

	virtual vec Eval()
	{
		return GetSubArray<T>(vXpr->Eval(), cmp, rng1);
	}

	virtual ~ClsSubArray1() {}
};

template<typename T, typename vec>
class ClsSubArray2: public Expression<vec>
{
protected:
	Expression<vec>* vXpr; bool cmp;
	Range *rng1, *rng2; 
public:
	ClsSubArray2(Expression<vec>* xpr, Range *rg1, Range *rg2, bool comp):
	  vXpr(xpr), rng1(rg1), rng2(rg2), cmp(comp) { }

	virtual vec Eval()
	{
		return GetSubArray<T>(vXpr->Eval(), cmp, rng1, rng2);
	}

	virtual ~ClsSubArray2() {}
};

template<typename T, typename vec>
class ClsSubArray3: public Expression<vec>
{
protected:
	Expression<vec>* vXpr; bool cmp;
	Range *rng1, *rng2, *rng3; 
public:
	ClsSubArray3(Expression<vec>* xpr, Range *rg1, Range *rg2, Range *rg3, bool comp):
	  vXpr(xpr), rng1(rg1), rng2(rg2), rng3(rg3), cmp(comp) { }

	virtual vec Eval()
	{
		return GetSubArray<T>(vXpr->Eval(), cmp, rng1, rng2, rng3);
	}

	virtual ~ClsSubArray3() {}
};

template<typename T, typename vec>
class ClsSubArray4: public Expression<vec>
{
protected:
	Expression<vec>* vXpr; bool cmp;
	Range *rng1, *rng2, *rng3, *rng4; 
public:
	ClsSubArray4(Expression<vec>* xpr, Range *rg1,
		Range *rg2, Range *rg3, Range *rg4, bool comp):
	  vXpr(xpr), rng1(rg1), rng2(rg2), rng3(rg3), rng4(rg4), cmp(comp) { }

	virtual vec Eval()
	{
		return GetSubArray<T>(vXpr->Eval(), cmp, rng1, rng2, rng3, rng4);
	}

	virtual ~ClsSubArray4() {}
};

template<typename T, typename vec>
class ClsSubArray5: public Expression<vec>
{
protected:
	Expression<vec>* vXpr; bool cmp;
	Range *rng1, *rng2, *rng3, *rng4, *rng5; 
public:
	ClsSubArray5(Expression<vec>* xpr, Range *rg1, Range *rg2,
		Range *rg3, Range *rg4, Range *rg5, bool comp):
	  vXpr(xpr), rng1(rg1), rng2(rg2), rng3(rg3),
		  rng4(rg4), rng5(rg5), cmp(comp) { }

	virtual vec Eval()
	{
		return GetSubArray<T>(vXpr->Eval(), cmp, rng1, rng2, rng3, rng4, rng5);
	}

	virtual ~ClsSubArray5() {}
};

template <typename T>
class ExprMaxMin: public Expression<T>
{
protected:
	bool ismax; // true for maximum, false for minimum
	Expression<T> *pXpr1, *pXpr2;
public:
	ExprMaxMin(Expression<T>* xpr1, Expression<T>* xpr2, bool bmax):
	  pXpr1(xpr1), pXpr2(xpr2), ismax(bmax) {}

	  virtual T Eval()
	  {
		  T v1 = pXpr1->Eval(), v2 = pXpr2->Eval();
		  if(ismax) // get max
			  return v1>v2 ? v1 : v2 ;
		  else  // get min
			  return v1>v2 ? v2 : v1 ;
	  }
	  virtual std::wstring Display() { return std::wstring(_T("")); }

	  virtual ~ExprMaxMin() {}
};

inline integer Range::GetStart() 
{ 
	if(pStart) return pStart->Eval(); 
	else return integer(INT_MIN);
}

inline integer Range::GetEnd()   
{ 
	if(pEnd) return pEnd->Eval(); 
	else return integer(INT_MAX);
}

template<typename T>
class ArgExpr: public BaseType
{
public:
	Expression<T>* expr;
	bool cbr; // true: call by reference, false: call by value

	ArgExpr(Expression<T>* xpr, bool callbyref = false):
	expr(xpr), cbr(callbyref) {}

	virtual ~ArgExpr() {}
};

class IntToReal: public ExpReal
{
protected:
	ExpIntg* expr;
	realnum rtn;

	void cache();
public:
	IntToReal(ExpIntg* xpr);

	virtual double Eval();
	virtual std::wstring Display();
	virtual ~IntToReal();
};

class IntToCplx: public ExpCplx
{
protected:
	ExpIntg* expr;
	Complex rtn;
	void cache();
public:
	IntToCplx(ExpIntg* xpr);

	virtual Complex Eval();
	virtual std::wstring Display();
	virtual ~IntToCplx();
};

class RealToInt: public ExpIntg
{
protected:
	ExpReal* expr;
	integer rtn;
	void cache();
public:
	RealToInt(ExpReal* xpr);
	virtual integer Eval();
	virtual std::wstring Display();
	virtual ~RealToInt();
};

class RealToCplx: public ExpCplx
{
protected:
	ExpReal* expr;
	Complex rtn;
	void cache();
public:
	RealToCplx(ExpReal* xpr);
	virtual Complex Eval();
	virtual std::wstring Display();
	virtual ~RealToCplx();
};

class RealToVctr: public ExpVctr
{
protected:
	ExpReal* expr;
	Vector rtn;
	void cache();

public:
	RealToVctr(ExpReal* xpr);

	virtual Vector Eval();
	
	virtual std::wstring Display();
	virtual ~RealToVctr();
};

class IntgToVctr: public ExpVctr
{
protected:
	ExpIntg* expr;
	Vector rtn;

	void cache();

public:
	IntgToVctr(ExpIntg* xpr);

	virtual Vector Eval();
	
	virtual std::wstring Display();
	virtual ~IntgToVctr();
};

template<typename T>
class ExprWhich: public Expression<T>
{
protected:

	ExpBool *pBool;
	Expression<T> *pLeft, *pRight;
	T rtn;

	void cache()
	{
		if(pBool->bLiteral && pLeft->bLiteral && pRight->bLiteral)
		{	
			rtn = pBool->Eval() ? pLeft->Eval() : pRight->Eval();
			bLiteral = true;
		}
		else bLiteral = false;
	}

public:
	ExprWhich(ExpBool *pExprBool, Expression<T> *pXpr1, Expression<T> *pXpr2):
	  pBool(pExprBool), pLeft(pXpr1), pRight(pXpr2) { cache(); }


	virtual T Eval()
	{
		if(bLiteral) return rtn;
		else return pBool->Eval() ? pLeft->Eval() : pRight->Eval();
	}

	virtual std::wstring Display()
	{
		return std::wstring(_T(""));
	}

	  virtual ~ExprWhich() {}
};

class UdtVarBase: public BaseType
{
public:
	StrArray memary; UserDefinedType* udtptr; 
	
public:
	UdtVarBase(UserDefinedType *uptr): udtptr(uptr)  { }

	virtual UserDefinedType* GetUdt() const = 0;
	virtual usertype* GetUserType() = 0;
	virtual UserData* GetUserData() = 0;
	virtual bool IsCompatible(UserDefinedType *ptr) = 0;
	virtual bool IsCompatible(LeftHandSide<UserData> *ptr) = 0;

	virtual void InsertMemberID(const std::wstring& memid) = 0;
	virtual std::wstring GetFullName() const = 0;

	virtual ~UdtVarBase() {}
};

class VarMemBase;

class VarUserBase: public UdtVarBase
{
protected:
	usertype* getusertype(); // for cache
public:
	UserData* data; 

	VarUserBase(UserData *udptr);
	
	virtual UserDefinedType* GetUdt() const;
	virtual usertype* GetUserType();
	virtual UserData *GetUserData();
	virtual bool IsCompatible(UserDefinedType *ptr);
	virtual bool IsCompatible(LeftHandSide<UserData> *ptr);
	virtual void InsertMemberID(const std::wstring& memid);
	virtual std::wstring GetFullName() const;

	bool IsCompatible(VarUserBase* ptr);
	bool IsCompatible(VarMemBase* ptr);
	
	//virtual usertype Eval();
	
	virtual ~VarUserBase();
};

template<typename T> 
class VarUser: public LeftHandSide<T>
{
public:
	VarUserBase *vubPtr; 

protected:
	
	T* getmember() 
	{
		if(vubPtr->memary.empty()) 
			throw UnicodeException(_T("Member ID not defined.\r\n"));

		usertype *ptr = vubPtr->data->GetPtr();
				
		UdtPair* thePair = ptr->GetElement(vubPtr->memary[0]);
		int size = int(vubPtr->memary.size()), depth=1;
				
		while(depth < size)
		{
			if(TypeUser == thePair->first)
				thePair = static_cast<usertype*>(thePair->second)->GetElement(vubPtr->memary[depth++]);
			else if(TypeUPtr == thePair->first)
			{
				thePair = static_cast<UPtrData*>(thePair->second)->GetData()->GetData().GetElement(vubPtr->memary[depth++]);
			}
			else break;
		}
		
		if(depth == size)
		{
			return static_cast<T*>(thePair->second);
		}
		else throw UnicodeException(_T("Element typemismatch or not found.\r\n"));
	}
public:
	
	VarUser(VarUserBase *ptr): vubPtr(ptr){  }
		
	virtual T& GetData()  { return *getmember();  }
	virtual T Eval()  { return GetData(); }
	
	virtual void GetVariables(DataSet& dataset) {  dataset.insert(vubPtr->data); }

	virtual const std::wstring& GetId() const { return vubPtr->data->GetId(); } 

	virtual std::wstring GetFullName() const
	{ 
		//std::wostringstream os; 
		//os<<GetId()<<vubPtr->GetFullName(); 
		return GetId()+vubPtr->GetFullName(); 
	} 

	virtual std::wstring Display() 
	{
		std::wostringstream os;
		if(typeid(T)!=typeid(voidtype)) os<<GetId();
		return os.str();
	}
	
	virtual ~VarUser() { }
};

template<> 
class VarUser<realnum>: public LeftHandSide<realnum>
{
public:
	VarUserBase *vubPtr; //realnum* memptr; 

protected:

	realnum* getmember() 
	{
		if(vubPtr->memary.empty()) 
			throw UnicodeException(_T("Member ID not defined.\r\n"));

		UdtPair* thePair = vubPtr->data->GetPtr()->GetElement(vubPtr->memary[0]);
		int size = int(vubPtr->memary.size()), depth=1;

		while(depth < size)
		{
			if(TypeUser == thePair->first)
				thePair = static_cast<usertype*>(thePair->second)->GetElement(vubPtr->memary[depth++]);
			else if(TypeUPtr == thePair->first)
				thePair = static_cast<UPtrData*>(thePair->second)->GetData()->GetData().GetElement(vubPtr->memary[depth++]);
			else break;
		}

		if(depth == size)
		{
			return static_cast<realnum*>(thePair->second);
		}
		else throw UnicodeException(_T("Element typemismatch or not found.\r\n"));
	}
public:
	
	VarUser(VarUserBase *ptr): vubPtr(ptr){  }
		
	virtual realnum& GetData() { return *getmember(); }
	virtual realnum Eval()  { return GetData(); }
	
	virtual realnum EvalPlus() { return GetData() + 1E-6; }
	virtual realnum EvalMinus() { return GetData() - 1E-6; }

	virtual void GetVariables(DataSet& dataset) {  dataset.insert(vubPtr->data); }

	virtual const std::wstring& GetId() const { return vubPtr->data->GetId(); } 
	
	virtual std::wstring GetFullName() const
	{ 
		//std::wostringstream os; 
		//os<<GetId()<<vubPtr->GetFullName(); 	
		return GetId()+vubPtr->GetFullName(); 
	}

	virtual std::wstring Display() 
	{
		std::wostringstream os; os<<GetId();
		return os.str();
	}
	
	virtual ~VarUser() { }
};

template<typename T, typename ary>
class VarUserAry: public LeftHandSideArray<T>
{
protected:
	VarUser<ary> *vmPtr; 
public:
	VarUserAry(VarUser<ary> *ptr): vmPtr(ptr) { }
	
	Array* GetDataPtr()  { return vmPtr->GetData(); }

	virtual T& GetData() { return *vmPtr->GetData().GetVec(); } 

	virtual T Eval() { return *vmPtr->GetData().GetVec(); }

	virtual const std::wstring& GetId() const 
		{ return vmPtr->GetId(); }
	virtual std::wstring GetFullName() const
		{ return vmPtr->GetFullName(); }
	
	virtual std::wstring Display() 
	{
		return std::wstring(L"");
	}

	virtual ~VarUserAry() {}
};


//////////////////////////////


class VarMemBase: public UdtVarBase
{
protected:
	usertype* getusertype(); // for cache
	
public:
	mutable UserData** data; Function *ptrFunc; 
	
	VarMemBase(UserDefinedType *uptr, Function *fptr=NULL, VarUserBase *vub=NULL);
	
	virtual UserDefinedType* GetUdt() const;
	virtual usertype* GetUserType();
	virtual UserData* GetUserData();
	virtual bool IsCompatible(UserDefinedType *ptr);
	virtual bool IsCompatible(LeftHandSide<UserData> *ptr);

	virtual void InsertMemberID(const std::wstring& memid);
	virtual std::wstring GetFullName() const;
	
	bool IsCompatible(VarMemBase* ptr);
	bool IsCompatible(VarUserBase* ptr);
	
	void SetDataPtr() const;
	virtual ~VarMemBase();
};

class UdtPtrBase: public UdtVarBase
{
public:
	PtrData<UserData>* pData;
public:
	
	UdtPtrBase(PtrData<UserData>* vptr);
	
	virtual void SetDataPtr(UserData* udptr);

	virtual UserDefinedType* GetUdt() const;
	virtual usertype* GetUserType();
	virtual UserData* GetUserData();

	virtual bool IsCompatible(UserDefinedType *ptr);
	virtual bool IsCompatible(LeftHandSide<UserData> *ptr);

	virtual void InsertMemberID(const std::wstring& memid);

	virtual std::wstring GetFullName() const;

	virtual ~UdtPtrBase();
};

class UdtPtrExp: public LeftHandSide<UserData>
{
protected:
	UdtPtrBase *vubPtr;
public:
	
	UdtPtrExp(UdtPtrBase *ptr): vubPtr(ptr) {}
	
	virtual UserData Eval() { return GetData(); }
	virtual UserData& GetData() { return *vubPtr->GetUserData(); }
	virtual UserDefinedType* GetUdt() { return vubPtr->GetUdt(); }
	virtual StrArray& GetMemArray() { return vubPtr->memary; }
	virtual UserData* GetUserData() { return vubPtr->GetUserData(); }
	virtual const std::wstring& GetId() const { return vubPtr->GetUserData()->GetId(); }
	virtual std::wstring GetFullName() const { return vubPtr->GetFullName(); }
	virtual bool IsCompatible(LeftHandSide<UserData> *ptr) 
		{ return vubPtr->GetUdt() == ptr->GetUdt(); }
	virtual bool IsCompatible(UserDefinedType *ptr)
		{ return vubPtr->GetUdt() == ptr; }

	virtual void InsertMemberID(const std::wstring& memid)
		{	vubPtr->InsertMemberID(memid); }

	virtual ~UdtPtrExp() {}
};

template<typename T> 
class VarUserPtr: public LeftHandSide<T>
{
protected:
	UdtPtrBase *vubPtr; 
	
	T* getmember() 
	{
		if(vubPtr->GetUserData() ==NULL)
			HandleSyntaxError(L"Pointer to User Defined Data is not initialized.\r\n");

		if(vubPtr->memary.empty()) 
			throw UnicodeException(_T("Member ID not defined.\r\n"));
			
		usertype *ptr = vubPtr->GetUserType();
		
		UdtPair* thePair = ptr->GetElement(vubPtr->memary[0]);
		int size = int(vubPtr->memary.size()), depth=1;

		while(depth<size)
		{
			if(TypeUser == thePair->first)
				thePair = static_cast<usertype*>(thePair->second)->GetElement(vubPtr->memary[depth++]);
			else if(TypeUPtr == thePair->first)
			{
				thePair = static_cast<UPtrData*>(thePair->second)->GetData()->GetData().GetElement(vubPtr->memary[depth++]);
			}
			else break;
		}

		if(depth == size)
		{
			return static_cast<T*>(thePair->second);
		}
		else throw UnicodeException(_T("Element typemismatch or not found.\r\n"));
	}
public:
	
	VarUserPtr(UdtPtrBase *ptr): vubPtr(ptr){  }
		
	virtual T& GetData()  { return *getmember();  }
	virtual T Eval()  { return GetData(); }
	
	virtual void GetVariables(DataSet& dataset) {  dataset.insert(vubPtr->pData); }

	virtual const std::wstring& GetId() const 
	{ return vubPtr->pData->GetId(); } 

	virtual std::wstring GetFullName() const
	{ 
		std::wostringstream os; 
		os<<GetId()<<L"."<<vubPtr->GetFullName(); 
		return os.str(); 
	} 

	virtual std::wstring Display() 
	{
		std::wostringstream os;
		if(typeid(T)!=typeid(voidtype)) os<<GetId();
		return os.str();
	}
	
	virtual ~VarUserPtr() { }
};


template<> 
class VarUserPtr<realnum>: public LeftHandSide<realnum>
{
protected:
	UdtPtrBase *vubPtr; 
	
	realnum* getmember() 
	{
		if(vubPtr->GetUserData()==NULL)
			HandleSyntaxError(L"Pointer to User Defined Data is not initialized.\r\n");

		if(vubPtr->memary.empty()) 
			throw UnicodeException(_T("Member ID not defined.\r\n"));

		usertype *ptr = vubPtr->GetUserType();
		
		UdtPair* thePair = ptr->GetElement(vubPtr->memary[0]);
		int size = int(vubPtr->memary.size()), depth=1;

		while(depth<size)
		{
			if(TypeUser == thePair->first)
				thePair = static_cast<usertype*>(thePair->second)->GetElement(vubPtr->memary[depth++]);
			else if(TypeUPtr == thePair->first)
				thePair = static_cast<UPtrData*>(thePair->second)->GetData()->GetData().GetElement(vubPtr->memary[depth++]);
			else break;
		}

		if(depth == size)
		{
			return static_cast<realnum*>(thePair->second);
		}
		else throw UnicodeException(_T("Element typemismatch or not found.\r\n"));
	}
public:
	
	VarUserPtr(UdtPtrBase *ptr): vubPtr(ptr){  }
		
	virtual realnum& GetData()  { return *getmember();  }
	virtual realnum Eval()  { return GetData(); }
	virtual realnum EvalPlus(){ return GetData()+1E-6; }
	virtual realnum EvalMinus(){ return GetData()-1E-6; }
	
	virtual void GetVariables(DataSet& dataset) {  dataset.insert(vubPtr->pData); }

	virtual const std::wstring& GetId() const 
	{ return vubPtr->GetUserData()->GetId(); } 

	virtual std::wstring GetFullName() const
	{ 
		std::wostringstream os; 
		os<<GetId()<<L"."<<vubPtr->GetFullName(); return os.str(); 
	} 

	virtual std::wstring Display() 
	{
		std::wostringstream os; os<<GetId();
		return os.str();
	}
	
	virtual ~VarUserPtr() { }
};


class VarMemExp: public LeftHandSide<UserData>
{
protected:
	VarMemBase *vmbPtr;
public:
	
	VarMemExp(VarMemBase *ptr): vmbPtr(ptr) {}
	
	virtual UserData Eval() { return GetData(); }
	virtual UserData& GetData() { return *vmbPtr->GetUserData(); }
	virtual UserDefinedType* GetUdt() { return vmbPtr->GetUdt(); }
	virtual UserData* GetUserData() { return vmbPtr->GetUserData(); }
	virtual StrArray& GetMemArray() { return vmbPtr->memary; }
	virtual const std::wstring& GetId() const { return vmbPtr->GetUserData()->GetId(); }
	virtual std::wstring GetFullName() const { return vmbPtr->GetFullName(); }
	virtual bool IsCompatible(LeftHandSide<UserData> *ptr) 
		{ return vmbPtr->udtptr == ptr->GetUdt(); }
	virtual bool IsCompatible(UserDefinedType *ptr)
		{ return vmbPtr->udtptr == ptr; }
	virtual void InsertMemberID(const std::wstring& memid)
		{ vmbPtr->InsertMemberID(memid); }

	virtual ~VarMemExp() {}
};

class VarUserExp: public LeftHandSide<UserData>
{
protected:
	VarUserBase *vubPtr;
public:
	
	VarUserExp(VarUserBase *ptr): vubPtr(ptr) {}
	
	virtual UserData Eval() { return GetData(); }
	virtual UserData& GetData() { return *vubPtr->GetUserData(); }
	virtual UserDefinedType* GetUdt() { return vubPtr->GetUdt(); }
	virtual StrArray& GetMemArray() { return vubPtr->memary; }
	virtual UserData* GetUserData() { return vubPtr->GetUserData(); }
	virtual const std::wstring& GetId() const { return vubPtr->GetUserData()->GetId(); }
	virtual std::wstring GetFullName() const { return vubPtr->GetFullName(); }
	virtual bool IsCompatible(LeftHandSide<UserData> *ptr) 
		{ return vubPtr->GetUdt() == ptr->GetUdt(); }
	virtual bool IsCompatible(UserDefinedType *ptr)
		{ return vubPtr->GetUdt() == ptr; }
	
	virtual void InsertMemberID(const std::wstring& memid)
		{ vubPtr->InsertMemberID(memid); }
	virtual ~VarUserExp() {}
};


template<typename T> 
class VarMem: public LeftHandSide<T>
{
public:
	mutable VarMemBase *vubPtr; 

protected:
	T* getmember() 
	{
		vubPtr->SetDataPtr();

		if(vubPtr->memary.empty()) 
			throw UnicodeException(_T("Member ID not defined.\r\n"));

		UdtPair* thePair = (*vubPtr->data)->GetPtr()->GetElement(vubPtr->memary[0]);
		int size = int(vubPtr->memary.size()), depth=1;

		while(depth < size)
		{
			if(TypeUser == thePair->first)
				thePair = static_cast<usertype*>(thePair->second)->GetElement(vubPtr->memary[depth++]);
			else if(TypeUPtr == thePair->first)
				thePair = static_cast<UPtrData*>(thePair->second)->GetData()->GetData().GetElement(vubPtr->memary[depth++]);
			else break;
		}

		if(depth == size)
		{
			return static_cast<T*>(thePair->second);	
		}
		else throw UnicodeException(_T("Element typemismatch or not found.\r\n"));
	}
public:
	
	VarMem(VarMemBase *ptr): vubPtr(ptr){  }
		
	virtual T& GetData() { return *getmember();  }
	
	virtual T Eval()  { return GetData(); }
	
	virtual void GetVariables(DataSet& dataset) {  dataset.insert((*vubPtr->data)); }

	virtual const std::wstring& GetId() const 
	{ 
		vubPtr->SetDataPtr();
		return (*vubPtr->data)->GetId(); 
	} 

	virtual std::wstring GetFullName() const
	{ 
		//std::wostringstream os; 
		//os<<GetId()<<vubPtr->GetFullName(); 
		return GetId()+vubPtr->GetFullName(); ; 
	} 

	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		if(typeid(T)!=typeid(voidtype))
			ostr<<GetId();
		return ostr.str();
	}
	
	virtual ~VarMem() { }
};

template<> 
class VarMem<realnum>: public LeftHandSide<realnum>
{
public:
	mutable VarMemBase *vubPtr; //realnum* memptr; 

protected:
	realnum* getmember() 
	{
		vubPtr->SetDataPtr();

		if(vubPtr->memary.empty()) 
			throw UnicodeException(_T("Member ID not defined.\r\n"));

		UdtPair* thePair = (*vubPtr->data)->GetPtr()->GetElement(vubPtr->memary[0]);
		int size = int(vubPtr->memary.size()), depth=1;

		while(depth < size)
		{
			if(TypeUser == thePair->first)
				thePair = static_cast<usertype*>(thePair->second)->GetElement(vubPtr->memary[depth++]);
			else if(TypeUPtr == thePair->first)
				thePair = static_cast<UPtrData*>(thePair->second)->GetData()->GetData().GetElement(vubPtr->memary[depth++]);
			else break;
		}

		if(depth == size)
		{
			return static_cast<realnum*>(thePair->second);
		}
		else throw UnicodeException(_T("Element typemismatch or not found.\r\n"));
	}
public:
	
	VarMem(VarMemBase *ptr): vubPtr(ptr){  }
		
	virtual realnum& GetData() { return *getmember(); }
	virtual realnum Eval()  { return GetData(); }
	
	virtual realnum EvalPlus() { return GetData() + 1E-6; }
	virtual realnum EvalMinus() { return GetData() - 1E-6; }

	virtual void GetVariables(DataSet& dataset) {  dataset.insert((*vubPtr->data)); }

	virtual const std::wstring& GetId() const 
	{ 
		vubPtr->SetDataPtr();
		return (*vubPtr->data)->GetId(); 
	} 
	
	virtual std::wstring GetFullName() const
	{ 
		//std::wostringstream os; 
		//os<<GetId()<<vubPtr->GetFullName(); 
		return GetId()+vubPtr->GetFullName(); 
	}

	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		ostr<<GetId();
		return ostr.str();
	}
	
	virtual ~VarMem() { }
};

class VarMemUPtrData: public UdtPtrBase
{
protected:
	VarMem<UPtrData> *pData;

public:

	VarMemUPtrData(VarMem<UPtrData>* vptr);
	
	virtual void SetDataPtr(UserData* udptr);

	virtual UserDefinedType* GetUdt() const;
	virtual usertype* GetUserType();
	virtual UserData* GetUserData();

	virtual bool IsCompatible(UserDefinedType *ptr);
	virtual bool IsCompatible(LeftHandSide<UserData> *ptr);

	virtual void InsertMemberID(const std::wstring& memid);

	virtual std::wstring GetFullName() const;

	virtual ~VarMemUPtrData();
};


class UserMemUPtrData: public UdtPtrBase
{
protected:
	VarUser<UPtrData> *pData;

public:

	UserMemUPtrData(VarUser<UPtrData>* vptr);
	
	virtual void SetDataPtr(UserData* udptr);

	virtual UserDefinedType* GetUdt() const;
	virtual usertype* GetUserType();
	virtual UserData* GetUserData();

	virtual bool IsCompatible(UserDefinedType *ptr);
	virtual bool IsCompatible(LeftHandSide<UserData> *ptr);

	virtual void InsertMemberID(const std::wstring& memid);

	virtual std::wstring GetFullName() const;

	virtual ~UserMemUPtrData();
};

class MemUPtrData: public UdtPtrBase
{
protected:
	UdtPtrBase *pData;

public:

	MemUPtrData(UdtPtrBase* vptr);
	
	virtual void SetDataPtr(UserData* udptr);

	virtual UserDefinedType* GetUdt() const;
	virtual usertype* GetUserType();
	virtual UserData* GetUserData();

	virtual bool IsCompatible(UserDefinedType *ptr);
	virtual bool IsCompatible(LeftHandSide<UserData> *ptr);

	virtual void InsertMemberID(const std::wstring& memid);

	virtual std::wstring GetFullName() const;

	virtual ~MemUPtrData();
};

template<typename T, typename ary>
class VarMemAry: public LeftHandSideArray<T>
{
protected:
	VarMem<ary> *vmPtr; 
public:
	VarMemAry(VarMem<ary> *ptr): vmPtr(ptr) { }
	
	Array* GetDataPtr()  { return vmPtr->GetData(); }

	virtual T& GetData() { return *vmPtr->GetData().GetVec(); } 
	
	virtual T Eval() { return *vmPtr->GetData().GetVec(); }
	
	virtual const std::wstring& GetId() const { return vmPtr->GetId(); }
	virtual std::wstring GetFullName() const 
	{ return vmPtr->GetFullName(); }
	
	virtual std::wstring Display() 
	{
		return std::wstring(L"");
	}

	virtual ~VarMemAry() {}
};

template<typename T> 
class Variable: public LeftHandSide<T>
{
protected:
	DataTemp<T>* data;
public:
	
	Variable(VarType vt)
	{ 
		if(vt.dPtr) data = dynamic_cast<DataTemp<T>*>(vt.dPtr);
		else data = NULL; // only when declare
	}

	Variable(DataTemp<T> *ptr): data(ptr) { }
		
	virtual T& GetData() { return data->GetData(); }
	
	virtual T Eval()  { return GetData(); }
	
	virtual void GetVariables(DataSet& dataset) {  dataset.insert(data); }

	virtual const std::wstring& GetId() const { return data->GetId(); } 

	virtual std::wstring GetFullName() const { return GetId(); }

	virtual std::wstring Display() 
	{
		std::wostringstream os;
		if(typeid(T)!=typeid(voidtype)) os<<GetId();
		return os.str();
	}
	
	virtual ~Variable() { }
};

template<> 
class Variable<realnum>: public LeftHandSide<realnum>
{
protected:
	DataTemp<realnum>* data;
public:
	
	Variable(VarType vt)
	{ 
		if(vt.dPtr) data = dynamic_cast<DataTemp<realnum>*>(vt.dPtr);
		else
			data = NULL; // only when declare
	}

	Variable(DataTemp<realnum>* ptr): data(ptr) { }
		
	virtual realnum& GetData() { return data->GetData(); }
	
	virtual realnum Eval()  { return GetData(); }
	
	virtual realnum EvalPlus() { return data->GetData()  + 1E-6; }
	virtual realnum EvalMinus() { return data->GetData() - 1E-6; }

	virtual void GetVariables(DataSet& dataset) {  dataset.insert(data); }

	virtual const std::wstring& GetId() const { return data->GetId(); } 

	virtual std::wstring GetFullName() const { return GetId(); }

	virtual std::wstring Display() 
	{
		std::wostringstream os; os<<GetId();
		return os.str();
	}
	
	virtual ~Variable() { }
};

template<> 
class Variable<stream>: public LeftHandSide<stream&>
{
protected:
	DataTemp<stream>* data;
public:
	
	Variable(VarType vt)
	{ 
		if(vt.dPtr) data = dynamic_cast<DataTemp<stream>*>(vt.dPtr);
		else data = NULL; // only when declare
	}

	Variable(DataTemp<stream> *ptr): data(ptr) { }
		
	virtual stream& GetData() { return data->GetData(); }
	virtual stream& Eval()  { return GetData(); }
	
	virtual void GetVariables(DataSet& dataset) {  dataset.insert(data); }

	virtual const std::wstring& GetId() const { return data->GetId(); } 

	virtual std::wstring GetFullName() const { return GetId(); }

	virtual std::wstring Display() 
	{
		std::wostringstream os; os<<GetId();
		return os.str();
	}
	
	virtual ~Variable() { }
};

template<typename T> 
class PtrVariable: public LeftHandSide<T*>
{
protected:
	PtrData<T>* data;
public:
	
	PtrVariable(PtrData<T>* ptr): data(ptr) { }
	
	virtual T*& GetData() { return *data->GetPtr(); }
	
	virtual T* Eval()  { return data->GetData(); }
	
	virtual void GetVariables(DataSet& dataset) {  dataset.insert(data); }

	virtual const std::wstring& GetId() const { return data->GetId(); } 

	virtual std::wstring Display() 
	{
		std::wostringstream os;
		if(typeid(T)!=typeid(voidtype)) os<<GetId();
		return os.str();
	}
	
	virtual ~PtrVariable() { }
};

template<typename T> 
class PtrToVar: public LeftHandSide<T>
{
protected:
	LeftHandSide<T*>* pLHS;

public:
	
	PtrToVar(LeftHandSide<T*> * ptr): pLHS(ptr) { }
	
	virtual T& GetData() { return *pLHS->GetData(); }
	
	virtual T Eval()  { return GetData(); }
	
	virtual const std::wstring& GetId() const 
	{ return pLHS->GetId(); } 

	virtual std::wstring Display() 
	{
		std::wostringstream os;
		os<<GetId(); return os.str();
	}
	
	virtual ~PtrToVar() { }
};

template<typename T> 
class PtrExpToVar: public LeftHandSide<T>
{
protected:
	Expression<T*>* pLHS; std::wstring id;
public:
	
	PtrExpToVar(Expression<T*> * ptr): pLHS(ptr) { }
	
	virtual T& GetData() { return *pLHS->Eval(); }
	
	virtual T Eval()  { return GetData(); }
	
	virtual const std::wstring& GetId() const 
	{ 
		LeftHandSide<T*>* ptr 
			= dynamic_cast<LeftHandSide<T*>*>(pLHS);
		if(ptr)
			return ptr->GetId();
		else return id;
	} 

	virtual std::wstring Display() 
	{
		std::wostringstream os;
		os<<GetId(); return os.str();
	}
	
	virtual ~PtrExpToVar() { }
};


template<> 
class PtrExpToVar<UserData>: public LeftHandSide<UserData>
{
protected:
	UdtPtrBase* pLHS; 
public:
	
	PtrExpToVar(UdtPtrBase* ptr): pLHS(ptr) { }
	
	virtual UserData& GetData() { return *pLHS->GetUserData(); }
	
	virtual UserData Eval()  { return GetData(); }
	
	virtual const std::wstring& GetId() const 
	{ 
		return const_cast<PtrExpToVar<UserData>*>(this)
			->pLHS->GetUserData()->GetId();
	} 

	virtual UserDefinedType* GetUdt() { return pLHS->GetUdt(); }

	virtual UserData* GetUserData()	{ return pLHS->GetUserData(); }

	virtual void InsertMemberID(const std::wstring& memid)
	{ pLHS->InsertMemberID(memid); }

	virtual std::wstring GetFullName() const { return GetId(); }

	virtual bool IsCompatible(LeftHandSide<UserData> *ptr)
	{  return ptr->IsCompatible(pLHS->GetUdt()); }

	virtual bool IsCompatible(UserDefinedType *ptr)
	{ return pLHS->GetUdt() == ptr; }
	
	virtual std::wstring Display() 
	{
		std::wostringstream os;
		os<<GetId(); return os.str();
	}
	
	virtual ~PtrExpToVar() { }
};


template<typename T, typename U> 
class IncDecVar: public Expression<T>
{
protected:
	bool isinc; // true for increment, false for decrement
	U* data;
public:
	
	IncDecVar(VarType vt, bool binc): isinc(binc)
	{ 
		if(vt.dPtr) data = dynamic_cast<U*>(vt.dPtr);
		else
			data = NULL; // only when declare
	}
		
	virtual T Eval()  
	{ 
		if(isinc)
			++data->GetData();
		else
			--data->GetData();
		
		return data->GetData(); 
	}
	
	virtual std::wstring Display() 
	{ return std::wstring(); }
	
	virtual ~IncDecVar() {  }
};

template<typename T>
class ExprPtr: public LeftHandSide<T>
{
protected:

	PtrData<T>* pData;
public:
	
	ExprPtr(PtrData<T>* vptr): pData(vptr) {}
	
	virtual const std::wstring& GetId() const { return pData->GetId(); }

	virtual T& GetData() { return *pData->GetData(); }
	
	virtual std::wstring GetFullName() const { return GetId();  }

	virtual T Eval() {  return GetData(); }

	virtual std::wstring Display() { return std::wstring(); }

	virtual ~ExprPtr() { }
};

template<>
class ExprPtr<UserData>: public LeftHandSide<UserData>
{
protected:
	PtrData<UserData>* pData;
public:
	
	ExprPtr(PtrData<UserData>* vptr): pData(vptr) {}
	
	virtual const std::wstring& GetId() const { return pData->GetId(); }

	virtual UserData* GetUserData() { return pData->GetData(); }
	virtual UserData& GetData() { return *pData->GetData(); }
			
	virtual std::wstring GetFullName() const { return GetId();  }

	virtual UserData Eval() {  return GetData(); }

	virtual UserDefinedType* GetUdt() { return pData->factory; }

	virtual std::wstring Display() { return std::wstring(); }

	virtual bool IsCompatible(LeftHandSide<UserData> *ptr)
	{ return ptr->IsCompatible(pData->factory); }

	virtual bool IsCompatible(UserDefinedType *ptr)
	{ return ptr == pData->factory; }

	virtual ~ExprPtr() { }
};

template<typename vec>
class ExprAptr: public LeftHandSideArray<vec>
{
protected:
	AptrData<vec>* pData;
	
public:
	
	ExprAptr(AptrData<vec>* vptr): pData(vptr) {}
	

	virtual vec& GetData() { return *pData->GetData(); }
	virtual vec Eval() { return GetData();  }

	virtual const std::wstring& GetId() const 
		{ return pData->GetId(); }
	virtual std::wstring GetFullName() const
		{ return GetId(); }
	
	virtual ~ExprAptr() { }
};

class VarAryBase
{
public:
	virtual int GetIndex1() const
	{ throw UnicodeException(_T("Called from VarAryBase::GetIndex1().\r\n")); }
	virtual int GetIndex2() const 
	{ throw UnicodeException(_T("Called from VarAryBase::GetIndex1().\r\n")); }
	virtual int GetIndex3() const
	{ throw UnicodeException(_T("Called from VarAryBase::GetIndex1().\r\n")); }
	virtual int GetIndex4() const
	{ throw UnicodeException(_T("Called from VarAryBase::GetIndex1().\r\n")); }
	virtual int GetIndex5() const
	{ throw UnicodeException(_T("Called from VarAryBase::GetIndex1().\r\n")); }

	virtual const std::wstring& GetId() const = 0; 
	virtual std::wstring GetFullName() const = 0;

	virtual bool IsCongruent(VarAryBase* vptr) const = 0;
	
	virtual ~VarAryBase() {}
};

template<typename T, typename ary>
class ExprAptr1: public VarAryBase, public LeftHandSide<T>
{
protected:
	AptrData<ary>* pData;
	ExpIntg *pIndex1;

public:
	
	ExprAptr1(AptrData<ary>* vptr, ExpIntg* idx1): 
	pData(vptr), pIndex1(idx1) {}
	
	virtual T& GetData() 
	{
		ary* ptr = pData->GetData();
		if(ptr)
			return ptr->at(int(pIndex1->Eval()));
		else
			throw UnicodeException(_T("Pointer is not initialized"));
	}

	virtual int GetIndex1() const { return int(pIndex1->Eval()); }

	virtual bool IsCongruent(VarAryBase *vptr) const
	{
		if(this->GetIndex1()==vptr->GetIndex1()) return true;
		else return false;
	}

	virtual T Eval() {  return GetData(); }
	virtual const std::wstring& GetId() const 
	{ return pData->GetId(); }

	virtual std::wstring GetFullName() const
	{
		std::wostringstream ostr;
		ostr<<this->GetId()<<_T("[")<<int(pIndex1->Eval())<<_T("]");
		return ostr.str();
	}

	virtual std::wstring Display() { return std::wstring(); }
	virtual ~ExprAptr1() { }
};

template<typename T, typename ary>
class ExprAptr2: public LeftHandSide<T>
{
protected:
	AptrData<ary>* pData;
	ExpIntg *pIndex1, *pIndex2;

public:
	
	ExprAptr2(AptrData<ary>* vptr, 
		ExpIntg* idx1, ExpIntg* idx2): 
	pData(vptr), pIndex1(idx1), pIndex2(idx2) {}
	
	virtual T& GetData()
	{
		ary* ptr = pData->GetData();
		if(ptr)
			return ptr->at(int(pIndex1->Eval())).at(int(pIndex2->Eval()));
		else
			throw UnicodeException(_T("Pointer is not initialized"));
	}
	
	virtual T Eval() { return GetData(); }

	virtual int GetIndex1() const {  return int(pIndex1->Eval()); }
	virtual int GetIndex2() const {  return int(pIndex2->Eval()); }

	virtual bool IsCongruent(VarAryBase *vptr) const
	{
		if(this->GetIndex1()==vptr->GetIndex1() &&
			this->GetIndex2()==vptr->GetIndex2()) return true;
		else return false;
	}

	virtual const std::wstring& GetId() const 
	{ return pData->GetId(); } 

	virtual std::wstring GetFullName() const
	{
		std::wostringstream os;
		os<<GetId()<<_T("[")<<pIndex1->Eval()<<_T(", ")<<pIndex2->Eval()<<_T("]"); return os.str();
	}

	virtual std::wstring Display() { return std::wstring(); }
	virtual ~ExprAptr2() { }
};

template<typename T, typename ary>
class ExprAptr3: public LeftHandSide<T>
{
protected:
	AptrData<ary>* pData;
	ExpIntg *pIndex1, *pIndex2, *pIndex3;

public:
	
	ExprAptr3(AptrData<ary>* vptr, ExpIntg* idx1, 
		ExpIntg* idx2, ExpIntg* idx3): 
	pData(vptr), pIndex1(idx1), pIndex2(idx2), pIndex3(idx3) {}
	
	virtual T& GetData()
	{
		ary* ptr = pData->GetData();
		if(ptr)
			return ptr->at(int(pIndex1->Eval())).at(int(pIndex2->Eval())).at(int(pIndex3->Eval()));
		else
			throw UnicodeException(_T("Pointer is not initialized"));
	}
	
	virtual int GetIndex1() const { return int(pIndex1->Eval()); }
	virtual int GetIndex2() const { return int(pIndex2->Eval()); }
	virtual int GetIndex3() const { return int(pIndex3->Eval()); }

	virtual bool IsCongruent(VarAryBase *vptr) const
	{
		if(	this->GetIndex1()==vptr->GetIndex1() &&
			this->GetIndex2()==vptr->GetIndex2() &&
			this->GetIndex3()==vptr->GetIndex3()) return true;
		else return false;
	}

	virtual const std::wstring& GetId() const { return pData->GetId(); }

	virtual std::wstring GetFullName() const
	{
		std::wostringstream os;
		os<<GetId()<<_T("[")<<pIndex1->Eval()<<_T(", ")<<pIndex2->Eval()<<_T(", ")
			<<pIndex3->Eval()<<_T("]"); return os.str();
	}

	virtual T Eval()  { return GetData(); }

	virtual std::wstring Display() { return std::wstring(); }
	virtual ~ExprAptr3() { }
};

template<typename T, typename ary>
class ExprAptr4: public LeftHandSide<T>
{
protected:
	AptrData<ary>* pData;
	ExpIntg *pIndex1, *pIndex2, *pIndex3, *pIndex4;

public:
	
	ExprAptr4(AptrData<ary>* vptr, ExpIntg* idx1, 
		ExpIntg* idx2, ExpIntg* idx3, ExpIntg* idx4): 
	pData(vptr), pIndex1(idx1), pIndex2(idx2), pIndex3(idx3), pIndex4(idx4) {}
	
	virtual T& GetData()
	{
		ary* ptr = pData->GetData();
		if(ptr)
			return ptr->at(int(pIndex1->Eval())).at(int(pIndex2->Eval())).
				at(int(pIndex3->Eval())).at(int(pIndex4->Eval()));
		else
			throw UnicodeException(_T("Pointer is not initialized"));
	}
	virtual T Eval() {  return GetData(); }

	virtual int GetIndex1() const { return int(pIndex1->Eval()); }
	virtual int GetIndex2() const { return int(pIndex2->Eval()); }
	virtual int GetIndex3() const { return int(pIndex3->Eval()); }
	virtual int GetIndex4() const { return int(pIndex4->Eval()); }

	virtual bool IsCongruent(VarAryBase *vptr) const
	{
		if(	this->GetIndex1()==vptr->GetIndex1() &&
			this->GetIndex2()==vptr->GetIndex2() &&
			this->GetIndex3()==vptr->GetIndex3() &&
			this->GetIndex4()==vptr->GetIndex4()) return true;
		else return false;
	}

	virtual const std::wstring& GetId() const { return pData->GetId(); }

	virtual std::wstring GetFullName() const
	{
		std::wostringstream os;
		os<<GetId()<<_T("[")<<pIndex1->Eval()<<_T(", ")<<pIndex2->Eval()<<_T(", ")
			<<pIndex3->Eval()<<_T(", ")<<pIndex4->Eval()<<_T("]"); return os.str();
	}

	virtual std::wstring Display() { return std::wstring(); }
	virtual ~ExprAptr4() { }
};

template<typename T, typename ary>
class ExprAptr5: public LeftHandSide<T>
{
protected:
	AptrData<ary>* pData;
	ExpIntg *pIndex1, *pIndex2, *pIndex3, *pIndex4, *pIndex5;

public:
	
	ExprAptr5(AptrData<ary>* vptr, ExpIntg* idx1, ExpIntg* idx2, 
		ExpIntg* idx3, ExpIntg* idx4, ExpIntg* idx5): 
	pData(vptr), pIndex1(idx1), pIndex2(idx2), pIndex3(idx3), pIndex4(idx4), pIndex5(idx5) {}
	
	virtual T& GetData()
	{
		ary* ptr = pData->GetData();
		if(ptr)
			return ptr->at(int(pIndex1->Eval())).at(int(pIndex2->Eval())).
				at(int(pIndex3->Eval())).at(int(pIndex4->Eval())).at(int(pIndex5->Eval()));
		else
			throw UnicodeException(_T("Pointer is not initialized"));
	}

	virtual T Eval()  { return GetData(); }

	virtual const std::wstring& GetId() const { return pData->GetId(); } 

	virtual int GetIndex1() const { return int(pIndex1->Eval()); }
	virtual int GetIndex2() const { return int(pIndex2->Eval()); }
	virtual int GetIndex3() const { return int(pIndex3->Eval()); }
	virtual int GetIndex4() const { return int(pIndex4->Eval()); }
	virtual int GetIndex5() const { return int(pIndex5->Eval()); }

	virtual bool IsCongruent(VarAryBase *vptr) const 
	{
		if(	this->GetIndex1()==vptr->GetIndex1() &&
			this->GetIndex2()==vptr->GetIndex2() &&
			this->GetIndex3()==vptr->GetIndex3() &&
			this->GetIndex4()==vptr->GetIndex4() &&
			this->GetIndex5()==vptr->GetIndex5()) return true;
		else return false;
	}

	virtual std::wstring GetFullName() const
	{
		std::wostringstream os;
		os<<GetId()<<_T("[")<<pIndex1->Eval()<<_T(", ")<<pIndex2->Eval()<<_T(", ")
			<<pIndex3->Eval()<<_T(", ")<<pIndex4->Eval()<<_T(", ")<<pIndex5->Eval()<<_T("]"); return os.str();
	}

	virtual std::wstring Display() { return std::wstring(); }
	virtual ~ExprAptr5() { }
};

template<typename T> 
class VarAry1D: public VarAryBase, public LeftHandSide<T>
{
protected:
	Array1D<T>* data; ExpIntg* index1;

public:

	VarAry1D(ExpIntg* i, Array1D<T>* ptr): index1(i), data(ptr) { }

	virtual int GetIndex1() const { return int(index1->Eval()); }
	virtual const std::wstring& GetId() const { return data->GetId(); } 
	virtual std::wstring GetFullName() const
	{
		std::wostringstream os;
		os<<data->GetId()<<_T("[")<<GetIndex1()<<_T("]"); return os.str();
	}

	virtual bool IsCongruent(VarAryBase *vptr) const
	{
		if(this->GetIndex1()==vptr->GetIndex1()) return true;
		else return false;
	}

	Array1D<T>* GetDataPtr() { return data; }

	virtual T Eval() { return GetData(); }
	virtual T& GetData() { return data->at(int(index1->Eval())); }

	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		ostr<<this->GetId()<<_T("[")<<int(index1->Eval())<<_T("]");
		return ostr.str();
	}
	
	virtual ~VarAry1D() { }
};

template<typename T> 
class VarAry2D: public VarAryBase, public LeftHandSide<T>
{
protected:
	Array2D<T>* data; ExpIntg *index1, *index2;

public:
	VarAry2D(ExpIntg* i1, ExpIntg* i2, Array2D<T>* ptr): 
		index1(i1), index2(i2), data(ptr) { }

	virtual int GetIndex1() const {  return int(index1->Eval()); }
	virtual int GetIndex2() const {  return int(index2->Eval()); }
	virtual const std::wstring& GetId() const { return data->GetId(); } 
	virtual std::wstring GetFullName() const
	{
		std::wostringstream os;
		os<<data->GetId()<<_T("[")<<GetIndex1()<<_T(", ")<<GetIndex2()<<_T("]"); return os.str();
	}

	virtual bool IsCongruent(VarAryBase *vptr) const
	{
		if(this->GetIndex1()==vptr->GetIndex1() &&
			this->GetIndex2()==vptr->GetIndex2()) return true;
		else return false;
	}

	Array2D<T>* GetDataPtr() { return data; }
	virtual T Eval() { return GetData(); }
	virtual T& GetData() 
		{ return data->at(int(index1->Eval()), int(index2->Eval())); }

	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		ostr<<this->GetId()<<_T("[")<<int(index1->Eval())<<_T(" , ")<<int(index2->Eval())<<_T("]");
		return ostr.str();
	}
	
	virtual ~VarAry2D() { }
};

template<typename T> 
class VarAry3D: public VarAryBase, public LeftHandSide<T>
{
protected:
	Array3D<T>* data;
	ExpIntg *index1, *index2, *index3;

public:

	VarAry3D(ExpIntg* i1,
		ExpIntg* i2, ExpIntg* i3, Array3D<T>* ptr):
	    index1(i1), index2(i2), index3(i3), data(ptr) { }

	Array3D<T>* GetDataPtr() { return data; }

	virtual const std::wstring& GetId() const { return data->GetId(); } 
	virtual std::wstring GetFullName() const
	{
		std::wostringstream os;
		os<<data->GetId()<<_T("[")<<GetIndex1()<<_T(", ")<<GetIndex2()<<_T(", ")
			<<GetIndex3()<<_T("]"); return os.str();
	}

	virtual int GetIndex1() const { return int(index1->Eval()); }
	virtual int GetIndex2() const { return int(index2->Eval()); }
	virtual int GetIndex3() const { return int(index3->Eval()); }
	virtual bool IsCongruent(VarAryBase *vptr) const
	{
		if(	this->GetIndex1()==vptr->GetIndex1() &&
			this->GetIndex2()==vptr->GetIndex2() &&
			this->GetIndex3()==vptr->GetIndex3()) return true;
		else return false;
	}

	virtual T Eval() { return GetData(); }
	virtual T& GetData() 
	{ return data->at(int(index1->Eval()), int(index2->Eval()), int(index3->Eval())); }

	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		ostr<<this->GetId()<<_T("[")<<int(index1->Eval())<<_T(" , ")<<int(index2->Eval())<<_T(" , ")<<int(index3->Eval())<<_T("]");
		return ostr.str();
	}
	
	virtual ~VarAry3D() { }
};

template<typename T> 
class VarAry4D: public VarAryBase, public LeftHandSide<T>
{
protected:
	Array4D<T>* data;
	ExpIntg *index1, *index2, *index3, *index4;

public:

	VarAry4D(ExpIntg* i1, ExpIntg* i2,
		ExpIntg* i3, ExpIntg* i4, Array4D<T>* ptr):
	    index1(i1), index2(i2), index3(i3), index4(i4), data(ptr) { }

	Array4D<T>* GetDataPtr() { return data; }
	
	virtual const std::wstring& GetId() const { return data->GetId(); } 
	virtual std::wstring GetFullName() const
	{
		std::wostringstream os;
		os<<data->GetId()<<_T("[")<<GetIndex1()<<_T(", ")<<GetIndex2()<<_T(", ")
			<<GetIndex3()<<_T(", ")<<GetIndex4()<<_T("]"); return os.str();
	}
	virtual int GetIndex1() const { return int(index1->Eval()); }
	virtual int GetIndex2() const { return int(index2->Eval()); }
	virtual int GetIndex3() const { return int(index3->Eval()); }
	virtual int GetIndex4() const { return int(index4->Eval()); }
	virtual bool IsCongruent(VarAryBase *vptr) const
	{
		if(	this->GetIndex1()==vptr->GetIndex1() &&
			this->GetIndex2()==vptr->GetIndex2() &&
			this->GetIndex3()==vptr->GetIndex3() &&
			this->GetIndex4()==vptr->GetIndex4()) return true;
		else return false;
	}

	virtual T Eval() { return GetData(); }
	virtual T& GetData() 
	{
		return data->at(int(index1->Eval()), int(index2->Eval()),
			int(index3->Eval()), int(index4->Eval())); 
	}

	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		ostr<<this->GetId()<<_T("[")<<int(index1->Eval())<<_T(" , ")<<int(index2->Eval())<<_T(" , ")
					   <<int(index3->Eval())<<_T(" , ")<<int(index4->Eval())<<_T("]");
		return ostr.str();
	}
	
	virtual ~VarAry4D() { }
};


template<typename T> 
class VarAry5D: public VarAryBase, public LeftHandSide<T>
{
protected:
	Array5D<T>* data;
	ExpIntg *index1, *index2, *index3, *index4, *index5;

public:

	VarAry5D(ExpIntg* i1,
		ExpIntg* i2, ExpIntg* i3, 
		ExpIntg* i4, ExpIntg* i5, Array5D<T>* ptr):
		index1(i1), index2(i2), index3(i3), index4(i4), index5(i5), data(ptr) { }

	Array5D<T>* GetDataPtr() { return data; }

	virtual const std::wstring& GetId() const { return data->GetId(); } 
	virtual std::wstring GetFullName() const
	{
		std::wostringstream os;
		os<<data->GetId()<<_T("[")<<GetIndex1()<<_T(", ")<<GetIndex2()<<_T(", ")
			<<GetIndex3()<<_T(", ")<<GetIndex4()<<_T(", ")<<GetIndex5()<<_T("]"); return os.str();
	}
	virtual int GetIndex1() const { return int(index1->Eval()); }
	virtual int GetIndex2() const { return int(index2->Eval()); }
	virtual int GetIndex3() const { return int(index3->Eval()); }
	virtual int GetIndex4() const { return int(index4->Eval()); }
	virtual int GetIndex5() const { return int(index5->Eval()); }
	virtual bool IsCongruent(VarAryBase *vptr) const 
	{
		if(	this->GetIndex1()==vptr->GetIndex1() &&
			this->GetIndex2()==vptr->GetIndex2() &&
			this->GetIndex3()==vptr->GetIndex3() &&
			this->GetIndex4()==vptr->GetIndex4() &&
			this->GetIndex5()==vptr->GetIndex5()) return true;

		else return false;
	}

	virtual T Eval() { return GetData(); }
	virtual T& GetData() 
	{
		return data->at(int(index1->Eval()), int(index2->Eval()),
			int(index3->Eval()), int(index4->Eval()), int(index5->Eval())); 
	}

	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		ostr<<this->GetId()<<_T("[")<<int(index1->Eval())<<_T(" , ")<<int(index2->Eval())<<_T(" , ")
					   <<int(index3->Eval())<<_T(" , ")<<int(index4->Eval())<<_T(" , ")
					   <<int(index5->Eval())<<_T("]");
		return ostr.str();
	}
	
	virtual ~VarAry5D() { }
};


template<typename T, typename vec> 
class VarAryEle1D: public VarAryBase, public LeftHandSide<T>
{
protected:
	LeftHandSide<vec> *pVec; ExpIntg* index1;

public:

	VarAryEle1D(LeftHandSide<vec> *ptr, ExpIntg* idx1): 
	  pVec(ptr), index1(idx1) { }

	virtual int GetIndex1() const { return int(index1->Eval()); }
	virtual const std::wstring& GetId() const { return pVec->GetId(); } 
	virtual std::wstring GetFullName() const 
	{ 
		std::wostringstream os;
		os<<pVec->GetFullName()<<L"["<<GetIndex1()<<L"]"; 
		return os.str();
	}

	virtual bool IsCongruent(VarAryBase *vptr) const
	{
		if(this->GetIndex1()==vptr->GetIndex1()) return true;
		else return false;
	}

	virtual T& GetData() { return pVec->GetData().at(int(index1->Eval())); }
	virtual T Eval() { return GetData(); }
	
	virtual std::wstring Display() { return GetFullName(); }
	
	virtual ~VarAryEle1D() { }
};


template<typename T, typename vec> 
class VarAryEle2D: public VarAryBase, public LeftHandSide<T>
{
protected:
	LeftHandSide<vec> *pVec; ExpIntg *index1, *index2;

public:
	VarAryEle2D(LeftHandSide<vec> *ptr, ExpIntg* idx1, ExpIntg* idx2): 
	  pVec(ptr), index1(idx1), index2(idx2) { }

	virtual int GetIndex1() const {  return int(index1->Eval()); }
	virtual int GetIndex2() const {  return int(index2->Eval()); }
	virtual const std::wstring& GetId() const { return pVec->GetId(); } 
	virtual std::wstring GetFullName() const 
	{ 
		std::wostringstream os;
		os<<pVec->GetFullName()<<L"["<<GetIndex1()<<L" , "<<GetIndex2()<<L"]"; 
		return os.str();
	}

	virtual bool IsCongruent(VarAryBase *vptr) const
	{
		if(this->GetIndex1()==vptr->GetIndex1() &&
			this->GetIndex2()==vptr->GetIndex2()) return true;
		else return false;
	}

	virtual T& GetData() 
		{ return pVec->GetData().at(int(index1->Eval())).at(int(index2->Eval())); }

	virtual T Eval() { return GetData(); }
	
	virtual std::wstring Display() { return GetFullName(); }
	
	virtual ~VarAryEle2D() { }
};

template<typename T, typename vec> 
class VarAryEle3D: public VarAryBase, public LeftHandSide<T>
{
protected:
	LeftHandSide<vec> *pVec; ExpIntg *index1, *index2, *index3;

public:

	VarAryEle3D(LeftHandSide<vec> *ptr, ExpIntg* idx1, 
		ExpIntg* idx2, ExpIntg* idx3): 
	  pVec(ptr), index1(idx1), index2(idx2), index3(idx3) { }

	virtual const std::wstring& GetId() const { return pVec->GetId(); } 
	virtual std::wstring GetFullName() const
	{
		std::wostringstream os;
		os<<pVec->GetFullName()<<L"["<<GetIndex1()<<L" , "<<GetIndex2()
			<<L" , "<<GetIndex3()<<L"]";

		return os.str();
	}

	virtual int GetIndex1() const { return int(index1->Eval()); }
	virtual int GetIndex2() const { return int(index2->Eval()); }
	virtual int GetIndex3() const { return int(index3->Eval()); }
	virtual bool IsCongruent(VarAryBase *vptr) const
	{
		if(	this->GetIndex1()==vptr->GetIndex1() &&
			this->GetIndex2()==vptr->GetIndex2() &&
			this->GetIndex3()==vptr->GetIndex3()) return true;
		else return false;
	}

	virtual T& GetData() 
	{ return pVec->GetData().at(int(index1->Eval())).at(int(index2->Eval())).at(int(index3->Eval())); }

	virtual T Eval() { return GetData(); }
	
	virtual std::wstring Display()  { return GetFullName(); }
		
	virtual ~VarAryEle3D() { }
};

template<typename T, typename vec> 
class VarAryEle4D: public VarAryBase, public LeftHandSide<T>
{
protected:
	LeftHandSide<vec> *pVec; 
	ExpIntg *index1, *index2, *index3, *index4;

public:

	VarAryEle4D(LeftHandSide<vec> *ptr, ExpIntg* idx1, 
		ExpIntg* idx2, ExpIntg* idx3, ExpIntg* idx4): 
	  pVec(ptr), index1(idx1), index2(idx2), index3(idx3), index4(idx4) { }

	virtual const std::wstring& GetId() const { return pVec->GetId(); } 
	virtual std::wstring GetFullName() const
	{
		std::wostringstream os;
		os<<pVec->GetFullName()<<L"["<<GetIndex1()<<L" , "<<GetIndex2()
			<<L" , "<<GetIndex3()<<L" , "<<GetIndex4()<<L"]";

		return os.str();
	}
	virtual int GetIndex1() const { return int(index1->Eval()); }
	virtual int GetIndex2() const { return int(index2->Eval()); }
	virtual int GetIndex3() const { return int(index3->Eval()); }
	virtual int GetIndex4() const { return int(index4->Eval()); }
	virtual bool IsCongruent(VarAryBase *vptr) const
	{
		if(	this->GetIndex1()==vptr->GetIndex1() &&
			this->GetIndex2()==vptr->GetIndex2() &&
			this->GetIndex3()==vptr->GetIndex3() &&
			this->GetIndex4()==vptr->GetIndex4()) return true;
		else return false;
	}

	virtual T& GetData() 
	{
		return pVec->GetData().at(int(index1->Eval())).at(int(index2->Eval())).
			at(int(index3->Eval())).at(int(index4->Eval())); 
	}

	virtual T Eval() { return GetData(); }

	virtual std::wstring Display() { return GetFullName(); }
	
	virtual ~VarAryEle4D() { }
};

template<typename T, typename vec> 
class VarAryEle5D: public VarAryBase, public LeftHandSide<T>
{
protected:
	LeftHandSide<vec> *pVec; 
	ExpIntg *index1, *index2, *index3, *index4, *index5;

public:

	VarAryEle5D(LeftHandSide<vec> *ptr, ExpIntg* idx1, 
		ExpIntg* idx2, ExpIntg* idx3, 
		ExpIntg* idx4, ExpIntg* idx5): 
	  pVec(ptr), index1(idx1), index2(idx2),
		  index3(idx3), index4(idx4), index5(idx5) { }
	
	virtual const std::wstring& GetId() const { return pVec->GetId(); } 
	virtual std::wstring GetFullName() const
	{
		std::wostringstream os;
		os<<pVec->GetFullName()<<L"["<<GetIndex1()<<L" , "<<GetIndex2()
			<<L" , "<<GetIndex3()<<L" , "<<GetIndex4()<<L" , "
			<<GetIndex5()<<L"]";

		return os.str();
	}
	virtual int GetIndex1() const { return int(index1->Eval()); }
	virtual int GetIndex2() const { return int(index2->Eval()); }
	virtual int GetIndex3() const { return int(index3->Eval()); }
	virtual int GetIndex4() const { return int(index4->Eval()); }
	virtual int GetIndex5() const { return int(index5->Eval()); }
	virtual bool IsCongruent(VarAryBase *vptr) const 
	{
		if(	this->GetIndex1()==vptr->GetIndex1() &&
			this->GetIndex2()==vptr->GetIndex2() &&
			this->GetIndex3()==vptr->GetIndex3() &&
			this->GetIndex4()==vptr->GetIndex4() &&
			this->GetIndex5()==vptr->GetIndex5()) return true;
		else return false;
	}

	virtual T& GetData() 
	{
		return pVec->GetData().at(int(index1->Eval())).at(int(index2->Eval()))
			.at(int(index3->Eval())).at(int(index4->Eval())).at(int(index5->Eval())); 
	}

	virtual T Eval() { return GetData(); }

	virtual std::wstring Display() { return GetFullName(); }
		
	virtual ~VarAryEle5D() { }
};

template<typename vec, typename ary> 
class VaArray: public LeftHandSideArray<vec>
{
protected:
	ary *data;

public:
	
	VaArray(ary* ptr): data(ptr) { }

	virtual vec Eval() { return GetData(); }
	virtual vec& GetData() { return *data->GetTop(); }

	virtual const std::wstring& GetId() const { return data->GetId(); } 
	virtual std::wstring GetFullName() const { return data->GetId(); }

	ary* GetDataPtr() { return data; }
		
	virtual ~VaArray() { }
};

template<typename T, typename vec> 
class MatrixPro: public Expression<vec>
{
protected:
	Expression<vec> *data1, *data2;; 

public:
	
	MatrixPro(Expression<vec>* aptr1, Expression<vec>* aptr2):
		data1(aptr1), data2(aptr2){ }

	virtual vec Eval() 
	{ 
		vec v; MatPro<T>(v, data1->Eval(), data2->Eval());
		return v;
	}

	virtual std::wstring Display() 
	{
		//std::wostringstream ostr;
		//ostr<<vrid<<_T("[]";
		return std::wstring();
	}
	
	virtual ~MatrixPro() { }
};

template<typename vec> 
class ArrayBin: public Expression<vec>
{
protected:
	Expression<vec> *data1, *data2; char optr; 

public:
	
	ArrayBin(Expression<vec>* aptr1, Expression<vec>* aptr2, char op):
		data1(aptr1), data2(aptr2), optr(op){ }

	virtual vec Eval() 
	{ 
		switch(optr)
		{
		case '*': return data1->Eval() * data2->Eval();
		case '/': return data1->Eval() / data2->Eval();
		case '+': return data1->Eval() + data2->Eval();
		case '-': return data1->Eval() - data2->Eval();
		
		default:
			throw UnicodeException(_T("Operation not allowed"));
		}
	}
	
	virtual ~ArrayBin() { }
};

template<typename T, typename vec> 
class MinMaxEle: public Expression<T>
{
protected:
	Expression<vec> *vPtr; char optr; 

public:
	
	MinMaxEle(Expression<vec>* aptr, char op): vPtr(aptr), optr(op){ }

	virtual T Eval() 
	{ 
		switch(optr)
		{
		case '+': return GetMaxEle<T>(vPtr->Eval());
		case '-': return GetMinEle<T>(vPtr->Eval());
		
		default:
			throw UnicodeException(_T("Operation not allowed"));
		}
	}
	
	virtual ~MinMaxEle() { }
};

template<typename tgt, typename src> 
class DotProduct: public Expression<tgt>
{
protected:
	Expression<src> *data1, *data2; 

public:
	
	DotProduct(Expression<src>* aptr1, Expression<src>* aptr2):
		data1(aptr1), data2(aptr2) { }

	virtual tgt Eval() 
	{ 
		tgt rlt; DotPro(rlt, data1->Eval(), data2->Eval());  return rlt;
	}

	virtual ~DotProduct() { }
};

template<typename tgt, typename src> 
class SDotProduct: public Expression<tgt>
{
protected:
	Expression<src> *vaPtr; ExpVctr *pExpr; bool left;

public:
	
	SDotProduct(Expression<src>* aptr, ExpVctr* xptr):
	  vaPtr(aptr), pExpr(xptr), left(true) { }
	
	SDotProduct(ExpVctr* xptr, Expression<src>* aptr):
	  vaPtr(aptr), pExpr(xptr), left(false) { }

	virtual tgt Eval() 
	{ 
		tgt rlt;
		if(left)
			SDotPro(rlt, vaPtr->Eval(), pExpr->Eval());  
		else
			SDotPro(rlt, pExpr->Eval(), vaPtr->Eval());  
		
		return rlt;
	}
		
	virtual ~SDotProduct() { }
};

template<typename vec> 
class ArrayBinVec: public Expression<vec>
{
protected:
	Expression<vec> *data1, *data2; char optr; 

public:
	
	ArrayBinVec(Expression<vec>* aptr1, Expression<vec>* aptr2, char op):
		data1(aptr1), data2(aptr2), optr(op){ }

	virtual vec Eval() 
	{ 
		switch(optr)
		{
		case '%': return data1->Eval() % data2->Eval();
		case '+': return data1->Eval() + data2->Eval();
		case '-': return data1->Eval() - data2->Eval();
		
		default:
			throw UnicodeException(_T("Operation not allowed"));
		}
	}
		
	virtual ~ArrayBinVec() { }
};

template<typename vec> 
class VaVRBin: public Expression<vec>
{
protected:
	Expression<vec>* data; ExpReal *pExpr;
	char optr; bool xpr_left;

public:
	
	VaVRBin(Expression<vec>* aptr, ExpReal* xpr, char op):
		data(aptr), pExpr(xpr), optr(op), xpr_left(false) { }

	VaVRBin(ExpReal* xpr, Expression<vec>* aptr, char op):
		data(aptr), pExpr(xpr), optr(op), xpr_left(true) { }

	virtual vec Eval() 
	{ 
		vec v;
		if(xpr_left)
		{
			switch(optr)
			{
			case '*': MulScalar(v, data->Eval(), pExpr->Eval()); return v;
					
			default:
				throw UnicodeException(_T("Operation not allowed"));
			}
		}
		else
		{

			switch(optr)
			{
			case '*': MulScalar(v, data->Eval(), pExpr->Eval()); return v;
			case '/': DivScalar(v, data->Eval(), pExpr->Eval()); return v;
			
			default:
				throw UnicodeException(_T("Operation not allowed"));
			}
		}
	}

	virtual ~VaVRBin() { }
};

template<typename T, typename vec> 
class VaAryBin: public Expression<vec>
{
protected:
	Expression<vec>* data; Expression<T> *pExpr;
	char optr; bool xpr_left;

public:
	
	VaAryBin(Expression<vec>* aptr, Expression<T>* xpr, char op):
		data(aptr), pExpr(xpr), optr(op), xpr_left(false) { }

	VaAryBin(Expression<T>* xpr, Expression<vec>* aptr, char op):
		data(aptr), pExpr(xpr), optr(op), xpr_left(true) { }

	virtual vec Eval() 
	{ 
		if(xpr_left)
		{
			switch(optr)
			{
			case '*': return pExpr->Eval() * data->Eval();
			case '/': return pExpr->Eval() / data->Eval();
			case '+': return pExpr->Eval() + data->Eval(); 
			case '-': return pExpr->Eval() - data->Eval();
			
			default:
				throw UnicodeException(_T("Operation not allowed"));
			}
		}
		else
		{
			switch(optr)
			{
			case '*': return data->Eval() * pExpr->Eval();
			case '/': return data->Eval() / pExpr->Eval();
			case '+': return data->Eval() + pExpr->Eval();  
			case '-': return data->Eval() - pExpr->Eval();
			
			default:
				throw UnicodeException(_T("Operation not allowed"));
			}
		}
	}

	virtual ~VaAryBin() { }
};

template<typename vec> 
class VaAryBin<Vector, vec>: public Expression<vec>
{
protected:
	Expression<vec>* data; ExpVctr *pExpr;
	char optr; bool xpr_left;

public:
	
	VaAryBin(Expression<vec>* aptr, ExpVctr* xpr, char op):
		data(aptr), pExpr(xpr), optr(op), xpr_left(false) { }

	VaAryBin(ExpVctr* xpr, Expression<vec>* aptr, char op):
		data(aptr), pExpr(xpr), optr(op), xpr_left(true) { }

	virtual vec Eval() 
	{ 
		if(xpr_left)
		{
			switch(optr)
			{
			case '+': return pExpr->Eval() + data->Eval(); 
			case '-': return pExpr->Eval() - data->Eval();
			case '%': 
				{
					vec v; CrossPro(v, pExpr->Eval(), data->Eval());
					return v;
				}
			
			default:
				throw UnicodeException(_T("Operation not allowed"));
			}
		}
		else
		{
			switch(optr)
			{
			case '+': return data->Eval() + pExpr->Eval();  
			case '-': return data->Eval() - pExpr->Eval();
			case '%': 
				{
					vec v; CrossPro(v, data->Eval(), pExpr->Eval());
					return v;
				}
			
			default:
				throw UnicodeException(_T("Operation not allowed"));
			}
		}
	}
	
	virtual ~VaAryBin() { }
};

template<typename T> 
class Literal: public Expression<T>
{
protected:
	T val;
public:
	Literal(T v=T()): val(v) { bLiteral=true; }

	virtual T Eval() { return val; }
	
	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		ostr<<val;
		return ostr.str();
	}
	
	virtual ~Literal() { }
};

template<> 
class Literal<Complex>: public ExpCplx
{
protected:
	ExpReal* lhs, *rhs; Complex rtn;

	void cache()
	{
		if(lhs->bLiteral && rhs->bLiteral)
		{	
			rtn = Complex(lhs->Eval(), rhs->Eval());
			bLiteral = true;
		}
		else bLiteral = false;
	}
public:
	Literal(ExpReal* l, ExpReal* r): 
	  lhs(l), rhs(r) { cache(); }

	virtual Complex Eval() 
	{
		if(bLiteral) return rtn;
		else return Complex(lhs->Eval(), rhs->Eval());
	}

	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		ostr<<Complex(lhs->Eval(), (rhs == NULL) ? 0: rhs->Eval());
		return ostr.str();
	}
		
	virtual ~Literal() { }
};

template<> 
class Literal<Vector>: public ExpVctr
{
protected:
	ExpReal *x, *y, *z; Vector rtn;
	
	void cache()
	{
		if(x->bLiteral && y->bLiteral && z->bLiteral)
		{
			rtn = Vector(x->Eval(), y->Eval(), z->Eval());
			bLiteral = true;
		}
		else bLiteral = false;
	}

public:
	Literal(ExpReal* i, 
		ExpReal* j, ExpReal* k): 
	  x(i), y(j), z(k) { cache(); }

	virtual Vector Eval() 
	{ 
		if(bLiteral) return rtn;  
		else return Vector(x->Eval(), y->Eval(), z->Eval());
	}

	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		ostr<<Vector(x->Eval(), y->Eval(), z->Eval() );
		return ostr.str();
	}
		
	virtual ~Literal() { }
};

template<typename T> 
class UnaryOpr: public Expression<T>
{
protected:
	Expression<T> *expr; int optr; // operator
	T rtn;
	void cache()
	{
		if(expr->bLiteral)
		{
			rtn = this->Eval(); bLiteral = true;
		}
	}

public:
	UnaryOpr(int opr, Expression<T>* xpr): optr(opr), expr(xpr) { cache(); }

	virtual T Eval()
	{
		if(bLiteral) return rtn;

		switch(optr)
		{
		case '-': return -expr->Eval();
		default:
			throw BaseType::BaseException(IDS_OPR_NOT_SUPPORTED);
		}
	}

	virtual T EvalPlus() { return Eval() + T(1E-6); }
	virtual T EvalMinus() { return Eval() - T(1E-6); }

	virtual void GetVariables(DataSet& dataset)
	{
		if(bLiteral) return;
		expr->GetVariables(dataset);
	}

	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		UniChar unicode;
		unicode.byte.low = optr; unicode.byte.high = '\0';
		ostr<<_T('(')<<unicode.wchar<<expr->Display()<<_T(')');
		return ostr.str();
	}

	virtual ~UnaryOpr() { }
};


template<> 
class UnaryOpr<Vector>: public Expression<Vector>
{
protected:
	Expression<Vector> *expr; int optr; // operator
	Vector rtn;

	void cache()
	{
		if(expr->bLiteral)
		{
			rtn = this->Eval(); bLiteral = true;
		}
	}

public:
	UnaryOpr(int opr, Expression<Vector>* xpr): optr(opr), expr(xpr) { cache(); }

	virtual Vector Eval()
	{
		if(bLiteral) return rtn;

		switch(optr)
		{
		case '-': return -expr->Eval();
		case '~': return getunitvector(expr->Eval());
		default:
			{
				std::wostringstream os; UniChar unicode;
				unicode.byte.low = optr; unicode.byte.high = '\0';
				os<<L"Unary operator \'"<<unicode.wchar<<L"\' for vector is not supported.\r\n";
				HandleSyntaxError(os.str());
				
				return rtn;	// to suppress warning
			}
		}
	}
	virtual void GetVariables(DataSet& dataset)
	{
		if(bLiteral) return;
		expr->GetVariables(dataset);
	}

	virtual std::wstring Display() 
	{
		std::wostringstream ostr; UniChar unicode;
		unicode.byte.low = optr; unicode.byte.high = '\0';
		ostr<<_T('(')<<unicode.wchar<<expr->Display()<<_T(')');
		return ostr.str();
	}

	virtual ~UnaryOpr() { }
};

template<> 
class UnaryOpr<Complex>: public Expression<Complex>
{
protected:
	Expression<Complex> *expr; int optr; // operator
	Complex rtn;

	void cache()
	{
		if(expr->bLiteral)
		{
			rtn = this->Eval(); bLiteral = true;
		}
	}

public:
	UnaryOpr(int opr, Expression<Complex>* xpr): optr(opr), expr(xpr) { cache(); }

	virtual Complex Eval()
	{
		if(bLiteral) return rtn;

		switch(optr)
		{
		case '-': return -expr->Eval();
		case '~':
			{
				rtn = expr->Eval(); rtn.get_imag() = - rtn.imag();
				
				return rtn;
			}
		default:
			{
				std::wostringstream os; UniChar unicode; 
				unicode.byte.low = optr; unicode.byte.high='\0';

				os<<L"Unary operator \'"<<unicode.wchar
					<<L"\' for complex number is not supported.\r\n";
				HandleSyntaxError(os.str());

				return rtn; // to suppress warning
			}
		}
	}
	
	virtual void GetVariables(DataSet& dataset)
	{
		if(bLiteral) return;
		expr->GetVariables(dataset);
	}

	virtual std::wstring Display() 
	{
		std::wostringstream ostr; UniChar unicode;
		unicode.byte.low = optr; unicode.byte.high = '\0';
		ostr<<_T('(')<<unicode.wchar<<expr->Display()<<_T(')');
		return ostr.str();
	}

	virtual ~UnaryOpr() { }
};

class UnaryBoolOpr: public ExpBool
{
protected:
	ExpBool *expr;
	int optr; // operator
	bool rtn;
	void cache()
	{
		if(expr->bLiteral)
		{
			rtn = this->Eval(); bLiteral = true;
		}
	}

public:
	UnaryBoolOpr(int opr, ExpBool* xpr): optr(opr), expr(xpr) { cache(); }

	virtual bool Eval() 
	{
		if(bLiteral) return rtn;

		switch(optr)
		{
		case '!': return !expr->Eval();
		default:
			throw BaseType::BaseException(IDS_OPR_NOT_SUPPORTED);
		}
	}

	virtual void GetVariables(DataSet& dataset)
	{
		if(bLiteral) return;
		expr->GetVariables(dataset);
	}

	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		UniChar unicode; unicode.byte.low = optr; unicode.byte.high = '\0';
		ostr<<_T('(')<<unicode.wchar<<expr->Display()<<_T(')');
		return ostr.str();
	}

	virtual ~UnaryBoolOpr() { }
};

class Factorial: public ExpIntg
{
protected:
	ExpIntg *expr;
	integer rtn;
	void cache();
public:
	Factorial(ExpIntg* xpr);

	virtual integer Eval();
	virtual std::wstring Display();
	virtual ~Factorial();
};

class Absolute: public ExpIntg
{
protected:
	ExpIntg *expr;
public:
	Absolute(ExpIntg* xpr);
	virtual integer Eval();
	virtual std::wstring Display();
	virtual ~Absolute();
};

class Permutation: public ExpIntg
{
protected:
	ExpIntg *nXpr;
	ExpIntg *rXpr;
	integer rtn;
	void cache();
public:
	Permutation(ExpIntg* n, ExpIntg* r);

	virtual integer Eval();
	virtual std::wstring Display();
	virtual ~Permutation();
};

class Combination: public ExpIntg
{
protected:
	ExpIntg *nXpr;
	ExpIntg *rXpr;
	integer rtn;

	void cache();

public:
	Combination(ExpIntg* n, ExpIntg* r);

	virtual integer Eval();
	virtual std::wstring Display() ;
	virtual ~Combination();
};

class Gamma: public ExpReal
{
protected:
	ExpReal *expr;
public:
	Gamma(ExpReal* xpr);
	virtual double Eval();
	virtual std::wstring Display();
	virtual ~Gamma();
};

template<typename T> 
class BinOpr: public Expression<T>
{
protected:
	int optr; // operator
	Expression<T> *lhs, *rhs; T rtn;

	void cache()
	{
		if(lhs->bLiteral && rhs->bLiteral) 
		{
			rtn = this->Eval(); bLiteral=true;
		}
	}
public:
	BinOpr(int opr, Expression<T>* ls, Expression<T>* rs):
	  optr(opr), lhs(ls), rhs(rs) { cache();  }
	
	virtual T Eval() 
	{ 
		if(bLiteral) return rtn;
		
		T left = lhs->Eval(), right = rhs->Eval();
		
		switch(optr)
		{
		case '+': return left + right;
		case '-': return left - right;
		case '*': return left * right;
		case '/':
			{
				if(right != T())
					return left/right;
				else // right hand side is zero
					throw ExpDivisionByZero();
			}
		default:
			throw BaseType::BaseException(IDS_OPR_NOT_SUPPORTED);
		}
	}

	virtual void GetVariables(DataSet& dataset)
	{
		if(bLiteral) return;

		lhs->GetVariables(dataset);
		rhs->GetVariables(dataset);
	}
	
	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		UniChar unicode; unicode.byte.low = optr; unicode.byte.high ='\0';
		ostr<<_T('(')<<lhs->Display()<<unicode.wchar<<rhs->Display()<<_T(')');
		return ostr.str();
	}

	virtual ~BinOpr() { }
};

template<> 
class BinOpr<integer>: public Expression<integer>
{
protected:
	int optr; // operator
	Expression<integer> *lhs, *rhs; integer rtn;

	void cache()
	{
		if(lhs->bLiteral && rhs->bLiteral) 
		{
			rtn = this->Eval(); bLiteral=true;
		}
	}
public:
	BinOpr(int opr, Expression<integer>* ls, Expression<integer>* rs):
	  optr(opr), lhs(ls), rhs(rs) { cache();  }
	
	 virtual integer Eval() 
	{ 
		extern integer combination(integer, integer);
		extern integer permutation(integer, integer);
		extern integer gcd(integer, integer);
		extern integer lcm(integer, integer);

		if(bLiteral) return rtn;
		
		integer left = lhs->Eval(), right = rhs->Eval();
		
		switch(optr)
		{
		case '+': return left + right;
		case '-': return left - right;
		case '*': return left * right;
		case '/':
			{
				if(right != integer()) 
					return left/right;
				else // right hand side is zero
					throw ExpDivisionByZero();
			}
		case '%': // modulo
			return left % right;
		
		case 'C': // combination
			return combination(left, right);
		case 'P': // permutation
			return permutation(left, right);
		case 'G': // greatest common divisor
			return gcd(left, right);
		case 'L': // least common multiple
			return lcm(left, right);

		default:
			throw BaseType::BaseException(IDS_OPR_NOT_SUPPORTED);
		}
	}

	virtual void GetVariables(DataSet& dataset)
	{
		if(bLiteral) return;

		lhs->GetVariables(dataset);
		rhs->GetVariables(dataset);
	}
	
	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		UniChar unicode; unicode.byte.low = optr; unicode.byte.high ='\0';
		ostr<<_T('(')<<lhs->Display()<<unicode.wchar<<rhs->Display()<<_T(')');
		return ostr.str();
	}

	virtual ~BinOpr() { }
};

template<> 
class BinOpr<realnum>: public ExpReal
{
protected:
	int optr; // operator
	ExpReal *lhs, *rhs; realnum rtn;

	void cache()
	{
		if(lhs->bLiteral && rhs->bLiteral) 
		{
			rtn = this->Eval(); bLiteral=true;
		}
	}
public:
	BinOpr(int opr, ExpReal* ls, ExpReal* rs):
	  optr(opr), lhs(ls), rhs(rs) { cache();  }
	
	virtual realnum Eval() 
	{ 
		if(bLiteral) return rtn;
		
		realnum left = lhs->Eval(), right = rhs->Eval();
		
		switch(optr)
		{
		case '+': return left + right;
		case '-': return left - right;
		case '*': return left * right;
		case '/':
			{
				if(right != realnum())
					return left/right;
				else // right hand side is zero
					throw ExpDivisionByZero();
			}
		default:
			throw BaseType::BaseException(IDS_OPR_NOT_SUPPORTED);
		}
	}

	virtual realnum EvalPlus() 
	{ 
		if(bLiteral) return rtn;
		
		realnum left = lhs->EvalPlus(), right = rhs->EvalPlus();
		
		switch(optr)
		{
		case '+': return left + right;
		case '-': return left - right;
		case '*': return left * right;
		case '/':
			{
				if(right != realnum()) return left/right;
				else
				{
					std::wostringstream os;
					os<<LoadUnicodeMsg(IDS_DIVISION_BY_ZERO).c_str()
						<<_T(" : ")<<rhs->Display()<<_T(" = ")<<rhs->Eval();
					//throw UnicodeException(os.str());

					throw ExpDivisionByZero();
				}
			}
		default:
			throw BaseType::BaseException(IDS_OPR_NOT_SUPPORTED);
		}
	}

	virtual realnum EvalMinus() 
	{ 
		if(bLiteral) return rtn;
		
		realnum left = lhs->EvalMinus(), right = rhs->EvalMinus();
		
		switch(optr)
		{
		case '+': return left + right;
		case '-': return left - right;
		case '*': return left * right;
		case '/':
			{
				if(right != realnum())
					return left/right;
				else
				{
					std::wostringstream os;
					os<<LoadUnicodeMsg(IDS_DIVISION_BY_ZERO).c_str()
						<<_T(" : ")<<rhs->Display()<<_T(" = ")<<rhs->Eval();
					//throw UnicodeException(os.str());

					throw ExpDivisionByZero();
				}
			}
		default:
			throw BaseType::BaseException(IDS_OPR_NOT_SUPPORTED);
		}
	}

	virtual void GetVariables(DataSet& dataset)
	{
		if(bLiteral) return;

		lhs->GetVariables(dataset);
		rhs->GetVariables(dataset);
	}
	
	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		UniChar unicode; unicode.byte.low = optr; unicode.byte.high ='\0';
		ostr<<_T('(')<<lhs->Display()<<unicode.wchar<<rhs->Display()<<_T(')');
		return ostr.str();
	}

	virtual ~BinOpr() { }
};

template<typename T> 
class BinBoolOpr: public ExpBool
{
protected:
	int optr; // operator
	Expression<T> *lhs, *rhs;

	bool rtn;

	void cache()
	{
		if(lhs->bLiteral && rhs->bLiteral)
		{
			rtn = this->Eval(); bLiteral = true;
		}
	}
	
public:
	BinBoolOpr(int opr, Expression<T>* ls, Expression<T>* rs):
	  optr(opr), lhs(ls), rhs(rs) { cache(); }
	
	virtual bool Eval() 
	{ 
		if(bLiteral) return rtn;

		switch(optr)
		{
		case '>': return lhs->Eval()  > rhs->Eval(); 
		case '<': return lhs->Eval()  < rhs->Eval();
		case '(': return lhs->Eval() <= rhs->Eval(); 
		case ')': return lhs->Eval() >= rhs->Eval(); 
		case '$': return lhs->Eval() == rhs->Eval(); 
		case '!': return lhs->Eval() != rhs->Eval();
			 
		default:
			throw BaseType::BaseException(IDS_OPR_NOT_SUPPORTED);
		}
	}

	virtual void GetVariables(DataSet& dataset)
	{
		if(bLiteral) return;

		lhs->GetVariables(dataset);
		rhs->GetVariables(dataset);
	}
	
	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		UniChar unicode; unicode.byte.low = optr; unicode.byte.high = '\0';
		ostr<<_T('(')<<lhs->Display()<<unicode.wchar<<rhs->Display()<<_T(')');
		return ostr.str();
	}

	virtual ~BinBoolOpr() { }
};

class Relation: public ExpBool
{
protected:
	int optr; // operator
	ExpBool *lhs, *rhs;
	
public:
	Relation(int opr, ExpBool* ls, ExpBool* rs):
	  optr(opr), lhs(ls), rhs(rs) { /* empty */ }
	
	virtual bool Eval() 
	{ 
		switch(optr)
		{
		case '|': return lhs->Eval() || rhs->Eval();
		case '&': return lhs->Eval() && rhs->Eval();
	 
		default:
			throw BaseType::BaseException(IDS_OPR_NOT_SUPPORTED);
		}
	}
	
	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		UniChar unicode; unicode.byte.low = optr; unicode.byte.high='\0';
		ostr<<_T('(')<<lhs->Display()<<unicode.wchar<<rhs->Display()<<_T(')');
		return ostr.str();
	}

	virtual ~Relation() { }
};

class ModOpr: public BinOpr<integer>
{
public:
	ModOpr(ExpIntg* ls, ExpIntg* rs):
	  BinOpr('%', ls, rs) { /* empty */ }
	
	virtual integer Eval() 
	{ 
		return lhs->Eval() % rhs->Eval();
	}
	
	virtual ~ModOpr() { }
};

template<> 
class BinOpr<Vector>: public ExpVctr
{
protected:
	int optr; // operator
	ExpVctr *lhs, *rhs; Vector rtn;
	void cache()
	{
		if(lhs->bLiteral && rhs->bLiteral) 
		{
			rtn = this->Eval(); bLiteral=true;
		}
	}
	
public:
	BinOpr(int opr, ExpVctr* ls, ExpVctr* rs):
	  optr(opr), lhs(ls), rhs(rs) { cache(); }
	
	virtual Vector Eval() 
	{ 
		if(bLiteral) return rtn;

		switch(optr)
		{
		case '+': return lhs->Eval() + rhs->Eval();
		case '-': return lhs->Eval() - rhs->Eval();
		case '%': return lhs->Eval() % rhs->Eval(); // cross product
		default:
			throw BaseType::BaseException(IDS_OPR_NOT_SUPPORTED);
		}
	}

	virtual void GetVariables(DataSet& dataset)
	{
		if(bLiteral) return;

		lhs->GetVariables(dataset);
		rhs->GetVariables(dataset);
	}
	
	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		UniChar unicode; unicode.byte.low = optr; unicode.byte.high = '\0';
		ostr<<_T('(')<<lhs->Display()<<unicode.wchar<<rhs->Display()<<_T(')');
		return ostr.str();
	}

	virtual ~BinOpr() { }
};

template<> 
class BinOpr<string>: public ExpStrn
{
protected:
	int optr; // operator
	ExpStrn *lhs, *rhs;
	string rtn;
	void cache()
	{
		if(lhs->bLiteral && rhs->bLiteral) 
		{
			rtn = this->Eval(); bLiteral=true;
		}
	}
	
public:
	BinOpr(int opr, ExpStrn* ls, ExpStrn* rs):
	  optr(opr), lhs(ls), rhs(rs) { cache(); }
	
	virtual string Eval() 
	{ 
		if(bLiteral) return rtn;
	
		string ls = lhs->Eval();
		string rs = rhs->Eval();

		switch(optr)
		{
		case '+': return ls + rs;
		
		default:
			throw BaseType::BaseException(IDS_OPR_NOT_SUPPORTED);
		}
	}

	virtual void GetVariables(DataSet& dataset)
	{
		if(bLiteral) return;

		lhs->GetVariables(dataset);
		rhs->GetVariables(dataset);
	}
	
	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		UniChar unicode; unicode.byte.low = optr; unicode.byte.high = '\0';
		ostr<<_T('(')<<lhs->Display()<<unicode.wchar<<rhs->Display()<<_T(')');
		return ostr.str();
	}

	virtual ~BinOpr() { }
};

// return vector
// scalar op vector
class RVBinOpr: public ExpVctr
{
protected:
	ExpReal* s;
	ExpVctr *lhs, *rhs;
	int optr;
	
public:
	RVBinOpr(int opr, ExpReal* scalar, ExpVctr* vctr);
   	RVBinOpr(int opr, ExpVctr* vctr, ExpReal* scalar);
	 
    virtual Vector Eval();
	virtual std::wstring Display();
	virtual ~RVBinOpr();
};

// vector dot product
class VVDotOpr: public ExpReal
{
protected:
	ExpVctr *lhs, *rhs;
	realnum rtn;

	void cache();
public:
	VVDotOpr(ExpVctr* left, ExpVctr* right);
    
	 virtual realnum Eval();
	
	virtual std::wstring Display();
	virtual ~VVDotOpr();
};

template<typename T, typename vec>
class ExpDet: public Expression<T>
{
protected:
	Expression<vec> *pExpr;

public:
	ExpDet(Expression<vec>* xpr): pExpr(xpr){}

	virtual T Eval()
	{
		vec V = pExpr->Eval();
		int rows, cols;
		GetVecSize<T>(rows, cols, V);

		if(rows>cols || rows<2)
			throw UnicodeException(_T("Invalid Matrix for the evaluation of determinant\r\n"));
		
		SqrArray<T> A(V);
		return Determinant<T>(A);
	}
	
	virtual std::wstring Display() 
	{
		std::wostringstream ostr;
		ostr<<'|'<<pExpr->Display()<<'|';
		return ostr.str();
	}

	virtual ~ExpDet() {}
};

template<typename T, typename rtn, typename vec>
class SolveCramer: public Expression<rtn>
{
protected:
	Expression<vec>* vXpr; 
	
public:
	SolveCramer(Expression<vec>* xpr): vXpr(xpr){ }

	virtual rtn Eval()
	{
		vec M = vXpr->Eval();
		int rows, cols;
		GetVecSize<T>(rows, cols, M);

		SqrArray<T> D(M);
		
		T det  = Determinant<T>(D); 

		if(det == T()) 
			throw UnicodeException(_T("Solution does not exist: Determinant is zero\r\n"));
			
		rtn R(rows); SqrArray<T> A(rows);

		for(int j=0; j<rows; ++j)
		{
			SET_USER_ABORT;

			A = D; for(int i=0; i<rows; ++i) 
				    A(i,j) = M[i][rows]; 

			R[j] = Determinant(A)/det;
		}
		
		return R;
	}

	virtual ~SolveCramer() {}
};

template<typename T, typename vec>
class SolveGaussJordan: public Expression<vec>
{
protected:
	Expression<vec>* vXpr; 
	
public:
	SolveGaussJordan(Expression<vec>* xpr): vXpr(xpr){ }

	virtual vec Eval()
	{
		vec A = vXpr->Eval();
		int M, N;
		GetVecSize<T>(M, N, A);
		int n=N-1;

		for(int i=0; i<M; ++i)
		{
			SET_USER_ABORT;

			T P=T(); int q = 0;

			for(int j=0; j<n; ++j)
			{
				SET_USER_ABORT;

				if(Magnitude(A[i][j]) > Magnitude(P))
				{
					q = j; P = A[i][j];
				}
			}

			if(Magnitude(P)<1e-10) continue;

			// Normalize equation
			for(int j=0; j<N; ++j)
			{
				SET_USER_ABORT;
				A[i][j] /= P;
			}

			// Pivot on coefficient no. Q in Equation I
			for(int k=0; k<M; ++k)
			{
				if(k==i) continue;
				P = A[k][q];
				for(int j=0; j<N; ++j)
				{
					SET_USER_ABORT;
					A[k][j] -= P*A[i][j];
				}
			}
		}

		// reorder
		for(int i=0; i<M; ++i)
		{
			T tmp;
			if(A[i][i]==T())
			{
				for(int k=i+1; k<M; ++k)
				{
					if(A[k][i] != T())
					{
						for(int j=0; j<N; ++j)
						{
							SET_USER_ABORT;
							tmp=A[i][j]; A[i][j] = A[k][j]; A[k][j]=tmp;
						}
						break;
					}
				}
			}
		}

		return A;
	}

	virtual ~SolveGaussJordan() {}
};

extern void makeunit(Array *pAry, int size);

template<typename T, typename vec, typename ary>
class SolveLUDecompo: public Expression<vec>
{
protected:
	Expression<vec>* vXpr; 
	ary *pArray;
	
public:
	SolveLUDecompo(Expression<vec>* xpr, ary *pAry): vXpr(xpr), pArray(pAry){ }

	virtual vec Eval()
	{
		vec M = vXpr->Eval();
		int rows, cols;
		GetVecSize<T>(rows, cols, M);
		makeunit(pArray, rows);
		T sum1, sum2;
		SqrArray<T> L(rows), U(rows);
		
		// Step 1
		for(int i=0; i<rows; ++i)
		{
			if((i==0) && (M[0][0]==T())) // swap rows
			{
				for(int k=i+1; k<rows; ++k)
				{
					if(M[k][0]==T()) continue;
					else
					{
						T tmp1, tmp2;
						for(int l=0; l<cols; ++l)
						{
							SET_USER_ABORT;

							tmp1 = M[i][l]; M[i][l] = M[k][l]; M[k][l]=tmp1;
							
							if(l < rows)
							{
								tmp2 = pArray->at(i, l); 
								pArray->at(i, l) = pArray->at(k, l); 
								pArray->at(k, l)=tmp2;
							}
						}
						break;
					}
				}
			}

			L(i, 0) = M[i][0];
		}

		// Step 2

		for(int j=1; j<rows; ++j)
		{
			SET_USER_ABORT;
			U(0, j) = M[0][j]/M[0][0];
		}

		for(int h=1; h<rows; ++h)
		{
			// Step 3
			for(int j=h; j<rows; ++j)
			{
				for(int i=j; i<rows; ++i)
				{
					sum1 = T(); 
					for(int k=0; k<j; ++k) 
					{
						SET_USER_ABORT;
						sum1+= L(i, k)*U(k, j);
					}

					L(i, j) = M[i][j] - sum1;
					
					if(i==j && L(i,j)==0)
						throw UnicodeException(_T("Diagonal element of Lower Matrix in LU Decomposition is zero.\r\n"));
				}
			}

			// Step 4
			for(int i=h; i<rows; ++i)
			{
				for(int j=i+1; j<rows; ++j)
				{
					sum2 = T();
					for(int k=0; k<i; ++k)
					{
						SET_USER_ABORT;
						sum2 += L(i, k)*U(k, j);
					}

					U(i, j) = (M[i][j]-sum2)/L(i, i);
				}
			}
		}

		for(int i=0; i<rows; ++i)
		{
			for(int j=0; j<rows; ++j)
			{
				SET_USER_ABORT;

				if(i>=j) M[i][j] = L(i, j);
				else M[i][j] = U(i, j);
			}
		}
		
		return M;
	}

	virtual ~SolveLUDecompo() {}
};

template<typename T, typename vec>
class SolveInverse: public Expression<vec>
{
protected:
	Expression<vec>* vXpr; 
		
public:
	SolveInverse(Expression<vec>* xpr): vXpr(xpr){ }

	virtual vec Eval()
	{
		vec A = vXpr->Eval();
		int rows, cols;
		GetVecSize<T>(rows, cols, A);
		
		vec B; SetVecSize<T>(rows, rows, B);

		// Step 1, make B as identity matrix
		for(int i=0; i<rows; ++i)
		{
			for(int j=0; j<rows; ++j)
			{
				SET_USER_ABORT;

				if(i==j) B[i][j] = 1.0;
				else B[i][j] = T();
			}
		}

		T t, s1; int q;
		realnum p, s;
		for(int j=0; j<rows; ++j)
		{
			p=0.0;
			for(int i=j; i<rows; ++i)
			{
				SET_USER_ABORT;
				s = Magnitude(A[i][j]);
				if(s > p) { q = i; p = s;}
			}

			if(p == 0.0)
				throw UnicodeException(_T("Matrix is singular. Can not evaluate inverse matrix.\r\n"));

			// Interchange rows J and Q
			if(j != q)
			{
				for(int k=0; k<rows; ++k)
				{
					SET_USER_ABORT;
					s1 = A[j][k]; t=B[j][k];
					A[j][k] = A[q][k];
					B[j][k] = B[q][k];
					A[q][k] = s1; B[q][k] = t;
				}
			}

			// normalize row j
			s1 = A[j][j];
			for(int k=0; k<rows; ++k)
			{
				SET_USER_ABORT;
				A[j][k] /= s1; B[j][k] /= s1;
			}

			// pivot on the A[j][j] coefficient. for each row except row j
			for(int i=0; i<rows; ++i)
			{
				if(i != j)
				{
					s1 = A[i][j];
					for(int k=0; k<rows; ++k)
					{
						SET_USER_ABORT;

						A[i][k] -= s1*A[j][k];
						B[i][k] -= s1*B[j][k];
					}
				}
			}
		}

		return B;
	}

	virtual ~SolveInverse() {}
};

template<typename T, typename rtn, typename vec>
class SolveLUMethod: public Expression<rtn>
{
protected:
	Expression<vec>* vXpr; 
public:
	SolveLUMethod(Expression<vec>* xpr): vXpr(xpr){ }

	virtual rtn Eval()
	{
		vec M = vXpr->Eval();
		int rows, cols;
		GetVecSize<T>(rows, cols, M);
		T sum;
		SqrArray<T> L(rows), U(rows);
		
		// Step 1
		for(int i=0; i<rows; ++i)
		{
			if((i==0) && (M[0][0]==T())) // swap rows
			{
				for(int k=i+1; k<rows; ++k)
				{
					SET_USER_ABORT;

					if(M[k][0]==T()) continue;
					else
					{
						T tmp;
						for(int l=0; l<cols; ++l)
						{
							SET_USER_ABORT;
							tmp = M[i][l]; M[i][l] = M[k][l]; M[k][l]=tmp;
						}
						break;
					}
				}
			}

			L(i, 0) = M[i][0];
		}

		// Step 2

		for(int j=1; j<rows; ++j)
		{
			SET_USER_ABORT;
			U(0, j) = M[0][j]/M[0][0];
		}

		for(int h=1; h<rows; ++h)
		{
			// Step 3
			for(int j=h; j<rows; ++j)
			{
				for(int i=j; i<rows; ++i)
				{
					sum = T(); 
					for(int k=0; k<j; ++k)
					{
						SET_USER_ABORT;
						sum+= L(i, k)*U(k, j);
					}
					L(i, j) = M[i][j] - sum;
					
					if(i==j && L(i,j)==T())
						throw UnicodeException(_T("Diagonal element of Lower Matrix in LU Decomposition is zero.\r\n"));
				}
			}

			// Step 4
			for(int i=h; i<rows; ++i)
			{
				for(int j=i+1; j<rows; ++j)
				{
					sum = T();
					for(int k=0; k<i; ++k) 
					{
						SET_USER_ABORT;
						sum += L(i, k)*U(k, j);
					}

					U(i, j) = (M[i][j]-sum)/L(i, i);
				}
			}
		}

		rtn Y(rows), X(rows);
		
		Y[0] = M[0][cols-1]/L(0, 0);
		for(int i=1; i<rows; ++i) 
		{
			sum = T();
			for(int j=0; j<i; ++j) 
			{
				SET_USER_ABORT;
				sum += L(i, j)*Y[j]; 
			}

			Y[i] = (M[i][cols-1]-sum)/L(i, i);
		}
		
		X[rows-1] = Y[rows-1];
		for(int i=rows-2; i>=0; --i)
		{
			sum = T();
			for(int j=i+1; j<rows; ++j) 
			{
				SET_USER_ABORT;
				sum+=U(i, j)*X[j];
			}

			X[i] = Y[i]-sum;
		}
				
		return X;
	}

	virtual ~SolveLUMethod() {}
};


template<typename T, typename rtn, typename vec>
class SolveGaussSeidel: public Expression<rtn>
{
protected:
	Expression<vec>* vXpr; 
public:
	SolveGaussSeidel(Expression<vec>* xpr): vXpr(xpr){ }

	virtual rtn Eval()
	{
		vec A = vXpr->Eval();
		int rows, cols;
		GetVecSize<T>(rows, cols, A);
		T sum;
		
		rtn X(rows);

		// Step 1
		for(int i=0; i<rows; ++i)
		{
			SET_USER_ABORT;
			if(A[i][i]==T())
			{
				SET_USER_ABORT;
				for(int k=i+1; k<rows; ++k)
					if(A[k][i]!=T())
					{
						
					}

			}

		}

		// Step 2
	}

	virtual ~SolveGaussSeidel() {}
};

class Statement: virtual public BaseType
{
public:
	StmtType stmttype;

	Statement(StmtType type=StmtNull): stmttype(type) {  }
	virtual void Run() = 0;
	virtual bool IsLabel(std::wstring label=_T("")) { return false; }
	virtual std::wstring GetName() { return _T(""); }
	virtual std::wstring GetVarName() { return _T(""); }
	virtual void Initialize(VarList& vl) { }

	virtual void SetThrownPtr(void* ptr) 
		{ throw RunTimeException(L"This should not be called.\r\n"); }

	virtual bool IsReturn() const { return false;}
	virtual bool IsGoto() const   { return false;}
	virtual void CheckIdentical(const Statement* const stmt) const { return; }
	
	virtual ~Statement() { }
};

class StmtInitArray: public Statement
{
protected:
	StmtVector stmtlist;
public:
	LHSArrayBase *pArrayBase;

	StmtInitArray(): pArrayBase(NULL){ }

	void Add(Statement *pStmt) { stmtlist.push_back(pStmt); }

	template<typename vec>
	bool IsLHSEqual(LeftHandSideArray<vec> *pArray)
	{
		LeftHandSideArray<vec> *pLHS = 
			dynamic_cast<LeftHandSideArray<vec>*>(pArrayBase);
		
		if(pLHS) { return pLHS->GetFullName() == pArray->GetFullName(); }
		else return false;
	}

	void Initialize() 
	{  
		int count = int(stmtlist.size());
		for(int i=0; i<count; ++i) stmtlist[i]->Run();
	}
	virtual void Run()
	{
		if(pArrayBase) return;
		Initialize();
	}

	virtual ~StmtInitArray() {}
};

class DeclareBase
{
public:
	DeclareBase() {}

	virtual void SetThrownPtr(void* ptr)=0;
	virtual void SetInitExpr(void* ptr)
	{
		throw UnicodeException(L"It should not be called.\r\n");
	}

	virtual ~DeclareBase() {}
};

class DeclareBaseArray: public DeclareBase
{
public:
	DeclareBaseArray() {}
	virtual ~DeclareBaseArray() {}
};

template<typename T>
class Declare: public DeclareBase, public Statement
{
protected:
	DataTemp<T>* pVar; Expression<T>* pExpr;
	bool isstatic; std::wstring vid;
public:
	T* pThrownValue;

	Declare(const wchar_t* varid, Expression<T>* expr = NULL, bool vstatic=false): 
	pExpr(expr), isstatic(vstatic), vid(varid), pThrownValue(NULL), Statement(StmtDecl) { }

	virtual void CheckIdentical(const Statement* const stmt) const 
	{ 
		const Declare<T>* const pStmt = 
			dynamic_cast<const Declare<T>* const>(stmt);

		if(!pStmt) HandleSyntaxError(_T("Function's argument type mismatch.\r\n")); 

		if(vid != pStmt->vid) 
			HandleSyntaxError(_T("Function's argument name mismatch.\r\n")); 
	}

	virtual void SetThrownPtr(void* ptr) { pThrownValue = static_cast<T*>(ptr); }
	virtual void SetInitExpr(void* ptr) { pExpr = static_cast<Expression<T>*>(ptr); }

	virtual std::wstring GetVarName() { return vid;}
	virtual void Initialize(VarList& vl) 
	{ pVar = new DataTemp<T>(vid, isstatic); vl.insert(pVar); }
	
	virtual void Run()
	{
		SET_USER_ABORT;
		if(pExpr)
		{
			pVar->GetData() = pExpr->Eval(); 
		}
		
		if(pThrownValue)
		{
			pVar->GetData() = *pThrownValue; pThrownValue = NULL;
		}

		if(this->isstatic) pExpr = NULL;
	}
	
	DataTemp<T>* GetVarPtr() const { return pVar;}

	virtual ~Declare() { }
};


template<>
class Declare<stream>: public Statement
{
protected:
	DataTemp<stream>* pVar; 
	bool isstatic; std::wstring vid;
public:
	Declare(const wchar_t* varid, bool vstatic=false): 
	isstatic(vstatic), vid(varid), Statement(StmtDecl) { }

	virtual void CheckIdentical(const Statement* const stmt) const 
	{ 
		const Declare<stream>* const pStmt = 
			dynamic_cast<const Declare<stream>* const>(stmt);

		if(!pStmt) HandleSyntaxError(_T("Function's argument type mismatch.\r\n")); 

		if(vid != pStmt->vid) 
			HandleSyntaxError(_T("Function's argument name mismatch.\r\n")); 
	}

	virtual std::wstring GetVarName() { return vid;}
	virtual void Initialize(VarList& vl) 
	{ pVar = new DataTemp<stream>(vid, isstatic);  vl.insert(pVar);  }
	
	virtual void Run() { }
	
	DataTemp<stream>* GetVarPtr() const { return pVar;}

	virtual ~Declare() { }
};
template<typename T>
class DeclareLiteral: public Statement
{
protected:
	LiteralData<T> *ldptr;

public:
	DeclareLiteral(const wchar_t* varid, Expression<T>* expr)
	{ 
		extern bool assign_set; assign_set = false;
		ldptr = new LiteralData<T>(varid, expr); 
	}

	virtual void Initialize(VarList& vl)  { vl.insert(ldptr); ldptr=NULL; }
	
	virtual void Run() { /* do nothing */ }
	
	virtual ~DeclareLiteral() { if(ldptr) delete ldptr; }
};

class DeclUdt: public Statement
{
protected:
	UserData* pVar; bool isstatic; std::wstring vid;
	UserDefinedType* factory; ExpUserPtr pExpr;
public:
	UserData *pThrownValue;

	DeclUdt(const wchar_t* varid, UserDefinedType* pUdt, ExpUserPtr xpr=NULL, bool vstatic=false): 
	isstatic(vstatic), vid(varid), factory(pUdt), 
		pThrownValue(NULL), pExpr(xpr), Statement(StmtDecl){ }

	virtual std::wstring GetVarName() { return vid;}
	virtual void Initialize(VarList& vl) 
	{ pVar = new UserData(vid, factory, isstatic); vl.insert(pVar); }
	
	virtual void SetThrownPtr(void* ptr) 
		{ pThrownValue = (UserData*)ptr; }

	virtual void Run() 
	{
		SET_USER_ABORT;

		if(pExpr)
		{
			pVar->Copy(pExpr->Eval());
		}
		if(pThrownValue)
		{
			pVar->Copy(*pThrownValue);
			pThrownValue = NULL;
		}

		if(isstatic) { pExpr=NULL; }
	}
	
	UserData* GetVarPtr() const { return pVar;}
	UserDefinedType* GetFactory() { return factory; }

	virtual ~DeclUdt() { }
};

class StatementList;

template<typename T>
class DecAry1D: public DeclareBaseArray, public Statement
{
protected:
	Array1D<T>* pVar; ExpIntg* pSize;
	std::wstring vid; bool isstatic;
	StmtInitArray *pStmtInit;
public:
	
	typename VctrType<T>::Vec1D *pThrownValue;

	DecAry1D(const wchar_t* vrid, ExpIntg* size=NULL, bool vstatic=false):
		pSize(size), vid(vrid), isstatic(vstatic), pThrownValue(NULL), 
			pStmtInit(NULL), Statement(StmtDecl) { }
	
	virtual void SetThrownPtr(void* ptr) 
		{ pThrownValue = (typename VctrType<T>::Vec1D *)ptr; }

	virtual void SetInitExpr(void* ptr) 
	{ pStmtInit = static_cast<StmtInitArray*>(ptr); }

	virtual std::wstring GetVarName() { return vid;}
	virtual void Initialize(VarList& vl) 
	{ pVar = new Array1D<T>(vid, isstatic);  vl.insert(pVar); }

	void SetVarType(VarType& vt)
	{
		extern void StrCopy(wchar_t* tgt, const wchar_t* src);
		StrCopy(vt.bufr , vid.c_str()); vt.dPtr = pVar; 
	}

	Array1D<T>* GetVarPtr() const { return pVar; }

	virtual void Run()
	{
		SET_USER_ABORT;

		if(pStmtInit) pStmtInit->Run();
		else if(pSize) 
			pVar->resize(int(pSize->Eval()));
		
		if(pThrownValue)
		{
			*pVar->GetTop() = *pThrownValue;  pThrownValue = NULL;
		}

		if(isstatic) { pSize = NULL; pStmtInit = NULL; }

	}
	virtual ~DecAry1D() { }
};

template<typename T>
class DecAry2D: public DeclareBaseArray, public Statement
{
protected: 
	Array2D<T> *pVar; std::wstring vid; bool isstatic;
	ExpIntg *pSize1, *pSize2; StmtInitArray *pStmtInit;

public:
	
	typename VctrType<T>::Vec2D *pThrownValue;

	DecAry2D(const wchar_t* vrid, ExpIntg* size1=NULL,
		ExpIntg* size2=NULL, bool vstatic=false):
		pSize1(size1), pSize2(size2), pThrownValue(NULL), 
			vid(vrid), isstatic(vstatic), pStmtInit(NULL), Statement(StmtDecl) {  }
	
	virtual void SetThrownPtr(void* ptr) 
		{ pThrownValue = (typename VctrType<T>::Vec2D *)ptr; }

	virtual void SetInitExpr(void* ptr)
		{  pStmtInit = static_cast<StmtInitArray*>(ptr); }

	virtual std::wstring GetVarName() { return vid;}
	virtual void Initialize(VarList& vl) 
	{ pVar = new Array2D<T>(vid, isstatic); vl.insert(pVar); }

	void SetVarType(VarType& vt)
	{
		extern void StrCopy(wchar_t* tgt, const wchar_t* src);
		StrCopy(vt.bufr , vid.c_str()); vt.dPtr = pVar; 
	}

	Array2D<T>* GetVarPtr() const { return pVar; }

	virtual void Run()
	{
		SET_USER_ABORT;
		
		if(pStmtInit) pStmtInit->Run(); 
		else if(pSize1 && pSize2)
			pVar->resize(int(pSize1->Eval()), int(pSize2->Eval()));
		
		if(pThrownValue)
		{
			*pVar->GetTop() = *pThrownValue; pThrownValue = NULL;
		}

		if(isstatic) { pSize1 = pSize2 = NULL; pStmtInit = NULL; }
	}
	virtual ~DecAry2D() { }
};

template<typename T>
class DecAry3D: public DeclareBaseArray, public Statement
{
protected:
	Array3D<T>* pVar; std::wstring vid; bool isstatic;
	ExpIntg *pSize1, *pSize2, *pSize3; StmtInitArray *pStmtInit;

public:
	typename VctrType<T>::Vec3D *pThrownValue;

	DecAry3D(const wchar_t* vrid, ExpIntg* size1 = NULL,
		ExpIntg* size2 = NULL, 
		ExpIntg* size3 = NULL, bool vstatic=false):
		pSize1(size1), pSize2(size2), pSize3(size3) , pThrownValue(NULL), 
			vid(vrid), isstatic(vstatic), pStmtInit(NULL), Statement(StmtDecl) { }
	
	virtual std::wstring GetVarName() { return vid;}
	
	virtual void SetThrownPtr(void* ptr) 
		{ pThrownValue = (typename VctrType<T>::Vec3D *)ptr; }

	virtual void SetInitExpr(void* ptr)
		{ pStmtInit = static_cast<StmtInitArray*>(ptr); }

	virtual void Initialize(VarList& vl) 
	{ pVar = new Array3D<T>(vid, isstatic); vl.insert(pVar); }

	void SetVarType(VarType& vt)
	{
		extern void StrCopy(wchar_t* tgt, const wchar_t* src);
		StrCopy(vt.bufr , vid.c_str()); vt.dPtr = pVar; 
	}

	Array3D<T>* GetVarPtr() const { return pVar; }

	virtual void Run()
	{
		SET_USER_ABORT;

		if(pStmtInit) pStmtInit->Run();
		else if(pSize1 && pSize2 && pSize3)
			pVar->resize(int(pSize1->Eval()), int(pSize2->Eval()), int(pSize3->Eval()));
		
		if(pThrownValue)
		{
			*pVar->GetTop() = *pThrownValue; pThrownValue = NULL;
		}

		if(isstatic) { pSize1 = pSize2 = pSize3 = NULL; pStmtInit = NULL; }
	}

	virtual ~DecAry3D() { }
};

template<typename T>
class DecAry4D: public DeclareBaseArray, public Statement
{
protected:
	Array4D<T>* pVar; std::wstring vid; bool isstatic;
	ExpIntg *pSize1, *pSize2, *pSize3, *pSize4; StmtInitArray *pStmtInit;

public:

	typename VctrType<T>::Vec4D *pThrownValue;

	DecAry4D(const wchar_t* vrid, ExpIntg* size1 = NULL,
		ExpIntg* size2 = NULL, ExpIntg* size3 = NULL, 
		ExpIntg* size4 = NULL, bool vstatic=false):
			pSize1(size1), pSize2(size2), pSize3(size3), pSize4(size4) , pThrownValue(NULL),
				vid(vrid), isstatic(vstatic), pStmtInit(NULL), Statement(StmtDecl) {  }
	
	virtual void SetThrownPtr(void* ptr) 
		{ pThrownValue = (typename VctrType<T>::Vec4D *)ptr; }

	virtual void SetInitExpr(void* ptr)
		{ pStmtInit = static_cast<StmtInitArray*>(ptr); }

	virtual std::wstring GetVarName() { return vid;}
	
	virtual void Initialize(VarList& vl) 
	{ pVar = new Array4D<T>(vid, isstatic);  vl.insert(pVar); }

	void SetVarType(VarType& vt)
	{
		extern void StrCopy(wchar_t* tgt, const wchar_t* src);
		StrCopy(vt.bufr , vid.c_str()); vt.dPtr = pVar; 
	}

	Array4D<T>* GetVarPtr() const { return pVar; }

	virtual void Run()
	{
		SET_USER_ABORT;

		if(pStmtInit) pStmtInit->Run();
		else if(pSize1 && pSize2 && pSize3 && pSize4)
			pVar->resize(int(pSize1->Eval()), int(pSize2->Eval()),
							 int(pSize3->Eval()), int(pSize4->Eval()));
		
		if(pThrownValue) { *pVar->GetTop() = *pThrownValue; pThrownValue=NULL; }
		
		if(isstatic) { pSize1 = pSize2 = pSize3 = pSize4 = NULL; pStmtInit = NULL; }
	}
	virtual ~DecAry4D() { }
};

template<typename T>
class DecAry5D: public DeclareBaseArray, public Statement
{
protected:
	Array5D<T>* pVar; std::wstring vid; bool isstatic;
	ExpIntg *pSize1, *pSize2, *pSize3, *pSize4, *pSize5;
	StmtInitArray *pStmtInit;

public:
	typename VctrType<T>::Vec5D *pThrownValue;

	DecAry5D(const wchar_t* vrid, ExpIntg* size1 = NULL,
		ExpIntg *size2 = NULL, ExpIntg * size3 = NULL, 
		ExpIntg *size4 = NULL, ExpIntg *size5 = NULL, bool vstatic=false):
			pSize1(size1), pSize2(size2), pSize3(size3), pSize4(size4), pSize5(size5) , pThrownValue(NULL),
				vid(vrid), isstatic(vstatic), pStmtInit(NULL), Statement(StmtDecl) { }
	
	virtual void SetThrownPtr(void* ptr) 
		{ pThrownValue = (typename VctrType<T>::Vec5D *)ptr; }

	virtual void SetInitExpr(void* ptr) 
		{ pStmtInit = static_cast<StmtInitArray*>(ptr); }

	virtual std::wstring GetVarName() { return vid; }
	
	virtual void Initialize(VarList& vl) 
	{ pVar = new Array5D<T>(vid, isstatic); vl.insert(pVar); }

	void SetVarType(VarType& vt)
	{
		extern void StrCopy(wchar_t* tgt, const wchar_t* src);
		StrCopy(vt.bufr , vid.c_str()); vt.dPtr = pVar; 
	}

	Array5D<T>* GetVarPtr() const { return pVar; }

	virtual void Run()
	{
		SET_USER_ABORT;
				
		if(pStmtInit) pStmtInit->Run();
		else if(pSize1 && pSize2 && pSize3 && pSize4 && pSize5)
		{
			pVar->resize(int(pSize1->Eval()), int(pSize2->Eval()),
			int(pSize3->Eval()), int(pSize4->Eval()), int(pSize5->Eval()));
		}
		
		if(pThrownValue) { *pVar->GetTop() = *pThrownValue; pThrownValue = NULL; }
		
		if(isstatic) { pSize1 = pSize2 = pSize3  = pSize4  = pSize5 = NULL; pStmtInit = NULL; }

	}
	virtual ~DecAry5D() { }
};

template<typename T>
class DeclarePtr: public DeclareBase, public Statement
{
protected:
	PtrData<T> *pVar; 
	LeftHandSide<T>* pData;
	std::wstring vid; bool isstatic;
public:
	
	T* pThrownValue; Expression<T*>* pExpr; 

	DeclarePtr(const wchar_t* vrid, LeftHandSide<T>* dPtr=NULL, bool vstatic=false): 
	pData(dPtr), vid(vrid), isstatic(vstatic), 
		pExpr(NULL), pThrownValue(NULL), Statement(StmtDecl) { }

	virtual std::wstring GetVarName() { return vid;}
	virtual void Initialize(VarList& vl) 
	{ pVar = new PtrData<T>(vid, isstatic); vl.insert(pVar); }
	
	virtual void SetThrownPtr(void* ptr) { pThrownValue = (T*)ptr; }
	virtual void SetInitExpr(void* ptr) 
	{ 
		extern bool assign_set; assign_set = false;
		pData = static_cast<LeftHandSide<T>*>(ptr);
	}

	virtual void Run()
	{
		SET_USER_ABORT;

		if(pData) 
		{
			*pVar->GetPtr() = &pData->GetData();
		}

		if(pThrownValue)
		{
			*pVar->GetPtr() = pThrownValue; pThrownValue = NULL;
		}

		if(pExpr)
		{
			*pVar->GetPtr() = pExpr->Eval();
		}

		if(isstatic) { pData = NULL; pExpr = NULL; pExpr = NULL;}
	}

	virtual ~DeclarePtr() { }
};

class usertype; 
class UserDefinedType;

template<>
class DeclarePtr<UserData>: public DeclareBase, public Statement
{
protected:
	PtrData<UserData> *pVar; LeftHandSide<UserData>* pData;
	std::wstring vid; bool isstatic;
public:
	UserDefinedType *factory;

	UserData* pThrownValue; Expression<UserData*>* pExpr; 

	DeclarePtr(const wchar_t* vrid, UserDefinedType *ftry, 
		LeftHandSide<UserData>* dPtr=NULL, bool vstatic=false): 
		pData(dPtr), vid(vrid), isstatic(vstatic), pExpr(NULL), 
			pThrownValue(NULL), factory(ftry), Statement(StmtDecl) { }

	virtual std::wstring GetVarName() { return vid;}
	virtual void Initialize(VarList& vl) 
	{ 
		pVar = new PtrData<UserData>(vid, factory, isstatic); 
		vl.insert(pVar); 
	}
	
	virtual void SetThrownPtr(void* ptr) { pThrownValue = (UserData*)ptr; }

	virtual void SetInitExpr(void* ptr) 
	{ 
		extern bool assign_set; assign_set = false;
		pData = static_cast<LeftHandSide<UserData>*>(ptr);
	}

	virtual void Run()
	{
		SET_USER_ABORT;

		if(pData) 
		{
			*pVar->GetPtr() = &pData->GetData();
		}

		if(pThrownValue)
		{
			*pVar->GetPtr() = pThrownValue; pThrownValue = NULL;
		}

		if(pExpr)
		{
			*pVar->GetPtr() = pExpr->Eval();
		}

		if(isstatic) { pData = NULL; pExpr = NULL; pExpr = NULL;}
	}

	virtual ~DeclarePtr() { }
};

template<typename vec>
class DeclareAptr: public DeclareBase, public Statement
{
protected:
	AptrData<vec> *pVar; LeftHandSideArray<vec> *pData;
	std::wstring vid; bool isstatic;
public:
		
	vec *pThrownValue;

	DeclareAptr(const wchar_t* vrid, LeftHandSideArray<vec> *dPtr=NULL, bool vstatic=false): 
	pData(dPtr), vid(vrid), isstatic(vstatic), 
		pThrownValue(NULL), Statement(StmtDecl) {   }

	virtual std::wstring GetVarName() { return vid;}
	virtual void Initialize(VarList& vl) 
	{ pVar = new AptrData<vec>(vid, isstatic); vl.insert(pVar); }

	virtual void SetThrownPtr(void* ptr) 
		{ pThrownValue = static_cast<vec*>(ptr); }
	virtual void SetInitExpr(void* ptr) 
		{ 
			extern bool assign_set; assign_set = false;
			pData = static_cast<LeftHandSideArray<vec>*>(ptr); 
		}

	virtual void Run()
	{
		SET_USER_ABORT;

		if(pData)
		{
			*pVar->GetPtr() = &pData->GetData();
			if(isstatic) pData = NULL;
		}

		if(pThrownValue)
		{
			*pVar->GetPtr() = pThrownValue; pThrownValue = NULL;
		}
	}

	virtual ~DeclareAptr() { }
};

/*
template<typename T, typename U>
class DeclPtr: public Statement
{
protected:
	PtrData<T> *pVar; U* pData;
	std::wstring vid; bool isstatic;
public:
	
	T* pThrownValue;

	DeclPtr(const wchar_t* vrid, U* dPtr=NULL, bool vstatic=false): 
	pData(dPtr), vid(vrid), isstatic(vstatic), pThrownValue(NULL), Statement(StmtDecl){ }

	virtual std::wstring GetVarName() { return vid;}
	virtual void Initialize(VarList& vl) 
	{ pVar = new PtrData<T>(vid, isstatic); vl.insert(pVar); }
	
	virtual void SetThrownPtr(void* ptr) { pThrownValue = (T*)ptr; }

	virtual void Run()
	{
		SET_USER_ABORT;

		if(pData) 
		{
			*pVar->GetPtr() = &pData->GetData();
			if(isstatic) { pData = NULL;}
		}

		if(pThrownValue)
		{
			*pVar->GetPtr() = pThrownValue; pThrownValue = NULL;
		}
	}

	virtual ~DeclPtr() { }
};

template<typename vec, typename ary>
class DeclAptr: public Statement
{
protected:
	AptrData<vec> *pVar; ary *pData;
	std::wstring vid; bool isstatic;
public:
		
	vec *pThrownValue;

	DeclAptr(const wchar_t* vrid, ary *dPtr=NULL, bool vstatic=false): 
	pData(dPtr), vid(vrid), isstatic(vstatic), 
		pThrownValue(NULL), Statement(StmtDecl) {   }

	virtual std::wstring GetVarName() { return vid;}
	virtual void Initialize(VarList& vl) 
	{ pVar = new AptrData<vec>(vid, isstatic); vl.insert(pVar); }

	virtual void SetThrownPtr(void* ptr) { pThrownValue = (vec*)ptr; }
	
	virtual void Run()
	{
		SET_USER_ABORT;

		if(pData)
		{
			*pVar->GetPtr() = pData->GetTop();
			if(isstatic) pData = NULL;
		}

		if(pThrownValue)
		{
			*pVar->GetPtr() = pThrownValue; pThrownValue = NULL;
		}
	}

	virtual ~DeclAptr() { }
};
*/

class BaseAssign: public Statement
{
public:
	std::wstring vrid;
	
	BaseAssign(const std::wstring id=L""): 
		vrid(id), Statement(StmtAsgn) {}

	void SetID(const std::wstring& id) { vrid = id; }
	
	std::wstring& GetID() { return vrid; }

	virtual ~BaseAssign() {}
};

class AssignExpUser: public BaseAssign
{
protected:
	LeftHandSide<UserData> *lhs; ExpUserPtr rhs;
	
public:
	AssignExpUser(LeftHandSide<UserData> *lptr, ExpUserPtr rptr);
	virtual void Run();

	virtual ~AssignExpUser();
};

template<typename T> class CallFunc;

class AssignUT2ExpUser: public BaseAssign
{
protected:
	LeftHandSide<UserData> *lhs; CallFunc<UserData> *rhs;
	
public:
	AssignUT2ExpUser(LeftHandSide<UserData> *lptr, CallFunc<UserData> *rptr);
	virtual void Run();

	virtual ~AssignUT2ExpUser();
};

class StmtSwapVUB: public Statement
{
protected:
	VarUserBase *lhs, *rhs;
	
public:
	StmtSwapVUB(VarUserBase *lptr, VarUserBase *rptr);
	virtual void Run();
	virtual ~StmtSwapVUB();
};

////////////////////////////////////////////////////////////

template<typename T>
class AssignmentLHS: public BaseAssign
{
protected:
	LeftHandSide<T> *pLHS; Expression<T> *pRHS;
	const char optr;
public:
	AssignmentLHS(LeftHandSide<T> *lhs, Expression<T>* rhs, const char opr = '='): 
	  optr(opr), pLHS(lhs), pRHS(rhs)
	  { 
		  if(opr=='&') Statement::stmttype = StmtExpr;  
	  }
	 
	 bool IsLHSEqual(LeftHandSide<T> *lhs) 
		{ return pLHS->GetFullName() == lhs->GetFullName(); }

	virtual void Run() 
	{ 
		if(Statement::stmttype == StmtExpr) return;

		switch(optr)
		{
		case '=': pLHS->GetData() = pRHS->Eval(); return;
		case '!': pLHS->GetData() += pRHS->Eval(); return;
		case '@': pLHS->GetData() -= pRHS->Eval(); return;
		case '#': pLHS->GetData() *= pRHS->Eval(); return;
		case '$': pLHS->GetData() /= pRHS->Eval(); return;

		case '%': ++pLHS->GetData(); return; // increment
		case '^': --pLHS->GetData(); return; // decrement
		
		default:
			throw UnicodeException(_T("Operation not defined.\r\n"));
		}
	}

	LeftHandSide<T> *GetLHS() { return pLHS; }
	Expression<T>* GetExpression() 
	{
		if(pRHS) return pRHS; 
		else HandleSyntaxError(L"AssignmentLHS<T>::GetExpression(), this should not happen.\r\n"); 
		
		return NULL; // to suppress warning
	}

	virtual ~AssignmentLHS() { }
};

template<>
class AssignmentLHS<integer>: public BaseAssign
{
protected:
	LeftHandSide<integer> *pLHS; Expression<integer> *pRHS;
	const char optr;
public:
	AssignmentLHS(LeftHandSide<integer> *lhs, Expression<integer>* rhs, const char opr = '='): 
	  optr(opr), pLHS(lhs), pRHS(rhs)
	  { 
		  if(opr=='&') Statement::stmttype = StmtExpr;  
	  }
	 
	 bool IsLHSEqual(LeftHandSide<integer> *lhs) 
		{ return pLHS->GetFullName() == lhs->GetFullName(); }

	virtual void Run() 
	{ 
		if(Statement::stmttype == StmtExpr) return;

		switch(optr)
		{
		case '=': pLHS->GetData() = pRHS->Eval(); return;
		case '!': pLHS->GetData() += pRHS->Eval(); return;
		case '@': pLHS->GetData() -= pRHS->Eval(); return;
		case '#': pLHS->GetData() *= pRHS->Eval(); return;
		case '$': pLHS->GetData() /= pRHS->Eval(); return;
		case '~': pLHS->GetData() %= pRHS->Eval(); return;

		case '%': ++pLHS->GetData(); return; // increment
		case '^': --pLHS->GetData(); return; // decrement
		
		default:
			throw UnicodeException(_T("Operation not defined.\r\n"));
		}
	}

	LeftHandSide<integer> *GetLHS() { return pLHS; }
	Expression<integer>* GetExpression() 
	{
		if(pRHS) return pRHS; 
		else HandleSyntaxError(L"AssignmentLHS<T>::GetExpression(), this should not happen.\r\n"); 
		
		return NULL; // to suppress warning
	}

	virtual ~AssignmentLHS() { }
};

template<>
class AssignmentLHS<Complex>: public BaseAssign
{
protected:
	LeftHandSide<Complex> *pLHS; ExpCplx *pRHS;
	const char optr;
public:
	AssignmentLHS(LeftHandSide<Complex> *lhs, ExpCplx* rhs, const char opr = '='): 
	  optr(opr), pLHS(lhs), pRHS(rhs)
	  {  
		  if(opr=='&') Statement::stmttype = StmtExpr; 
	  }
	
	 bool IsLHSEqual(LeftHandSide<Complex> *lhs) 
		{ return pLHS->GetFullName() == lhs->GetFullName(); }
	virtual void Run() 
	{ 
		if(Statement::stmttype == StmtExpr) return;

		switch(optr)
		{
		case '=': pLHS->GetData() = pRHS->Eval(); return;
		case '!': pLHS->GetData() += pRHS->Eval(); return;
		case '@': pLHS->GetData() -= pRHS->Eval(); return;
		case '#': pLHS->GetData() *= pRHS->Eval(); return;
		case '$': pLHS->GetData() /= pRHS->Eval(); return;
		
		default:
			throw UnicodeException(_T("Operation not defined.\r\n"));
		}
	}

	LeftHandSide<Complex> *GetLHS() { return pLHS; }

	ExpCplx* GetExpression() { return pRHS; }

	virtual ~AssignmentLHS() { }
};

template<>
class AssignmentLHS<Vector>: public BaseAssign
{
protected:
	LeftHandSide<Vector> *pLHS; ExpVctr *pRHS;
	const char optr;
public:
	AssignmentLHS(LeftHandSide<Vector> *lhs, ExpVctr* rhs, const char opr = '='): 
	  optr(opr), pLHS(lhs), pRHS(rhs)
	  {  if(opr=='&') Statement::stmttype = StmtExpr;  }
	
	 bool IsLHSEqual(LeftHandSide<Vector> *lhs) 
	 { return pLHS->GetFullName() == lhs->GetFullName(); }

	virtual void Run() 
	{ 
		if(Statement::stmttype == StmtExpr) return;

		switch(optr)
		{
		case '=': pLHS->GetData() = pRHS->Eval(); return;
		case '!': pLHS->GetData() += pRHS->Eval(); return;
		case '@': pLHS->GetData() -= pRHS->Eval(); return;
		case '%': pLHS->GetData() %= pRHS->Eval(); return;
				
		default:
			throw UnicodeException(_T("Operation not defined.\r\n"));
		}
	}

	LeftHandSide<Vector> *GetLHS() { return pLHS; }

	ExpVctr* GetExpression() { return pRHS; }

	virtual ~AssignmentLHS() { }
};

template<>
class AssignmentLHS<string>: public BaseAssign
{
protected:
	LeftHandSide<string> *pLHS; ExpStrn *pRHS;
	const char optr;

public:
	AssignmentLHS(LeftHandSide<string> *lhs, ExpStrn* rhs, const char opr = '='): 
	  optr(opr), pLHS(lhs), pRHS(rhs) 
	  {
		  if(opr=='&') Statement::stmttype = StmtExpr;  
	  }
	
	  bool IsLHSEqual(LeftHandSide<string> *lhs) 
	  { return pLHS->GetFullName() == lhs->GetFullName(); }

	virtual void Run() 
	{ 
		if(Statement::stmttype == StmtExpr) return;

		switch(optr)
		{
		case '=': pLHS->GetData() = pRHS->Eval(); return;
		case '!': pLHS->GetData() += pRHS->Eval(); return;
				
		default:
			throw UnicodeException(_T("Operation not defined.\r\n"));
		}
	}

	LeftHandSide<string> *GetLHS() { return pLHS; }

	ExpStrn* GetExpression() { return pRHS; }

	virtual ~AssignmentLHS() { }
};

template<>
class AssignmentLHS<bool>: public BaseAssign
{
protected:
	LeftHandSide<bool> *pLHS; ExpBool *pRHS;
	
public:
	AssignmentLHS(LeftHandSide<bool> *lhs, ExpBool* rhs, const int opr= '=' ): 
	  pLHS(lhs), pRHS(rhs) 
	  {
		  if(opr=='&') Statement::stmttype = StmtExpr;  
	  }
	
	  bool IsLHSEqual(LeftHandSide<bool> *lhs) 
	  { return pLHS->GetFullName() == lhs->GetFullName(); }

	virtual void Run() 
	{  
		if(Statement::stmttype == StmtExpr) return;

		pLHS->GetData() = pRHS->Eval();  
	}

	LeftHandSide<bool> *GetLHS() { return pLHS; }

	ExpBool* GetExpression() { return pRHS; }

	virtual ~AssignmentLHS() { }
};

//typedef AssignmentLHS<bool>		AssignBool, *AssignBoolPtr;
//typedef AssignmentLHS<integer>	AssignIntg, *AssignIntgPtr;
//typedef AssignmentLHS<realnum>	AssignReal, *AssignRealPtr;
//typedef AssignmentLHS<Complex>	AssignCplx, *AssignCplxPtr;
//typedef AssignmentLHS<Vector>	AssignVctr, *AssignVctrPtr;
//typedef AssignmentLHS<string>	AssignStrn, *AssignStrnPtr;


template<typename vec>
class AssignmentArrayLHS: public BaseAssign
{
protected:
	LeftHandSide<vec> *pLHS; Expression<vec> *pRHS;
	const char optr;
public:
	AssignmentArrayLHS(LeftHandSide<vec> *lhs, 
		Expression<vec>* rhs, const char opr = '='): 
	  optr(opr), pLHS(lhs), pRHS(rhs)
	  { 
		  if(opr=='&') Statement::stmttype = StmtExpr;  
	  }
	
	virtual void Run() 
	{ 
		if(Statement::stmttype == StmtExpr) return;
		
		switch(optr)
		{
		case '=': pLHS->GetData() = pRHS->Eval(); return;
		case '!': pLHS->GetData() += pRHS->Eval(); return;
		case '@': pLHS->GetData() -= pRHS->Eval(); return;
		case '#': pLHS->GetData() *= pRHS->Eval(); return;
		case '$': pLHS->GetData() /= pRHS->Eval(); return;
		
		default:
			throw UnicodeException(_T("Operation not defined.\r\n"));
		}
	}

	Expression<vec>* GetExpression() { return pRHS; }

	virtual ~AssignmentArrayLHS() { }
};

template<typename vec>
class AssignmentArrayVctrLHS: public BaseAssign
{
protected:
	LeftHandSide<vec> *pLHS; Expression<vec> *pRHS;
	const char optr;
public:
	AssignmentArrayVctrLHS(LeftHandSide<vec> *lhs, 
		Expression<vec>* rhs, const char opr = '='): 
	  optr(opr), pLHS(lhs), pRHS(rhs)
	  { if(opr=='&') Statement::stmttype = StmtExpr;  }
	
	virtual void Run() 
	{ 
		if(Statement::stmttype == StmtExpr) return;
		
		switch(optr)
		{
		case '=': pLHS->GetData() = pRHS->Eval(); return;
		case '!': pLHS->GetData() += pRHS->Eval(); return;
		case '@': pLHS->GetData() -= pRHS->Eval(); return;
		case '%': pLHS->GetData() %= pRHS->Eval(); return;
				
		default:
			throw UnicodeException(_T("Operation not defined.\r\n"));
		}
	}

	Expression<vec>* GetExpression() { return pRHS; }

	virtual ~AssignmentArrayVctrLHS() { }
};


////////////////////////////////////////////////////////////


template<typename T, typename U>
class Assignment: public BaseAssign
{
protected:
	U* pVar; Expression<T> *pExpr;
	const char optr;
public:
	Assignment(U *var, Expression<T>* xpr, const char opr = '='): 
	  optr(opr), pVar(var), pExpr(xpr), BaseAssign(var->GetId()) 
	  { if(opr=='&') Statement::stmttype = StmtExpr; }
	
	virtual void Run() 
	{ 
		if(Statement::stmttype == StmtExpr) return;

		switch(optr)
		{
		case '=': pVar->GetData() = pExpr->Eval(); return;
		
		case '!': pVar->GetData() += pExpr->Eval(); return;
		case '@': pVar->GetData() -= pExpr->Eval(); return;
		case '#': pVar->GetData() *= pExpr->Eval(); return;
		case '$': pVar->GetData() /= pExpr->Eval(); return;

		case '%': ++pVar->GetData(); return; // increment
		case '^': --pVar->GetData(); return; // decrement
		
		default:
			throw UnicodeException(_T("Operation not defined.\r\n"));
		}
	}

	Expression<T>* GetExpression() { return pExpr; }

	virtual ~Assignment() { }
};

template<>
class Assignment<string, VarUser<string> >: public BaseAssign
{
protected:
	VarUser<string>* pVar; ExpStrn *pExpr;
	const char optr;
public:
	Assignment(VarUser<string> *var, ExpStrn* xpr, const char opr = '='): 
	  optr(opr), pVar(var), pExpr(xpr), BaseAssign(var->GetId()) { }
	  	
	virtual void Run() 
	{ 
	
		switch(optr)
		{
		case '=': pVar->GetData() = pExpr->Eval(); return;
		
		case '!': pVar->GetData() += pExpr->Eval(); return;
		
		default:
			throw UnicodeException(_T("Operation not defined.\r\n"));
		}
	}

	ExpStrn* GetExpression() { return pExpr; }

	virtual ~Assignment() { }
};

template<>
class Assignment<bool, BoolData>: public BaseAssign
{
protected:
	BoolData* pVar; ExpBool *pExpr;
	const char optr;
public:
	Assignment(BoolData *var, ExpBool* xpr, const char opr = '='): 
	  optr(opr), pVar(var), pExpr(xpr), BaseAssign(var->GetId()) { }
	
	virtual void Run() 
	{ 
		switch(optr)
		{
		case '=': pVar->GetData() = pExpr->Eval(); break;
				
		default:
			throw UnicodeException(_T("Operation not defined"));
		}
	}

	ExpBool* GetExpression() { return pExpr; }

	virtual ~Assignment() { }
};

template<>
class Assignment<bool, VarUser<bool> >: public BaseAssign
{
protected:
	VarUser<bool>* pVar; ExpBool *pExpr;
	const char optr;
public:
	Assignment(VarUser<bool> *var, ExpBool* xpr, const char opr = '='): 
	  optr(opr), pVar(var), pExpr(xpr), BaseAssign(var->GetId()) { }
	
	virtual void Run() 
	{ 
		switch(optr)
		{
		case '=': pVar->GetData() = pExpr->Eval(); break;
				
		default:
			throw UnicodeException(_T("Operation not defined"));
		}
	}

	ExpBool* GetExpression() { return pExpr; }

	virtual ~Assignment() { }
};

template<>
class Assignment<Complex, CplxData>: public BaseAssign
{
protected:
	CplxData* pVar; ExpCplx *pExpr;
	const char optr;
public:
	Assignment(CplxData *var, ExpCplx* xpr, const char opr = '='): 
	  optr(opr), pVar(var), pExpr(xpr), BaseAssign(var->GetId()) 
	  { if(opr=='&') Statement::stmttype = StmtExpr; }
	
	virtual void Run() 
	{ 
		if(Statement::stmttype == StmtExpr) return;

		switch(optr)
		{
				
		case '=': pVar->GetData() = pExpr->Eval(); break;
		
		case '!': pVar->GetData() += pExpr->Eval(); break;
		case '@': pVar->GetData() -= pExpr->Eval(); break;
		case '#': pVar->GetData() *= pExpr->Eval(); break;
		case '$': pVar->GetData() /= pExpr->Eval(); break;
		
		default:
			throw UnicodeException(_T("Operation not defined"));
		}
	}

	ExpCplx* GetExpression() { return pExpr; }

	virtual ~Assignment() { }
};

class AssignCplxReal: public BaseAssign
{
protected:
	CplxData* pVar; ExpReal *pExpr;
	const char optr;
public:
	AssignCplxReal(CplxData *var, ExpReal* xpr, const char opr='='): 
	  pVar(var), pExpr(xpr), optr(opr), BaseAssign(L"No Name") { }
	
	virtual void Run() 
	{ 
		realnum real = pVar->GetData().real(), 
			v = pExpr->Eval();
		switch(optr)
		{
		case '=': pVar->GetData().real(v); return;
		
		case '!': pVar->GetData().real(real+v); return;
		case '@': pVar->GetData().real(real-v); return;
		case '#': pVar->GetData().real(real*v); return;
		case '$': pVar->GetData().real(real/v); return;

		case '%': pVar->GetData().real(++real); return; // increment
		case '^': pVar->GetData().real(--real); return; // decrement
		
		default:
			throw UnicodeException(_T("Operation not defined.\r\n"));
		}
	}

	ExpReal* GetExpression() { return pExpr; }

	virtual ~AssignCplxReal() { }
};

class AssignCplxImag: public BaseAssign
{
protected:
	CplxData* pVar; ExpReal *pExpr;
	const char optr;
public:
	AssignCplxImag(CplxData *var, ExpReal* xpr, const char opr='='): 
	  pVar(var), pExpr(xpr), optr(opr), BaseAssign(L"No Name") { }
	
	virtual void Run() 
	{ 
		realnum imag = pVar->GetData().imag(), 
			v = pExpr->Eval();
		switch(optr)
		{
		case '=': pVar->GetData().imag(v); return;
		
		case '!': pVar->GetData().imag(imag+v); return;
		case '@': pVar->GetData().imag(imag-v); return;
		case '#': pVar->GetData().imag(imag*v); return;
		case '$': pVar->GetData().imag(imag/v); return;

		case '%': pVar->GetData().imag(++imag); return; // increment
		case '^': pVar->GetData().imag(--imag); return; // decrement
		
		default:
			throw UnicodeException(_T("Operation not defined.\r\n"));
		}
	}

	ExpReal* GetExpression() { return pExpr; }

	virtual ~AssignCplxImag() { }
};

class AssignVctrX: public BaseAssign
{
protected:
	VctrData* pVar; ExpReal *pExpr;
public:
	AssignVctrX(VctrData *var, ExpReal* xpr): 
	  pVar(var), pExpr(xpr), BaseAssign(L"No Name") { }
	
	virtual void Run() 
	{ 
		pVar->GetData().x = pExpr->Eval();
	}

	ExpReal* GetExpression() { return pExpr; }

	virtual ~AssignVctrX() { }
};

class AssignVctrY: public BaseAssign
{
protected:
	VctrData* pVar; ExpReal *pExpr;
public:
	AssignVctrY(VctrData *var, ExpReal* xpr): 
	  pVar(var), pExpr(xpr), BaseAssign(L"No Name") { }
	
	virtual void Run() 
	{ 
		pVar->GetData().y = pExpr->Eval();
	}

	ExpReal* GetExpression() { return pExpr; }

	virtual ~AssignVctrY() { }
};

class AssignVctrZ: public BaseAssign
{
protected:
	VctrData* pVar; ExpReal *pExpr;
public:
	AssignVctrZ(VctrData *var, ExpReal* xpr): 
	  pVar(var), pExpr(xpr), BaseAssign(L"No Name") { }
	
	virtual void Run() 
	{ 
		pVar->GetData().z = pExpr->Eval();
	}

	ExpReal* GetExpression() { return pExpr; }

	virtual ~AssignVctrZ() { }
};

template<>
class Assignment<Complex, VarUser<Complex> >: public BaseAssign
{
protected:
	VarUser<Complex>* pVar; ExpCplx *pExpr;
	const char optr;
public:
	Assignment(VarUser<Complex> *var, ExpCplx* xpr, const char opr = '='): 
	  optr(opr), pVar(var), pExpr(xpr), BaseAssign(var->GetId()) 
	  { if(opr=='&') Statement::stmttype = StmtExpr; }
	
	virtual void Run() 
	{ 
		if(Statement::stmttype == StmtExpr) return;

		switch(optr)
		{
				
		case '=': pVar->GetData() = pExpr->Eval(); break;
		
		case '!': pVar->GetData() += pExpr->Eval(); break;
		case '@': pVar->GetData() -= pExpr->Eval(); break;
		case '#': pVar->GetData() *= pExpr->Eval(); break;
		case '$': pVar->GetData() /= pExpr->Eval(); break;
		
		default:
			throw UnicodeException(_T("Operation not defined"));
		}
	}

	ExpCplx* GetExpression() { return pExpr; }

	virtual ~Assignment() { }
};

template<>
class Assignment<Vector, VctrData>: public BaseAssign
{
protected:
	VctrData* pVar; ExpVctr *pExpr;
	const char optr;
public:
	Assignment(VctrData *var, ExpVctr* xpr, const char opr = '='): 
	  optr(opr), pVar(var), pExpr(xpr), BaseAssign(var->GetId()) 
	  { if(opr=='&') Statement::stmttype = StmtExpr; }
	
	virtual void Run() 
	{ 
		if(Statement::stmttype == StmtExpr) return;

		switch(optr)
		{
		case '=': pVar->GetData() = pExpr->Eval(); break;
		
		case '!': pVar->GetData() += pExpr->Eval(); break;
		case '@': pVar->GetData() -= pExpr->Eval(); break;
		case '%': pVar->GetData() %= pExpr->Eval(); break;
				
		default:
			throw UnicodeException(_T("Operation not defined"));
		}
	}

	ExpVctr* GetExpression() { return pExpr; }

	virtual ~Assignment() { }
};

template<>
class Assignment<string, StrnData>: public BaseAssign
{
protected:
	StrnData* pVar; ExpStrn *pExpr;
	const char optr;
public:
	Assignment(StrnData *var, ExpStrn* xpr, const char opr = '='): 
	  optr(opr), pVar(var), pExpr(xpr), BaseAssign(var->GetId()) { }
	  	
	virtual void Run() 
	{ 
		switch(optr)
		{
		case '=': pVar->GetData() = pExpr->Eval(); break;
		case '!': pVar->GetData() += pExpr->Eval(); break;
						
		default:
			throw UnicodeException(_T("Operation not defined.\r\n"));
		}
	}

	ExpStrn* GetExpression() { return pExpr; }

	virtual ~Assignment() { }
};

template<>
class Assignment<stream&, StrnData>: public BaseAssign
{
protected:
	StrmData* pVar; ExpStrn *pExpr;
	const char optr;
public:
	Assignment(StrmData *var, ExpStrn* xpr, const char opr = '='): 
	  optr(opr), pVar(var), pExpr(xpr), BaseAssign(var->GetId()) 
	  { if(opr=='&') Statement::stmttype = StmtExpr; }
	
	virtual void Run() 
	{ 
		if(Statement::stmttype == StmtExpr) return;

		switch(optr)
		{
		case '=': pVar->GetData().str(pExpr->Eval()); break;
		
		default:
			throw UnicodeException(_T("Operation not defined.\r\n"));
		}
	}

	virtual ~Assignment() { }
};

template<>
class Assignment<Vector, VarUser<Vector> >: public BaseAssign
{
protected:
	VarUser<Vector>* pVar; ExpVctr *pExpr;
	const char optr;
public:
	Assignment(VarUser<Vector> *var, ExpVctr* xpr, const char opr = '='): 
	  optr(opr), pVar(var), pExpr(xpr), BaseAssign(var->GetId()) 
	  { if(opr=='&') Statement::stmttype = StmtExpr; }
	
	virtual void Run() 
	{ 
		if(Statement::stmttype == StmtExpr) return;

		switch(optr)
		{
		case '=': pVar->GetData() = pExpr->Eval(); break;
		
		case '!': pVar->GetData() += pExpr->Eval(); break;
		case '@': pVar->GetData() -= pExpr->Eval(); break;
		case '%': pVar->GetData() %= pExpr->Eval(); break;
				
		default:
			throw UnicodeException(_T("Operation not defined"));
		}
	}

	ExpVctr* GetExpression() { return pExpr; }

	virtual ~Assignment() { }
};

template<typename T>
class AssignCplx: public BaseAssign
{
protected:
	LeftHandSide<T> *pVar; ExpReal *pExpr;
	const char optr;
public:
	AssignCplx(LeftHandSide<T> *var, ExpReal* xpr, const char opr): 
	  optr(opr), pVar(var), pExpr(xpr) {  }
	
	virtual void Run() 
	{ 
		switch(optr)
		{
		
		case '!': pVar->GetData() += pExpr->Eval(); break;
		case '@': pVar->GetData() -= pExpr->Eval(); break;
		case '#': pVar->GetData() *= pExpr->Eval(); break;
		case '$': pVar->GetData() /= pExpr->Eval(); break;
						
		default:
			throw UnicodeException(_T("Operation not defined"));
		}
	}
	virtual ~AssignCplx() { }
};

template<typename ltype>
class AssignVctr: public BaseAssign
{
protected:
	ltype* pVar; ExpReal *pExpr;
	const char optr;
public:
	AssignVctr(ltype *var, ExpReal* xpr, const char opr): 
	  optr(opr), pVar(var), pExpr(xpr){ }
	
	virtual void Run() 
	{ 
		switch(optr)
		{
		
		case '!': pVar->GetData() *= pExpr->Eval(); break;
		case '@': pVar->GetData() /= pExpr->Eval(); break;
						
		default:
			throw UnicodeException(_T("Operation not defined"));
		}
	}
	virtual ~AssignVctr() { }
};

template<typename T>
class AssignmentVarMem: public BaseAssign
{
protected:
	VarMem<T>* pVar; Expression<T> *pExpr;
	const char optr;
public:
	AssignmentVarMem(VarMem<T> *var, Expression<T>* xpr, const char opr = '='): 
	  optr(opr), pVar(var), pExpr(xpr), BaseAssign(L"No Name") 
	  { if(opr=='&') Statement::stmttype = StmtExpr; }
	
	virtual void Run() 
	{ 
		if(Statement::stmttype == StmtExpr) return;

		switch(optr)
		{
		case '=': pVar->GetData() = pExpr->Eval(); break;

		case '!': pVar->GetData() += pExpr->Eval(); return;
		case '@': pVar->GetData() -= pExpr->Eval(); return;
		case '#': pVar->GetData() *= pExpr->Eval(); return;
		case '$': pVar->GetData() /= pExpr->Eval(); return;

		case '%': ++pVar->GetData(); return; // increment
		case '^': --pVar->GetData(); return; // decrement
		
		default:
			throw UnicodeException(_T("Operation not defined.\r\n"));
		}
	}

	Expression<T>* GetExpression() { return pExpr; }

	virtual ~AssignmentVarMem() { }
};


template<>
class AssignmentVarMem<Complex>: public BaseAssign
{
protected:
	VarMem<Complex>* pVar; ExpCplx *pExpr;
	const char optr;
public:
	AssignmentVarMem(VarMem<Complex> *var, ExpCplx* xpr, const char opr = '='): 
	  optr(opr), pVar(var), pExpr(xpr), BaseAssign(L"No Name")  
	   { if(opr=='&') Statement::stmttype = StmtExpr; }
	
	virtual void Run() 
	{ 
		if(Statement::stmttype == StmtExpr) return;

		switch(optr)
		{
		case '=': pVar->GetData() = pExpr->Eval(); break;

		case '!': pVar->GetData() += pExpr->Eval(); return;
		case '@': pVar->GetData() -= pExpr->Eval(); return;
		case '#': pVar->GetData() *= pExpr->Eval(); return;
		case '$': pVar->GetData() /= pExpr->Eval(); return;

		default:
			throw UnicodeException(_T("Operation not defined.\r\n"));
		}
	}

	ExpCplx* GetExpression() { return pExpr; }

	virtual ~AssignmentVarMem() { }
};

template<>
class AssignmentVarMem<Vector>: public BaseAssign
{
protected:
	VarMem<Vector>* pVar; ExpVctr *pExpr;
	const char optr;
public:
	AssignmentVarMem(VarMem<Vector> *var, ExpVctr* xpr, const char opr = '='): 
	  optr(opr), pVar(var), pExpr(xpr), BaseAssign(L"No Name")
	   { if(opr=='&') Statement::stmttype = StmtExpr; }
	
	virtual void Run() 
	{ 
		if(Statement::stmttype == StmtExpr) return;

		switch(optr)
		{
		case '=': pVar->GetData() = pExpr->Eval(); break;

		case '!': pVar->GetData() += pExpr->Eval(); break;
		case '@': pVar->GetData() -= pExpr->Eval(); break;
		case '%': pVar->GetData() %= pExpr->Eval(); break;

		default:
			throw UnicodeException(_T("Operation not defined.\r\n"));
		}
	}

	ExpVctr* GetExpression() { return pExpr; }

	virtual ~AssignmentVarMem() { }
};

template<>
class AssignmentVarMem<bool>: public BaseAssign
{
protected:
	VarMem<bool>* pVar; ExpBool *pExpr;
	const char optr;
public:
	AssignmentVarMem(VarMem<bool> *var, ExpBool* xpr, const char opr = '='): 
	  optr(opr), pVar(var), pExpr(xpr), BaseAssign(L"No Name") { }
	
	virtual void Run() 
	{ 
		switch(optr)
		{
		case '=': pVar->GetData() = pExpr->Eval(); break;
		
		default:
			throw UnicodeException(_T("Operation not defined.\r\n"));
		}
	}

	ExpBool* GetExpression() { return pExpr; }

	virtual ~AssignmentVarMem() { }
};


template<>
class AssignmentVarMem<string>: public BaseAssign
{
protected:
	VarMem<string>* pVar; ExpStrn *pExpr;
	const char optr;
public:
	AssignmentVarMem(VarMem<string> *var, ExpStrn* xpr, const char opr = '='): 
	  optr(opr), pVar(var), pExpr(xpr), BaseAssign(L"No Name") { }
	
	virtual void Run() 
	{ 
		switch(optr)
		{
		case '=': pVar->GetData() = pExpr->Eval(); break;
		case '!': pVar->GetData() += pExpr->Eval(); return;
		
		default:
			throw UnicodeException(_T("Operation not defined.\r\n"));
		}
	}

	ExpStrn* GetExpression() { return pExpr; }

	virtual ~AssignmentVarMem() { }
};


template<typename T, typename ary>
class AssignVarMemAry: public BaseAssign
{
protected:
	VarMemAry<T, ary>* pVar; Expression<T> *pExpr;
	
public:
	AssignVarMemAry(VarMemAry<T, ary> *var, Expression<T>* xpr): 
	  pVar(var), pExpr(xpr), BaseAssign(L"No Name") {  }
	
	virtual void Run() 
	{ 
		pVar->GetData() = pExpr->Eval(); 
	}

	Expression<T>* GetExpression() { return pExpr; }

	virtual ~AssignVarMemAry() { }
};

template<typename T, typename ary>
class AssignVarUserAry: public BaseAssign
{
protected:
	VarUserAry<T, ary>* pVar; Expression<T> *pExpr;
	
public:
	AssignVarUserAry(VarUserAry<T, ary> *var, Expression<T>* xpr): 
	  pVar(var), pExpr(xpr), BaseAssign(L"No Name") {  }
	
	virtual void Run() 
	{ 
		pVar->GetData() = pExpr->Eval(); 
	}

	Expression<T>* GetExpression() { return pExpr; }

	virtual ~AssignVarUserAry() { }
};

class AssignVarMemCplxReal: public BaseAssign
{
protected:
	VarMem<Complex>* pVar; ExpReal *pExpr;
	const char optr;
public:
	AssignVarMemCplxReal(VarMem<Complex> *var, ExpReal* xpr, const char opr='='): 
	  pVar(var), pExpr(xpr), optr(opr), BaseAssign(L"No Name")  {  }
	
	virtual void Run() 
	{ 
		realnum real = pVar->GetData().real(), 
			v = pExpr->Eval();
		switch(optr)
		{
		case '=': pVar->GetData().real(v); return;
		
		case '!': pVar->GetData().real(real+v); return;
		case '@': pVar->GetData().real(real-v); return;
		case '#': pVar->GetData().real(real*v); return;
		case '$': pVar->GetData().real(real/v); return;

		case '%': pVar->GetData().real(++real); return; // increment
		case '^': pVar->GetData().real(--real); return; // decrement
		
		default:
			throw UnicodeException(_T("Operation not defined.\r\n"));
		}
	}

	ExpReal* GetExpression() { return pExpr; }

	virtual ~AssignVarMemCplxReal() { }
};

class AssignVarMemCplxImag: public BaseAssign
{
protected:
	VarMem<Complex>* pVar; ExpReal *pExpr;
	const char optr;
public:
	AssignVarMemCplxImag(VarMem<Complex> *var, ExpReal* xpr, const char opr='='): 
	  pVar(var), pExpr(xpr), optr(opr), BaseAssign(L"No Name")  {  }
	
	virtual void Run() 
	{ 
		realnum imag = pVar->GetData().imag(), 
			v = pExpr->Eval();
		switch(optr)
		{
		case '=': pVar->GetData().imag(v); return;
		
		case '!': pVar->GetData().imag(imag+v); return;
		case '@': pVar->GetData().imag(imag-v); return;
		case '#': pVar->GetData().imag(imag*v); return;
		case '$': pVar->GetData().imag(imag/v); return;

		case '%': pVar->GetData().imag(++imag); return; // increment
		case '^': pVar->GetData().imag(--imag); return; // decrement
		
		default:
			throw UnicodeException(_T("Operation not defined.\r\n"));
		}
	}

	ExpReal* GetExpression() { return pExpr; }

	virtual ~AssignVarMemCplxImag() { }
};

class AssignVarMemVctrX: public BaseAssign
{
protected:
	VarMem<Vector>* pVar; ExpReal *pExpr;
	
public:
	AssignVarMemVctrX(VarMem<Vector> *var, ExpReal* xpr): 
	  pVar(var), pExpr(xpr), BaseAssign(L"No Name")  {  }
	
	virtual void Run() 
	{ 
		pVar->GetData().x =  pExpr->Eval();
	}

	ExpReal* GetExpression() { return pExpr; }

	virtual ~AssignVarMemVctrX() { }
};

class AssignVarMemVctrY: public BaseAssign
{
protected:
	VarMem<Vector>* pVar; ExpReal *pExpr;
	
public:
	AssignVarMemVctrY(VarMem<Vector> *var, ExpReal* xpr): 
	  pVar(var), pExpr(xpr), BaseAssign(L"No Name")  {  }
	
	virtual void Run() 
	{ 
		pVar->GetData().y =  pExpr->Eval();
	}

	ExpReal* GetExpression() { return pExpr; }

	virtual ~AssignVarMemVctrY() { }
};

class AssignVarMemVctrZ: public BaseAssign
{
protected:
	VarMem<Vector>* pVar; ExpReal *pExpr;
	
public:
	AssignVarMemVctrZ(VarMem<Vector> *var, ExpReal* xpr): 
	  pVar(var), pExpr(xpr), BaseAssign(L"No Name")  {  }
	
	virtual void Run() 
	{ 
		pVar->GetData().z =  pExpr->Eval();
	}

	ExpReal* GetExpression() { return pExpr; }

	virtual ~AssignVarMemVctrZ() { }
};

template<typename T, typename ary>
class InitializeArray: public BaseAssign
{
protected:
	ary *pLHS; Expression<T>* pExpr; 
public:
	InitializeArray(ary *pLeft, Expression<T>* pRight): 
		pLHS(pLeft), pExpr(pRight), BaseAssign(pLeft->GetId()) { }
	
	virtual void Run() 
	{ pLHS->Initialize(pExpr->Eval()); }
	
	virtual ~InitializeArray() { }
};

template<typename vec, typename ary>
class AssignArray: public BaseAssign
{
protected:
	ary *pLHS; Expression<vec>* pExpr; const char optr;
public:
	AssignArray(ary *pLeft, Expression<vec>* pRight, const char opr): 
		pLHS(pLeft), pExpr(pRight), optr(opr), BaseAssign(pLeft->GetId()) { }
	
	virtual void Run() 
	{ 
		vec &v = *pLHS->GetTop();

		switch(optr)
		{
		case '=' : v = pExpr->Eval(); return;
		case '!' : 
			{
				if(v.size() == 0) 
					throw UnicodeException(_T("Array on the left hand side is not yet initialized."));
				v += pExpr->Eval(); return;
			}
			
		case '@' : 
			{
				if(v.size() == 0) 
					throw UnicodeException(_T("Array on the left hand side is not yet initialized."));
				v -= pExpr->Eval(); return;
			}
		case '#' : 
			{
				if(v.size() == 0) 
					throw UnicodeException(_T("Array on the left hand side is not yet initialized."));
				v *= pExpr->Eval(); return;
			}
		case '$' : 
			{
				if(v.size() == 0) 
					throw UnicodeException(_T("Array on the left hand side is not yet initialized."));
				v /= pExpr->Eval(); return;
			}
		
		default:
			throw UnicodeException(_T("Operation not defined"));
		}
	}
	
	virtual ~AssignArray() { }
};

template<typename vec, typename ary>
class AssignArrayVec: public BaseAssign
{
protected:
	ary *pLHS; Expression<vec>* pExpr; const char optr;
public:
	AssignArrayVec(ary *pLeft, Expression<vec>* pRight, const char opr): 
		pLHS(pLeft), pExpr(pRight), optr(opr), BaseAssign(pLeft->GetId()) { }
	
	virtual void Run() 
	{   
		vec& v = *pLHS->GetTop();

		switch(optr)
		{
		case '=' : v = pExpr->Eval(); return;
		
		case '!' : 
			{
				if(v.size() == 0) 
					throw UnicodeException(_T("Array on the left hand side is not yet initialized."));
				v += pExpr->Eval(); return;
			}
		case '@' : 
			{
				if(v.size() == 0) 
					throw UnicodeException(_T("Array on the left hand side is not yet initialized."));
				v -= pExpr->Eval(); return;
			}
		case '#' : 
			{
				if(v.size() == 0) 
					throw UnicodeException(_T("Array on the left hand side is not yet initialized."));
				v %= pExpr->Eval(); return;
			}
				
		default:
			throw UnicodeException(_T("Operation not defined"));
		}
	}
	
	virtual ~AssignArrayVec() { }
};


template<typename T, typename VarAry>
class AssignAryEle: public BaseAssign
{
protected:
	VarAry* pVar; Expression<T>* pExpr; const char optr;

public:
	AssignAryEle(VarAry* var, Expression<T>* xpr, char opr):
		pVar(var), pExpr(xpr), optr(opr), BaseAssign(var->GetId())
		{ if(optr=='&') Statement::stmttype = StmtExpr; }

		AssignAryEle(const std::wstring& id, VarAry* var, Expression<T>* xpr, char opr):
		pVar(var), pExpr(xpr), optr(opr), BaseAssign(id)
		{ if(optr=='&') Statement::stmttype = StmtExpr; }
	
	VarAry* GetVarAry() { return pVar; }

	virtual void Run() 
	{
		if(optr == '&') return;

		switch(optr)
		{
		case '+': ++pVar->GetData(); return;
		case '-': --pVar->GetData(); return;
		case '=': pVar->GetData() = pExpr->Eval(); return;

		case '!': pVar->GetData() += pExpr->Eval(); return;
		case '@': pVar->GetData() -= pExpr->Eval(); return;
		case '#': pVar->GetData() *= pExpr->Eval(); return;
		case '$': 
			{
				T value = pExpr->Eval();
				if(value == T()) throw UnicodeException(_T("Division By Zero"));

				pVar->GetData() /= value; return;
			}
		default:
			throw RunTimeException(_T("Unsupported Assignment Operator.\r\n"));
		}
	}
	
	Expression<T>* GetExpression() { return pExpr; }

	virtual ~AssignAryEle() { }
};

template<typename VarAry>
class AssignAryEle<Complex, VarAry>: public BaseAssign
{
protected:
	VarAry* pVar; ExpCplx* pExpr; const char optr;

public:
	AssignAryEle(VarAry* var, ExpCplx* xpr, char opr):
		pVar(var), pExpr(xpr), optr(opr), BaseAssign(var->GetId())
		{ if(optr=='&') Statement::stmttype = StmtExpr; }
	
		AssignAryEle(const std::wstring& id, VarAry* var, ExpCplx* xpr, char opr):
		pVar(var), pExpr(xpr), optr(opr), BaseAssign(id)
		{ if(optr=='&') Statement::stmttype = StmtExpr; }

	VarAry* GetVarAry() { return pVar; }

	virtual void Run() 
	{
		if(optr == '&') return;

		switch(optr)
		{
		case '=': pVar->GetData() = pExpr->Eval(); return;

		case '!': pVar->GetData() += pExpr->Eval(); return;
		case '@': pVar->GetData() -= pExpr->Eval(); return;
		case '#': pVar->GetData() *= pExpr->Eval(); return;
		case '$': 
			{
				Complex value = pExpr->Eval();
				if(value == Complex()) throw UnicodeException(_T("Division By Zero"));

				pVar->GetData() /= value; return;
			}
		default:
			throw RunTimeException(_T("Unsupported Assignment Operator.\r\n"));
		}
	}
	
	ExpCplx* GetExpression() { return pExpr; }

	virtual ~AssignAryEle() { }
};

template<typename VarAry>
class AssignAryEleCplxReal: public BaseAssign
{
protected:
	VarAry* pVar; ExpReal* pExpr; 

public:
	AssignAryEleCplxReal(VarAry* var, ExpReal* xpr):
		pVar(var), pExpr(xpr), BaseAssign(L"No Name") { }
	
	VarAry* GetVarAry() { return pVar; }

	virtual void Run() 
	{
		pVar->GetData().real(pExpr->Eval()); 
	}
	
	ExpReal* GetExpression() { return pExpr; }

	virtual ~AssignAryEleCplxReal() { }
};

template<typename VarAry>
class AssignAryEleCplxImag: public BaseAssign
{
protected:
	VarAry* pVar; ExpReal* pExpr; 

public:
	AssignAryEleCplxImag(VarAry* var, ExpReal* xpr):
		pVar(var), pExpr(xpr), BaseAssign(L"No Name") { }
	
	VarAry* GetVarAry() { return pVar; }

	virtual void Run() 
	{
		pVar->GetData().imag(pExpr->Eval()); 
	}
	
	ExpReal* GetExpression() { return pExpr; }

	virtual ~AssignAryEleCplxImag() { }
};

template<typename VarAry>
class AssignAryEleVctrX: public BaseAssign
{
protected:
	VarAry* pVar; ExpReal* pExpr; 

public:
	AssignAryEleVctrX(VarAry* var, ExpReal* xpr):
		pVar(var), pExpr(xpr), BaseAssign(L"No Name") { }
	
	VarAry* GetVarAry() { return pVar; }

	virtual void Run() 
	{
		pVar->GetData().x = pExpr->Eval(); 
	}
	
	ExpReal* GetExpression() { return pExpr; }

	virtual ~AssignAryEleVctrX() { }
};

template<typename VarAry>
class AssignAryEleVctrY: public BaseAssign
{
protected:
	VarAry* pVar; ExpReal* pExpr; 

public:
	AssignAryEleVctrY(VarAry* var, ExpReal* xpr):
		pVar(var), pExpr(xpr), BaseAssign(L"No Name") { }
	
	VarAry* GetVarAry() { return pVar; }

	virtual void Run() 
	{
		pVar->GetData().y = pExpr->Eval(); 
	}
	
	ExpReal* GetExpression() { return pExpr; }

	virtual ~AssignAryEleVctrY() { }
};

template<typename VarAry>
class AssignAryEleVctrZ: public BaseAssign
{
protected:
	VarAry* pVar; ExpReal* pExpr; 

public:
	AssignAryEleVctrZ(VarAry* var, ExpReal* xpr):
		pVar(var), pExpr(xpr), BaseAssign(L"No Name") { }
	
	VarAry* GetVarAry() { return pVar; }

	virtual void Run() 
	{
		pVar->GetData().z = pExpr->Eval(); 
	}
	
	ExpReal* GetExpression() { return pExpr; }

	virtual ~AssignAryEleVctrZ() { }
};

template<typename VarAry>
class AssignAryEle<Vector, VarAry>: public BaseAssign
{
protected:
	VarAry* pVar; ExpVctr* pExpr; const char optr;

public:
	AssignAryEle(VarAry* var, ExpVctr* xpr, char opr):
		pVar(var), pExpr(xpr), optr(opr), BaseAssign(var->GetId()) 
		{ if(optr=='&') Statement::stmttype = StmtExpr; }

		AssignAryEle(const std::wstring& id, VarAry* var, ExpVctr* xpr, char opr):
		pVar(var), pExpr(xpr), optr(opr), BaseAssign(id) 
		{ if(optr=='&') Statement::stmttype = StmtExpr; }
	
	VarAry* GetVarAry() { return pVar; }

	virtual void Run() 
	{
		if(optr=='&') return;

		switch(optr)
		{
		case '=': pVar->GetData() = pExpr->Eval(); return;

		case '!': pVar->GetData() += pExpr->Eval(); return;
		case '@': pVar->GetData() -= pExpr->Eval(); return;
		case '#': pVar->GetData() %= pExpr->Eval(); return;
		
		default:
			throw RunTimeException(_T("Unsupported Assignment Operator.\r\n"));
		}
	}

	ExpVctr* GetExpression() { return pExpr; }
	
	virtual ~AssignAryEle() { }
};

template<typename T>
class AssignmentPtr : public Statement
{
protected:
	PtrData<T>* pPtrData; LeftHandSide<T>* uPtr;
public:

	AssignmentPtr(PtrData<T> *var, LeftHandSide<T>* pu):
	  pPtrData(var), uPtr(pu) { }
	
	virtual void Run() 
		{ *pPtrData->GetPtr() = &uPtr->GetData(); }

	virtual ~AssignmentPtr() { }
};

template<typename T, typename U>
class AssignPtr : public Statement
{
protected:
	PtrData<T>* pPtrData; U* uPtr;
public:

	AssignPtr(PtrData<T> *var, U* pu):
	  pPtrData(var), uPtr(pu) { }
	
	virtual void Run() { *pPtrData->GetPtr() = &uPtr->GetData(); }

	virtual ~AssignPtr() { }
};

template<typename T>
class AssignVarToPtr: public Statement
{
protected:
	LeftHandSide<T*>* pLHS; LeftHandSide<T> *pRHS;
public:
	AssignVarToPtr(LeftHandSide<T*> *lhs, LeftHandSide<T> *rhs):
	  pLHS(lhs), pRHS(rhs) {}

	  virtual void Run()
	  {
		  pLHS->GetData() = &pRHS->GetData();
	  }

	  virtual ~AssignVarToPtr() {}
};

template<>
class AssignVarToPtr<UserData>: public Statement
{
protected:
	UdtPtrBase* pLHS; LeftHandSide<UserData> *pRHS;
public:
	AssignVarToPtr(UdtPtrBase *lhs, LeftHandSide<UserData> *rhs):
	  pLHS(lhs), pRHS(rhs) {}

	  virtual void Run()
	  {
		  pLHS->SetDataPtr(&pRHS->GetData());
	  }

	  virtual ~AssignVarToPtr() {}
};

template<typename T>
class AssignExpPtrToPtr: public Statement
{
protected:
	LeftHandSide<T*>* pLHS; Expression<T*> *pRHS;
public:
	AssignExpPtrToPtr(LeftHandSide<T*> *lhs, Expression<T*> *rhs):
	  pLHS(lhs), pRHS(rhs) {}

	  virtual void Run()
	  {
		  pLHS->GetData() = pRHS->Eval();
	  }

	  virtual ~AssignExpPtrToPtr() {}
};

template<>
class AssignExpPtrToPtr<UserData>: public Statement
{
protected:
	UdtPtrBase* pLHS; Expression<UserData*> *pRHS;
public:
	AssignExpPtrToPtr(UdtPtrBase* lhs, Expression<UserData*> *rhs):
	  pLHS(lhs), pRHS(rhs) {}

	  virtual void Run()
	  {
		  pLHS->SetDataPtr(pRHS->Eval());
	  }

	  virtual ~AssignExpPtrToPtr() {}
};

template<typename T, typename U>
class AssignPtrPtr : public Statement
{
protected:
	PtrData<T> *pPtrData1, *pPtrData2;
public:

	AssignPtrPtr(PtrData<T> *var1, PtrData<T> *var2):
	  pPtrData1(var1), pPtrData2(var2) { }
	
	virtual void Run() { *pPtrData1->GetPtr() = pPtrData2->GetData(); }

	virtual ~AssignPtrPtr() { }
};

template<typename vec, typename ary>
class AssignAptr : public Statement
{
protected:
	AptrData<vec>* pPtrData; ary* uPtr;
public:
	AssignAptr(AptrData<vec> *var, ary* pu):
	  pPtrData(var), uPtr(pu) { }
	
	virtual void Run() 
	{ 
		*pPtrData->GetPtr() = uPtr->GetTop(); 
	}

	virtual ~AssignAptr() { }
};

template<typename vec>
class AssignmentAptr: public Statement
{
protected:
	AptrData<vec>* pPtrData; LeftHandSide<vec>* pRHS;
public:
	AssignmentAptr(AptrData<vec> *var, LeftHandSide<vec>* pu):
	  pPtrData(var), pRHS(pu) { }
	
	virtual void Run() 
	{ 
		*pPtrData->GetPtr() = &pRHS->GetData(); 
	}

	virtual ~AssignmentAptr() { }
};

template<typename vec>
class AssignAptrAptr : public Statement
{
protected:
	AptrData<vec> *pPtrData1, *pPtrData2;
public:
	AssignAptrAptr(AptrData<vec> *var1, AptrData<vec> *var2):
	  pPtrData1(var1), pPtrData2(var2) { }
	
	virtual void Run() 
	{ 
		*pPtrData1->GetPtr() = pPtrData2->GetData();
	}

	virtual ~AssignAptrAptr() { }
};

template<typename T, typename vec>
class AssignAptr1: public Statement
{
protected:
	AptrData<vec>* pData; 
	ExpIntg *pIndex1; Expression<T>* pExpr;

public:
	
	AssignAptr1(AptrData<vec>* vptr, ExpIntg* idx1, Expression<T>* xpr): 
	pData(vptr), pIndex1(idx1), pExpr(xpr), Statement(StmtAsgn) {}
	
	virtual void Run() 
	{ 
		vec* ptr = pData->GetData();
		if(ptr)
			ptr->at(int(pIndex1->Eval())) = pExpr->Eval();
		else
			throw UnicodeException(_T("Pointer is not initialized"));
	}

	virtual std::wstring Display() { return std::wstring(); }
	virtual ~AssignAptr1() { }
};


template<typename T, typename vec>
class AssignAptr2: public Statement
{
protected:
	AptrData<vec>* pData; Expression<T>* pExpr;
	ExpIntg *pIndex1, *pIndex2;

public:
	
	AssignAptr2(AptrData<vec>* vptr, 
		ExpIntg* idx1, ExpIntg* idx2, Expression<T>* xpr): 
	pData(vptr), pIndex1(idx1), pIndex2(idx2), pExpr(xpr), Statement(StmtAsgn) {}
	
	virtual void Run() 
	{ 
		vec* ptr = pData->GetData();
		if(ptr)
			ptr->at(int(pIndex1->Eval())).at(int(pIndex2->Eval()))=pExpr->Eval();
		else
			throw UnicodeException(_T("Pointer is not initialized"));
	}

	virtual std::wstring Display() { return std::wstring(); }
	virtual ~AssignAptr2() { }
};

template<typename T, typename vec>
class AssignAptr3: public Statement
{
protected:
	AptrData<vec>* pData; Expression<T>* pExpr;
	ExpIntg *pIndex1, *pIndex2, *pIndex3;

public:
	
	AssignAptr3(AptrData<vec>* vptr, ExpIntg* idx1, 
		ExpIntg* idx2, ExpIntg* idx3, Expression<T>* xpr): 
	pData(vptr), pIndex1(idx1), pIndex2(idx2), pIndex3(idx3), pExpr(xpr), Statement(StmtAsgn) {}
	
	virtual void Run() 
	{ 
		vec* ptr = pData->GetData();
		if(ptr)
			ptr->at(int(pIndex1->Eval())).at(int(pIndex2->Eval())).at(int(pIndex3->Eval()))= pExpr->Eval();
		else
			throw UnicodeException(_T("Pointer is not initialized"));
	}

	virtual std::wstring Display() { return std::wstring(); }
	virtual ~AssignAptr3() { }
};

template<typename T, typename vec>
class AssignAptr4: public Statement
{
protected:
	AptrData<vec>* pData; Expression<T>* pExpr;
	ExpIntg *pIndex1, *pIndex2, *pIndex3, *pIndex4;

public:
	
	AssignAptr4(AptrData<vec>* vptr, ExpIntg* idx1, ExpIntg* idx2, 
		ExpIntg* idx3, ExpIntg* idx4, Expression<T>* xpr): 
	pData(vptr), pIndex1(idx1), pIndex2(idx2), pIndex3(idx3), pIndex4(idx4), pExpr(xpr), Statement(StmtAsgn) {}
	
	virtual void Run() 
	{ 
		vec* ptr = pData->GetData();
		if(ptr)
			ptr->at(int(pIndex1->Eval())).at(int(pIndex2->Eval())).
		 		 at(int(pIndex3->Eval())).at(int(pIndex4->Eval()))= pExpr->Eval();
		else
			throw UnicodeException(_T("Pointer is not initialized"));
	}

	virtual std::wstring Display() { return std::wstring(); }
	virtual ~AssignAptr4() { }
};

template<typename T, typename vec>
class AssignAptr5: public Statement
{
protected:
	AptrData<vec>* pData; Expression<T>* pExpr;
	ExpIntg *pIndex1, *pIndex2, *pIndex3, *pIndex4, *pIndex5;

public:
	
	AssignAptr5(AptrData<vec>* vptr, ExpIntg* idx1, ExpIntg* idx2, 
		ExpIntg* idx3, ExpIntg* idx4, ExpIntg* idx5, Expression<T>* xpr): 
	pData(vptr), pIndex1(idx1), pIndex2(idx2), pIndex3(idx3), pIndex4(idx4), pIndex5(idx5),
		pExpr(xpr), Statement(StmtAsgn) {}
	
	virtual void Run() 
	{ 
		vec* ptr = pData->GetData();
		if(ptr)
			ptr->at(int(pIndex1->Eval())).at(int(pIndex2->Eval())).
				 at(int(pIndex3->Eval())).at(int(pIndex4->Eval())).
				 at(int(pIndex5->Eval()))= pExpr->Eval();
		else
			throw UnicodeException(_T("Pointer is not initialized"));
	}

	virtual std::wstring Display() { return std::wstring(); }
	virtual ~AssignAptr5() { }
};

template<typename T, typename U>
class AssignPtrValue : public Statement
{
protected:
	const char optr;
	PtrData<T>* pPtrData; Expression<T>* pExpr;
public:

	AssignPtrValue(PtrData<T> *var, Expression<T>* xpr, const char opr = '=' ):
	  optr(opr), pPtrData(var), pExpr(xpr) { }
	
	virtual void Run()
	{ 
		switch(optr)
		{
		case '=': *pPtrData->GetData() = pExpr->Eval(); break;
		
		case '!': *pPtrData->GetData() += pExpr->Eval(); break;
		case '@': *pPtrData->GetData() -= pExpr->Eval(); break;
		case '#': *pPtrData->GetData() *= pExpr->Eval(); break;
		case '$': *pPtrData->GetData() /= pExpr->Eval(); break;

		case '%': ++(*pPtrData->GetData()); break; // increment
		case '^': --(*pPtrData->GetData()); break; // decrement

		default:
			throw UnicodeException(_T("Operation not defined"));
		}	
	}

	virtual ~AssignPtrValue() { }
};

template<>
class AssignPtrValue<bool, BoolData> : public Statement
{
protected:
	const char optr;
	PtrData<bool>* pPtrData; ExpBool* pExpr;
public:

	AssignPtrValue(PtrData<bool> *var, ExpBool* xpr, const char opr = '=' ):
	  optr(opr), pPtrData(var), pExpr(xpr) { }
	
	virtual void Run()
	{ 
		switch(optr)
		{
		case '=': *pPtrData->GetData() = pExpr->Eval(); break;
		
		default:
			throw UnicodeException(_T("Operation not defined"));
		}
	}

	virtual ~AssignPtrValue() { }
};

template<>
class AssignPtrValue<Complex, CplxData> : public Statement
{
protected:
	const char optr;
	PtrData<Complex>* pPtrData; ExpCplx* pExpr;
public:

	AssignPtrValue(PtrData<Complex> *var, ExpCplx* xpr, const char opr = '=' ):
	  optr(opr), pPtrData(var), pExpr(xpr) { }
	
	virtual void Run()
	{ 
		switch(optr)
		{
				
		case '=': *pPtrData->GetData() = pExpr->Eval(); break;
		
		case '!': *pPtrData->GetData() += pExpr->Eval(); break;
		case '@': *pPtrData->GetData() -= pExpr->Eval(); break;
		case '#': *pPtrData->GetData() *= pExpr->Eval(); break;
		case '$': *pPtrData->GetData() /= pExpr->Eval(); break;
		
		default:
			throw UnicodeException(_T("Operation not defined"));
		}
	}

	virtual ~AssignPtrValue() { }
};

class AssignPtrValueCplx : public Statement
{
protected:
	const char optr;
	PtrData<Complex>* pPtrData; ExpReal* pExpr;
public:

	AssignPtrValueCplx(PtrData<Complex> *var, ExpReal* xpr, const char opr):
	  optr(opr), pPtrData(var), pExpr(xpr) { }
	
	virtual void Run()
	{ 
		switch(optr)
		{
				
		case '!': *pPtrData->GetData() *= pExpr->Eval(); break;
		case '@': *pPtrData->GetData() /= pExpr->Eval(); break;
		
		default:
			throw UnicodeException(_T("Operation not defined"));
		}
	}

	virtual ~AssignPtrValueCplx() { }
};

class AssignPtrValueVctr : public Statement
{
protected:
	const char optr;
	PtrData<Vector>* pPtrData; ExpReal* pExpr;
public:

	AssignPtrValueVctr(PtrData<Vector> *var, ExpReal* xpr, const char opr):
	  optr(opr), pPtrData(var), pExpr(xpr) { }
	
	virtual void Run()
	{ 
		switch(optr)
		{
				
		case '!': *pPtrData->GetData() *= pExpr->Eval(); break;
		case '@': *pPtrData->GetData() /= pExpr->Eval(); break;
		
		default:
			throw UnicodeException(_T("Operation not defined"));
		}
	}

	virtual ~AssignPtrValueVctr() { }
};

template<>
class AssignPtrValue<Vector, VctrData> : public Statement
{
protected:
	const char optr;
	PtrData<Vector>* pPtrData; ExpVctr* pExpr;
public:

	AssignPtrValue(PtrData<Vector> *var, ExpVctr* xpr, const char opr = '=' ):
	  optr(opr), pPtrData(var), pExpr(xpr) { }
	
	virtual void Run()
	{ 
		switch(optr)
		{
		case '=': *pPtrData->GetData() = pExpr->Eval(); break;
		
		case '!': *pPtrData->GetData() += pExpr->Eval(); break;
		case '@': *pPtrData->GetData() -= pExpr->Eval(); break;
		case '%': *pPtrData->GetData() %= pExpr->Eval(); break;
				
		default:
			throw UnicodeException(_T("Operation not defined"));
		}
	}

	virtual ~AssignPtrValue() { }
};

template<typename T, typename U, typename VarAry>
class AssignPtrAryEle: public BaseAssign
{
protected:
	PtrData<T>* pPtrData; VarAry* pVar;
public:
	
	AssignPtrAryEle(PtrData<T>* ptr, VarAry* var):
			pVar(var), pPtrData(ptr),BaseAssign(var->GetId())   { }
	
	virtual void Run() { *pPtrData->GetPtr()=&pVar->GetData(); }
	
	virtual ~AssignPtrAryEle() { }
};

class Label: public Statement
{
protected:
	std::wstring slabel;
public:
	Label(std::wstring txt): slabel(txt), Statement(StmtLabel) {  }
	virtual void Run() { }
	virtual bool IsLabel(std::wstring label=_T("")) { return slabel == label; }
	virtual ~Label() { }
};

class Break: public Statement
{
public:
	Break(): Statement(StmtBreak) { }
	
	virtual void Run()
	{
		throw BreakException();
	}
	
	virtual ~Break() { }
};

class StatementList;

template<typename T>
class Return: public Statement
{
protected:
	
	Expression<T>* pExp;
	Stack<void*> *rtnStack;
	StatementList *pStmtList;

public:
	
	Return(Stack<void*> *stack, Expression<T>* xpr, StatementList *pSlist): 
	  Statement(StmtRtn), pExp(xpr), rtnStack(stack), pStmtList(pSlist) { }

	virtual bool IsReturn() const { return true; }

	virtual void Run();
		
	virtual ~Return() { }
};

class ReturnUdt: public Statement
{
protected:
	
	LeftHandSide<UserData>* pExp;
	Stack<void*> *rtnStack;
	StatementList *pStmtList;

public:
	
	ReturnUdt(Stack<void*> *stack, LeftHandSide<UserData>* xpr, StatementList *pSlist): 
	  Statement(StmtRtn), pExp(xpr), rtnStack(stack), pStmtList(pSlist) { }

	virtual bool IsReturn() const { return true; }

	virtual void Run();
			
	virtual ~ReturnUdt() { }
};

class GoTo: public Statement
{
protected:
	std::wstring slabel;
public:
	GoTo(std::wstring txt): Statement(StmtGoto), slabel(txt) { }

	virtual void Run()
	{
		throw GotoException(slabel.c_str());
	}
	
	virtual ~GoTo() { }
};

template<typename T>
class StmtDelete: public Statement
{
protected:
	LeftHandSide<T*> *pLHS;
public:
	StmtDelete(LeftHandSide<T*> *ptr): pLHS(ptr) {}
	virtual void Run()
	{
		extern MemoryManager gMM;

		gMM.Release(pLHS->GetData());
	
		pLHS->GetData() = NULL;
	}

	virtual ~StmtDelete() { }
};

template<>
class StmtDelete<UserData>: public Statement
{
protected:
	UdtPtrBase *pLHS;
public:
	StmtDelete(UdtPtrBase *ptr): pLHS(ptr) {}
	virtual void Run()
	{
		extern MemoryManager gMM;

		gMM.Release(pLHS->GetUserData());
		pLHS->SetDataPtr(NULL);
	}

	virtual ~StmtDelete() { }
};

class StmtAbort: public Statement
{
protected:
	ExpStrn *Expr;
public:
	StmtAbort(ExpStrn *xpr): Statement(StmtExit), Expr(xpr) { }

	virtual void Run()
	{
		throw UserAbortException(Expr->Eval().c_str());
	}
	
	virtual ~StmtAbort() { }
};

class StmtClear: public Statement
{

public:
	StmtClear(){ }

	virtual void Run() { ClearOutput(); }
	
	virtual ~StmtClear() { }
};

class Skip: public Statement
{
public:
	Skip(): Statement(StmtSkip) { }

	virtual void Run()
	{
		throw SkipException();
	}
	
	virtual ~Skip() { }
};


class StmtIf: public Statement
{
protected:
	
	ExpBool* pExpr;
	Statement *pStmt, *pElse; bool kind;
public:
	StmtIf(ExpBool* xpr, Statement *pstmt, bool b, Statement *pelse);
	
	virtual void Run();

	virtual bool IsReturn() const;

	virtual ~StmtIf() { }
};

class StmtIfElse: public Statement
{
protected:
	
	ExpBool* pExpr;
	Statement *pStmt1, *pStmt2;
public:
	StmtIfElse(ExpBool* xpr, Statement* stmt1, Statement* stmt2);
	
	virtual void Run();

	virtual bool IsReturn() const;

	virtual ~StmtIfElse() { }
};

template<typename T>
class ExprType: public Statement
{
protected:
	Expression<T>* expr;
public:

	ExprType(Expression<T>* xpr): expr(xpr) { }
	virtual void Run()
	{ 
		expr->Eval();

	}

	virtual ~ExprType() { }
};

class Function;

class StmtListBase: virtual public BaseType
{
protected:
	
	NameUdtMap udtmap; 
	UDFuncList funclist; 

	void copy(const StmtListBase& sb)
	{
		this->udtmap = sb.udtmap;
		this->funclist = sb.funclist;
	}
public:

	StmtListBase();
	StmtListBase(const StmtListBase& sb) { copy(sb); }
	StmtListBase& operator=(const StmtListBase& sb) { copy(sb); return *this; }

	void CheckImplemented() const;
	
	Function* FindFunction(const std::wstring& id);
	void AddUDFunction(const wchar_t* id, Function* fptr);
	void AddFactory(UserDefinedType *udt);

	NameUdtMap* GetFactoryMap();
		
	virtual ~StmtListBase();
};

class StatementList: 
	virtual public StmtListBase, virtual public Statement
{
protected:
	StmtVector stmtlist;  
	VarList varlist;

	void copy(const StatementList& sl) 
	{
		StmtListBase::copy(sl);

		this->stmttype = sl.stmttype;

		this->stmtlist = sl.stmtlist;
		this->varlist = sl.varlist;
		
		this->stmtindex = sl.stmtindex;
		this->funcstmt = sl.funcstmt;
	}
public:
	int stmtindex; // current stmtindex
	bool funcstmt;

	virtual int PrepareStack();
	virtual int CleanupStack();

	void CheckImplemented() const;

	virtual bool IsReturn() const;

	Statement* GetStatement(int index) { return stmtlist.at(index); }
	Statement* GetLastStmt() 
	{ 
		if(stmtlist.empty()) return NULL;
		else return stmtlist.back(); 
	}

	Statement* GetHeadStmt() 
	{ 
		if(stmtlist.empty()) return NULL;
		else return stmtlist.front(); 
	}

	void CheckIdentical(const Statement* pStmt, int index)
	{
		if(index>=varlist.size())
			HandleSyntaxError(_T("Number of function parameters exceeds than declared.\r\n"));

		stmtlist[index]->CheckIdentical(pStmt);
	}

	int GetStmtSize() const { return int(this->stmtlist.size()); }
	int GetStmtListSize() const
	{
		int rlt=0, count = int(this->stmtlist.size());
		for(int i=0; i<count; ++i)
		{
			if(dynamic_cast<StatementList*>(stmtlist[i])) ++rlt;
		}

		return rlt;
	}

	bool IsLastStmtList() const 
	{ 
		if(!stmtlist.empty() && dynamic_cast<StatementList*>(stmtlist.back()))
			return true;
		else return false;
	}

	StatementList();
	
	StatementList(const StatementList& sl) { copy(sl); }
	StatementList& operator=(const StatementList& sl) { copy(sl); return *this; }
	
	StatementList* Clone(StatementList* ptr) const 
	{ 
		*ptr = *this; 
		ptr->varlist.PreserveData(); return ptr; 
	}

	VarList* GetVarList();
		
	//int Add(Statement* stmt);
	int Add(Statement* stmt);
	int AddToHead(Statement* stmt);
	
	virtual void Run();
	StmtVector& GetStmtList();

	// if fails, returns -1, otherwise returns stmtlist index
	int FindLabel(std::wstring& id);
		
	template<typename T> 
	AssignmentLHS<T>* FindAssignment(LeftHandSide<T>* pLHS);

	template<typename vec>
	StmtInitArray* FindAssignment(LeftHandSideArray<vec> *pLHS);

	Statement* FindAssignment(const std::wstring& id);

	template<typename T, typename VarAry>
	AssignAryEle<T, VarAry>* FindAssignment(VarAry *vptr);

	template<typename T, typename VarAry>
	Statement* FindAssignment(const std::wstring& id, integer idx1);
	template<typename T, typename VarAry>
	Statement* FindAssignment(const std::wstring& id, integer idx1, integer idx2);
	template<typename T, typename VarAry>
	Statement* FindAssignment(const std::wstring& id, integer idx1, integer idx2, integer idx3);
	template<typename T, typename VarAry>
	Statement* FindAssignment(const std::wstring& id, integer idx1, integer idx2, integer idx3, integer idx4);
	template<typename T, typename VarAry>
	Statement* FindAssignment(const std::wstring& id, integer idx1, integer idx2, integer idx3, integer idx4, integer idx5);

	Statement* FindDeclare(const std::wstring& id);

	// if fails, returns -1, otherwise returns stmtlist index
	bool FindLoop();
	virtual ~StatementList();
};

template<typename T>
void Return<T>::Run()
{
	extern Stack<StatementList*> gStmtListStack;
	void *ptr = rtnStack->top();
	if(ptr && pExp) *((T*)ptr) = pExp->Eval();

	int i=1;
	StatementList *pSL = gStmtListStack.getoffset(i++);
	while(pStmtList != pSL)
	{
		pSL->CleanupStack();
		pSL = gStmtListStack.getoffset(i++);
	}
	
	throw ReturnException();
}


template<typename T, typename U>
T* LookupDataPtr(const std::wstring id)
{
	extern Stack<StatementList*> gStmtListStack;
	StatementList *pStatementList; VarList *pVarList;
	
	int count = gStmtListStack.size();
	for(int i=0; i<count; ++i)
	{
		SET_USER_ABORT;
		
		pStatementList = gStmtListStack.getoffset(i);
		// pVarList is always not NULL
		pVarList = pStatementList->GetVarList();
		type = pVarList->GetType(id);
		if(type != TypeNull)
		{
			U* ptr = dynamic_cast<U*>(pVarList->operator[](id));
			if(ptr) return &ptr->GetData();
			else throw UnicodeException(LoadUnicodeMsg(IDS_CALLFUNC_TYPE_MISMATCH).c_str());
		}
	}
	
	throw UnicodeException(_T("Data not found."));
}

class Function: public Statement
{
public:
	StatementList* pStmtList, *pSlist; 
	UserData *pUdt; Function *pParen;
	UserDefinedType *pUDTFactory; bool NotUseMember;


	DataType rtntype; std::wstring fname;
	
	Stack<void*> rtnStack;  ScanFile scanfile;
		
	Function(DataType rtn, const std::wstring& name);
	
	UserData *SetUdtFromParent();

	void InsertParam(Statement *pParam, int index);
	void CheckArgs(BaseType *xpr, int index);

	virtual void Run();
	virtual bool IsReturn() const;
	virtual std::wstring GetName();

	virtual ~Function();
};

class MyPunck: public std::numpunct<wchar_t>
{
public:
	typedef wchar_t char_type;
	typedef std::wstring string_type;

	explicit MyPunck(size_t r=2): std::numpunct<wchar_t>(r) {}

protected:
	
	wchar_t do_decimal_point() const 
	{ extern wchar_t decimal_point; return decimal_point;  }
	
	wchar_t do_thousands_sep() const 
	{ extern wchar_t thousand_point; return thousand_point;  }

	std::string do_grouping() const 
	{ extern std::string grouping_point; return grouping_point; }

};

class StmtOutput: public Statement
{
protected:
	typedef std::vector<ExpTypeless*> VecType;
	typedef std::vector<ExpTypeless*>::iterator ItrType;
	bool initialized;
	VecType exprvec; 
	Expression<stream&> *pStream;
	std::locale loc;
	std::wostringstream os;
	void SetStream();
public:
	StmtOutput(Expression<stream&> *pstrm=NULL);
	void InsertExpr(ExpTypeless* expr);
	virtual void Run();
	virtual ~StmtOutput();
};

class SetStream: public Statement
{
protected:
	Expression<stream&> *pStream;
	ExpStrn* Expr;
	
public:
	SetStream(Expression<stream&> *pstrm, ExpStrn* xpr):
	  pStream(pstrm), Expr(xpr) { }

	 virtual void Run() 
	  { pStream->Eval().str(Expr->Eval()); }

	 virtual ~SetStream(){};
};

class StmtInput: public Statement
{
protected:
	StrDataVec sdvec;
	CInputDialogBox dialog;
	Expression<stream&> *pStream;
	bool initialized;
public:
	StmtInput(Expression<stream&> *pstrm=NULL);
	
	void InsertPrompt(const wchar_t* prompt, ExpTypeless* pData);
	virtual void Run();
	virtual ~StmtInput();
};

struct ReturnSig { DataType type; Function* fptr; std::wstring fname; } ;

inline ReturnSig MakeReturn(DataType dt, const std::wstring& id, Function* ptr)
{
	ReturnSig rtsg;
	rtsg.type = dt; rtsg.fptr = ptr; rtsg.fname = id;
	return rtsg;
}


template<typename T, typename VarAry>
	AssignAryEle<T, VarAry>* StatementList::FindAssignment(VarAry *vptr)
{
	for(int i=stmtindex-1; i>=0; --i)
	{
		SET_USER_ABORT;

		if(stmtlist[i]) 
		{
			if(stmtlist[i]->stmttype==StmtExpr) 
			{
				AssignAryEle<T, VarAry> *asg_ptr = 
					dynamic_cast<AssignAryEle<T, VarAry>*>(stmtlist[i]);

				if(asg_ptr)
				{
					if(asg_ptr->GetVarAry()->IsCongruent(vptr))
						return asg_ptr;
					else continue;
				}
			}
			else
			{
				StatementList* pStmtList = 
					dynamic_cast<StatementList*>(stmtlist[i]);

				if(pStmtList)
				{
					AssignAryEle<T, VarAry> *asg_ptr = 
						pStmtList->FindAssignment<T, VarAry>(vptr);
					if(asg_ptr) return asg_ptr;
				}
			}
		}
	}
	return NULL;
}

template<typename T, typename VarAry>
Statement* StatementList::FindAssignment(const std::wstring& id, integer idx)
{
	for(int i=stmtindex-1; i>=0; --i)
	{
		SET_USER_ABORT;

		if(stmtlist[i]) 
		{
			if(stmtlist[i]->stmttype==StmtExpr) 
			{
				AssignAryEle<T, VarAry> *asg_ptr = 
					dynamic_cast<AssignAryEle<T, VarAry>*>(stmtlist[i]);

				if(asg_ptr)
				{
					if(asg_ptr->vrid == id)
					{
						VarAry *varary = asg_ptr->GetVarAry();
						if(varary->GetIndex1() == idx)
							return stmtlist[i];
						else
							continue;
					}
					else
						continue;
				}
			}
			
			if(stmtlist[i]->stmttype == StmtExpr)
			{
				StatementList* pStmtList = dynamic_cast<StatementList*>(stmtlist[i]);

				if(pStmtList)
				{
					Statement* pStmt = pStmtList->FindAssignment<T, VarAry>(id, idx);
					if(pStmt) return pStmt;
				}
			}
			
		}
	}
	return NULL;
}

template<typename T, typename VarAry>
Statement* StatementList::FindAssignment(const std::wstring& id, integer idx1, integer idx2)
{
	for(int i=stmtindex-1; i>=0; --i)
	{
		SET_USER_ABORT;

		if(stmtlist[i]) 
		{
			if(stmtlist[i]->stmttype==StmtExpr) 
			{
				AssignAryEle<T, VarAry> *asg_ptr = 
					dynamic_cast<AssignAryEle<T, VarAry>*>(stmtlist[i]);

				if(asg_ptr)
				{
					if(asg_ptr->vrid == id)
					{
						VarAry *varary = asg_ptr->GetVarAry();
						if((varary->GetIndex1()==idx1) && (varary->GetIndex2()==idx2))
							return stmtlist[i];
						else
							continue;
					}
					else
						continue;
				}
				//else
				//	throw UnicodeException(LoadUnicodeMsg(IDS_ASSGN_FIND_TYPE_MISMATCH).c_str());
			}
			
			if(stmtlist[i]->stmttype == StmtExpr)
			{
				StatementList* pStmtList = dynamic_cast<StatementList*>(stmtlist[i]);

				if(pStmtList)
				{
					Statement* pStmt = pStmtList->FindAssignment<T, VarAry>(id, idx1, idx2);
					if(pStmt) return pStmt;
				}
			}
			
		}
	}
	return NULL;
}

template<typename T, typename VarAry>
Statement* StatementList::FindAssignment(const std::wstring& id, integer idx1, integer idx2, integer idx3)
{
	for(int i=stmtindex-1; i>=0; --i)
	{
		SET_USER_ABORT;

		if(stmtlist[i]) 
		{
			if(stmtlist[i]->stmttype==StmtExpr) 
			{
				AssignAryEle<T, VarAry> *asg_ptr = 
					dynamic_cast<AssignAryEle<T, VarAry>*>(stmtlist[i]);

				if(asg_ptr)
				{
					if(asg_ptr->vrid == id)
					{
						VarAry *varary = asg_ptr->GetVarAry();
						if( (varary->GetIndex1()==idx1) && 
						    (varary->GetIndex2()==idx2) && 
							(varary->GetIndex3()==idx3) )
							return stmtlist[i];
						else
							continue;
					}
					else
						continue;
				}
				//else
				//	throw UnicodeException(LoadUnicodeMsg(IDS_ASSGN_FIND_TYPE_MISMATCH).c_str());
			}
			
			if(stmtlist[i]->stmttype == StmtExpr)
			{
				StatementList* pStmtList = dynamic_cast<StatementList*>(stmtlist[i]);

				if(pStmtList)
				{
					Statement* pStmt = pStmtList->FindAssignment<T, VarAry>(id, idx1, idx2, idx3);
					if(pStmt) return pStmt;
				}
			}
			
		}
	}
	return NULL;
}

template<typename T, typename VarAry>
Statement* StatementList::FindAssignment(const std::wstring& id, integer idx1, integer idx2, integer idx3, integer idx4)
{
	for(int i=stmtindex-1; i>=0; --i)
	{
		SET_USER_ABORT;

		if(stmtlist[i]) 
		{
			if(stmtlist[i]->stmttype==StmtExpr) 
			{
				AssignAryEle<T, VarAry> *asg_ptr = 
					dynamic_cast<AssignAryEle<T, VarAry>*>(stmtlist[i]);

				if(asg_ptr)
				{
					if(asg_ptr->vrid == id)
					{
						VarAry *varary = asg_ptr->GetVarAry();
						if( (varary->GetIndex1()==idx1) && 
						    (varary->GetIndex2()==idx2) && 
							(varary->GetIndex3()==idx3) && 
							(varary->GetIndex4()==idx4) )
							return stmtlist[i];
						else
							continue;
					}
					else
						continue;
				}
				//else
				//	throw UnicodeException(LoadUnicodeMsg(IDS_ASSGN_FIND_TYPE_MISMATCH).c_str());
			}
			
			if(stmtlist[i]->stmttype == StmtExpr)
			{
				StatementList* pStmtList = dynamic_cast<StatementList*>(stmtlist[i]);

				if(pStmtList)
				{
					Statement* pStmt = pStmtList->FindAssignment<T, VarAry>(id, idx1, idx2, idx3, idx4);
					if(pStmt) return pStmt;
				}
			}
			
		}
	}
	return NULL;
}

template<typename T, typename VarAry>
Statement* StatementList::FindAssignment(const std::wstring& id, 
										 integer idx1, integer idx2, integer idx3, integer idx4, integer idx5)
{
	for(int i=stmtindex-1; i>=0; --i)
	{
		SET_USER_ABORT;

		if(stmtlist[i]) 
		{
			if(stmtlist[i]->stmttype==StmtExpr) 
			{
				AssignAryEle<T, VarAry> *asg_ptr = 
					dynamic_cast<AssignAryEle<T, VarAry>*>(stmtlist[i]);

				if(asg_ptr)
				{
					if(asg_ptr->vrid == id)
					{
						VarAry *varary = asg_ptr->GetVarAry();
						if( (varary->GetIndex1()==idx1) && 
						    (varary->GetIndex2()==idx2) && 
							(varary->GetIndex3()==idx3) && 
							(varary->GetIndex4()==idx4) && 
							(varary->GetIndex5()==idx5) )
							return stmtlist[i];
						else
							continue;
					}
					else
						continue;
				}
				//else
				//	throw UnicodeException(LoadUnicodeMsg(IDS_ASSGN_FIND_TYPE_MISMATCH).c_str());
			}
			
			if(stmtlist[i]->stmttype == StmtExpr)
			{
				StatementList* pStmtList = dynamic_cast<StatementList*>(stmtlist[i]);

				if(pStmtList)
				{
					Statement* pStmt = pStmtList->FindAssignment<T, VarAry>(id, idx1, idx2, idx3, idx4, idx5);
					if(pStmt) return pStmt;
				}
			}
			
		}
	}
	return NULL;
}

template<typename T>
class StmtSelect: public Statement
{
protected:
	
	typedef std::pair<Expression<T>*, Statement*> Pair;
	typedef std::vector<Pair> StmtVec;
	typedef typename std::vector<Pair>::iterator Itr;
	
	StmtVec stmtvec; Expression<T> *expr;

public:
	
	StmtSelect(Expression<T>* xpr): Statement(StmtSele), expr(xpr) { }
	
	void InsertChoice(Expression<T>* xpr, Statement* pStmt)
	{
		stmtvec.push_back(Pair(xpr, pStmt));
	}
	
	virtual void Run()
	{
		extern Stack<StatementList*> gStmtListStack;

		SET_USER_ABORT;
		T value = expr->Eval();
		
		StatementList *pStmt = gStmtListStack.top();
		
		try
		{
			for(Itr itr = stmtvec.begin(); itr != stmtvec.end(); ++itr)
			{
				SET_USER_ABORT;

				if(itr->first)
				{
					if(value == itr->first->Eval())
						itr->second->Run();
					else
						continue;
				}
				else
					itr->second->Run();	
			}
		}
		catch(BreakException&)
		{
			while(pStmt!=gStmtListStack.top()) gStmtListStack.pop(); 
			return;
		}
				
		while(pStmt!=gStmtListStack.top()) gStmtListStack.pop();
	}
	

	virtual ~StmtSelect() { }
};

template<typename T, typename VarAry>
AssignAryEle<T, VarAry>* FindAssignment(VarAry *vptr)
{
	extern Stack<StatementList*> gStmtListStack;
	StatementList* pStatementList;
	AssignAryEle<T, VarAry>* pStmt;
	
	int count = gStmtListStack.size();
	for(int i=0; i<count; ++i)
	{
		SET_USER_ABORT;

		pStatementList = gStmtListStack.getoffset(i);
			
		pStmt = pStatementList->FindAssignment<T, VarAry>(vptr);
		if(pStmt) return pStmt;
	}

	return NULL; 
}

template<typename T, typename VarAry>
Statement* FindAssignment(const std::wstring& id, integer idx)
{
	extern Stack<StatementList*> gStmtListStack;
	StatementList* pStatementList; Statement* pStmt;
	
	int count = gStmtListStack.size();
	for(int i=0; i<count; ++i)
	{
		SET_USER_ABORT;

		pStatementList = gStmtListStack.getoffset(i);
		pStmt = pStatementList->FindAssignment<T, VarAry>(id, idx);			
		if(pStmt) return pStmt;
	}
	return NULL; 
}

template<typename T, typename VarAry>
Statement* FindAssignment(const std::wstring& id, integer idx1, integer idx2)
{
	extern Stack<StatementList*> gStmtListStack;
	StatementList* pStatementList; Statement* pStmt;
	
	int count = gStmtListStack.size();
	for(int i=0; i<count; ++i)
	{
		SET_USER_ABORT;

		pStatementList = gStmtListStack.getoffset(i);
		pStmt = pStatementList->FindAssignment<T, VarAry>(id, idx1, idx2);			
		if(pStmt) return pStmt;
	}
	return NULL; 
}

template<typename T, typename VarAry>
Statement* FindAssignment(const std::wstring& id, integer idx1, integer idx2, integer idx3)
{
	extern Stack<StatementList*> gStmtListStack;
	StatementList* pStatementList; Statement* pStmt;
	
	int count = gStmtListStack.size();
	for(int i=0; i<count; ++i)
	{
		SET_USER_ABORT;

		pStatementList = gStmtListStack.getoffset(i);
		pStmt = pStatementList->FindAssignment<T, VarAry>(id, idx1, idx2, idx3);			
		if(pStmt) return pStmt;
	}
	return NULL; 
}

template<typename T, typename VarAry>
Statement* FindAssignment(const std::wstring& id, integer idx1, integer idx2, integer idx3, integer idx4)
{
	extern Stack<StatementList*> gStmtListStack;
	StatementList* pStatementList; Statement* pStmt;
	
	int count = gStmtListStack.size();
	for(int i=0; i<count; ++i)
	{
		SET_USER_ABORT;

		pStatementList = gStmtListStack.getoffset(i);
		pStmt = pStatementList->FindAssignment<T, VarAry>(id, idx1, idx2, idx3, idx4);			
		if(pStmt) return pStmt;
	}
	return NULL; 
}

template<typename T, typename VarAry>
Statement* FindAssignment(const std::wstring& id, integer idx1, integer idx2, integer idx3, integer idx4, integer idx5)
{
	extern Stack<StatementList*> gStmtListStack;
	StatementList* pStatementList; Statement* pStmt;
	
	int count = gStmtListStack.size();
	for(int i=0; i<count; ++i)
	{
		SET_USER_ABORT;

		pStatementList = gStmtListStack.getoffset(i);
		pStmt = pStatementList->FindAssignment<T, VarAry>(id, idx1, idx2, idx3, idx4, idx5);			
		if(pStmt) return pStmt;
	}
	return NULL; 
}

class StmtFor: public Statement
{
protected:
	ExpBool* expr;
	Statement* stmt2;
public:
	Statement *stmt;
	Statement* stmt1;

	StmtFor(Statement* st1, ExpBool* xpr, Statement* st2, Statement* st);
	virtual void Run();
	virtual bool IsReturn() const;
	virtual ~StmtFor();
};

class StmtGraph1D: public Statement
{
protected:
	RealData *pVar;

	ExpReal *pExpr1, *pExpr2, *pExpr3;
	std::wstring bufname;

public:

	StmtGraph1D(const wchar_t* bname, RealData *var, ExpReal* xpr1, ExpReal* xpr2, ExpReal* xpr3):
	  Statement(StmtGr1D), bufname(bname), pVar(var), pExpr1(xpr1), pExpr2(xpr2), pExpr3(xpr3) { }

	virtual void Run()
	{
		//////////////////// Put code here

		realnum x1 = pExpr1->Eval(), x2 = pExpr2->Eval();
		realnum maxx = GetMax<realnum>(x1, x2);
		realnum minx = GetMin<realnum>(x1, x2);

		int count = 1000;
		realnum dx = (maxx - minx)/realnum(count), rtn;
		
		std::wostringstream os;
	
		for(realnum x = minx; x <= maxx; x += dx)
		{
			//pVar->GetData() = x;
			//os<<pVar->GetId()<<_T(" = ")<<pExpr3->Eval()<<_T("\r\n";
			EVAL_XPR(pVar, x, rtn, pExpr3);
		}

		PutString(os.str());
	}
	
	virtual ~StmtGraph1D() { }
};

class StmtWhile: public Statement
{
protected:
	ExpBool* pExpr;
	Statement *pStmt;

public:
	StmtWhile(ExpBool* expr, Statement* stmt);
	virtual void Run();
	
	virtual bool IsReturn() const;
	
	virtual ~StmtWhile();
};

class StmtThrowBase: public Statement
{
public:
	StmtThrowBase(): Statement(StmtThrw) {}

	virtual void Run() { throw; /* *this; */ }

	virtual bool IsReturn() const { return true; }
	
	virtual ~StmtThrowBase() { }
};

class StmtThrowExp: public StmtThrowBase
{
protected:
	LeftHandSide<UserData> *pExpr;  //StatementList *pStmtList;

public:

	UserData theValue;

	StmtThrowExp(LeftHandSide<UserData>* xpr):  
	pExpr(xpr){}
	
	virtual void Run()
	{
		theValue = pExpr->GetData(); throw *this;
	}

	virtual ~StmtThrowExp() { }
};

template<typename T> 
class StmtThrow: public StmtThrowBase
{
protected:
	Expression<T>* pExpr;  //StatementList *pStmtList;

public:

	T theValue;

	StmtThrow(Expression<T>* xpr):  pExpr(xpr) {}
	
	virtual void Run()
	{
		theValue = pExpr->Eval();  throw *this;
	}

	virtual ~StmtThrow() { }
};


class StmtTryCatchBase: public Statement
{
protected:
	Statement *pStmtTry; StatementList *pStmtCatch;

public:
	StmtTryCatchBase(Statement *stmttry, StatementList *stmtcatch):
	  pStmtTry(stmttry), pStmtCatch(stmtcatch), Statement(StmtCath) { }
	
	  virtual bool IsReturn() const
	  {	return pStmtTry->IsReturn() && pStmtCatch->IsReturn() ;  }
	
	virtual void Run()
	{
		try
		{
			pStmtTry->Run();
		}
		catch(StmtThrowBase&)
		{
			pStmtCatch->Run();
		}
	}
		
	virtual ~StmtTryCatchBase() {}
};

template<typename T>
class StmtTryCatch: public StmtTryCatchBase
{

public:
	StmtTryCatch(Statement *stmttry, StatementList *stmtcatch):
	  StmtTryCatchBase(stmttry, stmtcatch) { }
	
	virtual void Run()
	{
		try
		{
			pStmtTry->Run();
		}
		catch(StmtThrow<T>& exp)
		{
			Statement *pStmt = pStmtCatch->GetStatement(0);

			T theValue = exp.theValue; 
			
			pStmt->SetThrownPtr((void*)&theValue);
			
			pStmtCatch->Run();
		}
	}
		
	virtual ~StmtTryCatch() {}
};

template<typename T>
class StmtTryCatchPtr: public StmtTryCatchBase
{

public:
	StmtTryCatchPtr(Statement *stmttry, StatementList *stmtcatch):
	  StmtTryCatchBase(stmttry, stmtcatch) { }
	
	virtual void Run()
	{
		try
		{
			pStmtTry->Run();
		}
		catch(StmtThrow<T>& exp)
		{
			Statement *pStmt = pStmtCatch->GetStatement(0);

			T *theValue = &exp.theValue; 
			
			pStmt->SetThrownPtr((void*)theValue);
			
			pStmtCatch->Run();
		}
	}
		
	virtual ~StmtTryCatchPtr() {}
};

template<>
class StmtTryCatch<UserData>: public StmtTryCatchBase
{

public:
	StmtTryCatch(Statement *stmttry, StatementList *stmtcatch):
	  StmtTryCatchBase(stmttry, stmtcatch) { }
	
	virtual void Run()
	{
		try
		{
			pStmtTry->Run();
		}
		catch(StmtThrowExp& exp)
		{
			DeclUdt *pStmt = 
				dynamic_cast<DeclUdt*>(pStmtCatch->GetStatement(0));
			
			if(exp.theValue.factory != pStmt->GetFactory()) throw;
			
			pStmt->SetThrownPtr(&exp.theValue);

			pStmtCatch->Run();
		}
	}
		
	virtual ~StmtTryCatch() {}
};


template<>
class StmtTryCatchPtr<UserData>: public StmtTryCatchBase
{

public:
	StmtTryCatchPtr(Statement *stmttry, StatementList *stmtcatch):
	  StmtTryCatchBase(stmttry, stmtcatch) { }
	
	virtual void Run()
	{
		try
		{
			pStmtTry->Run();
		}
		catch(StmtThrowExp& exp)
		{
			DeclarePtr<UserData> *pStmt = 
				dynamic_cast<DeclarePtr<UserData>*>(pStmtCatch->GetStatement(0));

			if(exp.theValue.factory != pStmt->factory) throw;

			UserData *theValue = &exp.theValue; 
			
			pStmt->SetThrownPtr((void*)theValue);
			
			pStmtCatch->Run();
		}
	}
		
	virtual ~StmtTryCatchPtr() {}
};

class StmtDo: public Statement
{
protected:
	ExpBool* pExpr;
	Statement *pStmt;

public:
	Statement *pUpdate;

	StmtDo(ExpBool* expr, Statement* stmt);
	virtual void Run();
	
	virtual bool IsReturn() const;

	virtual ~StmtDo();
};

